@@ -32,7 +32,6 @@
 #include "system.h"
 #include "gretl_restrict.h"
 #include "gretl_func.h"
-#include "modelspec.h"
 #include "libset.h"
 #include "forecast.h"
 #include "cmd_private.h"
@@ -84,9 +83,10 @@ static int saved_object_action (const char *line, double ***pZ,
 				DATAINFO *pdinfo, MODEL **models,
 				PRN *prn);
 
-static void usage(void)
+static void usage (int err)
 {
     logo();
+
     printf(_("\nYou may supply the name of a data file on the command line.\n"
 	     "Options:\n"
 	     " -b or --batch     Process a command script and exit.\n"
@@ -99,7 +99,12 @@ static void usage(void)
 	     " gretlcli -b myfile.inp >myfile.out\n"
 	     "Example of run mode usage:\n"
 	     " gretlcli -r myfile.inp\n"));
-    exit(EXIT_SUCCESS);
+
+    if (err) {
+	exit(EXIT_FAILURE);
+    } else {
+	exit(EXIT_SUCCESS);
+    }
 }
 
 static int cli_workdir_callback (const char *s)
@@ -170,29 +175,6 @@ static void nls_init (void)
     reset_local_decpoint();
 }
 
-static void force_language (int f)
-{
-    if (f == ENGLISH) {
-	setlocale(LC_ALL, "C");
-    } else if (f == BASQUE) {
-# ifdef WIN32
-	setlocale(LC_ALL, "eu");
-# else
-	setlocale(LC_ALL, "eu_ES");
-# endif
-    }
-
-# ifdef WIN32
-    if (f == ENGLISH) { 
-	SetEnvironmentVariable("LC_ALL", "C");
-	putenv("LC_ALL=C");
-    } else if (f == BASQUE) {
-	SetEnvironmentVariable("LC_ALL", "eu");
-	putenv("LC_ALL=eu");
-    }	
-# endif
-}
-
 #endif /* ENABLE_NLS */
 
 static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
@@ -215,8 +197,6 @@ static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
     clear_model(models[0]);
     clear_model(models[1]);
 
-    free_modelspec();
-
     if (cmd->opt & OPT_P) {
 	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
     } else {
@@ -280,7 +260,7 @@ static int maybe_get_input_line_continuation (char *line)
 	return 0;
     }
 
-    contd = top_n_tail(line, &err);
+    contd = top_n_tail(line, MAXLINE, &err);
 
     while (contd && !err) {
 	*tmp = '\0';
@@ -305,7 +285,7 @@ static int maybe_get_input_line_continuation (char *line)
 		compress_spaces(line);
 	    }
 	}
-	contd = top_n_tail(line, &err);
+	contd = top_n_tail(line, MAXLINE, &err);
     }
 
     return err;
@@ -373,19 +353,25 @@ int main (int argc, char *argv[])
     
     if (argc > 1) {
 	int force_lang = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
+	int opt = parseopt(&argc, &argv, filearg, &force_lang);
+
+	if (opt & OPT_ERROR) {
+	    usage(1);
+	}
 
 	switch (opt) {
 	case OPT_BATCH:
 	    batch = 1;
-	    if (*filearg == '\0') usage();
+	    if (*filearg == '\0') usage(1);
 	    strcpy(runfile, filearg);
 	    cli_get_data = 1;
 	    break;
 	case OPT_HELP:
+	    usage(0);
+	    break;
 	case OPT_DBOPEN:
 	case OPT_WEBDB:
-	    usage();
+	    usage(1);
 	    break;
 	case OPT_VERSION:
 	    logo();
@@ -394,7 +380,7 @@ int main (int argc, char *argv[])
 	case OPT_RUNIT:
 	    runit = 1;
 	    if (*filearg == '\0') {
-		usage();
+		usage(1);
 	    }
 	    strcpy(runfile, filearg); 
 	    cli_get_data = 1;
@@ -488,6 +474,7 @@ int main (int argc, char *argv[])
 				     &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_DTA:
+	case GRETL_SAV:
 	case GRETL_JMULTI:
 	case GRETL_OCTAVE:
 	case GRETL_WF1:
@@ -622,7 +609,6 @@ int main (int argc, char *argv[])
 
     free(line);
 
-    free_modelspec();
     gretl_print_destroy(prn);
     gretl_cmd_free(&cmd);
     libgretl_cleanup();
@@ -743,6 +729,15 @@ static gretlopt plot_opt (gretlopt opt, int batch)
 
 #define ENDRUN (NC + 1)
 
+/* exec_line: this is called to execute both interactive and script
+   commands.  Note that most commands get passed on to the libgretl
+   function gretl_cmd_exec(), but some commands that require special
+   action are dealt with here.  All estimation commands are passed on
+   to libgretl.
+
+   see also gui_exec_line() in gui2/library.c
+*/
+
 static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 {
     char *line = s->line;
@@ -774,12 +769,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	k = saved_object_action(line, pZ, pdinfo, models, prn);
 	if (k == 1) return 0;   /* action was OK, or ignored */
 	if (k == -1) return 1;  /* action was faulty */
-
-	/* are we ready for this? */
-	if (!data_status && !cmd_ignore(cmd) && !ready_for_command(line)) {
-	    fprintf(stderr, _("You must open a data file first\n"));
-	    return 1;
-	}
     }
 
     /* tell libgretl if we're in batch mode */
@@ -828,7 +817,9 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     if (cmd->ci == LOOP || gretl_compiling_loop()) {  
 	/* accumulating loop commands */
 	if (!ok_in_loop(cmd->ci)) {
-	    printf(_("Command '%s' ignored; not available in loop mode\n"), line);
+	    printf("> %s\n", line);
+	    pprintf(prn, _("Sorry, this command is not available in loop mode\n"));
+	    err = E_NOTIMP;
 	} else {
 	    if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
 		eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
@@ -855,7 +846,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     check_for_loop_only_options(cmd->ci, cmd->opt, prn);
 
-    s->callback = cli_exec_callback;
+    gretl_exec_state_set_callback(s, cli_exec_callback);
 
     switch (cmd->ci) {
 
@@ -959,7 +950,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
 	printf(_("type a filename to store output (enter to quit): "));
 	get_a_filename(outfile);
-	top_n_tail(outfile, NULL);
+	top_n_tail(outfile, 0, NULL);
 
 	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
 	    && strcmp(outfile, "q")) {
@@ -1012,7 +1003,9 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    fb = pop_input_file();
 	} else {
 	    strcpy(s->runfile, runfile);
-	    fprintf(stderr, _("%s opened OK\n"), runfile);
+	    if (libset_get_bool(VERBOSE_INCLUDE)) {
+		pprintf(prn, _("%s opened OK\n"), runfile);
+	    }
 	    if (cmd->ci == INCLUDE) {
 		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    } else {
@@ -1035,17 +1028,9 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
     }
 
-    if (!err && (is_model_cmd(cmd->word) || s->alt_model)
-	&& !is_quiet_model_test(cmd->ci, cmd->opt)) { 
-	attach_subsample_to_model(models[0], pdinfo);
-#if MSPEC_DEBUG
-	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
-		(models[0])->ID, get_model_count());
-#endif
-	if (is_model_cmd(cmd->word)) {
-	    err = modelspec_save(models[0]);
-	}
-	maybe_stack_model(models[0], cmd, prn);
+    if (!err && s->pmod != NULL) { 
+	attach_subsample_to_model(s->pmod, pdinfo);
+	err = maybe_stack_model(s->pmod, cmd, prn);
     }
 
     if (system_save_flag_is_set(s->sys)) {
