@@ -95,6 +95,20 @@ enum {
 
 /* functions relating to distribution graphics */
 
+int use_alt_form (int d, double *parms)
+{
+    if (d == CHISQ_DIST && parms[0] > 69) {
+	return 1;
+    } else if (d == BINOMIAL_DIST) {
+	int n = parms[1];
+	double p = parms[0];
+
+	return (n*p > 8 && n*(1-p) > 8);
+    } else {
+	return 0;
+    }
+}
+
 static int get_dist_and_params (const char *s, int *d, double *x)
 {
     int ret = 1;
@@ -160,14 +174,14 @@ htest_graph_title (int dist, double x, double *parms)
     gchar *s = NULL;
 
     if (dist == NORMAL_DIST) {
-	s = g_strdup(G_("Gaussian sampling distribution"));
+	s = g_strdup(_("Gaussian sampling distribution"));
     } else if (dist == T_DIST) {
-	s = g_strdup_printf(G_("t(%d) sampling distribution"), (int) parms[0]);
+	s = g_strdup_printf(_("t(%d) sampling distribution"), (int) parms[0]);
     } else if (dist == CHISQ_DIST) {
-	s = g_strdup_printf(G_("Chi-square(%d) sampling distribution"), 
+	s = g_strdup_printf(_("Chi-square(%d) sampling distribution"), 
 			    (int) parms[0]);
     } else if (dist == F_DIST) {
-	s = g_strdup_printf(G_("F(%d, %d) sampling distribution"), 
+	s = g_strdup_printf(_("F(%d, %d) sampling distribution"), 
 			    (int) parms[0], (int) parms[1]);
     }
 
@@ -180,19 +194,19 @@ dist_graph_title (int dist, double *parms)
     gchar *s = NULL;
 
     if (dist == NORMAL_DIST) {
-	s = g_strdup_printf(G_("N(%g, %g)"), parms[0], parms[1] * parms[1]);
+	s = g_strdup_printf(_("N(%g, %g)"), parms[0], parms[1] * parms[1]);
     } else if (dist == T_DIST) {
-	s = g_strdup_printf(G_("t(%d)"), (int) parms[0]);
+	s = g_strdup_printf(_("t(%d)"), (int) parms[0]);
     } else if (dist == CHISQ_DIST) {
-	s = g_strdup_printf(G_("Chi-square(%d)"), (int) parms[0]);
+	s = g_strdup_printf(_("Chi-square(%d)"), (int) parms[0]);
     } else if (dist == F_DIST) {
-	s = g_strdup_printf(G_("F(%d, %d)"), (int) parms[0], (int) parms[1]);
+	s = g_strdup_printf(_("F(%d, %d)"), (int) parms[0], (int) parms[1]);
     } else if (dist == BINOMIAL_DIST) {
-	s = g_strdup_printf(G_("Binomial(%d, %g)"), (int) parms[1], parms[0]);
+	s = g_strdup_printf(_("Binomial(%d, %g)"), (int) parms[1], parms[0]);
     } else if (dist == POISSON_DIST) {
-	s = g_strdup_printf(G_("Poisson(%g)"), parms[0]);
+	s = g_strdup_printf(_("Poisson(%g)"), parms[0]);
     } else if (dist == WEIBULL_DIST) {
-	s = g_strdup_printf(G_("Weibull(%g, %g)"), parms[0], parms[1]);
+	s = g_strdup_printf(_("Weibull(%g, %g)"), parms[0], parms[1]);
     }
 
     return s;
@@ -241,13 +255,13 @@ enum {
 #define F_F      "f(x,m,n)=Binv(0.5*m,0.5*n)*(m/n)**(0.5*m)*" \
                  "x**(0.5*m-1.0)/(1.0+m/n*x)**(0.5*(m+n))"
 #define F_STUD   "stud(x,m)=Binv(0.5*m,0.5)/sqrt(m)*(1.0+(x*x)/m)**(-0.5*(m+1.0))"
-#define F_COMB   "comb(n,k)=n!/(k!*(n-k)!)"
-#define F_BINOM  "binom(k,n,p)=comb(int(n),int(k))*p**k*(1-p)**(n-k)"
 #define F_POIS   "poisson(z,k)=exp(-z)*(z**k)/(int(k))!"
 #define F_WEIB   "weibull(x,shp,scl)=(shp/scl)*(x/scl)**(shp-1.0)*exp(-(x/scl)**shp)"
+#define F_BINOM  "binom(k,n,p)=k<0||k>n?0.0:exp(lgamma(n+1)-lgamma(n-k+1)-lgamma(k+1)" \
+                 "+k*log(p)+(n-k)*log(1.0-p))"
 
 static void 
-dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax)
+dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax, int alt)
 {
     double x[3] = {0};
     char st = 0;
@@ -274,14 +288,12 @@ dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax)
 	    x[2] = 0.005;
 	}
     } else if (d == BINOMIAL_DIST) {
-	int n = parms[1];
-	double p = parms[0];
-
-	if (n*p > 5 && n*(1-p) > 5) {
-	    /* use normal approx */
+	if (alt) {
+	    int n = parms[1];
+	    double p = parms[0];
 	    double m = n * p;
 	    double s = sqrt(m * (1 - p));
-    
+
 	    *xmin = m - 4 * s;
 	    *xmax = m + 4 * s;
 	    if (*xmin < 0) {
@@ -349,7 +361,7 @@ static double dist_xmax (int d, double *parms)
     return gretl_get_critval(st, x);
 }
 
-static int n_literal_lines (int d, int alt, int ptype)
+static int n_literal_lines (int d, int ptype)
 {
     int n = 0;
 
@@ -369,7 +381,7 @@ static int n_literal_lines (int d, int alt, int ptype)
 	n = 5;
 	break;
     case BINOMIAL_DIST:
-	n = (alt)? 4 : 5;
+	n = 4;
 	break;
     }
 
@@ -398,7 +410,7 @@ range_from_test_stat (int d, double x, double *parms, double *spike,
 	*spike = .25;
 	fprintf(fp, "set xrange [%.3f:%.3f]\n", -x1, x1);
 	fprintf(fp, "set yrange [0:.50]\n");
-	fprintf(fp, "set xlabel '%s'\n", G_("Standard errors"));
+	fprintf(fp, "set xlabel '%s'\n", _("Standard errors"));
     } else {
 	x1 = dist_xmax(d, parms);
 	if (x > x1) {
@@ -410,11 +422,11 @@ range_from_test_stat (int d, double x, double *parms, double *spike,
 }
 
 static void
-range_from_dist (int d, double *parms, FILE *fp)
+range_from_dist (int d, double *parms, int alt, FILE *fp)
 {
     double x, tmin, tmax;
 
-    dist_xmin_xmax(d, parms, &tmin, &tmax);
+    dist_xmin_xmax(d, parms, &tmin, &tmax, alt);
 
     fprintf(fp, "set trange [%g:%g]\n", tmin, tmax);
 
@@ -435,7 +447,7 @@ spec_range_from_dist (int d, double *parms, int alt, GPT_SPEC *spec)
     double *t_range = spec->range[3];
     double tmin, tmax;
 
-    dist_xmin_xmax(d, parms, &tmin, &tmax);
+    dist_xmin_xmax(d, parms, &tmin, &tmax, alt);
 
     if (t_range[0] == 0 && tmin < 0) {
 	; /* don't adjust? */
@@ -518,19 +530,16 @@ static void htest_graph (int d, double x, double *parms)
 	return;
     }
 
-    if (d == CHISQ_DIST && parms[0] > 69) {
-	alt = 1;
-    } 
+    alt = use_alt_form(d, parms);
 
-    fputs("set key right top\n", fp);
+    print_keypos_string(GP_KEY_RIGHT_TOP, fp);
     
     gretl_push_c_numeric_locale();
 
     range_from_test_stat(d, x, parms, &spike, fp);
 
     /* header */
-    fprintf(fp, "# literal lines = %d\n", 
-	    n_literal_lines(d, alt, PLOT_H_TEST));
+    fprintf(fp, "# literal lines = %d\n", n_literal_lines(d, PLOT_H_TEST));
     title = dist_comment_line(d, parms);
     fprintf(fp, "%s\n", title);
     g_free(title);
@@ -577,7 +586,7 @@ static void htest_graph (int d, double x, double *parms)
     title = htest_graph_title(d, x, parms);
     fprintf(fp, " title '%s' w lines , \\\n", title);
     fprintf(fp, "'-' using 1:2 title '%s' w impulses\n",
-	    G_("test statistic"));
+	    _("test statistic"));
     fprintf(fp, "%g %g\n", x, spike);
     fputs("e\n", fp);
     g_free(title);
@@ -605,22 +614,17 @@ static void dist_graph (int d, double *parms)
 	return;
     }
 
-    if (d == CHISQ_DIST && parms[0] > 69) {
-	alt = 1;
-    } else if (d == BINOMIAL_DIST && parms[1] > 150) {
-	alt = 1;
-    }
+    alt = use_alt_form(d, parms);
 
-    fputs("set key right top\n", fp);
+    print_keypos_string(GP_KEY_RIGHT_TOP, fp);
     fputs("set parametric\n", fp);
 
     gretl_push_c_numeric_locale();
 
-    range_from_dist(d, parms, fp);
+    range_from_dist(d, parms, alt, fp);
 
     /* header */
-    fprintf(fp, "# literal lines = %d\n", 
-	    n_literal_lines(d, alt, PLOT_PROB_DIST));
+    fprintf(fp, "# literal lines = %d\n", n_literal_lines(d, PLOT_PROB_DIST));
     title = dist_comment_line(d, parms);
     fprintf(fp, "%s\n", title);
     g_free(title);
@@ -653,7 +657,6 @@ static void dist_graph (int d, double *parms)
 	if (alt) {
 	    fprintf(fp, "%s\n", F_NORM);
 	} else {
-	    fprintf(fp, "%s\n", F_COMB);
 	    fprintf(fp, "%s\n", F_BINOM);
 	}
 	break;
@@ -765,11 +768,7 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
     }
 
     /* alternate forms for some plots */
-    if (d == CHISQ_DIST && parms[0] > 69) {
-	alt = 1;
-    } else if (d == BINOMIAL_DIST && parms[1] > 150) {
-	alt = 1;
-    }
+    alt = use_alt_form(d, parms);
 
     /* maybe adjust t-range */
     spec_range_from_dist(d, parms, alt, spec);
@@ -861,8 +860,7 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 	if (alt && !got[NORMAL_DIST]) {
 	    f1 = F_NORM;
 	} else if (!alt && !got[BINOMIAL_DIST]) {
-	    f1 = F_COMB;
-	    f2 =  F_BINOM;
+	    f1 = F_BINOM;
 	}
 	break;
     case POISSON_DIST:
@@ -892,12 +890,12 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 
     i = spec->n_lines - 1;
 
-    strcpy(spec->lines[i].scale, "NA");
+    spec->lines[i].scale = NADBL;
 
     if (d == BINOMIAL_DIST || d == POISSON_DIST) {
-	strcpy(spec->lines[i].style, "linespoints");
+	spec->lines[i].style = GP_STYLE_LINESPOINTS;
     } else {
-	strcpy(spec->lines[i].style, "lines");
+	spec->lines[i].style = GP_STYLE_LINES;
     }
 
     title = dist_graph_title(d, parms);
@@ -1500,8 +1498,10 @@ static void np_test (GtkWidget *w, test_t *test)
     if (err) {
 	gui_errmsg(err);
     } else {
-	view_buffer(prn, 78, 380, "gretl: nonparametric test", 
-		    PRINT, NULL);
+	gchar *title = g_strdup_printf("gretl: %s", _("nonparametric test"));
+
+	view_buffer(prn, 78, 380, title, PRINT, NULL);
+	g_free(title);			       
     }
 }
 
@@ -2112,7 +2112,7 @@ static void populate_stats (GtkWidget *w, gpointer p)
     test_t *test = g_object_get_data(G_OBJECT(p), "test");
     int pos = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(p), "pos"));
     gchar **pbuf = g_object_get_data(G_OBJECT(p), "pbuf");
-    int t, n = datainfo->t2 - datainfo->t1 + 1;
+    int t, n = sample_size(datainfo);
     int vx = -1, vy = -1;
     GretlOp yop = 0;
     gchar *buf = NULL;
@@ -2251,7 +2251,7 @@ static void add_vars_to_combo (GtkWidget *w, int code, int pos)
 
 static void toggle_combo_ok (GtkWidget *toggle, gpointer p)
 {
-    if (GTK_TOGGLE_BUTTON(toggle)->active) {
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle))) {
 	gtk_widget_set_sensitive(GTK_WIDGET(p), TRUE);
 	populate_stats(NULL, p);
     } else {
@@ -2371,7 +2371,7 @@ static void add_test_combo (GtkWidget *tbl, gint *rows,
     gtk_widget_set_sensitive(tmp, FALSE);
 
     entry = gtk_bin_get_child(GTK_BIN(tmp));
-    g_signal_connect(G_OBJECT(GTK_ENTRY(entry)), "key_press_event",
+    g_signal_connect(G_OBJECT(GTK_ENTRY(entry)), "key-press-event",
 		     G_CALLBACK(catch_combo_key), tmp);
     g_signal_connect(G_OBJECT(GTK_COMBO_BOX(tmp)), "changed",
 		     G_CALLBACK(select_child_callback), tmp);
@@ -2463,17 +2463,6 @@ static int n_ok_dummies (void)
     return nv;
 }
 
-static gint toggle_verbose_state (GtkWidget *w, GtkWidget *b)
-{
-    if (GTK_TOGGLE_BUTTON(w)->active) {
-	gtk_widget_set_sensitive(b, FALSE);
-    } else {
-	gtk_widget_set_sensitive(b, TRUE);
-    }
-
-    return FALSE;
-}
-
 static void make_nptest_tab (CalcChild *child, int idx) 
 {
     test_t **tests = child->calcp;
@@ -2570,9 +2559,7 @@ static void make_nptest_tab (CalcChild *child, int idx)
 
     if (idx == NP_DIFF) {
 	gtk_widget_set_sensitive(tmp, FALSE);
-	g_signal_connect(G_OBJECT(test->radio[0]), "toggled",
-			 G_CALLBACK(toggle_verbose_state),
-			 tmp);
+	desensitize_widget_from_check(test->radio[0], tmp);
     }
 }
 
@@ -2923,7 +2910,7 @@ static int calc_help_code (int c)
 static void real_stats_calculator (int code, gpointer data) 
 {
     GtkWidget *tmp = NULL;
-    static GtkWidget *winptr[6];
+    static GtkWidget *winptr[CALC_MAX];
     GtkWidget *oldwin;
     CalcChild *child;
     const char *calc_titles[] = {
@@ -3022,6 +3009,162 @@ static void real_stats_calculator (int code, gpointer data)
     gtk_widget_show(child->dlg);
 }
 
+/* for gnuplot: convert '^' to '**' for exonentiation */
+
+static gchar *formula_mod (const gchar *s)
+{
+    const gchar *p = s;
+    int n = strlen(s) + 1;
+    gchar *q, *ret;
+
+    while (*p) {
+	if (*p == '^') n++;
+	p++;
+    }
+
+    ret = g_malloc(n);
+    q = ret;
+
+    while (*s) {
+	if (*s == '^') {
+	    *q++ = '*';
+	    *q++ = '*';
+	    s++;
+	} else {
+	    *q++ = *s++;
+	}
+    }  
+
+    *q = '\0';
+
+    return ret;
+}
+
+struct curve_plotter {
+    GtkWidget *dlg;
+    GtkWidget *entry;
+    gchar *formula;
+    double xmin;
+    double xrange;
+};
+
+static void do_plot_curve (GtkWidget *w, struct curve_plotter *p)
+{
+    const gchar *s = gtk_entry_get_text(GTK_ENTRY(p->entry));
+    FILE *fp = NULL;
+    int err = 0;
+
+    if (s == NULL || *s == '\0') {
+	return;
+    }
+
+    g_free(p->formula);
+
+    if (strchr(s, '^')) {
+	p->formula = formula_mod(s);
+    } else {
+	p->formula = g_strdup(s);
+    }
+
+    if (gnuplot_init(PLOT_CURVE, &fp)) { 
+	return;
+    }
+
+    print_keypos_string(GP_KEY_RIGHT_TOP, fp);
+    
+    gretl_push_c_numeric_locale();
+
+    fprintf(fp, "set xrange [%g:%g]\n", p->xmin, p->xmin + p->xrange);
+    fprintf(fp, "plot \\\n");
+    fprintf(fp, "%s notitle w lines\n", p->formula);
+
+    gretl_pop_c_numeric_locale();
+
+    fclose(fp);
+
+    err = gnuplot_make_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph();
+	gtk_widget_destroy(p->dlg);
+    }
+}
+
+/* plot a curve specified via formula (no data required) */
+
+static void plot_curve (void)
+{
+    static struct curve_plotter plotter;
+    GtkWidget *dialog, *hbox, *vbox;
+    GtkWidget *button, *tmp;
+    GtkObject *adj;
+
+    if (plotter.dlg != NULL) {
+	gtk_window_present(GTK_WINDOW(plotter.dlg));
+	return;
+    }
+
+    plotter.dlg = dialog = 
+	gretl_dialog_new(_("gretl: plot a curve"), mdata->main, 0);
+
+    g_signal_connect(G_OBJECT(dialog), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed), &plotter.dlg);
+
+    if (plotter.xrange == 0) {
+	plotter.xrange = 10;
+    }
+
+    vbox = GTK_DIALOG(dialog)->vbox;
+
+    /* gnuplot formula entry box */
+    hbox = gtk_hbox_new(FALSE, 5);
+    tmp = gtk_label_new(_("formula"));
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    plotter.entry = tmp = gtk_entry_new();
+    gtk_entry_set_width_chars(GTK_ENTRY(tmp), 32);
+    if (plotter.formula != NULL) {
+	gtk_entry_set_text(GTK_ENTRY(tmp), plotter.formula);
+    }
+    gtk_entry_set_activates_default(GTK_ENTRY(tmp), TRUE);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+
+    /* x-axis spinners (min and range) */
+    hbox = gtk_hbox_new(FALSE, 5);
+    tmp = gtk_label_new(_("x minimum"));
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    adj = gtk_adjustment_new(plotter.xmin, -100, 100, 1, 0, 0);
+    tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+    g_signal_connect(tmp, "value-changed", 
+		     G_CALLBACK(set_double_from_spinner), &plotter.xmin);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 0);
+    tmp = gtk_label_new("  ");
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    tmp = gtk_label_new(_("x range"));
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+    adj = gtk_adjustment_new(plotter.xrange, 1, 1000, 1, 0, 0);
+    tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+    g_signal_connect(tmp, "value-changed", 
+		     G_CALLBACK(set_double_from_spinner), &plotter.xrange);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 0); 
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+    
+    /* "Cancel" button */
+    cancel_delete_button(GTK_DIALOG(dialog)->action_area, dialog, NULL);
+
+    /* "OK" button */
+    button = ok_button(GTK_DIALOG(dialog)->action_area);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(do_plot_curve), &plotter);
+    gtk_widget_grab_default(button);
+
+    /* Help button */
+    context_help_button(GTK_DIALOG(dialog)->action_area, GPT_CURVE);
+
+    gtk_widget_show_all(dialog);
+}
+
 static int stats_calculator_code (GtkAction *action)
 {
     const gchar *s = gtk_action_get_name(action);
@@ -3038,6 +3181,8 @@ static int stats_calculator_code (GtkAction *action)
 	return CALC_GRAPH;
     else if (!strcmp(s, "AddRandom"))
 	return CALC_RAND;
+    else if (!strcmp(s, "PlotCurve"))
+	return CALC_PLOT;
     else
 	return 0;
 }
@@ -3051,9 +3196,14 @@ void stats_calculator (GtkAction *action, gpointer data)
 		     code == CALC_TEST ||
 		     code == CALC_NPTEST ||
 		     code == CALC_GRAPH ||
-		     code == CALC_RAND);
+		     code == CALC_RAND ||
+		     code == CALC_PLOT);
 
-    real_stats_calculator(code, data);
+    if (code == CALC_PLOT) {
+	plot_curve();
+    } else {
+	real_stats_calculator(code, data);
+    }
 }
 
 void dist_graph_add (gpointer p)
