@@ -26,13 +26,14 @@
 #include "datafiles.h"
 #include "textbuf.h"
 #include "textutil.h"
-#include "boxplots.h"
 #include "dlgutils.h"
 #include "fileselect.h"
 #include "ssheet.h"
 #include "treeutils.h"
 #include "datawiz.h"
 
+#include "boxplots.h"
+
 /* these live in dialogs.c */
 extern GtkWidget *active_edit_id;
 extern GtkWidget *active_edit_name;
@@ -173,6 +174,8 @@ struct open_data_code open_data_codes[] = {
     { APPEND_WF1,      "AppendWF1" },
     { OPEN_DTA,        "OpenDTA" },
     { APPEND_DTA,      "AppendDTA" },
+    { OPEN_SAV,        "OpenSAV" },
+    { APPEND_SAV,      "AppendSAV" },
     { OPEN_JMULTI,     "OpenJMulTi" },
     { APPEND_JMULTI,   "AppendJMulTi" },
     { OPEN_ODS,        "OpenODS" },
@@ -240,6 +243,10 @@ void open_data (GtkAction *action)
     case APPEND_DTA:
 	file_selector(_("Open Stata file"), code, FSEL_DATA_NONE, NULL);
 	break;
+    case OPEN_SAV:
+    case APPEND_SAV:
+	file_selector(_("Open SPSS file"), code, FSEL_DATA_NONE, NULL);
+	break;
     case OPEN_JMULTI:
     case APPEND_JMULTI:
 	file_selector(_("Open JMulTi file"), code, FSEL_DATA_NONE, NULL);
@@ -385,6 +392,7 @@ static int model_action_code (GtkAction *action)
     int ci = gretl_command_number(s);
 
     if (ci == 0) {
+	/* look up "GUI special" */
 	if (!strcmp(s, "CORC"))
 	    ci = CORC;
 	else if (!strcmp(s, "HILU"))
@@ -397,6 +405,20 @@ static int model_action_code (GtkAction *action)
 	    ci = PANEL_B;
 	else if (!strcmp(s, "VLAGSEL"))
 	    ci = VLAGSEL;
+	else if (!strcmp(s, "blogit"))
+	    ci = LOGIT;
+	else if (!strcmp(s, "ologit"))
+	    ci = OLOGIT;
+	else if (!strcmp(s, "mlogit"))
+	    ci = MLOGIT;
+	else if (!strcmp(s, "bprobit"))
+	    ci = PROBIT;
+	else if (!strcmp(s, "oprobit"))
+	    ci = OPROBIT;
+	else if (!strcmp(s, "iv-liml"))
+	    ci = IV_LIML;
+	else if (!strcmp(s, "iv-gmm"))
+	    ci = IV_GMM;
     }
 
     return ci;
@@ -407,48 +429,50 @@ void fit_resid_callback (GtkAction *action, gpointer data)
     const gchar *s = gtk_action_get_name(action);
     windata_t *mydata = (windata_t *) data; 
     MODEL *pmod = mydata->data;
-    int code = GENR_RESID; 
+    int code = M_UHAT; 
 
     if (!strcmp(s, "yhat")) {
-	code = GENR_FITTED;
+	code = M_YHAT;
     } else if (!strcmp(s, "uhat")) {
-	code = GENR_RESID;
+	code = M_UHAT;
     } else if (!strcmp(s, "uhat2")) {
-	code = GENR_RESID2;
+	code = M_UHAT2;
     } else if (!strcmp(s, "h")) {
-	code = GENR_H;
+	code = M_H;
     } else if (!strcmp(s, "ahat")) {
-	code = GENR_AHAT;
+	code = M_AHAT;
     }
 
     add_fit_resid(pmod, code, 0);
 }
 
+/* callback for adding a scalar from a model */
+
 void model_stat_callback (GtkAction *action, gpointer data)
 {
     const gchar *s = gtk_action_get_name(action);
     windata_t *vwin = (windata_t *) data; 
     MODEL *pmod = vwin->data;
-    int code = ESS; 
+    int code = M_ESS; 
 
     if (!strcmp(s, "ess")) {
-	code = ESS;
+	code = M_ESS;
     } else if (!strcmp(s, "se")) {
-	code = SIGMA;
+	code = M_SIGMA;
     } else if (!strcmp(s, "rsq")) {
-	code = R2;
+	code = M_RSQ;
     } else if (!strcmp(s, "trsq")) {
-	code = TR2;
+	code = M_TRSQ;
     } else if (!strcmp(s, "df")) {
-	code = DF;
+	code = M_DF;
     } else if (!strcmp(s, "lnL")) {
-	code = LNL;
+	code = M_LNL;
     } else if (!strcmp(s, "AIC")) {
-	code = AIC;
+	code = M_AIC;
     } else if (!strcmp(s, "BIC")) {
-	code = BIC;
+	code = M_BIC;
     } else if (!strcmp(s, "HQC")) {
-	code = HQC;
+	code = M_HQC;
     }
 
     add_model_stat(pmod, code);
@@ -457,10 +481,8 @@ void model_stat_callback (GtkAction *action, gpointer data)
 void model_callback (GtkAction *action, gpointer data) 
 {
     int code = model_action_code(action);
-    int presel = 0;   
 
-    selection_dialog(_("gretl: specify model"), do_model, code,
-		     presel);
+    modelspec_dialog(code);
 }
 
 void model_genr_callback (GtkAction *action, gpointer data)
@@ -521,12 +543,12 @@ void selector_callback (GtkAction *action, gpointer data)
     strcpy(title, "gretl: ");
 
     if (ci == COINT || ci == COINT2) {
-	selection_dialog(_("gretl: cointegration test"), do_coint, ci, 0);
+	selection_dialog(_("gretl: cointegration test"), do_coint, ci);
     } else if (ci == VAR || ci == VECM) {
 	selection_dialog((ci == VAR)? _("gretl: VAR") : _("gretl: VECM"),
-			 do_vector_model, ci, 0);
+			 do_vector_model, ci);
     } else if (ci == VLAGSEL) {
-	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, ci, 0);
+	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, ci);
     } else if (ci == GR_XY || ci == GR_IMP || ci == GR_DUMMY
 	       || ci == SCATTERS || ci == GR_3D
 	       || ci == GR_XYZ) {
@@ -552,7 +574,7 @@ void selector_callback (GtkAction *action, gpointer data)
 	default:
 	    return;
 	}
-	selection_dialog(_("gretl: define graph"), selfunc, ci, 0);
+	selection_dialog(_("gretl: define graph"), selfunc, ci);
     } else if (ci == ADD || ci == OMIT) {
 	simple_selection(_("gretl: model tests"), do_add_omit, ci, vwin);
     } else if (ci == VAROMIT) {
@@ -602,6 +624,9 @@ static int gretl_callback_code (const gchar *s)
     return 0;
 }
 
+/* callback for menu items, where we want to prepare an "edit dialog"
+   for handle the request */
+
 void gretl_callback (GtkAction *action, gpointer data)
 {
     const char *title = NULL;
@@ -643,25 +668,28 @@ void gretl_callback (GtkAction *action, gpointer data)
 	query = N_("Specify variables to plot:");
 	okfunc = do_box_graph;
 	varclick = VARCLICK_INSERT_NAME;
-	defstr = get_boxplots_string();
+	defstr = get_last_boxplots_string();
 	break;
     case GMM:
 	title = N_("gretl: GMM");
 	query = N_("GMM: Specify function and orthogonality conditions:");
 	okfunc = do_gmm_model;
 	varclick = VARCLICK_INSERT_TEXT;
+	data = NULL;
 	break;	
     case MLE:
 	title = N_("gretl: maximum likelihood");
 	query = N_("MLE: Specify function, and derivatives if possible:");
 	okfunc = do_mle_model;
 	varclick = VARCLICK_INSERT_TEXT;
+	data = NULL;
 	break;	
     case NLS:
 	title = N_("gretl: nonlinear least squares");
 	query = N_("NLS: Specify function, and derivatives if possible:");
 	okfunc = do_nls_model;
 	varclick = VARCLICK_INSERT_TEXT;
+	data = NULL;
 	break;	
     case SYSTEM:
 	title = N_("gretl: simultaneous equations system");
@@ -690,6 +718,53 @@ void gretl_callback (GtkAction *action, gpointer data)
 		cmd, varclick, &cancel);   
 }
 
+void revise_nl_model (MODEL *pmod)
+{
+    const char *title = NULL;
+    const char *query = NULL;
+    const char *defstr;
+    void (*okfunc)() = NULL;
+    int cancel = 0;
+
+    switch (pmod->ci) {
+    case GMM:
+	title = N_("gretl: GMM");
+	query = N_("GMM: Specify function and orthogonality conditions:");
+	okfunc = do_gmm_model;
+	break;	
+    case MLE:
+	title = N_("gretl: maximum likelihood");
+	query = N_("MLE: Specify function, and derivatives if possible:");
+	okfunc = do_mle_model;
+	break;	
+    case NLS:
+	title = N_("gretl: nonlinear least squares");
+	query = N_("NLS: Specify function, and derivatives if possible:");
+	okfunc = do_nls_model;
+	break;	
+    }
+
+    defstr = gretl_model_get_data(pmod, "nlinfo");
+
+    edit_dialog(_(title), _(query), defstr, okfunc, pmod, 
+		pmod->ci, VARCLICK_INSERT_TEXT, &cancel);   
+}
+
+void revise_system_model (void *ptr)
+{
+    equation_system *sys = (equation_system *) ptr;
+    int cancel = 0;
+
+    edit_dialog(_("gretl: simultaneous equations system"), 
+		_("Specify simultaneous equations:"),
+		NULL,
+		do_eqn_system,
+		sys,
+		SYSTEM,
+		VARCLICK_INSERT_TEXT,
+		&cancel);
+}
+
 void genr_callback (void)
 {
     edit_dialog(_("gretl: add var"), 
