@@ -24,6 +24,7 @@
 #include "database.h"
 #include "datafiles.h"
 #include "gretl_www.h"
+#include "gretl_untar.h"
 #include "menustate.h"
 #include "treeutils.h"
 #include "winstack.h"
@@ -201,16 +202,14 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbinfo)
     }
 
     if (dbinfo->structure == CROSS_SECTION) {
-	err = boxplots(list, NULL, dbZ, dbinfo, 0);
-	if (err) {
-	    errbox(_("boxplot command failed"));
-	}
+	err = boxplots(list, dbZ, dbinfo, OPT_NONE);
     } else {
 	err = gnuplot(list, NULL, (const double **) *dbZ, dbinfo,
 		      OPT_G | OPT_O | OPT_T);
-	if (err) {
-	    errbox(_("gnuplot command failed"));
-	}
+    }
+
+    if (err) {
+	errbox(_("gnuplot command failed"));
     }
 
     if (!err) {
@@ -1043,7 +1042,7 @@ static void db_drag_connect (windata_t *vwin, int i)
 			&gretl_drag_targets[i],
 			1, GDK_ACTION_COPY);
 
-    g_signal_connect(G_OBJECT(vwin->listbox), "drag_data_get",
+    g_signal_connect(G_OBJECT(vwin->listbox), "drag-data-get",
 		     G_CALLBACK(do_db_drag),
 		     vwin);
 }
@@ -1363,7 +1362,7 @@ static GtkWidget *database_window (windata_t *vwin)
 
     box = gtk_vbox_new(FALSE, 0);
     vwin_add_list_box(vwin, GTK_BOX(box), 4, TRUE, types, titles, 0);
-    g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+    g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 		     G_CALLBACK(popup_menu_handler), 
 		     vwin->popup);
     gtk_widget_show(box);
@@ -1812,10 +1811,10 @@ static int ggz_extract (char *ggzname)
     if (fcbk != NULL) fclose(fcbk);
 
     if (cblen == 0) {
-	remove(cbname);
+	gretl_remove(cbname);
     }
 
-    remove(ggzname);
+    gretl_remove(ggzname);
 
     return err;
 }
@@ -1842,6 +1841,7 @@ enum {
 
 static char *get_writable_target (int code, int op, char *objname)
 {
+    const char *ext;
     FILE *fp;
     char *targ;
     int err = 0;
@@ -1852,16 +1852,29 @@ static char *get_writable_target (int code, int op, char *objname)
     }
 
     if (code == REMOTE_DB) {
-	build_path(targ, paths.binbase, objname, ".ggz");
+	ext = ".ggz";
+    } else if (code == REMOTE_FUNC_FILES) {
+	ext = ".gfn";
+    } else {
+	ext = ".tar.gz";
+    }
+
+    if (code == REMOTE_DB) {
+	build_path(targ, paths.binbase, objname, ext);
+    } else if (code == REMOTE_DATA_PKGS) {
+	    char pkgdir[MAXLEN];
+
+	    get_default_dir(pkgdir, SAVE_DATA_PKG);
+	    build_path(targ, pkgdir, objname, ext);
     } else {
 	if (op == TMP_INSTALL) {
 	    build_path(targ, paths.dotdir, "dltmp", NULL);
 	    err = gretl_tempname(targ);
 	} else {
-	    char fndir[MAXLEN];
+	    char pkgdir[MAXLEN];
 
-	    get_default_dir(fndir, SAVE_FUNCTIONS);
-	    build_path(targ, fndir, objname, ".gfn");
+	    get_default_dir(pkgdir, SAVE_FUNCTIONS);
+	    build_path(targ, pkgdir, objname, ext);
 	}
     } 
 
@@ -1875,7 +1888,7 @@ static char *get_writable_target (int code, int op, char *objname)
 	if (fp == NULL) {
 	    if (errno == EACCES && code != REMOTE_FUNC_FILES) { 
 		/* write to user dir instead? */
-		build_path(targ, paths.workdir, objname, ".ggz");
+		build_path(targ, paths.workdir, objname, ext);
 	    } else {
 		file_write_errbox(targ);
 		free(targ);
@@ -1889,9 +1902,40 @@ static char *get_writable_target (int code, int op, char *objname)
     return targ;
 }
 
+static int unpack_book_data (const char *fname)
+{
+    char *p, path[FILENAME_MAX];
+    int err = 0;
+
+    errno = 0;
+
+    strcpy(path, fname);
+
+    p = strrchr(path, SLASH);
+    if (p == NULL && SLASH == '\\') {
+	p = strrchr(path, '/');
+    }
+    if (p != NULL) {
+	*p = '\0';
+    }
+
+    chdir(path);
+
+    if (errno != 0) {
+        gretl_errmsg_set_from_errno("chdir");
+	err = E_FOPEN;
+    }
+
+    if (!err) {
+	err = gretl_untar(fname);
+    }
+
+    return err;
+}
+
 /* note : 'vwin' here is the source viewer window displaying the
-   remote file (database or function package) that is being installed
-   onto the local machine.
+   remote file (database or datafiles package or function package)
+   that is being installed onto the local machine.
 */
 
 static int real_install_file_from_server (windata_t *vwin, int op)
@@ -1910,25 +1954,26 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	    gtk_tree_model_get(model, &iter, 0, &objname, -1);
 	}
     } else {
-	/* remote function packages */
+	/* remote datafiles or function package */
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			     vwin->active_var, 0, &objname);
-	
     }
 
     if (objname == NULL || *objname == '\0') {
-	free(objname);
+	g_free(objname);
 	return 1;
     }    
 
     target = get_writable_target(vwin->role, op, objname);
     if (target == NULL) {
-	free(objname);
+	g_free(objname);
 	return 1;
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
 	err = retrieve_remote_function_package(objname, target);
+    } else if (vwin->role == REMOTE_DATA_PKGS) {
+	err = retrieve_remote_datafiles_package(objname, target);
     } else {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	err = retrieve_remote_db(objname, target, GRAB_NBO_DATA);
@@ -1953,7 +1998,17 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	    } else {
 		gui_show_function_info(target, VIEW_FUNC_INFO);
 	    }
+	} else if (vwin->role == REMOTE_DATA_PKGS) {
+	    fprintf(stderr, "downloaded '%s'\n", target);
+	    err = unpack_book_data(target);
+	    remove(target);
+	    if (err) {
+		errbox(_("Error unzipping compressed data"));
+	    } else {
+		infobox("Restart gretl to access this database");
+	    }
 	} else {
+	    fprintf(stderr, "downloaded '%s'\n", target);
 	    err = ggz_extract(target);
 	    if (err) {
 		if (err != E_FOPEN) {
@@ -1972,7 +2027,7 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	}
     }
 
-    free(objname);
+    g_free(objname);
     free(target);
 
     return err;
@@ -2088,7 +2143,7 @@ int write_db_description (const char *binname, const char *descrip)
 	}
 	fclose(fnew);
 	fclose(fbak);
-	remove(idxtmp);
+	gretl_remove(idxtmp);
     }
 
     return err;
@@ -2121,70 +2176,86 @@ read_db_files_in_dir (DIR *dir, int dbtype, const char *dbdir,
     return ndb;
 }
 
-static void get_local_object_status (char *fname, int role, char *status, 
+static void get_local_object_status (const char *fname, int role, 
+				     const char **status, 
 				     time_t remtime)
 {
-    char fndir[MAXLEN];
+    char filedir[MAXLEN];
     char fullname[MAXLEN];
     struct stat fbuf;
     int err;
 
     if (role == REMOTE_DB) {
 	build_path(fullname, paths.binbase, fname, NULL);
+    } else if (role == REMOTE_DATA_PKGS) {
+	build_path(filedir, paths.gretldir, "data", NULL);
+	build_path(fullname, filedir, fname, NULL);
     } else {
-	build_path(fndir, paths.gretldir, "functions", NULL);
-	build_path(fullname, fndir, fname, NULL);
+	build_path(filedir, paths.gretldir, "functions", NULL);
+	build_path(fullname, filedir, fname, NULL);
     }
 
-    if ((err = stat(fullname, &fbuf)) == -1) {
+    if ((err = gretl_stat(fullname, &fbuf)) == -1) {
 	if (errno == ENOENT) {
-#ifdef G_OS_WIN32
-	    strcpy(status, _("Not installed"));
+#ifdef G_OS_WIN32 /* FIXME? */
+	    *status = N_("Not installed");
 #else
 	    /* try user dir too, if not on Windows */
 	    if (role == REMOTE_DB) {
 		build_path(fullname, paths.workdir, fname, NULL);
+	    } else if (role == REMOTE_DATA_PKGS) {
+		build_path(fullname, paths.workdir, fname, NULL);
 	    } else {
-		build_path(fndir, paths.workdir, "functions", NULL);
-		build_path(fullname, fndir, fname, NULL);
+		build_path(filedir, paths.workdir, "functions", NULL);
+		build_path(fullname, filedir, fname, NULL);
 	    }
-	    err = stat(fullname, &fbuf);
+	    err = gretl_stat(fullname, &fbuf);
 	    if (err == -1) {
-		if (role == REMOTE_FUNC_FILES) {
+		if (role == REMOTE_FUNC_FILES || role == REMOTE_DATA_PKGS) {
 		    /* try default working dir */
 		    char *tmp = gretl_default_workdir(&paths);
 
 		    if (tmp != NULL) {
-			build_path(fndir, tmp, "functions", NULL);
-			build_path(fullname, fndir, fname, NULL);
+			if (role == REMOTE_FUNC_FILES) {
+			    build_path(filedir, tmp, "functions", NULL);
+			    build_path(fullname, filedir, fname, NULL);
+			} else {
+			    build_path(fullname, tmp, fname, NULL);
+			}
 			err = stat(fullname, &fbuf);
 			free(tmp);
 		    }
-		}
+		} 
 		if (err == -1) {
-		    strcpy(status, _("Not installed"));
+		    *status = N_("Not installed");
 		}
 	    } 
 #endif
 	} else {
-	    strcpy(status, _("Unknown: access error"));
+	    *status = N_("Unknown: access error");
 	}
     }
 
     if (!err) {
-	if (difftime(remtime, fbuf.st_ctime) > 360) {
-	    strcpy(status, _("Not up to date"));
+	double dt;
+
+	dt = difftime(remtime, fbuf.st_ctime);
+	if (dt > 360) {
+	    dt = difftime(remtime, fbuf.st_mtime);
+	}
+	if (dt > 360) {
+	    *status = N_("Not up to date");
 	} else {
-	    strcpy(status, _("Up to date"));
+	    *status = N_("Up to date");
 	}
     }
 }
 
-static int read_remote_filetime (char *line, char *fname, time_t *date)
+static int read_remote_filetime (char *line, char *fname, time_t *date,
+				 char *buf)
 {
-    char mon[4], hrs[9];
-    int mday, yr;
-    struct tm mytime;
+    char month[4], hrs[9];
+    int mday, yr, mon = 0;
     const char *months[] = {
 	"Jan", "Feb", "Mar", "Apr",
 	"May", "Jun", "Jul", "Aug",
@@ -2201,29 +2272,35 @@ static int read_remote_filetime (char *line, char *fname, time_t *date)
     */
 
     if (sscanf(line, "%*s%*s%3s%2d%8s%4d%31s", 
-	       mon, &mday, hrs, &yr, fname) != 5) {
+	       month, &mday, hrs, &yr, fname) != 5) {
 	return 1;
     }
 
-    hrs[2] = 0;
-
-    mytime.tm_sec = 0;
-    mytime.tm_min = 0;   
-    mytime.tm_wday = 0;   
-    mytime.tm_yday = 0;   
-    mytime.tm_isdst = -1; 
-    mytime.tm_hour = atoi(hrs);
-    mytime.tm_year = yr - 1900;
-    mytime.tm_mday = mday;
-    mytime.tm_mon = 0;
-
     for (i=0; i<12; i++) {
-	if (strcmp(mon, months[i]) == 0) {
-	    mytime.tm_mon = i;
+	if (!strcmp(month, months[i])) {
+	    mon = i;
 	}
-    }
+    }    
 
-    *date = mktime(&mytime);
+    if (buf != NULL) {
+	sprintf(buf, "%d-%02d-%02d", yr, mon + 1, mday);
+    } else {
+	struct tm mytime;
+
+	hrs[2] = 0;
+
+	mytime.tm_sec = 0;
+	mytime.tm_min = 0;   
+	mytime.tm_wday = 0;   
+	mytime.tm_yday = 0;   
+	mytime.tm_isdst = -1; 
+	mytime.tm_hour = atoi(hrs);
+	mytime.tm_year = yr - 1900;
+	mytime.tm_mday = mday;
+	mytime.tm_mon = mon;
+
+	*date = mktime(&mytime);
+    }
 
     return 0;
 }
@@ -2302,8 +2379,9 @@ gint populate_remote_db_list (windata_t *vwin)
     GtkTreeIter iter, child_iter; 
     char *getbuf = NULL;
     char line[1024];
-    char fname[32], status[20];
+    char fname[32];
     char src[96], srcbak[96];
+    const char *status;
     gchar *row[3];
     time_t remtime;
     int start, parent, kids;
@@ -2328,7 +2406,7 @@ gint populate_remote_db_list (windata_t *vwin)
 	if (strstr(line, "idx")) {
 	    continue;
 	}
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 	if (bufgets(line, sizeof line, getbuf)) {
@@ -2372,13 +2450,15 @@ gint populate_remote_db_list (windata_t *vwin)
 	if (strstr(line, "idx")) {
 	    continue;
 	}
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 
-	get_local_object_status(fname, vwin->role, status, remtime);
+	status = "";
+	get_local_object_status(fname, vwin->role, &status, remtime);
 	row[0] = strip_extension(fname);
-	row[2] = status;
+	row[1] = NULL;
+	row[2] = _(status);
 
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
@@ -2390,9 +2470,7 @@ gint populate_remote_db_list (windata_t *vwin)
 		parent = 1;
 		kids = ndb;
 	    }	
-	} else {
-	    row[1] = NULL;
-	}
+	} 
 
 	if (parent) {
 	    /* header for child databases */
@@ -2439,7 +2517,8 @@ gint populate_remote_func_list (windata_t *vwin)
     GtkTreeIter iter;  
     char *getbuf = NULL;
     char line[1024];
-    char fname[32], status[20];
+    char fname[32];
+    const char *status;
     char *basename;
     char *descrip;
     char *version;
@@ -2465,11 +2544,12 @@ gint populate_remote_func_list (windata_t *vwin)
 	descrip = NULL;
 	version = NULL;
 
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 
-	get_local_object_status(fname, vwin->role, status, remtime);
+	status = "";
+	get_local_object_status(fname, vwin->role, &status, remtime);
 	basename = strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
@@ -2493,7 +2573,7 @@ gint populate_remote_func_list (windata_t *vwin)
 			   0, basename, 
 			   1, version,
 			   2, descrip, 
-			   3, status,
+			   3, _(status),
 			   -1);
 
 	free(descrip);
@@ -2515,6 +2595,88 @@ gint populate_remote_func_list (windata_t *vwin)
     return err;
 }
 
+/* Fill a list box with names and short descriptions
+   of data file packages, retrieved from server.
+*/
+
+gint populate_remote_data_pkg_list (windata_t *vwin)
+{
+    GtkListStore *store;
+    GtkTreeIter iter;  
+    char *getbuf = NULL;
+    char line[256];
+    char fname[32];
+    char tstr[16];
+    const char *status;
+    char *basename;
+    char *descrip;
+    int n, err = 0;
+
+    err = list_remote_data_packages(&getbuf);
+
+    if (err) {
+	show_network_error(NULL);
+	free(getbuf);
+	return err;
+    }
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			   (GTK_TREE_VIEW(vwin->listbox)));
+    gtk_list_store_clear(store);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
+    n = 0;
+    bufgets_init(getbuf);
+
+    while (bufgets(line, sizeof line, getbuf)) {
+	descrip = NULL;
+	*tstr = '\0';
+
+	if (read_remote_filetime(line, fname, NULL, tstr)) {
+	    continue;
+	}
+
+	status = "";
+	basename = strip_extension(fname);
+
+	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
+	    utf8_correct(line);
+	    descrip = gretl_strdup(line + 2);
+	} 
+
+	if (descrip == NULL) {
+	    descrip = gretl_strdup("");
+	}
+
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 
+			   0, basename, 
+			   1, descrip, 
+			   2, tstr,
+			   -1);
+
+	free(descrip);
+	n++;
+    }
+
+    bufgets_finalize(getbuf);
+    free(getbuf);
+
+    if (n == 0) {
+	warnbox(_("No data packages found"));
+	err = 1;
+    }
+
+#if 0
+    if (!err) {
+	funcpkg_drag_connect(vwin);
+    }
+#endif
+
+    return err;
+}
+
 static DIR *open_gretl_db_dir (char *dbdir)
 {
     DIR *dir = NULL;
