@@ -33,6 +33,7 @@
 #include "selector.h"
 #include "toolbar.h"
 #include "winstack.h"
+#include "fileselect.h"
 
 #include "gretl_xml.h"
 #include "gretl_func.h"
@@ -93,7 +94,14 @@ struct fpkg_response {
     int try_server;
 };
 
-#define REMOTE_ACTION(c) (c == REMOTE_DB || c == REMOTE_FUNC_FILES)
+#define REMOTE_ACTION(c) (c == REMOTE_DB || \
+                          c == REMOTE_FUNC_FILES || \
+                          c == REMOTE_DATA_PKGS)
+
+static void
+read_fn_files_in_dir (int role, DIR *dir, const char *path, 
+		      GtkListStore *store, GtkTreeIter *iter,
+		      int *nfn, int *maxlen);
 
 static void 
 fpkg_response_init (struct fpkg_response *f, gpointer p)
@@ -565,13 +573,25 @@ static int build_file_collections (void)
 
 char *strip_extension (char *s)
 {
-    char *p = strrchr(s, '.');
+    char *p = strstr(s, ".tar.gz");
+
+    if (p != NULL) {
+	char *q = strstr(s, "_data");
+
+	if (q != NULL) {
+	    *q = '\0';
+	} else {
+	    *p = '\0';
+	}
+    } else {
+	p = strrchr(s, '.');
     
-    if (p != NULL && 
-	(!strcmp(p, ".gdt") || !strcmp(p, ".inp") ||
-	 !strcmp(p, ".bin") || !strcmp(p, ".gfn") ||
-	 !strcmp(p, ".bn7"))) {
-	*p = '\0';
+	if (p != NULL && 
+	    (!strcmp(p, ".gdt") || !strcmp(p, ".inp") ||
+	     !strcmp(p, ".bin") || !strcmp(p, ".gfn") ||
+	     !strcmp(p, ".bn7"))) {
+	    *p = '\0';
+	}
     }
 
     return s;
@@ -675,7 +695,7 @@ void browser_open_data (GtkWidget *w, gpointer data)
     set_datapage(coll->title);
 
     verify_open_data(vwin, OPEN_DATA);
-} 
+}
 
 void browser_open_ps (GtkWidget *w, gpointer data)
 {
@@ -698,6 +718,21 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     view_file(scriptfile, 0, 0, 78, 370, VIEW_SCRIPT);
 } 
 
+static gint enter_opens_file (GtkWidget *w, GdkEventKey *key, 
+			      windata_t *vwin)
+{
+    if (key->keyval == GDK_Return || key->keyval == GDK_o) {
+	if (vwin->role == TEXTBOOK_DATA) {
+	    browser_open_data(w, vwin);
+	} else if (vwin->role == PS_FILES) {
+	    browser_open_ps(w, vwin);
+	}
+	return TRUE;
+    } else {
+	return FALSE;
+    }
+}
+
 enum {
     VIEW_FN_PKG_INFO,
     VIEW_FN_PKG_CODE,
@@ -728,7 +763,8 @@ static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
         return 0;
     }
 
-    err = remove(fname);
+    err = gretl_remove(fname);
+
     if (err) {
 	file_write_errbox(fname);
     } else {
@@ -786,6 +822,31 @@ windata_t *gui_show_function_info (const char *fname, int role)
     return vwin;
 }
 
+void set_funcs_dir_callback (windata_t *vwin, char *path)
+{
+    DIR *dir = opendir(path);
+
+    if (dir != NULL) {
+	GtkListStore *store;
+	GtkTreeIter iter;
+	int maxlen = 0;
+	int nfn = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->listbox), "nfn"));
+	int nfn0 = nfn;	
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	read_fn_files_in_dir(vwin->role, dir, path, store, &iter,
+			     &nfn, &maxlen);
+	if (nfn > nfn0) {
+	    infobox("Found %d additional function file(s)", nfn - nfn0);
+	    g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
+	} else {
+	    warnbox("No additional function files were found");
+	}
+	closedir(dir);
+    }
+}
+
 static void browser_functions_handler (windata_t *vwin, int task)
 {
     char fnfile[FILENAME_MAX];
@@ -1032,6 +1093,16 @@ static void build_db_popup (windata_t *vwin)
     }
 }
 
+static void build_data_pkg_popup (windata_t *vwin)
+{
+    if (vwin->popup == NULL) {
+	vwin->popup = gtk_menu_new();
+	add_popup_item(_("Install"), vwin->popup, 
+		       G_CALLBACK(install_file_from_server), 
+		       vwin);
+    }
+}
+
 static void show_server_dbs (GtkWidget *w, gpointer p)
 {
     display_files(REMOTE_DB, p);
@@ -1047,11 +1118,22 @@ static void show_server_funcs (GtkWidget *w, gpointer p)
     display_files(REMOTE_FUNC_FILES, p);
 }
 
+static void show_server_data_pkgs (GtkWidget *w, gpointer p)
+{
+    display_files(REMOTE_DATA_PKGS, p);
+}
+
 static void show_local_funcs (GtkWidget *w, gpointer p)
 {
     display_files(FUNC_FILES, p);
 }
 
+static void alt_funcs_dir (GtkWidget *w, windata_t *vwin)
+{
+    file_selector_with_parent(NULL, SET_FDIR, FSEL_DATA_VWIN, vwin, 
+                              vwin->main);
+}
+
 enum {
     BTN_EDIT = 1,
     BTN_INFO,
@@ -1065,11 +1147,17 @@ enum {
     BTN_NEW,
     BTN_FIND,
     BTN_OPEN,
+    BTN_DIR,
     BTN_CLOSE
 };
 
 static GretlToolItem files_items[] = {
     { N_("Open"),           GTK_STOCK_OK,         NULL,                          BTN_OPEN },
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
+    { N_("Select directory"), GTK_STOCK_OPEN,  G_CALLBACK(alt_funcs_dir),   BTN_DIR },
+#else
+    { N_("Select directory"), GTK_STOCK_DIRECTORY,  G_CALLBACK(alt_funcs_dir),   BTN_DIR },
+#endif
     { N_("Edit"),           GTK_STOCK_EDIT,       G_CALLBACK(browser_edit_func), BTN_EDIT },
     { N_("Info"),           GTK_STOCK_INFO,       NULL,                          BTN_INFO },
     { N_("View code"),      GTK_STOCK_PROPERTIES, G_CALLBACK(show_function_code), BTN_CODE },
@@ -1088,7 +1176,7 @@ static int n_files_items = G_N_ELEMENTS(files_items);
 
 #define common_item(f) (f == BTN_FIND || f == BTN_CLOSE)
 
-#define local_funcs_item(f) (f == BTN_EDIT || f == BTN_NEW || \
+#define local_funcs_item(f) (f == BTN_DIR || f == BTN_EDIT || f == BTN_NEW || \
 			     f == BTN_DEL || f == BTN_CODE)
 
 static int files_item_get_callback (GretlToolItem *item, int role)
@@ -1098,7 +1186,9 @@ static int files_item_get_callback (GretlToolItem *item, int role)
     } else if (local_funcs_item(item->flag)) {
 	return (role == FUNC_FILES);
     } else if (item->flag == BTN_INST) {
-	return (role == REMOTE_DB || role == REMOTE_FUNC_FILES);
+	return (role == REMOTE_DB || 
+		role == REMOTE_FUNC_FILES ||
+		role == REMOTE_DATA_PKGS);
     } else if (item->flag == BTN_EXEC) {
 	return (role == FUNC_FILES || role == REMOTE_FUNC_FILES);
     }
@@ -1132,7 +1222,9 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 	    item->func = G_CALLBACK(show_server_funcs);
 	} else if (role == NATIVE_DB) {
 	    item->func = G_CALLBACK(show_server_dbs);
-	} 
+	} else if (role == TEXTBOOK_DATA) {
+	    item->func = G_CALLBACK(show_server_data_pkgs);
+	}
     } else if (item->flag == BTN_HOME) {
 	/* home: show only for on-server items */
 	if (role == REMOTE_FUNC_FILES) {
@@ -1250,7 +1342,7 @@ static void set_up_viewer_drag_target (windata_t *vwin)
 		      &gretl_drag_targets[i], 1,
 		      GDK_ACTION_COPY);
 
-    g_signal_connect(G_OBJECT(vwin->listbox), "drag_data_received",
+    g_signal_connect(G_OBJECT(vwin->listbox), "drag-data-received",
 		     callback, NULL);
 }
 
@@ -1277,6 +1369,8 @@ void display_files (int code, gpointer p)
 	title = g_strdup(_("gretl: databases on server"));
     } else if (code == REMOTE_FUNC_FILES) {
 	title = g_strdup(_("gretl: function packages on server"));
+    } else if (code == REMOTE_DATA_PKGS) {
+	title = g_strdup(_("gretl: data packages on server"));
     }
 
     vwin = gretl_browser_new(code, title, 0);
@@ -1317,19 +1411,24 @@ void display_files (int code, gpointer p)
 
 	build_datafiles_popup(vwin);
 	while ((coll = pop_data_collection())) {
-	    g_signal_connect(G_OBJECT(coll->page), "button_press_event",
+	    g_signal_connect(G_OBJECT(coll->page), "button-press-event",
 			     G_CALLBACK(popup_menu_handler), 
 			     vwin->popup);
 	}
 	reset_data_stack();
     } else if (code == FUNC_FILES || code == REMOTE_FUNC_FILES) {
 	build_funcfiles_popup(vwin);
-	g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 			 G_CALLBACK(popup_menu_handler), 
 			 vwin->popup);
     } else if (code == NATIVE_DB || code == REMOTE_DB) {
 	build_db_popup(vwin);
-	g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
+			 G_CALLBACK(popup_menu_handler), 
+			 vwin->popup);
+    } else if (code == REMOTE_DATA_PKGS)  {
+	build_data_pkg_popup(vwin);
+	g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 			 G_CALLBACK(popup_menu_handler), 
 			 vwin->popup);
     }	
@@ -1368,7 +1467,7 @@ void display_files (int code, gpointer p)
 	gtk_widget_grab_focus(vwin->listbox);
 	if (code == NATIVE_DB || code == FUNC_FILES) {
 	    set_up_viewer_drag_target(vwin);
-	}
+	} 
     }
 
     if (err && code == FUNC_FILES && fresp.try_server == 1) {
@@ -1420,10 +1519,48 @@ static int get_func_info (const char *fname, const char *fndir,
     return err;
 }
 
-static int
-read_fn_files_in_dir (int role, DIR *dir, const char *fndir, 
+static int fn_file_is_duplicate (const char *fname, 
+				 const char *version,
+				 GtkListStore *store,
+				 int imax)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gchar *fname_i;
+    gchar *version_i;
+    int i, n, ret = 0;
+
+    model = GTK_TREE_MODEL(store);
+
+    if (!gtk_tree_model_get_iter_first(model, &iter)) {
+	return 0;
+    }
+
+    n = strlen(fname) - 4;
+
+    for (i=0; i<imax; i++) {
+	gtk_tree_model_get(model, &iter, 
+			   0, &fname_i, 
+			   1, &version_i,
+			   -1);
+	if (strncmp(fname, fname_i, n) == 0 &&
+	    strcmp(version, version_i) == 0) {
+	    ret = 1;
+	} 
+	g_free(fname_i);
+	g_free(version_i);
+	if (ret || !gtk_tree_model_iter_next(model, &iter)) {
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+static void
+read_fn_files_in_dir (int role, DIR *dir, const char *path, 
 		      GtkListStore *store, GtkTreeIter *iter,
-		      int *maxlen)
+		      int *nfn, int *maxlen)
 {
     struct dirent *dirent;
     char fullname[MAXLEN];
@@ -1431,7 +1568,7 @@ read_fn_files_in_dir (int role, DIR *dir, const char *fndir,
     char *fname;
     char *descrip;
     char *version;
-    int n, nfn = 0;
+    int n, imax = *nfn;
     int err;
 
     while ((dirent = readdir(dir)) != NULL) {
@@ -1445,31 +1582,31 @@ read_fn_files_in_dir (int role, DIR *dir, const char *fndir,
 	}
 	n = strlen(fname);
 	if (!g_ascii_strcasecmp(fname + n - 4, ".gfn")) {
-	    err = get_func_info(fname, fndir, &descrip, &version);
+	    err = get_func_info(fname, path, &descrip, &version);
 	    if (!err) {
-		gtk_list_store_append(store, iter);
-		build_path(fullname, fndir, fname, NULL);
-		n -= 4;
-		fname[n] = '\0';
-		if (n > *maxlen) {
-		    *maxlen = n;
+		if (!fn_file_is_duplicate(fname, version, store, imax)) {
+		    gtk_list_store_append(store, iter);
+		    build_path(fullname, path, fname, NULL);
+		    n -= 4;
+		    fname[n] = '\0';
+		    if (n > *maxlen) {
+			*maxlen = n;
+		    }
+		    loaded = function_package_is_loaded(fullname);
+		    gtk_list_store_set(store, iter, 
+				       0, fname, 
+				       1, version,
+				       2, descrip, 
+				       3, loaded, 
+				       4, path, -1);
+		    *nfn += 1;
 		}
-		loaded = function_package_is_loaded(fullname);
-		gtk_list_store_set(store, iter, 
-				   0, fname, 
-				   1, version,
-				   2, descrip, 
-				   3, loaded, 
-				   4, fndir, -1);
 		free(descrip);
 		free(version);
-		nfn++;
 	    }
 	}
 	g_free(fname);
     }
-
-    return nfn;
 }
 
 static gint 
@@ -1523,17 +1660,26 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     build_path(fndir, paths.gretldir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
-	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-				    &maxlen);
+	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+			     &nfn, &maxlen);
 	closedir(dir);
     }
 
     /* plus any function files in the user's working dir */
+    strcpy(fndir, paths.workdir);
+    dir = opendir(fndir);
+    if (dir != NULL) {
+	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+			     &nfn, &maxlen);
+	closedir(dir);
+    }    
+
+    /* plus any function files under "functions" in the user's working dir */
     build_path(fndir, paths.workdir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
-	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-				    &maxlen);
+	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+			     &nfn, &maxlen);
 	closedir(dir);
     }
 
@@ -1541,8 +1687,8 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     build_path(fndir, paths.dotdir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
-	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-				    &maxlen);
+	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+			     &nfn, &maxlen);
 	closedir(dir);
     } 
 
@@ -1552,8 +1698,8 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 	build_path(fndir, tmp, "functions", NULL);
 	dir = opendir(fndir);
 	if (dir != NULL) {
-	    nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-				    &maxlen);
+	    read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+				 &nfn, &maxlen);
 	    closedir(dir);
 	}
 	free(tmp);
@@ -1575,6 +1721,7 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 	}
 	return 1;
     } else {
+	g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
 	sort_pkglist(vwin);
     }
 
@@ -1642,6 +1789,8 @@ gint populate_filelist (windata_t *vwin, gpointer p)
 	return populate_remote_db_list(vwin);
     } else if (vwin->role == REMOTE_FUNC_FILES) {
 	return populate_remote_func_list(vwin);
+    } else if (vwin->role == REMOTE_DATA_PKGS) {
+	return populate_remote_data_pkg_list(vwin);
     } else if (vwin->role == FUNC_FILES) {
 	return populate_func_list(vwin, p);
     } else {
@@ -1655,6 +1804,11 @@ static GtkWidget *files_window (windata_t *vwin)
 	N_("File"), 
 	N_("Summary")
     };
+    const char *remote_data_titles[] = {
+	N_("File"), 
+	N_("Source"),
+	N_("Date")
+    };
     const char *ps_titles[] = {
 	N_("Script"), 
 	N_("Topic"), 
@@ -1727,6 +1881,11 @@ static GtkWidget *files_window (windata_t *vwin)
 	full_width = 580;
 	use_tree = 1;
 	break;
+    case REMOTE_DATA_PKGS:
+	titles = remote_data_titles;
+	cols = 3;
+	full_width = 600;
+	break;
     case PS_FILES:
 	titles = ps_titles;
 	cols = 3;
@@ -1827,6 +1986,8 @@ static GtkWidget *files_notebook (windata_t *vwin, int code)
 	sprintf(wnum, "%d", j);
 	g_object_set_data(G_OBJECT(notebook), wnum, coll->page);
 	g_object_set_data(G_OBJECT(coll->page), "coll", coll);
+	g_signal_connect(G_OBJECT(coll->page), "key-press-event",
+			 G_CALLBACK(enter_opens_file), vwin);
 	j++;
     }
 
