@@ -68,7 +68,8 @@ enum {
     DW_N_PRIME    = 1 << 2,
     DW_VLIST_DONE = 1 << 3,
     DW_NP_DONE    = 1 << 4,
-    DW_NO_PANEL   = 1 << 5
+    DW_NO_PANEL   = 1 << 5,
+    DW_SSHEET     = 1 << 6
 };
 
 #define dw_n_is_prime(o) (o->flags & DW_N_PRIME)
@@ -77,7 +78,7 @@ enum {
 typedef struct dw_opts_ dw_opts;
 
 struct dw_opts_ {
-    int flags;          /* state bit-flags */
+    gretlopt flags;     /* state bit-flags */
     int n_radios;       /* number of radio-button options */
     int deflt;          /* default setting for current radio variable */ 
     int plf;            /* panel: least factor > 1 of # of observations */
@@ -163,22 +164,15 @@ static void prep_spreadsheet (GtkWidget *widget, dialog_t *dlg)
 static void maybe_start_editing (void)
 {
     int cancel = 0;
-    int resp;
 
-    resp = yes_no_dialog(_("gretl: new dataset"), 
-			 _("Do you want to start entering data values\n"
-			 "using gretl's spreadsheet?"), 0);
+    edit_dialog(_("gretl: name variable"), 
+		_("Enter name for first variable\n"
+		  "(max. 15 characters)"),
+		NULL, prep_spreadsheet, NULL, 
+		CREATE_DATASET, VARCLICK_NONE, 
+		&cancel);
 
-    if (resp == GRETL_YES) {
-	edit_dialog(_("gretl: name variable"), 
-		    _("Enter name for first variable\n"
-		      "(max. 15 characters)"),
-		    NULL, prep_spreadsheet, NULL, 
-		    CREATE_DATASET, VARCLICK_NONE, 
-		    &cancel);
-    } 
-
-    if (resp == GRETL_NO || cancel) {
+    if (cancel) {
 	/* accept the default blank dataset */
 	register_data(NULLDATA_STARTED);
     }	
@@ -249,15 +243,19 @@ static void maybe_unrestrict_dataset (void)
 
 /* respond to the "Apply" button in the wizard */
 
-static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
+static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts,
+			      GtkWidget *dlg)
 {
     char setline[32];
+    char record[128];
     gretlopt opt = OPT_NONE;
     int create = (opts->flags & DW_CREATE);
     int delmiss = (opts->flags & DW_DROPMISS);
     int delete_markers = 0;
     int err = 0;
 
+    *record = '\0';
+
     /* preliminaries */
     if (time_series(dwinfo)) {
 	ntodate_full(dwinfo->stobs, dwinfo->t1, dwinfo);
@@ -277,6 +275,10 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
 	if (!err) {
 	    err = set_panel_structure_from_vars(uv, tv, Z, datainfo);
 	}
+	if (!err) {
+	    sprintf(record, "setobs %s %s --panel-vars",
+		    datainfo->varname[uv], datainfo->varname[tv]);
+	}
 	goto finalize;
     }
 
@@ -285,6 +287,7 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
 	dwinfo->pd == datainfo->pd &&
 	strcmp(dwinfo->stobs, datainfo->stobs) == 0) {
 	if (create || delmiss) {
+	    /* recording? */
 	    goto finalize;
 	} else {
 	    infobox(_("No changes were made"));
@@ -329,16 +332,17 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
 	opt = OPT_N;
     }
 
-#if DWDEBUG
-    fprintf(stderr, "setline = '%s', opt = %ld\n", setline, opt);
-#endif
-
-    err = set_obs(setline, Z, datainfo, opt);
+    err = set_obs(setline, &Z, datainfo, opt);
 
 #if DWDEBUG
-    fprintf(stderr, "set_obs returned %d\n", err);
+    fprintf(stderr, "setline = '%s', opt = %ld; set_obs returned %d\n", 
+	    setline, opt, err);
 #endif
 
+    if (!err) {
+	sprintf(record, "%s%s", setline, print_flags(opt, SETOBS));
+    }
+
  finalize:
 
     if (!err && delmiss) {
@@ -348,7 +352,8 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
     if (err) {
 	gui_errmsg(err);
     } else if (create) {
-	if (datainfo->n < 1001) {
+	if (opts->flags & DW_SSHEET) {
+	    gtk_widget_hide(dlg);
 	    maybe_start_editing();
 	} else {
 	    register_data(NULLDATA_STARTED);
@@ -360,6 +365,10 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts)
 	mark_dataset_as_modified();
     }
 
+    if (!err && *record != '\0') {
+	record_command_line(record);
+    }
+
 #if DWDEBUG
     fprintf(stderr, "dwiz_make_changes: returning %d\n", err);
 #endif
@@ -531,7 +540,7 @@ static void dwiz_set_radio_opt (GtkWidget *w, dw_opts *opts)
     *opts->setvar = val;
 }
 
-static void make_confirmation_text (char *ctxt, DATAINFO *dwinfo, int *flags)
+static void make_confirmation_text (char *ctxt, DATAINFO *dwinfo, gretlopt *flags)
 {
     if (dwinfo->structure == CROSS_SECTION) {
 	sprintf(ctxt, _("%s, observations 1 to %d"), _("Cross-sectional data"), 
@@ -1259,16 +1268,7 @@ static void dwiz_make_panel_spinners (dw_opts *opts,
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);
 }
 
-static void set_purge_missobs (GtkWidget *w, int *flags)
-{
-    if (GTK_TOGGLE_BUTTON(w)->active) {
-	*flags |= DW_DROPMISS;
-    } else {
-	*flags &= ~DW_DROPMISS;
-    }    
-}
-
-static void maybe_add_missobs_purger (GtkWidget *vbox, int *flags)
+static void maybe_add_missobs_purger (GtkWidget *vbox, gretlopt *flags)
 {
     double missfrac = 0.0;
     int active = 0;
@@ -1282,16 +1282,15 @@ static void maybe_add_missobs_purger (GtkWidget *vbox, int *flags)
 
     if (active || (missfrac > 0 && missfrac < 0.12)) {
 	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
-	GtkWidget *chk = gtk_check_button_new_with_label
-	    (N_("purge missing observations"));
+	GtkWidget *chk;
 
+	chk = gretl_option_check_button(_("purge missing observations"),
+					flags, DW_DROPMISS);
 	gtk_box_pack_start(GTK_BOX(hbox), chk, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 	if (active) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk), TRUE);
 	}
-	g_signal_connect(G_OBJECT(chk), "toggled", 
-			 G_CALLBACK(set_purge_missobs), flags);
 	gtk_widget_show_all(hbox);
     }
 }
@@ -1577,6 +1576,17 @@ static void dwiz_button_visibility (GtkWidget *dlg, int step)
     }
 }
 
+static void add_editing_option (GtkWidget *vbox, gretlopt *flags)
+{
+    GtkWidget *hbox, *b;
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    b = gretl_option_check_button(_("start entering data values"),
+				  flags, DW_SSHEET);
+    gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);    
+}
+
 /* build the appropriate notebook page for the given step */
 
 static void dwiz_prepare_page (GtkNotebook *nb,
@@ -1598,6 +1608,9 @@ static void dwiz_prepare_page (GtkNotebook *nb,
 
 	make_confirmation_text(ctxt, dwinfo, &opts->flags);
 	gtk_label_set_text(GTK_LABEL(w), ctxt);
+	if ((opts->flags & DW_CREATE) && datainfo->n < 1001) {
+	    add_editing_option(page, &opts->flags);
+	}
     } else {
 	/* all other pages */
 	set_up_dw_opts(opts, step, dwinfo);
@@ -1629,7 +1642,7 @@ static void dwiz_finalize (GtkWidget *dlg, DATAINFO *dwinfo,
     dw_opts *opts = g_object_get_data(G_OBJECT(dlg), "opts");
 
     if (!cancel) {
-	dwiz_make_changes(dwinfo, opts);
+	dwiz_make_changes(dwinfo, opts, dlg);
     } else if (opts->flags & DW_CREATE) {
 	/* aborting creation of new dataset */
 	gui_clear_dataset();
@@ -1862,7 +1875,9 @@ static void data_structure_wizard (int create)
 
     for (i=0; i<=DW_CONFIRM; i++) {
 	n = g_utf8_strlen(_(wizcode_string(i)), -1);
-	if (n > smax) smax = n;
+	if (n > smax) {
+	    smax = n;
+	}
     }
 
     /* make all the notebook pages */
