@@ -107,7 +107,8 @@ static void set_canceled (GtkWidget *w, int *c)
 
 static void maybe_set_canceled (GtkDialog *d, int resp, int *c)
 {
-    if (resp == GTK_RESPONSE_DELETE_EVENT) {
+    if (resp == GTK_RESPONSE_DELETE_EVENT ||
+	resp == GTK_RESPONSE_CANCEL) {
 	*c = 1;
     }
 }
@@ -144,7 +145,9 @@ static void invalidate_opt (GtkWidget *w, int *opt)
 
 static void maybe_invalidate_opt (GtkDialog *d, int resp, int *opt)
 {
-    if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
+    if (resp == GTK_RESPONSE_NONE || 
+	resp == GTK_RESPONSE_DELETE_EVENT ||
+	resp == GTK_RESPONSE_CANCEL) {
 	*opt = -1;
     }
 }
@@ -245,6 +248,7 @@ static void gretl_dialog_set_resizeable (GtkWidget *w, gboolean s)
 }
 
 static GtkWidget *current_dialog;
+static GtkWidget *open_edit_dialog;
 
 int maybe_raise_dialog (void)
 {
@@ -253,7 +257,10 @@ int maybe_raise_dialog (void)
     if (current_dialog != NULL) {
 	gtk_window_present(GTK_WINDOW(current_dialog));
 	ret = 1;
-    }
+    } else if (open_edit_dialog != NULL) {
+	gtk_window_present(GTK_WINDOW(open_edit_dialog));
+	ret = 1;
+    }	
 
     return ret;
 }
@@ -282,6 +289,11 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 	gtk_window_set_title(GTK_WINDOW(d), title);
     }
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7)
+    g_signal_connect(G_OBJECT(d), "key-press-event", 
+		     G_CALLBACK(esc_kills_window), NULL);
+#endif
+
     gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(d)->action_area), 
 			      GTK_BUTTONBOX_END);
     set_dialog_border_widths(d);
@@ -319,6 +331,98 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
     return d;
 }
 
+/* make the sensitivity of widget @w positively dependent on the state
+   of check button @check */
+
+void sensitize_widget_from_check (GtkWidget *check, GtkWidget *w)
+{
+    gboolean s;
+
+    s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+    gtk_widget_set_sensitive(w, s);
+}
+
+/* make the sensitivity of widget @w negatively dependent on the state
+   of check button @check */
+
+void desensitize_widget_from_check (GtkWidget *check, GtkWidget *w)
+{
+    gboolean s;
+
+    s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+    gtk_widget_set_sensitive(w, !s);
+}
+
+void set_double_from_spinner (GtkSpinButton *b, double *x)
+{
+    *x = gtk_spin_button_get_value(b);
+}
+
+void set_int_from_spinner (GtkSpinButton *b, int *k)
+{
+    *k = gtk_spin_button_get_value_as_int(b);
+}
+
+static void toggle_gretl_option (GtkToggleButton *b, gretlopt *popt)
+{
+    gretlopt val = 
+	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(b), "optval"));
+
+    if (gtk_toggle_button_get_active(b)) {
+	*popt |= val;
+    } else {
+	*popt &= ~val;
+    }
+}
+
+/* Create a check button which sets @val in @popt when
+   activated, and unsets the flag when deactivated */
+
+GtkWidget *gretl_option_check_button (const char *label,
+				      gretlopt *popt,
+				      gretlopt val)
+{
+    GtkWidget *button;
+
+    button = gtk_check_button_new_with_label(label);
+    g_object_set_data(G_OBJECT(button), "optval",
+		      GINT_TO_POINTER(val));
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(toggle_gretl_option), popt); 
+
+    return button;
+}
+
+static void toggle_gretl_option_switched (GtkToggleButton *b, gretlopt *popt)
+{
+    gretlopt val = 
+	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(b), "optval"));
+
+    if (gtk_toggle_button_get_active(b)) {
+	*popt &= ~val;
+    } else {
+	*popt |= val;
+    }
+}
+
+/* Create a check button which unsets @val in @popt when
+   activated, and sets the flag when deactivated */
+
+GtkWidget *gretl_option_check_button_switched (const char *label,
+					       gretlopt *popt,
+					       gretlopt val)
+{
+    GtkWidget *button;
+
+    button = gtk_check_button_new_with_label(label);
+    g_object_set_data(G_OBJECT(button), "optval",
+		      GINT_TO_POINTER(val));
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(toggle_gretl_option_switched), popt); 
+
+    return button;
+}
+
 /* "edit dialog" apparatus */
 
 GtkWidget *active_edit_id;
@@ -335,8 +439,6 @@ struct dialog_t_ {
     gretlopt opt;
 };
 
-static GtkWidget *open_edit_dialog;
-
 static void destroy_dialog_data (GtkWidget *w, gpointer data) 
 {
     dialog_t *d = (dialog_t *) data;
@@ -356,7 +458,9 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
 
 static void cancel_on_delete (GtkDialog *d, int resp, int *c)
 {
-    if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
+    if (resp == GTK_RESPONSE_NONE || 
+	resp == GTK_RESPONSE_DELETE_EVENT ||
+	resp == GTK_RESPONSE_CANCEL) {
 	*c = 1;
     }
 }
@@ -365,8 +469,14 @@ gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key,
 			   gpointer unused)
 {
     if (key->keyval == GDK_Escape) { 
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7
+	/* remedial action for old gtk */
+	if (GTK_IS_DIALOG(w)) {
+	    gtk_dialog_response(GTK_DIALOG(w), GTK_RESPONSE_CANCEL);
+	}
+#endif
         gtk_widget_destroy(w);
-	return TRUE; /* handled! */
+	return TRUE;
     } else {
 	return FALSE;
     }
@@ -389,7 +499,7 @@ dialog_data_new (gpointer p, gint code, const char *title,
     d->popup = NULL;
 
     if (canceled != NULL) {
-	g_signal_connect(G_OBJECT(d->dialog), "delete_event",
+	g_signal_connect(G_OBJECT(d->dialog), "delete-event",
 			 G_CALLBACK(cancel_on_delete), 
 			 canceled);
 	d->blocking = 1;
@@ -405,7 +515,7 @@ dialog_data_new (gpointer p, gint code, const char *title,
 
     g_signal_connect(G_OBJECT(d->dialog), "destroy", 
 		     G_CALLBACK(destroy_dialog_data), d);
-    g_signal_connect(G_OBJECT(d->dialog), "key_press_event", 
+    g_signal_connect(G_OBJECT(d->dialog), "key-press-event", 
 		     G_CALLBACK(esc_kills_window), NULL);
 
     return d;
@@ -489,7 +599,7 @@ gchar *edit_dialog_special_get_text (dialog_t *dlg)
 
     if (buf == NULL) {
 	gtk_widget_destroy(dlg->dialog);
-    } 
+    }
 
     set_edit_save_buf(buf, dlg->code);
 
@@ -779,86 +889,32 @@ static void set_sys_method (GtkComboBox *box, dialog_t *d)
     g_free(str);
 }
 
-static gboolean opt_t_callback (GtkWidget *w, dialog_t *dlg)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	dlg->opt |= OPT_T;
-    } else {
-	dlg->opt &= ~OPT_T;
-    }
-
-    return FALSE;
-}
-
-static gboolean opt_v_callback (GtkWidget *w, dialog_t *dlg)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	dlg->opt |= OPT_V;
-    } else {
-	dlg->opt &= ~OPT_V;
-    }
-
-    return FALSE;
-}
-
-static gboolean opt_r_callback (GtkWidget *w, dialog_t *dlg)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	dlg->opt |= OPT_R;
-    } else {
-	dlg->opt &= ~OPT_R;
-    }
-
-    return FALSE;
-}
-
-static gboolean opt_b_callback (GtkWidget *w, dialog_t *dlg)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	dlg->opt |= OPT_B;
-    } else {
-	dlg->opt &= ~OPT_B;
-    }
-
-    return FALSE;
-}
-
-static gboolean opt_f_callback (GtkWidget *w, dialog_t *dlg)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	dlg->opt |= OPT_F;
-    } else {
-	dlg->opt &= ~OPT_F;
-    }
-
-    return FALSE;
-}
-
 static GtkWidget *dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
-					gretlopt opt)
+					gretlopt opt, MODEL *pmod)
 {
     GtkWidget *b = NULL;
 
     if (opt == OPT_T) {
-	b = gtk_check_button_new_with_label(_("Iterated estimation"));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(opt_t_callback), dlg);
+	b = gretl_option_check_button(_("Iterated estimation"),
+				      &dlg->opt, OPT_T); /* OPT_T vs OPT_I?? */
+	if (pmod != NULL && (pmod->opt & OPT_I)) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
+	}
     } else if (opt == OPT_V) {
-	b = gtk_check_button_new_with_label(_("Show details of iterations"));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(opt_v_callback), dlg);
+	b = gretl_option_check_button(_("Show details of iterations"),
+				      &dlg->opt, OPT_V);
     } else if (opt == OPT_R) {
-	b = gtk_check_button_new_with_label(_("Robust standard errors"));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(opt_r_callback), dlg);
+	b = gretl_option_check_button(_("Robust standard errors"),
+				      &dlg->opt, OPT_R);
+	if (pmod != NULL && (pmod->opt & OPT_R)) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
+	}
     } else if (opt == OPT_B) {
-	b = gtk_check_button_new_with_label(_("Use bootstrap"));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(opt_b_callback), dlg);
+	b = gretl_option_check_button(_("Use bootstrap"),
+				      &dlg->opt, OPT_B);
     } else if (opt == OPT_F) {
-	b = gtk_check_button_new_with_label(_("Show full restricted estimates"));
-	g_signal_connect(G_OBJECT(b), "toggled", 
-			 G_CALLBACK(opt_f_callback), dlg);
+	b = gretl_option_check_button(_("Show full restricted estimates"),
+				      &dlg->opt, OPT_F);
     } 
 
     if (b != NULL) {
@@ -919,7 +975,7 @@ GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt)
     return gretl_opts_combo_full(opts, deflt, NULL, NULL);
 }
 
-static void build_gmm_combo (GtkWidget *vbox, dialog_t *d)
+static void build_gmm_combo (GtkWidget *vbox, dialog_t *d, MODEL *pmod)
 {
     GtkWidget *combo, *hbox;
     static const char *strs[] = {
@@ -940,6 +996,14 @@ static void build_gmm_combo (GtkWidget *vbox, dialog_t *d)
     gmm_opts.vals = opts;
     gmm_opts.optp = &d->opt;
 
+    if (pmod != NULL) {
+	if (pmod->opt & OPT_T) {
+	    deflt = 1;
+	} else if (pmod->opt & OPT_I) {
+	    deflt = 2;
+	}
+    }
+
     combo = gretl_opts_combo(&gmm_opts, deflt);
 
     hbox = gtk_hbox_new(FALSE, 5);
@@ -1000,13 +1064,16 @@ static void dlg_display_sys (dialog_t *d)
     GtkWidget *w;
     int hsize = 62;
 
-    w = gtk_label_new(sys->name);
-    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
-		       w, TRUE, TRUE, 10);
-    gtk_widget_show(w);
+    if (d->code == ESTIMATE) {
+	/* estimating an existing system, not editing */
+	w = gtk_label_new(sys->name);
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
+			   w, TRUE, TRUE, 10);
+	gtk_widget_show(w);
+    }
 
-    d->edit = dlg_text_edit_new(&hsize, FALSE);
+    d->edit = dlg_text_edit_new(&hsize, d->code == SYSTEM);
     dialog_table_setup(d, hsize);
     dlg_text_set_from_sys(sys, d); 
     gretl_dialog_set_resizeable(d->dialog, TRUE);
@@ -1026,9 +1093,8 @@ static int edit_dialog_help_code (int ci, void *p)
 {
     int hc = ci;
 
-    if (ci == SYSTEM && p != NULL) {
-	hc = 0;
-    } else if (ci == PRINT || ci == CREATE_DATASET || ci == MINIBUF) {
+    if (ci == ESTIMATE || ci == CREATE_DATASET ||
+	ci == PRINT || ci == MINIBUF) {
 	hc = 0;
     } else if (ci == RESTRICT) {
 	windata_t *vwin = (windata_t *) p;
@@ -1083,23 +1149,36 @@ static int vecm_model_window (windata_t *vwin)
     return vwin->role == VECM;
 }
 
+static void edit_dialog_add_note (const char *s, GtkWidget *vbox)
+{
+    GtkWidget *w;
+    gchar *lbl;
+
+    lbl = g_strdup_printf("%s\n%s", s, _("(Please refer to Help for guidance)"));
+    w = gtk_label_new(lbl);
+    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
+    gtk_box_pack_start(GTK_BOX(vbox), w, TRUE, TRUE, 10);
+    gtk_widget_show(w);
+    g_free(lbl);
+}
+
 void edit_dialog (const char *title, const char *info, const char *deflt, 
-		  void (*okfunc)(), void *okptr,
-		  guint cmdcode, guint varclick, 
-		  int *canceled)
+		  void (*okfunc)(), void *okptr, guint ci, 
+		  guint varclick, int *canceled)
 {
     dialog_t *d;
     GtkWidget *w;
     GtkWidget *top_vbox, *button_box;
+    MODEL *pmod = NULL;
     int hlpcode, modal = 0;
     int clear = 0;
 
-    if (open_edit_dialog != NULL && cmdcode != MINIBUF) {
+    if (open_edit_dialog != NULL && ci != MINIBUF) {
 	gtk_window_present(GTK_WINDOW(open_edit_dialog));
 	return;
     }
 
-    d = dialog_data_new(okptr, cmdcode, title, canceled);
+    d = dialog_data_new(okptr, ci, title, canceled);
     if (d == NULL) return;
 
     open_edit_dialog = d->dialog;
@@ -1112,44 +1191,45 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
 			      GTK_BUTTONBOX_END);
 
-    if (cmdcode == SYSTEM && d->data != NULL) {
-	/* revisiting saved equation system */
+    if (ci == ESTIMATE) {
+	/* estimating saved equation system */
 	dlg_display_sys(d);
-    } else if (cmdcode == NLS || cmdcode == MLE || 
-	       cmdcode == SYSTEM || cmdcode == RESTRICT ||
-	       cmdcode == GMM) {
+    } else if (ci == SYSTEM && d->data != NULL) {
+	/* repecifying equation system */
+	edit_dialog_add_note(info, top_vbox);
+	dlg_display_sys(d);
+	clear = 1;
+    } else if (ci == NLS || ci == MLE || ci == GMM ||
+	       ci == SYSTEM || ci == RESTRICT) {
 	int hsize = 62;
-	gchar *lbl;
-
-	lbl = g_strdup_printf("%s\n%s", info,
-			      _("(Please refer to Help for guidance)"));
-	w = gtk_label_new(lbl);
-	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
-	gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 10);
-	gtk_widget_show(w);
-	g_free(lbl);
 
+	edit_dialog_add_note(info, top_vbox);
 	d->edit = dlg_text_edit_new(&hsize, TRUE);
 	dialog_table_setup(d, hsize);
 	gretl_dialog_set_resizeable(d->dialog, TRUE);
 
-	/* insert previous text, if any and if the command
-	   is the same as previously -- or insert skeleton
-	   of command
-	*/
-	if (dlg_text_set_previous(d) ||
-	    dlg_text_set_gmm_skel(d)) {
+	if (ci != RESTRICT && deflt != NULL) {
+	    /* re-specifying a nonlinear model */
+	    textview_set_text(d->edit, deflt);
+	    if (ci == NLS || ci == MLE || ci == GMM) {
+		pmod = okptr;
+	    }
+	} else if (dlg_text_set_previous(d) ||
+		   dlg_text_set_gmm_skel(d)) {
+	    /* insert previous text, if any and if the command
+	       is the same as previously -- or insert skeleton
+	       of command
+	    */
 	    clear = 1;
 	}
 
-	if (cmdcode != RESTRICT && cmdcode != GMM) {
-	    g_signal_connect(G_OBJECT(d->edit), "button_press_event", 
+	if (ci != RESTRICT && ci != GMM) {
+	    g_signal_connect(G_OBJECT(d->edit), "button-press-event", 
 			     G_CALLBACK(edit_dialog_popup_handler), d);
 	}
     } else {
 	if (info != NULL) {
 	    w = gtk_label_new(info);
-	    /* GTK_JUSTIFY_CENTER? */
 	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
 	    gtk_widget_show(w);
@@ -1175,24 +1255,24 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 			 G_CALLBACK(raise_and_focus_dialog), d);
     }
 
-    if (cmdcode == SMPLBOOL && dataset_is_restricted()) {
+    if (ci == SMPLBOOL && dataset_is_restricted()) {
 	sample_replace_buttons(top_vbox, d);
-    } else if (cmdcode == SYSTEM) {
+    } else if (ci == SYSTEM || ci == ESTIMATE) {
 	GtkWidget *bt, *bv;
 
-	bt = dialog_option_switch(top_vbox, d, OPT_T);
-	bv = dialog_option_switch(top_vbox, d, OPT_V);
+	bt = dialog_option_switch(top_vbox, d, OPT_T, NULL);
+	bv = dialog_option_switch(top_vbox, d, OPT_V, NULL);
 	system_estimator_list(top_vbox, d, bt, bv);
-    } else if (cmdcode == NLS || cmdcode == MLE) {
-	dialog_option_switch(top_vbox, d, OPT_V);
-	dialog_option_switch(top_vbox, d, OPT_R);
-    } else if (cmdcode == GMM) {
-	dialog_option_switch(top_vbox, d, OPT_V);
-	build_gmm_combo(top_vbox, d);
-    } else if (cmdcode == RESTRICT && ols_model_window(okptr)) {
-	dialog_option_switch(top_vbox, d, OPT_B);
-    } else if (cmdcode == RESTRICT && vecm_model_window(okptr)) {
-	dialog_option_switch(top_vbox, d, OPT_F);
+    } else if (ci == NLS || ci == MLE) {
+	dialog_option_switch(top_vbox, d, OPT_V, pmod);
+	dialog_option_switch(top_vbox, d, OPT_R, pmod);
+    } else if (ci == GMM) {
+	dialog_option_switch(top_vbox, d, OPT_V, pmod);
+	build_gmm_combo(top_vbox, d, pmod);
+    } else if (ci == RESTRICT && ols_model_window(okptr)) {
+	dialog_option_switch(top_vbox, d, OPT_B, NULL);
+    } else if (ci == RESTRICT && vecm_model_window(okptr)) {
+	dialog_option_switch(top_vbox, d, OPT_F, NULL);
     }
 
     if (varclick == VARCLICK_INSERT_ID) { 
@@ -1201,7 +1281,7 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	active_edit_name = d->edit;
     } else if (varclick == VARCLICK_INSERT_TEXT) { 
 	active_edit_text = d->edit;
-    } else if (cmdcode != SYSTEM) {
+    } else if (ci != ESTIMATE) {
 	modal = 1;
     }
 
@@ -1233,32 +1313,32 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     gtk_widget_show(w);    
 
     /* "Help" button, if wanted */
-    hlpcode = edit_dialog_help_code(cmdcode, okptr);
+    hlpcode = edit_dialog_help_code(ci, okptr);
     if (hlpcode > 0) {
 	context_help_button(button_box, hlpcode);
 	modal = 0;
     }
 
-    /* schedule destruction */
-    if (cmdcode == MODEL_GENR || cmdcode == RESTRICT) {
+    if (ci == MODEL_GENR || ci == RESTRICT) {
 	windata_t *vwin = (windata_t *) okptr;
 
+	/* schedule destruction */
 	g_signal_connect(G_OBJECT(vwin->main), "destroy",
 			 G_CALLBACK(cancel_vwin_edit), 
 			 NULL);
-    }
-
-    gtk_window_set_destroy_with_parent(GTK_WINDOW(d->dialog), TRUE);
-
-    gtk_widget_show(d->dialog); 
+    } 
 
     if (modal) {
-	gretl_set_window_modal(d->dialog);
-    }
+	g_signal_connect(G_OBJECT(d->dialog), "show", 
+			 G_CALLBACK(gretl_set_window_modal), NULL);
+    }    
 
     if (d->blocking) {
-	gtk_main();
+	g_signal_connect(G_OBJECT(d->dialog), "show", 
+			 G_CALLBACK(gtk_main), NULL);
     }
+
+    gtk_widget_show(d->dialog); 
 }
 
 char *entry_box_get_trimmed_text (GtkWidget *w)
