@@ -15,21 +15,19 @@ void delete_from_filelist (int filetype, const char *fname);
 
 void add_files_to_menus (void);
 
-#if defined(USE_GNOME)
+void trim_homedir (char *fname);
 
-# include <gconf/gconf-client.h>
+void rc_save_file_lists (FILE *fp);
 
-void save_file_lists (GConfClient *client);
-void read_file_lists (GConfClient *client);
+GList *get_working_dir_list (void);
 
-#elif defined(G_OS_WIN32)
+#ifdef G_OS_WIN32
 
 void save_file_lists (void);
 void read_file_lists (void);
 
 #else
 
-void save_file_lists (FILE *fp);
 void read_file_lists (FILE *fp, char *prev);
 
 #endif
