@@ -313,19 +313,6 @@ enum {
     HIDX_SWITCH
 };
 
-const char *print_today (void)
-{
-    static char timestr[16];
-    struct tm *local;
-    time_t t;
-
-    t = time(NULL);
-    local = localtime(&t);
-    strftime(timestr, 15, "%Y-%m-%d", local);
-
-    return timestr;
-}
-
 static gboolean update_iface (GtkComboBox *menu, 
 			      function_info *finfo)
 {
@@ -1251,7 +1238,7 @@ void prepare_functions_save (void)
 {
     function_info *finfo;
     int *list = NULL;
-    int i;
+    int i, err = 0;
 
     if (storelist == NULL) {
 	return;
@@ -1262,9 +1249,9 @@ void prepare_functions_save (void)
 	return;
     }
 
-    list = gretl_list_from_string(storelist);
-    if (list == NULL) {
-	nomem();
+    list = gretl_list_from_string(storelist, &err);
+    if (err) {
+	gui_errmsg(err);
 	free(finfo);
 	return;
     }
