@@ -32,4 +32,8 @@ int no_user_functions_check (void);
 
 void get_default_package_name (char *fname, gpointer p, int mode);
 
+void update_sample_script (windata_t *vwin);
+
+void fnsave_set_script (const char *fname, gpointer p);
+
 #endif /* FNSAVE_H */
