@@ -53,6 +53,10 @@
 # include "gretlwin32.h"
 #endif
 
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6
+# define OLD_GTK
+#endif
+
 /* update.c */
 extern int silent_update_query (void);
 extern int update_query (void); 
@@ -88,32 +92,114 @@ mdata_handle_drag  (GtkWidget          *widget,
 		    guint               time,
 		    gpointer            p);
 
-#ifdef USE_GNOME
-static char *optrun, *optdb;
-static int opteng, optbasque, optdump;
+static char *optrun, *optdb, *optwebdb;
+static int opteng, optbasque, optdump, optver;
+#ifdef G_OS_WIN32
+static int optdebug;
+#endif
 
-static const struct poptOption options[] = {
-    { "run", 'r', POPT_ARG_STRING, &optrun, 0, 
-      N_("open a script file on startup"), "SCRIPT" },
-    { "db", 'd', POPT_ARG_STRING, &optdb, 0, 
-      N_("open a database on startup"), "DATABASE" },
-    { "webdb", 'w', POPT_ARG_STRING, &optdb, 0, 
-      N_("open a remote (web) database on startup"), "REMOTE_DB" },
-    { "english", 'e', POPT_ARG_NONE, &opteng, 0, 
-      N_("force use of English"), NULL },
-    { "basque", 'q', POPT_ARG_NONE, &optbasque, 0, 
-      N_("force use of Basque"), NULL },
-    { "dump", 'c', POPT_ARG_NONE, &optdump, 0, 
-      N_("dump gretl configuration to file"), NULL },
-    { NULL, '\0', 0, NULL, 0, NULL, NULL },
+static gchar *param_msg = 
+    N_("\nYou may supply the name of a data file on the command line");
+
+#ifdef OLD_GTK
+
+struct start_opts {
+    const char *longopt;
+    char shortopt;
+    const char *optstr;
+    const char *filetype;
 };
-#endif /* USE_GNOME */
+
+static struct start_opts options[] = {
+    { "run",     'r', N_("open a script file on startup"), "SCRIPT" },
+    { "db",      'd', N_("open a database on startup"),    "DATABASE"},
+    { "webdb",   'w', N_("open a remote (web) database on startup"), "REMOTE_DB" },
+    { "english", 'e', N_("force use of English"), NULL },
+    { "basque",  'q', N_("force use of Basque"), NULL },
+    { "dump",    'c', N_("dump gretl configuration to file"), NULL },
+    { "version", 'v', N_("print version information"), NULL }, 
+    { NULL, 0, NULL, NULL },
+};
+
+static void gui_help (void) 
+{
+    int i;
+
+    putchar('\n');
+
+    for (i=0; options[i].longopt != NULL; i++) {
+	if (options[i].filetype != NULL) {
+	    printf("gretl --%s %s : %s\n", options[i].longopt, options[i].filetype, 
+		   _(options[i].optstr));
+	} else {
+	    printf("gretl --%s : %s\n", options[i].longopt, _(options[i].optstr));
+	}
+    }
+
+    printf("%s\n", _(param_msg));
+}
+
+static void old_gretl_init (int *pargc, char ***pargv)
+{
+    static char fname[FILENAME_MAX];
+    int force_lang = 0;
+    gretlopt opt;
+
+    opt = parseopt(pargc, pargv, fname, &force_lang);
+
+    if (force_lang == LANG_C) {
+	opteng = 1;
+    } else if (force_lang == LANG_EU) {
+	optbasque = 1;
+    }
+
+    if (opt & (OPT_HELP | OPT_VERSION | OPT_ERROR)) {
+	gui_logo(NULL);
+	if (opt != OPT_VERSION) {
+	    gui_help();
+	}
+	exit(EXIT_SUCCESS);
+    } else if (opt & OPT_DUMP) {
+	optdump = 1;
+    } else if (opt & OPT_RUNIT) {
+	optrun = fname;
+    } else if (opt & OPT_DBOPEN) {
+	optdb = fname;
+    } else if (opt & OPT_WEBDB) {
+	optwebdb = fname;
+    }
+}
+
+#else
+
+static GOptionEntry options[] = {
+    { "run", 'r', 0, G_OPTION_ARG_FILENAME, &optrun, 
+      N_("open a script file on startup"), "SCRIPT" },
+    { "db", 'd', 0, G_OPTION_ARG_STRING, &optdb, 
+      N_("open a database on startup"), "DATABASE" },
+    { "webdb", 'w', 0, G_OPTION_ARG_STRING, &optwebdb, 
+      N_("open a remote (web) database on startup"), "REMOTE_DB" },
+    { "english", 'e', 0, G_OPTION_ARG_NONE, &opteng, 
+      N_("force use of English"), NULL },
+    { "basque", 'q', 0, G_OPTION_ARG_NONE, &optbasque, 
+      N_("force use of Basque"), NULL },
+    { "dump", 'c', 0, G_OPTION_ARG_NONE, &optdump, 
+      N_("dump gretl configuration to file"), NULL },
+#ifdef G_OS_WIN32
+    { "debug", 'b', 0, G_OPTION_ARG_NONE, &optdebug, 
+      N_("send debugging info to console"), NULL }, 
+#endif
+    { "version", 'v', 0, G_OPTION_ARG_NONE, &optver, 
+      N_("print version information"), NULL }, 
+    { NULL, '\0', 0, 0, NULL, NULL, NULL },
+};
+
+#endif
 
 windata_t *mdata;
 DATAINFO *datainfo;
 
 char scriptfile[MAXLEN];
-char sessionfile[MAXLEN];
 char tryfile[MAXLEN];
 
 PATHS paths;                /* useful paths */
@@ -182,25 +268,6 @@ static void email_data (gpointer p, guint u, GtkWidget *w)
 }
 #endif
 
-
-static void gui_usage (void)
-{
-    gui_logo(NULL);
-
-    printf(I_("\nYou may supply the name of a data file on the command line.\n"
-	      "Options:\n"
-	      " -c or --dump      Dump the configuration file.\n"
-	      " -d or --db        Open a local database.\n"
-	      " -e or --english   Force use of English rather than translation.\n"
-	      " -q or --basque    Force use of Basque translation.\n"
-	      " -r or --run       Open a command script.\n"
-	      " -w or --webdb     Open a remote database.\n"
-	      " -h or --help      Print this info and exit.\n"
-	      " -v or --version   Print version info and exit.\n"));
-
-    exit(EXIT_SUCCESS);
-}
-
 static void noalloc (void)
 {
     fputs(I_("Out of memory!\n"), stderr);
@@ -212,7 +279,9 @@ static void get_runfile (char *fname)
     *tryfile = '\0';
 
 #ifdef G_OS_WIN32
-    if (unmangle(fname, tryfile)) return;
+    if (filename_to_win32(tryfile, fname)) {
+	return;
+    }
 #else
     strncat(tryfile, fname, MAXLEN - 1);
 #endif
@@ -290,6 +359,7 @@ static void real_nls_init (void)
 	strcpy(p, "share/locale");
     }
 
+    /* FIXME GUI language choice? */
     loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
     bindtextdomain(PACKAGE, localedir);
@@ -324,38 +394,10 @@ void nls_init (void)
     real_nls_init();
 }
 
-void force_language (int lang)
-{
-    if (lang == ENGLISH) {
-	putenv("LANGUAGE=english");
-	setlocale(LC_ALL, "C");
-    } else if (lang == BASQUE) {
-# ifdef G_OS_WIN32
-	setlocale(LC_ALL, "eu");
-# else
-	setlocale(LC_ALL, "eu_ES");
-# endif
-    }
-
-# ifdef G_OS_WIN32
-    if (lang == ENGLISH) {
-	SetEnvironmentVariable("LC_ALL", "C");
-	putenv("LC_ALL=C");
-	textdomain("none");
-    } else if (lang == BASQUE) {
-	SetEnvironmentVariable("LC_ALL", "eu");
-	putenv("LC_ALL=eu");
-    }
-# endif
-
-    if (lang == ENGLISH) {
-	force_english_help();
-    }
-}
-
 #endif /* ENABLE_NLS */
 
 #ifndef G_OS_WIN32
+
 static void record_filearg (char *targ, const char *src)
 {
     if (*src == '.') {
@@ -369,6 +411,7 @@ static void record_filearg (char *targ, const char *src)
 	strcpy(targ, src);
     }
 }
+
 #endif
 
 static int have_data (void)
@@ -376,18 +419,13 @@ static int have_data (void)
     return datainfo != NULL && datainfo->v > 0;
 }
 
-int main (int argc, char *argv[])
+int main (int argc, char **argv)
 {
-    int open_datafile = 0;
     int ftype = 0;
     char dbname[MAXLEN];
     char filearg[MAXLEN];
-    int opt = 0;
-#ifdef USE_GNOME
-    GnomeProgram *program;
-#endif
-#ifdef G_OS_WIN32
-    int debug = 0;
+#ifndef OLD_GTK
+    GError *opterr = NULL;
 #endif
     int err = 0;
 
@@ -399,92 +437,53 @@ int main (int argc, char *argv[])
     *scriptfile = '\0';
     *paths.datfile = '\0';
     *dbname = '\0';
+    *filearg = '\0';
 
-    /* Initialize gnome or GTK */
-#ifdef USE_GNOME
-    program = gnome_program_init ("gretl", GRETL_VERSION,
-				  LIBGNOMEUI_MODULE, argc, argv,
-				  GNOME_PARAM_POPT_TABLE, options,
-				  GNOME_PARAM_HUMAN_READABLE_NAME,
-				  _("The GNOME 2.0 econometrics package"),
-				  GNOME_PARAM_APP_DATADIR, DATADIR,
-				  GNOME_PARAM_NONE);
-#else
+#ifdef OLD_GTK
     gtk_init(&argc, &argv);
-#endif /* USE_GNOME */
+    old_gretl_init(&argc, &argv);
+#else
+    gtk_init_with_args(&argc, &argv, param_msg, options, "gretl", &opterr);
+    if (opterr != NULL) {
+	g_print("%s\n", opterr->message);
+	exit(EXIT_FAILURE);
+    }
+#endif
 
     libgretl_init();
     gretl_set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
 
 #ifdef G_OS_WIN32
-    gretl_win32_init(argv[0]);
+    gretl_win32_init(argv[0], optdebug);
 #else 
     gretl_config_init();
 #endif
 
-    set_workdir_callback(gui_set_working_dir);
-
-    if (argc > 1) {
-	int force_lang = 0;
-
-	opt = parseopt((const char **) argv, argc, filearg, &force_lang);
-
-#ifdef G_OS_WIN32
-	if (opt & OPT_DEBUG) {
-	    debug = 1;
-	}
-#endif
-
-	switch (opt) {
-	case OPT_HELP:
-	    gui_usage();
-	    break;
-	case OPT_VERSION:
-	    gui_logo(NULL);
-	    exit(EXIT_SUCCESS);
-	    break;
-	case OPT_RUNIT:
-#ifdef USE_GNOME
-	    get_runfile(optrun);
-#else
-	    if (*filearg == '\0') {
-		gui_usage();
-	    }
-	    get_runfile(filearg);
-#endif
-	    break;
-	case OPT_DBOPEN:
-	case OPT_WEBDB:
-#ifdef USE_GNOME
-	    strncpy(dbname, optdb, MAXLEN - 1);
-#else
-	    if (*filearg == '\0') {
-		gui_usage();
-	    }
-	    strcpy(dbname, filearg);
-#endif
-	    if (opt == OPT_DBOPEN) {
-		fix_dbname(dbname);
-	    }
-	    break;
-	case OPT_DUMP:
-	    dump_rc();
-	    exit(EXIT_SUCCESS);
-	    break;
-	default:
-	    /* let's suppose any string argument is a data file */
-	    if (*filearg != '\0') {
-		open_datafile = 1;
-	    }
-	    break;
-	}
+    if (optver) {
+	gui_logo(NULL);
+	exit(EXIT_SUCCESS);
+    } else if (optdump) {
+	dump_rc();
+	exit(EXIT_SUCCESS);
+    } else if (optrun) {
+	get_runfile(optrun);
+    } else if (optdb != NULL) {
+	strncat(dbname, optdb, MAXLEN - 1);
+	fix_dbname(dbname);
+    } else if (optwebdb != NULL) {
+	strncat(dbname, optdb, MAXLEN - 1);
+    } 
 
 #ifdef ENABLE_NLS
-	if (force_lang) {
-	    force_language(force_lang);
-	}
+    if (opteng) {
+	force_language(LANG_C);
+	force_english_help();
+    } else if (optbasque) {
+	force_language(LANG_EU);
+    }
 #endif
-    } 
+
+    set_workdir_callback(gui_set_working_dir);
 
     /* allocate data information struct */
     datainfo = datainfo_new();
@@ -504,17 +503,22 @@ int main (int argc, char *argv[])
     session_init();
     init_fileptrs();
 
-    /* get the data file, if specified on the command line */
-    if (open_datafile) {
+    if (argc > 1) {
+	/* Process what is presumably a filename argument
+	   given on the command line (by now any options will
+	   have been extracted from the argv array).
+	*/
 	PRN *prn; 
 
 	prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
 	if (err) exit(EXIT_FAILURE);
 
+	strncat(filearg, argv[1], MAXLEN - 1);
+
 	*paths.datfile = '\0';
 
 #ifdef G_OS_WIN32
-	if (unmangle(filearg, paths.datfile)) {
+	if (filename_to_win32(paths.datfile, filearg)) {
 	    exit(EXIT_FAILURE);
 	}
 #else
@@ -543,6 +547,7 @@ int main (int argc, char *argv[])
 	case GRETL_GNUMERIC:
 	case GRETL_ODS:
 	case GRETL_DTA:
+	case GRETL_SAV:
 	case GRETL_JMULTI:
 	case GRETL_OCTAVE:
 	case GRETL_WF1:
@@ -560,11 +565,11 @@ int main (int argc, char *argv[])
 	    *tryfile = '\0';
 	    *paths.datfile = '\0';
 	    fix_dbname(dbname);
-	    opt = OPT_DBOPEN;
+	    optdb = dbname;
 	    break;
 	case GRETL_UNRECOGNIZED:
-	    gui_usage();
 	default:
+	    fprintf(stderr, "%s: unrecognized file type", tryfile);
 	    exit(EXIT_FAILURE);
 	    break;
 	}
@@ -600,16 +605,16 @@ int main (int argc, char *argv[])
     restore_sample_state(FALSE);
     main_menubar_state(FALSE);
 
-    /* run init script, if found? */
+    /* FIXME run init script, if found? */
 
     if (have_data()) {
 	register_startup_data(tryfile);
 	maybe_display_string_table();
-	*tryfile = 0;
+	*tryfile = '\0';
     }
 
     /* opening a script or session from the command line? */
-    if (tryfile[0] != '\0') { 
+    if (*tryfile != '\0') { 
 	if (gretl_is_pkzip_file(tryfile)) {
 	    ftype = GRETL_SESSION;
 	}
@@ -626,23 +631,12 @@ int main (int argc, char *argv[])
     }
 
     /* try opening specified database */
-    if (opt == OPT_DBOPEN) {
+    if (optdb != NULL) {
 	open_named_db_index(dbname);
-    } else if (opt == OPT_WEBDB) {
+    } else if (optwebdb != NULL) {
 	open_named_remote_db_index(dbname);
     }
 
-#ifdef G_OS_WIN32
-    if (debug) {
-	char *loc;
-
-	gretl_win32_debug();
-	loc = setlocale(LC_ALL, "");
-	fprintf(stderr, "setlocale gives '%s'\n", 
-		(loc != NULL)? loc : "NULL");
-    }
-#endif
-
     /* Enter the event loop */
     gtk_main();
 
@@ -663,6 +657,7 @@ int main (int argc, char *argv[])
     }
 
     destroy_file_collections();
+    free_command_stack();
 
     return EXIT_SUCCESS;
 }
@@ -681,49 +676,74 @@ static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
     }
 }
 
+/* if a keystroke would take us to row 0 (e.g. page up), countermand
+   and got to row 1 instead
+*/
+
+static void mdata_avoid_zero (GtkTreeView *view, gpointer p)
+{
+    GtkTreePath *path = NULL;
+    int i;
+
+    gtk_tree_view_get_cursor(view, &path, NULL);
+
+    if (path != NULL) {
+	i = gtk_tree_path_get_indices(path)[0];
+	if (i == 0) {
+	    GtkTreePath *newp;
+
+	    newp = gtk_tree_path_new_from_indices(1, -1);
+	    gtk_tree_view_set_cursor(view, newp, NULL, FALSE);
+	    gtk_tree_path_free(newp);
+	}
+	gtk_tree_path_free(path);
+    }
+}
+
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
     GdkModifierType mods;
+    int k = key->keyval;
 
     gdk_window_get_pointer(w->window, NULL, NULL, &mods);
 
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
     if (!(mods & GDK_MOD1_MASK)) {
-	if (key->keyval == GDK_a) {
+	if (k == GDK_a) {
 	    audio_render_window(vwin, AUDIO_LISTBOX);
-	} else if (key->keyval == GDK_x) {
+	} else if (k == GDK_x) {
 	    stop_talking();
 	}
     }
 #endif
 
-    if (key->keyval == GDK_h || key->keyval == GDK_F1) {
+    if (k == GDK_h || k == GDK_F1) {
 	/* invoke help */
 	plain_text_cmdref(NULL);
 	return FALSE;
     }
 
-    if (key->keyval == GDK_g) {
+    if (k == GDK_g) {
 	/* invoke genr */
 	genr_callback();
 	return FALSE;
     }
 
-    if (key->keyval == GDK_r) {
+    if (k == GDK_r) {
 	refresh_data();
 	return FALSE;
     }
 
-    if (key->keyval == GDK_x && (mods & GDK_MOD1_MASK)) {
+    if (k == GDK_x && (mods & GDK_MOD1_MASK)) {
 	/* invoke command minibuffer */
 	minibuf_callback();
 	return FALSE;
     }
 
-    if (key->keyval == GDK_Return                        /* display variable(s) */
-	|| key->keyval == GDK_Delete                     /* delete variable(s) */
-	|| key->keyval == GDK_e || key->keyval == GDK_F2 /* edit variable's info */
-	|| key->keyval == GDK_t                          /* graph variable */
+    if (k == GDK_Return              /* display variable(s) */
+	|| k == GDK_Delete           /* delete variable(s) */
+	|| k == GDK_e || k == GDK_F2 /* edit variable's info */
+	|| k == GDK_t                /* graph variable */
 	) {
 	int selcount, vnum = 0;
 
@@ -731,26 +751,28 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 
 	if (selcount == 1 && vnum != 0) {
 	    mdata->active_var = vnum;
-	    if (key->keyval == GDK_e || key->keyval == GDK_F2) {
+	    if (k == GDK_e || k == GDK_F2) {
 		varinfo_dialog(mdata->active_var, 1);
-	    } else if (key->keyval == GDK_t) {
+	    } else if (k == GDK_t) {
 		do_graph_var(mdata->active_var);
-	    } else if (key->keyval == GDK_Return) {
+	    } else if (k == GDK_Return) {
 		display_var();
-	    } else if (key->keyval == GDK_Delete) {
+	    } else if (k == GDK_Delete) {
 		delete_single_var(mdata->active_var);
 	    }
 	} else if (selcount > 1) {
-	    if (key->keyval == GDK_Delete) {
+	    if (k == GDK_Delete) {
 		delete_selected_vars();
-	    } else if (key->keyval == GDK_Return) {
+	    } else if (k == GDK_Return) {
 		display_selected();
 	    }
 	}
     } 
 
-    /* suppress useless keystrokes */
-    if (key->keyval != GDK_Up && key->keyval != GDK_Down) {
+    /* suppress echo of useless keystrokes */
+    if (k != GDK_Up && k != GDK_Down &&
+	k != GDK_Page_Up && k != GDK_Page_Down &&
+	k != GDK_Home && k != GDK_End) {
 	return TRUE;
     }
 
@@ -817,16 +839,15 @@ static int get_line_pos (GtkTreeModel *mod)
 
 void populate_varlist (void)
 {
+    static gint check_connected;
+    static gint click_connected;
     GtkTreeStore *store;
     GtkTreeSelection *select;
     GtkTreeIter iter;    
-    char id[4];
+    char id[8];
     int pos = 0;
     int i;
 
-    static gint check_connected;
-    static gint click_connected;
-
     store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
 
     if (store != NULL) {
@@ -914,17 +935,20 @@ void populate_varlist (void)
 	g_signal_connect(G_OBJECT(select), "changed",
 			 G_CALLBACK(check_varmenu_state),
 			 mdata);
+	g_signal_connect(G_OBJECT(mdata->listbox), "cursor-changed",
+			 G_CALLBACK(mdata_avoid_zero),
+			 NULL);
 	check_connected = 1;
     }
 
     if (!click_connected) {
-	g_signal_connect(G_OBJECT(mdata->listbox), "button_press_event",
+	g_signal_connect(G_OBJECT(mdata->listbox), "button-press-event",
 			 G_CALLBACK(main_popup_handler), 
 			 mdata);
-	g_signal_connect(G_OBJECT(mdata->listbox), "button_press_event",
+	g_signal_connect(G_OBJECT(mdata->listbox), "button-press-event",
 			 G_CALLBACK(main_varclick),
 			 mdata);
-	g_signal_connect(G_OBJECT(mdata->listbox), "key_press_event",
+	g_signal_connect(G_OBJECT(mdata->listbox), "key-press-event",
 			 G_CALLBACK(catch_mdata_key),
 			 mdata);
 	click_connected = 1;
@@ -1095,7 +1119,8 @@ static void scale_main_window (void)
 static GtkWidget *make_main_window (void) 
 {
     GtkWidget *main_vbox;
-    GtkWidget *box, *dlabel, *align;
+    GtkWidget *box, *dlabel;
+    GtkWidget *align;
     const char *titles[] = {
 	N_("ID #"), 
 	N_("Variable name"), 
@@ -1121,19 +1146,15 @@ static GtkWidget *make_main_window (void)
 	scale_main_window();
     }
 
-#ifdef USE_GNOME
-    mdata->main = gnome_app_new("gretl", _("Econometrics program"));
-#else
     mdata->main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#endif
 
 #ifdef G_OS_WIN32
     set_up_windows_look();
 #endif
 
-    g_signal_connect(G_OBJECT(mdata->main), "configure_event",
+    g_signal_connect(G_OBJECT(mdata->main), "configure-event",
 		     G_CALLBACK(mainwin_config), NULL);
-    g_signal_connect(G_OBJECT(mdata->main), "delete_event",
+    g_signal_connect(G_OBJECT(mdata->main), "delete-event",
 		     G_CALLBACK(exit_check), NULL);
     g_signal_connect(G_OBJECT(mdata->main), "destroy",
 		     G_CALLBACK(gtk_main_quit), NULL);
@@ -1149,47 +1170,41 @@ static GtkWidget *make_main_window (void)
 
     main_vbox = gtk_vbox_new(FALSE, 4);
     gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 8);
-#ifdef USE_GNOME
-    gnome_app_set_contents(GNOME_APP(mdata->main), main_vbox);
-#else
     gtk_container_add(GTK_CONTAINER(mdata->main), main_vbox);
-#endif
     g_object_set_data(G_OBJECT(mdata->main), "vbox", main_vbox);
 
     if (set_up_main_menu()) {
 	exit(EXIT_FAILURE);
     }
 
+    /* put the main menu bar in place */
     gtk_box_pack_start(GTK_BOX(main_vbox), mdata->mbar, FALSE, TRUE, 0);
-    gtk_widget_show(mdata->mbar);
 
+    /* label for name of datafile */
     dlabel = gtk_label_new(_(" No datafile loaded ")); 
-    gtk_widget_show(dlabel);
-
     g_object_set_data(G_OBJECT(mdata->main), "dlabel", dlabel);
 
+    /* will hold the list of variables */
     box = gtk_vbox_new(FALSE, 0);
+
     align = gtk_alignment_new(0, 0, 0, 0);
     gtk_box_pack_start(GTK_BOX(box), align, FALSE, FALSE, 0);
     gtk_widget_show(align);
     gtk_container_add(GTK_CONTAINER(align), dlabel);
    
     vwin_add_list_box(mdata, GTK_BOX(box), 3, FALSE, types, titles, 1);
-    gtk_widget_show(box);
 
     gtk_drag_dest_set(mdata->listbox,
 		      GTK_DEST_DEFAULT_ALL,
 		      gretl_drag_targets, 2,
 		      GDK_ACTION_COPY);
 
-    g_signal_connect(G_OBJECT(mdata->listbox), "drag_data_received",
+    g_signal_connect(G_OBJECT(mdata->listbox), "drag-data-received",
 		     G_CALLBACK(mdata_handle_drag),
 		     NULL);
 
     gtk_box_pack_start(GTK_BOX(main_vbox), box, TRUE, TRUE, 0);
-
     mdata->status = gtk_label_new("");
-    
     gtk_box_pack_start(GTK_BOX(main_vbox), mdata->status, FALSE, TRUE, 0);
 
     /* put stuff into list box, activate menus */
@@ -1201,9 +1216,7 @@ static GtkWidget *make_main_window (void)
     set_fixed_font();
 
     /* and a proportional font for menus, etc */
-#ifndef USE_GNOME
     set_app_font(NULL);
-#endif
 
     gtk_widget_show_all(mdata->main); 
 
@@ -1217,6 +1230,11 @@ static GtkWidget *make_main_window (void)
     return main_vbox;
 }
 
+static void iconview_callback (void)
+{
+    view_session(NULL);
+}
+
 GtkActionEntry main_entries[] = {
     /* File */
     { "File",         NULL, N_("_File"), NULL, NULL, NULL }, 
@@ -1232,6 +1250,7 @@ GtkActionEntry main_entries[] = {
     { "OpenODS",      NULL, N_("_Open Document..."), NULL, NULL, G_CALLBACK(open_data) },
     { "OpenWF1",      NULL, N_("_Eviews..."), NULL, NULL, G_CALLBACK(open_data) },
     { "OpenDTA",      NULL, N_("_Stata..."), NULL, NULL, G_CALLBACK(open_data) },
+    { "OpenSAV",      NULL, N_("_SPSS..."), NULL, NULL, G_CALLBACK(open_data) },
     { "OpenJMulTi",   NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(open_data) },
 
     { "AppendData",     NULL, N_("_Append data"), NULL, NULL, NULL }, 
@@ -1244,6 +1263,7 @@ GtkActionEntry main_entries[] = {
     { "AppendODS",      NULL, N_("_Open Document..."), NULL, NULL, G_CALLBACK(open_data) },
     { "AppendWF1",      NULL, N_("_Eviews..."), NULL, NULL, G_CALLBACK(open_data) },
     { "AppendDTA",      NULL, N_("_Stata..."), NULL, NULL, G_CALLBACK(open_data) },
+    { "AppendSAV",      NULL, N_("_SPSS..."), NULL, NULL, G_CALLBACK(open_data) },
     { "AppendJMulTi",   NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(open_data) },
 
     { "SaveData",  GTK_STOCK_SAVE, N_("_Save data"), "<control>S", NULL, G_CALLBACK(auto_store) },
@@ -1296,6 +1316,7 @@ GtkActionEntry main_entries[] = {
     { "StatsTables", NULL, N_("_Statistical tables"), NULL, NULL, G_CALLBACK(stats_calculator) },
     { "PValues", NULL, N_("_P-value finder"), NULL, NULL, G_CALLBACK(stats_calculator) },
     { "DistGraphs", NULL, N_("_Distribution graphs"), NULL, NULL, G_CALLBACK(stats_calculator) },
+    { "PlotCurve", NULL, N_("_Plot a curve"), NULL, NULL, G_CALLBACK(stats_calculator) },
     { "TestStats", NULL, N_("_Test statistic calculator"), NULL, NULL, G_CALLBACK(stats_calculator) },
     { "NonparamTests", NULL, N_("_Nonparametric tests"), NULL, NULL, G_CALLBACK(stats_calculator) },
     { "SetSeed", NULL, N_("_Seed for random numbers"), NULL, NULL, G_CALLBACK(rand_seed_dialog) },
@@ -1314,10 +1335,8 @@ GtkActionEntry main_entries[] = {
       G_CALLBACK(options_dialog_callback) },
     { "FixedFont", GTK_STOCK_SELECT_FONT, N_("_Fixed font..."), NULL, NULL, 
       G_CALLBACK(font_selector) },
-#if !defined(USE_GNOME)
     { "MenuFont", GTK_STOCK_SELECT_FONT, N_("_Menu font..."), NULL, NULL, 
       G_CALLBACK(font_selector) },
-#endif
 
     /* Data */
     { "Data", NULL, N_("_Data"), NULL, NULL, NULL },
@@ -1337,11 +1356,11 @@ GtkActionEntry main_entries[] = {
     { "DataTranspose", NULL, N_("_Transpose data..."), NULL, NULL, G_CALLBACK(gui_transpose_data) },
     { "DataSort", NULL, N_("_Sort data..."), NULL, NULL, G_CALLBACK(gui_sort_data) },
     { "DataRefresh", NULL, N_("_Refresh window"), NULL, NULL, G_CALLBACK(refresh_data) },
-    { "EditScalars", NULL, N_("_Scalars..."), NULL, NULL, G_CALLBACK(edit_scalars) },
 
     /* View */
     { "View", NULL, N_("_View"), NULL, NULL, NULL },
-    { "IconView", NULL, N_("_Icon view"), NULL, NULL, G_CALLBACK(view_session) },
+    { "IconView", NULL, N_("_Icon view"), NULL, NULL, G_CALLBACK(iconview_callback) },
+    { "EditScalars", NULL, N_("_Scalars"), NULL, NULL, G_CALLBACK(edit_scalars) },
     { "GraphVars", NULL, N_("_Graph specified vars"), NULL, NULL, NULL },
     { "TSPlot", NULL, N_("_Time series plot..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "ScatterPlot", NULL, N_("X-Y _scatter..."), NULL, NULL, G_CALLBACK(selector_callback) },
@@ -1363,15 +1382,15 @@ GtkActionEntry main_entries[] = {
 
     /* Add */
     { "Add", NULL, N_("_Add"), NULL, NULL, NULL },
-    { "logs", NULL, N_("_Logs of selected variables"), NULL, NULL, G_CALLBACK(add_logs_etc) },
-    { "square", NULL, N_("_Squares of selected variables"), NULL, NULL, G_CALLBACK(add_logs_etc) },
-    { "lags", NULL, N_("_Lags of selected variables"), NULL, NULL, G_CALLBACK(add_logs_etc) },
+    { "logs", NULL, N_("_Logs of selected variables"), NULL, NULL, G_CALLBACK(logs_etc_callback) },
+    { "square", NULL, N_("_Squares of selected variables"), NULL, NULL, G_CALLBACK(logs_etc_callback) },
+    { "lags", NULL, N_("_Lags of selected variables"), NULL, NULL, G_CALLBACK(logs_etc_callback) },
     { "diff", NULL, N_("_First differences of selected variables"), NULL, NULL, 
-      G_CALLBACK(add_logs_etc) },
+      G_CALLBACK(logs_etc_callback) },
     { "ldiff", NULL, N_("_Log differences of selected variables"), NULL, NULL, 
-      G_CALLBACK(add_logs_etc) },
+      G_CALLBACK(logs_etc_callback) },
     { "sdiff", NULL, N_("_Seasonal differences of selected variables"), NULL, NULL, 
-      G_CALLBACK(add_logs_etc) },
+      G_CALLBACK(logs_etc_callback) },
     { "AddIndex", NULL, N_("_Index variable"), NULL, NULL, G_CALLBACK(add_index) },
     { "AddTime", NULL, N_("_Time trend"), NULL, NULL, G_CALLBACK(add_index) },
     { "AddRandom", NULL, N_("_Random variable..."), NULL, NULL, G_CALLBACK(stats_calculator) },
@@ -1379,7 +1398,7 @@ GtkActionEntry main_entries[] = {
     { "UnitDums", NULL, N_("_Unit dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "TimeDums", NULL, N_("_Time dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "dummify", NULL, N_("Dummies for selected _discrete variables"), NULL, NULL, 
-      G_CALLBACK(add_logs_etc) },
+      G_CALLBACK(logs_etc_callback) },
     { "NewMatrix", NULL, N_("_Define matrix..."), NULL, NULL, G_CALLBACK(gui_new_matrix) },
 
     /* Sample */
@@ -1435,10 +1454,13 @@ GtkActionEntry main_entries[] = {
     /* Model */
     { "Model", NULL, N_("_Model"), NULL, NULL, NULL },
     { "ols", NULL, N_("_Ordinary Least Squares..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "ivreg", NULL, N_("_Instrumental variables"), NULL, NULL, NULL },
+    { "tsls", NULL, N_("_Two-Stage Least Squares..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "iv-liml", NULL, N_("_LIML..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "iv-gmm", NULL, N_("_GMM..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "LinearModels", NULL, N_("Other _linear models"), NULL, NULL, NULL },
     { "wls", NULL, N_("_Weighted Least Squares..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "hsk", NULL, N_("H_eteroskedasticity corrected..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "tsls", NULL, N_("_Two-Stage Least Squares..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "hsk", NULL, N_("H_eteroskedasticity corrected..."), NULL, NULL, G_CALLBACK(model_callback) },
 #ifdef ENABLE_GMP
     { "mpols", NULL, N_("High _precision OLS..."), NULL, NULL, G_CALLBACK(model_callback) }, 
 #endif
@@ -1462,13 +1484,19 @@ GtkActionEntry main_entries[] = {
     { "PANEL_B", NULL, N_("_Between model..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "arbond", NULL, N_("_Arellano-Bond..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "NonlinearModels", NULL, N_("_Nonlinear models"), NULL, NULL, NULL },
-    { "logit", NULL, N_("_Logit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "probit", NULL, N_("_Probit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "logit", NULL, N_("_Logit"), NULL, NULL, NULL }, 
+    { "blogit", NULL, N_("_Binary..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "ologit", NULL, N_("_Ordered..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "mlogit", NULL, N_("_Multinomial..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "probit", NULL, N_("_Probit"), NULL, NULL, NULL }, 
+    { "bprobit", NULL, N_("_Binary..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "oprobit", NULL, N_("_Ordered..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "tobit", NULL, N_("To_bit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "heckit", NULL, N_("_Heckit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "poisson", NULL, N_("Poi_sson..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "logistic", NULL, N_("Lo_gistic..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "nls", NULL, N_("_Nonlinear Least Squares..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
+    { "intreg", NULL, N_("_Interval regression..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "RobustModels", NULL, N_("_Robust estimation"), NULL, NULL, NULL },
     { "lad", NULL, N_("Least _Absolute Deviation..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "quantreg", NULL, N_("_Quantile regression..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1490,14 +1518,12 @@ GtkActionEntry main_entries[] = {
 
 static void add_conditional_items (GtkUIManager *ui)
 {
-#if !defined(USE_GNOME)
     gtk_ui_manager_add_ui(ui, gtk_ui_manager_new_merge_id(ui),
 			  "/MenuBar/Tools/Preferences",
 			  N_("_Menu font..."),
 			  "MenuFont",
 			  GTK_UI_MANAGER_MENUITEM, 
 			  FALSE);
-#endif
 
 #ifdef HAVE_X12A
     gtk_ui_manager_add_ui(ui, gtk_ui_manager_new_merge_id(ui),
@@ -1724,22 +1750,22 @@ mdata_handle_drag  (GtkWidget *widget,
 	strcat(tmp, dfname + skip);
     }
 
-    /* handle spaces in filenames */
+    /* handle spaces and such */
     unescape_url(tmp);
 
 #ifdef G_OS_WIN32
-    slash_convert(tmp, TO_BACKSLASH);
+    filename_to_win32(tryfile, tmp);
+#else
+    strcpy(tryfile, tmp);
 #endif
 
-    strcpy(tryfile, tmp);
-
-    if (gretl_is_pkzip_file(tmp)) {
+    if (has_suffix(tryfile, ".gretl") && gretl_is_pkzip_file(tryfile)) {
 	verify_open_session();
-    } else if (has_suffix(tmp, ".inp")) {
+    } else if (has_suffix(tryfile, ".inp")) {
 	do_open_script(EDIT_SCRIPT);
-    } else if (has_suffix(tmp, ".R")) {
+    } else if (has_suffix(tryfile, ".R")) {
 	do_open_script(EDIT_R);
-    } else if (has_suffix(tmp, ".plt")) {
+    } else if (has_suffix(tryfile, ".plt")) {
 	do_open_script(EDIT_GP);
     } else {
 	verify_open_data(NULL, 0);
