@@ -26,9 +26,6 @@
 # include "winconfig.h"
 #else
 # include "config.h"
-# ifdef USE_GNOME
-#   include <gnome.h>
-# endif
 #endif
 
 #include <gtk/gtk.h>
@@ -42,16 +39,16 @@
 #include <stdlib.h>
 #include <time.h>
 
-#if defined(G_OS_WIN32) || defined(USE_GNOME) || (GTK_MINOR_VERSION >= 10)
-# define NATIVE_PRINTING
-#endif
-
-#ifndef G_OS_WIN32
-# if GTK_MINOR_VERSION >= 10
+#ifndef _WIN32
+# if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 10
 #  define GTK_PRINTING
 # endif
 #endif
 
+#if defined(_WIN32) || defined(GTK_PRINTING)
+# define NATIVE_PRINTING
+#endif
+
 #define GNULL (gconstpointer) NULL
 
 #include <gtksourceview/gtksourceview.h>
@@ -63,9 +60,9 @@
 #include "library.h"
 #include "settings.h"
 #include "helpfiles.h"
+#include "gui_recode.h"
 
 #ifdef ENABLE_NLS
-# include "gui_recode.h"
 # include "locale.h"
 #endif
 
@@ -88,7 +85,6 @@ extern float gui_scale;
 
 /* global filenames */
 extern char scriptfile[MAXLEN];
-extern char sessionfile[MAXLEN];
 extern char tryfile[MAXLEN];
 
 /* global option-related vars */
