@@ -46,13 +46,15 @@
 # define GRETL_STOCK_EDIT "gretl-edit"
 # define GTK_STOCK_EDIT GRETL_STOCK_EDIT
 # define GRETL_STOCK_SCRIPT "gretl-script"
+# define GTK_STOCK_ABOUT NULL
 #endif
 
 enum windata_flags {
-    VWIN_HELP_ACTIVE  = 1 << 0,
-    VWIN_BUSY         = 1 << 1,
-    VWIN_DELETE_FNAME = 1 << 2,
-    VWIN_STICKY       = 1 << 3
+    VWIN_HELP_ACTIVE     = 1 << 0,
+    VWIN_BUSY            = 1 << 1,
+    VWIN_DELETE_FNAME    = 1 << 2,
+    VWIN_STICKY          = 1 << 3,
+    VWIN_CONTENT_CHANGED = 1 << 4
 };
 
 typedef struct _windata_t windata_t;
@@ -60,14 +62,14 @@ typedef struct _windata_t windata_t;
 #include <gtksourceview/gtksourceview.h>
 
 struct _windata_t {
-    GtkWidget *dialog;
-    GtkWidget *vbox;
-    GtkWidget *listbox; 
-    GtkWidget *mbar;
-    GtkWidget *w;
-    GtkWidget *status;
-    GtkWidget *popup;
-    GtkItemFactory *ifac; 
+    GtkWidget *main;      /* top-level GTK window */
+    GtkWidget *vbox;      /* vbox within main */
+    GtkWidget *text;      /* text or sourceview object */
+    GtkWidget *listbox;   /* or: box containing tree or list */
+    GtkWidget *mbar;      /* menubar, or toolbar */
+    GtkWidget *status;    /* status label */
+    GtkWidget *popup;     /* popup menu */
+    GtkUIManager *ui;     /* UI definition */
     windata_t *gretl_parent;
     windata_t **gretl_children;
     gpointer data;
@@ -90,6 +92,15 @@ struct dialog_opts_ {
     const char **strs;
 };
 
+typedef struct GretlToolItem_ GretlToolItem;
+
+struct GretlToolItem_ {
+    const gchar *tip;
+    const gchar *icon;
+    GCallback func;
+    int flag;
+};
+
 #define window_is_busy(w)    (w->flags & VWIN_BUSY)
 
 #define window_help_is_active(w)    (w->flags & VWIN_HELP_ACTIVE)
