@@ -189,82 +189,41 @@ void win32_start_R_async (void)
     g_free(Rline);
 }
 
-static char *substr (char *targ, const char *src, const char *p)
+/* Convert @src, a filename that might be in UTF-8 and might contain
+   forward slashes, to canonical win32 form.  We assume that @targ can
+   hold MAXLEN bytes.
+*/
+
+int filename_to_win32 (char *targ, const char *src)
 {
+    GError *gerr = NULL;
+    gsize bytes;
+    int err = 0;
+
     *targ = '\0';
 
-    if (p == NULL) {
-	strcpy(targ, src);
-    } else {
-	strncat(targ, src, p - src);
-    }
+    if (string_is_utf8(src)) {
+	gchar *tr = g_locale_from_utf8(src, -1, NULL, &bytes, &gerr);
 
-    return targ;
-}
-
-int unmangle (const char *dosname, char *longname)
-{
-    HANDLE handle;
-    WIN32_FIND_DATA fdata;
-    char tmp[MAXLEN];
-    const char *p;
-    int err = 0, done = 0;
-    char drive;
-
-    *longname = '\0';
-    
-    if (sscanf(dosname, "%c:\\", &drive) != 1 ||
-	dosname[strlen(dosname) - 1] == '\\') {
-	strcpy(longname, dosname);
-	return 0;
-    }
-
-    sprintf(longname, "%c:", tolower(drive));
-    p = dosname + 2;
-
-    while (!done) {
-	p = strchr(p + 1, '\\');
-	if (p == NULL) {
-	    done = 1;
-	} 
-	substr(tmp, dosname, p);
-	handle = FindFirstFile(tmp, &fdata);
-	if (handle != INVALID_HANDLE_VALUE) {
-	    strcat(longname, "\\");
-	    strcat(longname, fdata.cFileName);
-	    FindClose(handle);
-	} else {
-	    *longname = '\0';
+	if (gerr != NULL) {
+	    fprintf(stderr, "Couldn't handle '%s': %s\n", src, gerr->message);
+	    g_error_free(gerr);
 	    err = 1;
-	    break;
+	} else {
+	    strncat(targ, tr, MAXLEN - 1);
+	    g_free(tr);
 	}
+    } else {
+	strncat(targ, src, MAXLEN - 1);
+    }
+
+    if (!err) {
+	slash_convert(targ, TO_BACKSLASH);
     }
 
     return err;
 }
 
-void win_help (void)
-{
-    char hlpshow[MAXLEN];
-    int found = 0;
-
-    sprintf(hlpshow, "hh.exe \"%s\\%s\"", paths.gretldir, _("gretl.chm"));
-    
-    if (WinExec(hlpshow, SW_SHOWNORMAL) < 32) {
-	if (strcmp("gretl.chm", _("gretl.chm"))) {
-	    /* try falling back on untranslated helpfile */
-	    sprintf(hlpshow, "hh.exe \"%s\\gretl.chm\"", paths.gretldir);
-	    if (WinExec(hlpshow, SW_SHOWNORMAL) >= 32) found = 1;
-	}
-    } else {
-	found = 1;
-    }
-
-    if (!found) {
-	errbox(_("Couldn't access help file"));
-    }
-}
-
 static void dummy_output_handler (const gchar *log_domain,
 				  GLogLevelFlags log_level,
 				  const gchar *message,
@@ -413,53 +372,21 @@ static int set_network_cfg_filename (const char *prog)
     return 0;
 }
 
-#if 0
-static int set_gd_fontpath (void)
+void gretl_win32_init (const char *progname, int debug)
 {
-    char fpath[MAX_PATH];
-    char *envstr;
-    UINT usz;
-
-    usz = GetWindowsDirectory(fpath, MAX_PATH);
-    if (usz == 0 || usz > MAX_PATH - 6) {
-        return 1;
+    if (debug) {
+        redirect_io_to_console();
     }
 
-    fpath[2] = '/';
-    strcat(fpath, "/fonts");
-
-    SetEnvironmentVariable("GDFONTPATH", fpath);
-
-    envstr = malloc(strlen(fpath) + 12);
-    if (envstr != NULL) {
-	sprintf(envstr, "GDFONTPATH=%s", fpath);
-	putenv(envstr);
-	free(envstr);
-    }
-
-    return 0;        
-}
-#endif
-
-void gretl_win32_init (const char *progname)
-{
     set_network_cfg_filename(progname);
 
     wimp_init();
-    read_rc(); /* get config info from registry */
+    read_rc(debug); /* get config info from registry */
     set_gretl_startdir();
-#if 0
-    set_gd_fontpath();
-#endif
     hush_warnings();
     ws_startup(); 
 }
 
-void gretl_win32_debug (void)
-{
-    redirect_io_to_console();
-}
-
 static int win_copy_buf (const char *buf, int fmt, size_t buflen)
 {
     HGLOBAL winclip;
@@ -789,16 +716,73 @@ int win32_open_file (const char *fname)
     return err;
 }
 
-void win32_raise_window (GtkWidget *w)
+static const char *font_weight_string (int weight)
 {
-    GtkWidget *top = gtk_widget_get_toplevel(w);
+    if (weight >= FW_THIN && weight <= FW_LIGHT) {
+	return " Light";
+    }
+    if (weight >= FW_NORMAL && weight <= FW_DEMIBOLD) {
+	return "";
+    }
+    if (weight >= FW_BOLD) {
+	return " Bold";
+    }
+    return "";
+}
 
-    if (top != NULL && top->window != NULL) {
-	HWND h = GDK_WINDOW_HWND(top->window);
+static void fontname_to_win32 (const char *src, int flag,
+			       char *name, int *pts)
+{
+    int sz;
 
-	SetWindowPos(h, HWND_TOP, 0, 0, 0, 0, /* was HWND_TOPMOST */
-		     SWP_NOMOVE | SWP_NOSIZE);
-	gtk_widget_grab_focus(w);
+    if (sscanf(src, "%31[^0123456789]%d", name, &sz) == 2) {
+	size_t i, n = strlen(name);
+
+	for (i=n-1; i>0; i--) {
+	    if (name[i] == ' ') name[i] = 0;
+	    else break;
+	}
+	*pts = sz * 10; /* measured in tenths of a point */
+    } else {
+	*name = 0;
+	strncat(name, src, 31);
+	if (flag == FIXED_FONT_SELECTION) *pts = 100;
+	else *pts = 80;
     }
 }
 
+void win32_font_selector (char *fontname, int flag)
+{
+    CHOOSEFONT cf;            /* common dialog box structure */
+    LOGFONT lf;               /* logical font structure */
+
+    ZeroMemory(&cf, sizeof cf);
+    cf.lStructSize = sizeof cf;
+    cf.Flags = CF_SCREENFONTS | CF_TTONLY | CF_LIMITSIZE | CF_INITTOLOGFONTSTRUCT;
+    cf.nSizeMin = 6;
+    cf.nSizeMax = 24;
+
+    ZeroMemory(&lf, sizeof lf);
+
+    cf.Flags |= CF_NOSCRIPTSEL;
+    
+    lf.lfWeight = FW_REGULAR;
+    lf.lfCharSet = DEFAULT_CHARSET;
+
+    if (flag == FIXED_FONT_SELECTION) {
+	cf.Flags |= CF_FIXEDPITCHONLY;
+    } 
+
+    fontname_to_win32(fontname, flag, lf.lfFaceName, &(cf.iPointSize));
+
+    cf.lpLogFont = &lf;
+
+    if (ChooseFont(&cf) == TRUE && *(cf.lpLogFont->lfFaceName)) {
+	sprintf(fontname, "%s%s %d", cf.lpLogFont->lfFaceName, 
+		font_weight_string(cf.lpLogFont->lfWeight), 
+		cf.iPointSize / 10);
+    } else {
+	/* signal cancellation */
+	*fontname = '\0';
+    }
+}
