@@ -26,19 +26,10 @@
 #include <libxml/xmlreader.h>
 
 #include "gtksourcelanguagesmanager.h"
-
 #include "gtksourcelanguage-private.h"
-#include "gtksourceview-i18n.h"
-
 #include "gtksourcelanguage.h"
 #include "gtksourcetag.h"
 
-#define DEFAULT_GCONF_BASE_DIR		"/apps/gtksourceview"
-
-#define DEFAULT_LANGUAGE_DIR		DATADIR "/gtksourceview"
-#define USER_LANGUAGE_DIR		"gtksourceview-1.0/language-specs"
-#define USER_CONFIG_BASE_DIR	".gnome2"
-
 enum {
 	PROP_0,
 	PROP_LANG_SPECS_DIRS
@@ -116,10 +107,10 @@ gtk_source_languages_manager_class_init (GtkSourceLanguagesManagerClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_LANG_SPECS_DIRS,
 					 g_param_spec_pointer ("lang_files_dirs",
-						 	       _("Language specification directories"),
-							       _("List of directories where the "
+						 	       "Language specification directories",
+							       "List of directories where the "
 								 "language specification files (.lang) "
-								 "are located"),
+								 "are located",
 							       (G_PARAM_READWRITE | 
 							        G_PARAM_CONSTRUCT_ONLY)));
 }
@@ -177,7 +168,13 @@ gtk_source_languages_manager_instance_init (GtkSourceLanguagesManager *lm)
 	lm->priv = g_new0 (GtkSourceLanguagesManagerPrivate, 1);	
 }
 
-
+/**
+ * gtk_source_languages_manager_new:
+ *
+ * Creates a new language manager.
+ *
+ * Return value: a #GtkSourceLanguagesManager.
+ **/
 GtkSourceLanguagesManager *
 gtk_source_languages_manager_new (void)
 {
@@ -208,13 +205,98 @@ gtk_source_languages_manager_finalize (GObject *object)
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+#ifdef G_OS_WIN32
+
+#include <windows.h> /* read registry */
+
+# define MAXLEN 512
+
+static int read_gretldir_reg_val (HKEY tree, char *keyval)
+{
+    unsigned long datalen = MAXLEN;
+    int error = 0;
+    HKEY regkey;
+
+    if (RegOpenKeyEx(
+                     tree,                        /* handle to open key */
+                     "Software\\gretl",           /* subkey name */
+                     0,                           /* reserved */
+                     KEY_READ,                    /* access mask */
+                     &regkey                      /* key handle */
+                     ) != ERROR_SUCCESS) {
+        return 1;
+    }
+
+    if (RegQueryValueEx(
+                        regkey,
+                        "gretldir",
+                        NULL,
+                        NULL,
+                        keyval,
+                        &datalen
+                        ) != ERROR_SUCCESS) {
+        error = 1;
+    }
+
+    RegCloseKey(regkey);
+
+    return error;
+}
+
+static void get_gretl_lang_dir (char *targ)
+{
+    size_t n;
+
+    read_gretldir_reg_val(HKEY_LOCAL_MACHINE, targ);
+
+    if (*targ == '\0') {
+        strcpy(targ, "c:\\Program Files\\gretl");
+    }
+
+    n = strlen(targ);
+    if (targ[n-1] != '\\') strcat(targ, "\\");
+
+    strcat(targ, "share\\gtksourceview-1.0\\language-specs");
+}
+
 static void 
 gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
 					     const GSList		*dirs)
+{
+    char gretl_lang_dir[MAXLEN];
+
+    g_return_if_fail (GTK_IS_SOURCE_LANGUAGES_MANAGER (lm));
+    g_return_if_fail (lm->priv->language_specs_directories == NULL);
+			
+    if (dirs == NULL) {
+	get_gretl_lang_dir(gretl_lang_dir);
+	lm->priv->language_specs_directories =
+	    g_slist_prepend (lm->priv->language_specs_directories,
+			     g_strdup (gretl_lang_dir));
+	return;
+    }
+
+    while (dirs != NULL) {
+	lm->priv->language_specs_directories = 
+	    g_slist_prepend (lm->priv->language_specs_directories,
+			     g_strdup ((const gchar*)dirs->data));
+	dirs = g_slist_next (dirs);
+    }
+}
+
+#else /* !G_OS_WIN32 */
+
+#define DEFAULT_LANGUAGE_DIR	DATADIR "/gtksourceview"
+#define USER_LANGUAGE_DIR	"gtksourceview-1.0/language-specs"
+#define USER_CONFIG_BASE_DIR	".gnome2"
+
+static void
+gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager *lm,
+					     const GSList              *dirs)
 {
 	g_return_if_fail (GTK_IS_SOURCE_LANGUAGES_MANAGER (lm));
 	g_return_if_fail (lm->priv->language_specs_directories == NULL);
-			
+
 	if (dirs == NULL)
 	{
 	        gchar *env_language_dir = getenv("GTKSOURCEVIEW_LANGUAGE_DIR");
@@ -228,7 +310,7 @@ gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
 						USER_CONFIG_BASE_DIR, USER_LANGUAGE_DIR, 
 						NULL));
 
-                if (env_language_dir != NULL) 
+                if (env_language_dir != NULL && strcmp(DEFAULT_LANGUAGE_DIR, env_language_dir)) 
                 {
                         lm->priv->language_specs_directories = 
                                 g_slist_prepend (lm->priv->language_specs_directories,
@@ -238,7 +320,7 @@ gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
 
 		return;
 	}
-
+	
 	while (dirs != NULL)
 	{
 		lm->priv->language_specs_directories = 
@@ -249,6 +331,16 @@ gtk_source_languages_manager_set_specs_dirs (GtkSourceLanguagesManager	*lm,
 	}
 }
 
+#endif /* G_OS_WIN32 vs other */
+
+/**
+ * gtk_source_languages_manager_get_lang_files_dirs:
+ * @lm: a #GtkSourceLanguagesManager.
+ * 
+ * Gets a list of language files directories for the given language manager.
+ *
+ * Return value: a list of language files directories (as strings).
+ **/
 const GSList *
 gtk_source_languages_manager_get_lang_files_dirs (GtkSourceLanguagesManager *lm)
 {
@@ -257,10 +349,28 @@ gtk_source_languages_manager_get_lang_files_dirs (GtkSourceLanguagesManager *lm)
 	return lm->priv->language_specs_directories;
 }
 
+static void
+prepend_lang (gchar *id, GtkSourceLanguage *lang, GtkSourceLanguagesManager *lm)
+{
+	lm->priv->available_languages = 
+		g_slist_prepend (lm->priv->available_languages, lang);
+}
+
+/**
+ * gtk_source_languages_manager_get_available_languages:
+ * @lm: a #GtkSourceLanguagesManager.
+ * 
+ * Gets a list of available languages for the given language manager.
+ * This function returns a pointer to a internal list, so there is no need to
+ * free it after usage.
+ *
+ * Return value: a list of #GtkSourceLanguage.
+ **/
 const GSList *
 gtk_source_languages_manager_get_available_languages (GtkSourceLanguagesManager *lm)
 {
-	GSList *filenames;
+	GSList *filenames, *l;
+	GHashTable *lang_hash;
 
 	g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGES_MANAGER (lm), NULL);
 
@@ -272,34 +382,50 @@ gtk_source_languages_manager_get_available_languages (GtkSourceLanguagesManager
 	/* Build list of availables languages */
 	filenames = get_lang_files (lm);
 	
-	while (filenames != NULL)
+	lang_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+	for (l = filenames; l != NULL; l = l->next)
 	{
 		GtkSourceLanguage *lang;
-		
-		lang = _gtk_source_language_new_from_file ((const gchar*)filenames->data,
+
+		lang = _gtk_source_language_new_from_file ((const gchar*)l->data,
 							   lm);
 
 		if (lang == NULL)
 		{
 			g_warning ("Error reading language specification file '%s'", 
-				   (const gchar*)filenames->data);
-		}
-		else
-		{	
-			lm->priv->available_languages = 
-				g_slist_prepend (lm->priv->available_languages, lang);
+				   (const gchar*)l->data);
+			continue;
 		}
 
-		filenames = g_slist_next (filenames);
+		if (g_hash_table_lookup (lang_hash, lang->priv->id) == NULL)
+		{	
+			g_hash_table_insert (lang_hash, 
+					     lang->priv->id,
+					     lang);
+		}
 	}
 
 	slist_deep_free (filenames);
 
-	/* TODO: remove duplicates - Paolo */
+	g_hash_table_foreach (lang_hash, (GHFunc) prepend_lang, lm);
+
+	g_hash_table_destroy (lang_hash);
 
 	return lm->priv->available_languages;
 }
 
+/**
+ * gtk_source_languages_manager_get_language_from_mime_type:
+ * @lm: a #GtkSourceLanguagesManager.
+ * @mime_type: a mime type.
+ * 
+ * Gets the #GtkSourceLanguage which is associated with the given @mime_type
+ * in the language manager.
+ *
+ * Return value: a #GtkSourceLanguage, or %NULL if there is no language
+ * associated with the given @mime_type.
+ **/
 GtkSourceLanguage *
 gtk_source_languages_manager_get_language_from_mime_type (GtkSourceLanguagesManager 	*lm,
 							  const gchar 			*mime_type)
