@@ -19,8 +19,6 @@
 
 #include "gretl.h"
 
-#ifdef ENABLE_NLS
-
 static int seven_bit_string (const unsigned char *s)
 {
     while (*s) {
@@ -60,29 +58,6 @@ gchar *my_filename_from_utf8 (char *fname)
     return fname;
 }
 
-gchar *my_locale_from_utf8 (const gchar *src)
-{
-    const gchar *cset;
-    gsize bytes;
-    GError *err = NULL;
-    gchar *ret = NULL;
-
-    if (g_get_charset(&cset)) {
-	/* g_get_charset returns TRUE if the returned 
-	   charset is UTF-8 */ 
-	return g_strdup(src);
-    }
-
-    ret = g_locale_from_utf8(src, -1, NULL, &bytes, &err);
-
-    if (err) {
-	errbox(err->message);
-	g_error_free(err);
-    }
-
-    return ret;
-}
-
 /* returns new copy of fname, converted if need be */
 
 gchar *my_filename_to_utf8 (const char *fname)
@@ -96,7 +71,7 @@ gchar *my_filename_to_utf8 (const char *fname)
     } else {
 	/* On Windows, with GTK >= 2.6, the GLib filename
 	   encoding is UTF-8; however, filenames coming from
-	   a native Windows file dialog will be in the
+	   a native Windows source will be in the
 	   locale charset 
 	*/
 #ifdef G_OS_WIN32
@@ -114,6 +89,31 @@ gchar *my_filename_to_utf8 (const char *fname)
     return ret;
 }
 
+#ifdef ENABLE_NLS
+
+gchar *my_locale_from_utf8 (const gchar *src)
+{
+    const gchar *cset;
+    gsize bytes;
+    GError *err = NULL;
+    gchar *ret = NULL;
+
+    if (g_get_charset(&cset)) {
+	/* g_get_charset returns TRUE if the returned 
+	   charset is UTF-8 */ 
+	return g_strdup(src);
+    } 
+
+    ret = g_locale_from_utf8(src, -1, NULL, &bytes, &err);
+
+    if (err) {
+	errbox(err->message);
+	g_error_free(err);
+    }
+
+    return ret;
+}
+
 /* Used when, e.g. loading a script into a GTK window: the
    script might be in a locale encoding other than UTF-8
    (if it was created in a third-party editor).
@@ -249,10 +249,11 @@ int maybe_recode_gp_file_to_utf8 (const char *fname)
     fclose(fout);
 
     if (recoded) {
+	fprintf(stderr, "recoded old graph file\n");
 	err = copyfile(trname, fname);
     }
 
-    remove(trname);
+    gretl_remove(trname);
     g_free(trname);
 
     return err;
