@@ -52,6 +52,8 @@ void mark_vwin_content_saved (windata_t *vwin);
 
 void mark_vwin_content_changed (windata_t *vwin);
 
+void vwin_set_filename (windata_t *vwin, const char *fname);
+
 int latex_is_ok (void);
 
 gint query_save_text (GtkWidget *w, GdkEvent *event, windata_t *vwin);
@@ -106,6 +108,8 @@ int gretl_file_get_contents (const gchar *fname, gchar **contents);
 
 void start_R (const char *buf, int send_data, int interactive);
 
+const char *print_today (void);
+
 #ifndef G_OS_WIN32
 int browser_open (const char *url);
 #endif
