@@ -27,21 +27,19 @@
 #include "forecast.h"
 #include "texprint.h"
 
+#include "gretl_scalar.h"
+
 #ifdef G_OS_WIN32
 # include <windows.h>
 # include "gretlwin32.h"
 #endif
 
-#ifndef GTK_PRINTING
-# ifdef USE_GNOME
-#  define USE_GNOMEPRINT
-# endif
-#endif
-
 #define PAGE_LINES 47
 
 #ifdef NATIVE_PRINTING
 
+/* printing is enabled, either via Windows or GTK+ */
+
 static gchar *user_string (void)
 {
     const gchar *username, *realname;
@@ -93,7 +91,7 @@ static char *header_string (void)
 
 #if defined(G_OS_WIN32)
 
-void winprint (char *fullbuf, char *selbuf)
+void print_window_content (char *fullbuf, char *selbuf)
 {
     HDC dc;
     PRINTDLG pdlg;
@@ -242,7 +240,7 @@ void winprint (char *fullbuf, char *selbuf)
 
 #undef WGRDEBUG
 
-int winprint_graph (char *emfname)
+int win32_print_graph (char *emfname)
 {
     HENHMETAFILE hemf;
     HDC dc;
@@ -334,344 +332,7 @@ int winprint_graph (char *emfname)
     return !printok;
 }
 
-#elif defined(USE_GNOMEPRINT)
-
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#define GRETL_PRINT_CONFIG_FILE "gretl-print-config"
-#define GRETL_PBM_TMP "gretltmp.pbm"
-#define GRETL_PNG_TMP "gretltmp.png"
-
-static GdkPixbuf *png_mono_pixbuf (const char *fname);
-
-static GnomePrintConfig *load_gretl_print_config_from_file (void)
-{
-    gchar *file_name;
-    gchar *contents;
-    GnomePrintConfig *gretl_print_config;
-    int err;
-	
-    file_name = gnome_util_home_file(GRETL_PRINT_CONFIG_FILE);
-
-    err = gretl_file_get_contents(file_name, &contents);
-    g_free(file_name);
-
-    if (!err) {
-	gretl_print_config = gnome_print_config_from_string(contents, 0);
-	g_free(contents);
-    } else {
-	gretl_print_config = gnome_print_config_default();
-    }
-
-    return gretl_print_config;
-}
-
-static void
-save_gretl_print_config_to_file (GnomePrintConfig *gretl_print_config)
-{
-    gint fd;
-    gchar *str;
-    gint bytes;
-    gchar *file_name;
-    gboolean res;
-
-    g_return_if_fail(gretl_print_config != NULL);
-
-    str = gnome_print_config_to_string(gretl_print_config, 0);
-    g_return_if_fail(str != NULL);
-	
-    file_name = gnome_util_home_file(GRETL_PRINT_CONFIG_FILE);
-
-    fd = open(file_name, O_WRONLY | O_CREAT | O_TRUNC, 0600);
-    g_free(file_name);
-
-    if (fd == -1) goto save_error;
-	
-    bytes = strlen(str);
-    res = (write(fd, str, bytes) == bytes);
-
-    if (!res) goto save_error;
-	
-    close(fd);
-    g_free(str);
-	
-    return;
-	
- save_error:
-    g_warning("gretl cannot save print config file.");
-    g_free(str);
-}
-
-#define GUCAST (const guchar *)
-
-/* still under gnomeprint conditional */
-
-void winprint (char *fullbuf, char *selbuf)
-{
-    GnomePrintJob *job;
-    GnomePrintContext *gpc;
-    GnomePrintConfig *config;
-    GtkWidget *dialog;
-    gint response;
-    gboolean preview = FALSE;
-    GnomeFont *font = NULL;
-    gchar *hdrstart;
-    char *p, linebuf[90], hdr[90];
-    int x, y, line, page;
-    size_t len;
-
-    config = load_gretl_print_config_from_file();
-    job = gnome_print_job_new(config);
-    gpc = gnome_print_job_get_context(job);
-    config = gnome_print_job_get_config(job);
-
-    dialog = gnome_print_dialog_new(job, 
-				    GUCAST "print gretl output", 
-				    0);
-    response = gtk_dialog_run(GTK_DIALOG(dialog));
-
-    switch (response) {
-    case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-	break;
-    case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-	preview = TRUE;
-	break;
-    default:
-	goto winprint_bailout;
-    }
-
-    gnome_print_beginpage(gpc, GUCAST _("gretl output"));
-
-    font = gnome_font_find_closest(GUCAST "Monospace", 10);
-    if (font == NULL) {
-	fprintf(stderr, "gnomeprint couldn't find \"Monospace\"\n");
-	goto winprint_bailout;
-    }
-
-    gnome_print_setfont(gpc, font);
-    /* gnome_print_setrgbcolor(gpc, 0, 0, 0); */
-
-    p = (selbuf != NULL)? selbuf : fullbuf;
-
-    page = 1;
-    x = 72;
-    hdrstart = header_string();
-    while (*p) { /* pages loop */
-	line = 0;
-	y = 756;
-	if (page > 1) {
-	    gnome_print_beginpage(gpc, GUCAST _("gretl output"));
-	    gnome_print_setfont(gpc, font); 
-	}
-	sprintf(hdr, _("%s, page %d"), hdrstart, page++);
-	gnome_print_moveto(gpc, x, y);
-	gnome_print_show(gpc, GUCAST hdr);
-	y = 720;
-	while (*p && line < PAGE_LINES) { /* lines loop */
-	    len = strcspn(p, "\n");
-	    *linebuf = '\0';
-	    strncat(linebuf, p, len);
-	    gnome_print_moveto(gpc, x, y);
-	    gnome_print_show(gpc, GUCAST linebuf);
-	    p += len + 1;
-	    y -= 14; /* line spacing */
-	    line++;
-	}
-	gnome_print_showpage(gpc);
-    }
-
-    g_free(hdrstart);
-
-    gnome_print_job_close(job);
-
-    if (preview) {
-	gtk_widget_show(gnome_print_job_preview_new(job, 
-						    GUCAST "Print preview"));
-    } else {
-	gnome_print_job_print(job);
-    }
-
-    save_gretl_print_config_to_file(config);
-
- winprint_bailout:
-
-    g_object_unref(G_OBJECT(config));
-    g_object_unref(G_OBJECT(gpc));
-    g_object_unref(G_OBJECT(job));
-
-    gtk_widget_destroy (dialog);
-    if (font != NULL) gnome_font_unref(font);
-
-    free(fullbuf);
-    if (selbuf) free(selbuf);
-}
-
-static void
-print_image_from_pixbuf (GnomePrintContext *gpc, GdkPixbuf *pixbuf)
-{
-    guchar *raw_image;
-    gboolean has_alpha;
-    gint rowstride, height, width;
-        
-    raw_image = gdk_pixbuf_get_pixels (pixbuf);
-    has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
-    rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-    height    = gdk_pixbuf_get_height (pixbuf);
-    width     = gdk_pixbuf_get_width (pixbuf);
-        
-    if (has_alpha) {
-	gnome_print_rgbaimage (gpc, GUCAST raw_image, width, height, 
-			       rowstride);
-    } else {
-	gnome_print_rgbimage (gpc, GUCAST raw_image, width, height, 
-			      rowstride);
-    }
-}
-
-void gtk_print_graph (const char *fname)
-{
-    GnomePrintJob *job;
-    GnomePrintConfig *config;
-    GnomePrintContext *gpc; 
-    GdkPixbuf *pbuf;
-    GtkWidget *dialog;
-    gboolean preview = FALSE;
-    gint response;
-    int image_left_x = 530, image_bottom_y = 50;
-    int width, height;
-
-    config = load_gretl_print_config_from_file();
-    job = gnome_print_job_new(config);
-    gpc = gnome_print_job_get_context(job);
-    config = gnome_print_job_get_config(job);
-
-    dialog = gnome_print_dialog_new(job, 
-				    GUCAST _("print gretl graph"), 
-				    0);
-    response = gtk_dialog_run(GTK_DIALOG(dialog));
-
-    switch (response) {
-    case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-	break;
-    case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-# if 0
-	preview = TRUE;
-	break;
-# else
-	/* preview doesn't work for images */
-	dummy_call();
-# endif
-    default:
-	g_object_unref(G_OBJECT(config));
-	g_object_unref(G_OBJECT(gpc));
-	g_object_unref(G_OBJECT(job));
-	gtk_widget_destroy(dialog);
-	return;
-    }
-
-    pbuf = png_mono_pixbuf(fname);
-   
-    if (pbuf == NULL) {
-	errbox(_("Failed to generate graph"));
-	g_object_unref(G_OBJECT(config));
-	g_object_unref(G_OBJECT(gpc));
-	g_object_unref(G_OBJECT(job));
-	gtk_widget_destroy(dialog);
-	return;
-    }
-
-    width = gdk_pixbuf_get_width(pbuf);
-    height = gdk_pixbuf_get_height(pbuf);
-
-    gnome_print_beginpage(gpc, GUCAST _("gretl output"));
-    gnome_print_gsave(gpc);
-    gnome_print_translate(gpc, image_left_x, image_bottom_y);
-    gnome_print_rotate(gpc, 90);
-    gnome_print_scale(gpc, width, height);
-    print_image_from_pixbuf(gpc, pbuf);
-    gnome_print_grestore(gpc);
-    gnome_print_showpage(gpc);
-
-    gnome_print_job_close(job);
-
-    if (preview) {
-	gtk_widget_show(gnome_print_job_preview_new(job, 
-						    GUCAST "Print preview"));
-    } else {
-	gnome_print_job_print(job);
-    }  
-
-    save_gretl_print_config_to_file(config);
-
-    g_object_unref(G_OBJECT(config));
-    g_object_unref(G_OBJECT(gpc));
-    g_object_unref(G_OBJECT(job));
-
-    gtk_widget_destroy(dialog);
-}
-
-/* still under gnomeprint conditional */
-
-static GdkPixbuf *png_mono_pixbuf (const char *fname)
-{
-    FILE *fsrc, *ftmp;
-    char cmd[MAXLEN], temp[MAXLEN], fline[MAXLEN];
-    GdkPixbuf *pbuf = NULL;
-
-    sprintf(temp, "%sgpttmp", paths.dotdir);
-
-    ftmp = gretl_tempfile_open(temp);
-    if (ftmp == NULL) {
-	return NULL;
-    }
-
-    fsrc = gretl_fopen(fname, "r");
-    if (fsrc == NULL) {
-	fclose(ftmp);
-	remove(temp);
-	return NULL;
-    }
-
-    fprintf(ftmp, "set term pbm mono\n"
-	    "set output '%s%s'\n", 
-	    paths.dotdir, GRETL_PBM_TMP);
-
-    while (fgets(fline, MAXLEN-1, fsrc)) {
-	if (strncmp(fline, "set term", 8) && 
-	    strncmp(fline, "set output", 10)) {
-	    fputs(fline, ftmp);
-	}
-    }
-
-    fclose(fsrc);
-    fclose(ftmp);
-
-    /* run gnuplot on the temp plotfile */
-    sprintf(cmd, "\"%s\" \"%s\"", paths.gnuplot, temp);
-    if (system(cmd)) {
-	remove(temp);
-	return NULL;
-    }
-
-    remove(temp);
-
-    build_path(temp, paths.dotdir, GRETL_PBM_TMP, NULL);
-    pbuf = gdk_pixbuf_new_from_file(temp, NULL);
-    remove(temp);
-
-    return pbuf;
-}
-
-#endif /* USE_GNOMEPRINT */
-
-#ifdef GTK_PRINTING
+#elif defined(GTK_PRINTING)
 
 /* native GTK printing with recent GTK+ */
 
@@ -797,7 +458,7 @@ static void job_set_n_pages (GtkPrintOperation *op, struct print_info *pinfo)
 
 static GtkPrintSettings *settings = NULL;
 
-void winprint (char *fullbuf, char *selbuf)
+void print_window_content (char *fullbuf, char *selbuf)
 {
     GtkPrintOperation *op;
     GtkPrintOperationResult res;
@@ -822,8 +483,8 @@ void winprint (char *fullbuf, char *selbuf)
 
     job_set_n_pages(op, &pinfo); 
 
-    g_signal_connect(op, "begin_print", G_CALLBACK(begin_text_print), &pinfo);
-    g_signal_connect(op, "draw_page", G_CALLBACK(draw_text_page), &pinfo);
+    g_signal_connect(op, "begin-print", G_CALLBACK(begin_text_print), &pinfo);
+    g_signal_connect(op, "draw-page", G_CALLBACK(draw_text_page), &pinfo);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
 				  GTK_WINDOW(mdata->main), &err);
@@ -865,7 +526,7 @@ static int make_png_file (const char *fname,
     fsrc = gretl_fopen(fname, "r");
     if (fsrc == NULL) {
 	fclose(ftmp);
-	remove(temp);
+	gretl_remove(temp);
 	return 1;
     }
 
@@ -885,11 +546,11 @@ static int make_png_file (const char *fname,
     /* run gnuplot on the temp plotfile */
     sprintf(cmd, "\"%s\" \"%s\"", paths.gnuplot, temp);
     if (system(cmd)) {
-	remove(temp);
+	gretl_remove(temp);
 	return 1;
     }
 
-    remove(temp);
+    gretl_remove(temp);
 
     return 0;
 }
@@ -968,8 +629,8 @@ void gtk_print_graph (const char *fname)
     gtk_print_operation_set_use_full_page(op, TRUE);
     gtk_print_operation_set_unit(op, GTK_UNIT_POINTS);
 
-    g_signal_connect(op, "begin_print", G_CALLBACK(begin_image_print), pngname);
-    g_signal_connect(op, "draw_page", G_CALLBACK(draw_image), NULL);
+    g_signal_connect(op, "begin-print", G_CALLBACK(begin_image_print), pngname);
+    g_signal_connect(op, "draw-page", G_CALLBACK(draw_image), NULL);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
 				  GTK_WINDOW(mdata->main), &err);
@@ -1110,23 +771,33 @@ rtfprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
     pputs(prn, "}}\n");
 }
 
-static void printftex (double zz, PRN *prn, int endrow)
+/* print value in (non-correlation) matrix */
+
+static void tex_matnum (double x, PRN *prn)
 {
-    if (na(zz)) {
+    char s[32];
+
+    tex_sprint_double_digits(x, s, 5);
+    pprintf(prn, "%s & ", s);
+}
+
+static void printftex (double x, PRN *prn, int endrow)
+{
+    if (na(x)) {
 	if (endrow) {
-	    pprintf(prn, "\\multicolumn{1}{c}{%s}\\\\", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{2}{c}{%s}\\\\", I_("undefined"));
 	} else {
-	    pprintf(prn, "\\multicolumn{1}{c}{%s} & ", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{2}{c}{%s} & ", I_("undefined"));
 	}
     } else {
 	char s[32];
 
-	tex_dcolumn_double(zz, s);
+	tex_rl_double(x, s);
 
 	if (endrow) {
-	    pprintf(prn, "$%s$\\\\", s);
+	    pprintf(prn, "%s\\\\", s);
 	} else {
-	    pprintf(prn, "$%s$ & ", s);
+	    pprintf(prn, "%s & ", s);
 	}
     }	
 }
@@ -1134,6 +805,7 @@ static void printftex (double zz, PRN *prn, int endrow)
 static void 
 texprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
 {
+    char pt = get_local_decpoint();
     char date1[OBSLEN], date2[OBSLEN], vname[16], tmp[128];
     int i, vi;
 
@@ -1150,21 +822,23 @@ texprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
 	sprintf(tmp, I_("for the variable %s (%d valid observations)"), 
 		vname, summ->n);
 	pprintf(prn, "%s\\\\[8pt]\n\n", tmp);
-	pputs(prn, "\\begin{tabular}{rrrr}\n");
+	pprintf(prn, "\\begin{tabular}{r@{%c}lr@{%c}lr@{%c}lr@{%c}l}\n",
+		pt, pt, pt, pt);
     } else {
 	if (summ->missing) {
 	    pprintf(prn, "%s\\\\[8pt]\n\n", I_("(missing values were skipped)"));
 	} else {
 	    pputs(prn, "\n\\vspace{8pt}\n\n");
 	}
-	pputs(prn, "\\begin{tabular}{lrrrr}\n");
+	pprintf(prn, "\\begin{tabular}{lr@{%c}lr@{%c}lr@{%c}lr@{%c}l}\n",
+		pt, pt, pt, pt);
 	pprintf(prn, "%s &", I_("Variable"));
     }
 
-    pprintf(prn, " \\multicolumn{1}{c}{%s}%%\n"
-	    " & \\multicolumn{1}{c}{%s}%%\n"
-	    "  & \\multicolumn{1}{c}{%s}%%\n"
-	    "   & \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
+    pprintf(prn, " \\multicolumn{2}{c}{%s}%%\n"
+	    " & \\multicolumn{2}{c}{%s}%%\n"
+	    "  & \\multicolumn{2}{c}{%s}%%\n"
+	    "   & \\multicolumn{2}{c}{%s} \\\\[1ex]\n",
 	    I_("Mean"), I_("Median"), I_("Minimum"), I_("Maximum"));
 
     for (i=0; i<summ->list[0]; i++) {
@@ -1188,10 +862,10 @@ texprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
 	pprintf(prn, "%s & ", I_("Variable"));
     }
 
-    pprintf(prn, " \\multicolumn{1}{c}{%s}%%\n"
-	    " & \\multicolumn{1}{c}{%s}%%\n"
-	    "  & \\multicolumn{1}{c}{%s}%%\n"
-	    "   & \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
+    pprintf(prn, " \\multicolumn{2}{c}{%s}%%\n"
+	    " & \\multicolumn{2}{c}{%s}%%\n"
+	    "  & \\multicolumn{2}{c}{%s}%%\n"
+	    "   & \\multicolumn{2}{c}{%s} \\\\[1ex]\n",
 	    I_("Std.\\ Dev."), I_("C.V."), I_("Skewness"), I_("Ex.\\ kurtosis"));
 
     for (i=0; i<summ->list[0]; i++) {
@@ -1423,7 +1097,7 @@ texprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 		if (vmat->ci == CORR) {
 		    tex_outxx(vmat->vec[idx], prn);
 		} else {
-		    printftex(vmat->vec[idx], prn, 0);
+		    tex_matnum(vmat->vec[idx], prn);
 		}
 	    }
 	    tex_escape(vname, vmat->names[j]);
@@ -1441,7 +1115,7 @@ texprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 		if (vmat->ci == CORR) {
 		    tex_outxx(vmat->vec[idx], prn);
 		} else {
-		    printftex(vmat->vec[idx], prn, 0);
+		    tex_matnum(vmat->vec[idx], prn);
 		}
 	    }
 	    tex_escape(vname, vmat->names[ij2]);
@@ -1640,9 +1314,9 @@ void special_print_fit_resid (const FITRESID *fr,
 static void texprint_fcast_x (double x, int places, char *str)
 {
     if (places != PMAX_NOT_AVAILABLE && !na(x)) {
-	sprintf(str, "%.*f", places, x);
+	tex_rl_float(x, str, places);
     } else {
-	tex_dcolumn_double(x, str);
+	tex_rl_double(x, str);
     }
 }
 
@@ -1655,15 +1329,14 @@ static void texprint_fcast_without_errs (const FITRESID *fr,
     char pt = get_local_decpoint();
     int t;
 
-    pputs(prn, "%% The table below needs the \"dcolumn\" and "
-	  "\"longtable\" packages\n\n");
+    pputs(prn, "%% The table below needs the \"longtable\" package\n\n");
 
     pprintf(prn, "\\begin{center}\n"
 	    "\\begin{longtable}{%%\n"
 	    "r%% col 1: obs\n"
 	    "  l%% col 2: varname\n"
-	    "    D{%c}{%c}{-1}}%% col 3: fitted\n",
-	    pt, pt);
+	    "    r@{%c}l}%% col 3: fitted\n",
+	    pt);
 
     tex_escape(vname, fr->depvar);
 
@@ -1685,7 +1358,8 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 				      const DATAINFO *pdinfo, 
 				      PRN *prn)
 {
-    double maxerr;
+    double maxerr, tval = 0;
+    double conf = 100 * (1 - fr->alpha);
     int pmax = fr->pmax;
     int errpmax = fr->pmax;
     char actual[32], fitted[32], sderr[32], lo[32], hi[32];
@@ -1694,42 +1368,42 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
     int t;
 
     pputs(prn, "\\begin{center}\n");
+
     if (fr->asymp) {
-	pprintf(prn, _("For 95\\%% confidence intervals, $z(.025) = %.2f$\n\n"), 
-		1.96);
+	tval = normal_critval(fr->alpha / 2);
+	pprintf(prn, I_("For %g\\%% confidence intervals, $z(%g) = %.2f$\n\n"), 
+		conf, fr->alpha / 2, tval);
     } else {
-	pprintf(prn, I_("For 95\\%% confidence intervals, $t(%d, .025) = %.3f$\n\n"), 
-		fr->df, fr->tval);
+	tval = student_critval(fr->df, fr->alpha / 2);
+	pprintf(prn, I_("For %g\\%% confidence intervals, $t(%d, %g) = %.3f$\n\n"), 
+		conf, fr->df, fr->alpha / 2, tval);
     }
+
     pputs(prn, "\\end{center}\n");
 
-    pputs(prn, "%% The table below needs the \"dcolumn\" and "
-	  "\"longtable\" packages\n\n");
+    pputs(prn, "%% The table below needs the "
+	  "\"longtable\" package\n\n");
 
     pprintf(prn, "\\begin{center}\n"
 	    "\\begin{longtable}{%%\n"
 	    "r%% col 1: obs\n"
-	    "  l%% col 2: varname\n"
-	    "    D{%c}{%c}{-1}%% col 3: fitted\n"
-	    "      D{%c}{%c}{-1}%% col 4: std error\n"
-	    "        D{%c}{%c}{-1}%% col 5: conf int lo\n"
-	    "         D{%c}{%c}{-1}}%% col 5: conf int hi\n",
-	    pt, pt, pt, pt, pt, pt, pt, pt);
+	    "  r@{%c}l%% col 2: actual\n"
+	    "    r@{%c}l%% col 3: fitted\n"
+	    "      r@{%c}l%% col 4: std error\n"
+	    "        r@{%c}l%% col 5: conf int lo\n"
+	    "         r@{%c}l}%% col 5: conf int hi\n",
+	    pt, pt, pt, pt, pt);
 
     tex_escape(vname, fr->depvar);
+    sprintf(hi, I_("%g\\%% interval"), conf);
 
-    pprintf(prn, "%s & %s & \\multicolumn{1}{c}{%s}\n"
-	    " & \\multicolumn{1}{c}{%s}\n"
-	    "  & \\multicolumn{2}{c}{%s} \\\\\n",
+    pprintf(prn, "%s & \\multicolumn{2}{c}{%s} "
+	    " & \\multicolumn{2}{c}{%s}\n"
+	    "  & \\multicolumn{2}{c}{%s}\n"
+	    "   & \\multicolumn{4}{c}{%s} \\\\[1ex]\n",
 	    I_("Obs"), vname,
 	    I_("prediction"), I_("std. error"),
-	    /* xgettext:no-c-format */
-	    I_("95\\% confidence interval"));
-
-    pprintf(prn, "& & & & \\multicolumn{1}{c}{%s} & "
-	    "\\multicolumn{1}{c}{%s} \\\\\n", 
-	    I_("low"), I_("high"));
-
+	    hi);
 
     if (pmax < 4) {
 	errpmax = pmax + 1;
@@ -1741,7 +1415,7 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 	if (na(fr->sderr[t])) {
 	    xlo = xhi = NADBL;
 	} else {
-	    maxerr = fr->tval * fr->sderr[t];
+	    maxerr = tval * fr->sderr[t];
 	    xlo = fr->fitted[t] - maxerr;
 	    xhi = fr->fitted[t] + maxerr;
 	}
@@ -1795,19 +1469,24 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 				      const DATAINFO *pdinfo, 
 				      PRN *prn)
 {
-    int t;
-    double maxerr;
+    double maxerr, tval = 0;
+    double conf = 100 * (1 - fr->alpha);
     char tmp[128];
+    int t;
 
     if (fr->asymp) {
-	sprintf(tmp, I_("For 95%% confidence intervals, z(.025) = %.2f"), 
-		1.96);
+	tval = normal_critval(fr->alpha / 2);
+	sprintf(tmp, I_("For %g%% confidence intervals, z(%g) = %.2f"), 
+		conf, fr->alpha / 2, tval);
     } else {
-	sprintf(tmp, I_("For 95%% confidence intervals, t(%d, .025) = %.3f"), 
-		fr->df, fr->tval);
+	tval = student_critval(fr->df, fr->alpha / 2);
+	sprintf(tmp, I_("For %g%% confidence intervals, t(%d, %g) = %.3f"), 
+		conf, fr->df, fr->alpha / 2, tval);
     }
     pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n\\par\n", tmp);
 
+    sprintf(tmp, I_("%g%% interval"), conf);
+
     pputs(prn, "{" FCE_ROW "\\intbl ");
     pprintf(prn, 
 	    " \\qc %s\\cell"
@@ -1818,19 +1497,18 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 	    " \\intbl \\row\n", 
 	    I_("Obs"), fr->depvar, I_("prediction"), 
 	    I_("std. error"),
-	    /* xgettext:no-c-format */
-	    I_("95% confidence interval"));
+	    tmp);
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	rtf_print_obs_marker(t, pdinfo, prn);
-	maxerr = fr->tval * fr->sderr[t];
+	maxerr = tval * fr->sderr[t];
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
 	printfrtf(fr->sderr[t], prn, 0);
 	if (na(fr->sderr[t])) {
 	    pputs(prn, "\\qc \\cell \\intbl \\row\n");
 	} else {
-	    maxerr = fr->tval * fr->sderr[t];
+	    maxerr = tval * fr->sderr[t];
 	    pprintf(prn, "\\qc (%#.*g, %#.*g)\\cell \\intbl \\row\n", 
 		    GRETL_DIGITS, fr->fitted[t] - maxerr, 
 		    GRETL_DIGITS, fr->fitted[t] + maxerr);
@@ -1868,23 +1546,24 @@ texprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
     pprintf(prn, " %s & ", vname);
 
     if (isnan(cf->coeff[i])) {
-	pprintf(prn, "\\multicolumn{1}{c}{%s} & ", I_("undefined"));
+	pprintf(prn, "\\multicolumn{2}{c}{%s} & ", I_("undefined"));
     } else {
 	char coeff[32];
 
-	tex_dcolumn_double(cf->coeff[i], coeff);
+	tex_rl_double(cf->coeff[i], coeff);
 	pprintf(prn, "%s & ", coeff);
     }
 
     if (isnan(cf->maxerr[i])) {
-	pprintf(prn, "\\multicolumn{2}{c}{%s}", I_("undefined"));
+	pprintf(prn, "\\multicolumn{4}{c}{%s}", I_("undefined"));
     } else {
 	char lo[32], hi[32];
 
-	tex_dcolumn_double(cf->coeff[i] - cf->maxerr[i], lo);
-	tex_dcolumn_double(cf->coeff[i] + cf->maxerr[i], hi);
+	tex_rl_double(cf->coeff[i] - cf->maxerr[i], lo);
+	tex_rl_double(cf->coeff[i] + cf->maxerr[i], hi);
 	pprintf(prn, "%s & %s", lo, hi);
     }
+
     pputs(prn, "\\\\\n");
 }
 
@@ -1897,26 +1576,20 @@ static void texprint_confints (const CoeffIntervals *cf, PRN *prn)
 
     pprintf(prn, "$t(%d, %g) = %.3f$\n\n", cf->df, tail, cf->t);
 
-    pputs(prn, "%% The table below needs the \"dcolumn\" package\n\n");
-
     pprintf(prn, "\\begin{center}\n"
-	    "\\begin{tabular}{rD{%c}{%c}{-1}D{%c}{%c}{-1}D{%c}{%c}{-1}}\n",
-	    pt, pt, pt, pt, pt, pt);
+	    "\\begin{tabular}{rr@{%c}lr@{%c}lr@{%c}l}\n",
+	    pt, pt, pt);
 
     cstr = g_strdup_printf(I_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
 
     pprintf(prn, " %s%%\n"
-	    " & \\multicolumn{1}{c}{%s}%%\n"
-	    "  & \\multicolumn{2}{c}{%s}\\\\\n",
+	    " & \\multicolumn{2}{c}{%s}%%\n"
+	    "  & \\multicolumn{4}{c}{%s}\\\\[1ex]\n",
 	    I_("Variable"), I_("Coefficient"),
 	    cstr);
 
     g_free(cstr);
 
-    pprintf(prn, " & & \\multicolumn{1}{c}{%s}%%\n"
-	    "  & \\multicolumn{1}{c}{%s}\\\\\n",
-	    I_("low"), I_("high"));
-
     for (i=0; i<cf->ncoeff; i++) {
 	texprint_coeff_interval(cf, i, prn);
     }
@@ -1983,18 +1656,11 @@ void special_print_confints (const CoeffIntervals *cf, PRN *prn)
     }
 }
 
-/* copy data to buffer in CSV format and place on clipboard */
-
-#define SCALAR_DIGITS 12
-
-static int data_to_buf_as_csv (const int *list, PRN *prn)
+int scalars_to_prn (PRN *prn)
 {
-    int i, t, l0 = list[0];
-    int *pmax = NULL;
-    int tsamp = datainfo->t2 - datainfo->t1 + 1;
-    double xx;
-
-    if (l0 == 0) return 1;
+    const char *sname;
+    double sval;
+    int i, n = n_saved_scalars();
 
     if (datainfo->delim == ',' && ',' == datainfo->decpoint) {
 	errbox(_("You can't use the same character for "
@@ -2002,44 +1668,18 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	return 1;
     }
 
-    pmax = mymalloc(l0 * sizeof *pmax);
-    if (pmax == NULL) {
-	return 1;
-    }
-
-    for (i=1; i<=l0; i++) {
-	pmax[i-1] = get_precision(&Z[list[i]][datainfo->t1], 
-				  tsamp, 8);
-    }	
-
     if (datainfo->decpoint != ',') {
 	gretl_push_c_numeric_locale();
     }
 
-    /* obs column heading? */
-    if (datainfo->S != NULL || datainfo->structure != CROSS_SECTION) {
-	pprintf(prn, "obs%c", datainfo->delim);
-    }
 
-    /* variable names */
-    for (i=1; i<=l0; i++) {
-	pprintf(prn, "%s", datainfo->varname[list[i]]);
-	pputc(prn, (i < l0)? datainfo->delim : '\n');
-    }
-
-    /* actual data values */
-    for (t=datainfo->t1; t<=datainfo->t2; t++) {
-	csv_obs_to_prn(t, datainfo, prn);
-	for (i=1; i<=l0; i++) { 
-	    xx = Z[list[i]][t];
-	    if (na(xx)) {
-		pputs(prn, "NA");
-	    } else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
-		pprintf(prn, "%.10g", xx);
-	    } else {
-		pprintf(prn, "%.*f", pmax[i-1], xx);
-	    }
-	    pputc(prn, (i < l0)? datainfo->delim : '\n');
+    for (i=0; i<n; i++) {
+	sname = gretl_scalar_get_name(i);
+	sval = gretl_scalar_get_value_by_index(i);
+	if (na(sval)) {
+	    pprintf(prn, "%s%cNA\n", sname, datainfo->delim);
+	} else {
+	    pprintf(prn, "%s%c%.15g\n", sname, datainfo->delim, sval);
 	}
     }
 
@@ -2047,13 +1687,29 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	gretl_pop_c_numeric_locale();
     }
 
-    if (pmax != NULL) {
-	free(pmax);
-    }
-
     return 0;
 }
 
+static int data_to_buf_as_rtf (const int *list, PRN *prn)
+{
+    int err;
+
+    gretl_print_set_format(prn, GRETL_FORMAT_RTF);
+    err = print_data_sorted(list, NULL, (const double **) Z, 
+			    datainfo, prn);
+    return err;
+}
+
+static int data_to_buf_as_csv (const int *list, PRN *prn)
+{
+    int err;
+
+    gretl_print_set_format(prn, GRETL_FORMAT_CSV);
+    err = print_data_sorted(list, NULL, (const double **) Z, 
+			    datainfo, prn);
+    return err;
+}
+
 static int real_csv_to_clipboard (const int *list)
 {
     PRN *prn = NULL;
@@ -2085,9 +1741,9 @@ int csv_to_clipboard (void)
 
     data_save_selection_wrapper(COPY_CSV, GINT_TO_POINTER(opt));
 
-    if (storelist != NULL && *storelist != 0) {
-	list = gretl_list_from_string(storelist);	
-	if (list != NULL) {
+    if (storelist != NULL && *storelist != '\0') {
+	list = gretl_list_from_string(storelist, &err);	
+	if (!err && list != NULL) {
 	    err = real_csv_to_clipboard(list);
 	    free(list);
 	}
@@ -2114,10 +1770,38 @@ int csv_selected_to_clipboard (void)
     return err;
 }
 
+int scalars_to_clipboard_as_csv (void)
+{
+    PRN *prn = NULL;
+    int err = 0;
+
+    if (n_saved_scalars() == 0) {
+	warnbox(_("No scalar variables are currently defined"));
+	return 0;
+    }
+
+    if (delimiter_dialog(NULL)) {
+	return 0;
+    }
+
+    if (bufopen(&prn)) {
+	return 1;
+    }
+
+    err = scalars_to_prn(prn);
+    if (!err) {
+	prn_to_clipboard(prn, GRETL_FORMAT_CSV);
+    }
+
+    gretl_print_destroy(prn);
+
+    return err;
+}
+
 #include "series_view.h"
 #include "fileselect.h"
 
-int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
+int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 {
     int *list = series_view_get_list(vwin);
     PRN *prn = NULL;
@@ -2136,18 +1820,24 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 	if (fmt == GRETL_FORMAT_CSV) {
 	    datainfo->delim = ',';
 	    datainfo->decpoint = '.';
-	} else {
+	} else if (fmt == GRETL_FORMAT_TAB) {
+	    fmt |= GRETL_FORMAT_CSV;
 	    datainfo->delim = '\t';
 	}
 
 	if (series_view_is_sorted(vwin)) {
-	    prn = vwin_print_sorted_as_csv(vwin);
+	    prn = vwin_print_sorted_with_format(vwin, fmt);
 	    if (prn == NULL) {
 		err = 1;
 	    }
 	} else {
 	    err = bufopen(&prn);
-	    if (!err) {
+	}
+
+	if (!err) {
+	    if (fmt == GRETL_FORMAT_RTF) {
+		err = data_to_buf_as_rtf(list, prn);
+	    } else {
 		err = data_to_buf_as_csv(list, prn);
 	    }
 	}
@@ -2155,6 +1845,8 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 	if (!err) {
 	    if (action == W_COPY) {
 		err = prn_to_clipboard(prn, fmt);
+	    } else if (fmt == GRETL_FORMAT_RTF) {
+		file_selector(_("Save"), SAVE_RTF, FSEL_DATA_PRN, prn);
 	    } else {
 		file_selector(_("Save data"), EXPORT_CSV, FSEL_DATA_PRN, prn);
 	    }
