@@ -22,13 +22,11 @@
 #ifndef GUIPRINT_H
 #define GUIPRINT_H
 
-#ifdef G_OS_WIN32
-int winprint_graph (char *emfname);
-#endif
-
 #ifdef NATIVE_PRINTING
-void winprint (char *fullbuf, char *selbuf);
-# ifndef G_OS_WIN32
+void print_window_content (char *fullbuf, char *selbuf);
+# ifdef G_OS_WIN32
+int win32_print_graph (char *emfname);
+# else
 void gtk_print_graph (const char *fname);
 # endif
 #endif
@@ -56,7 +54,9 @@ int csv_to_clipboard (void);
 
 int csv_selected_to_clipboard (void);
 
-int csv_copy_listed_vars (windata_t *vwin, int fmt, int action);
+int copy_vars_formatted (windata_t *vwin, int fmt, int action);
+
+int scalars_to_clipboard_as_csv (void);
 
 int font_has_minus (PangoFontDescription *desc);
 
