@@ -140,7 +140,6 @@ static struct gui_help_item gui_help_items[] = {
     { SYS_RESTR,      "restrict-system" },
     { VECM_RESTR,     "restrict-vecm" },
     { LAGS_DIALOG,    "lags-dialog" },
-    { COPY_FORMATS,   "copy-formats" },
     { MINIBUF,        "minibuffer" },
     { VLAGSEL,        "VAR-lagselect" },
     { VAROMIT,        "VAR-omit" },
@@ -152,6 +151,8 @@ static struct gui_help_item gui_help_items[] = {
     { DATASORT,       "datasort" },
     { WORKDIR,        "working-dir" },
     { DFGLS,          "dfgls" },
+    { GPT_ADDLINE,    "addline" }, 
+    { GPT_CURVE  ,    "curve" }, 
     { -1,             NULL },
 };
 
@@ -1584,14 +1585,16 @@ static void find_in_text (GtkWidget *widget, gpointer data)
 
 static void 
 get_tree_model_haystack (GtkTreeModel *mod, GtkTreeIter *iter, int col,
-			 char *haystack)
+			 char *haystack, int vnames)
 {
     gchar *tmp;
 
     gtk_tree_model_get(mod, iter, col, &tmp, -1);
     if (tmp != NULL) {
 	strcpy(haystack, tmp);
-	lower(haystack);
+	if (!vnames) {
+	    lower(haystack);
+	}
 	g_free(tmp);
     } else {
 	*haystack = '\0';
@@ -1606,18 +1609,28 @@ static void find_in_listbox (GtkWidget *w, gpointer p)
     char pstr[16];
     GtkTreeModel *model;
     GtkTreeIter iter;
+    gpointer vp;
+    int vnames = 0;
     int found = -1;
 
     /* if searching in the main gretl window, start on line 1 */
     minvar = (win == mdata)? 1 : 0;
 
+    /* are we confining the search to variable names? */
+    vp = g_object_get_data(G_OBJECT(p), "vnames_only");
+    if (vp != NULL) {
+	vnames = GPOINTER_TO_INT(vp);
+    }
+
     if (needle != NULL) {
 	g_free(needle);
 	needle = NULL;
     }
 
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
-    lower(needle);
+    if (!vnames) {
+	lower(needle);
+    }
 
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox));
 
@@ -1633,17 +1646,17 @@ static void find_in_listbox (GtkWidget *w, gpointer p)
 
     while (found < 0) {
 	/* try looking in column 1 first */
-	get_tree_model_haystack(model, &iter, 1, haystack);
+	get_tree_model_haystack(model, &iter, 1, haystack, vnames);
 
 	found = look_for_string(haystack, needle, 0);
 
-	if (found < 0) {
+	if (found < 0 && !vnames) {
 	    if (win == mdata) {
 		/* then column 2 */
-		get_tree_model_haystack(model, &iter, 2, haystack);
+		get_tree_model_haystack(model, &iter, 2, haystack, vnames);
 	    } else {
 		/* then column 0 */
-		get_tree_model_haystack(model, &iter, 0, haystack);
+		get_tree_model_haystack(model, &iter, 0, haystack, vnames);
 	    }
 	    found = look_for_string(haystack, needle, 0);
 	}
@@ -1699,6 +1712,17 @@ static void parent_find (GtkWidget *finder, windata_t *caller)
     }
 }
 
+static void toggle_vname_search (GtkToggleButton *tb, GtkWidget *w)
+{
+    if (gtk_toggle_button_get_active(tb)) {
+	g_object_set_data(G_OBJECT(w), "vnames_only",
+			  GINT_TO_POINTER(1));
+    } else {
+	g_object_set_data(G_OBJECT(w), "vnames_only",
+			  GINT_TO_POINTER(0));
+    }
+}
+
 static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 {
     GtkWidget *label;
@@ -1720,9 +1744,9 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 		     G_CALLBACK(close_find_dialog),
 		     find_window);
 
-    hbox = gtk_hbox_new(TRUE, TRUE);
+    hbox = gtk_hbox_new(FALSE, 5);
     label = gtk_label_new(_(" Find what:"));
-    gtk_widget_show (label);
+    gtk_widget_show(label);
     find_entry = gtk_entry_new();
 
     if (needle != NULL) {
@@ -1735,12 +1759,23 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 
     gtk_widget_show(find_entry);
 
-    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), find_entry, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), find_entry, TRUE, TRUE, 5);
     gtk_widget_show(hbox);
 
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG(find_window)->vbox), 
-		       hbox, TRUE, TRUE, 0);
+		       hbox, TRUE, TRUE, 5);
+
+    if (vwin == mdata) {
+	hbox = gtk_hbox_new(FALSE, 5);
+	button = gtk_check_button_new_with_label(_("Variable names only (case sensitive)"));
+	g_signal_connect(G_OBJECT(button), "toggled",
+			 G_CALLBACK(toggle_vname_search), find_window);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+	gtk_widget_show_all(hbox);
+	gtk_box_pack_start(GTK_BOX (GTK_DIALOG(find_window)->vbox), 
+			   hbox, FALSE, FALSE, 0);
+    }
 
     hbox = GTK_DIALOG(find_window)->action_area;
 
@@ -1774,10 +1809,17 @@ static int get_writable_path (char *path, const char *fname)
 {
     static int sysdoc_writable = -1;
     static int userdoc_writable = -1;
-
     FILE *fp;
     int err = 0;
 
+    if (sysdoc_writable == 1) {
+	sprintf(path, "%sdoc%c%s", paths.gretldir, SLASH, fname);
+	return 0;
+    } else if (userdoc_writable == 1) {
+	sprintf(path, "%sdoc%c%s", paths.dotdir, SLASH, fname);
+	return 0;
+    }
+
     if (sysdoc_writable < 0) {
 	sysdoc_writable = 0;
 	sprintf(path, "%sdoc", paths.gretldir);
@@ -1787,12 +1829,13 @@ static int get_writable_path (char *path, const char *fname)
 	    if (fp != NULL) {
 		sysdoc_writable = 1;
 		fclose(fp);
-		remove(path);
+		gretl_remove(path);
 	    } 
 	} 
     }
 
     if (!sysdoc_writable && userdoc_writable < 0) {
+	/* can't write to 'sys' dir, user dir not tested yet */
 	userdoc_writable = 0;
 	sprintf(path, "%sdoc", paths.dotdir);
 	if (gretl_mkdir(path) == 0) {
@@ -1801,7 +1844,7 @@ static int get_writable_path (char *path, const char *fname)
 	    if (fp != NULL) {
 		userdoc_writable = 1;
 		fclose(fp);
-		remove(path);
+		gretl_remove(path);
 	    } 
 	} 
     }
@@ -1813,7 +1856,13 @@ static int get_writable_path (char *path, const char *fname)
     return err;
 }
 
-static int find_or_download_pdf (int uguide, int i, char *fullpath)
+enum {
+    GRETL_GUIDE = 1,
+    GRETL_REF,
+    GNUPLOT_REF
+};
+
+static int find_or_download_pdf (int code, int i, char *fullpath)
 {
     const char *guide_files[] = {
 	"gretl-guide.pdf",
@@ -1835,7 +1884,15 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
 	i = 0;
     }
 
-    fname = (uguide)? guide_files[i] : ref_files[i];
+    if (code == GRETL_GUIDE) {
+	fname = guide_files[i];
+    } else if (code == GRETL_REF) {
+	fname = ref_files[i];
+    } else if (code == GNUPLOT_REF) {
+	fname = "gnuplot.pdf";
+    } else {
+	return E_DATA;
+    }
 
     /* is the file available in public dir? */
     sprintf(fullpath, "%sdoc%c%s", paths.gretldir, SLASH, fname);
@@ -1874,21 +1931,8 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
     return err;
 }
 
-void display_pdf_help (GtkAction *action)
+static void show_pdf (const char *fname)
 {
-    char fname[FILENAME_MAX];
-    int err, uguide = 1;
-
-    if (action != NULL && strcmp(gtk_action_get_name(action), "UserGuide")) {
-	/* PDF command ref wanted */
-	uguide = 0;
-    }
-
-    err = find_or_download_pdf(uguide, get_manpref(), fname);
-    if (err) {
-	return;
-    }
-
 #if defined(G_OS_WIN32)
     win32_open_file(fname);
 #elif defined(OSX_BUILD)
@@ -1897,3 +1941,32 @@ void display_pdf_help (GtkAction *action)
     gretl_fork("viewpdf", fname);
 #endif
 }
+
+void display_pdf_help (GtkAction *action)
+{
+    char fname[FILENAME_MAX];
+    int err, code = GRETL_GUIDE;
+
+    if (action != NULL && strcmp(gtk_action_get_name(action), "UserGuide")) {
+	/* PDF command ref wanted */
+	code = GRETL_REF;
+    }
+
+    err = find_or_download_pdf(code, get_manpref(), fname);
+
+    if (!err) {
+	show_pdf(fname);
+    }
+}
+
+void display_gnuplot_help (void)
+{
+    char fname[FILENAME_MAX];
+    int err;
+
+    err = find_or_download_pdf(GNUPLOT_REF, 0, fname);
+
+    if (!err) {
+	show_pdf(fname);
+    }
+}
