@@ -57,6 +57,8 @@ int gretl_command_strcpy (const char *s);
 
 int gretl_command_strcat (const char *s);
 
+int record_command_line (const char *s);
+
 int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
 gint bufopen (PRN **pprn);
@@ -123,10 +125,12 @@ int do_VAR_omit (selector *sr);
 
 int do_confidence_region (selector *sr);
 
-void do_lmtest (GtkAction *action, gpointer p);
+void do_modtest (GtkAction *action, gpointer p);
 
 void do_autocorr (GtkAction *action, gpointer p);
 
+void do_dwpval (GtkAction *action, gpointer p);
+
 void do_chow_cusum (GtkAction *action, gpointer p);
 
 void do_reset (GtkAction *action, gpointer p);
@@ -227,7 +231,9 @@ void do_add_obs (void);
 
 void do_remove_obs (void);
 
-void add_logs_etc (GtkAction *action);
+void add_logs_etc (int ci);
+
+void logs_etc_callback (GtkAction *action);
 
 void add_system_resid (GtkAction *action, gpointer p);
 
