@@ -163,11 +163,6 @@ void main_menubar_state (gboolean s)
 #define x12_ts(d) ((d)->structure == TIME_SERIES && \
                    (d->pd == 4 || d->pd == 12))
 
-const gchar *ts_ui = 
-    "<ui>"
-    
-    "</ui>";
-
 void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
@@ -237,7 +232,11 @@ void session_menu_state (gboolean s)
 	flip(mdata->ui, "/MenuBar/View/IconView", s);
 	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", s);
 	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSessionAs", s);
-    }	
+    }
+
+    if (!s && mdata->main != NULL) {
+	set_main_window_title(NULL, FALSE);
+    }
 }
 
 void restore_sample_state (gboolean s)
@@ -298,9 +297,10 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
 	do_corrgm();
     else if (!strcmp(item, _("Spectrum"))) 
 	do_pergm(NULL);
-    else if (!strcmp(item, _("ARIMA model"))) 
-	selection_dialog(_("gretl: specify model"), do_model, ARMA, v);
-    else if (!strcmp(item, _("Dickey-Fuller test"))) 
+    else if (!strcmp(item, _("ARIMA model"))) {
+	selector_set_varnum(v);
+	modelspec_dialog(ARMA);
+    } else if (!strcmp(item, _("Dickey-Fuller test"))) 
 	unit_root_test(ADF);
     else if (!strcmp(item, _("KPSS test"))) 
 	unit_root_test(KPSS);
@@ -352,6 +352,10 @@ static gint selection_popup_click (GtkWidget *w, gpointer p)
  	show_spreadsheet(SHEET_EDIT_VARLIST);
     } else if (!strcmp(item, _("Delete")))  {
 	delete_selected_vars();
+    } else if (!strcmp(item, _("Add logs")))  {
+	add_logs_etc(LOGS);
+    } else if (!strcmp(item, _("Add differences")))  {
+	add_logs_etc(DIFF);
     }
 
     gtk_widget_destroy(mdata->popup);
@@ -373,6 +377,7 @@ GtkWidget *build_var_popup (void)
 	N_("Edit values"),
 	N_("Copy to clipboard"),
 	N_("Delete"),
+	NULL,
 	N_("Define new variable...")
     };
     GtkWidget *menu;
@@ -382,6 +387,12 @@ GtkWidget *build_var_popup (void)
     menu = gtk_menu_new();
 
     for (i=0; i<n; i++) {
+	if (items[i] == NULL) {
+	    item = gtk_separator_menu_item_new();
+	    gtk_widget_show(item);
+	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	    continue;
+	}
 	if ((i == 5 || i == 6) && !dataset_is_time_series(datainfo)) {
 	    continue;
 	}
@@ -410,7 +421,10 @@ GtkWidget *build_selection_popup (void)
 	N_("XY scatterplot"),
 	N_("Copy to clipboard"),
 	N_("Edit values"),
-	N_("Delete")
+	N_("Delete"),
+	NULL,
+	N_("Add logs"),
+	N_("Add differences")
     };
     GtkWidget *menu;
     GtkWidget *item;
@@ -419,7 +433,13 @@ GtkWidget *build_selection_popup (void)
     menu = gtk_menu_new();
 
     for (i=0; i<n; i++) {
-	if (!dataset_is_time_series(datainfo) && i == 3) {
+	if (items[i] == NULL) {
+	    item = gtk_separator_menu_item_new();
+	    gtk_widget_show(item);
+	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	    continue;
+	}
+	if (!dataset_is_time_series(datainfo) && (i == 3 || i == 11)) {
 	    continue;
 	}
 	if (!extended_ts(datainfo) && i == 4) {
@@ -444,6 +464,25 @@ void clear_sample_label (void)
     gtk_label_set_text(GTK_LABEL(dlabel), _(" No datafile loaded "));
 }
 
+void set_main_window_title (const char *name, gboolean modified)
+{
+    if (name == NULL) {
+	gtk_window_set_title(GTK_WINDOW(mdata->main), "gretl");
+    } else {
+	/* FIXME encoding on Windows? */
+	gchar *title;
+
+	if (modified) {
+	    title = g_strdup_printf("gretl: %s *", name);
+	} else {
+	    title = g_strdup_printf("gretl: %s", name);
+	}
+
+	gtk_window_set_title(GTK_WINDOW(mdata->main), title);
+	g_free(title);
+    }
+}
+
 void set_sample_label (DATAINFO *pdinfo)
 {
     GtkWidget *dlabel;
