@@ -155,9 +155,9 @@ static int model_table_precheck (MODEL *pmod, int add_mode)
 	return 1;
     }    
 
-    /* NLS, MLE and GMM models won't work */
+    /* NLS, MLE and GMM models won't work, nor INTREG, IVREG */
     if (pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM ||
-	pmod->ci == ARBOND) {
+	pmod->ci == ARBOND || pmod->ci == INTREG || pmod->ci == IVREG) {
 	mtable_errmsg(_("Sorry, this model can't be put in the model table"),
 		      gui);
 	return 1;
@@ -170,13 +170,6 @@ static int model_table_precheck (MODEL *pmod, int add_mode)
 	return 1;
     }
 
-    /* nor TSLS */
-    if (pmod->ci == TSLS) {
-	mtable_errmsg(_("Sorry, TSLS models can't be put in the model table"),
-		      gui);
-	return 1;
-    }    
-
     if (n_models > 0) {
 	int dv = model_table_depvar();
 
@@ -369,17 +362,17 @@ static const char *short_estimator_string (const MODEL *pmod, PRN *prn)
 	    return N_("WLS");
 	}
     } else if (pmod->ci == PANEL) {
-	if (gretl_model_get_int(pmod, "fixed-effects")) {
+	if (pmod->opt & OPT_F) {
 	    return N_("Within");
-	} else if (gretl_model_get_int(pmod, "random-effects")) {
+	} else if (pmod->opt & OPT_U) {
 	    return N_("GLS");
 	} else {
 	    return N_("Between");
 	}
     } else if (pmod->ci == AR1) {
-	if (gretl_model_get_int(pmod, "hilu")) {
+	if (pmod->opt & OPT_H) {
 	    return N_("HILU");
-	} else if (gretl_model_get_int(pmod, "pwe")) {
+	} else if (pmod->opt & OPT_P) {
 	    return N_("PWE");
 	} else {
 	    return N_("CORC");
