@@ -22,7 +22,6 @@
 #include "gretl.h"
 #include "session.h"
 #include "gpt_control.h"
-#include "boxplots.h"
 #include "objectsave.h"
 
 #include "cmd_private.h"
@@ -112,6 +111,8 @@ int maybe_save_model (const CMD *cmd, MODEL *pmod, PRN *prn)
 	}
 	if (!err) {
 	    set_as_last_model(cpy, GRETL_OBJ_EQN);
+	} else {
+	    errmsg(err, prn);
 	}
     } else {
 	set_as_last_model(pmod, GRETL_OBJ_EQN);
@@ -161,10 +162,6 @@ int maybe_save_graph (const CMD *cmd, const char *fname, GretlObjType type,
     char gname[MAXSAVENAME];
     int ret, err = 0;
 
-    if (type == GRETL_OBJ_PLOT && fname == NULL) {
-	fname = get_boxdump_name();
-    }
-
     gretl_cmd_get_savename(gname);
     if (*gname == 0) {
 	return 0;
@@ -310,7 +307,7 @@ int saved_object_action (const char *line, PRN *prn)
     /* special: display icon view window */
     if (!strncmp(line, "iconview", 8)) {
 	if (data_status) {
-	    view_session();
+	    view_session(NULL);
 	    return 1;
 	} else {
 	    return -1;
