@@ -30,12 +30,17 @@ struct iterinfo_t {
 
 void clear_selector (void);
 
-void selection_dialog (const char *title, int (*callback)(), guint cmdcode,
-		       int preselect);
+void selection_dialog (const char *title, int (*callback)(), guint cmdcode);
 
 void simple_selection (const char *title, int (*callback)(), guint cmdcode,
 		       gpointer p);
 
+void modelspec_dialog (int ci);
+
+void selector_set_varnum (int v);
+
+void selector_from_model (void *ptr, int ci);
+
 char *main_window_selection_as_string (void);
 
 void data_save_selection_wrapper (int file_code, gpointer p);
@@ -50,6 +55,10 @@ gpointer selector_get_data (const selector *sr);
 
 gretlopt selector_get_opts (const selector *sr);
 
+int selector_get_depvar_number (const selector *sr);
+
+int selector_get_VAR_order (const selector *sr);
+
 const char *selector_entry_text (const selector *sr);
 
 int selector_error (const selector *sr);
