@@ -22,7 +22,6 @@
 #include "gretl.h"
 #include "session.h"
 #include "selector.h"
-#include "boxplots.h"
 #include "ssheet.h"
 #include "plotspec.h"
 #include "gpt_control.h"
@@ -45,7 +44,6 @@
 #include "system.h"
 #include "gretl_xml.h"
 #include "gretl_func.h"
-#include "modelspec.h"
 #include "matrix_extra.h"
 #include "cmd_private.h"
 #include "gretl_scalar.h"
@@ -84,8 +82,9 @@ typedef struct SESSION_GRAPH_ SESSION_GRAPH;
 typedef struct gui_obj_ gui_obj;
 
 enum {
-    SESSION_CHANGED = 1 << 0,
-    SESSION_SAVED   = 1 << 1
+    SESSION_CHANGED  = 1 << 0,
+    SESSION_SAVED    = 1 << 1,
+    SESSION_EMBEDDED = 1 << 2
 };
 
 struct SESSION_ {
@@ -189,6 +188,11 @@ static char *dataset_items[] = {
     N_("Copy as CSV...")
 };
 
+static char *scalars_items[] = {
+    N_("Edit"),
+    N_("Copy as CSV...")
+};
+
 static char *info_items[] = {
     N_("View"),
     N_("Edit"),
@@ -206,7 +210,7 @@ static char *matrix_items[] = {
 
 SESSION session;            /* hold models, graphs */
 
-static int session_file_open;
+static char sessionfile[MAXLEN];
 
 static GtkWidget *iconview;
 static GtkWidget *icon_table;
@@ -216,8 +220,8 @@ static GtkWidget *model_table_popup;
 static GtkWidget *generic_popup;
 static GtkWidget *graph_popup;
 static GtkWidget *graph_page_popup;
-static GtkWidget *boxplot_popup;
 static GtkWidget *data_popup;
+static GtkWidget *scalars_popup;
 static GtkWidget *info_popup;
 static GtkWidget *matrix_popup;
 static GtkWidget *save_item;
@@ -232,20 +236,21 @@ static gint in_icon;
 static gui_obj *gui_object_new (gchar *name, int sort, gpointer data);
 static gui_obj *session_add_icon (gpointer data, int sort, int mode);
 static void session_build_popups (void);
-static void global_popup_activated (GtkWidget *widget, gpointer data);
-static void object_popup_activated (GtkWidget *widget, gpointer data);
-static void data_popup_activated (GtkWidget *widget, gpointer data);
-static void info_popup_activated (GtkWidget *widget, gpointer data);
-static void matrix_popup_activated (GtkWidget *widget, gpointer data);
+static void global_popup_callback (GtkWidget *widget, gpointer data);
+static void object_popup_callback (GtkWidget *widget, gpointer data);
+static void data_popup_callback (GtkWidget *widget, gpointer data);
+static void scalars_popup_callback (GtkWidget *widget, gpointer data);
+static void info_popup_callback (GtkWidget *widget, gpointer data);
+static void matrix_popup_callback (GtkWidget *widget, gpointer data);
 static void session_delete_icon (gui_obj *obj);
 static void open_gui_graph (gui_obj *obj);
-static void open_boxplot (gui_obj *obj);
 static gboolean session_view_click (GtkWidget *widget, 
 				    GdkEventButton *event,
 				    gpointer data);
 static int real_delete_model_from_session (SESSION_MODEL *model);
 static void make_short_label_string (char *targ, const char *src);
 static gui_obj *get_gui_obj_by_data (void *finddata);
+static void add_user_matrix_callback (void);
 
 static int session_graph_count;
 static int session_bplot_count;
@@ -261,6 +266,9 @@ void mark_session_changed (void)
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, TRUE);
 	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", TRUE);
+	if (*session.name != '\0') {
+	    set_main_window_title(session.name, TRUE);
+	}
     }
 }
 
@@ -270,6 +278,7 @@ static void mark_session_saved (void)
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, FALSE);
 	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", FALSE);
+	set_main_window_title(session.name, FALSE);
     }
 }
 
@@ -415,6 +424,16 @@ session_append_graph (const char *grname,
     return graph;
 }
 
+static void session_switch_log_location (int code)
+{
+    gchar *fullpath;
+
+    fullpath = g_strdup_printf("%s%s%c", paths.dotdir, 
+			       session.dirname, SLASH);
+    set_session_log(fullpath, code);
+    g_free(fullpath);
+}
+
 /* first arg should be a MAXLEN string */
 
 static char *session_file_make_path (char *path, const char *fname)
@@ -629,10 +648,14 @@ void save_output_as_text_icon (windata_t *vwin)
     ensure_unique_text_name(tname);
     err = session_append_text(tname, buf);
 
-    if (!err && iconlist != NULL) {
-	SESSION_TEXT * text = get_session_text_by_name(tname);
+    if (err) {
+	return;
+    } else if (iconlist != NULL) {
+	    SESSION_TEXT * text = get_session_text_by_name(tname);
 
-	session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
+	    session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
+    } else if (autoicon_on()) {
+	view_session(NULL);
     }
 }
 
@@ -653,6 +676,8 @@ static int real_add_model_to_session (void *ptr, const char *name,
 
     if (iconlist != NULL) {
 	session_add_icon(mod, type, ICON_ADD_SINGLE);
+    } else if (autoicon_on()) {
+	view_session(NULL);
     }
 
     return 0;
@@ -682,6 +707,10 @@ real_add_graph_to_session (const char *fname, const char *grname,
 	session_add_icon(graph, type, ICON_ADD_SINGLE);
     }
 
+    if (autoicon_on() && iconlist == NULL) {
+	view_session(NULL);
+    }
+
     return (replace)? ADD_OBJECT_REPLACE : ADD_OBJECT_OK;
 }
 
@@ -700,7 +729,7 @@ static int session_dir_ok (void)
 	errno = 0;
 
 	sprintf(session.dirname, ".gretl-%d", (int) p);
-	if (chdir(paths.dotdir)) {
+	if (gretl_chdir(paths.dotdir)) {
 	    perror("moving to user directory");
 	    ret = 0;
 	} else if (gretl_mkdir(session.dirname)) {
@@ -711,13 +740,13 @@ static int session_dir_ok (void)
     return ret;
 }
 
-int add_graph_to_session (char *fname, char *fullname)
+int add_graph_to_session (char *fname, char *fullname, int type)
 {
     char shortname[MAXSAVENAME];
     char graphname[MAXSAVENAME];
     int err = 0;
 
-    if (Z == NULL) {
+    if (type != GRETL_OBJ_PLOT && Z == NULL) {
 	/* we may be called via the "stats calculator" when
 	   there's no dataset yet */
 	err = open_nulldata(&Z, datainfo, 0, 10, NULL);
@@ -735,21 +764,27 @@ int add_graph_to_session (char *fname, char *fullname)
 	return 1;
     }
 
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
 
-    sprintf(shortname, "graph.%d", session_graph_count + 1);
-    session_file_make_path(fullname, shortname);
-    sprintf(graphname, "%s %d", _("Graph"), session_graph_count + 1);
+    if (type == GRETL_OBJ_PLOT) {
+	sprintf(shortname, "plot.%d", session_bplot_count + 1);
+	session_file_make_path(fullname, shortname);
+	sprintf(graphname, "%s %d", _("Boxplot"), session_bplot_count + 1);
+    } else {
+	sprintf(shortname, "graph.%d", session_graph_count + 1);
+	session_file_make_path(fullname, shortname);
+	sprintf(graphname, "%s %d", _("Graph"), session_graph_count + 1);
+    }
 
     /* move temporary plot file to permanent */
     if (copyfile(fname, fullname)) {
 	return 1;
     } 
 
-    remove(fname);
+    gretl_remove(fname);
     strcpy(fname, shortname);
 
-    err = real_add_graph_to_session(shortname, graphname, GRETL_OBJ_GRAPH);
+    err = real_add_graph_to_session(shortname, graphname, type);
     if (err == ADD_OBJECT_FAIL) {
 	err = 1;
     }
@@ -757,32 +792,6 @@ int add_graph_to_session (char *fname, char *fullname)
     return err;
 }
 
-void add_boxplot_to_session (const char *boxtmp)
-{
-    char fname[MAXSAVENAME];
-    char grname[MAXSAVENAME];
-    char grpath[MAXLEN];
-
-    errno = 0;
-
-    if (!session_dir_ok()) {
-	errbox(_("Failed to copy graph file"));
-	return;
-    }
-
-    chdir(paths.dotdir);
-
-    sprintf(fname, "plot.%d", session_bplot_count + 1);
-    session_file_make_path(grpath, fname);
-    sprintf(grname, "%s %d", _("Boxplot"), session_bplot_count + 1);
-
-    if (copyfile(boxtmp, grpath)) {
-	return;
-    } 
-
-    real_add_graph_to_session(fname, grname, GRETL_OBJ_PLOT);
-}
-
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type)
 {
@@ -796,7 +805,7 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	return ADD_OBJECT_FAIL;
     }
 
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
 
     strcpy(name, gname);
     space_to_score(name);
@@ -806,7 +815,7 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	errbox(_("Failed to copy graph file"));
 	return ADD_OBJECT_FAIL;
     } 
-    remove(fname);
+    gretl_remove(fname);
 
     return real_add_graph_to_session(name, gname, type);
 }
@@ -972,7 +981,7 @@ static int unzip_session_file (const char *fname, char **zdirname)
 	return 1;
     }
     
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
 
     gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
 					       &handle);
@@ -1053,6 +1062,8 @@ static int set_session_dirname (const char *zdirname)
     FILE *fp;
     int err = 0;
 
+    g_return_val_if_fail(zdirname != NULL, 1);
+
     strcpy(session.dirname, zdirname);
     sprintf(test, "%s%csession.xml", session.dirname, SLASH);
     fp = gretl_fopen(test, "r");
@@ -1066,11 +1077,15 @@ static int set_session_dirname (const char *zdirname)
     return err;
 }
 
+/* note: the name of the file to be opened is in the global var
+   'tryfile' */
+
 void do_open_session (void)
 {
     struct sample_info sinfo;
-    char sname[MAXLEN];
-    char fname[MAXLEN];
+    char xmlname[MAXLEN]; /* path to master session XML file */
+    char gdtname[MAXLEN]; /* path to session data file */
+    char fname[MAXLEN];   /* multi-purpose temp variable */
     gchar *zdirname = NULL;
     FILE *fp;
     int err = 0;
@@ -1089,64 +1104,74 @@ void do_open_session (void)
     /* close existing session, if any, and initialize */
     close_session(NULL, &Z, datainfo, OPT_NONE);
 
-    strcpy(sessionfile, tryfile);
-    fprintf(stderr, I_("\nReading session file %s\n"), sessionfile);
+    fprintf(stderr, I_("\nReading session file %s\n"), tryfile);
 
-    chdir(paths.dotdir);
-    err = unzip_session_file(sessionfile, &zdirname);
+    gretl_chdir(paths.dotdir);
+    err = unzip_session_file(tryfile, &zdirname);
 
     if (err) {
 	/* FIXME more explicit error message */
 	g_free(zdirname);
-	file_read_errbox(sessionfile);
-	return;
+	file_read_errbox(tryfile);
+	goto bailout;
     } 
 
-    session_name_from_session_file(sname, sessionfile);
-    strcpy(session.name, sname);
+    session_name_from_session_file(session.name, tryfile);
 
     err = set_session_dirname(zdirname);
     if (err) {
 	g_free(zdirname);
 	fprintf(stderr, "Failed on set_session_dirname\n");
 	file_read_errbox("session.xml");
-	return;
+	goto bailout;
     }
 
     g_free(zdirname);
-    session_file_make_path(fname, "session.xml");
-    err = read_session_xml(fname, &sinfo);
+    session_file_make_path(xmlname, "session.xml");
+
+    /* try getting the name of the session data file first */
+    err = get_session_datafile_name(xmlname, &sinfo);
 
     if (err) {
-	/* FIXME more explicit error message */
 	fprintf(stderr, "Failed on read_session_xml: err = %d\n", err);
 	file_read_errbox("session.xml");
-	return;
+	goto bailout;
     }
 
+    /* construct path to session data file */
     session_file_make_path(paths.datfile, sinfo.datafile);
     fp = gretl_fopen(paths.datfile, "r");
 
     if (fp != NULL) {
-	/* OK */
-	strcpy(fname, paths.datfile);
+	/* OK, write good name into gdtname */
+	strcpy(gdtname, paths.datfile);
 	fclose(fp);
     } else {
-	/* try remedial action */
+	/* try remedial action, transform filename? */
 	fprintf(stderr, "'%s' : not found, trying to fix\n", paths.datfile);
-	err = get_session_dataname(fname, &sinfo);
+	err = get_session_dataname(gdtname, &sinfo);
     }
 
     if (!err) {
-	err = gretl_read_gdt(fname, NULL, &Z, datainfo, OPT_P, NULL);
+	err = gretl_read_gdt(gdtname, NULL, &Z, datainfo, OPT_P, NULL);
     }
 
     if (err) {
 	/* FIXME more explicit error message */
 	file_read_errbox(sinfo.datafile);
-	return;
+	goto bailout;
     } else {
-	fprintf(stderr, "Opened session datafile '%s'\n", fname);
+	fprintf(stderr, "Opened session datafile '%s'\n", gdtname);
+	data_status = USER_DATA;
+    }
+
+    /* having opened the data file, get the rest of the info from
+       session.xml */    
+    err = read_session_xml(xmlname, &sinfo);
+    if (err) {
+	fprintf(stderr, "Failed on read_session_xml: err = %d\n", err);
+	file_read_errbox("session.xml");
+	goto bailout;
     }
 
     session_file_make_path(fname, "matrices.xml");
@@ -1170,7 +1195,7 @@ void do_open_session (void)
 
     if (err) {
 	errbox(_("Couldn't set sample"));
-	return;
+	goto bailout;
     }
 
     datainfo->t1 = sinfo.t1;
@@ -1183,14 +1208,23 @@ void do_open_session (void)
 	free(sinfo.mask);
     }
 
-    mkfilelist(FILE_LIST_SESSION, sessionfile);
+    set_main_window_title(session.name, FALSE);
 
-    /* sync gui with session */
-    session_file_open = 1;
-    session_menu_state(TRUE);
+ bailout:
 
-    view_session();
-    mark_session_saved();
+    if (err) {
+	delete_from_filelist(FILE_LIST_SESSION, tryfile);
+    } else {
+	strcpy(sessionfile, tryfile);
+	mkfilelist(FILE_LIST_SESSION, sessionfile);
+
+	/* sync gui with session */
+	session_menu_state(TRUE);
+
+	view_session(NULL);
+	mark_session_saved();
+	session_switch_log_location(LOG_OPEN);
+    }
 }
 
 int is_session_file (const char *fname)
@@ -1233,11 +1267,11 @@ static void remove_session_dir (void)
     char *fullpath = g_strdup_printf("%s%s", paths.dotdir,
 				     session.dirname);
 
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
     gretl_deltree(session.dirname);
 #endif
 }
@@ -1257,6 +1291,7 @@ void session_init (void)
     *session.dirname = '\0';
 
     winstack_init();
+    set_matrix_add_callback(add_user_matrix_callback);    
 }
 
 void free_session (void)
@@ -1348,7 +1383,7 @@ int highest_numbered_variable_in_session (void)
 
 int session_file_is_open (void)
 {
-    return (session_file_open && *sessionfile != '\0');
+    return (*sessionfile != '\0');
 }
 
 void gui_clear_dataset (void)
@@ -1370,8 +1405,7 @@ void gui_clear_dataset (void)
     main_menubar_state(FALSE);
 }
 
-static void 
-session_clear_data (double ***pZ, DATAINFO *pdinfo, int realclean)
+static void session_clear_data (double ***pZ, DATAINFO *pdinfo)
 {
     gui_restore_sample(pZ, pdinfo);
     gui_clear_dataset();
@@ -1381,66 +1415,56 @@ session_clear_data (double ***pZ, DATAINFO *pdinfo, int realclean)
     clear_model(models[1]);
     clear_model(models[2]);
 
-    if (realclean) {
-	free_command_stack(); 
-    }
-
-    free_modelspec();
+    free_command_stack(); 
     reset_model_count();
-
     lib_cmd_destroy_context();
 }
 
 void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
 		    gretlopt opt)
 {
-    int realclean = 1;
-
-    /* if we're running the "command log" to get text output
-       (SESSION_EXEC), then don't destroy saved stuff */
-    if (s != NULL && (s->flags & SESSION_EXEC)) {
-	realclean = 0;
-    }
+    int logcode = LOG_NULL;
 
 #if SESSION_DEBUG
     fprintf(stderr, "close_session: starting cleanup\n");
 #endif
-    session_clear_data(pZ, pdinfo, realclean); 
 
-    if (realclean) {
-	free_session();
-
-	clear_model_table(NULL);
-	clear_graph_page();
-
-	session_menu_state(FALSE);
-	session_file_open = 0;
-	*scriptfile = '\0';
-	*sessionfile = '\0';
-
-	if (iconview != NULL) {
-	    gtk_widget_destroy(iconview);
-	}
-
-	session.status = 0;
+    if (pdinfo != NULL && pdinfo->v > 0) {
+	logcode = LOG_CLOSE;
+	session_clear_data(pZ, pdinfo); 
     }
 
+    free_session();
+
+    clear_model_table(NULL);
+    clear_graph_page();
+
+    session_menu_state(FALSE);
+    *scriptfile = '\0';
+    *sessionfile = '\0';
+
+    if (iconview != NULL) {
+	gtk_widget_destroy(iconview);
+    }
+
+    session.status = 0;
+
     winstack_destroy();
     clear_selector();
     edit_dialog_special_get_text(NULL);
 
-    if (realclean) {
-	if (opt & OPT_P) {
-	    libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
-	} else {
-	    libgretl_session_cleanup(SESSION_CLEAR_FULL);
-	}
+    if (opt & OPT_P) {
+	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
+    } else {
+	libgretl_session_cleanup(SESSION_CLEAR_FULL);
     }
 
     session_graph_count = 0;
     session_bplot_count = 0;
 
     reset_plot_count();
+
+    set_session_log(NULL, logcode);
 }
 
 static int session_overwrite_check (const char *fname)
@@ -1555,7 +1579,10 @@ int save_session (char *fname)
     int len, err = 0;
     GError *gerr = NULL;
 
-    if (session_overwrite_check(fname)) {
+    if (fname == NULL) {
+	/* saving session 'as is' */
+	fname = sessionfile;
+    } else if (session_overwrite_check(fname)) {
 	return 0;
     }
 
@@ -1574,17 +1601,21 @@ int save_session (char *fname)
     }
 
     /* paths below are relative to this */
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
 
     if (strcmp(dirname, session.dirname)) {
 	/* rename session directory */
-	rename(session.dirname, dirname);
+	gretl_rename(session.dirname, dirname);
 	strcpy(session.dirname, dirname);
     }
 
     make_session_dataname(datname);
     write_session_xml(datname);
     err = save_session_dataset(datname);
+
+    if (!err) {
+	session_switch_log_location(LOG_SAVE);
+    }
     
     if (!err) {
 	gretl_make_zipfile = gui_get_plugin_function("gretl_make_zipfile", 
@@ -1604,6 +1635,9 @@ int save_session (char *fname)
     } else {
 	mkfilelist(FILE_LIST_SESSION, fname);
 	mark_session_saved();
+	if (fname != sessionfile) {
+	    strcpy(sessionfile, fname);
+	}
     }
 
     return err;
@@ -1663,6 +1697,8 @@ static gchar *graph_str (SESSION_GRAPH *graph)
     session_file_make_path(tmp, graph->fname);
     fp = gretl_fopen(tmp, "r");
 
+    /* FIXME boxplots */
+
     if (fp != NULL) {
 	char line[128], title[64];
 	char xlabel[24], ylabel[24];
@@ -1685,6 +1721,7 @@ static gchar *graph_str (SESSION_GRAPH *graph)
 
 #ifdef ENABLE_NLS
 	if (gottitle) {
+	    /* FIXME? encoding */
 	    buf = my_locale_to_utf8(title);
 	} else if (gotxy == 2) {
 	    char *s = g_strdup_printf("%s %s %s", ylabel, _("versus"), xlabel);
@@ -1708,35 +1745,6 @@ static gchar *graph_str (SESSION_GRAPH *graph)
     return buf;
 }
 
-static char *boxplot_str (SESSION_GRAPH *graph)
-{
-    FILE *fp;
-    char *str = NULL;
-
-    fp = gretl_fopen(graph->fname, "r");
-
-    if (fp != NULL) {
-	char vname[VNAMELEN], line[48];
-
-	str = malloc(MAXLEN);
-	if (str == NULL) {
-	    return NULL;
-	}
-	*str = '\0';
-
-	while (fgets(line, 47, fp) && strlen(str) < MAXLEN-48) {
-	    chopstr(line);
-	    if (sscanf(line, "%*d varname = %15s", vname) == 1) { 
-		strcat(str, strchr(line, '=') + 2);
-		strcat(str, " ");
-	    }
-	}
-	fclose(fp);
-    }
-
-    return str;
-}
-
 static int maybe_raise_object_window (gpointer p)
 {
     GtkWidget *w = match_window_by_data(p);
@@ -1777,7 +1785,7 @@ static int display_session_model (SESSION_MODEL *sm)
 	int cancel = 0;
 
 	edit_dialog(sm->name, NULL, NULL, do_saved_eqn_system, sys, 
-		    SYSTEM, VARCLICK_NONE, &cancel); 
+		    ESTIMATE, VARCLICK_NONE, &cancel); 
     }
 
     return 0;
@@ -1806,15 +1814,6 @@ void session_model_callback (void *ptr, int action)
     } 
 }	
 
-static void open_boxplot (gui_obj *obj)
-{
-    SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
-    char tmp[MAXLEN];
-
-    session_file_make_path(tmp, graph->fname);
-    retrieve_boxplot(tmp);
-}
-
 static void open_matrix (gui_obj *obj)
 {
     user_matrix *um = (user_matrix *) obj->data;
@@ -1908,9 +1907,9 @@ static void remove_session_graph_file (const char *gfname)
 {
     char fname[MAXLEN];
 
-    chdir(paths.dotdir);
+    gretl_chdir(paths.dotdir);
     session_file_make_path(fname, gfname);
-    remove(fname);
+    gretl_remove(fname);
 }
 
 static int real_delete_graph_from_session (SESSION_GRAPH *junk)
@@ -2326,7 +2325,11 @@ static void add_user_matrix_callback (void)
 	    session_add_icon(get_user_matrix_by_index(n-1), GRETL_OBJ_MATRIX, 
 			     ICON_ADD_SINGLE);
 	}
+    } else if (autoicon_on()) {
+	view_session(NULL);
     }
+
+    mark_session_changed();
 }
 
 static void add_all_icons (void) 
@@ -2336,15 +2339,14 @@ static void add_all_icons (void)
 
     active_object = NULL;
 
-    set_matrix_add_callback(add_user_matrix_callback);
-
     if (data_status) {
-	session_add_icon(NULL, GRETL_OBJ_INFO,   ICON_ADD_BATCH);  /* data info */
-	session_add_icon(NULL, GRETL_OBJ_DSET,   ICON_ADD_BATCH);  /* data file */
-	session_add_icon(NULL, GRETL_OBJ_NOTES,  ICON_ADD_BATCH);  /* session notes */
-	session_add_icon(NULL, GRETL_OBJ_STATS,  ICON_ADD_BATCH);  /* summary stats */
-	session_add_icon(NULL, GRETL_OBJ_CORR,   ICON_ADD_BATCH);  /* correlation matrix */
-	session_add_icon(NULL, GRETL_OBJ_MODTAB, ICON_ADD_BATCH);  /* model table */
+	session_add_icon(NULL, GRETL_OBJ_INFO,    ICON_ADD_BATCH);  /* data info */
+	session_add_icon(NULL, GRETL_OBJ_DSET,    ICON_ADD_BATCH);  /* data file */
+	session_add_icon(NULL, GRETL_OBJ_SCALARS, ICON_ADD_BATCH);  /* scalars */
+	session_add_icon(NULL, GRETL_OBJ_NOTES,   ICON_ADD_BATCH);  /* session notes */
+	session_add_icon(NULL, GRETL_OBJ_STATS,   ICON_ADD_BATCH);  /* summary stats */
+	session_add_icon(NULL, GRETL_OBJ_CORR,    ICON_ADD_BATCH);  /* correlation matrix */
+	session_add_icon(NULL, GRETL_OBJ_MODTAB,  ICON_ADD_BATCH);  /* model table */
 	if (show_graph_page) {
 	    session_add_icon(NULL, GRETL_OBJ_GPAGE, ICON_ADD_BATCH); /* graph page */
 	}
@@ -2432,14 +2434,15 @@ static void object_popup_show (gui_obj *obj, GdkEventButton *event)
 	w = generic_popup; 
 	break;
     case GRETL_OBJ_GRAPH: 
-	w = graph_popup; 
-	break;
     case GRETL_OBJ_PLOT: 
-	w = boxplot_popup; 
+	w = graph_popup; 
 	break;
     case GRETL_OBJ_DSET: 
 	w = data_popup; 
 	break;
+    case GRETL_OBJ_SCALARS: 
+	w = scalars_popup; 
+	break;
     case GRETL_OBJ_INFO: 
 	w = info_popup; 
 	break;
@@ -2495,10 +2498,8 @@ static gboolean session_view_click (GtkWidget *widget,
 	case GRETL_OBJ_SYS:
 	    display_session_model(obj->data); 
 	    break;
-	case GRETL_OBJ_PLOT:
-	    open_boxplot(obj); 
-	    break;
 	case GRETL_OBJ_GRAPH:
+	case GRETL_OBJ_PLOT:
 	    open_gui_graph(obj); 
 	    break;
 	case GRETL_OBJ_TEXT:
@@ -2507,6 +2508,9 @@ static gboolean session_view_click (GtkWidget *widget,
 	case GRETL_OBJ_DSET:
 	    show_spreadsheet(SHEET_EDIT_DATASET); 
 	    break;
+	case GRETL_OBJ_SCALARS:
+	    edit_scalars();
+	    break;
 	case GRETL_OBJ_MATRIX:
 	    open_matrix(obj); 
 	    break;
@@ -2540,7 +2544,7 @@ static gboolean session_view_click (GtkWidget *widget,
 	    obj->sort == GRETL_OBJ_INFO || obj->sort == GRETL_OBJ_GPAGE ||
 	    obj->sort == GRETL_OBJ_PLOT || obj->sort == GRETL_OBJ_MODTAB ||
 	    obj->sort == GRETL_OBJ_VAR  || obj->sort == GRETL_OBJ_SYS ||
-	    obj->sort == GRETL_OBJ_MATRIX) {
+	    obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_SCALARS) {
 	    object_popup_show(obj, (GdkEventButton *) event);
 	}
 	return TRUE;
@@ -2549,14 +2553,14 @@ static gboolean session_view_click (GtkWidget *widget,
     return FALSE;
 }
 
-static void global_popup_activated (GtkWidget *widget, gpointer data)
+static void global_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
 
     if (!strcmp(item, _("Save session"))) {
-	if (session_file_open && sessionfile[0]) {
+	if (sessionfile[0]) {
 	    if (session.status == SESSION_CHANGED) {
-		save_session(sessionfile);
+		save_session(NULL);
 	    } else {
 		mark_session_saved();
 	    }
@@ -2570,7 +2574,7 @@ static void global_popup_activated (GtkWidget *widget, gpointer data)
     }
 }
 
-static void info_popup_activated (GtkWidget *widget, gpointer data)
+static void info_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
 
@@ -2581,7 +2585,7 @@ static void info_popup_activated (GtkWidget *widget, gpointer data)
     }
 }
 
-static void matrix_popup_activated (GtkWidget *widget, gpointer data)
+static void matrix_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
     gui_obj *obj = active_object;
@@ -2609,7 +2613,7 @@ static void matrix_popup_activated (GtkWidget *widget, gpointer data)
     }
 }
 
-static void data_popup_activated (GtkWidget *widget, gpointer data)
+static void data_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
 
@@ -2624,7 +2628,28 @@ static void data_popup_activated (GtkWidget *widget, gpointer data)
     }
 }
 
-static void object_popup_activated (GtkWidget *widget, gpointer data)
+static void scalars_popup_callback (GtkWidget *widget, gpointer data)
+{
+    gchar *item = (gchar *) data;
+
+    if (!strcmp(item, _("Edit"))) {
+	edit_scalars();
+    } else if (!strcmp(item, _("Copy as CSV..."))) {
+	scalars_to_clipboard_as_csv();
+    }
+}
+
+static void object_set_window_title (windata_t *vwin, gui_obj *obj)
+{
+    if (vwin != NULL && obj != NULL) {
+	gchar *title = g_strdup_printf("gretl: %s", obj->name);
+
+	gtk_window_set_title(GTK_WINDOW(vwin->main), title);
+	g_free(title);	
+    }
+}
+
+static void object_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
     gui_obj *obj = active_object;
@@ -2640,21 +2665,21 @@ static void object_popup_activated (GtkWidget *widget, gpointer data)
 	    display_model_table_wrapper();
 	} else if (obj->sort == GRETL_OBJ_GPAGE) {
 	    display_graph_page();
-	} else if (obj->sort == GRETL_OBJ_GRAPH) {
+	} else if (obj->sort == GRETL_OBJ_GRAPH || 
+		   obj->sort == GRETL_OBJ_PLOT) {
 	    open_gui_graph(obj);
-	} else if (obj->sort == GRETL_OBJ_PLOT) {
-	    open_boxplot(obj);
-	}
+	} 
     } else if (!strcmp(item, _("Edit plot commands"))) {
 	if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
 	    SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
 	    char fullname[MAXLEN];
+	    windata_t *vwin;
 
-	    chdir(paths.dotdir);
+	    gretl_chdir(paths.dotdir);
 	    session_file_make_path(fullname, graph->fname);
 	    remove_png_term_from_plotfile_by_name(fullname);
-	    view_file(fullname, 1, 0, 78, 400, 
-		      (obj->sort == GRETL_OBJ_GRAPH)? EDIT_GP : EDIT_BOX);
+	    vwin = view_file(fullname, 1, 0, 78, 400, EDIT_GP);
+	    object_set_window_title(vwin, obj);
 	}
     } else if (!strcmp(item, _("Rename"))) {
 	rename_object_dialog(obj);
@@ -2750,7 +2775,7 @@ static void session_drag_setup (gui_obj *obj)
                        targ, 1,
                        GDK_ACTION_COPY);
 
-    g_signal_connect (G_OBJECT(w), "drag_data_received",
+    g_signal_connect (G_OBJECT(w), "drag-data-received",
                       G_CALLBACK(session_data_received),
                       NULL);
 }
@@ -2769,7 +2794,7 @@ static void graph_drag_connect (GtkWidget *w, SESSION_GRAPH *graph)
     gtk_drag_source_set(w, GDK_BUTTON1_MASK,
                         &gretl_drag_targets[GRETL_GRAPH_FILE],
                         1, GDK_ACTION_COPY);
-    g_signal_connect(G_OBJECT(w), "drag_data_get",
+    g_signal_connect(G_OBJECT(w), "drag-data-get",
                      G_CALLBACK(drag_graph), graph);
 }
 
@@ -2787,7 +2812,7 @@ static void model_drag_connect (GtkWidget *w, SESSION_MODEL *mod)
     gtk_drag_source_set(w, GDK_BUTTON1_MASK,
                         &gretl_drag_targets[GRETL_MODEL_PTR],
                         1, GDK_ACTION_COPY);
-    g_signal_connect(G_OBJECT(w), "drag_data_get",
+    g_signal_connect(G_OBJECT(w), "drag-data-get",
                      G_CALLBACK(drag_model), mod);
 }
 
@@ -2824,6 +2849,9 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
     case GRETL_OBJ_DSET:
 	name = g_strdup(_("Data set"));
 	break;
+    case GRETL_OBJ_SCALARS:
+	name = g_strdup(_("Scalars"));
+	break;
     case GRETL_OBJ_INFO:
 	name = g_strdup(_("Data info"));
 	break;
@@ -2878,11 +2906,10 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	    MODEL *pmod = (MODEL *) mod->ptr;
 
 	    str = model_cmd_str(pmod);
-	} else if (sort == GRETL_OBJ_GRAPH) {
+	} else if (sort == GRETL_OBJ_GRAPH ||
+		   sort == GRETL_OBJ_PLOT) {
 	    str = graph_str(graph);
-	} else if (sort == GRETL_OBJ_PLOT) {
-	    str = boxplot_str(graph);
-	}
+	} 
 	if (str != NULL) {
 	    gretl_tooltips_add(GTK_WIDGET(obj->icon), str);
 	    free(str);
@@ -2902,8 +2929,8 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
     return obj;
 }
 
-static GtkWidget *create_popup_item (GtkWidget *popup, char *str, 
-				     GtkCallback callback)
+static GtkWidget *create_pop_item (GtkWidget *popup, char *str, 
+				   GtkCallback callback)
 {
     GtkWidget *item;
 
@@ -2923,96 +2950,98 @@ static GtkWidget *create_popup_item (GtkWidget *popup, char *str,
 
 static void session_build_popups (void)
 {
-    size_t i;
+    size_t i, n;
 
     if (global_popup == NULL) {
 	global_popup = gtk_menu_new();
-	for (i=0; i<sizeof global_items / sizeof global_items[0]; i++) {
-	    create_popup_item(global_popup, 
-			      _(global_items[i]), 
-			      global_popup_activated);
+	n = G_N_ELEMENTS(global_items);
+	if (session.status & SESSION_EMBEDDED) {
+	    n--;
+	}
+	for (i=0; i<n; i++) {
+	    create_pop_item(global_popup, _(global_items[i]), 
+			    global_popup_callback);
 	}
     }
 
     if (model_popup == NULL) {
 	model_popup = gtk_menu_new();
-	for (i=0; i<sizeof model_items / sizeof model_items[0]; i++) {
-	    create_popup_item(model_popup, 
-			      _(model_items[i]), 
-			      object_popup_activated);
+	n = G_N_ELEMENTS(model_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(model_popup, _(model_items[i]), 
+			    object_popup_callback);
 	}
     }
 
     if (model_table_popup == NULL) {
 	model_table_popup = gtk_menu_new();
-	for (i=0; i<sizeof model_table_items / sizeof model_table_items[0]; i++) {
-	    create_popup_item(model_table_popup, 
-			      _(model_table_items[i]), 
-			      object_popup_activated);
+	n = G_N_ELEMENTS(model_table_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(model_table_popup, _(model_table_items[i]), 
+			    object_popup_callback);
 	}
     }
 
     if (generic_popup == NULL) {
 	generic_popup = gtk_menu_new();
-	for (i=0; i<sizeof generic_items / sizeof generic_items[0]; i++) {
-	    create_popup_item(generic_popup, 
-			      _(generic_items[i]), 
-			      object_popup_activated);
+	n = G_N_ELEMENTS(generic_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(generic_popup, _(generic_items[i]), 
+			    object_popup_callback);
 	}
     }
 
     if (graph_popup == NULL) {
 	graph_popup = gtk_menu_new();
-	for (i=0; i<sizeof graph_items / sizeof graph_items[0]; i++) {
-	    create_popup_item(graph_popup, 
-			      _(graph_items[i]), 
-			      object_popup_activated);
+	n = G_N_ELEMENTS(graph_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(graph_popup, _(graph_items[i]), 
+			    object_popup_callback);
 	}
     }
 
     if (graph_page_popup == NULL) {
 	graph_page_popup = gtk_menu_new();
-	for (i=0; i<sizeof graph_page_items / sizeof graph_page_items[0]; i++) {
-	    create_popup_item(graph_page_popup, 
-			      _(graph_page_items[i]), 
-			      object_popup_activated);
-	}
-    }
-
-    if (boxplot_popup == NULL) {
-	boxplot_popup = gtk_menu_new();
-	for (i=0; i<sizeof graph_items / sizeof graph_items[0]; i++) {
-	    if (strstr(graph_items[i], "GUI")) continue;
-	    create_popup_item(boxplot_popup, 
-			      _(graph_items[i]), 
-			      object_popup_activated);
+	n = G_N_ELEMENTS(graph_page_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(graph_page_popup, _(graph_page_items[i]), 
+			    object_popup_callback);
 	}
     }
 
     if (data_popup == NULL) {
 	data_popup = gtk_menu_new();
-	for (i=0; i<sizeof dataset_items / sizeof dataset_items[0]; i++) {
-	    create_popup_item(data_popup, 
-			      _(dataset_items[i]), 
-			      data_popup_activated);	    
+	n = G_N_ELEMENTS(dataset_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(data_popup, _(dataset_items[i]), 
+			    data_popup_callback);	    
+	}
+    }
+
+    if (scalars_popup == NULL) {
+	scalars_popup = gtk_menu_new();
+	n = G_N_ELEMENTS(scalars_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(scalars_popup, _(scalars_items[i]), 
+			    scalars_popup_callback);	    
 	}
     }
 
     if (info_popup == NULL) {
 	info_popup = gtk_menu_new();
-	for (i=0; i<sizeof info_items / sizeof info_items[0]; i++) {
-	    create_popup_item(info_popup, 
-			      _(info_items[i]), 
-			      info_popup_activated);	    
+	n = G_N_ELEMENTS(info_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(info_popup, _(info_items[i]), 
+			    info_popup_callback);	    
 	}
     }
 
     if (matrix_popup == NULL) {
 	matrix_popup = gtk_menu_new();
-	for (i=0; i<sizeof matrix_items / sizeof matrix_items[0]; i++) {
-	    create_popup_item(matrix_popup, 
-			      _(matrix_items[i]), 
-			      matrix_popup_activated);	    
+	n = G_N_ELEMENTS(matrix_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(matrix_popup, _(matrix_items[i]), 
+			    matrix_popup_callback);	    
 	}
     }
 }
@@ -3039,35 +3068,46 @@ static void iconview_connect_signals (GtkWidget *iconview)
 {
     g_signal_connect(G_OBJECT(iconview), "destroy",
 		     G_CALLBACK(session_view_free), NULL);
-    g_signal_connect(G_OBJECT(iconview), "key_press_event",
+    g_signal_connect(G_OBJECT(iconview), "key-press-event",
 		     G_CALLBACK(catch_iconview_key), NULL);
     g_signal_connect(G_OBJECT(iconview), "configure-event",
 		     G_CALLBACK(iconview_resize_callback), NULL);
 }
 
-void view_session (void)
+/* @parent: this should be non-NULL only if want to embed the "icon
+   view" of a session inside another widget, which is something that
+   I've experimented with, briefly, but is not an active option as of
+   this writing. AC, 2009-05-02
+*/
+
+void view_session (GtkWidget *parent)
 {
     GtkWidget *hbox, *scroller;
     gchar *title;
 
     if (iconview != NULL) {
-	gdk_window_show(iconview->window);
-	gdk_window_raise(iconview->window);
+	if (parent == NULL) {
+	    gtk_window_present(GTK_WINDOW(iconview));
+	}
 	return;
     }
 
     session_view_init();
 
-    title = g_strdup_printf("gretl: %s", 
-			    (session.name[0])? session.name : 
-			    _("current session"));
-    
-    iconview = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(iconview), title);
-    g_free(title);
-    gtk_window_set_default_size(GTK_WINDOW(iconview), 400, 300);
-    gtk_container_set_border_width(GTK_CONTAINER(iconview), 0);
+    if (parent == NULL) {
+	iconview = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	title = g_strdup_printf("gretl: %s", _("icon view"));
+	gtk_window_set_title(GTK_WINDOW(iconview), title);
+	g_free(title);
+	gtk_window_set_default_size(GTK_WINDOW(iconview), 400, 300);
+	session.status &= ~SESSION_EMBEDDED;
+    } else {
+	iconview = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent), iconview, TRUE, TRUE, 0);
+	session.status |= SESSION_EMBEDDED;
+    }
 
+    gtk_container_set_border_width(GTK_CONTAINER(iconview), 0);
     iconview_connect_signals(iconview);
 
     session_build_popups();
@@ -3080,7 +3120,7 @@ void view_session (void)
     gtk_container_set_border_width(GTK_CONTAINER(scroller), 0);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    g_signal_connect(G_OBJECT(scroller), "button_press_event",
+    g_signal_connect(G_OBJECT(scroller), "button-press-event",
 		     G_CALLBACK(session_view_click), NULL);
 
     gtk_box_pack_start(GTK_BOX(hbox), scroller, TRUE, TRUE, 0); 
@@ -3107,9 +3147,8 @@ void view_session (void)
 
 static void make_short_label_string (char *targ, const char *src)
 {
-    if (strlen(src) > SHOWNAMELEN) {
-	*targ = '\0';
-	strncat(targ, src, SHOWNAMELEN - 3);
+    if (g_utf8_strlen(src, -1) > SHOWNAMELEN) {
+	g_utf8_strncpy(targ, src, SHOWNAMELEN - 3);
 	strcat(targ, "...");
     } else {
 	strcpy(targ, src);
@@ -3143,11 +3182,11 @@ static void create_gobj_icon (gui_obj *obj, const char **xpm)
     g_object_ref(obj->icon);
     g_object_ref(obj->label);
 
-    g_signal_connect(G_OBJECT(obj->icon), "button_press_event",
+    g_signal_connect(G_OBJECT(obj->icon), "button-press-event",
 		     G_CALLBACK(session_view_click), obj);
-    g_signal_connect(G_OBJECT(obj->icon), "enter_notify_event",
+    g_signal_connect(G_OBJECT(obj->icon), "enter-notify-event",
 		     G_CALLBACK(icon_entered), obj);
-    g_signal_connect(G_OBJECT(obj->icon), "leave_notify_event",
+    g_signal_connect(G_OBJECT(obj->icon), "leave-notify-event",
 		     G_CALLBACK(icon_left), obj);
 }
 
@@ -3172,6 +3211,7 @@ static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
     case GRETL_OBJ_PLOT:    xpm = boxplot_xpm;     break;
     case GRETL_OBJ_GRAPH:   xpm = gnuplot_xpm;     break;
     case GRETL_OBJ_DSET:    xpm = dot_sc_xpm;      break;
+    case GRETL_OBJ_SCALARS: xpm = dot_sc_xpm;      break;
     case GRETL_OBJ_INFO:    xpm = xfm_info_xpm;    break;
     case GRETL_OBJ_TEXT:
     case GRETL_OBJ_NOTES:   xpm = text_xpm;        break;
@@ -3222,11 +3262,35 @@ static int is_idempotent (const gretl_matrix *m,
     return gretl_matrix_is_idempotent(m);
 }
 
+static void print_int_formatted (char *s, int k, PRN *prn)
+{
+    int len = 12, n = strlen(s) - g_utf8_strlen(s, -1);
+    char fmt[16];
+
+    if (n > 0) {
+	len += n;
+    }
+
+    sprintf(fmt, "%%-%ds %%3d\n", len);
+    pprintf(prn, fmt, s, k);
+}
+
+static void print_double_formatted (char *s, double x, PRN *prn)
+{
+    int len = 16, n = strlen(s) - g_utf8_strlen(s, -1);
+    char fmt[16];
+
+    if (n > 0) {
+	len += n;
+    }
+
+    sprintf(fmt, "%%-%ds %%.8g\n", len);    
+    pprintf(prn, fmt, s, x);    
+}
+
 void 
 view_matrix_properties (const gretl_matrix *m, const char *name)
 {
-    const char *xfmt = "%-16s %.8g\n";
-    const char *ifmt = "%-12s %3d\n";
     gretl_matrix *A = NULL;
     gretl_matrix *evals = NULL;
     PRN *prn;
@@ -3250,9 +3314,9 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 	goto done;
     } 
 
-    pprintf(prn, ifmt, _("Rows"), m->rows);
-    pprintf(prn, ifmt, _("Columns"), m->cols);
-    pprintf(prn, ifmt, _("Rank"), gretl_matrix_rank(m, &err));
+    print_int_formatted(_("Rows"), m->rows, prn);
+    print_int_formatted(_("Columns"), m->cols, prn);
+    print_int_formatted(_("Rank"), gretl_matrix_rank(m, &err), prn);
 
     s = gretl_matrix_get_structure(m);
 
@@ -3298,13 +3362,13 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 
     pputc(prn, '\n');
 
-    pprintf(prn, xfmt, _("1-norm"), gretl_matrix_one_norm(m));
-    pprintf(prn, xfmt, _("Infinity-norm"), gretl_matrix_infinity_norm(m));
+    print_double_formatted(_("1-norm"), gretl_matrix_one_norm(m), prn);
+    print_double_formatted(_("Infinity-norm"), gretl_matrix_infinity_norm(m), prn);
 	    
     if (m->rows == m->cols) {
 	double det;
 
-	pprintf(prn, xfmt, _("Trace"), gretl_matrix_trace(m, &err));
+	print_double_formatted(_("Trace"), gretl_matrix_trace(m, &err), prn);
 	if (A == NULL) {
 	    A = gretl_matrix_copy(m);
 	} else {
@@ -3313,7 +3377,7 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 	if (A != NULL) {
 	    det = gretl_matrix_determinant(A, &err);
 	    if (!err) {
-		pprintf(prn, xfmt, _("Determinant"), det);
+		print_double_formatted(_("Determinant"), det, prn);
 	    }
 	}
     }
