@@ -36,6 +36,13 @@ enum {
     ADD_OBJECT_FAIL
 };
 
+enum {
+    LOG_SAVE,
+    LOG_OPEN,
+    LOG_CLOSE,
+    LOG_NULL
+};
+
 int save_session (char *fname);
 
 int session_is_modified (void);
@@ -48,9 +55,7 @@ int real_add_text_to_session (PRN *prn, const char *tname);
 
 void save_output_as_text_icon (windata_t *vwin);
 
-int add_graph_to_session (char *fname, char *fullname);
-
-void add_boxplot_to_session (const char *boxtmp);
+int add_graph_to_session (char *fname, char *fullname, int type);
 
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type);
@@ -90,7 +95,7 @@ int is_session_file (const char *fname);
 
 int is_session_model (void *p);
 
-void view_session (void);
+void view_session (GtkWidget *parent);
 
 void save_session_callback (GtkAction *action);
 
