@@ -65,7 +65,7 @@ static void read_rc (void);
 static char fixedfontname[MAXLEN] = "Courier New 10";
 #else
 # ifdef OSX_BUILD
-static char fixedfontname[MAXLEN] = "Luxi Mono 12";
+static char fixedfontname[MAXLEN] = "Monospace 12";
 # else
 static char fixedfontname[MAXLEN] = "Monospace 10";
 # endif
@@ -73,12 +73,10 @@ static char fixedfontname[MAXLEN] = "Monospace 10";
 
 #if defined(G_OS_WIN32)
 static char appfontname[MAXLEN] = "tahoma 8";
-#elif !defined(USE_GNOME)
-# ifdef OSX_BUILD
-static char appfontname[MAXLEN] = "Luxi Sans 12";
+#elif defined(OSX_BUILD)
+static char appfontname[MAXLEN] = "Sans 12";
 # else
 static char appfontname[MAXLEN] = "Sans 10";
-# endif
 #endif
 
 PangoFontDescription *fixed_font;
@@ -86,12 +84,15 @@ PangoFontDescription *fixed_font;
 static int usecwd;
 static int shellok;
 static int manpref;
-char gpcolors[32];
+static int autoicon = 1;
+char gpcolors[64];
 static char datapage[24];
 static char scriptpage[24];
 
 static int hc_by_default;
-static int langpref;
+#ifdef ENABLE_NLS
+static char langpref[32];
+#endif
 static char hc_xsect[5] = "HC1";
 static char hc_tseri[5] = "HAC";
 static char hc_panel[9] = "Arellano";
@@ -173,12 +174,14 @@ RCVAR rc_vars[] = {
 #endif
     { "shellok", N_("Allow shell commands"), NULL, &shellok, 
       BOOLSET, 0, TAB_MAIN, NULL },
-#ifdef ENABLE_NLS
-    { "langpref", N_("Language preference"), NULL, &langpref, 
-      RADIOSET | INTSET, 0, TAB_MAIN, NULL },
-#endif
+    { "autoicon", N_("Show icon view automatically"), NULL, &autoicon, 
+      BOOLSET, 0, TAB_MAIN, NULL },
     { "usecwd", N_("Set working directory from shell"), NULL, &usecwd, 
       INVISET | BOOLSET, 0, TAB_NONE, NULL },
+#ifdef ENABLE_NLS
+    { "langpref", N_("Language preference"), NULL, langpref, 
+      LISTSET, 32, TAB_MAIN, NULL },
+#endif
 #ifndef G_OS_WIN32 
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       MACHSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
@@ -221,10 +224,8 @@ RCVAR rc_vars[] = {
       BOOLSET, 1, TAB_DBS, NULL },
     { "Fixed_font", N_("Fixed font"), NULL, fixedfontname, 
       USERSET, MAXLEN, TAB_NONE, NULL },
-#if !defined(USE_GNOME)
     { "App_font", N_("Menu font"), NULL, appfontname, 
       USERSET, MAXLEN, TAB_NONE, NULL },
-#endif
     { "DataPage", "Default data page", NULL, datapage, 
       INVISET, sizeof datapage, TAB_NONE, NULL },
     { "ScriptPage", "Default script page", NULL, scriptpage, 
@@ -292,6 +293,11 @@ const char *get_scriptpage (void)
     return scriptpage;
 }
 
+int autoicon_on (void)
+{
+    return autoicon;
+}
+
 static gretlopt set_paths_opt = OPT_X;
 
 void force_english_help (void)
@@ -326,6 +332,7 @@ static void set_gd_fontpath (void)
     }
 
     gdpath = getenv("GDFONTPATH");
+
     if (gdpath != NULL) {
 	if (strstr(gdpath, "gretl") == NULL &&
 	    strstr(gdpath, "GRETL") == NULL) {
@@ -337,6 +344,8 @@ static void set_gd_fontpath (void)
     }
 
     if (newpath != NULL) {
+	/* Vera is supplied with gretl, so it should work */
+	strcpy(paths.pngfont, "Vera 9");
 	setenv("GDFONTPATH", newpath, 1);
 	g_free(newpath);
 	gnuplot_has_ttf(1);
@@ -350,21 +359,19 @@ static void record_shell_opt (void)
     sprintf(shellstamp, "%s.gretl_shell_stamp", paths.dotdir);
 
     if (shellok) {
-	FILE *fp = fopen(shellstamp, "w");
+	FILE *fp = gretl_fopen(shellstamp, "w");
 
 	if (fp != NULL) {
 	    fputs("ok\n", fp);
 	    fclose(fp);
 	}
     } else {
-	remove(shellstamp);
+	gretl_remove(shellstamp);
     }
 }
 
 #endif
 
-#ifndef USE_GNOME
-
 const char *get_app_fontname (void)
 {
     return appfontname;
@@ -401,8 +408,6 @@ void set_app_font (const char *fontname)
     }
 }
 
-#endif
-
 static void slash_terminate (char *path)
 {
     if (path == NULL || *path == '\0') return;
@@ -412,13 +417,24 @@ static void slash_terminate (char *path)
     }
 }
 
-static void get_functions_dir (char *dirname)
+static void get_pkg_dir (char *dirname, int action)
 {
+    const char *subdir = NULL;
     char *target = NULL;
     FILE *fp;
     int err, ok = 0;
 
-    sprintf(dirname, "%sfunctions", paths.gretldir);
+    if (action == SAVE_FUNCTIONS) {
+	subdir = "functions";
+    } else if (action == SAVE_DATA_PKG) {
+	subdir = "data";
+    } else {
+	return;
+    }
+
+    /* try 'system' location first */
+
+    sprintf(dirname, "%s%s", paths.gretldir, subdir);
     err = gretl_mkdir(dirname);
     if (!err) {
 	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
@@ -427,7 +443,7 @@ static void get_functions_dir (char *dirname)
 	    if (fp != NULL) {
 		ok = 1;
 		fclose(fp);
-		remove(target);
+		gretl_remove(target);
 	    }
 	    free(target);
 	}
@@ -435,19 +451,28 @@ static void get_functions_dir (char *dirname)
     
     if (ok) return;
 
-    sprintf(dirname, "%sfunctions", paths.dotdir); /* ?? */
-    err = gretl_mkdir(dirname);
-    if (!err) {
-	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
-	if (target != NULL) {
-	    fp = gretl_fopen(target, "w");
-	    if (fp != NULL) {
-		ok = 1;
-		fclose(fp);
-		remove(target);
-	    }
-	    free(target);
+    /* try user's directory */
+
+    if (action == SAVE_DATA_PKG) {
+	strcpy(dirname, paths.workdir);
+	target = g_strdup_printf("%s%s", dirname, "wtest");
+    } else {
+	/* should we really use dotdir here? */
+	sprintf(dirname, "%s%s", paths.dotdir, subdir);
+	err = gretl_mkdir(dirname);
+	if (!err) {
+	    target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
 	}
+    } 
+
+    if (target != NULL) {
+	fp = gretl_fopen(target, "w");
+	if (fp != NULL) {
+	    ok = 1;
+	    fclose(fp);
+	    gretl_remove(target);
+	}
+	free(target);
     }
 
     if (ok) return;
@@ -495,8 +520,8 @@ void get_default_dir (char *s, int action)
 {
     *s = '\0';
 
-    if (action == SAVE_FUNCTIONS) {
-	get_functions_dir(s);
+    if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
+	get_pkg_dir(s, action);
     } else if (action == OPEN_RATS_DB) {
 	strcpy(s, paths.ratsbase);
     } else {
@@ -628,7 +653,11 @@ int is_executable (const char *s, uid_t myid, gid_t mygrp)
     struct stat buf;
     int ok = 0;
 
-    if (stat(s, &buf) == 0 && (buf.st_mode & (S_IFREG|S_IFLNK))) {
+    if (gretl_stat(s, &buf) != 0) {
+	return 0;
+    }
+
+    if (buf.st_mode & (S_IFREG|S_IFLNK)) {
 	if (buf.st_uid == myid && (buf.st_mode & S_IXUSR)) {
 	    ok = 1;
 	} else if (buf.st_gid == mygrp && (buf.st_mode & S_IXGRP)) {
@@ -738,7 +767,14 @@ static void root_check (void)
 
 void gretl_config_init (void)
 {
-    sprintf(rcfile, "%s/.gretl2rc", getenv("HOME"));
+    char *custprof = getenv("GRETL_PROFILE");
+
+    if (custprof == NULL) {
+	sprintf(rcfile, "%s/.gretl2rc", getenv("HOME"));
+    } else {
+	strcpy(rcfile, custprof);
+    }
+
     read_rc();
     set_gretl_startdir();
     set_gd_fontpath();
@@ -898,6 +934,37 @@ static gboolean takes_effect_on_restart (void)
     return FALSE;
 }
 
+#ifdef ENABLE_NLS
+
+static gboolean try_switch_locale (GtkComboBox *box, gpointer p)
+{
+    static int lasterr;
+    gchar *langstr;
+    int err;
+
+    if (lasterr) {
+	lasterr = 0;
+	return FALSE;
+    }
+
+    langstr = gtk_combo_box_get_active_text(box);
+    err = test_locale(langstr);
+    g_free(langstr);
+
+    if (err) {
+	lasterr = err;
+	gui_errmsg(err);
+	gretl_error_clear();
+	gtk_combo_box_set_active(box, 0);
+    } else {
+	infobox(_("This change will take effect when you restart gretl"));
+    }
+
+    return FALSE;
+}
+
+#endif
+
 #define HIDE_SPANISH_MANUAL 1
 
 static const char **get_radio_setting_strings (void *var, int *n)
@@ -908,11 +975,6 @@ static const char **get_radio_setting_strings (void *var, int *n)
         N_("Italian"),
 	N_("Spanish")
     };
-    static const char *lang_strs[] = {
-        N_("Use local language if possible"),
-        N_("Use English"),
-	N_("Use Basque")
-    };
     const char **strs = NULL;
 
     *n = 0;
@@ -923,10 +985,7 @@ static const char **get_radio_setting_strings (void *var, int *n)
 #if HIDE_SPANISH_MANUAL
 	*n -= 1;
 #endif
-    } else if (var == &langpref) {
-	strs = lang_strs;
-	*n = sizeof lang_strs / sizeof lang_strs[0];
-    }
+    } 
 
     return strs;
 }
@@ -962,7 +1021,8 @@ static const char **get_list_setting_strings (void *var, int *n)
 }
 
 static void 
-get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse)
+get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse,
+		 int *n_list)
 {
     int i;
 
@@ -970,6 +1030,8 @@ get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse)
 	if (rc_vars[i].tab == page) {
 	    if (rc_vars[i].flags & BROWSER) {
 		*n_browse += 1;
+	    } else if (rc_vars[i].flags & LISTSET) {
+		*n_list += 1;
 	    }
 	    if (rc_vars[i].flags & BOOLSET) {
 		*n_bool += 1;
@@ -977,7 +1039,7 @@ get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse)
 		*n_bool += 1;
 	    } else if (!(rc_vars[i].flags & INVISET)) {
 		*n_str += 1;
-	    }
+	    } 
 	}
     }
 }
@@ -987,9 +1049,6 @@ static void radio_change_value (GtkWidget *w, int *v)
     if (GTK_TOGGLE_BUTTON(w)->active) {
 	gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
 
-	if (v == &langpref) {
-	    takes_effect_on_restart();
-	}
 	*v = i;
     }
 }
@@ -997,13 +1056,16 @@ static void radio_change_value (GtkWidget *w, int *v)
 static void make_prefs_tab (GtkWidget *notebook, int tab) 
 {
     GtkWidget *b_table = NULL, *s_table = NULL;
+    GtkWidget *l_table = NULL;
     GtkWidget *box, *w = NULL;
-    int s_len = 1, b_len = 0;
-    int s_cols, b_cols = 0;
+    int s_len = 1, b_len = 0, l_len = 1;
+    int s_cols, b_cols = 0, l_cols = 0;
     int b_col = 0;
     int n_str = 0;
     int n_bool = 0;
     int n_browse = 0;
+    int n_list = 0;
+    int langs = 0;
     RCVAR *rc;
     int i;
    
@@ -1026,10 +1088,20 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     gtk_widget_show(w);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, w);   
 
-    get_table_sizes(tab, &n_str, &n_bool, &n_browse);
+    get_table_sizes(tab, &n_str, &n_bool, &n_browse, &n_list);
 
     s_cols = (n_browse > 0)? 3 : 2;
 
+    if (tab == TAB_VCV && n_list > 0) {
+	/* VCV tab -- put the list entries first, right aligned */
+	l_cols = 2;
+	l_table = gtk_table_new(l_len, l_cols, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(l_table), 5);
+	gtk_table_set_col_spacings(GTK_TABLE(l_table), 5);
+	gtk_box_pack_start(GTK_BOX(box), l_table, FALSE, FALSE, 0);
+	gtk_widget_show(l_table);
+    }	
+
     if (n_str > 0) {
 	s_table = gtk_table_new(s_len, s_cols, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(s_table), 5);
@@ -1047,6 +1119,17 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	gtk_widget_show(b_table);
     }
 
+    if (tab != TAB_VCV && n_list > 0) {
+	/* non-VCV tab: language list comes last */
+	l_cols = 2;
+	langs = 1;
+	l_table = gtk_table_new(l_len, l_cols, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(l_table), 5);
+	gtk_table_set_col_spacings(GTK_TABLE(l_table), 5);
+	gtk_box_pack_start(GTK_BOX(box), l_table, FALSE, FALSE, 0);
+	gtk_widget_show(l_table);
+    }
+
     for (i=0; rc_vars[i].key != NULL; i++) {
 	rc = &rc_vars[i];
 
@@ -1157,38 +1240,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_widget_set_sensitive(button, FALSE);
 		gtk_widget_set_sensitive(rc->widget, FALSE);
 	    }
-	} else if (rc->flags & LISTSET) {
-	    char *strvar = (char *) rc->var;
-	    const char **strs;
-	    int j, ww, nopt, active = 0;
-
-	    s_len++;
-
-	    gtk_table_resize(GTK_TABLE(s_table), s_len, s_cols);
-	    w = gtk_label_new(_(rc->description));
-	    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
-	    gtk_table_attach_defaults(GTK_TABLE(s_table), 
-				      w, 0, 1, s_len - 1, s_len);
-	    gtk_widget_show(w);
-
-	    rc->widget = gtk_combo_box_new_text();
-	    gtk_table_attach(GTK_TABLE(s_table), rc->widget, 
-			     1, 2, s_len - 1, s_len,
-			     0, 0, 0, 0);
-	    strs = get_list_setting_strings(rc->var, &nopt);
-	    for (j=0; j<nopt; j++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), 
-					  strs[j]);
-		if (!strcmp(strs[j], strvar)) {
-		    active = j;
-		}
-	    }
-	    gtk_combo_box_set_active(GTK_COMBO_BOX(rc->widget), active);
-	    /* this will need generalizing if we add another LISTSET
-	       variable */
-	    ww = get_string_width("XXArellanoXXXXX");
-	    gtk_widget_set_size_request(rc->widget, ww, -1);
-	    gtk_widget_show(rc->widget);
 	} else if (rc->flags & RADIOSET) {
 	    int nopt, j, rcval = *(int *) (rc->var);
 	    int rcol;
@@ -1234,6 +1285,69 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 				 G_CALLBACK(radio_change_value),
 				 rc->var);
 	    }
+	} else if (rc->flags & LISTSET) {
+	    int j, active = 0;
+
+	    l_len++;
+
+	    gtk_table_resize(GTK_TABLE(l_table), l_len, l_cols);
+	    w = gtk_label_new(_(rc->description));
+	    if (langs) {
+		gtk_table_attach(GTK_TABLE(l_table), w, 
+				 0, 1, l_len - 1, l_len,
+				 0, 0, 0, 0);
+	    } else {		
+		gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
+		gtk_table_attach_defaults(GTK_TABLE(l_table), 
+					  w, 0, 1, l_len - 1, l_len);
+	    } 
+	    gtk_widget_show(w);
+
+	    rc->widget = gtk_combo_box_new_text();
+	    gtk_table_attach(GTK_TABLE(l_table), rc->widget, 
+			     1, 2, l_len - 1, l_len,
+			     0, 0, 0, 0);
+	    if (langs) {
+#ifdef ENABLE_NLS
+		char *strvar = (char *) rc->var;
+		const char *str;
+
+		for (j=LANG_AUTO; j<LANG_MAX; j++) {
+		    str = lang_string_from_id(j);
+		    gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), str);
+		    if (!strcmp(str, strvar)) {
+			active = j;
+		    }
+		}
+#endif
+	    } else {
+		char *strvar = (char *) rc->var;
+		const char **strs;
+		int nopt;
+		
+		strs = get_list_setting_strings(rc->var, &nopt);
+		for (j=0; j<nopt; j++) {
+		    gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), 
+					      strs[j]);
+		    if (!strcmp(strs[j], strvar)) {
+			active = j;
+		    }
+		}
+	    }
+	    if (tab == TAB_VCV) {
+		int ww = get_string_width("XXArellanoXXXXX");
+
+		gtk_widget_set_size_request(rc->widget, ww, -1);
+	    }
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(rc->widget), active);
+	    gtk_widget_show(rc->widget);
+#ifdef ENABLE_NLS
+	    if (langs) {
+		g_signal_connect(G_OBJECT(rc->widget), "changed",
+				 G_CALLBACK(try_switch_locale), 
+				 NULL);
+	    }
+#endif
 	} else if (!(rc->flags & INVISET)) { 
 	    /* visible string variable */
 	    char *strvar = (char *) rc->var;
@@ -1266,7 +1380,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_widget_set_sensitive(w, FALSE);
 	    }
 	} 
-    } 
+    }
 
     if (tab == TAB_VCV) {
 	/* we need a help button */
@@ -1288,102 +1402,37 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     }
 }
 
-#ifdef ENABLE_NLS
-
-# ifdef G_OS_WIN32
-
-struct langname {
-    const char *abbr;
-    const char *full;
-};
-
-static void set_lcnumeric (void)
-{
-    if (lcnumeric) {
-	struct langname names[] = {
-	    { "es", "Spanish" },
-	    { "eu", "Basque" },
-	    { "fr", "French" },
-	    { "it", "Italian" },
-	    { "pl", "Polish" },
-	    { "de", "German" },
-	    { "pt", "Portuguese" },
-	    { "ru", "Russian" },
-	    { NULL, NULL }
-	};
-	char *lang = getenv("LANG");
-	char *set = NULL;
-	int i;
-
-	if (lang != NULL) {
-	    for (i=0; names[i].abbr != NULL; i++) {
-		if (!strncmp(lang, names[i].abbr, 2)) {
-		    set = setlocale(LC_NUMERIC, names[i].full);
-		    if (set == NULL) {
-			set = setlocale(LC_NUMERIC, names[i].abbr);
-		    }
-		    if (set != NULL) {
-			break;
-		    }
-		}
-	    }
-	}
-
-	if (set == NULL) {
-	    setlocale(LC_NUMERIC, "");
-	    putenv("LC_NUMERIC=");
-	}
-    } else {
-	setlocale(LC_NUMERIC, "C");
-	putenv("LC_NUMERIC=C");
-    }
-
-    reset_local_decpoint();
-}
-
-# else /* ! G_OS_WIN32 */
-
-static void set_lcnumeric (void)
-{
-    if (lcnumeric) {
-	char *lang = getenv("LANG");
-	char *set = NULL;
-
-	if (lang != NULL) {
-	    set = setlocale(LC_NUMERIC, lang);
-	    fprintf(stderr, "setlocale(LC_NUMERIC, \"%s\") returned %s\n", 
-		    lang, set);
-	} else {
-	    fprintf(stderr, "set_lcnumeric: getenv(\"LANG\") gave NULL\n");
-	}
-	if (set == NULL) {
-	    setlocale(LC_NUMERIC, "");
-	    putenv("LC_NUMERIC=");
-	}
-    } else {
-	setlocale(LC_NUMERIC, "C");
-	putenv("LC_NUMERIC=C");
-    }
-
-    reset_local_decpoint();
-}
-
-# endif
-
-#endif /* ENABLE_NLS */
-
 static void set_gp_colors (void)
 {
+    const char *s = gpcolors;
     char cstr[N_GP_COLORS][8];
-    int i, nc;
+    int i, nc = 0;
 
-    *cstr[0] = *cstr[1] = *cstr[2] = *cstr[3] = '\0';
+    for (i=0; i<N_GP_COLORS; i++) {
+	if (sscanf(s, "%7s", cstr[i]) == 1) {
+	    nc++;
+	    s += 7;
+	    if (*s == ' ') {
+		s++;
+	    } else {
+		break;
+	    }
+	} else {
+	    *cstr[i] = '\0';
+	    break;
+	}
+    }
 
-    nc = sscanf(gpcolors, "%7s %7s %7s %7s", 
-		cstr[0], cstr[1], cstr[2], cstr[3]);
-
-    for (i=0; i<nc; i++) {
-	set_graph_palette_from_string(i, cstr[i]);
+    if (nc == 4) {
+	/* old-style */
+	for (i=0; i<3; i++) {
+	    set_graph_palette_from_string(i, cstr[i]);
+	}
+	set_graph_palette_from_string(BOXCOLOR, cstr[3]);
+    } else {
+	for (i=0; i<nc; i++) {
+	    set_graph_palette_from_string(i, cstr[i]);
+	}
     }
 }
 
@@ -1418,15 +1467,17 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 {
     const gchar *str;
     char *strvar;
+    int *intvar;
     int i = 0;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	if (rc_vars[i].widget != NULL) {
 	    if (rc_vars[i].flags & BOOLSET) {
+		intvar = (int *) rc_vars[i].var;
 		if (GTK_TOGGLE_BUTTON(rc_vars[i].widget)->active) {
-		    *(int *) (rc_vars[i].var) = TRUE;
+		    *intvar = TRUE;
 		} else {
-		    *(int *) (rc_vars[i].var) = FALSE;
+		    *intvar = FALSE;
 		}
 	    } else if ((rc_vars[i].flags & USERSET) || 
 		       (rc_vars[i].flags & ROOTSET) ||
@@ -1439,13 +1490,21 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 		}
 	    } else if (rc_vars[i].flags & LISTSET) {
 		GtkComboBox *box = GTK_COMBO_BOX(rc_vars[i].widget);
-		gchar *boxval;
 
-		boxval = gtk_combo_box_get_active_text(box);
-		strvar = (char *) rc_vars[i].var;
-		*strvar = '\0';
-		strcat(strvar, boxval);
-		g_free(boxval);
+		if (rc_vars[i].flags & INTSET) {
+		    int boxval = gtk_combo_box_get_active(box);
+		    
+		    intvar = (int *) rc_vars[i].var;
+		    *intvar = boxval;
+		} else {
+		    gchar *boxval;
+
+		    boxval = gtk_combo_box_get_active_text(box);
+		    strvar = (char *) rc_vars[i].var;
+		    *strvar = '\0';
+		    strcat(strvar, boxval);
+		    g_free(boxval);
+		}
 	    }
 	}
     }
@@ -1599,6 +1658,9 @@ static void maybe_fix_viewpdf (void)
 
 static int common_read_rc_setup (void)
 {
+# ifdef ENABLE_NLS
+    int langid = 0;
+# endif
     int err = 0;
 
     libset_set_bool(SHELL_OK, shellok);
@@ -1624,10 +1686,14 @@ static int common_read_rc_setup (void)
 # endif
 
 # ifdef ENABLE_NLS
-    set_lcnumeric();
-    if (langpref > 0) {
-	force_language(langpref);
-    }
+    langid = lang_id_from_name(langpref);
+    set_lcnumeric(langid, lcnumeric);
+    if (langid > 0) {
+	force_language(langid);
+	if (langid == LANG_C) {
+	    force_english_help();
+	}
+    } 
 # endif
 
     return err;
@@ -1639,7 +1705,7 @@ void write_rc (void)
 {
     char bval[6];
     char ival[16];
-    const char *strval;
+    char *strval;
     int i = 0, err = 0;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
@@ -1654,25 +1720,25 @@ void write_rc (void)
 	    err += write_reg_val(HKEY_CURRENT_USER, 
 				 "gretl", 
 				 rc_vars[i].key, 
-				 bval);
+				 bval, GRETL_TYPE_BOOL);
 	} else if (rc_vars[i].flags & INTSET) {
 	    sprintf(ival, "%d", *(int *) rc_vars[i].var);
 	    err += write_reg_val(HKEY_CURRENT_USER, 
 				 "gretl", 
 				 rc_vars[i].key, 
-				 ival);	    
+				 ival, GRETL_TYPE_INT);	    
 	} else if (rc_vars[i].flags & MACHSET) {
 	    strval = (char *) rc_vars[i].var;
 	    err += write_reg_val(HKEY_LOCAL_MACHINE, 
 				 get_reg_base(rc_vars[i].key),
 				 rc_vars[i].key, 
-				 strval);
+				 strval, GRETL_TYPE_STRING);
 	} else {
 	    strval = (char *) rc_vars[i].var;
 	    err += write_reg_val(HKEY_CURRENT_USER, 
 				 get_reg_base(rc_vars[i].key),
 				 rc_vars[i].key, 
-				 strval);
+				 strval, GRETL_TYPE_STRING);
 	}
     }
 
@@ -1735,12 +1801,16 @@ static int get_network_settings (void)
     return gotini;
 }
 
-void read_rc (void) 
+void read_rc (int debug) 
 {
     char value[MAXSTR];
     char *strvar;
     int i;
 
+    if (chinese_locale()) {
+	strcpy(fixedfontname, "MS Gothic 10");
+    }
+
     if (get_network_settings() && *paths.workdir != '\0') {
 	int err = set_gretl_work_dir(paths.workdir, &paths);
 
@@ -1781,6 +1851,11 @@ void read_rc (void)
 			       rc_vars[i].key, 
 			       value);
 	}
+
+        if (debug) {
+            fprintf(stderr, "reg: err = %d, '%s' -> '%s'\n", err, rc_vars[i].key,
+                    value);
+        }
 	    
 	if (!err && *value != '\0') {
 	    if (rc_vars[i].flags & BOOLSET) {
@@ -1907,24 +1982,15 @@ static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
 	strcpy(fixedfontname, fontname);
 	set_fixed_font();
 	write_rc();
-    } 
-
-# ifndef USE_GNOME /* gnome handles the app font */
-    else if (which == APP_FONT_SELECTION) {
+    } else if (which == APP_FONT_SELECTION) {
 	set_app_font(fontname);
 	write_rc();
     }
-# endif
 
     g_free(fontname);
     gtk_widget_destroy(GTK_WIDGET(fs));
 }
 
-static void fontsel_quit (GtkWidget *w, gpointer p)
-{
-    gtk_main_quit();
-}
-
 void font_selector (GtkAction *action)
 {
     static GtkWidget *fontsel = NULL;
@@ -1933,15 +1999,8 @@ void font_selector (GtkAction *action)
     char *title = NULL;
     const char *fontname = NULL;
 
-# ifdef USE_GNOME
-    if (which == APP_FONT_SELECTION) return; /* shouldn't happen */
-# endif
-
     if (fontsel != NULL) {
-	if (!GTK_WIDGET_VISIBLE(fontsel)) {
-	    gtk_widget_show(fontsel);
-	}
-        gdk_window_raise(fontsel->window);
+	gtk_window_present(GTK_WINDOW(fontsel));
         return;
     }
 
@@ -1949,14 +2008,10 @@ void font_selector (GtkAction *action)
 	title = _("Font for gretl output windows");
 	filter = GTK_FONT_HACK_LATIN_MONO;
 	fontname = fixedfontname;
-    } 
-
-# ifndef USE_GNOME
-    else if (which == APP_FONT_SELECTION) {
+    } else if (which == APP_FONT_SELECTION) {
 	title = _("Font for menus and labels");
 	fontname = appfontname;
     }
-# endif
 
     fontsel = gtk_fontsel_hack_dialog_new(title);
     gtk_fontsel_hack_dialog_set_filter
@@ -1971,7 +2026,7 @@ void font_selector (GtkAction *action)
 		     G_CALLBACK(gtk_widget_destroyed),
 		     &fontsel);
     g_signal_connect(G_OBJECT(fontsel), "destroy",
-		     G_CALLBACK(fontsel_quit),
+		     G_CALLBACK(gtk_main_quit),
 		     NULL);
     g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_ok_button(fontsel)),
 		     "clicked", 
@@ -1989,79 +2044,25 @@ void font_selector (GtkAction *action)
 
 #else /* end non-win32 font selection, start win32 */
 
-static const char *font_weight_string (int weight)
-{
-    if (weight >= FW_THIN && weight <= FW_LIGHT) {
-	return " Light";
-    }
-    if (weight >= FW_NORMAL && weight <= FW_DEMIBOLD) {
-	return "";
-    }
-    if (weight >= FW_BOLD) {
-	return " Bold";
-    }
-    return "";
-}
-
-static void fontname_to_win32 (const char *src, int fixed,
-			       char *name, int *pts)
-{
-    int sz;
-
-    if (sscanf(src, "%31[^0123456789]%d", name, &sz) == 2) {
-	size_t i, n = strlen(name);
-
-	for (i=n-1; i>0; i--) {
-	    if (name[i] == ' ') name[i] = 0;
-	    else break;
-	}
-	*pts = sz * 10; /* measured in tenths of a point */
-    } else {
-	*name = 0;
-	strncat(name, src, 31);
-	if (fixed) *pts = 100;
-	else *pts = 80;
-    }
-}
-
 void font_selector (GtkAction *action)
 {
-    int which = fontsel_code(action);
-    CHOOSEFONT cf;            /* common dialog box structure */
-    LOGFONT lf;               /* logical font structure */
-    char fontname[48];
+    int flag = fontsel_code(action);
+    char fontname[128];
 
-    ZeroMemory(&cf, sizeof cf);
-    cf.lStructSize = sizeof cf;
-    cf.Flags = CF_SCREENFONTS | CF_TTONLY | CF_LIMITSIZE | CF_INITTOLOGFONTSTRUCT;
-    cf.nSizeMin = 6;
-    cf.nSizeMax = 24;
+    if (flag == FIXED_FONT_SELECTION) {
+	strcpy(fontname, fixedfontname);
+    } else {
+	strcpy(fontname, appfontname);
+    }
 
-    ZeroMemory(&lf, sizeof lf);
+    win32_font_selector(fontname, flag);
 
-    cf.Flags |= CF_NOSCRIPTSEL;
-    
-    lf.lfWeight = FW_REGULAR;
-    lf.lfCharSet = DEFAULT_CHARSET;
-
-    if (which == FIXED_FONT_SELECTION) {
-	cf.Flags |= CF_FIXEDPITCHONLY;
-	fontname_to_win32(fixedfontname, 1, lf.lfFaceName, &(cf.iPointSize));
-    } else if (which == APP_FONT_SELECTION) {
-	fontname_to_win32(appfontname, 0, lf.lfFaceName, &(cf.iPointSize));
-    } 
-
-    cf.lpLogFont = &lf;
-
-    if (ChooseFont(&cf) == TRUE && *(cf.lpLogFont->lfFaceName)) {
-	sprintf(fontname, "%s%s %d", cf.lpLogFont->lfFaceName, 
-		font_weight_string(cf.lpLogFont->lfWeight), 
-		cf.iPointSize / 10);
-	if (which == FIXED_FONT_SELECTION) {
+    if (*fontname != '\0') {
+	if (flag == FIXED_FONT_SELECTION) {
 	    strcpy(fixedfontname, fontname);
 	    set_fixed_font();
 	    write_rc();
-	} else if (which == APP_FONT_SELECTION) {
+	} else {
 	    set_app_font(fontname);
 	    write_rc();
 	} 	    
@@ -2070,175 +2071,11 @@ void font_selector (GtkAction *action)
 
 #endif /* end win32 font selection */
 
-/* graph color selection apparatus */
-
-#define XPMROWS 19
-#define XPMCOLS 17
-
-#define scale_round(v) ((v) * 255.0 / 65535.0)
-
-static GtkWidget *get_image_for_color (const gretlRGB *color)
+void update_persistent_graph_colors (void)
 {
-    static char **xpm = NULL;
-    GdkPixbuf *icon;
-    GtkWidget *image;
-    char colstr[8] = {0};
-    int i;
-
-    if (color == NULL) {
-	return NULL;
-    }
-
-    if (xpm == NULL) {
-	xpm = strings_array_new_with_length(XPMROWS, XPMCOLS);
-	if (xpm != NULL) {
-	    for (i=0; i<XPMROWS; i++) {
-		if (i == 0) {
-		    strcpy(xpm[i], "16 16 2 1");
-		} else if (i == 1) {
-		    strcpy(xpm[i], "X      c #000000");
-		} else if (i == 2) {
-		    strcpy(xpm[i], ".      c #000000");
-		} else if (i == 3 || i == XPMROWS - 1) {
-		    strcpy(xpm[i], "................");
-		} else {
-		    strcpy(xpm[i], ".XXXXXXXXXXXXXX.");
-		}
-	    }
-	}
-    }
-
-    if (xpm == NULL) {
-	return NULL;
-    }
-
-    print_rgb_hash(colstr, color);
-
-    for (i=0; i<6; i++) {
-	xpm[1][10+i] = colstr[i+1];
-    }    
-
-    icon = gdk_pixbuf_new_from_xpm_data((const char **) xpm);
-    image = gtk_image_new_from_pixbuf(icon);
-    
-    return image;
-}
-
-static void color_select_callback (GtkWidget *button, GtkWidget *w)
-{
-    GtkWidget *csel;
-    GtkWidget *color_button, *image;
-    GdkColor gcolor;
-    gretlRGB rgb;
-    gint i;
-
-    color_button = g_object_get_data(G_OBJECT(w), "color_button");
-    csel = GTK_COLOR_SELECTION_DIALOG(w)->colorsel;
-
-    gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &gcolor);
-
-    rgb.r = (unsigned char) (scale_round(gcolor.red));
-    rgb.g = (unsigned char) (scale_round(gcolor.green));
-    rgb.b = (unsigned char) (scale_round(gcolor.blue));
-
-    i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "colnum"));
-
-    set_graph_palette(i, rgb);
     print_palette_string(gpcolors);
-
-    /* update the "image" widget */
-    image = g_object_get_data(G_OBJECT(color_button), "image");
-    gtk_widget_destroy(image);
-    image = get_image_for_color(&rgb);
-    gtk_widget_show(image);
-    gtk_container_add(GTK_CONTAINER(color_button), image);
-    g_object_set_data(G_OBJECT(color_button), "image", image);
-  
-    gtk_widget_destroy(w);
 }
 
-static void color_cancel (GtkWidget *button, GtkWidget *w)
-{
-    gtk_widget_destroy(w);
-}
-
-GtkWidget *color_patch_button (int cnum)
-{
-    GtkWidget *image, *button;
-
-    image = get_image_for_color(get_graph_color(cnum));
-
-    if (image == NULL) {
-	button = gtk_button_new_with_label(_("Select color"));
-    } else {
-	button = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(button), image);
-	g_object_set_data(G_OBJECT(button), "image", image);
-    }	
-
-    return button;
-}
-
-/* reset color patch button after selecting the option to
-   restore the default plot colors */
-
-void color_patch_button_reset (GtkWidget *button, int cnum)
-{
-    GtkWidget *image;
-
-    image = g_object_get_data(G_OBJECT(button), "image");
-    gtk_widget_destroy(image);
-    image = get_image_for_color(get_graph_color(cnum));
-    gtk_widget_show(image);
-    gtk_container_add(GTK_CONTAINER(button), image);
-    g_object_set_data(G_OBJECT(button), "image", image);
-
-    if (cnum == BOXCOLOR || cnum == BOXCOLOR - 1) {
-	print_palette_string(gpcolors);
-    }
-}
-
-void graph_color_selector (GtkWidget *w, gpointer p)
-{
-    GtkWidget *cdlg;
-    GtkWidget *button;
-    gint i = GPOINTER_TO_INT(p);
-    char colstr[8];
-    const gretlRGB *rgb;
-    GdkColor gcolor;
-
-    rgb = get_graph_color(i);
-    if (rgb == NULL) {
-	fprintf(stderr, "graph_get_color(%d) gave NULL\n", i);
-	return;
-    }
-
-    print_rgb_hash(colstr, rgb);
-    gdk_color_parse(colstr, &gcolor);
-
-    cdlg = gtk_color_selection_dialog_new(_("gretl: graph color selection"));
-
-    g_object_set_data(G_OBJECT(cdlg), "colnum", GINT_TO_POINTER(i));
-    g_object_set_data(G_OBJECT(cdlg), "color_button", w);
-
-    gtk_color_selection_set_current_color(GTK_COLOR_SELECTION
-					  (GTK_COLOR_SELECTION_DIALOG(cdlg)->colorsel),
-					  &gcolor);					  
-
-    button = GTK_COLOR_SELECTION_DIALOG(cdlg)->ok_button;
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(color_select_callback), cdlg);
-
-    button = GTK_COLOR_SELECTION_DIALOG(cdlg)->cancel_button;
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(color_cancel), cdlg);
-
-    gtk_widget_show(cdlg);
-    gtk_window_set_modal(GTK_WINDOW(cdlg), TRUE);
-}
-
-/* end graph color selection apparatus */
-
 void dump_rc (void) 
 {
     char dumper[MAXLEN];
