@@ -2,7 +2,7 @@
 #define SETTINGS_H
 
 #ifdef G_OS_WIN32
-void read_rc (void);
+void read_rc (int debug);
 #else
 void gretl_config_init (void);
 #endif
@@ -13,6 +13,8 @@ int using_hc_by_default (void);
 
 int get_manpref (void);
 
+int autoicon_on (void);
+
 void write_rc (void);
 
 void dump_rc (void);
@@ -25,20 +27,12 @@ void font_selector (GtkAction *action);
 
 void set_fixed_font (void);
 
-#ifndef USE_GNOME
+void update_persistent_graph_colors (void);
 
 void set_app_font (const char *fontname);
 
 const char *get_app_fontname (void);
 
-#endif
-
-void graph_color_selector (GtkWidget *w, gpointer p);
-
-GtkWidget *color_patch_button (int cnum);
-
-void color_patch_button_reset (GtkWidget *button, int cnum);
-
 void get_default_dir (char *s, int action);
 
 int gui_set_working_dir (char *dirname);
