@@ -66,9 +66,11 @@ typedef struct {
     GtkWidget *entry;
     GtkWidget *popup;
     GtkWidget *save;
+    GtkWidget *apply;
     GtkUIManager *ui;
     GtkCellRenderer *dumbcell;
     GtkCellRenderer *datacell;
+    GdkPixbuf *pbuf;
     gchar location[64];
     gretl_matrix *matrix;
     gretl_matrix *oldmat;
@@ -104,6 +106,7 @@ static void matrix_props_callback (GtkAction *action, gpointer data);
 static void matrix_edit_callback (GtkAction *action, gpointer data);
 static int update_sheet_from_matrix (Spreadsheet *sheet);
 static void size_matrix_window (Spreadsheet *sheet);
+static void set_ok_transforms (Spreadsheet *sheet);
 
 static void 
 spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col);
@@ -179,6 +182,8 @@ static void sheet_set_modified (Spreadsheet *sheet, gboolean s)
 
     if (sheet->save != NULL) {
 	gtk_widget_set_sensitive(sheet->save, s);
+    } else if (sheet->apply != NULL) {
+	gtk_widget_set_sensitive(sheet->apply, s);
     }
 }
 
@@ -477,6 +482,21 @@ maybe_update_store (Spreadsheet *sheet, const gchar *new_text,
     g_free(old_text);
 }
 
+static int scalar_try_genr (gchar **ps)
+{
+    double x;
+    int err = 0;
+
+    x = generate_scalar(*ps + 1, &Z, datainfo, &err);
+
+    if (!err) {
+	g_free(*ps);
+	*ps = g_strdup_printf("%.*g", DBL_DIG, x);
+    }
+
+    return err;
+}
+
 static void sheet_cell_edited (GtkCellRendererText *cell,
 			       const gchar *path_string,
 			       const gchar *user_text,
@@ -495,11 +515,15 @@ static void sheet_cell_edited (GtkCellRendererText *cell,
 	new_text = g_strdup("");
     } else {
 	new_text = g_strdup(user_text);
-	if (sheet->flags & SHEET_USE_COMMA) {
-	    /* accept point also: convert to locale */
-	    charsub(new_text, '.', ',');
+	if (*new_text == '=' && sheet->cmd == SHEET_EDIT_SCALARS) {
+	    err = scalar_try_genr(&new_text);
+	} else {
+	    if (sheet->flags & SHEET_USE_COMMA) {
+		/* accept point also: convert to locale */
+		charsub(new_text, '.', ',');
+	    }
+	    err = check_atof(new_text);
 	}
-	err = check_atof(new_text);
 	if (err) {
 	    errbox(gretl_errmsg_get());
 	    g_free(new_text);
@@ -566,7 +590,7 @@ static int real_add_new_series (Spreadsheet *sheet, const char *varname)
     }
 
 #if SSDEBUG
-    fprintf(stderr, "real_add_new_var, after add_data_column: sheet->totcols=%d\n", 
+    fprintf(stderr, "real_add_new_series, after add_data_column: sheet->totcols=%d\n", 
 	    sheet->totcols);
 #endif
 
@@ -581,7 +605,8 @@ static int real_add_new_series (Spreadsheet *sheet, const char *varname)
     return 0;
 }
 
-static int real_add_new_scalar (Spreadsheet *sheet, const char *vname)
+static int real_add_new_scalar (Spreadsheet *sheet, const char *vname,
+				double val)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkListStore *store;
@@ -589,7 +614,17 @@ static int real_add_new_scalar (Spreadsheet *sheet, const char *vname)
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
     gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, 0, vname, 1, "", -1);
+
+    if (na(val)) {
+	gtk_list_store_set(store, &iter, 0, vname, 1, "", -1);
+    } else {
+	gchar *numstr;
+
+	numstr = g_strdup_printf("%.*g", DBL_DIG, val);
+	gtk_list_store_set(store, &iter, 0, vname, 1, numstr, 
+			   2, sheet->pbuf, -1);
+	g_free(numstr);
+    }
 
     sheet->datarows += 1;
     spreadsheet_scroll_to_foot(sheet, sheet->datarows - 1, 1);
@@ -599,15 +634,6 @@ static int real_add_new_scalar (Spreadsheet *sheet, const char *vname)
     return 0;
 }
 
-static int real_add_new_var (Spreadsheet *sheet, const char *varname)
-{
-    if (sheet->cmd == SHEET_EDIT_SCALARS) {
-	return real_add_new_scalar(sheet, varname);
-    } else {
-	return real_add_new_series(sheet, varname);
-    }
-}
-
 static void 
 spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col)
 {
@@ -722,18 +748,66 @@ real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
     sheet_set_modified(sheet, TRUE);
 }
 
+static void add_new_scalar (GtkWidget *widget, dialog_t *dlg) 
+{
+    Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
+    const gchar *buf;
+    gchar *vname = NULL;
+    gchar *expr = NULL;
+    char varname[VNAMELEN];
+    double val = NADBL;
+    int err = 0;
+
+    buf = edit_dialog_get_text(dlg);
+
+    if (buf == NULL || *buf == '\0') {
+	return;
+    }
+
+    if (strchr(buf, '=')) {
+	gchar *p, *tmp = g_strdup(buf);
+
+	p = strchr(tmp, '=');
+	*p = '\0';
+	tailstrip(tmp);
+	vname = g_strdup(tmp);
+	expr = g_strdup(p + 1);
+	g_free(tmp);
+
+	if (gui_validate_varname(vname, GRETL_TYPE_DOUBLE)) {
+	    err = 1;
+	} else {
+	    *varname = 0;
+	    strncat(varname, vname, VNAMELEN - 1);
+	    val = generate_scalar(expr, &Z, datainfo, &err);
+	}
+    } else if (gui_validate_varname(buf, GRETL_TYPE_DOUBLE)) {
+	return;
+    } else {
+	*varname = 0;
+	strncat(varname, buf, VNAMELEN - 1);
+    }
+
+    g_free(vname);
+    g_free(expr);
+
+    if (!err) {
+	close_dialog(dlg);
+	if (real_add_new_scalar(sheet, varname, val)) {
+	    nomem();
+	}
+    }
+}
+
 static void name_new_var (GtkWidget *widget, dialog_t *dlg) 
 {
     Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
-    GretlType t;
     const gchar *buf;
     char varname[VNAMELEN];
 
-    t = (sheet->cmd == SHEET_EDIT_SCALARS)? GRETL_TYPE_DOUBLE :
-	GRETL_TYPE_SERIES;
-
     buf = edit_dialog_get_text(dlg);
-    if (buf == NULL || gui_validate_varname(buf, t)) {
+
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_SERIES)) {
 	return;
     }
 
@@ -742,7 +816,7 @@ static void name_new_var (GtkWidget *widget, dialog_t *dlg)
 
     close_dialog(dlg);
 
-    if (real_add_new_var(sheet, varname)) {
+    if (real_add_new_series(sheet, varname)) {
 	nomem();
     }
 }
@@ -774,6 +848,17 @@ static void name_var_dialog (Spreadsheet *sheet)
 		0, VARCLICK_NONE, &cancel);
 }
 
+static void new_scalar_dialog (Spreadsheet *sheet) 
+{
+    int cancel = 0;
+    
+    edit_dialog(_("gretl: scalar"), 
+		_("Enter name, or name=value,\n"
+		  "for new scalar"),
+		NULL, add_new_scalar, sheet, 
+		0, VARCLICK_NONE, &cancel);
+}
+
 static void new_case_dialog (Spreadsheet *sheet) 
 {
     int cancel = 0;
@@ -829,12 +914,17 @@ static GtkListStore *make_sheet_liststore (Spreadsheet *sheet)
 {
     GtkListStore *store;
     GType *types;
+    int ncols;
     int i;
 
     /* obs col, data cols */
-    sheet->totcols = sheet->datacols + 1;
+    ncols = sheet->totcols = sheet->datacols + 1;
 
-    types = mymalloc(sheet->totcols * sizeof *types);
+    if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	ncols++;
+    }
+
+    types = mymalloc(ncols * sizeof *types);
     if (types == NULL) {
 	return NULL;
     }
@@ -843,7 +933,11 @@ static GtkListStore *make_sheet_liststore (Spreadsheet *sheet)
 	types[i] = G_TYPE_STRING;
     }
 
-    store = gtk_list_store_newv(sheet->totcols, types);
+    if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	types[i] = GDK_TYPE_PIXBUF;
+    }
+
+    store = gtk_list_store_newv(ncols, types);
     free(types);
 
     return store;
@@ -1045,15 +1139,38 @@ static void popup_sheet_add_var (GtkWidget *w, Spreadsheet *sheet)
     name_var_dialog(sheet);
 }
 
+static void popup_sheet_add_scalar (GtkWidget *w, Spreadsheet *sheet)
+{
+    new_scalar_dialog(sheet);
+}
+
+static void sheet_delete_scalar (Spreadsheet *sheet, GtkTreePath *path)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkTreeIter iter;
+    GtkListStore *store;
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
+    gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
+    gtk_list_store_remove(store, &iter);
+    sheet->datarows--;
+    sheet_set_modified(sheet, TRUE);
+}
+
 static void build_sheet_popup (Spreadsheet *sheet)
 {
-    if (sheet->popup != NULL) return;
-
     sheet->popup = gtk_menu_new();
 
-    add_popup_item(_("Add Variable"), sheet->popup, 
-		   G_CALLBACK(popup_sheet_add_var),
-		   sheet);
+    if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	add_popup_item(_("Add..."), 
+		       sheet->popup, 
+		       G_CALLBACK(popup_sheet_add_scalar),
+		       sheet);
+    } else {
+	add_popup_item(_("Add Variable"), sheet->popup, 
+		       G_CALLBACK(popup_sheet_add_var),
+		       sheet);
+    }
 
     if (sheet->flags & SHEET_ADD_OBS_OK) {
 	add_popup_item(_("Add Observation"), sheet->popup,
@@ -1122,6 +1239,7 @@ static void update_matrix_from_sheet_full (Spreadsheet *sheet)
     }
 
     sheet_set_modified(sheet, FALSE);
+    set_ok_transforms(sheet);
 }
 
 /* put modified values from the spreadsheet into the array
@@ -1133,12 +1251,15 @@ static void update_scalars_from_sheet (Spreadsheet *sheet)
     GtkTreeModel *model;
     GtkTreeIter iter;
     gchar *vname, *val;
+    const char *sname;
     double x;
-    int i, err = 0;
+    int i, j, ns;
+    int err = 0;
 
     model = gtk_tree_view_get_model(view);
     gtk_tree_model_get_iter_first(model, &iter);
 
+    /* deal with added or modified scalars fist */
     for (i=0; i<sheet->datarows && !err; i++) {
 	gtk_tree_model_get(model, &iter, 0, &vname, 1, &val, -1);
 	x = (*val == '\0')? NADBL : atof(val);
@@ -1152,6 +1273,30 @@ static void update_scalars_from_sheet (Spreadsheet *sheet)
 	gtk_tree_model_iter_next(model, &iter);
     }
 
+    ns = n_saved_scalars();
+
+    if (ns > sheet->datarows) {
+	/* must have deleted some scalar(s) via GUI */
+	int match, n = 0, ndel = ns - sheet->datarows;
+
+	i = 0;
+	while ((sname = gretl_scalar_get_name(i)) && n < ndel) {
+	    gtk_tree_model_get_iter_first(model, &iter);
+	    match = 0;
+	    for (j=0; j<sheet->datarows && !match; j++) {
+		gtk_tree_model_get(model, &iter, 0, &vname, -1);
+		match = !strcmp(sname, vname);
+		g_free(vname);
+	    }
+	    if (!match) {
+		gretl_scalar_delete(sname, NULL);
+		n++;
+	    } else {
+		i++;
+	    }
+	}
+    }
+
     if (err) {
 	gui_errmsg(err);
     }
@@ -1448,6 +1593,9 @@ static int update_sheet_from_matrix (Spreadsheet *sheet)
 	}	    
 	for (j=0; j<sheet->datacols; j++) {
 	    x = gretl_matrix_get(sheet->matrix, i, j);
+	    if (x == -0.0) {
+		x = 0.0;
+	    }
 	    sprintf(tmpstr, "%.*g", MATRIX_DIGITS, x);
 	    gtk_list_store_set(store, &iter, j + 1, tmpstr, -1);
 	}
@@ -1519,21 +1667,27 @@ static int add_matrix_data_to_sheet (Spreadsheet *sheet)
 
 static int add_scalars_to_sheet (Spreadsheet *sheet)
 {
-    gchar vname[VNAMELEN];
-    gchar val[32];
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreeIter iter;
     GtkListStore *store;
+    gchar vname[VNAMELEN];
+    gchar val[32];
     double x;
     int i;
 
+    if (sheet->pbuf == NULL) {
+	sheet->pbuf = gtk_widget_render_icon(sheet->view, GTK_STOCK_DELETE,
+					     GTK_ICON_SIZE_MENU, NULL);
+    }
+
     store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     if (n_saved_scalars() == 0) {
 	/* starting from scratch */
 	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 0, sheet->mname, 1, "", -1);
+	gtk_list_store_set(store, &iter, 0, sheet->mname, 1, "", 
+			   2, sheet->pbuf, -1);
 	return 0;
     }
 
@@ -1548,7 +1702,8 @@ static int add_scalars_to_sheet (Spreadsheet *sheet)
 	    sprintf(val, "%.*g", DBL_DIG, x);
 	}
 	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 0, vname, 1, val, -1);
+	gtk_list_store_set(store, &iter, 0, vname, 1, val, 
+			   2, sheet->pbuf, -1);
     }
 
     return 0;
@@ -1676,6 +1831,16 @@ static gint get_data_col_width (void)
     return width;
 }
 
+static gint get_delete_col_width (void)
+{
+    static gint width;
+
+    if (width == 0) {
+	width = get_string_width("XXXXXX");
+    }
+    return width;
+}
+
 static void 
 set_up_sheet_column (GtkTreeViewColumn *column, gint width, gboolean expand)
 {
@@ -1733,9 +1898,13 @@ catch_sheet_edit_key (GtkWidget *view, GdkEventKey *key,
 	    return TRUE;
 	}
     } else if (key->keyval == GDK_Return) {
+#if CELLDEBUG
 	fprintf(stderr, "catch_edit_key: GDK_Return\n");
+#endif
     } else if (key->keyval == GDK_Down) {
+#if CELLDEBUG
 	fprintf(stderr, "catch_edit_key: GDK_Down\n");
+#endif
     }
 
     return FALSE;
@@ -1761,7 +1930,7 @@ static void cell_edit_start (GtkCellRenderer *r,
 #if 1 /* produces "random" broken behaviour */
 	sheet->entry = GTK_WIDGET(ed);
 #endif
-	g_signal_connect(G_OBJECT(ed), "key_press_event",
+	g_signal_connect(G_OBJECT(ed), "key-press-event",
 			 G_CALLBACK(catch_sheet_edit_key), sheet);
 	g_signal_connect(GTK_OBJECT(ed), "destroy",
 			 G_CALLBACK(nullify_sheet_entry), sheet);
@@ -1973,6 +2142,9 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	    if (colnum == 0) {
 		/* don't respond to a click in a non-data column */
 		ret = TRUE;
+	    } else if (sheet->cmd == SHEET_EDIT_SCALARS && colnum == 2) {
+		/* call to delete scalar */
+		sheet_delete_scalar(sheet, path);
 	    } else {
 		/* activate clicked cell for editing */
 		gtk_tree_view_set_cursor(GTK_TREE_VIEW(sheet->view), 
@@ -2070,6 +2242,9 @@ static int build_sheet_view (Spreadsheet *sheet)
 	}
 	sheet->colnames = NULL;
     } else if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	GtkCellRenderer *pixcell;
+	
+	/* numerical value */
 	column = gtk_tree_view_column_new_with_attributes(_("Value"),
 							  sheet->datacell,
 							  "text", 
@@ -2079,6 +2254,18 @@ static int build_sheet_view (Spreadsheet *sheet)
 	set_up_sheet_column(column, width, TRUE);
 	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(1));
 	g_object_set_data(G_OBJECT(column), "sheet", sheet);
+
+	/* deletion icon */
+	pixcell = gtk_cell_renderer_pixbuf_new();
+	column = gtk_tree_view_column_new_with_attributes(_("Delete"),
+							  pixcell,
+							  "pixbuf", 
+							  2, 
+							  NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
+	gtk_tree_view_column_set_resizable(column, FALSE);
+	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(2));
+	g_object_set_data(G_OBJECT(column), "sheet", sheet);
     } else {
 	colnum = 0;
 	for (i=1; i<=sheet->varlist[0]; i++) {
@@ -2195,8 +2382,10 @@ static Spreadsheet *spreadsheet_new (SheetCmd c)
     sheet->popup = NULL;
     sheet->ui = NULL;
     sheet->save = NULL;
+    sheet->apply = NULL;
     sheet->dumbcell = NULL;
     sheet->datacell = NULL;
+    sheet->pbuf = NULL;
     sheet->datacols = sheet->datarows = 0;
     sheet->totcols = 0;
     sheet->added_vars = 0;
@@ -2364,7 +2553,8 @@ static void size_scalars_window (Spreadsheet *sheet)
     int nc = 2;
     int w, h;
 
-    w = get_row_label_width() + nc * get_data_col_width() + 30;
+    w = get_row_label_width() + nc * get_data_col_width() + 
+	get_delete_col_width() + 30;
     if (w > 640) {
 	w = 640;
     }
@@ -2512,7 +2702,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 	g_signal_connect(G_OBJECT(sheet->win), "destroy",
 			 G_CALLBACK(gtk_main_quit), NULL);
     }
-    g_signal_connect(G_OBJECT(sheet->win), "delete_event",
+    g_signal_connect(G_OBJECT(sheet->win), "delete-event",
 		     G_CALLBACK(sheet_delete_event), sheet);
 
     main_vbox = gtk_vbox_new(FALSE, 5);
@@ -2572,6 +2762,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 			 G_CALLBACK(simple_exit_sheet), sheet);
 	gtk_widget_show(tmp);
     } else if (sheet->matrix != NULL) {
+	/* editing a new matrix */
 	tmp = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
 	gtk_container_add(GTK_CONTAINER(button_box), tmp);
 	g_signal_connect(G_OBJECT(tmp), "enter-notify-event",
@@ -2605,6 +2796,8 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 	g_signal_connect(G_OBJECT(tmp), "clicked",
 			 G_CALLBACK(get_data_from_sheet), sheet);
 	gtk_widget_show(tmp);
+	sheet->apply = tmp;
+	gtk_widget_set_sensitive(sheet->apply, FALSE);
 
 	tmp = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
 	gtk_container_add(GTK_CONTAINER(button_box), tmp);
@@ -2615,7 +2808,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 	gtk_widget_show(tmp);
     }
 
-    g_signal_connect(G_OBJECT(sheet->view), "button_press_event",
+    g_signal_connect(G_OBJECT(sheet->view), "button-press-event",
 		     G_CALLBACK(catch_spreadsheet_click),
 		     sheet);
 
@@ -2677,6 +2870,7 @@ static void start_scalars (GtkWidget *w, dialog_t *dlg)
     char varname[VNAMELEN];
  
     buf = edit_dialog_get_text(dlg);
+
     if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_DOUBLE)) {
 	return;
     }
@@ -2700,8 +2894,8 @@ static void maybe_add_scalar (void)
     if (resp == GRETL_YES) {
 	int cancel = 0;
 
-	edit_dialog(_("gretl: name variable"), 
-		    _("Enter name for new variable\n"
+	edit_dialog(_("gretl: name scalar"), 
+		    _("Enter name for new scalar\n"
 		      "(max. 15 characters)"),
 		    NULL, start_scalars, NULL, 
 		    0, VARCLICK_NONE, &cancel);
@@ -3008,17 +3202,17 @@ static int matrix_from_list (selector *sr)
 	errbox("No variables are selected");
 	return 1;
     } else {
-	list = gretl_list_from_string(buf);
+	list = gretl_list_from_string(buf, &err);
     }
 
-    if (list == NULL) {
-	nomem();
-	return 1;
+    if (err) {
+	gui_errmsg(err);
+	return err;
     }
 
-    s->m = gretl_matrix_data_subset_skip_missing(list, (const double **) Z, 
-						 datainfo->t1, datainfo->t2, 
-						 &err);
+    s->m = gretl_matrix_data_subset(list, (const double **) Z, 
+				    datainfo->t1, datainfo->t2, 
+				    M_MISSING_SKIP, &err);
 
     if (!err) {
 	err = add_or_replace_user_matrix(s->m, s->name);
@@ -3138,6 +3332,9 @@ static void edit_matrix (gretl_matrix *m, const char *name,
     }
 }
 
+/* note that both @m and @name may be NULL depending on how
+   we are called */
+
 static void real_gui_new_matrix (gretl_matrix *m, const char *name)
 {
     struct gui_matrix_spec spec;
@@ -3172,6 +3369,8 @@ static void real_gui_new_matrix (gretl_matrix *m, const char *name)
     edit_matrix(spec.m, spec.name, block);
 }
 
+/* callback for "Define matrix..." in main window */
+
 void gui_new_matrix (void)
 {
     real_gui_new_matrix(NULL, NULL);
