@@ -24,15 +24,20 @@ typedef enum {
     SHEET_EDIT_VARLIST,
     SHEET_EDIT_DATASET,
     SHEET_NEW_DATASET,
-    SHEET_EDIT_MATRIX
+    SHEET_EDIT_MATRIX,
+    SHEET_EDIT_SCALARS
 } SheetCmd;
 
 void show_spreadsheet (SheetCmd c);
 
+void edit_scalars (void);
+
 void gui_new_matrix (void);
 
 void edit_user_matrix_by_name (const char *name);
 
 int dataset_locked (void);
 
+gint get_string_width (const gchar *str);
+
 #endif /* SSHEET_H */
