@@ -566,19 +566,19 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(vwin->text), editable);
 
     if (gretl_script_role(vwin->role)) {
-	g_signal_connect(G_OBJECT(vwin->text), "key_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "key-press-event",
 			 G_CALLBACK(script_key_handler), vwin);
-	g_signal_connect(G_OBJECT(vwin->text), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button-press-event",
 			 G_CALLBACK(script_popup_handler), 
 			 vwin);
-	g_signal_connect(G_OBJECT(vwin->text), "button_release_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button-release-event",
 			 G_CALLBACK(interactive_script_help), vwin);
     } else if (foreign_script_role(vwin->role)) {
-	g_signal_connect(G_OBJECT(vwin->text), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button-press-event",
 			 G_CALLBACK(script_popup_handler), 
 			 vwin);
     } else if (vwin->role == VIEW_LOG) {
-	g_signal_connect(G_OBJECT(vwin->text), "button_release_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button-release-event",
 			 G_CALLBACK(interactive_script_help), vwin);
     }	
 }
@@ -1572,24 +1572,25 @@ static void get_cmdword (const char *s, char *word)
 #define starts_comment(p) (*p == '/' && *(p+1) == '*')
 #define ends_comment(p)   (*p == '*' && *(p+1) == '/')
 
-static void check_for_comment (const char *s, int *ignore)
+static void check_for_comment (const char *s, int *incomm)
 {
+    int commbak = *incomm;
     const char *p = s;
     int quoted = 0;
 
     while (*p) {
-	if (!quoted && !*ignore && *p == '#') {
+	if (!quoted && !*incomm && *p == '#') {
 	    break;
 	}
-	if (!*ignore && bare_quote(p, s)) {
+	if (!*incomm && bare_quote(p, s)) {
 	    quoted = !quoted;
 	}
 	if (!quoted) {
 	    if (starts_comment(p)) {
-		*ignore = 1;
+		*incomm = 1;
 		p += 2;
 	    } else if (ends_comment(p)) {
-		*ignore = 0;
+		*incomm = 0;
 		p += 2;
 		p += strspn(p, " ");
 	    }
@@ -1598,6 +1599,12 @@ static void check_for_comment (const char *s, int *ignore)
 	    p++;
 	}
     }
+
+    if (*incomm && commbak) {
+	/* on the second or subsequent line of a multiline
+	   comment */
+	*incomm = 2;
+    }
 }
 
 static void normalize_indent (GtkTextBuffer *tbuf, 
@@ -1609,7 +1616,7 @@ static void normalize_indent (GtkTextBuffer *tbuf,
     int next_indent = 0;
     char word[9], line[1024];
     const char *ins;
-    int ignore = 0;
+    int incomment = 0;
     int i, nsp;
 
     if (buf == NULL) {
@@ -1625,17 +1632,24 @@ static void normalize_indent (GtkTextBuffer *tbuf,
 	    gtk_text_buffer_insert(tbuf, start, line, -1);
 	    continue;
 	}
-	check_for_comment(line, &ignore);
-	if (ignore) {
+	check_for_comment(line, &incomment);
+#if 0
+	if (incomment) {
 	    /* in multiline comment */
 	    gtk_text_buffer_insert(tbuf, start, line, -1);
 	    continue;
 	}
+#endif
 	ins = line + strspn(line, " \t");
-	*word = '\0';
-	get_cmdword(ins, word);
-	adjust_indent(word, &this_indent, &next_indent);
+	if (!incomment) {
+	    *word = '\0';
+	    get_cmdword(ins, word);
+	    adjust_indent(word, &this_indent, &next_indent);
+	}
 	nsp = this_indent * tabwidth;
+	if (incomment == 2) {
+	    nsp += 3;
+	}
 	for (i=0; i<nsp; i++) {
 	    gtk_text_buffer_insert(tbuf, start, " ", -1);
 	}
