@@ -28,6 +28,10 @@
 #include "texprint.h"
 #include "system.h"
 
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7
+# include "dlgutils.h"
+#endif
+
 struct search_replace {
     GtkWidget *w;
     GtkWidget *f_entry;
@@ -65,6 +69,11 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_window_set_title(GTK_WINDOW(s->w), _("gretl: replace"));
     gtk_container_set_border_width(GTK_CONTAINER(s->w), 5);
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7)
+    g_signal_connect(G_OBJECT(s->w), "key-press-event", 
+		     G_CALLBACK(esc_kills_window), NULL);
+#endif
+
     /* Find part */
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_("Find:"));
@@ -412,18 +421,18 @@ void model_tex_copy (GtkAction *action, gpointer data)
 
 static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
 {
-    GtkTextBuffer *textbuf = 
-	gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
     gchar *cpybuf = NULL;
 
-    if (!select) {
-	cpybuf = textview_get_text(vwin->text); 
-    } else if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL)) {
+    if (select) {
+	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	GtkTextIter selstart, selend;
 
-	gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend);
-	cpybuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
-    } 
+	if (gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend)) {
+	    cpybuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
+	}
+    } else {	
+	cpybuf = textview_get_text(vwin->text); 
+    }
 
     return cpybuf;
 }
@@ -462,8 +471,9 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	special_text_handler(vwin, fmt, action);
     } else if (MULTI_FORMAT_ENABLED(vwin->role) && SPECIAL_FORMAT(fmt)) {
 	special_text_handler(vwin, fmt, action);
-    } else if (fmt == GRETL_FORMAT_CSV || fmt == GRETL_FORMAT_TAB) {
-	csv_copy_listed_vars(vwin, fmt, action);
+    } else if (fmt == GRETL_FORMAT_CSV || fmt == GRETL_FORMAT_TAB ||
+	       fmt == GRETL_FORMAT_RTF) {
+	copy_vars_formatted(vwin, fmt, action);
     } else if (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) {
 	cpybuf = text_window_get_copy_buf(vwin, 0);
     } else if (fmt == GRETL_FORMAT_SELECTION) {
@@ -482,6 +492,7 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	}
 
 	textprn = gretl_print_new_with_buffer(cpybuf);
+
 	if (action == W_COPY) {
 	    prn_to_clipboard(textprn, fmt);
 	} else {
@@ -522,7 +533,7 @@ void window_print (GtkAction *action, windata_t *vwin)
 	selbuf = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
     }
 
-    winprint(buf, selbuf);
+    print_window_content(buf, selbuf);
 }
 
 #endif
