@@ -29,6 +29,7 @@
 #include "series_view.h"
 #include "cmdstack.h"
 #include "dlgutils.h"
+#include "fileselect.h"
 #include "toolbar.h"
 
 #include "usermat.h"
@@ -75,6 +76,7 @@ enum {
     MAIL_ITEM,
     HELP_ITEM,
     CMD_HELP_ITEM,
+    GP_HELP_ITEM,
     SORT_ITEM,
     SORT_BY_ITEM,
     FORMAT_ITEM,
@@ -82,7 +84,8 @@ enum {
     EDIT_SCRIPT_ITEM,
     STICKIFY_ITEM,
     ALPHA_ITEM,
-    REFRESH_ITEM
+    REFRESH_ITEM,
+    OPEN_ITEM
 } viewbar_flags;
 
 static GtkIconFactory *gretl_stock_ifac;
@@ -231,8 +234,6 @@ static void text_copy_callback (GtkWidget *w, windata_t *vwin)
 {
     if (vwin_selection_present(vwin)) {
 	window_copy(vwin, GRETL_FORMAT_SELECTION);
-    } else if (vwin->role == VIEW_SCALAR) {
-	scalar_to_clipboard(vwin);
     } else if (!script_role(vwin->role) && !editor_role(vwin->role)) {
 	copy_format_dialog(vwin, W_COPY);
     } else {
@@ -254,6 +255,25 @@ static void mail_script_callback (GtkWidget *w, windata_t *vwin)
     send_file(vwin->fname);
 }
 
+static void file_open_callback (GtkWidget *w, windata_t *vwin)
+{
+    if (query_save_text(NULL, NULL, vwin)) {
+	return;
+    }
+
+    file_selector(_("Open script file"), OPEN_SCRIPT, FSEL_DATA_VWIN, vwin);
+
+    if (vwin->flags & VWIN_CONTENT_CHANGED) {
+	mark_vwin_content_saved(vwin);
+	vwin_set_filename(vwin, tryfile);
+    }
+}
+
+static void toolbar_new_callback (GtkWidget *w, windata_t *vwin)
+{
+    do_new_script(vwin->role);
+}
+
 static void save_plot_commands_callback (GtkWidget *w, windata_t *vwin)
 {
     auto_save_plot(vwin);
@@ -283,27 +303,23 @@ static void script_index (GtkWidget *w, windata_t *vwin)
 
 static void cmd_log_refresh (GtkWidget *w, windata_t *vwin)
 {
-    gchar *logfile = NULL;
-    gchar *newtext = NULL;
-    int err;
+    gchar *newtext;
+    int err = 0;
 
-    logfile = g_strdup_printf("%ssession.inp", paths.dotdir);
-    err = dump_command_stack(logfile, 0);
+    newtext = get_logfile_content(&err);
 
-    if (!err) {
-	err = gretl_file_get_contents(logfile, &newtext);
-    }
-
-    if (!err) {
+    if (err) {
+	gui_errmsg(err);
+    } else {
 	GtkTextBuffer *buf;
 
 	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	gtk_text_buffer_set_text(buf, "", -1);
-	textview_set_text(vwin->text, newtext);
-	g_free(newtext);
+	if (newtext != NULL) {
+	    textview_set_text(vwin->text, newtext);
+	    g_free(newtext);
+	}
     }
-
-    g_free(logfile);
 }
 
 static void matrix_savename (GtkWidget *w, dialog_t *dlg)
@@ -370,35 +386,138 @@ static void add_data_callback (GtkWidget *w, windata_t *vwin)
     }	
 }
 
-#define xround(x) (((x-floor(x))>.5)? ceil(x) : floor(x))
+static void real_coeffint_set_alpha (GtkWidget *w, GtkWidget *dialog)
+{
+    windata_t *vwin = g_object_get_data(G_OBJECT(dialog), "vwin");
+    double *x = g_object_get_data(G_OBJECT(dialog), "xptr");
+    CoeffIntervals *cf = vwin->data;
+    GtkTextBuffer *buf;
+    const char *newtext;
+    PRN *prn;
+
+    if (bufopen(&prn)) {
+	return;
+    }
+
+    reset_coeff_intervals(cf, 1.0 - *x);
+    text_print_model_confints(cf, prn);
+    newtext = gretl_print_get_buffer(prn);
+    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+    gtk_text_buffer_set_text(buf, "", -1);
+    textview_set_text(vwin->text, newtext);
+    gretl_print_destroy(prn); 
+
+    gtk_widget_destroy(dialog);
+}
+
+static void alpha_button_callback (GtkToggleButton *b, double *x)
+{
+    if (gtk_toggle_button_get_active(b)) {
+	int i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(b), "i"));
+
+	if (i == 0) {
+	    *x = 0.90;
+	} else if (i == 1) {
+	    *x = 0.95;
+	} else if (i == 2) {
+	    *x = 0.99;
+	}
+    } 
+}
+
+static void toggle_alpha_spin (GtkToggleButton *b, GtkWidget *w)
+{
+    gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(b));
+}
 
 static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
 {
     CoeffIntervals *cf = vwin->data;
-    GtkTextBuffer *buf;
-    const char *newtext;
-    double alpha, x = 100 * (1 - cf->alpha);
-    int cval = (int) xround(x);
-    PRN *prn;
-    int resp;
+    GtkWidget *dialog, *tmp, *hbox;
+    GtkWidget *vbox, *b, *hb2;
+    GSList *group = NULL;
+    GtkObject *adj;
+    gchar txt[16];
+    double x = 1.0 - cf->alpha;
+    gboolean defset = FALSE;
+    int i;
 
-    resp = spin_dialog("gretl: alpha", NULL,
-		       &cval, _("Confidence level, percent"),
-		       60, 99, 0);
-
-    if (resp < 0 || bufopen(&prn)) {
+    if (maybe_raise_dialog()) {
 	return;
     }
 
-    alpha = (100.0 - cval) / 100.0;
-    reset_coeff_intervals(cf, alpha);
-    text_print_model_confints(cf, prn);
-    newtext = gretl_print_get_buffer(prn);
-    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+    dialog = gretl_dialog_new(_("gretl: coefficient confidence intervals"), 
+			      vwin->main, GRETL_DLG_BLOCK);
 
-    gtk_text_buffer_set_text(buf, "", -1);
-    textview_set_text(vwin->text, newtext);
-    gretl_print_destroy(prn);
+    hbox = gtk_hbox_new(FALSE, 5);
+    hb2 = gtk_hbox_new(FALSE, 5);
+    tmp = gtk_label_new(_("Confidence level"));
+    gtk_box_pack_start(GTK_BOX(hb2), tmp, FALSE, FALSE, 0);
+    tmp = gtk_label_new("1 - α :");
+    gtk_box_pack_start(GTK_BOX(hb2), tmp, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), hb2, TRUE, TRUE, 10);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+		       hbox, FALSE, FALSE, 5);
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    vbox = gtk_vbox_new(FALSE, 5);
+
+    /* radio button for 90%, 95%, 99% confidence */
+
+    for (i=0; i<3; i++) {
+	double a = (i == 0)? 0.90 : (i == 1)? 0.95 : 0.99;
+
+	sprintf(txt, "%.2f", a);
+	b = gtk_radio_button_new_with_label(group, txt);
+	gtk_box_pack_start(GTK_BOX(vbox), b, FALSE, FALSE, 0);
+	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
+	if (a == x) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
+	    defset = TRUE;
+	} else {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), FALSE);
+	}
+	g_object_set_data(G_OBJECT(b), "i", GINT_TO_POINTER(i));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(alpha_button_callback), 
+			 &x);
+    }
+
+    /* radio button for "other" confidence level, plus spinner */
+
+    hb2 = gtk_hbox_new(FALSE, 0);
+    b = gtk_radio_button_new_with_label(group, _("Other"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), !defset);
+    gtk_box_pack_start(GTK_BOX(hb2), b, FALSE, FALSE, 0);
+    adj = gtk_adjustment_new(x, 0.60, 0.99, 0.01, 0, 0);
+    tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.01, 2);
+    g_signal_connect(tmp, "value-changed", 
+		     G_CALLBACK(set_double_from_spinner), &x);
+    gtk_widget_set_sensitive(tmp, !defset);
+    g_signal_connect(G_OBJECT(b), "toggled", 
+		     G_CALLBACK(toggle_alpha_spin),
+		     tmp);
+    gtk_entry_set_activates_default(GTK_ENTRY(tmp), TRUE);
+    gtk_box_pack_start(GTK_BOX(hb2), tmp, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hb2, FALSE, FALSE, 0);
+
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 10);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+		       hbox, FALSE, FALSE, 0);
+
+    /* Cancel button */
+    cancel_options_button(GTK_DIALOG(dialog)->action_area, dialog, NULL);
+
+    g_object_set_data(G_OBJECT(dialog), "vwin", vwin);
+    g_object_set_data(G_OBJECT(dialog), "xptr", &x);
+
+    /* "OK" button */
+    tmp = ok_button(GTK_DIALOG(dialog)->action_area);
+    g_signal_connect(G_OBJECT(tmp), "clicked", 
+		     G_CALLBACK(real_coeffint_set_alpha), dialog);
+    gtk_widget_grab_default(tmp);
+
+    gtk_widget_show_all(dialog);
 }
 
 static void set_output_sticky (GtkWidget *w, windata_t *vwin)
@@ -465,6 +584,8 @@ static void set_plot_icon (GretlToolItem *item)
 }
 
 static GretlToolItem viewbar_items[] = {
+    { N_("New window"), GTK_STOCK_NEW, G_CALLBACK(toolbar_new_callback), OPEN_ITEM },
+    { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
     { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(view_window_save), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
 #ifdef NATIVE_PRINTING
@@ -476,8 +597,8 @@ static GretlToolItem viewbar_items[] = {
     { N_("Find..."), GTK_STOCK_FIND, G_CALLBACK(text_find), 0 },
     { N_("Replace..."), GTK_STOCK_FIND_AND_REPLACE, G_CALLBACK(text_replace), EDIT_ITEM },
     { N_("Undo"), GTK_STOCK_UNDO, G_CALLBACK(text_undo), EDIT_ITEM },
-    { N_("Sort"), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(series_view_sort), SORT_ITEM },    
-    { N_("Sort by..."), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(series_view_sort_by), SORT_BY_ITEM },
+    { N_("Sort"), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(series_view_toggle_sort), SORT_ITEM },    
+    { N_("Sort by..."), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(multi_series_view_sort_by), SORT_BY_ITEM },
     { N_("Configure tabs..."), GTK_STOCK_PREFERENCES, G_CALLBACK(script_tabs_dialog), EDIT_SCRIPT_ITEM },
     { N_("Send To..."), GRETL_STOCK_MAIL, G_CALLBACK(mail_script_callback), MAIL_ITEM },
     { N_("Scripts index"), GTK_STOCK_INDEX, G_CALLBACK(script_index), INDEX_ITEM },
@@ -491,6 +612,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_sticky), STICKIFY_ITEM },
     { N_("Help on command"), GTK_STOCK_HELP, G_CALLBACK(activate_script_help), CMD_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(window_help), HELP_ITEM },
+    { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
     { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), 0 }
 };
 
@@ -501,20 +623,21 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
                     r == EDIT_R || \
 	            r == VIEW_SCRIPT)
 
+#define open_ok(r) (r == EDIT_SCRIPT || \
+                    r == EDIT_GP || \
+                    r == EDIT_R)
+
 #define edit_ok(r) (r == EDIT_SCRIPT || \
                     r == EDIT_HEADER || \
                     r == EDIT_NOTES || \
                     r == EDIT_FUNC_CODE || \
 	            r == EDIT_GP || \
-                    r == EDIT_BOX || \
 		    r == EDIT_R || \
                     r == SCRIPT_OUT)
 
 #define save_as_ok(r) (r != EDIT_HEADER && \
 	               r != EDIT_NOTES && \
-	               r != EDIT_FUNC_CODE && \
-		       r != EDIT_BOX && \
-		       r != VIEW_SCALAR)
+	               r != EDIT_FUNC_CODE)
 
 #define help_ok(r) (r == LEVERAGE || \
 		    r == COINT2 || \
@@ -527,7 +650,7 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 			r == VIEW_LOG)
 
 #define sort_ok(r)    (r == VIEW_SERIES)
-#define format_ok(r)  (r == VIEW_SERIES || r == VIEW_SCALAR)
+#define format_ok(r)  (r == VIEW_SERIES)
 #define plot_ok(r)    (r == VIEW_SERIES)
 
 #define add_data_ok(r) (r == PCA || r == LEVERAGE || \
@@ -539,7 +662,8 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 */
 
 static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin, 
-				    int latex_ok, int sortby_ok)
+				    int latex_ok, int sortby_ok,
+				    int format_ok)
 {
     GCallback func = item->func;
     int f = item->flag;
@@ -547,6 +671,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 
     if (!edit_ok(r) && f == EDIT_ITEM) {
 	return NULL;
+    } else if (!open_ok(r) && f == OPEN_ITEM) {
+	return NULL;
     } else if (!exec_ok(r) && f == EXEC_ITEM) {
 	return NULL;
     } else if (!cmd_help_ok(r) && f == CMD_HELP_ITEM) {
@@ -555,8 +681,6 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (!help_ok(r) && f == HELP_ITEM) {
 	return NULL;
-    } else if (r == VIEW_SCALAR && f == 0) {
-	return NULL;
     } else if ((!latex_ok || !MULTI_FORMAT_ENABLED(r)) && f == TEX_ITEM) {
 	return NULL;
     } else if (!add_data_ok(r) && f == ADD_DATA_ITEM) {
@@ -569,7 +693,7 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (!plot_ok(r) && f == PLOT_ITEM) {
 	return NULL;
-    } else if (!format_ok(r) && f == FORMAT_ITEM) {
+    } else if (!format_ok && f == FORMAT_ITEM) {
 	return NULL;
     } else if (r != EDIT_SCRIPT && r != EDIT_FUNC_CODE && f == EDIT_SCRIPT_ITEM) {
 	return NULL;
@@ -581,6 +705,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != VIEW_LOG && f == REFRESH_ITEM) {
 	return NULL;
+    } else if (r != EDIT_GP && f == GP_HELP_ITEM) {
+	return NULL;
     } else if (f == SAVE_ITEM) { 
 	if (!edit_ok(r) || r == SCRIPT_OUT) {
 	    /* script output doesn't already have a filename */
@@ -662,39 +788,21 @@ GtkToolItem *gretl_toolbar_insert (GtkWidget *tbar,
     return button;
 }
 
-/* "text_out": if this argument is non-zero, it means that vwin
-   carries text output which could appropriately be saved, either
-   to file or in the form of a chunk of text to be added as a
-   session icon.
-*/
-
-void vwin_add_viewbar (windata_t *vwin, int text_out)
+static void viewbar_add_items (windata_t *vwin)
 {
-    GtkWidget *hbox;
+    int sortby_ok = has_sortable_data(vwin);
+    int format_ok = can_format_data(vwin);
+    int latex_ok = latex_is_ok();
     GtkToolItem *button;
     GretlToolItem *item;
     GCallback func;
-    int sortby_ok = has_sortable_data(vwin);
-    int latex_ok = latex_is_ok();
     int i;
-
-    if (gretl_stock_ifac == NULL) {
-	gretl_stock_icons_init();
-    }
-
-    if (text_out || vwin->role == SCRIPT_OUT) {
-	g_object_set_data(G_OBJECT(vwin->main), "text_out", GINT_TO_POINTER(1));
-    }
-
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
-
-    vwin->mbar = gretl_toolbar_new();
-
+ 
     for (i=0; i<n_viewbar_items; i++) {
 	item = &viewbar_items[i];
 
-	func = item_get_callback(item, vwin, latex_ok, sortby_ok);
+	func = item_get_callback(item, vwin, latex_ok, sortby_ok,
+				 format_ok);
 	if (func == NULL) {
 	    continue;
 	}
@@ -716,37 +824,48 @@ void vwin_add_viewbar (windata_t *vwin, int text_out)
 	    }
 	}
     }
+}
+
+/* "text_out": if this argument is non-zero, it means that vwin
+   carries text output which could appropriately be saved, either
+   to file or in the form of a chunk of text to be added as a
+   session icon.
+*/
+
+void vwin_add_viewbar (windata_t *vwin, int text_out)
+{
+    GtkWidget *hbox;
+
+    if (gretl_stock_ifac == NULL) {
+	gretl_stock_icons_init();
+    }
+
+    if (text_out || vwin->role == SCRIPT_OUT) {
+	g_object_set_data(G_OBJECT(vwin->main), "text_out", GINT_TO_POINTER(1));
+    }
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+
+    vwin->mbar = gretl_toolbar_new();
+
+    viewbar_add_items(vwin);
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
     gtk_widget_show_all(hbox);
 }
 
+static void remove_child (GtkWidget *child, GtkWidget *cont)
+{
+    gtk_container_remove(GTK_CONTAINER(cont), child);
+}
+
 void viewbar_add_edit_items (windata_t *vwin)
 {
-    GretlToolItem *item;
-    GtkToolItem *button;
-    int i, pos = 0;
-
-    for (i=0; i<n_viewbar_items; i++) {
-	item = &viewbar_items[i];
-	if (item->flag == SAVE_ITEM ||
-	    item->flag == EDIT_ITEM ||
-	    item->flag == EDIT_SCRIPT_ITEM) {
-	    button = gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, pos);
-	    if (item->flag == SAVE_ITEM) {
-		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
-		g_object_set_data(G_OBJECT(vwin->mbar), "save_button",
-				  button);
-	    } else if (item->flag == SAVE_AS_ITEM) {
-		g_object_set_data(G_OBJECT(vwin->mbar), "save_as_button",
-				  button);
-	    }
-	    gtk_widget_show(GTK_WIDGET(button));
-	}
-	if (item->flag != EDIT_GP) {
-	    pos++;
-	}
-    }
+    gtk_container_foreach(GTK_CONTAINER(vwin->mbar),
+			  (GtkCallback) remove_child, vwin->mbar);
+    viewbar_add_items(vwin);
+    gtk_widget_show_all(vwin->mbar);
 }
 
 GtkWidget *build_text_popup (windata_t *vwin)
@@ -768,7 +887,7 @@ GtkWidget *build_text_popup (windata_t *vwin)
 		func = NULL;
 	    }
 	} else {
-	    func = item_get_callback(item, vwin, 0, 0);
+	    func = item_get_callback(item, vwin, 0, 0, 0);
 	}
 	if (func != G_CALLBACK(NULL)) {
 	    if (func == G_CALLBACK(text_paste)) {
@@ -826,7 +945,7 @@ static void tbar_xy_graph (void)
 	} else {
 	    selection_dialog(_("gretl: define graph"), 
 			     do_graph_from_selector,
-			     GR_XY, 0);
+			     GR_XY);
 	}
     } else {
 	warnbox(_("Please open a data file first"));
@@ -836,7 +955,7 @@ static void tbar_xy_graph (void)
 static void tbar_model (void)
 {
     if (data_status) {
-	selection_dialog(_("gretl: specify model"), do_model, OLS, 0);
+	selection_dialog(_("gretl: specify model"), do_model, OLS);
     } else {
 	warnbox(_("Please open a data file first"));
     }
@@ -845,7 +964,7 @@ static void tbar_model (void)
 static void tbar_iconview (void)
 {
     if (data_status) {
-	view_session();
+	view_session(NULL);
     } else {
 	warnbox(_("Please open a data file first"));
     }
