@@ -93,6 +93,9 @@ static int GLS_demean_detrend (double *y, int T, int test)
     return err;
 }
 
+/* generate the various differences and lags required for
+   the ADF test */
+
 static int *
 adf_prepare_vars (int order, int varno, int nseas, int *d0,
 		  double ***pZ, DATAINFO *pdinfo,
@@ -216,23 +219,65 @@ static double *df_gls_ct_cval (int T)
     return df_gls_ct_cvals[i];
 }
 
+/* display an F-test for the joint significance of the lagged
+   \delta y terms in ADF test */
+
+static void show_lags_test (MODEL *pmod, int order, PRN *prn)
+{
+    int *llist = gretl_list_new(order);
+    double F;
+    int i;
+
+    if (llist != NULL) {
+	for (i=0; i<order; i++) {
+	    /* lagged differences */
+	    llist[i+1] = pmod->list[pmod->ifc + 3 + i];
+	}
+
+	F = wald_omit_F(llist, pmod);
+
+	if (!na(F)) {
+	    pprintf(prn, "   %s: F(%d, %d) = %.3f [%.4f]\n",
+		    _("lagged differences"), order, pmod->dfd, F, 
+		    snedecor_cdf_comp(order, pmod->dfd, F));
+	}
+
+	free(llist);
+    }
+}
+
+static void DF_header (const char *s, int p, PRN *prn)
+{
+    if (p <= 0) {
+	pprintf(prn, _("\nDickey-Fuller test for %s\n"), s);	
+    } else {
+	pprintf(prn, _("\nAugmented Dickey-Fuller test for %s\n"), s);
+	if (p == 1) {
+	    pprintf(prn, _("including one lag of (1-L)%s"), s);
+	} else {
+	    pprintf(prn, _("including %d lags of (1-L)%s"), p, s);
+	}
+	pputc(prn, '\n');
+    }
+}
+
 static void 
 print_adf_results (int order, int auto_order, double DFt, double pv, 
-		   const MODEL *dfmod, int dfnum, const char *vname, 
+		   MODEL *dfmod, int dfnum, const char *vname, 
 		   int *blurb_done, unsigned char flags, int i, 
 		   int niv, int nseas, gretlopt opt, PRN *prn)
 {
     const char *models[] = {
-	"(1 - L)y = (a-1)*y(-1) + e",
-	"(1 - L)y = b0 + (a-1)*y(-1) + e",
-	"(1 - L)y = b0 + b1*t + (a-1)*y(-1) + e",
-	"(1 - L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + e"
+	"(1-L)y = (a-1)*y(-1) + e",
+	"(1-L)y = b0 + (a-1)*y(-1) + e",
+	"(1-L)y = b0 + b1*t + (a-1)*y(-1) + e",
+	"(1-L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + e"
     };
     const char *aug_models[] = {
-	"(1 - L)y = (a-1)*y(-1) + ... + e",
-	"(1 - L)y = b0 + (a-1)*y(-1) + ... + e",
-	"(1 - L)y = b0 + b1*t + (a-1)*y(-1) + ... + e",
-	"(1 - L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + ... + e"
+	"(1-L)y = (a-1)*y(-1) + ... + e",
+	"(1-L)y = b0 + (a-1)*y(-1) + ... + e",
+	"(1-L)y = b0 + b1*t + (a-1)*y(-1) + ... + e",
+	"(1-L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + ... + e"
     };
     const char *teststrs[] = {
 	N_("test without constant"),
@@ -262,24 +307,7 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
     } 
 
     if (*blurb_done == 0) {
-	if (flags & ADF_EG_RESIDS) {
-	    pputc(prn, '\n');
-	    pprintf(prn, _("lag order %d\n"), order);
-	} else if (flags & ADF_EG_TEST) {
-	    if (order > 0) {
-		pprintf(prn, _("\nAugmented Dickey-Fuller test, order %d, for %s\n"),
-			order, vname);
-	    } else {
-		pprintf(prn, _("\nDickey-Fuller test for %s\n"), vname);
-	    }
-	} else {
-	    if (order > 0 && !auto_order) {
-		pprintf(prn, _("\nAugmented Dickey-Fuller tests, order %d, for %s\n"),
-			order, vname);
-	    } else {
-		pprintf(prn, _("\nDickey-Fuller tests for %s\n"), vname);
-	    }
-	} 
+	DF_header(vname, order, prn);
 	pprintf(prn, _("sample size %d\n"), dfmod->nobs);
 	pputs(prn, _("unit-root null hypothesis: a = 1"));
 	pputs(prn, "\n\n");
@@ -297,18 +325,18 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
 	pputc(prn, '\n');
     }
 
-    if (!(flags & ADF_EG_TEST)) {
-	pprintf(prn, "   %s: %s\n", _("model"), 
-		(order > 0)? aug_models[i] : models[i]);
-	if (auto_order) {
-	    pprintf(prn, "   %s %d\n", _("lag order:"), order);
-	}
-	if (!na(dfmod->rho)) {
-	    pprintf(prn, "   %s: %.3f\n", _("1st-order autocorrelation coeff. for e"), 
-		    dfmod->rho);
-	}
+    pprintf(prn, "   %s: %s\n", _("model"), 
+	    (order > 0)? aug_models[i] : models[i]);
+
+    if (!na(dfmod->rho)) {
+	pprintf(prn, "   %s: %.3f\n", _("1st-order autocorrelation coeff. for e"), 
+		dfmod->rho);
     }
 
+    if (order > 1) {
+	show_lags_test(dfmod, order, prn);
+    }	
+
     if (opt & OPT_G) {
 	strcpy(taustr, "tau");
     } else {
@@ -345,8 +373,9 @@ static int auto_adjust_order (int *list, int order_max,
 	kmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
 
 	if (kmod.errcode) {
+	    fprintf(stderr, "auto_adjust_order: k = %d, err = %d\n", k,
+		    kmod.errcode);
 	    clear_model(&kmod);
-	    fprintf(stderr, "adf: model failed in auto_adjust_order()\n");
 	    k = -1;
 	    break;
 	}
@@ -389,8 +418,8 @@ static void copy_list_values (int *targ, const int *src)
     }
 }
 
-static double df_pvalue_from_plugin (double tau, int n, int niv, int itv,
-				     gretlopt opt)
+double df_pvalue_from_plugin (double tau, int n, int niv, int itv,
+			      gretlopt opt)
 {
     char datapath[FILENAME_MAX];
     void *handle;
@@ -498,6 +527,23 @@ static int real_adf_test (int varno, int order, int niv,
     fprintf(stderr, "real_adf_test: got order = %d\n", order);
 #endif
 
+    if (opt & OPT_E) {
+	/* testing down */
+	auto_order = 1;
+    }
+
+    if (order < 0) {
+	/* testing down: backward compatibility */
+	auto_order = 1;
+	order = -order;
+    }
+
+    order_max = order;
+
+#if ADF_DEBUG
+    fprintf(stderr, "real_adf_test: order = %d, auto_order = %d\n", order, auto_order);
+#endif
+
     if (flags & ADF_EG_RESIDS) {
 	/* final step of Engle-Granger test: the (A)DF test regression
 	   will contain no deterministic terms, but the selection of the
@@ -510,28 +556,18 @@ static int real_adf_test (int varno, int order, int niv,
 
     if (opt & OPT_F) {
 	/* difference the variable before testing */
+	int t1 = pdinfo->t1;
+
+	pdinfo->t1 = 0;
 	varno = diffgenr(varno, DIFF, pZ, pdinfo);
+	pdinfo->t1 = t1;
 	if (varno < 0) {
 	    return E_DATA;
 	}
     }
 
-    if (opt & OPT_E) {
-	auto_order = 1;
-    }
-
-    if (order < 0) {
-	auto_order = 1;
-	order = -order;
-    }
-
-    order_max = order;
-
-#if ADF_DEBUG
-    fprintf(stderr, "real_adf_test: order = %d, auto_order = %d\n", order, auto_order);
-#endif
-
     if ((opt & OPT_D) && pdinfo->pd > 1) {
+	/* add seasonal dummies */
 	nseas = pdinfo->pd - 1;
     }
 
@@ -672,11 +708,11 @@ static int real_adf_test (int varno, int order, int niv,
 	    /* verbose */
 	    dfmod.aux = (order > 0)? AUX_ADF : AUX_DF;
 	    if (!na(pv)) {
-		gretl_model_set_int(&dfmod, "dfnum", dfnum + 2);
+		gretl_model_set_int(&dfmod, "dfnum", dfnum);
 		gretl_model_set_double(&dfmod, "dfpval", pv);
 	    }
 	    printmodel(&dfmod, pdinfo, OPT_NONE, prn);
-	} else if (!(opt & OPT_Q)) {
+	} else if (!(opt & OPT_Q) && !(flags & ADF_EG_RESIDS)) {
 	    pputc(prn, '\n');
 	}
 
@@ -991,6 +1027,53 @@ coint_check_opts (gretlopt opt, int *detcode, gretlopt *adf_opt)
 	*adf_opt = OPT_R;
     }
 
+    if (opt & OPT_E) {
+	*adf_opt |= OPT_E;
+    }
+
+    return 0;
+}
+
+/* Engle-Granger: try to ensure a uniform sample for the individual
+   (A)DF tests and the test on the cointegrating regression
+*/
+
+static int coint_set_sample (const int *list, int nv, int order,
+			     double **Z, DATAINFO *pdinfo)
+{
+    int anymiss;
+    int i, t;
+
+    for (t=pdinfo->t1; t<pdinfo->t2; t++) {
+	anymiss = 0;
+	for (i=1; i<=nv; i++) {
+	    if (na(Z[list[i]][t])) {
+		anymiss = 1;
+		break;
+	    }
+	}
+	if (!anymiss) {
+	    break;
+	}
+    }
+
+    pdinfo->t1 = t + order + 1;
+    
+    for (t=pdinfo->t2; t>pdinfo->t1; t--) {
+	anymiss = 0;
+	for (i=1; i<=nv; i++) {
+	    if (na(Z[list[i]][t])) {
+		anymiss = 1;
+		break;
+	    }
+	}
+	if (!anymiss) {
+	    break;
+	}
+    }
+
+    pdinfo->t2 = t;
+
     return 0;
 }
 
@@ -1000,8 +1083,10 @@ coint_check_opts (gretlopt opt, int *detcode, gretlopt *adf_opt)
  * @list: specifies the variables to use.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
- * @opt: if OPT_N, do not an include a constant in the
- *       cointegrating regression.
+ * @opt: if %OPT_N, do not an include a constant in the
+ *       cointegrating regression, if %OPT_S, skip
+ *       DF tests for individual variables; if %OPT_E,
+ *       test down from maximum lag order.
  * @prn: gretl printing struct.
  *
  * Carries out the Engle-Granger test for cointegration.  
@@ -1012,6 +1097,9 @@ coint_check_opts (gretlopt opt, int *detcode, gretlopt *adf_opt)
 int coint (int order, const int *list, double ***pZ, 
 	   DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+    int test_t1, test_t2;
     gretlopt adf_opt = OPT_C;
     MODEL cmod;
     int detcode = UR_CONST;
@@ -1030,11 +1118,22 @@ int coint (int order, const int *list, double ***pZ,
 	return err;
     }
 
+    /* backward compatibility: let a negative lag order
+       indicate that we should test down */
+    if (order < 0) {
+	order = -order;
+	adf_opt |= OPT_E;
+    }
+
     gretl_model_init(&cmod);
 
     if (!(opt & OPT_S)) {
 	/* test all candidate vars for unit root */
+	coint_set_sample(clist, nv, order, *pZ, pdinfo);
 	for (i=1; i<=nv; i++) {
+	    if (step == 1) {
+		pputc(prn, '\n');
+	    }
 	    pprintf(prn, _("Step %d: testing for a unit root in %s\n"),
 		    step++, pdinfo->varname[clist[i]]);
 	    real_adf_test(clist[i], order, 1, pZ, pdinfo, adf_opt, 
@@ -1042,8 +1141,17 @@ int coint (int order, const int *list, double ***pZ,
 	}
     }
 
+    if (step == 1) {
+	pputc(prn, '\n');
+    }
     pprintf(prn, _("Step %d: cointegrating regression\n"), step++);
 
+    test_t1 = pdinfo->t1;
+    test_t2 = pdinfo->t2;
+
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
     cmod = lsq(clist, pZ, pdinfo, OLS, OPT_NONE);
     err = cmod.errcode;
     if (err) {
@@ -1063,8 +1171,11 @@ int coint (int order, const int *list, double ***pZ,
     strcpy(pdinfo->varname[k], "uhat");
     cmod.uhat = NULL;
 
-    pputc(prn, '\n');
-    pprintf(prn, _("Step %d: Dickey-Fuller test on residuals\n"), step);
+    pprintf(prn, _("Step %d: testing for a unit root in %s\n"),
+	    step, pdinfo->varname[k]);
+
+    pdinfo->t1 = test_t1;
+    pdinfo->t2 = test_t2;
 
     /* Run (A)DF test on the residuals */
     real_adf_test(k, order, nv, pZ, pdinfo, adf_opt, 
@@ -1074,6 +1185,7 @@ int coint (int order, const int *list, double ***pZ,
 		 "(a) The unit-root hypothesis is not rejected for the individual"
 		 " variables.\n(b) The unit-root hypothesis is rejected for the "
 		 "residuals (uhat) from the \n    cointegrating regression.\n"));
+    pputc(prn, '\n');
 
  bailout:
     
@@ -1083,5 +1195,8 @@ int coint (int order, const int *list, double ***pZ,
 	dataset_drop_variable(k, pZ, pdinfo);
     }
 
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
     return err;
 }
