@@ -650,7 +650,7 @@ int bhhh_max (LL_FUNC loglik,
 	    stepsize *= 2.0;
 	}
 
-	/* print interation info, if wanted */
+	/* print iteration info, if wanted */
 	if (opt & OPT_V) {
 	    print_iter_info(iters, minfo->ll, C_LOGLIK, k, minfo->theta, delta, 
 			    stepsize, prn);
@@ -684,12 +684,8 @@ int bhhh_max (LL_FUNC loglik,
 	    gretl_matrix_free(G);
 	}
 	if (minfo->opts & PRESERVE_OPG_MODEL) {
-	    int qr_bak = libset_get_bool(USE_QR);
-
-	    /* run OPG once more using QR, to get packed VCV */
-	    libset_set_bool(USE_QR, 1);
-	    *bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A);
-	    libset_set_bool(USE_QR, qr_bak);
+	    /* run OPG once more using QR decomp, to get packed VCV */
+	    *bmod = lsq(blist, &tZ, tinfo, OLS, OPT_A | OPT_Q);
 	    minfo->pmod = bmod;
 	    gretl_model_set_int(bmod, "iters", iters);
 	    if (minfo->bign > minfo->t2) {
