@@ -23,6 +23,7 @@
 #include "csvdata.h"
 
 #include <ctype.h>
+#include <errno.h>
 #include <glib.h>
 
 #define QUOTE      '\''
@@ -33,11 +34,13 @@ enum {
     CSV_HAVEDATA = 1 << 0,
     CSV_GOTDELIM = 1 << 1,
     CSV_GOTTAB   = 1 << 2,
-    CSV_BLANK1   = 1 << 3,
-    CSV_OBS1     = 1 << 4,
-    CSV_TRAIL    = 1 << 5,
-    CSV_AUTONAME = 1 << 6,
-    CSV_NONNUM   = 1 << 7
+    CSV_GOTSEMI  = 1 << 3,
+    CSV_BLANK1   = 1 << 4,
+    CSV_OBS1     = 1 << 5,
+    CSV_TRAIL    = 1 << 6,
+    CSV_AUTONAME = 1 << 7,
+    CSV_NONNUM   = 1 << 8,
+    CSV_REVERSED = 1 << 9
 };
 
 typedef struct csvdata_ csvdata;
@@ -46,7 +49,9 @@ struct csvdata_ {
     int flags;
     char delim;
     int markerpd;
+    int maxlen;
     int real_n;
+    char *line;
     double **Z;
     DATAINFO *dinfo;
     int ncols, nrows;
@@ -55,32 +60,38 @@ struct csvdata_ {
     int *codelist;
     char *descrip;
     gretl_string_table *st;
+    int *cols_list;
+    int *width_list;
 };
 
 #define csv_has_trailing_comma(c) (c->flags & CSV_TRAIL)
 #define csv_has_obs_column(c)     (c->flags & CSV_OBS1)
 #define csv_has_blank_column(c)   (c->flags & CSV_BLANK1)
 #define csv_got_tab(c)            (c->flags & CSV_GOTTAB)
+#define csv_got_semi(c)           (c->flags & CSV_GOTSEMI)
 #define csv_got_delim(c)          (c->flags & CSV_GOTDELIM)
 #define csv_autoname(c)           (c->flags & CSV_AUTONAME)
 #define csv_skip_column(c)        (c->flags & (CSV_OBS1 | CSV_BLANK1))
 #define csv_have_data(c)          (c->flags & CSV_HAVEDATA)
 #define csv_force_nonnum(c)       (c->flags & CSV_NONNUM)
+#define csv_data_reversed(c)      (c->flags & CSV_REVERSED)
 
 #define csv_set_trailing_comma(c)   (c->flags |= CSV_TRAIL)
 #define csv_unset_trailing_comma(c) (c->flags &= ~CSV_TRAIL)
 #define csv_set_obs_column(c)       (c->flags |= CSV_OBS1)
 #define csv_set_blank_column(c)     (c->flags |= CSV_BLANK1)
 #define csv_set_got_tab(c)          (c->flags |= CSV_GOTTAB)
+#define csv_set_got_semi(c)         (c->flags |= CSV_GOTSEMI)
 #define csv_set_got_delim(c)        (c->flags |= CSV_GOTDELIM)
 #define csv_set_autoname(c)         (c->flags |= CSV_AUTONAME)
 #define csv_set_force_nonnum(c)     (c->flags |= CSV_NONNUM)
+#define csv_set_data_reversed(c)    (c->flags |= CSV_REVERSED)
 
 #define csv_skipping(c)        (*c->skipstr != '\0')
 #define csv_has_non_numeric(c) (c->st != NULL)
 
 static int 
-time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn);
+time_series_label_check (DATAINFO *pdinfo, int reversed, char *skipstr, PRN *prn);
 
 static void csvdata_free (csvdata *c)
 {
@@ -100,6 +111,15 @@ static void csvdata_free (csvdata *c)
 	free(c->codelist);
     }
 
+    if (c->line != NULL) {
+	free(c->line);
+    }  
+
+    if (c->cols_list != NULL) {
+	free(c->cols_list);
+	free(c->width_list);
+    }  
+
     destroy_dataset(c->Z, c->dinfo);
 
     free(c);
@@ -117,7 +137,9 @@ static csvdata *csvdata_new (double **Z, const DATAINFO *pdinfo,
     c->flags = (opt & OPT_C)? CSV_NONNUM : 0;
     c->delim = '\t';
     c->markerpd = -1;
+    c->maxlen = 0;
     c->real_n = 0;
+    c->line = NULL;
     c->Z = NULL;
     c->dinfo = NULL;
     c->ncols = 0;
@@ -127,6 +149,8 @@ static csvdata *csvdata_new (double **Z, const DATAINFO *pdinfo,
     c->codelist = NULL;
     c->descrip = NULL;
     c->st = NULL;
+    c->cols_list = NULL;
+    c->width_list = NULL;
 
     c->dinfo = datainfo_new();
     if (c->dinfo == NULL) {
@@ -145,6 +169,68 @@ static csvdata *csvdata_new (double **Z, const DATAINFO *pdinfo,
     return c;
 }
 
+static int csvdata_add_cols_list (csvdata *c, const char *s)
+{
+    int *list, *clist = NULL, *wlist = NULL;
+    int i, n, m, err = 0;
+
+    list = gretl_list_from_string(s, &err);
+
+    if (!err) {
+	n = list[0];
+	if (n == 0 || n % 2 != 0) {
+	    err = E_DATA;
+	} else {
+	    m = n / 2;
+	    clist = gretl_list_new(m);
+	    wlist = gretl_list_new(m);
+	    if (clist == NULL || wlist == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+    }
+
+    if (!err) {
+	int j = 1;
+
+	for (i=1; i<=n; i+=2, j++) {
+	    clist[j] = list[i];
+	    wlist[j] = list[i+1];
+	}
+
+	/* clist = column start list: must be a set of increasing
+	   positive integers; and wlist = respective column widths,
+	   must all be positive 
+	*/
+	for (i=1; i<=m; i++) {
+	    if (wlist[i] <= 0 || clist[i] <= 0 || 
+		(i > 1 && clist[i] <= clist[i-1])) {
+		err = E_DATA;
+		break;
+	    } else if (wlist[i] >= CSVSTRLEN) {
+		fprintf(stderr, "Warning: field %d too wide (%d), truncating\n", 
+			i, wlist[i]);
+		wlist[i] = CSVSTRLEN - 1;
+	    }
+	}
+    }
+
+    free(list);
+
+    if (!err) {
+	c->cols_list = clist;
+	c->width_list = wlist;
+    } else {
+	free(clist);
+	free(wlist);
+	if (err == E_DATA) {
+	    gretl_errmsg_set(_("Invalid column specification"));
+	}
+    }
+
+    return err;
+}
+
 static int add_obs_marker (DATAINFO *pdinfo, int n)
 {
     char **S;
@@ -278,12 +364,13 @@ static int csv_weekly_data (double ***pZ, DATAINFO *pdinfo)
 
 #define DAY_DEBUG 1
 
-static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
+static int check_daily_dates (DATAINFO *pdinfo, int *pd, int *reversed, PRN *prn)
 {
     int T = pdinfo->n;
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[T - 1];
     int fulln = 0, n, t, nbak;
+    int alt_pd = 0;
     int oldpd = pdinfo->pd;
     double oldsd0 = pdinfo->sd0;
     long ed1, ed2;
@@ -309,18 +396,30 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
 
     if (!err) {
 	ed2 = get_epoch_day(lbl2);
-	if (ed2 <= ed1) {
+	if (ed2 < 0) {
 	    err = 1;
+	} else if (ed2 < ed1) {
+#if DAY_DEBUG    
+	    fprintf(stderr, "check_daily_dates: data are reversed?\n");
+#endif
+	    pdinfo->sd0 = ed2;
+	    *reversed = 1;
 	} else {
 	    pdinfo->sd0 = ed1;
 	}
     }
 
+ recompute:
+
+    alt_pd = 0;
+    nbak = 0;
+
     if (!err) {
-	int n1 = calendar_obs_number(lbl1, pdinfo);
-	int n2 = calendar_obs_number(lbl2, pdinfo);
+	int n1 = calendar_obs_number((*reversed)? lbl2 : lbl1, pdinfo);
+	int n2 = calendar_obs_number((*reversed)? lbl1 : lbl2, pdinfo);
 
 	fulln = n2 - n1 + 1;
+
 	if (T > fulln) {
 	    err = 1;
 	} else {
@@ -346,30 +445,54 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
     }
 
     nbak = 0;
+
     for (t=0; t<pdinfo->n && !err; t++) {
-	n = calendar_obs_number(pdinfo->S[t], pdinfo);
+	int wd, s = (*reversed)? (pdinfo->n - 1 - t) : t;
+
+	wd = get_day_of_week(pdinfo->S[s]);
+
+	if (pdinfo->pd == 5 && (wd == 6 || wd == 0)) {
+	    /* Got Sat or Sun, can't be 5-day daily? */
+	    alt_pd = (wd == 6)? 6 : 7;
+	    pprintf(prn, "Found a Saturday (%s): re-trying with pd = %d\n", 
+		    pdinfo->S[s], alt_pd);
+	    break;
+	} else if (pdinfo->pd == 6 && wd == 0) {
+	    /* Got Sun, can't be 6-day daily? */
+	    alt_pd = 7;
+	    pprintf(prn, "Found a Sunday (%s): re-trying with pd = %d\n", 
+		    pdinfo->S[s], alt_pd);
+	    break;
+	}
+	    
+	n = calendar_obs_number(pdinfo->S[s], pdinfo);
 	if (n < t) {
 	    pprintf(prn, "Daily dates error at t = %d:\n"
 		    "  calendar_obs_number() for '%s' = %d but t = %d\n", 
 		    t, pdinfo->S[t], n, t);
 	    err = 1;
 	} else if (n > fulln - 1) {
-	    pprintf(prn, "Error: date '%s' out of bounds\n", pdinfo->S[t]);
+	    pprintf(prn, "Error: date '%s' out of bounds\n", pdinfo->S[s]);
 	    err = 1;
 	} else if (nbak > 0 && n == nbak) {
-	    pprintf(prn, "Error: date '%s' is repeated\n", pdinfo->S[t]);
+	    pprintf(prn, "Error: date '%s' is repeated\n", pdinfo->S[s]);
 	    err = 1;
 	}
 	nbak = n;
     }
 
+    if (alt_pd > 0) {
+	pdinfo->pd = alt_pd;
+	goto recompute;
+    }
+
     if (err) {
 	pdinfo->pd = oldpd;
 	pdinfo->sd0 = oldsd0;
 	pdinfo->structure = CROSS_SECTION;
     } else {
-	strcpy(pdinfo->stobs, lbl1);
-	strcpy(pdinfo->endobs, lbl2);
+	strcpy(pdinfo->stobs, (*reversed)? lbl2 : lbl1);
+	strcpy(pdinfo->endobs, (*reversed)? lbl1 : lbl2);
 	pdinfo->t2 = pdinfo->n - 1;
 	if (nmiss > 0 && *pd == 0) {
 	    pdinfo->markers = DAILY_DATE_STRINGS;
@@ -377,8 +500,8 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
     }
 
 #if DAY_DEBUG
-    fprintf(stderr, "check_daily_dates: pd = %d, err = %d\n", 
-	    pdinfo->pd, err);
+    fprintf(stderr, "check_daily_dates: pd = %d, reversed = %d, err = %d\n", 
+	    pdinfo->pd, *reversed, err);
 #endif
 
     return (err)? -1 : pdinfo->pd;
@@ -398,12 +521,13 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd, PRN *prn)
 
 #define fakequarter(m) (m==3 || m==6 || m==9 || m==12) 
 
-static int complete_qm_labels (DATAINFO *pdinfo, char *skipstr,
-			       int *ppd, const char *fmt,
+static int complete_qm_labels (DATAINFO *pdinfo, int reversed,
+			       char *skipstr, int *ppd, 
+			       const char *fmt,
 			       PRN *prn)
 {
     char bad[16], skip[8];
-    int t, y, p, Ey, Ep;
+    int t, s, yr, per, Ey, Ep;
     int pmin = 1;
     int pd, pd0;
     int ret = 1;
@@ -412,33 +536,35 @@ static int complete_qm_labels (DATAINFO *pdinfo, char *skipstr,
 
  restart:
 
-    if (sscanf(pdinfo->S[0], fmt, &y, &p) != 2) {
+    s = (reversed)? (pdinfo->n - 1) : 0;
+    if (sscanf(pdinfo->S[s], fmt, &yr, &per) != 2) {
 	return 0;
     }
 
     for (t=1; t<pdinfo->n; t++) {
-	Ey = (p == pd)? y + 1 : y;
-	Ep = (p == pd)? pmin : p + pmin;
-	if (sscanf(pdinfo->S[t], fmt, &y, &p) != 2) {
+	s = (reversed)? (pdinfo->n - 1 - t) : t;
+	Ey = (per == pd)? yr + 1 : yr;
+	Ep = (per == pd)? pmin : per + pmin;
+	if (sscanf(pdinfo->S[s], fmt, &yr, &per) != 2) {
 	    ret = 0;
-	} else if (Ep == 1 && pd == pd0 && p == pd + 1 
+	} else if (Ep == 1 && pd == pd0 && per == pd + 1 
 		   && skipstr != NULL) {
 	    *skip = *bad = '\0';
-	    strncat(skip, pdinfo->S[t] + 4, 7); 
-	    strncat(bad, pdinfo->S[t], 15); 
+	    strncat(skip, pdinfo->S[s] + 4, 7); 
+	    strncat(bad, pdinfo->S[s], 15); 
 	    pd = pd0 + 1;
 	    goto restart;
-	} else if (p == Ep + 2 && pmin == 1 && fakequarter(p)) {
+	} else if (per == Ep + 2 && pmin == 1 && fakequarter(per)) {
 	    *bad = '\0';
-	    strncat(bad, pdinfo->S[t], 15); 
+	    strncat(bad, pdinfo->S[s], 15); 
 	    pmin = 3;
 	    goto restart;
-	} else if (y != Ey || p != Ep) {
+	} else if (yr != Ey || per != Ep) {
 	    ret = 0;
 	}
 	if (!ret) {
 	    pprintf(prn, "   %s: not a consistent date\n", 
-		    pdinfo->S[t]);
+		    pdinfo->S[s]);
 	    break;
 	}
     }
@@ -458,13 +584,17 @@ static int complete_qm_labels (DATAINFO *pdinfo, char *skipstr,
     return ret;
 }
 
-static int complete_year_labels (const DATAINFO *pdinfo)
+static int complete_year_labels (const DATAINFO *pdinfo, int reversed)
 {
-    int t, yr, yrbak = atoi(pdinfo->S[0]);
+    int s, t, yr, yrbak;
     int ret = 1;
 
+    s = (reversed)? (pdinfo->n - 1) : 0;
+    yrbak = atoi(pdinfo->S[s]);
+
     for (t=1; t<pdinfo->n; t++) {
-	yr = atoi(pdinfo->S[t]);
+	s = (reversed)? (pdinfo->n - 1 - t) : t;
+	yr = atoi(pdinfo->S[s]);
 	if (yr != yrbak + 1) {
 	    ret = 0;
 	    break;
@@ -508,6 +638,7 @@ int get_date_order (int f0, int fn)
 	/* first field must be day */
 	return DDMMYYYY;
     } else {
+	/* could be wrong here */
 	return MMDDYYYY;
     }
 }
@@ -529,23 +660,22 @@ static void retransform_daily_dates (DATAINFO *pdinfo)
 
 static int transform_daily_dates (DATAINFO *pdinfo, int dorder)
 {
+    char *label;
     int t, yr, mon, day;
-    char sep1[2], sep2[2];
-    int sret, err = 0;
+    char s1, s2;
+    int n, err = 0;
 
     for (t=0; t<pdinfo->n && !err; t++) {
+	label = pdinfo->S[t];
 	if (dorder == YYYYMMDD) {
-	    sret = sscanf(pdinfo->S[t], "%d%1[/-.]%d%1[/-.]%d", 
-			  &yr, sep1, &mon, sep2, &day);
+	    n = sscanf(label, "%d%c%d%c%d", &yr, &s1, &mon, &s2, &day);
 	} else if (dorder == DDMMYYYY) {
-	    sret = sscanf(pdinfo->S[t], "%d%1[/-.]%d%1[/-.]%d", 
-			  &day, sep1, &mon, sep2, &yr);
+	    n = sscanf(label, "%d%c%d%c%d", &day, &s1, &mon, &s2, &yr);
 	} else {
-	    sret = sscanf(pdinfo->S[t], "%d%1[/-.]%d%1[/-.]%d", 
-			  &mon, sep1, &day, sep2, &yr);
+	    n = sscanf(label, "%d%c%d%c%d", &mon, &s1, &day, &s2, &yr);
 	}
-	if (sret == 5) {
-	    sprintf(pdinfo->S[t], "%02d/%02d/%02d", yr, mon, day);
+	if (n == 5) {
+	    sprintf(label, "%02d/%02d/%02d", yr, mon, day);
 	} else {
 	    err = 1;
 	}
@@ -554,18 +684,43 @@ static int transform_daily_dates (DATAINFO *pdinfo, int dorder)
     return err;
 }
 
+void reverse_data (double **Z, DATAINFO *pdinfo, PRN *prn)
+{
+    char tmp[OBSLEN];
+    double x;
+    int T = pdinfo->n / 2;
+    int i, t, s;
+
+    pprintf(prn, M_("reversing the data!\n"));
+
+    for (t=0; t<T; t++) {
+	s = pdinfo->n - 1 - t;
+	for (i=1; i<pdinfo->v; i++) {
+	    x = Z[i][t];
+	    Z[i][t] = Z[i][s];
+	    Z[i][s] = x;
+	}
+	if (pdinfo->S != NULL) {
+	    strcpy(tmp, pdinfo->S[t]);
+	    strcpy(pdinfo->S[t], pdinfo->S[s]);
+	    strcpy(pdinfo->S[s], tmp);
+	}
+    }
+}
+
 static int 
-csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr, 
-		      PRN *prn)
+csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
+		      char *skipstr, PRN *prn)
 {
     int d1[3], d2[3];
-    char sep1[2], sep2[2];
+    char s1, s2;
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[pdinfo->n - 1];
     int dorder = 0;
 
-    if (sscanf(lbl1, "%d%1[/-.]%d%1[/-.]%d", &d1[0], sep1, &d1[1], sep2, &d1[2]) == 5 &&
-	sscanf(lbl2, "%d%1[/-.]%d%1[/-.]%d", &d2[0], sep1, &d2[1], sep2, &d2[2]) == 5) {
+    if (sscanf(lbl1, "%d%c%d%c%d", &d1[0], &s1, &d1[1], &s2, &d1[2]) == 5 &&
+	sscanf(lbl2, "%d%c%d%c%d", &d2[0], &s1, &d2[1], &s2, &d2[2]) == 5 &&
+	s1 == s2 && ispunct(s1)) {
 	int yr1, mon1, day1;
 	int yr2, mon2, day2;
 	int pd, ret = 0;
@@ -600,19 +755,18 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr,
 	    yr2 = d2[2];
 	}		
 	    
-	if (yr2 >= yr1 && 
-	    mon1 > 0 && mon1 < 13 &&
+	if (mon1 > 0 && mon1 < 13 &&
 	    mon2 > 0 && mon2 < 13 && 
 	    day1 > 0 && day1 < 32 &&
 	    day2 > 0 && day2 < 32) {
 	    /* looks promising for calendar dates */
-	    if (dorder != YYYYMMDD || *sep1 != '/' || *sep2 != '/') {
+	    if (dorder != YYYYMMDD || s1 != '/' || s2 != '/') {
 		if (transform_daily_dates(pdinfo, dorder)) {
 		    return -1;
 		}
 	    }
 	    pprintf(prn, "Could be %s - %s\n", lbl1, lbl2);
-	    ret = check_daily_dates(pdinfo, &pd, prn);
+	    ret = check_daily_dates(pdinfo, &pd, reversed, prn);
 	    if (ret >= 0 && pd > 0) {
 		if (pd == 52) {
 		    if (csv_weekly_data(pZ, pdinfo)) {
@@ -628,13 +782,16 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr,
 		} else {
 		    compress_daily(pdinfo, pd);
 		    ret = time_series_label_check(pdinfo, 
+						  *reversed,
 						  skipstr, 
 						  prn);
 		}
 	    } 
 	    return ret;
 	}
-    } 
+    } else {
+	pprintf(prn, "'%s' and '%s': couldn't get dates\n", lbl1, lbl2);
+    }
 
     return -1;
 }
@@ -642,7 +799,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr,
 static int pd_from_date_label (const char *lbl, char *year, char *subp,
 			       char *format, PRN *prn)
 {
-    const char *subchars = ".:QqMmPp";
+    const char *subchars = ".:QqMmPp-";
     int len = strlen(lbl);
     int try, pd = -1;
 
@@ -698,8 +855,8 @@ static int pd_from_date_label (const char *lbl, char *year, char *subp,
     return pd;
 }
 
-static int 
-time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
+static int time_series_label_check (DATAINFO *pdinfo, int reversed,
+				    char *skipstr, PRN *prn)
 {
     char year[5], sub[3];
     char format[8] = {0};
@@ -707,10 +864,11 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
     char *lbl2 = pdinfo->S[pdinfo->n - 1];
     int pd = -1;
 
-    pd = pd_from_date_label(lbl1, year, sub, format, prn);
+    pd = pd_from_date_label((reversed)? lbl2 : lbl1, year, sub, 
+			    format, prn);
 
     if (pd == 1) {
-	if (complete_year_labels(pdinfo)) {
+	if (complete_year_labels(pdinfo, reversed)) {
 	    pdinfo->pd = pd;
 	    strcpy(pdinfo->stobs, year);
 	    pdinfo->sd0 = atof(pdinfo->stobs);
@@ -722,7 +880,7 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
     } else if (pd == 4 || pd == 12) {
 	int savepd = pd;
 
-	if (complete_qm_labels(pdinfo, skipstr, &pd, format, prn)) {
+	if (complete_qm_labels(pdinfo, reversed, skipstr, &pd, format, prn)) {
 	    pdinfo->pd = pd;
 	    if (savepd == 12 && pd == 4) {
 		int s = atoi(sub) / 3;
@@ -742,6 +900,26 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
     return pd;
 }
 
+static int dates_maybe_reversed (const char *s1, const char *s2,
+				 PRN *prn)
+{
+    char d1[5], d2[5];
+    int ret = 0;
+
+    *d1 = *d2 = '\0';
+
+    strncat(d1, s1, 4);
+    strncat(d2, s2, 4);
+
+    ret = atoi(d1) > atoi(d2);
+
+    if (ret) {
+	pputs(prn, M_("   dates are reversed?\n"));
+    }
+    
+    return ret;
+}
+
 /* Attempt to parse csv row labels as dates.  Return -1 if this
    doesn't work out, or 0 if the labels seem to be just integer
    observation numbers, else return the inferred data frequency.  The
@@ -750,16 +928,18 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
 */
 
 int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo, 
-			    char *skipstr, PRN *prn)
+			    int *reversed, char *skipstr, 
+			    PRN *prn)
 {
     char endobs[OBSLEN];
     int n = pdinfo->n;
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[n - 1];
     int len1 = strlen(lbl1);
+    int pd = -1;
 
     if (skipstr != NULL && *skipstr != '\0') {
-	return time_series_label_check(pdinfo, skipstr, prn);
+	return time_series_label_check(pdinfo, *reversed, skipstr, prn);
     }
 
     pprintf(prn, M_("   first row label \"%s\", last label \"%s\"\n"), 
@@ -781,20 +961,27 @@ int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo,
 
     if (len1 == 8 || len1 == 10) {
 	/* daily data? */
-	return csv_daily_date_check(pZ, pdinfo, skipstr, prn);
+	pd = csv_daily_date_check(pZ, pdinfo, reversed, skipstr, prn);
     } else if (len1 >= 4) {
 	/* annual, quarterly, monthly? */
 	if (isdigit((unsigned char) lbl1[0]) &&
 	    isdigit((unsigned char) lbl1[1]) &&
 	    isdigit((unsigned char) lbl1[2]) && 
 	    isdigit((unsigned char) lbl1[3])) {
-	    return time_series_label_check(pdinfo, skipstr, prn);
+	    *reversed = dates_maybe_reversed(lbl1, lbl2, prn);
+	    pd = time_series_label_check(pdinfo, *reversed, skipstr, prn);
 	} else {
 	    pputs(prn, M_("   definitely not a four-digit year\n"));
 	}
     }
 
-    return -1;
+    if (pd <= 0 && *reversed) {
+	/* give up the "reversed" notion if we didn't get
+	   a workable time-series interpretation */
+	*reversed = 0;
+    }
+
+    return pd;
 }
 
 /* The function below checks for the maximum line length in the given
@@ -853,6 +1040,9 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
 	    if (c == '\t') {
 		csv_set_got_tab(cdata);
 	    }
+	    if (c == ';') {
+		csv_set_got_semi(cdata);
+	    }
 	    if (c == cdata->delim) {
 		csv_set_got_delim(cdata);
 	    }
@@ -874,19 +1064,27 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
     return maxlen;
 }
 
-static int count_csv_fields (const char *line, char delim)
+#define nonspace_delim(d) (d != ',' && d != ';')
+
+static int count_csv_fields (const char *s, char delim)
 {
     int cbak, nf = 0;
-    const char *p = line;
 
-    if (*p == delim && *p == ' ') p++;
+    if (*s == delim && *s == ' ') {
+	s++;
+    }
 
-    while (*p) {
-	if (*p == delim) nf++;
-	cbak = *p;
-	p++;
-	/* Problem: (when) should trailing delimiter be read as implicit "NA"? */
-	if (*p == '\0' && cbak == delim && cbak != ',') {
+    while (*s) {
+	if (*s == delim) {
+	    nf++;
+	}
+	cbak = *s;
+	s++;
+	/* Problem: (when) should a trailing delimiter be read as an
+	   implicit NA?  For now we'll so treat it if the delimiter
+	   is not white space.
+	*/
+	if (*s == '\0' && cbak == delim && nonspace_delim(delim)) {
 	    nf--;
 	}
     }
@@ -909,10 +1107,10 @@ static void remove_quoted_commas (char *s)
     }
 }
 
-static void compress_csv_line (char *line, csvdata *c)
+static void compress_csv_line (csvdata *c)
 {
-    int n = strlen(line);
-    char *p = line + n - 1;
+    int n = strlen(c->line);
+    char *p = c->line + n - 1;
 
     if (*p == '\n') {
 	*p = '\0';
@@ -922,22 +1120,22 @@ static void compress_csv_line (char *line, csvdata *c)
     if (*p == '\r') *p = '\0';
 
     if (c->delim == ',') {
-	remove_quoted_commas(line);
+	remove_quoted_commas(c->line);
     }
 
     if (c->delim != ' ') {
-	delchar(' ', line);
+	delchar(' ', c->line);
     } else {
-	compress_spaces(line);
+	compress_spaces(c->line);
     }
 
-    delchar('"', line);
+    delchar('"', c->line);
 
     if (csv_has_trailing_comma(c)) {
 	/* chop trailing comma */
-	n = strlen(line);
+	n = strlen(c->line);
 	if (n > 0) {
-	    line[n-1] = '\0';
+	    c->line[n-1] = '\0';
 	}
     }
 }
@@ -1112,6 +1310,33 @@ static int non_numeric_check (csvdata *c, PRN *prn)
     return err;
 }
 
+/* remedial function: we may be trying to read a "CSV" file
+   that uses ',' as decimal separator, on a platform where
+   '.' is the decimal separator
+*/
+
+static double try_comma_atof (const char *orig)
+{
+    char *test, s[32];
+    double x;
+
+    if (strlen(orig) > 31) {
+	return NON_NUMERIC;
+    }
+
+    strcpy(s, orig);
+    charsub(s, ',', '.');
+
+    errno = 0;
+    x = strtod(s, &test);
+
+    if (*test == '\0' && errno != ERANGE) {
+	return x;
+    } else {
+	return NON_NUMERIC;
+    }
+}
+
 static int process_csv_obs (csvdata *c, int i, int t, PRN *prn)
 {
     int ix, err = 0;
@@ -1128,7 +1353,12 @@ static int process_csv_obs (csvdata *c, int i, int t, PRN *prn)
     } else if (csv_missval(c->str, i, t+1, prn)) {
 	c->Z[i][t] = NADBL;
     } else if (check_atof(c->str)) {
-	c->Z[i][t] = NON_NUMERIC;
+	if (c->delim != ',' && get_local_decpoint() != ',' &&
+	    strchr(c->str, ',') != NULL) {
+	    c->Z[i][t] = try_comma_atof(c->str);
+	} else {
+	    c->Z[i][t] = NON_NUMERIC;
+	}
     } else {
 	c->Z[i][t] = atof(c->str);
     }
@@ -1216,8 +1446,7 @@ csv_msg = N_("\nPlease note:\n"
 	     "- The remainder of the file must be a rectangular "
 	     "array of data.\n");
 
-static int csv_fields_check (char *line, int maxlen, FILE *fp,
-			     csvdata *c, PRN *prn)
+static int csv_fields_check (FILE *fp, csvdata *c, PRN *prn)
 {
     int gotdata = 0;
     int chkcols = 0;
@@ -1225,18 +1454,18 @@ static int csv_fields_check (char *line, int maxlen, FILE *fp,
 
     c->ncols = c->nrows = 0;
 
-    while (csv_fgets(line, maxlen, fp) && !err) {
+    while (csv_fgets(c->line, c->maxlen, fp) && !err) {
 
 	/* skip comment lines */
-	if (*line == '#') {
+	if (*c->line == '#') {
 	    continue;
 	}
 
 	/* skip blank lines -- but finish if the blank comes after data */
-	if (string_is_blank(line)) {
+	if (string_is_blank(c->line)) {
 	    if (gotdata) {
 		if (!csv_have_data(c)) {
-		    c->descrip = get_csv_descrip(line, maxlen, fp);
+		    c->descrip = get_csv_descrip(c->line, c->maxlen, fp);
 		}
 		break;
 	    } else {
@@ -1245,26 +1474,43 @@ static int csv_fields_check (char *line, int maxlen, FILE *fp,
 	}
 	
 	c->nrows += 1;
-	compress_csv_line(line, c);
+
+	if (c->cols_list != NULL) {
+	    tailstrip(c->line);
+	    gotdata = 1;
+	    chkcols = strlen(c->line);
+	    if (chkcols < c->cols_list[c->cols_list[0]]) {
+		gretl_errmsg_set(_("Invalid column specification"));
+		err = E_DATA;
+		break;
+	    } else {
+		continue;
+	    }
+	}
+
+	compress_csv_line(c);
 
 	if (!gotdata) {
 	    /* scrutinize first "real" line */
-	    check_first_field(line, c, prn);
+	    check_first_field(c->line, c, prn);
 	    gotdata = 1;
 	} 
 
-	chkcols = count_csv_fields(line, c->delim);
+	chkcols = count_csv_fields(c->line, c->delim);
 	if (c->ncols == 0) {
 	    c->ncols = chkcols;
 	    pprintf(prn, M_("   number of columns = %d\n"), c->ncols);	    
 	} else if (chkcols != c->ncols) {
 	    pprintf(prn, M_("   ...but row %d has %d fields: aborting\n"),
 		    c->nrows, chkcols);
-	    pputs(prn, M_(csv_msg));
 	    err = E_DATA;
 	}
     }
 
+    if (!err && c->cols_list != NULL) {
+	c->ncols = c->cols_list[0];
+    }
+
     return err;
 }
 
@@ -1296,8 +1542,7 @@ csv_reconfigure_for_markers (double ***pZ, DATAINFO *pdinfo)
 
 #define obs_labels_no_varnames(o,c,n)  (!o && c->v > 3 && n == c->v - 2)
 
-static int csv_varname_scan (csvdata *c, char *line, int maxlen, FILE *fp,
-			     PRN *prn, PRN *mprn)
+static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
 {
     char *p;
     int obscol = csv_has_obs_column(c);
@@ -1306,17 +1551,17 @@ static int csv_varname_scan (csvdata *c, char *line, int maxlen, FILE *fp,
 
     pputs(mprn, M_("scanning for variable names...\n"));
 
-    while (csv_fgets(line, maxlen, fp)) {
-	if (*line == '#' || string_is_blank(line)) {
+    while (csv_fgets(c->line, c->maxlen, fp)) {
+	if (*c->line == '#' || string_is_blank(c->line)) {
 	    continue;
 	} else {
 	    break;
 	}
     }
 
-    compress_csv_line(line, c);   
+    compress_csv_line(c);   
 
-    p = line;
+    p = c->line;
     if (c->delim == ' ' && *p == ' ') p++;
     iso_to_ascii(p);
     pprintf(mprn, M_("   line: %s\n"), p);
@@ -1355,7 +1600,6 @@ static int csv_varname_scan (csvdata *c, char *line, int maxlen, FILE *fp,
 		    iso_to_ascii(c->dinfo->varname[nv]);
 		    strip_illegals(c->dinfo->varname[nv]);
 		    if (check_varname(c->dinfo->varname[nv])) {
-			fprintf(stderr, "bombing here\n");
 			errmsg(1, prn);
 			err = E_DATA;
 		    }
@@ -1399,9 +1643,68 @@ static int csv_varname_scan (csvdata *c, char *line, int maxlen, FILE *fp,
     return err;
 }
 
+/* read numerical data when we've been given a fixed column-reading
+   specification */
+
+static int fixed_format_read (csvdata *c, FILE *fp, PRN *prn)
+{
+    char *p;
+    int i, k, n, m, t = 0;
+    int err = 0;
+
+    c->real_n = c->dinfo->n;
+
+    while (csv_fgets(c->line, c->maxlen, fp) && !err) {
+
+	tailstrip(c->line);
+
+	if (*c->line == '#' || string_is_blank(c->line)) {
+	    continue;
+	}
+
+	m = strlen(c->line);
+
+	for (i=1; i<=c->ncols; i++) {
+	    k = c->cols_list[i];
+	    n = c->width_list[i];
+	    if (k + n - 1 > m) {
+		/* attempting to read out of bounds */
+		fprintf(stderr, "row %d, column %d: start=%d, width=%d, "
+			"but line length = %d\n", t+1, i, k, n, m);
+		err = E_DATA;
+		break;
+	    }		
+	    p = c->line + k - 1;
+	    *c->str = '\0';
+	    strncat(c->str, p, n);
+	    if (csv_missval(c->str, i, t+1, prn)) {
+		c->Z[i][t] = NADBL;
+	    } else if (check_atof(c->str)) {
+		if (c->delim != ',' && get_local_decpoint() != ',' &&
+		    strchr(c->str, ',') != NULL) {
+		    c->Z[i][t] = try_comma_atof(c->str);
+		} else {
+		    err = E_DATA;
+		}
+	    } else {
+		c->Z[i][t] = atof(c->str);
+	    }
+	}
+
+	if (++t == c->dinfo->n) {
+	    break;
+	}
+    }
+
+    if (err == E_DATA) {
+	gretl_errmsg_set(_("Invalid column specification"));
+    }
+
+    return err;
+}
+
 static int 
-real_read_labels_and_data (csvdata *c, char *line, int maxlen, 
-			   FILE *fp, PRN *prn)
+real_read_labels_and_data (csvdata *c, FILE *fp, PRN *prn)
 {
     char *p;
     int i, k, nv, t = 0;
@@ -1409,19 +1712,19 @@ real_read_labels_and_data (csvdata *c, char *line, int maxlen,
 
     c->real_n = c->dinfo->n;
 
-    while (csv_fgets(line, maxlen, fp) && !err) {
+    while (csv_fgets(c->line, c->maxlen, fp) && !err) {
 
-	if (*line == '#' || string_is_blank(line)) {
+	if (*c->line == '#' || string_is_blank(c->line)) {
 	    continue;
 	}
 
-	if (csv_skipping(c) && strstr(line, c->skipstr)) {
+	if (csv_skipping(c) && strstr(c->line, c->skipstr)) {
 	    c->real_n -= 1;
 	    continue;
 	}
 
-	compress_csv_line(line, c);
-	p = line;
+	compress_csv_line(c);
+	p = c->line;
 	if (c->delim == ' ' && *p == ' ') p++;
 
 	for (k=0; k<c->ncols; k++) {
@@ -1471,17 +1774,20 @@ real_read_labels_and_data (csvdata *c, char *line, int maxlen,
     return err;
 }
 
-static int csv_read_data (csvdata *c, char *line, int maxlen,
-			  FILE *fp, PRN *prn, PRN *mprn)
+static int csv_read_data (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
 {
+    int reversed = csv_data_reversed(c);
     int err;
 
     pputs(mprn, M_("scanning for row labels and data...\n"));
-    err = real_read_labels_and_data(c, line, maxlen, fp, prn);
+    err = real_read_labels_and_data(c, fp, prn);
 
     if (!err && csv_skip_column(c)) {
-	c->markerpd = test_markers_for_dates(&c->Z, c->dinfo, c->skipstr, 
-					     prn);
+	c->markerpd = test_markers_for_dates(&c->Z, c->dinfo, &reversed,
+					     c->skipstr, prn);
+	if (reversed) {
+	    csv_set_data_reversed(c);
+	}
     }
 
     return err;
@@ -1519,14 +1825,25 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 		gretlopt opt, PRN *prn)
 {
     csvdata *c = NULL;
+    const char *cols = NULL;
     int popit = 0;
-    int i, maxlen;
     FILE *fp = NULL;
     PRN *mprn = NULL;
-    char *line = NULL;
     int newdata = (*pZ == NULL);
+    char save_delim = pdinfo->delim;
+    int fixed_format = 0;
     long datapos;
-    int err = 0;
+    int i, err = 0;
+
+    if (opt & OPT_Q) {
+	/* quiet */
+	prn = NULL;
+    }
+
+    if (opt & OPT_F) {
+	/* fixed format: should have --cols=XXX specification */
+	cols = get_optval_string(OPEN, OPT_F);
+    }
 
 #ifdef ENABLE_NLS
     if (prn != NULL) {
@@ -1549,38 +1866,54 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
     if (c == NULL) {
 	err = E_ALLOC;
 	goto csv_bailout;
-    }	
+    }
+
+    if (cols != NULL && *cols != '\0') {
+	fixed_format = 1;
+	pprintf(mprn, M_("using fixed column format\n"));
+	err = csvdata_add_cols_list(c, cols);
+	if (err) {
+	    goto csv_bailout;
+	}
+    }
 
     if (mprn != NULL) {
 	csv_parsing_header(fname, mprn);
     }
 
     /* get line length, also check for binary data, etc. */
-    maxlen = csv_max_line_length(fp, c, prn);    
-    if (maxlen <= 0) {
+    c->maxlen = csv_max_line_length(fp, c, prn);    
+    if (c->maxlen <= 0) {
 	err = E_DATA;
 	goto csv_bailout;
     } 
 
-    if (!csv_got_delim(c)) {
+    if (!fixed_format && !csv_got_delim(c)) {
 	/* set default delimiter */
 	if (csv_got_tab(c)) {
 	    c->delim = c->dinfo->delim = '\t';
+	} else if (csv_got_semi(c)) {
+	    c->delim = c->dinfo->delim = ';';
 	} else {
 	    c->delim = c->dinfo->delim = ' ';
 	}
     }
 
-    pprintf(mprn, M_("using delimiter '%c'\n"), c->delim);
-    pprintf(mprn, M_("   longest line: %d characters\n"), maxlen - 1);
+ alt_delim:
+
+    if (!fixed_format) {
+	pprintf(mprn, M_("using delimiter '%c'\n"), c->delim);
+    }
+
+    pprintf(mprn, M_("   longest line: %d characters\n"), c->maxlen - 1);
 
     if (csv_has_trailing_comma(c) && c->delim != ',') {
 	csv_unset_trailing_comma(c);
     }
 
     /* buffer to hold lines */
-    line = malloc(maxlen);
-    if (line == NULL) {
+    c->line = malloc(c->maxlen);
+    if (c->line == NULL) {
 	err = E_ALLOC;
 	goto csv_bailout;
     }  
@@ -1588,13 +1921,24 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
     rewind(fp);
 
     /* read lines, check for consistency in number of fields */
-    err = csv_fields_check(line, maxlen, fp, c, mprn);
-    if (err) {
+    err = csv_fields_check(fp, c, mprn);
+    if (err && !fixed_format) {
+	if (c->delim != ';' && csv_got_semi(c)) {
+	    c->delim = c->dinfo->delim = ';';
+	    err = 0;
+	    goto alt_delim;
+	}
+	pputs(prn, M_(csv_msg));
 	goto csv_bailout;
     }
 
-    c->dinfo->n = c->nrows - 1; /* allow for var headings */
-    c->dinfo->v = (csv_skip_column(c))? c->ncols : c->ncols + 1;
+    if (fixed_format) {
+	c->dinfo->n = c->nrows;
+	c->dinfo->v = c->ncols + 1;
+    } else {
+	c->dinfo->n = c->nrows - 1; /* allow for var headings */
+	c->dinfo->v = (csv_skip_column(c))? c->ncols : c->ncols + 1;
+    }
 
     pprintf(mprn, M_("   number of variables: %d\n"), c->dinfo->v - 1);
     pprintf(mprn, M_("   number of non-blank lines: %d\n"), c->nrows);
@@ -1622,7 +1966,17 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     rewind(fp);
 
-    err = csv_varname_scan(c, line, maxlen, fp, prn, mprn);
+    if (fixed_format) {
+	err = fixed_format_read(c, fp, prn);
+	if (err) {
+	    goto csv_bailout;
+	} else {
+	    csv_set_autoname(c);
+	    goto csv_continue;
+	}
+    }
+
+    err = csv_varname_scan(c, fp, prn, mprn);
     if (err) {
 	goto csv_bailout;
     }
@@ -1634,12 +1988,12 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     datapos = ftell(fp);
 
-    err = csv_read_data(c, line, maxlen, fp, prn, mprn);
+    err = csv_read_data(c, fp, prn, mprn);
 
     if (!err && csv_skipping(c)) {
 	/* try again */
 	fseek(fp, datapos, SEEK_SET);
-	err = csv_read_data(c, line, maxlen, fp, prn, NULL);
+	err = csv_read_data(c, fp, prn, NULL);
     }
 
     if (!err) {
@@ -1647,7 +2001,7 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	if (!err && csv_has_non_numeric(c)) {
 	    /* try once more */
 	    fseek(fp, datapos, SEEK_SET);
-	    err = csv_read_data(c, line, maxlen, fp, prn, NULL);
+	    err = csv_read_data(c, fp, prn, NULL);
 	}
     }	
 
@@ -1659,6 +2013,12 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	goto csv_bailout;
     }
 
+    if (csv_data_reversed(c)) {
+	reverse_data(c->Z, c->dinfo, mprn);
+    }
+
+ csv_continue:
+
     c->dinfo->t1 = 0;
     c->dinfo->t2 = c->dinfo->n - 1;
 
@@ -1714,7 +2074,6 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	fclose(fp);
     }
 
-    free(line);
     csvdata_free(c);
 
     if (err == E_ALLOC) {
@@ -1725,5 +2084,7 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
     console_off();
 #endif
 
+    pdinfo->delim = save_delim;
+
     return err;
 }
