@@ -636,9 +636,42 @@ static int get_dot_pos (const char *s)
 
 #define DATES_DEBUG 0
 
+static int match_obs_marker (const char *s, const DATAINFO *pdinfo)
+{
+    char test[OBSLEN];
+    int t;
+
+#if DATES_DEBUG
+    fprintf(stderr, "dateton: checking marker strings\n");
+#endif
+
+    maybe_unquote_label(test, s);
+
+    for (t=0; t<pdinfo->n; t++) {
+	if (!strcmp(test, pdinfo->S[t])) {
+	    /* handled */
+	    return t;
+	}
+    }
+
+    if (isalpha(*s)) {
+	/* try harder */
+	int k = strlen(test);
+
+	for (t=0; t<pdinfo->n; t++) {
+	    if (!strncmp(test, pdinfo->S[t], k)) {
+		return t;
+	    }
+	}
+    }
+
+    return -1;
+}
+
 static int 
 real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 {
+    int handled = 0;
     int t, n = -1;
 
     /* first check if this is calendar data and if so,
@@ -672,53 +705,38 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 	} else {
 	    /* automatic calendar dates */
 	    n = calendar_obs_number(date, pdinfo);
+	    handled = 1;
 	} 
-    } 
-
-    /* now try treating as an undated time series */
-
-    else if (dataset_is_daily(pdinfo) ||
-	     dataset_is_weekly(pdinfo)) {
+    } else if (dataset_is_daily(pdinfo) ||
+	       dataset_is_weekly(pdinfo)) {
 #if DATES_DEBUG
-	fprintf(stderr, "dateton: treating as undated time series\n");
+	fprintf(stderr, "dateton: trying undated time series\n");
 #endif
-	if (sscanf(date, "%d", &t) && t > 0) {
+	t = positive_int_from_string(date);
+	if (t > 0) {
 	    n = t - 1;
+	    handled = 1;
 	}
-    } 
-
-    /* string observation markers other than dates? */
-
-    else if (pdinfo->markers && pdinfo->S != NULL) {
-	char test[OBSLEN];
-
-#if DATES_DEBUG
-	fprintf(stderr, "dateton: working from marker strings\n");
-#endif
-	maybe_unquote_label(test, date);
-	for (t=0; t<pdinfo->n; t++) {
-	    if (!strcmp(test, pdinfo->S[t])) {
-		/* handled */
-		return t;
-	    }
+    } else if (dataset_is_decennial(pdinfo)) {
+	t = positive_int_from_string(date);
+	if (t > 0) {
+	    n = (t - pdinfo->sd0) / 10;
+	    handled = 1;
+	}	
+    } else if (pdinfo->markers && pdinfo->S != NULL) {
+	t = match_obs_marker(date, pdinfo);
+	if (t >= 0) {
+	    return t;
 	}
 	/* else maybe just a straight obs number */
-	if (sscanf(date, "%d", &t) && t > 0) {
+	t = positive_int_from_string(date);
+	if (t > 0) {
 	    n = t - 1;
+	    handled = 1;
 	}
     }
 
-    /* decennial data? */
-
-    else if (dataset_is_decennial(pdinfo)) {
-	if (sscanf(date, "%d", &t) && t > 0) {
-	    n = (t - pdinfo->sd0) / 10;
-	}	
-    }
-
-    /* treat as "regular" numeric obs number or date */
-
-    else {
+    if (!handled) {
 	int dotpos1, dotpos2;
 
 #if DATES_DEBUG
@@ -760,7 +778,7 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 
     if (!nolimit && pdinfo->n > 0 && n >= pdinfo->n) {
 	fprintf(stderr, "n = %d, pdinfo->n = %d: out of bounds\n", n, pdinfo->n);
-	sprintf(gretl_errmsg, _("Observation number out of bounds"));
+	gretl_errmsg_set(_("Observation number out of bounds"));
 	n = -1; 
     }
 
@@ -1244,7 +1262,7 @@ int write_data (const char *fname, int *list,
     int i, t, v, l0;
     GretlDataFormat fmt;
     char datfile[MAXLEN], hdrfile[MAXLEN], lblfile[MAXLEN];
-    int tsamp = pdinfo->t2 - pdinfo->t1 + 1;
+    int tsamp = sample_size(pdinfo);
     int n = pdinfo->n;
     char delim = 0;
     FILE *fp = NULL;
@@ -2300,9 +2318,12 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-    if (simple_structure(pdinfo) && simple_range_match(pdinfo, addinfo, &offset)) {
-	/* we'll allow undated data to be merged with x-sectional or
-	   dated, sideways, provided the number of observations
+    /* below: had additional condition: simple_structure(pdinfo)
+       relaxed this on 2009-05-15 */
+
+    if (simple_range_match(pdinfo, addinfo, &offset)) {
+	/* we'll allow undated data to be merged with the existing
+	   dateset, sideways, provided the number of observations
 	   matches OK */
 	addsimple = 1;
     } else if (dataset_is_panel(pdinfo) && 
@@ -2862,6 +2883,8 @@ int import_other (const char *fname, int ftype,
 	importer = get_plugin_function("wf1_get_data", &handle);
     } else if (ftype == GRETL_DTA) {
 	importer = get_plugin_function("dta_get_data", &handle);
+    } else if (ftype == GRETL_SAV) {
+	importer = get_plugin_function("sav_get_data", &handle);
     } else if (ftype == GRETL_JMULTI) {
 	importer = get_plugin_function("jmulti_get_data", &handle);
     } else {
@@ -3050,6 +3073,8 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths)
 	return GRETL_WF1;
     if (has_suffix(fname, ".dta"))
 	return GRETL_DTA;
+    if (has_suffix(fname, ".sav"))
+	return GRETL_SAV;
     if (has_suffix(fname, ".bin"))
 	return GRETL_NATIVE_DB;
     if (has_suffix(fname, ".rat"))
