@@ -669,8 +669,7 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
     return pdinfo;
 }
 
-DATAINFO *
-create_auxiliary_dataset (double ***pZ, int nvar, int nobs)
+DATAINFO *create_auxiliary_dataset (double ***pZ, int nvar, int nobs)
 {
     DATAINFO *pdinfo;
 
@@ -1422,7 +1421,13 @@ int dataset_copy_variable_as (int v, const char *newname,
 	}
 	strcpy(pdinfo->varname[vnew], newname);
 	STACK_LEVEL(pdinfo, vnew) += 1;
-	/* FIXME other varinfo stuff?? */
+	/* FIXME other varinfo stuff? */
+#if 0
+	fprintf(stderr, "copied var %d ('%s', level %d) as var %d ('%s', level %d): ",
+		v, pdinfo->varname[v], STACK_LEVEL(pdinfo, v),
+		vnew, newname, STACK_LEVEL(pdinfo, vnew));
+	fprintf(stderr, "Z[%d][0] = %g\n", vnew, (*pZ)[vnew][0]);
+#endif
     }
 
     return err;
@@ -1503,23 +1508,45 @@ int overwrite_err (const char *name)
 }
 
 /**
- * rename_var_by_id:
- * @idstr: string representation of the ID number of the
- * variable to be renamed.
- * @vname: new name to give the variable.
+ * series_is_parent:
  * @pdinfo: dataset information.
+ * @v: ID number of variable to test.
  * 
- * Returns: 0 on sucess, %E_DATA on error.
+ * Returns: 1 if variable @v is "parent" to a transformed
+ * variable (e.g. a log, lag or difference), othewise 0.
  */
 
-int dataset_rename_variable (DATAINFO *pdinfo, int v, 
-			     const char *name)
+int series_is_parent (const DATAINFO *pdinfo, int v)
+{
+    const char *s = pdinfo->varname[v];
+    int i;
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (i != v && !strcmp(s, pdinfo->varinfo[i]->parent)) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+/**
+ * dataset_rename_series:
+ * @pdinfo: dataset information.
+ * @v: ID number of the variable to be renamed.
+ * @name: new name to give the variable.
+ * 
+ * Returns: 0 on sucess, non-zero on error.
+ */
+
+int dataset_rename_series (DATAINFO *pdinfo, int v, const char *name)
 {
     if (v < 0 || v >= pdinfo->v) {
 	return E_DATA;
     }
 
-    if (object_is_const(pdinfo->varname[v])) {
+    if (object_is_const(pdinfo->varname[v]) ||
+	series_is_parent(pdinfo, v)) {
 	return overwrite_err(pdinfo->varname[v]);
     }
 
@@ -2002,8 +2029,15 @@ int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 
 #if FULLDEBUG
     for (i=0; i<pdinfo->v; i++) {
-	fprintf(stderr, "var %d (%s, level %d) %s\n", i, pdinfo->varname[i],
-		STACK_LEVEL(pdinfo, i), (i >= newv)? "deleting" : "");
+	if ((*pZ)[i] == NULL) {
+	    fprintf(stderr, "var %d (%s, level %d, val = NULL) %s\n", 
+		    i, pdinfo->varname[i], STACK_LEVEL(pdinfo, i), 
+		    (i >= newv)? "deleting" : "");
+	} else {
+	    fprintf(stderr, "var %d (%s, level %d, val[0] = %g) %s\n", 
+		    i, pdinfo->varname[i], STACK_LEVEL(pdinfo, i), 
+		    (*pZ)[i][0], (i >= newv)? "deleting" : "");
+	}
     }
 #endif
 
@@ -2743,7 +2777,7 @@ int dataset_resample (int n, unsigned int seed,
     double **RZ = NULL;
     DATAINFO *rinfo = NULL;
     char **S = NULL;
-    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int T = sample_size(pdinfo);
     int v = pdinfo->v;
     int i, j, s, t;
     int err = 0;
@@ -2849,6 +2883,10 @@ int modify_dataset (int op, const int *list, const char *s,
     static int resampled;
     int k = 0, err = 0;
 
+    if (pZ == NULL || *pZ == NULL || pdinfo == NULL) {
+	return E_NODATA;
+    }
+
     if (op == DS_CLEAR) {
 	/* must be handled by the calling program */
 	return E_NOTIMP;
