@@ -22,6 +22,9 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "missing_private.h"
+#include "gretl_bfgs.h"
+
+#include <errno.h>
 
 #define TINY 1.0e-13
 
@@ -65,17 +68,16 @@ static double logit_pdf (double x)
     return l;
 }
 
-static void Lr_chisq (MODEL *pmod, double **Z)
+static void Lr_chisq (MODEL *pmod, const double **Z)
 {
     int t, zeros, ones = 0, m = pmod->nobs;
     double Lr, chisq;
     
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (model_missing(pmod, t)) {
-	    continue;
-	}
-	if (Z[pmod->list[1]][t] == 1.0) {
-	    ones++;
+	if (!model_missing(pmod, t)) {
+	    if (Z[pmod->list[1]][t] == 1.0) {
+		ones++;
+	    }
 	} 
     }
 
@@ -85,11 +87,15 @@ static void Lr_chisq (MODEL *pmod, double **Z)
     Lr += (double) zeros * log((double) zeros /(double) m);
 
     chisq = 2.0 * (pmod->lnL - Lr);
-    gretl_model_set_double(pmod, "chisq", chisq);
-    
-    /* McFadden pseudo-R^2 */
-    pmod->rsq = 1.0 - pmod->lnL / Lr;
-    pmod->adjrsq = NADBL;
+
+    if (chisq < 0) {
+	pmod->rsq = pmod->adjrsq = pmod->chisq = NADBL;
+    } else {
+	pmod->chisq = chisq;
+	/* McFadden pseudo-R^2 */
+	pmod->rsq = 1.0 - pmod->lnL / Lr;
+	pmod->adjrsq = 1.0 - (pmod->lnL - pmod->ncoeff) / Lr;
+    }
 }
 
 static double 
@@ -113,35 +119,6 @@ logit_probit_llhood (const double *y, const MODEL *pmod, int opt)
     return lnL;
 }
 
-static int add_slopes_to_model (MODEL *pmod, double fbx)
-{
-    double *slopes;
-    size_t ssize = pmod->ncoeff * sizeof *slopes;
-    int i;
-
-    slopes = malloc(ssize);
-
-    if (slopes == NULL) {
-	return 1;
-    }
-
-    for (i=0; i<pmod->ncoeff; i++) {
-	if (pmod->list[i+2] == 0) {
-	    continue;
-	}
-	slopes[i] = pmod->coeff[i] * fbx;
-    }
-
-    if (gretl_model_set_data(pmod, "slopes", slopes, 
-			     GRETL_TYPE_DOUBLE_ARRAY,
-			     ssize)) {
-	free(slopes);
-	return 1;
-    }
-
-    return 0;
-}
-
 static double *hess_wts (MODEL *pmod, const double **Z, int ci) 
 {
     int t, tw, n = pmod->t2 - pmod->t1 + 1;
@@ -288,25 +265,13 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     gretl_matrix_qform(H, GRETL_MOD_NONE, S,
 		       V, GRETL_MOD_NONE);
 
-    pmod->vcv = malloc((k * (k + 1) / 2) * sizeof *pmod->vcv);
-    if (pmod->vcv == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
+    err = gretl_model_write_vcv(pmod, V);
 
-    /* set VCV and std err values */
-    for (i=0; i<k; i++) {
-	for (j=0; j<=i; j++) {
-	    x = gretl_matrix_get(V, i, j);
-	    pmod->vcv[ijton(i, j, k)] = x;
-	    if (i == j) {
-		pmod->sderr[i] = sqrt(x);
-	    }
-	}
+    if (!err) {
+	gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
+	pmod->opt |= OPT_R;
     }
 
-    gretl_model_set_int(pmod, "ml_vcv", VCV_QML);
-
  bailout:
 
     gretl_matrix_free(G);
@@ -564,8 +529,10 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
     return err;
 }
 
-static char *classifier_check (int *list, double **Z, DATAINFO *pdinfo,
-			       gretlopt opt, PRN *prn, int *err)
+static char *classifier_check (int *list, const double **Z, 
+			       const DATAINFO *pdinfo,
+			       gretlopt opt, PRN *prn, 
+			       int *err)
 {
     char *mask = NULL;
     const double *y = Z[list[1]];
@@ -638,29 +605,143 @@ static char *classifier_check (int *list, double **Z, DATAINFO *pdinfo,
     return mask;
 }
 
-static MODEL 
-binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo, 
-		     int ci, gretlopt opt, PRN *prn)
+/* construct an array holding the means of the independent variables
+   in the binary model */
+
+static double *model_get_x_means (MODEL *pmod, const double **Z)
 {
-    int *list = NULL;
-    char *mask = NULL;
-    int i, t, depvar, nx;
-    int oldt1 = pdinfo->t1;
-    int oldt2 = pdinfo->t2;
-    int *act_pred = NULL;
-    MODEL dmod;
-    double xx, fbx, f, F;
-    double *xbar = NULL;
-    double *beta = NULL;
+    double *xbar;
+    int i, vi, t;
 
-    gretl_model_init(&dmod);
-
-    list = gretl_list_copy(inlist);
-    if (list == NULL) {
-	dmod.errcode = E_ALLOC;
-	goto bailout;
+    xbar = malloc(pmod->ncoeff * sizeof *xbar);
+    if (xbar == NULL) {
+	return NULL;
     }
 
+    for (i=0; i<pmod->ncoeff; i++) {
+	vi = pmod->list[i+2];
+	if (vi == 0) {
+	    xbar[i] = 1.0;
+	} else {
+	    xbar[i] = 0.0;
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!model_missing(pmod, t)) {
+		    xbar[i] += Z[vi][t];
+		}
+	    }
+	    xbar[i] /= pmod->nobs;
+	}
+    }
+
+    return xbar;
+}
+
+/* f(Xb) calculated at the means of the independent variables */
+
+static double binary_fXb (MODEL *pmod, const double **Z)
+{
+    double *xbar;
+    double Xb = 0.0;
+    int i;
+
+    xbar = model_get_x_means(pmod, Z);
+    if (xbar == NULL) {
+	return NADBL;
+    }
+
+    for (i=0; i<pmod->ncoeff; i++) {
+	Xb += pmod->coeff[i] * xbar[i];
+    }
+
+    free(xbar);
+
+    return (pmod->ci == LOGIT)? logit_pdf(Xb) : normal_pdf(Xb);
+}
+
+/* Special "slope" calculation for a dummy regressor in binary model:
+   this is F(~Xb + b_j) - F(~Xb) where ~Xb denotes the sum of
+   (coefficient times mean) for all regressors other than the dummy in
+   question and b_j indicates the coefficient on the dummy.  That is,
+   the calculation measures the effect on the probability of Y = 1 of
+   the discrete change 0 to 1 in x_j.
+*/
+
+static double dumslope (MODEL *pmod, const double *xbar, int j)
+{
+    double s, Xb = 0.0;
+    int i;
+
+    for (i=0; i<pmod->ncoeff; i++) {
+	if (i != j) {
+	    Xb += pmod->coeff[i] * xbar[i];
+	}
+    } 
+
+    if (pmod->ci == LOGIT) {
+	s = logit(Xb + pmod->coeff[j]) - logit(Xb);
+    } else {
+	s = normal_cdf(Xb + pmod->coeff[j]) - normal_cdf(Xb);
+    }
+
+    return s;
+}
+
+static int add_slopes_to_model (MODEL *pmod, const double **Z)
+{
+    double *xbar, *slopes;
+    double Xb, fXb;
+    size_t ssize;
+    int i, vi, err = 0;
+
+    xbar = model_get_x_means(pmod, Z);
+    if (xbar == NULL) {
+	return E_ALLOC;
+    }    
+
+    ssize = pmod->ncoeff * sizeof *slopes;
+    slopes = malloc(ssize);
+    if (slopes == NULL) {
+	free(xbar);
+	return E_ALLOC;
+    }
+
+    Xb = 0.0;
+    for (i=0; i<pmod->ncoeff; i++) {
+	Xb += pmod->coeff[i] * xbar[i];
+    }
+
+    fXb = (pmod->ci == LOGIT)? logit_pdf(Xb) : normal_pdf(Xb);
+    pmod->sdy = fXb;
+
+    for (i=0; i<pmod->ncoeff; i++) {
+	vi = pmod->list[i+2];
+	if (vi == 0) {
+	    slopes[i] = 0.0;
+	} else if (gretl_isdummy(pmod->t1, pmod->t2, Z[vi])) {
+	    slopes[i] = dumslope(pmod, xbar, i);
+	} else {
+	    slopes[i] = pmod->coeff[i] * fXb;
+	}
+    }
+
+    err = gretl_model_set_data(pmod, "slopes", slopes, 
+			       GRETL_TYPE_DOUBLE_ARRAY,
+			       ssize);
+
+    free(xbar);
+    if (err) {
+	free(slopes);
+    }
+
+    return err;
+}
+
+static void binary_model_add_stats (MODEL *pmod, const double *y)
+{
+    int *act_pred;
+    double f, F, xx = 0.0;
+    int i, t;
+
     /* space for actual/predicted matrix */
     act_pred = malloc(4 * sizeof *act_pred);
     if (act_pred != NULL) {
@@ -669,10 +750,73 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
-			  (const double **) *pZ);
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	double xb = pmod->yhat[t];
+	int yt;
 
-    mask = classifier_check(list, *pZ, pdinfo, opt, prn,
+	if (model_missing(pmod, t)) {
+	    continue;
+	}
+
+	yt = (int) y[t];
+	xx += yt;
+
+	if (act_pred != NULL) {
+	    i = 2 * yt + (xb > 0.0);
+	    act_pred[i] += 1;
+	}
+
+	if (pmod->ci == LOGIT) {
+	    pmod->yhat[t] = exp(xb) / (1.0 + exp(xb)); 
+	    pmod->uhat[t] = yt - pmod->yhat[t];
+	} else {
+	    f = normal_pdf(xb);
+	    F = normal_cdf(xb);
+	    pmod->yhat[t] = F; 
+	    pmod->uhat[t] = (yt != 0)? f/F : -f/(1-F);
+	}
+    }
+
+    pmod->ybar = xx / pmod->nobs;
+
+    if (act_pred != NULL) {
+	gretl_model_set_data(pmod, "discrete_act_pred", act_pred, 
+			     GRETL_TYPE_INT_ARRAY, 
+			     4 * sizeof *act_pred);
+    }
+
+    mle_criteria(pmod, 0);
+}
+
+static MODEL 
+binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo, 
+		     int ci, gretlopt opt, PRN *prn)
+{
+    int oldt1 = pdinfo->t1;
+    int oldt2 = pdinfo->t2;
+    int *list = NULL;
+    char *mask = NULL;
+    double *beta = NULL;
+    const double **Z;
+    MODEL dmod;
+    int i, depvar, nx;
+
+    /* FIXME do we need to insist on a constant in this sort
+       of model? */
+
+    gretl_model_init(&dmod);
+
+    list = gretl_list_copy(inlist);
+    if (list == NULL) {
+	dmod.errcode = E_ALLOC;
+	goto bailout;
+    }
+
+    Z = (const double **) *pZ;
+
+    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, Z);
+
+    mask = classifier_check(list, Z, pdinfo, opt, prn,
 			    &dmod.errcode);
     if (dmod.errcode) {
 	goto bailout;
@@ -681,14 +825,6 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     depvar = list[1];
     nx = list[0] - 1;
 
-    /* allocate space for means of indep vars, coeffs */
-    xbar = malloc(nx * sizeof *xbar);
-    beta = malloc(nx * sizeof *beta);
-    if (xbar == NULL || beta == NULL) {
-	dmod.errcode = E_ALLOC;
-	goto bailout;
-    }
-
     if (mask != NULL) {
 	dmod.errcode = copy_to_reference_missmask(mask);
     }
@@ -700,6 +836,9 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+    /* re-attach */
+    Z = (const double **) *pZ;
+
     if (dmod.errcode) {
 	goto bailout;
     }
@@ -708,20 +847,15 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     printmodel(&dmod, pdinfo, OPT_NONE, prn);
 #endif
 
-    for (i=0; i<dmod.ncoeff; i++) {
-	beta[i] = dmod.coeff[i];
-	xbar[i] = 0.0;
-	for (t=dmod.t1; t<=dmod.t2; t++) {
-	    if (!model_missing(&dmod, t)) {
-		xbar[i] += (*pZ)[list[i+2]][t];
-	    }
-	}
-	xbar[i] /= dmod.nobs;
+    beta = copyvec(dmod.coeff, dmod.ncoeff);
+
+    if (beta == NULL) {
+	dmod.errcode = E_ALLOC;
+    } else {
+	dmod.errcode = do_BRMR(list, &dmod, ci, beta, Z, pdinfo, 
+			       (opt & OPT_V)? prn : NULL);
     }
 
-    dmod.errcode = do_BRMR(list, &dmod, ci, beta, 
-			   (const double **) *pZ, pdinfo, 
-			   (opt & OPT_V)? prn : NULL);
     if (dmod.errcode) {
 	goto bailout;
     }
@@ -731,88 +865,34 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
 	dmod.coeff[i] = beta[i];
     }
 
-    dmod.lnL = logit_probit_llhood((*pZ)[depvar], &dmod, ci);
-    Lr_chisq(&dmod, *pZ);
+    dmod.lnL = logit_probit_llhood(Z[depvar], &dmod, ci);
+    Lr_chisq(&dmod, Z);
     dmod.ci = ci;
 
     /* calculate standard errors etc */
-    dmod.errcode = logit_probit_vcv(&dmod, opt, (const double **) *pZ);
-    if (dmod.errcode) {
-	goto bailout;
-    }
+    dmod.errcode = logit_probit_vcv(&dmod, opt, Z);
 
-    /* apparatus for calculating slopes at means */
-    xx = 0.0;
-    for (i=0; i<dmod.ncoeff; i++) {
-	xx += dmod.coeff[i] * xbar[i];
-    }
-
-    if (ci == LOGIT) {
-	fbx = logit_pdf(xx);
-#if LPDEBUG
-	fprintf(stderr, "xx = %.8g, fbx = %.8g\n", xx, fbx);
-#endif
-    } else {
-	fbx = normal_pdf(xx);
-    }
-
-    if (opt & OPT_P) {
-	gretl_model_set_int(&dmod, "show-pvals", 1);
-    } else if (add_slopes_to_model(&dmod, fbx)) {
-	dmod.errcode = E_ALLOC;
-	goto bailout;
-    }
-
-    /* calculate additional statistics */
-    xx = 0.0;
-    for (t=dmod.t1; t<=dmod.t2; t++) {
-	double xb = dmod.yhat[t];
-	int yt;
-
-	if (model_missing(&dmod, t)) {
-	    continue;
-	}
-
-	yt = (int) (*pZ)[depvar][t];
-	xx += yt;
-
-	if (act_pred != NULL) {
-	    i = 2 * yt + (xb > 0.0);
-	    act_pred[i] += 1;
-	}
-
-	if (dmod.ci == LOGIT) {
-	    dmod.yhat[t] = exp(xb) / (1.0 + exp(xb)); 
-	    dmod.uhat[t] = yt - dmod.yhat[t];
+    if (!dmod.errcode) {
+	if (opt & OPT_P) {
+	    /* showing p-values, not slopes */
+	    dmod.opt |= OPT_P;
+	    dmod.sdy = binary_fXb(&dmod, Z);
 	} else {
-	    f = normal_pdf(xb);
-	    F = normal_cdf(xb);
-	    dmod.yhat[t] = F; 
-	    dmod.uhat[t] = yt ? f/F : -f/(1-F);
+	    dmod.errcode = add_slopes_to_model(&dmod, Z);
 	}
     }
 
-    xx /= dmod.nobs;
-    dmod.ybar = xx;
-    dmod.sdy = fbx;
-
-    if (act_pred != NULL) {
-	gretl_model_set_data(&dmod, "discrete_act_pred", act_pred, 
-			     GRETL_TYPE_INT_ARRAY, 
-			     4 * sizeof *act_pred);
-    }
-
-    mle_criteria(&dmod, 0);
-
-    if (opt & OPT_A) {
-	dmod.aux = AUX_AUX;
-    } else {
-	dmod.ID = model_count_plus();
+    if (!dmod.errcode) {
+	binary_model_add_stats(&dmod, Z[depvar]);
+	if (opt & OPT_A) {
+	    dmod.aux = AUX_AUX;
+	} else {
+	    dmod.ID = model_count_plus();
+	}
     }
 
  bailout:
 
-    free(xbar);
     free(beta);
     free(list);
     free(mask);
@@ -823,8 +903,696 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     return dmod;
 }
 
+/* struct for holding multinomial logit info */
+
+typedef struct mnl_info_ mnl_info;
+
+struct mnl_info_ {
+    gretl_matrix_block *B;
+    gretl_matrix *y;  /* dependent variable */
+    gretl_matrix *X;  /* regressors */
+    gretl_matrix *b;  /* coefficients, matrix form */
+    gretl_matrix *Xb; /* coeffs times regressors */
+    gretl_matrix *P;  /* probabilities */
+    double *theta;    /* coeffs for BFGS */
+    int n;            /* number of categories (excluding base) */
+    int k;            /* number of coeffs per category */
+    int npar;         /* total number of parameters */
+    int T;            /* number of observations */
+};
+
+static void mnl_info_destroy (mnl_info *mnl)
+{
+    if (mnl != NULL) {
+	gretl_matrix_block_destroy(mnl->B);
+	free(mnl->theta);
+	free(mnl);
+    }
+}
+
+static mnl_info *mnl_info_new (int n, int k, int T)
+{
+    mnl_info *mnl = malloc(sizeof *mnl);
+    int i;
+
+    if (mnl != NULL) {
+	mnl->npar = k * n;
+	mnl->theta = malloc(mnl->npar * sizeof *mnl->theta);
+	if (mnl->theta == NULL) {
+	    free(mnl);
+	    return NULL;
+	}
+	mnl->B = gretl_matrix_block_new(&mnl->y, T, 1,
+					&mnl->X, T, k,
+					&mnl->b, k, n,
+					&mnl->Xb, T, n,
+					&mnl->P, T, n,
+					NULL);
+	if (mnl->B == NULL) {
+	    free(mnl->theta);
+	    free(mnl);
+	    return NULL;
+	} else {
+	    for (i=0; i<mnl->npar; i++) {
+		mnl->theta[i] = 0.0;
+	    }
+	    mnl->n = n;
+	    mnl->k = k;
+	    mnl->T = T;
+	}
+    }
+
+    return mnl;
+}
+
+/* compute loglikelihood for multinomial logit */
+
+static double mn_logit_loglik (const double *theta, void *ptr)
+{
+    mnl_info *mnl = (mnl_info *) ptr;
+    double x, xti, ll = 0.0;
+    int yt, i, t;
+
+    errno = 0;
+
+    for (i=0; i<mnl->npar; i++) {
+	mnl->b->val[i] = theta[i];
+    }
+
+    gretl_matrix_multiply(mnl->X, mnl->b, mnl->Xb);
+
+    for (t=0; t<mnl->T && !errno; t++) {
+	x = 1.0;
+	for (i=0; i<mnl->n; i++) {
+	    /* sum row i of exp(Xb) */
+	    xti = exp(gretl_matrix_get(mnl->Xb, t, i));
+	    x += xti;
+	}
+	ll -= log(x);
+	yt = gretl_vector_get(mnl->y, t);
+	if (yt > 0) {
+	    ll += gretl_matrix_get(mnl->Xb, t, yt-1);
+	}
+    }
+
+    if (errno != 0) {
+	ll = NADBL;
+    }
+
+    return ll;
+}
+
+static int mn_logit_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll, 
+			   void *ptr)
+{
+    mnl_info *mnl = (mnl_info *) ptr;
+    double x, xti, pti, p, g;
+    int i, j, k, t, yt;
+
+    errno = 0;
+
+    for (i=0; i<npar; i++) {
+	s[i] = 0.0;
+    }
+
+    for (t=0; t<mnl->T && !errno; t++) {
+	x = 1.0;
+	for (i=0; i<mnl->n; i++) {
+	    /* sum row i of exp(Xb) */
+	    xti = exp(gretl_matrix_get(mnl->Xb, t, i));
+	    x += xti;
+	    gretl_matrix_set(mnl->P, t, i, xti);
+	}
+
+	if (!errno) {
+	    yt = gretl_vector_get(mnl->y, t);
+	    k = 0;
+	    for (i=0; i<mnl->n; i++) {
+		pti = gretl_matrix_get(mnl->P, t, i) / x;
+		gretl_matrix_set(mnl->P, t, i, pti);
+		p = (i == (yt-1)) - pti;
+		for (j=0; j<mnl->k; j++) {
+		    g = p * gretl_matrix_get(mnl->X, t, j);
+		    s[k++] += g;
+		}
+	    }
+	}
+    }
+
+    return 1;
+}
+
+/* multinomial logit; form the negative inverse of the analytical
+   Hessian */
+
+static gretl_matrix *mnl_hess_inverse (mnl_info *mnl, int *err)
+{
+    gretl_matrix_block *B;
+    gretl_matrix *H;
+    gretl_matrix *x;
+    gretl_matrix *xx;
+    gretl_matrix *hjk;
+    double xti, ptj, ptk;
+    int r, c;
+    int i, j, k, t;
+
+    H = gretl_zero_matrix_new(mnl->npar, mnl->npar);
+    if (H == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    B = gretl_matrix_block_new(&x, 1, mnl->k,
+			       &xx, mnl->k, mnl->k,
+			       &hjk, mnl->k, mnl->k,
+			       NULL);
+    if (B == NULL) {
+	*err = E_ALLOC;
+	gretl_matrix_free(H);
+	return NULL;
+    }
+
+    r = c = 0;
+
+    for (j=0; j<mnl->n; j++) {
+	for (k=0; k<=j; k++) {
+	    gretl_matrix_zero(hjk);
+	    for (t=0; t<mnl->T; t++) {
+		for (i=0; i<mnl->k; i++) {
+		    xti = gretl_matrix_get(mnl->X, t, i);
+		    gretl_vector_set(x, i, xti);
+		}
+		gretl_matrix_multiply_mod(x, GRETL_MOD_TRANSPOSE,
+					  x, GRETL_MOD_NONE,
+					  xx, GRETL_MOD_NONE);
+		ptj = gretl_matrix_get(mnl->P, t, j);
+		ptk = gretl_matrix_get(mnl->P, t, k);
+		gretl_matrix_multiply_by_scalar(xx, ptj * ((j == k) - ptk));
+		gretl_matrix_add_to(hjk, xx);
+	    }
+	    gretl_matrix_inscribe_matrix(H, hjk, r, c, GRETL_MOD_NONE);
+	    if (j != k) {
+		gretl_matrix_inscribe_matrix(H, hjk, c, r, GRETL_MOD_NONE);
+	    }
+	    c += mnl->k;
+	}
+	r += mnl->k;
+	c = 0;
+    }
+
+    gretl_matrix_block_destroy(B);
+
+    *err = gretl_invert_symmetric_matrix(H);
+
+    if (*err) {
+	gretl_matrix_free(H);
+	H = NULL;
+    }
+
+    return H;
+}
+
+static gretl_matrix *mnl_opg_matrix (mnl_info *mnl, int *err)
+{
+    gretl_matrix *G;
+    gretl_matrix *GG;
+    double p, g;
+    int yt, i, j, k, t;
+
+    G = gretl_matrix_alloc(mnl->T, mnl->npar);
+    if (G == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    GG = gretl_matrix_alloc(mnl->npar, mnl->npar);
+    if (GG == NULL) {
+	*err = E_ALLOC;
+	gretl_matrix_free(G);
+	return NULL;
+    }
+
+    for (t=0; t<mnl->T; t++) {
+	yt = gretl_vector_get(mnl->y, t);
+	k = 0;
+	for (i=0; i<mnl->n; i++) {
+	    p = (i == (yt-1)) - gretl_matrix_get(mnl->P, t, i);
+	    for (j=0; j<mnl->k; j++) {
+		g = p * gretl_matrix_get(mnl->X, t, j);
+		gretl_matrix_set(G, t, k++, g);
+	    }
+	}
+    }
+
+    gretl_matrix_multiply_mod(G, GRETL_MOD_TRANSPOSE,
+			      G, GRETL_MOD_NONE,
+			      GG, GRETL_MOD_NONE);
+
+    gretl_matrix_free(G);
+
+    return GG;
+}
+
+static int mnl_add_variance_matrix (MODEL *pmod, mnl_info *mnl,
+				    gretlopt opt)
+{
+    gretl_matrix *H = NULL;
+    gretl_matrix *G = NULL;
+    gretl_matrix *V = NULL;
+    int err = 0;
+
+    H = mnl_hess_inverse(mnl, &err);
+    if (err) {
+	return err;
+    }
+
+    if (opt & OPT_R) {
+	G = mnl_opg_matrix(mnl, &err);
+	if (!err) {
+	    V = gretl_matrix_alloc(mnl->npar, mnl->npar);
+	    if (V == NULL) {
+		err = E_ALLOC;
+	    } else {
+		err = gretl_matrix_qform(H, GRETL_MOD_NONE, G,
+					 V, GRETL_MOD_NONE);
+	    }
+	}
+    }
+
+    if (!err) {
+	if (opt & OPT_R) {
+	    err = gretl_model_write_vcv(pmod, V);
+	    if (!err) {
+		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
+		pmod->opt |= OPT_R;
+	    }
+	} else {
+	    err = gretl_model_write_vcv(pmod, H);
+	    if (!err) {
+		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
+	    }
+	}
+    }
+
+    gretl_matrix_free(H);
+    gretl_matrix_free(G);
+    gretl_matrix_free(V);
+
+    return err;
+}
+
+/* Construct 'yhat' and 'uhat'.  Maybe this is too simple-minded; we
+   just find, for each observation, the y-value for which the
+   probability is maximized and set that as yhat[t].  We then set
+   uhat[t] as a binary "hit" (residual = 0) or "miss" (residual = 1).
+   Constructing a "quantitative" residual as y[t] - yhat[t] seems
+   spurious, since there's no meaningful metric for the "distance"
+   between y and yhat when y is an unordered response.
+*/
+
+static void mn_logit_yhat (MODEL *pmod, mnl_info *mnl,
+			   const int *yvals)
+{
+    double p, pmax;
+    int i, s, t, iymax;
+    int ncorrect = 0;
+
+    s = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (na(pmod->yhat[t])) {
+	    continue;
+	}
+	pmax = 1.0;
+	iymax = 0;
+	for (i=0; i<mnl->n; i++) {
+	    p = gretl_matrix_get(mnl->Xb, s, i);
+	    if (p > pmax) {
+		pmax = p;
+		iymax = i + 1;
+	    }
+	}
+	if (iymax == (int) mnl->y->val[s]) {
+	    ncorrect++;
+	    pmod->uhat[t] = 0;
+	} else {
+	    pmod->uhat[t] = 1;
+	}
+	if (yvals != NULL) {
+	    pmod->yhat[t] = yvals[iymax];
+	} else {
+	    pmod->yhat[t] = iymax;
+	}
+	s++;
+    }
+
+    gretl_model_set_int(pmod, "correct", ncorrect);
+}
+
+struct sorter {
+    double x;
+    int t;
+};
+
+/* In case the dependent variable is not in canonical form for
+   multinomial logit, construct a transformed version */
+
+static int make_canonical_depvar (MODEL *pmod, const double *y, 
+				  gretl_matrix *yvec)
+{
+    struct sorter *s;
+    double nexty, bady;
+    int i, t, n = pmod->nobs;
+
+    s = malloc(n * sizeof *s);
+    if (s == NULL) {
+	return E_ALLOC;
+    }
+
+    i = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    s[i].x = y[t];
+	    s[i].t = i;
+	    i++;
+	}
+    }
+
+    qsort(s, n, sizeof *s, gretl_compare_doubles);
+
+    /* normalize to a minimum of zero */
+    if (s[0].x != 0) {
+	double ymin = s[0].x;
+
+	for (i=0; i<n && s[i].x == ymin; i++) {
+	    s[i].x = 0.0;
+	}
+    }
+
+    /* ensure that the sorted values increase by steps of one */
+    for (i=1; i<n; i++) {
+	if (s[i].x != s[i-1].x) {
+	    nexty = s[i-1].x + 1;
+	    if (s[i].x != nexty) {
+		bady = s[i].x;
+		while (i < n && s[i].x == bady) {
+		    s[i++].x = nexty;
+		}
+		i--; /* compensate for outer i++ */
+	    } 
+	} 
+    }
+
+    /* write canonical version of y into yvec */
+    for (i=0; i<n; i++) {
+	yvec->val[s[i].t] = s[i].x;
+    }
+
+    free(s);
+
+    return 0;
+}
+
+/* multinomial logit: count the distinct values taken on by the
+   dependent variable.  If the variable does not take the form of a
+   sequence of consecutive integers with a base of zero, flag this
+   by returning via @yvals the vector of distinct values. To assist
+   with later computations, we also return a frequency count for
+   the distinct y values in @valcount.
+*/
+
+static int mn_value_count (const double *y, MODEL *pmod, 
+			   int **valcount, int **yvals)
+{
+    double *sy;
+    int *vc = NULL;
+    int *v = NULL;
+    int want_yvals = 0;
+    int s, t, n;
+
+    sy = malloc(pmod->nobs * sizeof *sy);
+    if (sy == NULL) {
+	pmod->errcode = E_ALLOC;
+	return 0;
+    }
+
+    s = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    sy[s++] = y[t];
+	}
+    }
+
+    qsort(sy, pmod->nobs, sizeof *sy, gretl_compare_doubles);
+
+    n = count_distinct_values(sy, pmod->nobs);
+
+    vc = malloc(n * sizeof *vc);
+    v = malloc(n * sizeof *v);
+    
+    if (vc == NULL || v == NULL) {
+	pmod->errcode = E_ALLOC;
+	free(sy);
+	free(vc);
+	return 0;
+    }
+
+    for (s=0; s<n; s++) {
+	vc[s] = 0;
+    }
+
+    if (sy[0] != 0.0) {
+	/* we'll need v later */
+	want_yvals = 1;
+    }
+
+    v[0] = (int) sy[0];
+    s = 0;
+
+    for (t=0; t<pmod->nobs && !pmod->errcode; t++) {
+	if (sy[t] != floor(sy[t])) {
+	    pmod->errcode = E_DATA;
+	    gretl_errmsg_set("logit: the dependent variable must form a sequence of "
+			     "integers");
+	} else if (t > 0 && sy[t] != sy[t-1]) {
+	    v[++s] = (int) sy[t];
+	    if (sy[t] != sy[t-1] + 1) {
+		/* not consecutive: need v later */
+		want_yvals = 1;
+	    }
+	    vc[s] = 1;
+	} else {
+	    vc[s] += 1;
+	}
+    }
+
+    if (pmod->errcode) {
+	free(vc);
+	n = 0;
+    } else {
+	*valcount = vc;
+	if (want_yvals) {
+	    *yvals = v;
+	    v = NULL;
+	}
+    }
+
+    free(v);
+    free(sy);
+
+    return n;
+}
+
+const char *mn_logit_coeffsep (const MODEL *pmod, const DATAINFO *pdinfo, int i)
+{
+    static char ret[32];
+    const char *vname = gretl_model_get_depvar_name(pmod, pdinfo);
+    const int *list = (const int *) gretl_model_get_data(pmod, "yvals");
+    int val = (list != NULL)? list[i+1] : i;
+
+    sprintf(ret, "%s = %d", vname, val);
+    return ret;
+}
+
+/* transcribe multinomial logit results into @pmod, which was
+   initialized via OLS, and add covariance matrix
+*/
+
+static void mnl_finish (mnl_info *mnl, MODEL *pmod,
+			const int *valcount,
+			const int *yvals,
+			const DATAINFO *pdinfo,
+			gretlopt opt)
+{
+    int i;
+
+    pmod->errcode = gretl_model_write_coeffs(pmod, mnl->theta, mnl->npar);
+
+    if (!pmod->errcode) {
+	pmod->errcode = mnl_add_variance_matrix(pmod, mnl, opt);
+    }
+
+    if (!pmod->errcode) {
+	pmod->ci = LOGIT;
+	pmod->lnL = mn_logit_loglik(mnl->theta, mnl);
+	mle_criteria(pmod, 0);
+
+	gretl_model_set_int(pmod, "multinom", mnl->n);
+	gretl_model_set_int(pmod, "cblock", mnl->k);
+
+	pmod->ess = pmod->sigma = NADBL;
+	pmod->fstt = pmod->rsq = pmod->adjrsq = NADBL;
+
+	gretl_model_allocate_params(pmod, mnl->npar);
+    }
+
+    if (!pmod->errcode) {
+	int j, vj, k = 0;
+
+	for (i=0; i<mnl->n; i++) {
+	    for (j=0; j<mnl->k; j++) {
+		vj = pmod->list[j+2];
+		strcpy(pmod->params[k++], pdinfo->varname[vj]);
+	    }
+	}
+	mn_logit_yhat(pmod, mnl, yvals);
+	set_model_id(pmod);
+    }
+
+    if (!pmod->errcode && yvals != NULL) {
+	int *list = gretl_list_new(mnl->n + 1);
+
+	if (list != NULL) {
+	    for (i=0; i<=mnl->n; i++) {
+		list[i+1] = yvals[i];
+	    }
+	    gretl_model_set_list_as_data(pmod, "yvals", list);
+	}
+    }
+
+    if (!pmod->errcode) {
+	/* add overall likelihood ratio test */
+	int ni, df = pmod->ncoeff;
+	double L0 = 0.0;
+
+	if (pmod->ifc) {
+	    df -= mnl->n;
+	}
+
+	for (i=0; i<=mnl->n; i++) {
+	    ni = valcount[i];
+	    if (pmod->ifc) {
+		L0 += ni * log((double) ni / mnl->T);
+	    } else {
+		L0 += ni * log(1.0 / (mnl->n + 1));
+	    }
+	}
+
+	pmod->chisq = 2.0 * (pmod->lnL - L0);
+	pmod->dfn = df;
+    }
+
+    if (pmod->xpx != NULL) {
+	free(pmod->xpx);
+	pmod->xpx = NULL;
+    }
+
+    pmod->opt |= OPT_M;
+}
+
+/* multinomial logit */
+
+static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn)
+{
+    int maxit = 1000;
+    int fncount = 0;
+    int grcount = 0;
+    MODEL mod;
+    mnl_info *mnl;
+    int *valcount = NULL;
+    int *yvals = NULL;
+    int n, k = list[0] - 1;
+    int i, vi, t, s;
+
+    /* we'll start with OLS to flush out data issues */
+    mod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+    if (mod.errcode) {
+	return mod;
+    }
+
+    n = mn_value_count((*pZ)[list[1]], &mod, &valcount, &yvals);
+    if (mod.errcode) {
+	return mod;
+    }
+
+    n--; /* exclude the first value */
+
+    if (n * k > mod.nobs) {
+	mod.errcode = E_DF;
+	return mod;
+    }
+
+    mnl = mnl_info_new(n, k, mod.nobs);
+    if (mnl == NULL) {
+	mod.errcode = E_ALLOC;
+	return mod;
+    }
+
+    if (yvals != NULL) {
+	/* the dependent variable needs transforming */
+	mod.errcode = make_canonical_depvar(&mod, (*pZ)[list[1]], mnl->y);
+	if (mod.errcode) {
+	    goto bailout;
+	}
+    } else {
+	s = 0;
+	for (t=mod.t1; t<=mod.t2; t++) {
+	    if (!na(mod.yhat[t])) {
+		mnl->y->val[s++] = (*pZ)[list[1]][t];
+	    }
+	}
+    }
+
+    for (i=0; i<k; i++) {
+	vi = list[i+2];
+	s = 0;
+	for (t=mod.t1; t<=mod.t2; t++) {
+	    if (!na(mod.yhat[t])) {
+		gretl_matrix_set(mnl->X, s++, i, (*pZ)[vi][t]);
+	    }
+	}
+    } 
+
+    if (mod.ifc) {
+	double lf0 = log(valcount[0]);
+	int j = 0;
+
+	for (i=1; i<=mnl->n; i++) {
+	    mnl->theta[j] = log(valcount[i]) - lf0;
+	    j += mnl->k;
+	}
+    }
+
+    mod.errcode = BFGS_max(mnl->theta, mnl->npar, maxit, 0.0, 
+			   &fncount, &grcount, mn_logit_loglik, C_LOGLIK,
+			   mn_logit_score, mnl, 
+			   (prn != NULL)? OPT_V : OPT_NONE, prn);
+
+    if (!mod.errcode) {
+	mnl_finish(mnl, &mod, valcount, yvals, pdinfo, opt);
+    }  
+
+ bailout:  
+
+    mnl_info_destroy(mnl);
+    free(valcount);
+    free(yvals);
+
+    return mod;
+}
+
 static int 
-ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
+discrete_depvar_ok (double **Z, const DATAINFO *pdinfo, int v)
 {
     if (!var_is_discrete(pdinfo, v) && 
 	!gretl_is_oprobit_ok(pdinfo->t1, pdinfo->t2, Z[v])) {
@@ -863,14 +1631,17 @@ ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
 MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, 
 		    int ci, gretlopt opt, PRN *prn)
 {
+    PRN *vprn = NULL;
     int yv = list[1];
 
-    if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[yv])) {
+    if (ci == LOGIT && (opt & OPT_M)) {
+	vprn = (opt & OPT_V)? prn : NULL;
+	return mnl_model(list, pZ, pdinfo, opt, vprn);
+    } else if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[yv])) {
 	return binary_logit_probit(list, pZ, pdinfo, ci, opt, prn);
-    } else if (ordered_model_ok(*pZ, pdinfo, yv)) {
-	PRN *oprn = (opt & OPT_V)? prn : NULL;
-
-	return ordered_model(list, ci, pZ, pdinfo, opt, oprn);
+    } else if (discrete_depvar_ok(*pZ, pdinfo, yv)) {
+	vprn = (opt & OPT_V)? prn : NULL;
+	return ordered_model(list, ci, pZ, pdinfo, opt, vprn);
     } else {
 	MODEL dmod;
 
@@ -982,10 +1753,6 @@ static int cholesky_decomp (double *xpx, int nv)
     return 0; 
 }
 
-/* logistic model: doesn't exactly belong here but it seems like
-   the most suitable place for it 
-*/
-
 int logistic_ymax_lmax (const double *y, const DATAINFO *pdinfo,
 			double *ymax, double *lmax)
 {
@@ -1012,6 +1779,7 @@ int logistic_ymax_lmax (const double *y, const DATAINFO *pdinfo,
     } else if (*ymax < 100.0) {
 	*lmax = 100.0;
     } else {
+	/* admittedly arbitrary */
 	*lmax = 1.1 * *ymax;
     }
 	    
@@ -1067,8 +1835,8 @@ static int make_logistic_depvar (double ***pZ, DATAINFO *pdinfo,
 static int rewrite_logistic_stats (const double **Z, const DATAINFO *pdinfo,
 				   MODEL *pmod, int dv, double lmax)
 {
+    double x, ess, sigma;
     int t;
-    double x;
 
     pmod->ybar = gretl_mean(pmod->t1, pmod->t2, Z[dv]);
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, Z[dv]);
@@ -1076,7 +1844,8 @@ static int rewrite_logistic_stats (const double **Z, const DATAINFO *pdinfo,
     /* make the VCV matrix before messing with the model stats */
     makevcv(pmod, pmod->sigma);
 
-    pmod->ess = 0.0;
+    ess = 0.0;
+
     for (t=0; t<pdinfo->n; t++) {
 	x = pmod->yhat[t];
 	if (na(x)) {
@@ -1084,34 +1853,15 @@ static int rewrite_logistic_stats (const double **Z, const DATAINFO *pdinfo,
 	}
 	pmod->yhat[t] = lmax / (1.0 + exp(-x));
 	pmod->uhat[t] = Z[dv][t] - pmod->yhat[t];
-	pmod->ess += pmod->uhat[t] * pmod->uhat[t];
+	ess += pmod->uhat[t] * pmod->uhat[t];
     }
 
-    pmod->sigma = sqrt(pmod->ess / pmod->dfd);
-
-    pmod->tss = 0.0;
-    for (t=pmod->t1; t<=pmod->t2; t++) {
-	x = Z[dv][t];
-	if (!na(x)) {
-	    pmod->tss += (x - pmod->ybar) * (x - pmod->ybar);
-	}
-    }
-
-    pmod->fstt = pmod->dfd * (pmod->tss - pmod->ess) / (pmod->dfn * pmod->ess);
-
-    pmod->rsq = pmod->adjrsq = NADBL;
-
-    if (pmod->tss > 0) {
-	pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
-	if (pmod->dfd > 0) {
-	    double den = pmod->tss * pmod->dfd;
-
-	    pmod->adjrsq = 1.0 - (pmod->ess * (pmod->nobs - 1) / den);
-	}
-    }
+    sigma = sqrt(ess / pmod->dfd);
 
     pmod->list[1] = dv;
     gretl_model_set_double(pmod, "lmax", lmax);
+    gretl_model_set_double(pmod, "ess_orig", ess);
+    gretl_model_set_double(pmod, "sigma_orig", sigma);
     pmod->ci = LOGISTIC;
     ls_criteria(pmod);
 
@@ -1269,3 +2019,45 @@ int fishers_exact_test (const Xtab *tab, PRN *prn)
 
     return 0;
 }
+
+/**
+ * intreg:
+ * @list: high/low (2 variables) plus list of regressors.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @opt: if includes %OPT_R form robust (QML) estimates of standard
+ * errors and covariance matrix; if includes %OPT_V give verbose
+ * operation.
+ * @prn: printing struct in case additional information is
+ * wanted (signalled via %OPT_V).
+ *
+ * Returns: a #MODEL struct, containing interval estimates of the
+ * model specified by @list.
+ */
+
+MODEL intreg (int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt, 
+	      PRN *prn)
+{
+    MODEL intmod;
+    void *handle;
+    MODEL (* interval_estimate) (int *, double ***, DATAINFO *, gretlopt, 
+				 PRN *);
+
+    gretl_error_clear();
+
+    interval_estimate = get_plugin_function("interval_estimate", &handle);
+    if (interval_estimate == NULL) {
+	gretl_model_init(&intmod);
+	intmod.errcode = E_FOPEN;
+	return intmod;
+    }
+
+    intmod = (*interval_estimate) (list, pZ, pdinfo, opt, prn);
+
+    close_plugin(handle);
+
+    set_model_id(&intmod);
+
+    return intmod;
+}
+
