@@ -20,7 +20,7 @@
 #ifndef DISCRETE_H
 #define DISCRETE_H
 
-MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, 
 		    int ci, gretlopt opt, PRN *prn);
 
 MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
@@ -31,6 +31,12 @@ int logistic_ymax_lmax (const double *y, const DATAINFO *pdinfo,
 
 int fishers_exact_test (const Xtab *tab, PRN *prn);
 
+MODEL intreg (int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt, 
+	      PRN *prn);
+
+const char *mn_logit_coeffsep (const MODEL *pmod, const DATAINFO *pdinfo, 
+			       int i);
+
 #endif /* DISCRETE_H */
 
 
