@@ -58,6 +58,9 @@ MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo);
 MODEL panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 		   gretlopt opt, PRN *prn);
 
+MODEL ivreg (const int *list, double ***pZ, DATAINFO *pdinfo,
+	     gretlopt opt);
+
 MODEL arbond_model (const int *list, const char *istr, const double **Z, 
 		    const DATAINFO *pdinfo, gretlopt opt, 
 		    PRN *prn);
@@ -90,9 +93,7 @@ int makevcv (MODEL *pmod, double sigma);
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo);
 
-int gretl_XTX_XTy (const int *list, int t1, int t2, 
-		   const double **Z, int nwt, double rho, int pwe,
-		   double *xpx, double *xpy, const char *mask);
+double *gretl_XTX (const MODEL *pmod, const double **Z, int *err);
 
 #endif /* ESTIMATE_H */
 
