@@ -19,9 +19,11 @@
 
 #include "libgretl.h"
 #include "matrix_extra.h"
+#include "gretl_scalar.h"
 #include "forecast.h"
 #include "var.h"
 #include "system.h"
+#include "libset.h"
 
 #define ARF_DEBUG 0
 
@@ -140,16 +142,18 @@ static FITRESID *fit_resid_new_with_length (int n, int add_errs)
     f->method = 0;
     f->model_ID = 0;
     f->asymp = 0;
+    f->std = 0;
     f->model_t1 = 0;
     f->t0 = 0;
     f->t1 = 0;
     f->t2 = 0;
     f->df = 0;
+    f->k = 0;
     f->nobs = 0;
     f->pmax = PMAX_NOT_AVAILABLE;
 
     f->sigma = NADBL;
-    f->tval = NADBL;
+    f->alpha = 0.05;
 
     f->actual = NULL;
     f->fitted = NULL;
@@ -287,7 +291,15 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
 			 const DATAINFO *pdinfo, int *err)
 {
     FITRESID *fr;
-    int dv, t;
+    int t, dv = -1;
+
+    if (pmod->ci != INTREG) {
+	dv = gretl_model_get_depvar(pmod);
+	if (dv < 0 || dv >= pdinfo->v) {
+	    *err = E_DATA;
+	    return NULL;
+	}
+    }
 
     fr = fit_resid_new_for_model(pmod, pdinfo, pmod->t1, pmod->t2,
 				 0, err);
@@ -297,6 +309,9 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
 
     if (LIMDEP(pmod->ci)) {
 	fr->sigma = NADBL;
+    } else if (pmod->ci == GARCH && (pmod->opt & OPT_U)) {
+	fr->sigma = 1.0;
+	fr->std = 1;
     } else {
 	fr->sigma = gretl_model_get_double(pmod, "sigma_orig");
 	if (na(fr->sigma)) {
@@ -304,23 +319,34 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
 	}
     }
 
-    dv = gretl_model_get_depvar(pmod);
-
     for (t=0; t<fr->nobs; t++) {
-	fr->actual[t] = Z[dv][t];
+	if (dv < 0) {
+	    if (na(pmod->yhat[t]) || na(pmod->uhat[t])) {
+		fr->actual[t] = NADBL;
+	    } else {
+		fr->actual[t] = pmod->yhat[t] + pmod->uhat[t];
+	    }
+	} else {
+	    fr->actual[t] = Z[dv][t];
+	}
 	fr->fitted[t] = pmod->yhat[t];
 	fr->resid[t] = pmod->uhat[t];
     }
 
     fit_resid_set_dec_places(fr);
-    strcpy(fr->depvar, pdinfo->varname[dv]);
+    
+    if (dv < 0) {
+	strcpy(fr->depvar, "implicit y");
+    } else {
+	strcpy(fr->depvar, pdinfo->varname[dv]);
+    }
     
     return fr;
 }
 
 /* local shortcut to get a model's list of regressors */
 
-static int *model_xlist (MODEL *pmod)
+static const int *model_xlist (MODEL *pmod)
 {
     int *xlist = (int *) gretl_model_get_data(pmod, "xlist");
 
@@ -338,11 +364,10 @@ static int *model_xlist (MODEL *pmod)
    variable among the regressors: return 1 if so, 0 if not
 */
 
-static int 
-has_depvar_lags (MODEL *pmod, const DATAINFO *pdinfo)
+static int has_depvar_lags (MODEL *pmod, const DATAINFO *pdinfo)
 {
     const char *yname;
-    int *xlist;
+    const int *xlist;
     int i, vi;
 
     xlist = model_xlist(pmod);
@@ -377,7 +402,7 @@ static int process_lagged_depvar (MODEL *pmod,
 				  const DATAINFO *pdinfo,
 				  int **depvar_lags)
 {
-    int *xlist = NULL;
+    const int *xlist = NULL;
     int *dvlags = NULL;
     int anylags;
     int err = 0;
@@ -428,7 +453,7 @@ static int
 has_real_exog_regressors (MODEL *pmod, const int *dvlags,
 			  const double **Z, const DATAINFO *pdinfo)
 {
-    int *xlist = model_xlist(pmod);
+    const int *xlist = model_xlist(pmod);
     int i, xi, ret = 0;
 
     if (xlist != NULL) {
@@ -562,9 +587,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	}
 
 	/* forecast value */
-	fc->yhat[t] = gretl_matrix_dot_product(Xs, GRETL_MOD_NONE,
-					       b, GRETL_MOD_NONE,
-					       NULL);
+	fc->yhat[t] = gretl_vector_dot_product(Xs, b, NULL);
 
 	/* forecast variance */
 	vyh = gretl_scalar_qform(Xs, V, &err);
@@ -590,10 +613,13 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
     return err;
 }
 
-/* Generate forecasts from nonlinear least squares model, using
-   the string specification of the regression function that
-   was saved as data on the model (see nls.c).  For now we don't
-   attempt to calculate forecast error variance.
+/* Generate forecasts from nonlinear least squares model, using the
+   string specification of the regression function that was saved as
+   data on the model (see nls.c).  If the NLS formula is dynamic and
+   the user has not requested a static forecast, we do an
+   autoregressive genr out of sample.  If we're doing a static
+   forecast we add a simple-minded forecast error, namely the standard
+   error of the NLS regression.
 */
 
 static int nls_fcast (Forecast *fc, const MODEL *pmod, 
@@ -601,34 +627,93 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
 {
     int oldt1 = pdinfo->t1;
     int oldt2 = pdinfo->t2;
-    const char *nlfunc;    
+    int fcv = pdinfo->v;
+    int yno = 0;
+    const char *nlfunc;   
+    double *y = NULL;
     char formula[MAXLINE];
-    int t, oldv = pdinfo->v;
-    int err = 0;
+    int t, err = 0;
 
     nlfunc = gretl_model_get_data(pmod, "nl_regfunc");
     if (nlfunc == NULL) {
 	err = E_DATA;
     }
 
-    if (!err) {
-	pdinfo->t1 = fc->t1;
-	pdinfo->t2 = fc->t2;
-	sprintf(formula, "$nl_y = %s", nlfunc);
-	err = generate(formula, pZ, pdinfo, OPT_P, NULL);
+    if (fc->method == FC_AUTO) {
+	yno = pmod->list[1];
+	y = copyvec((*pZ)[yno], pdinfo->n);
+	if (y == NULL) {
+	    err = E_ALLOC;
+	} 
     }
 
     if (!err) {
-	/* transcribe values from last generated var to target */
-	for (t=fc->t1; t<=fc->t2; t++) {
-	    fc->yhat[t] = (*pZ)[oldv][t];
+	/* upper limit of static forecast */
+	int t2 = (fc->method == FC_STATIC)? fc->t2 : pmod->t2;
+
+	if (t2 >= fc->t1) {
+	    /* non-null static range */
+	    pdinfo->t1 = fc->t1;
+	    pdinfo->t2 = t2;
+	    sprintf(formula, "$nl_y = %s", nlfunc);
+	    err = generate(formula, pZ, pdinfo, OPT_P, NULL);
+	    if (!err) {
+		for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		    fc->yhat[t] = (*pZ)[fcv][t];
+		}
+	    }
 	}
-	err = dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
+
+	if (!err && fc->method == FC_AUTO && fc->t2 > pmod->t2) {
+	    /* dynamic forecast out of sample */
+	    pdinfo->t1 = pmod->t2 + 1;
+	    pdinfo->t2 = fc->t2;
+	    strcpy(formula, pmod->depvar);
+	    err = generate(formula, pZ, pdinfo, OPT_P, NULL);
+	    if (!err) {
+		for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		    fc->yhat[t] = (*pZ)[yno][t];
+		}
+	    }
+	}
+    }
+
+    if (pdinfo->v > fcv) {
+	err = dataset_drop_last_variables(pdinfo->v - fcv, pZ, pdinfo);
     }
 
     pdinfo->t1 = oldt1;
     pdinfo->t2 = oldt2;
 
+    if (y != NULL) {
+	/* restore original dependent variable */
+	for (t=0; t<pdinfo->n; t++) {
+	    (*pZ)[yno][t] = y[t];
+	}
+	free(y);
+    }
+
+    if (!err && fc->method == FC_STATIC && fc->sderr != NULL) {
+#if 1   /* not fully tested! */
+	err = nls_boot_calc(pmod, pZ, pdinfo, fc->t1, fc->t2, fc->sderr);
+	if (!err) {
+	    double et, s2 = pmod->sigma * pmod->sigma;
+
+	    for (t=fc->t1; t<=fc->t2; t++) {
+		if (!na(fc->sderr[t])) {
+		    et = fc->sderr[t];
+		    fc->sderr[t] = sqrt(et * et + s2);
+		}
+	    }
+	}
+#else
+	/* kinda simple-minded */
+	for (t=fc->t1; t<=fc->t2; t++) {
+	    fc->sderr[t] = pmod->sigma;
+	}
+#endif
+    }
+
     return err;
 }
 
@@ -878,7 +963,7 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
 {
     double xval;
     int xvars, yno;
-    int *xlist = NULL;
+    const int *xlist = NULL;
     double *h = NULL;
     double *mh = NULL;
     double *phi = NULL;
@@ -1011,7 +1096,7 @@ static double arma_variance (const double *phi, int p,
 */
 
 static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
-				 const double *beta, int *xlist, 
+				 const double *beta, const int *xlist, 
 				 const double **Z)
 {
     double *Xb;
@@ -1080,7 +1165,7 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     double xval, yval, vl;
     double mu = NADBL;
     int xvars, yno;
-    int *xlist = NULL;
+    const int *xlist = NULL;
     int p, q, px = 0, npsi = 0;
     int fcstart = fc->t1;
     int ar_smax, ma_smax;
@@ -1456,7 +1541,7 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
 	set_up_ar_fcast_variance(fc, pmod, pmax, npsi, &phi, &psi, &errphi);
     }
 
-    pwe = gretl_model_get_int(pmod, "pwe");
+    pwe = (pmod->opt & OPT_P);
 
     for (t=fc->t1; t<=fc->t2; t++) {
 	miss = 0;
@@ -1593,17 +1678,91 @@ static double fcast_transform (double xb, int ci, int t,
     return yf;
 }
 
-/* compute forecasts for linear models without autoregressive errors,
-   version without computation of forecast standard errors
+static int 
+integrated_fcast (Forecast *fc, const MODEL *pmod, int yno,
+		  const double **Z, const DATAINFO *pdinfo)
+{
+    double s2 = NADBL;
+    int dyn_start; /* start of dynamic forecast */
+    double xval, yht = 0.0;
+    int i, vi, t;
+
+    if (fc->method == FC_STATIC) {
+	dyn_start = pdinfo->n; /* i.e., never */
+    } else if (fc->method == FC_DYNAMIC) {
+	dyn_start = fc->t1;
+    } else {
+	/* FC_AUTO: dynamic out of sample */
+	dyn_start = pmod->t2 + 1;
+    }
+
+    if (fc->sderr != NULL) {
+	s2 = pmod->sigma * pmod->sigma;
+	fc->sderr[dyn_start - 1] = 0.0;
+    }
+
+    for (t=fc->t1; t<=fc->t2; t++) {
+	int miss = 0;
+
+	if (t <= dyn_start) {
+	    yht = Z[yno][t-1];
+	    if (na(yht)) {
+		miss = 1;
+	    }
+	}
+
+	for (i=0; i<pmod->ncoeff && !miss; i++) {
+	    vi = pmod->list[i+2];
+	    xval = Z[vi][t];
+	    if (na(xval)) {
+		miss = 1;
+	    } else {
+		yht += xval * pmod->coeff[i];
+	    }
+	}
+
+	if (miss) {
+	    if (t >= dyn_start) {
+		/* all subsequent values are NA */
+		int s;
+
+		for (s=t; s<=fc->t2; s++) {
+		    fc->yhat[s] = NADBL;
+		}
+		break;
+	    } else {
+		fc->yhat[t] = NADBL;
+	    }
+	} else {
+	    fc->yhat[t] = yht;
+	    if (fc->sderr != NULL && t >= dyn_start) {
+		fc->sderr[t] = fc->sderr[t-1] + s2;
+	    }
+	}
+    }
+
+    if (fc->sderr != NULL) {
+	fc->sderr[dyn_start - 1] = NADBL;
+	for (t=dyn_start; t<=fc->t2; t++) {
+	    if (!na(fc->sderr[t])) {
+		fc->sderr[t] = sqrt(fc->sderr[t]);
+	    }
+	}
+    }
+
+    return 0;
+}
+
+/* Compute forecasts for linear models without autoregressive errors.
+   We don't compute forecast standard errors, unless we're integrating
+   the forecast from a static model estimated via OLS.
 */
 
-static int linear_fcast (Forecast *fc, const MODEL *pmod, 
-			 const double **Z, const DATAINFO *pdinfo)
+static int linear_fcast (Forecast *fc, const MODEL *pmod, int yno,
+			 const double **Z)
 {
     const double *offvar = NULL;
-    int yno = pmod->list[1];
-    double lmax = NADBL;
-    double xval;
+    double xval, lmax = NADBL;
     int i, vi, t;
 
     if (pmod->ci == POISSON) {
@@ -1618,8 +1777,8 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
     }
 
     for (t=fc->t1; t<=fc->t2; t++) {
+	double yht = 0.0;
 	int miss = 0;
-	double yh = 0.0;
 
 	for (i=0; i<pmod->ncoeff && !miss; i++) {
 	    int lag;
@@ -1633,7 +1792,7 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
 	    if (na(xval)) {
 		miss = 1;
 	    } else {
-		yh += xval * pmod->coeff[i];
+		yht += xval * pmod->coeff[i];
 	    }
 	}
 
@@ -1641,15 +1800,17 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod,
 	    fc->yhat[t] = NADBL;
 	} else if (FCAST_SPECIAL(pmod->ci)) {
 	    /* special handling for LOGIT and others */
-	    fc->yhat[t] = fcast_transform(yh, pmod->ci, t, offvar, lmax);
+	    fc->yhat[t] = fcast_transform(yht, pmod->ci, t, offvar, lmax);
 	} else {
-	    fc->yhat[t] = yh;
+	    fc->yhat[t] = yht;
 	}
     }
 
     return 0;
 }
 
+#define dynamic_nls(m) (m->ci == NLS && gretl_model_get_int(m, "dynamic"))
+
 static int get_forecast_method (Forecast *fc,
 				MODEL *pmod, 
 				const DATAINFO *pdinfo,
@@ -1657,19 +1818,16 @@ static int get_forecast_method (Forecast *fc,
 {
     int dyn_ok = 0;
     int dyn_errs_ok = 0;
-    int err = 0;
+    int err;
+
+    err = incompatible_options(opt, OPT_D | OPT_S);
+    if (err) {
+	return err;
+    }
 
     fc->dvlags = NULL;
     fc->method = FC_STATIC;
 
-    if ((opt & OPT_D) && (opt & OPT_S)) {
-	/* conflicting options: remove them */
-	fputs("got conflicting options, static and dynamic\n", stderr);
-	opt &= ~OPT_D;
-	opt &= ~OPT_S;
-	err = 1;
-    }
-
     /* do setup for possible lags of the dependent variable,
        unless OPT_S for "static" has been given */
     if (dataset_is_time_series(pdinfo) && !(opt & OPT_S) &&
@@ -1678,21 +1836,27 @@ static int get_forecast_method (Forecast *fc,
     }
 
     if (!(opt & OPT_S)) {
-	if (pmod->ci == ARMA || fc->dvlags != NULL) {
-	    /* dynamic forecast is possible, and not ruled out by 
-	       "static" option */
+	/* user didn't give the "static" option */
+	if (pmod->ci == ARMA || fc->dvlags != NULL || 
+	    dynamic_nls(pmod) || (opt & OPT_I)) {
+	    /* dynamic forecast is possible */
 	    dyn_ok = 1;
 	}
 	if (SIMPLE_AR_MODEL(pmod->ci) || pmod->ci == GARCH) {
 	    dyn_errs_ok = 1;
 	}
-    }    
+    } 
 
     if (!dyn_ok && (opt & OPT_D)) {
 	/* "dynamic" option given, but can't be honored */
-	fputs("requested dynamic option, but it is not applicable\n", stderr);
-	opt &= ~OPT_D;
-	err = 1;
+	return inapplicable_option_error(FCAST, OPT_D);
+    }
+
+    /* NLS: we can only do dynamic out of sample (fc->t1 > pmod->t2) */
+    if (pmod->ci == NLS && dyn_ok && (opt & OPT_D)) {
+	if (fc->t1 <= pmod->t2) {
+	    return inapplicable_option_error(FCAST, OPT_D);
+	}
     }
 
     if (opt & OPT_D) {
@@ -1748,6 +1912,53 @@ static int fc_add_eps (Forecast *fc, int n)
     return err;
 }
 
+/* find the earlist starting point at which we have a previous
+   valid observation on the level of Z[yno] with which to
+   initialize an integrated forecast 
+*/
+
+static int revise_fr_start (FITRESID *fr, int yno, const double **Z,
+			    const DATAINFO *pdinfo)
+{
+    int t, t0;
+
+    if (fr->t0 == 0) {
+	/* certainly can't start before obs 1 */
+	fr->t0 = 1;
+    }
+
+    /* previous obs */
+    t0 = fr->t0 - 1;
+
+    /* find first non-missing value */
+    for (t=t0; t<pdinfo->n; t++) {
+	if (!na(Z[yno][t])) {
+	    break;
+	}
+    }
+
+    t0 = t;
+
+    if (t0 >= pdinfo->n - 2) {
+	return E_MISSDATA;
+    }
+
+    fr->t0 = t0 + 1;
+
+    if (fr->t1 < fr->t0) {
+	fr->t1 = fr->t0;
+    }
+
+    if (fr->t2 < fr->t1) {
+	fr->t2 = fr->t1;
+    }
+
+    fr->fitted[t0] = Z[yno][t0];
+    fr->resid[t0] = 0.0;
+
+    return 0;
+}
+
 /* driver for various functions that compute forecasts
    for different sorts of models */
 
@@ -1758,22 +1969,48 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     Forecast fc;
     const double **Z = (const double **) *pZ;
     int yno = gretl_model_get_depvar(pmod);
+    int integrate = (opt & OPT_I);
     int dummy_AR = 0;
     int DM_errs = 0;
     int dyn_errs = 0;
-    int tdyn = 0;
+    int asy_errs = 0;
+    int int_errs = 0;
     int nf = 0;
-    int t, err = 0;
+    int t, err;
 
     forecast_init(&fc);
 
+    if (integrate) {
+	/* check we we really can do an integrated forecast */
+	int d, parent;
+
+	d = is_standard_diff(yno, pdinfo, &parent);
+	if (!d) {
+	    err = inapplicable_option_error(FCAST, OPT_I);
+	} else {
+	    /* 'yno' should refer to the level variable */
+	    yno = parent;
+	    err = revise_fr_start(fr, yno, (const double **) *pZ, pdinfo);
+	}
+	if (err) {
+	    return err;
+	}
+    }
+
     fc.t1 = fr->t1;
     fc.t2 = fr->t2;
     fc.model_t2 = pmod->t2;
 
-    get_forecast_method(&fc, pmod, pdinfo, opt);
+    err = get_forecast_method(&fc, pmod, pdinfo, opt);
+    if (err) {
+	return err;
+    }
 
-    if (!FCAST_SPECIAL(pmod->ci)) {
+    if (pmod->ci == NLS && fc.method == FC_STATIC) {
+	asy_errs = 1;
+    }
+
+    if (!FCAST_SPECIAL(pmod->ci) && !integrate) {
 	if (!AR_MODEL(pmod->ci) && fc.dvlags == NULL) {
 	    /* we'll do Davidson-MacKinnon error variance */
 	    DM_errs = 1;
@@ -1786,6 +2023,11 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	}
     }
 
+    if (integrate && pmod->ci == OLS && 
+	fc.dvlags == NULL && fc.method != FC_STATIC) {
+	int_errs = 1;
+    }
+
     if (dyn_errs && !AR_MODEL(pmod->ci) && fc.dvlags != NULL) {
 	/* create dummy AR info structure for model with lagged
 	   dependent variable */
@@ -1798,7 +2040,7 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	}
     }
 
-    if (DM_errs || dyn_errs) {
+    if (DM_errs || dyn_errs || asy_errs || int_errs) {
 	err = fit_resid_add_sderr(fr);
     }
 
@@ -1813,6 +2055,7 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	fc_add_eps(&fc, pdinfo->n);
     }
 
+    /* compute the actual forecast */
     if (DM_errs) {
 	err = static_fcast_with_errs(&fc, pmod, Z, pdinfo);
     } else if (pmod->ci == NLS) {
@@ -1823,19 +2066,13 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	err = arma_fcast(&fc, pmod, Z, pdinfo);
     } else if (pmod->ci == GARCH) {
 	err = garch_fcast(&fc, pmod, Z, pdinfo);
+    } else if (integrate) {
+	err = integrated_fcast(&fc, pmod, yno, Z, pdinfo);
     } else {
-	err = linear_fcast(&fc, pmod, Z, pdinfo);
-    }
-
-    /* start of dynamic range */
-    if (fc.method == FC_DYNAMIC) {
-	tdyn = fr->t1;
-    } else if (fc.method == FC_STATIC) {
-	tdyn = fr->t2 + 1;
-    } else {
-	tdyn = pmod->t2 + 1;
+	err = linear_fcast(&fc, pmod, yno, Z);
     }
 
+    /* free any auxiliary info */
     forecast_free(&fc);
 
     if (dummy_AR) {
@@ -1850,8 +2087,13 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 		nf++;
 	    }	    
 	} else if (t >= fr->t0 && t >= pmod->t1 && t <= pmod->t2) {
-	    fr->fitted[t] = pmod->yhat[t];
-	    fr->resid[t] = pmod->uhat[t];
+	    if (integrate) {
+		fr->fitted[t] = fr->fitted[t-1] + pmod->yhat[t];
+		fr->resid[t] = fr->resid[t-1] + pmod->uhat[t];
+	    } else {
+		fr->fitted[t] = pmod->yhat[t];
+		fr->resid[t] = pmod->uhat[t];
+	    }
 	}	    
 	fr->actual[t] = (*pZ)[yno][t];
     }
@@ -1859,15 +2101,7 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     if (nf == 0) {
 	err = E_MISSDATA;
     } else {
-	if (pmod->ci == ARMA) {
-	    /* asymptotic normal */
-	    fr->tval = 1.96;
-	} else {
-	    fr->tval = tcrit95(pmod->dfd);
-	}
-
 	fit_resid_set_dec_places(fr);
-
 	strcpy(fr->depvar, pdinfo->varname[yno]);
 	fr->df = pmod->dfd;
     }
@@ -1875,16 +2109,47 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     return err;
 }
 
+static int fcast_get_limit (const char *s, double ***pZ, DATAINFO *pdinfo)
+{
+    double x;
+    int t, err = 0;
+
+    if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
+    } else {
+	x = generate_scalar(s, pZ, pdinfo, &err);
+    }
+
+    if (x < 1 || x > pdinfo->n) {
+	gretl_errmsg_set(_("Observation number out of bounds"));
+	t = -1;
+    } else {
+	t = x - 1;
+    }
+
+    return t;
+}
+
 static int parse_forecast_string (const char *s, 
 				  gretlopt opt, 
 				  int t2est,
-				  const DATAINFO *pdinfo,
+				  double ***pZ,
+				  DATAINFO *pdinfo,
 				  int *pt1, int *pt2,
-				  char *vname)
+				  int *pk, char *vname)
 {
-    char t1str[16], t2str[16];
+    char f[4][32];
+    char *t1str = NULL, *t2str = NULL;
+    char *kstr = NULL, *vstr = NULL;
+    int nmax, nmin = (opt & OPT_R)? 1 : 0;
     int t1 = 0, t2 = 0;
-    int n, err = 0;
+    int nf, err = 0;
+    
+    /* "static" and "rolling" can't be combined */
+    err = incompatible_options(opt, OPT_S | OPT_R);
+    if (err) {
+	return err;
+    }
 
     if (!strncmp(s, "fcasterr", 8)) {
         s += 8;
@@ -1894,17 +2159,98 @@ static int parse_forecast_string (const char *s,
 
     *vname = '\0';
 
-    n = sscanf(s, "%15s %15s %15s", t1str, t2str, vname);
-    if (n == 1) {
-	strcpy(vname, t1str);
-	n = 0;
-    } else if (n == 3) {
-	n = 2;
+    /* How many fields should we be looking for in the user input?
+       If OPT_R ("rolling") is given, the max is 4: 
+
+       t1, t2, k (steps-ahead), vname
+
+       Otherwise the max is 3 (no k-value):
+
+       t1, t2, vname
+
+       However, if OPT_O ("out-of-sample") is given, we should not 
+       expect t1 and t2, so the max becomes 2 (if OPT_R), else 1.
+
+       Also note: t1 and t2 need not be given (even in the absence
+       if OPT_O) since these default to the current sample range.
+    */
+
+    if (opt & OPT_O) {
+	nmax = (opt & OPT_R)? 2 : 1;
+    } else {
+	nmax = (opt & OPT_R)? 4 : 3;
+    }   
+
+    if (*s == '\0') {
+	nf = 0;
+    } else {
+	nf = sscanf(s, "%31s %31s %31s %31s", f[0], f[1], f[2], f[3]);
     }
 
-    if (n == 2) {
+    if (nf < nmin || nf > nmax) {
+	fprintf(stderr, "fcast: expected %d to %d fields in input, got %d\n",
+		nmin, nmax, nf);
+	return E_DATA;
+    }
+
+    if (opt & OPT_R) {
+	if (nf == 4) {
+	    /* t1, t2, k, vname */
+	    t1str = f[0];
+	    t2str = f[1];
+	    kstr = f[2];
+	    vstr = f[3];
+	} else if (nf == 3) {
+	    /* t1, t2, k */
+	    t1str = f[0];
+	    t2str = f[1];
+	    kstr = f[2];
+	} else if (nf == 2) {
+	    /* k, vname */
+	    kstr = f[0];
+	    vstr = f[1];
+	} else if (nf == 1) {
+	    /* just k */
+	    kstr = f[0];
+	}
+    } else {
+	if (nf == 3) {
+	    /* t1, t2, vname */
+	    t1str = f[0];
+	    t2str = f[1];
+	    vstr = f[2];
+	} else if (nf == 2) {
+	    /* t1, t2 */
+	    t1str = f[0];
+	    t2str = f[1];
+	} else if (nf == 1) {
+	    /* just vname */
+	    vstr = f[0];
+	}
+    }
+
+    if ((opt & OPT_O) && (t1str != NULL || t2str != NULL)) {
+	fprintf(stderr, "fcast: got unexpected t1 and/or t2 field in input\n");
+	return E_DATA;
+    }	
+
+    if (kstr != NULL && pk != NULL) {
+	*pk = positive_int_from_string(kstr);
+    }
+
+    if (vstr != NULL) {
+	strncat(vname, vstr, VNAMELEN - 1);
+    }
+
+    if (t1str != NULL && t2str != NULL) {
 	t1 = dateton(t1str, pdinfo);
+	if (t1 < 0) {
+	    t1 = fcast_get_limit(t1str, pZ, pdinfo);
+	}
 	t2 = dateton(t2str, pdinfo);
+	if (t2 < 0) {
+	    t2 = fcast_get_limit(t2str, pZ, pdinfo);
+	}	
 	if (t1 < 0 || t2 < 0 || t2 < t1) {
 	    err = E_DATA;
 	}
@@ -1942,6 +2288,9 @@ static int parse_forecast_string (const char *s,
  * a static forecast.  By default, the forecast is static within
  * the data range over which the model was estimated, and dynamic
  * out of sample (in cases where a dynamic forecast is meaningful).
+ * If @opt includes %OPT_I, integrate the forecast (only relevant
+ * if the dependent variable in the model in question is recognized
+ * as the first difference of another variable).
  * @err: location to receive error code.
  *
  * Allocates a #FITRESID structure and fills it out with forecasts
@@ -2292,14 +2641,16 @@ static int add_fcast_to_dataset (FITRESID *fr, const char *vname,
 
 	strcpy(VARLABEL(pdinfo, v), _("predicted values"));
 
-	if (v < oldv) {
-	    pprintf(prn, _("Replaced series %s (ID %d)"),
-		    vname, v);
-	} else {
-	    pprintf(prn, _("Generated series %s (ID %d)"),
-		    vname, v);
+	if (gretl_messages_on()) {
+	    if (v < oldv) {
+		pprintf(prn, _("Replaced series %s (ID %d)"),
+			vname, v);
+	    } else {
+		pprintf(prn, _("Generated series %s (ID %d)"),
+			vname, v);
+	    }
+	    pputc(prn, '\n');
 	}
-	pputc(prn, '\n');
     }
 
     return err;
@@ -2311,7 +2662,7 @@ static int model_do_forecast (const char *str, MODEL *pmod,
 {
     char vname[VNAMELEN];
     FITRESID *fr;
-    int t1, t2;
+    int t1, t2, k = -1;
     int err;
 
     if (pmod->ci == ARBOND || pmod->ci == HECKIT) {
@@ -2325,13 +2676,33 @@ static int model_do_forecast (const char *str, MODEL *pmod,
 	return E_DATA;
     }
 
-    err = parse_forecast_string(str, opt, pmod->t2, pdinfo, 
-				&t1, &t2, vname);
+    /* OPT_I for integrate: reject for non-OLS, or if the dependent 
+       variable is not recognized as a first difference */
+    if (opt & OPT_I) {
+	int dv = gretl_model_get_depvar(pmod);
+
+	if (pmod->ci != OLS || !is_standard_diff(dv, pdinfo, NULL)) {
+	    return inapplicable_option_error(FCAST, OPT_I);
+	}
+    }
+
+    err = parse_forecast_string(str, opt, pmod->t2, pZ, pdinfo, 
+				&t1, &t2, &k, vname);
     if (err) {
 	return err;
     }
 
-    fr = get_forecast(pmod, t1, t2, 0, pZ, pdinfo, opt, &err);
+    if (*vname != '\0') {
+	/* saving to named series */
+	opt |= (OPT_A | OPT_Q);
+    }
+
+    if (opt & OPT_R) {
+	fr = rolling_OLS_k_step_fcast(pmod, pZ, pdinfo, t1, t2, 
+				      k, 0, &err);
+    } else {
+	fr = get_forecast(pmod, t1, t2, 0, pZ, pdinfo, opt, &err);
+    }
 
     if (!err && !(opt & OPT_Q)) {
 	err = text_print_forecast(fr, pdinfo, opt, prn);
@@ -2347,7 +2718,7 @@ static int model_do_forecast (const char *str, MODEL *pmod,
     }
 
     free_fit_resid(fr);
-    
+
     return err;
 }
 
@@ -2428,12 +2799,13 @@ static int fill_system_forecast (FITRESID *fr, int i, int yno,
 }
 
 static int system_do_forecast (const char *str, void *ptr, int type,
-			       const double **Z, DATAINFO *pdinfo, 
+			       double ***pZ, DATAINFO *pdinfo, 
 			       gretlopt opt, PRN *prn)
 {
     char vname[VNAMELEN];
     GRETL_VAR *var = NULL;
     equation_system *sys = NULL;
+    const double **Z;
     const int *ylist = NULL;
     const gretl_matrix *F = NULL;
     int t1 = 0, t2 = 0;
@@ -2458,7 +2830,9 @@ static int system_do_forecast (const char *str, void *ptr, int type,
 	ylist = sys->ylist;
     }
 
-    err = parse_forecast_string(str, opt, t2est, pdinfo, &t1, &t2, vname);
+    err = parse_forecast_string(str, opt, t2est, pZ, pdinfo, &t1, &t2, NULL, vname);
+
+    Z = (const double **) *pZ;
 
     if (!err) {
 	if (var != NULL) {
@@ -2504,10 +2878,8 @@ static int system_do_forecast (const char *str, void *ptr, int type,
     	if (asy) {
 	    /* asymptotic normal */
 	    fr->df = var->T;
-	    fr->tval = 1.96;
 	} else {
 	    fr->df = df;
-	    fr->tval = tcrit95(fr->df);
 	}
     
 	for (i=imin; i<=imax && !err; i++) {
@@ -2525,13 +2897,6 @@ static int system_do_forecast (const char *str, void *ptr, int type,
     return err;
 }
 
-static int count_fcast_params (const char *s)
-{
-    char s1[16], s2[16], s3[16];
-
-    return sscanf(s, "%*s %15s %15s %15s", s1, s2, s3);
-}
-
 /**
  * do_forecast:
  * @str: command string, which may include a starting 
@@ -2543,8 +2908,9 @@ static int count_fcast_params (const char *s)
  * a static forecast.  By default, the forecast is static within
  * the data range over which the model was estimated, and dynamic
  * out of sample (in cases where this distinction is meaningful).
- * %OPT_Q can be used to suppress printing of the forecast;
- * %OPT_P can be used to ensure that the values are printed.
+ * %OPT_R: do rolling/recursive forecast.
+ * %OPT_Q: suppress printing of the forecast;
+ * %OPT_P: ensure that the values are printed.
  *
  * In the case of "simple" models with an autoregressive error term 
  * (%AR, %AR1) the predicted values incorporate the forecastable portion 
@@ -2565,24 +2931,13 @@ int do_forecast (const char *str, double ***pZ, DATAINFO *pdinfo,
 	return E_BADSTAT;
     }
 
-    if (opt & OPT_E) {
-	; /* OPT_E -> "fcasterr" compatibility */
+    if ((opt & (OPT_R | OPT_I)) && type != GRETL_OBJ_EQN) {
+	/* "rolling" or "integrate": single equations only */
+	err = E_NOTIMP;
     } else if (type == GRETL_OBJ_EQN) {
-	/* single equation, old fcast compatibility */
-	int n = count_fcast_params(str);
-
-	if (n == 1 || n == 3) {
-	    /* add named fcast series directly, don't print */
-	    opt |= (OPT_A | OPT_Q);
-	}
-    }
-
-    if (type == GRETL_OBJ_EQN) {
 	err = model_do_forecast(str, ptr, pZ, pdinfo, opt, prn);
     } else if (type == GRETL_OBJ_SYS || type == GRETL_OBJ_VAR) {
-	err = system_do_forecast(str, ptr, type, 
-				 (const double **) *pZ,
-				 pdinfo, opt, prn);
+	err = system_do_forecast(str, ptr, type, pZ, pdinfo, opt, prn);
     } else {
 	err = E_DATA;
     }
@@ -2713,10 +3068,8 @@ FITRESID *get_system_forecast (void *p, int ci, int i,
     if (asy) {
 	/* asymptotic normal */
 	fr->df = var->T;
-	fr->tval = 1.96;
     } else {
 	fr->df = df;
-	fr->tval = tcrit95(fr->df);
     }
 
     *err = fill_system_forecast(fr, i, yno, var, sys, 
@@ -2735,10 +3088,7 @@ FITRESID *get_system_forecast (void *p, int ci, int i,
  * @pmod: the model from which forecasts are wanted.
  * @Z: data array.
  * @pdinfo: dataset information.
- * @dyn_ok: location to receive 1 if the "dynamic" option is
- * applicable, 0 otherwise.
- * @add_obs_ok: location to receive 1 if it looks as if we can
- * extend the forecast by adding blank observations, 0 otherwise.
+ * @flags: location to receive flags from among #ForecastFlags.
  * @dt2max: location to receive the last observation that can
  * be supported for a dynamic forecast.
  * @st2max: location to receive the last observation that can
@@ -2749,41 +3099,54 @@ FITRESID *get_system_forecast (void *p, int ci, int i,
  */
 
 void forecast_options_for_model (MODEL *pmod, const double **Z,
-				 const DATAINFO *pdinfo,
-				 int *dyn_ok, int *add_obs_ok,
+				 const DATAINFO *pdinfo, int *flags, 
 				 int *dt2max, int *st2max)
 {
     int *dvlags = NULL;
-    int exo = 1;
+    int dv, exo = 1;
+
+    *flags = 0;
+
+    dv = gretl_model_get_depvar(pmod);
+
+    if (is_standard_diff(dv, pdinfo, NULL)) {
+	*flags |= FC_INTEGRATE_OK;
+    }
+
+    if (pmod->ci == NLS) {
+	/* we'll try winging it! */
+	if (gretl_model_get_int(pmod, "dynamic") && pmod->t2 < pdinfo->n - 1) {
+	    *flags |= FC_AUTO_OK;
+	}
+	return;
+    }
 
-    *dyn_ok = 0;
-    *add_obs_ok = 0;
     *dt2max = pmod->t2;
     *st2max = pmod->t2;
 
     if (pmod->ci == ARMA || pmod->ci == GARCH) {
-	*dyn_ok = 1;
+	*flags |= FC_DYNAMIC_OK;
     } else if (AR_MODEL(pmod->ci)) {
-	*dyn_ok = 1;
+	*flags |= FC_DYNAMIC_OK;
     } else if (dataset_is_time_series(pdinfo) &&
 	       has_depvar_lags(pmod, pdinfo)) {
-	*dyn_ok = 1;
+	*flags |= FC_DYNAMIC_OK;
     }
 
-    if (*dyn_ok) {
+    if (*flags & FC_DYNAMIC_OK) {
 	int err = process_lagged_depvar(pmod, pdinfo, &dvlags);
 
 	if (!err) {
 	    exo = has_real_exog_regressors(pmod, dvlags, Z, pdinfo);
 	}
 	if (!exo) {
-	    *add_obs_ok = 1;
+	    *flags |= FC_ADDOBS_OK;
 	    *dt2max = pdinfo->n - 1;
 	}
     } 
 
     if (exo) {
-	int *xlist = model_xlist(pmod);
+	const int *xlist = model_xlist(pmod);
 
 	if (xlist != NULL) {
 	    *dt2max = fcast_get_t2max(xlist, dvlags, pmod, Z, pdinfo, FC_DYNAMIC);
@@ -2796,19 +3159,93 @@ void forecast_options_for_model (MODEL *pmod, const double **Z,
     }
 }
 
-/* recursive one-step ahead forecasts and forecast errors, for models
-   estimated via OLS */
+static int y_lag (int v, int parent, const DATAINFO *pdinfo)
+{
+    if (pdinfo->varinfo[v]->transform == LAGS) {
+	int pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
+
+	if (pv == parent) {
+	    return pdinfo->varinfo[v]->lag;
+	}
+    }
+
+    return 0;
+}
+
+static int k_step_init (MODEL *pmod, const DATAINFO *pdinfo, 
+			double **Z, int k, 
+			double **py, int **pllist)
+{
+    double *y = NULL;
+    int *llist = NULL;
+    int vy = pmod->list[1];
+    int i, nl = 0;
+
+    for (i=2; i<=pmod->list[0]; i++) {
+	if (y_lag(pmod->list[i], vy, pdinfo)) {
+	    nl++;
+	}
+    }
+
+    if (nl == 0) {
+	return 0;
+    }
+
+    y = malloc(pdinfo->n * sizeof *y);
+    llist = gretl_list_new(pmod->list[0] - 1);
+
+    if (y == NULL || llist == NULL) {
+	free(y);
+	free(llist);
+	return E_ALLOC;
+    }
+
+    for (i=0; i<pdinfo->n; i++) {
+	y[i] = Z[vy][i];
+    }
+
+    for (i=2; i<=pmod->list[0]; i++) {
+	llist[i-1] = y_lag(pmod->list[i], vy, pdinfo);
+    }    
+
+    *py = y;
+    *pllist = llist;
+
+    return 0;
+}
+
+static int rolling_fcast_adjust_obs (MODEL *pmod, int *t1, int t2, int k)
+{
+    /* the earliest possible forecast start */
+    int t1min = pmod->t1 + pmod->ncoeff + k - 1;
+    int err = 0;
+
+    if (*t1 < t1min) {
+	*t1 = t1min;
+    }
+
+    /* minimal forecast range = 1, otherwise error */
+    if (t2 < *t1) {
+	err = E_OBS;
+    }
+
+    return err;
+}
+
+/* recursive k-step ahead forecasts, for models estimated via OLS */
 
 FITRESID * 
-rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-			    int t1, int t2, int pre_n, int *err)
+rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+			  int t1, int t2, int k, int pre_n, int *err)
 {
     FITRESID *fr;
     int orig_t1 = pdinfo->t1;
     int orig_t2 = pdinfo->t2;
-    double xit, yf;
-    int nf = t2 - t1 + 1;
+    double *y = NULL;
+    int *llist = NULL;
+    double xit, yf = NADBL;
     MODEL mod;
+    int j, p, vi, nf;
     int i, s, t;
 
     if (pmod->ci != OLS) {
@@ -2816,47 +3253,85 @@ rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return NULL;
     }
 
+    if (k < 1) {
+	gretl_errmsg_set("recursive forecast: steps-ahead must be >= 1");
+	*err = E_DATA;
+	return NULL;
+    }    
+
     if (gretl_model_get_int(pmod, "daily_repack")) {
 	*err = E_DATA;
 	return NULL;
     }
 
-    if (t1 - pmod->t1 < pmod->ncoeff || t2 < t1) {
-	*err = E_OBS;
-	return NULL;
-    }
-
-    fr = fit_resid_new_for_model(pmod, pdinfo, t1, t2, pre_n, err); 
+    /* check feasibility of forecast range */
+    *err = rolling_fcast_adjust_obs(pmod, &t1, t2, k);
     if (*err) {
 	return NULL;
     }
 
-    fr->method = FC_ONESTEP;
+    if (k > 1) {
+	*err = k_step_init(pmod, pdinfo, *pZ, k, &y, &llist);
+	if (*err) {
+	    return NULL;
+	}
+    }
 
+    fr = fit_resid_new_for_model(pmod, pdinfo, t1, t2, pre_n, err); 
+    if (*err) {
+	free(y);
+	free(llist);
+	return NULL;
+    }
+
+    fr->method = FC_KSTEP;
+    fr->k = k;
+
+    /* sample range for initial estimation */
     pdinfo->t1 = pmod->t1;
-    pdinfo->t2 = t1 - 1;
+    pdinfo->t2 = t1 - k; /* start of fcast range minus k */
+
+    /* number of forecasts */
+    nf = t2 - t1 + 1;
+
+    fprintf(stderr, "rolling fcast: pdinfo->t1=%d, pdinfo->t2=%d, t1=%d, t2=%d, k=%d, nf=%d\n",
+	    pdinfo->t1, pdinfo->t2, t1, t2, k, nf);
 
     for (t=0; t<pdinfo->n; t++) {
 	fr->actual[t] = (*pZ)[pmod->list[1]][t];
     }
 
     for (s=0; s<nf; s++) {
-	mod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A);
+	mod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A | OPT_Z);
 	if (mod.errcode) {
 	    *err = mod.errcode;
 	    clear_model(&mod);
 	    break;
 	}
 
+	/* the first obs following the estimation sample */
 	t = pdinfo->t2 + 1;
-	yf = 0.0;
-	for (i=0; i<mod.ncoeff; i++) {
-	    xit = (*pZ)[mod.list[i+2]][t];
-	    if (na(xit)) {
-		yf = NADBL;
-		break;
-	    } else {
-		yf += mod.coeff[i] * xit;
+
+	for (j=0; j<k; j++) {
+	    yf = 0.0;
+	    /* steps ahead */
+	    for (i=0; i<mod.ncoeff; i++) {
+		vi = mod.list[i+2];
+		p = (llist != NULL)? llist[i+1] : 0;
+		if (p > 0 && p <= j) {
+		    xit = y[t-p];
+		} else {
+		    xit = (*pZ)[vi][t];
+		}
+		if (na(xit)) {
+		    yf = NADBL;
+		    break;
+		} else {
+		    yf += mod.coeff[i] * xit;
+		}
+	    }
+	    if (y != NULL && j < k - 1) {
+		y[t++] = yf;
 	    }
 	}
 
@@ -2880,6 +3355,9 @@ rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	fit_resid_set_dec_places(fr);
 	strcpy(fr->depvar, pdinfo->varname[pmod->list[1]]);
     }
+
+    free(y);
+    free(llist);
 	
     return fr;
 }
