@@ -24,12 +24,20 @@ typedef enum {
     FC_STATIC,
     FC_DYNAMIC,
     FC_AUTO,
-    FC_ONESTEP
+    FC_KSTEP
 } ForecastMethod;
 
+typedef enum {
+    FC_AUTO_OK      = 1 << 0,
+    FC_DYNAMIC_OK   = 1 << 1,
+    FC_ADDOBS_OK    = 1 << 2,
+    FC_INTEGRATE_OK = 1 << 3
+} ForecastFlags;
+
 struct FITRESID_ {
     int model_ID;
     int asymp;
+    int std;
     int model_t1;
     int method;
     double *actual;
@@ -37,10 +45,11 @@ struct FITRESID_ {
     double *resid;
     double *sderr;
     double sigma;
-    double tval;
+    double alpha;
     int pmax;
     int df;
     int t0, t1, t2;
+    int k; 
     int nobs;
     char depvar[VNAMELEN];
 };
@@ -63,15 +72,14 @@ int do_forecast (const char *str, double ***pZ, DATAINFO *pdinfo,
 		 gretlopt opt, PRN *prn);
 
 void forecast_options_for_model (MODEL *pmod, const double **Z,
-				 const DATAINFO *pdinfo,
-				 int *dyn_ok, int *add_obs_ok,
+				 const DATAINFO *pdinfo, int *flags,
 				 int *dt2max, int *st2max);
 
 gretl_matrix *get_forecast_matrix (int idx, int *err);
 
 FITRESID *
-rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-			    int t1, int t2, int pre_n, int *err);
+rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+			  int t1, int t2, int k, int pre_n, int *err);
 
 void forecast_matrix_cleanup (void);
 
