@@ -62,7 +62,7 @@ int sort_series (const double *x, double *y, int f,
     double *z = NULL;
     struct val_mark *vm = NULL;
     int markers = 0;
-    int n = pdinfo->t2 - pdinfo->t1 + 1;
+    int n = sample_size(pdinfo);
     int i, t;
 
     if (pdinfo->S != NULL && !complex_subsampled()) {
@@ -140,7 +140,7 @@ int gretl_sort_by (const double *x, const double *y,
 		   double *z, const DATAINFO *pdinfo)
 {
     struct pair_sorter *xy;
-    int n = pdinfo->t2 - pdinfo->t1 + 1;
+    int n = sample_size(pdinfo);
     int i, t;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
@@ -217,7 +217,7 @@ int rank_series (const double *x, double *y, int f,
 {
     double *sx = NULL;
     double *rx = NULL;
-    int n = pdinfo->t2 - pdinfo->t1 + 1;
+    int n = sample_size(pdinfo);
     int m = n;
     int i, t;
 
@@ -420,6 +420,37 @@ int fracdiff_series (const double *x, double *y, double d,
     return 0;
 }
 
+/**
+ * boxcox_series:
+ * @x: array of original data.
+ * @y: array into which to write the result.
+ * @d: lambda parameter.
+ * @pdinfo: data set information.
+ *
+ * Calculates in @y the Box-Cox transformation for the 
+ * input series @x.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int boxcox_series (const double *x, double *y, double d,
+		   const DATAINFO *pdinfo)
+{
+    int t;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (na(x[t])) {
+	    y[t] = NADBL;
+	} else if (d == 0) {
+	    y[t] = (x[t] > 0)? log(x[t]) : NADBL; 
+	} else {
+	    y[t] = (pow(x[t], d) - 1) / d;
+	}
+    } 
+    
+    return 0;
+}
+
 int cum_series (const double *x, double *y, const DATAINFO *pdinfo)
 {
     int t, s = pdinfo->t1;
@@ -442,7 +473,7 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
 {
     int t1 = pdinfo->t1;
     int t2 = pdinfo->t2;
-    double *z = NULL;
+    int *z = NULL;
     int i, t, n;
 
     array_adjust_t1t2(x, &t1, &t2);
@@ -457,14 +488,80 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
-    /* generate uniform random series */
-    gretl_rand_uniform(z, 0, n - 1);
+    /* generate n uniform drawings from [t1 .. t2] */
+    gretl_rand_int_minmax(z, n, t1, t2);
 
-    /* sample from source series based on indices */
-    for (t=t1; t<=t2; t++) {
-	i = t1 + n * z[t-t1];
-	i = (i > t2)? t2 : i;
-	y[t] = x[i];
+    /* sample from source series x based on indices z */
+    for (t=t1, i=0; t<=t2; t++, i++) {
+	y[t] = x[z[i]];
+    }
+
+    free(z);
+
+    return 0;
+}
+
+int block_resample_series (const double *x, double *y, int blocklen,
+			   const DATAINFO *pdinfo)
+{
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int *z = NULL;
+    int m, rem, bt2, x0;
+    int i, s, t, n;
+
+    if (blocklen <= 0) {
+	return E_DATA;
+    }
+
+    if (blocklen == 1) {
+	return resample_series(x, y, pdinfo);
+    }    
+
+    array_adjust_t1t2(x, &t1, &t2);
+
+    n = t2 - t1 + 1;
+
+    m = n / blocklen;
+    rem = n % blocklen;
+
+    /* Let n now represent the number of blocks of @blocklen
+       contiguous observations which we need to select; the
+       last of these may not be fully used.
+    */     
+    n = m + (rem > 0);
+
+    /* the last selectable starting point for a block */
+    bt2 = t2 - blocklen + 1;
+
+    if (bt2 < t1) {
+	return E_DATA;
+    }
+
+    z = malloc(n * sizeof *z);
+    if (z == NULL) {
+	return E_ALLOC;
+    }
+
+    /* Generate uniform random series: we want n drawings from the
+       range [t1 .. t2 - blocklen + 1], each of which will be
+       interpreted as the starting point of a block to be used.
+    */
+    gretl_rand_int_minmax(z, n, t1, bt2);
+
+    /* Sample from the source series using blocks given by the random
+       indices: note that the last block will be incomplete if rem > 0
+    */
+    t = t1;
+    for (i=0; i<n; i++) {
+	x0 = z[i];
+	for (s=0; s<blocklen; s++) {
+	    if (t <= t2) {
+		y[t++] = x[x0+s];
+	    } else {
+		break;
+	    }
+	}
     }
 
     free(z);
@@ -477,58 +574,50 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
  * @x: array of original data.
  * @y: array into which to write the result.
  * @pdinfo: data set information.
- * @A: matrix for autoregressive polynomial.
- * @C: matrix for moving average polynomial.
+ * @A: vector for autoregressive polynomial.
+ * @C: vector for moving average polynomial.
+ * @y0: initial value of output series.
  *
- * Filters x according to y_t = C(L)/A(L) x_t; the matrices A and C
- * must have two columns, or be empty. The rows of A and C must
- * contain the coefficients for the powers other than 0 (assumed 1),
- * with the exponents on column one and the corresponding coefficient
- * on column two. For example, the matrix corresponding to the
- * polynomial C(L) = 1 - 0.75 L^2 should be given as { 2, -0.75 };
- * negative exponents (leads) are allowed for the C polynomial only.
+ * Filters x according to y_t = C(L)/A(L) x_t.  If the intended
+ * AR order is p, @A should be a vector of length p.  If the 
+ * intended MA order is q, @C should be vector of length (q+1), 
+ * the first entry giving the coefficient at lag 0.  However, if 
+ * @C is %NULL this is taken to mean that the lag-0 MA coefficient 
+ * is unity (and all others are zero).
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-
 int filter_series (const double *x, double *y, const DATAINFO *pdinfo, 
-		   gretl_matrix *A, gretl_matrix *C, double y0)
+		   gretl_vector *A, gretl_vector *C, double y0)
 {
     int t1 = pdinfo->t1;
     int t2 = pdinfo->t2;
-    int t, s, i, m, n;
-    int amin = 0, amax = 0;
-    int cmin = 0, cmax = 0;
-    double coef, *e;
+    int t, s, i, n;
+    int amax, cmax;
+    double xlag, coef, *e;
     int err = 0;
 
-    if (!gretl_is_null_matrix(C)) {
-	for (i=0; i<C->rows; i++) {
-	    m = gretl_matrix_get(C, i, 0);
-	    if (m < cmin) {
-		cmin = m; 
-	    } else if (m > cmax) {
-		cmax = m; 
-	    } 
+    if (gretl_is_null_matrix(C)) {
+	cmax = 0;
+    } else {
+	cmax = gretl_vector_get_length(C);
+	if (cmax == 0) {
+	    /* if present, C must be a vector */
+	    return E_NONCONF;
 	}
     }
 
-    if (!gretl_is_null_matrix(A)) {
-	for (i=0; i<A->rows; i++) {
-	    m = gretl_matrix_get(A, i, 0);
-	    if (m < amin) {
-		amin = m; 
-	    } else if (m > amax) {
-		amax = m; 
-	    } 
+    if (gretl_is_null_matrix(A)) {
+	amax = 0;
+    } else {
+	amax = gretl_vector_get_length(A);
+	if (amax == 0) {
+	    /* if present, A must be a vector */
+	    return E_NONCONF;
 	}
     }
 
-    if (amin < 0) {
-	return E_DATA;
-    } 
-
     err = array_adjust_t1t2(x, &t1, &t2);
     if (err) {
 	return E_DATA;
@@ -542,50 +631,51 @@ int filter_series (const double *x, double *y, const DATAINFO *pdinfo,
     }
 
     s = 0;
-    if (gretl_is_null_matrix(C)) {
+    if (cmax) {
+	for (t=t1; t<=t2; t++) {
+	    e[s] = 0;
+	    for (i=0; i<cmax; i++) {
+		xlag = (t-i >= t1)? x[t-i] : 0;
+		if (na(xlag)) {
+		    e[s] = NADBL;
+		    break;
+		} else {
+		    coef = gretl_vector_get(C, i);
+		    e[s] += xlag * coef;
+		}
+	    } 
+	    s++;
+	}
+    } else {
+	/* implicitly MA(0) = 1 */
 	for (t=t1; t<=t2; t++) {
 	    e[s++] = x[t];
 	}
-    } else {
-	for (t=t1; t<=t2; t++) {
-	    if (s >= cmax && t <= t2+cmin) {
-		e[s] = x[t];
-		for (i=0; i<C->rows; i++) {
-		    m = gretl_matrix_get(C, i, 0);
-		    coef = gretl_matrix_get(C, i, 1);
-		    e[s] += x[t-m] * coef;
-		} 
-	    } else {
-		e[s] = NADBL;
-	    }
-	    s++;
-	} 
     }
 
     s = 0;
-    if (gretl_is_null_matrix(A)) {
+    if (amax) {
 	for (t=t1; t<=t2; t++) {
-	    y[t] = e[s++];
+	    if (na(e[s])) {
+		y[t] = NADBL;
+	    } else {
+		y[t] = e[s];
+		for (i=0; i<amax; i++) {
+		    xlag = (t-i>t1)? y[t-i-1] : y0;
+		    if (na(xlag)) {
+			y[t] = NADBL;
+			break;
+		    } else {
+			coef = gretl_vector_get(A, i);
+			y[t] += coef * xlag;
+		    }
+		} 
+	    }
+	    s++;
 	}
     } else {
-	double xlag;
-
-	for (t=t1; t<=t2-amin; t++) {
-	    y[t] = e[s];
-	    for (i=0; i<A->rows; i++) {
-		m = gretl_matrix_get(A, i, 0);
-		coef = gretl_matrix_get(A, i, 1);
-#if 1
-		xlag = (t < m || na(y[t-m]))? y0 : y[t-m];
-		y[t] -= coef * xlag;
-#else
-		xlag = (s < m)? y0 : y[t-m];
-		if (!na(xlag)) {
-		    y[t] -= coef * xlag;
-		}
-#endif
-	    } 
-	    s++;
+	for (t=t1; t<=t2; t++) {
+	    y[t] = e[s++];
 	}
     }
 
@@ -1623,13 +1713,17 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     const double *x = NULL;
     int i, t;
 
-    if (code == GENR_H) {
+    if (code == M_H) {
 	x = gretl_model_get_data(pmod, "garch_h");
-	if (x == NULL) return E_DATA;
-    } else if (code == GENR_AHAT) {
+	if (x == NULL) return E_BADSTAT;
+    } else if (code == M_AHAT) {
 	x = gretl_model_get_data(pmod, "ahat");
-	if (x == NULL) return E_DATA;
-    }	
+	if (x == NULL) return E_BADSTAT;
+    } else if (code == M_UHAT || code == M_UHAT2) {
+	if (pmod->uhat == NULL) return E_BADSTAT;
+    } else if (code == M_YHAT) {
+	if (pmod->yhat == NULL) return E_BADSTAT;
+    }
 
     if (dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -1641,22 +1735,30 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	(*pZ)[i][t] = NADBL;
     }
 
-    if (code == GENR_RESID) {
+    if (code == M_UHAT) {
 	sprintf(vname, "uhat%d", pmod->ID);
-	sprintf(vlabel, _("residual from model %d"), pmod->ID);
+	if (pmod->ci == GARCH && (pmod->opt & OPT_U)) {
+	    sprintf(vlabel, _("standardized residual from model %d"), pmod->ID);
+	} else {
+	    sprintf(vlabel, _("residual from model %d"), pmod->ID);
+	}
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    (*pZ)[i][t] = pmod->uhat[t];
 	}
-    } else if (code == GENR_FITTED) {
+    } else if (code == M_YHAT) {
 	sprintf(vname, "yhat%d", pmod->ID);
 	sprintf(vlabel, _("fitted value from model %d"), pmod->ID);
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    (*pZ)[i][t] = pmod->yhat[t];
 	}
-    } else if (code == GENR_RESID2) { 
+    } else if (code == M_UHAT2) { 
 	/* squared residuals */
 	sprintf(vname, "usq%d", pmod->ID);
-	sprintf(vlabel, _("squared residual from model %d"), pmod->ID);
+	if (pmod->ci == GARCH && (pmod->opt & OPT_U)) {
+	    sprintf(vlabel, _("squared standardized residual from model %d"), pmod->ID);
+	} else {
+	    sprintf(vlabel, _("squared residual from model %d"), pmod->ID);
+	}
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (na(pmod->uhat[t])) {
 		(*pZ)[i][t] = NADBL;
@@ -1664,14 +1766,14 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		(*pZ)[i][t] = pmod->uhat[t] * pmod->uhat[t];
 	    }
 	}
-    } else if (code == GENR_H) { 
+    } else if (code == M_H) { 
 	/* garch variance */
 	sprintf(vname, "h%d", pmod->ID);
 	sprintf(vlabel, _("fitted variance from model %d"), pmod->ID);
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    (*pZ)[i][t] = x[t];
 	}
-    } else if (code == GENR_AHAT) { 
+    } else if (code == M_AHAT) { 
 	/* fixed-effects constants */
 	sprintf(vname, "ahat%d", pmod->ID);
 	sprintf(vlabel, _("per-unit constants from model %d"), pmod->ID);
@@ -1789,12 +1891,7 @@ int get_t_from_obs_string (const char *s, const double **Z,
 	} else {
 	    if (gretl_is_scalar(s)) {
 		t = gretl_scalar_get_value(s);
-	    } else if (strlen(s) == 1) {
-		t = loop_scalar_read(s[0]);
-#if OBS_DEBUG
-		fprintf(stderr, " loop_scalar_read gave t = %d\n", t);
-#endif
-	    }
+	    } 
 
 	    if (t > pdinfo->n) {
 		char try[16];
@@ -2174,3 +2271,494 @@ int list_linear_combo (double *y, const int *list,
 
     return err;
 }
+
+/* Imhof: draws on the RATS code in IMHOF.SRC from Estima, 2004.
+
+   Imhof Procedure for computing P(u'Au < x) for a quadratic form in
+   Normal(0,1) variables. This can be used for ratios of quadratic
+   forms as well, since P((u'Au/u'Bu) < x) = P(u'(A-xB)u < 0).
+
+   In the Durbin-Watson context, 'B' is the identity matrix and
+   'x' is the Durbin-Watson statistic.
+
+   References:
+
+   Imhof, J.P (1961), Computing the Distribution of Quadratic Forms of
+   Normal Variables, Biometrika 48, 419-426.
+
+   Abrahamse, A.P.J and Koerts, J. (1969), On the Theory and
+   Application of the General Linear Model, Rotterdam University
+   Press.
+*/
+
+static double imhof_bound (const double *lambda, int k, int *err)
+{
+    double e1 = 0.0001; /* Max truncation error due to finite 
+			   upper bound on domain */
+    double e2 = 0.0001; /* Cutoff for deciding whether an 
+			   eigenvalue is effectively zero */
+    double absl, bound;
+    double nl = 0.0, sum = 0.0;
+    int i;
+
+    for (i=0; i<k; i++) {
+	absl = fabs(lambda[i]);
+	if (absl > e2) {
+	    nl += 1.0;
+	    sum += log(absl);
+	}
+    }
+
+    if (nl == 0.0) {
+	fprintf(stderr, "imhof_bound: got no non-zero eigenvalues\n");
+	*err = E_DATA;
+	return NADBL;
+    }
+
+    /* The key factor in the integrand is the product of 
+       (1+(lambda(i)*x)**2)**(1/4) across i. Since, for those 
+       factors with small |lambda(i)|, this won't go to zero very
+       quickly, we count only the terms on the bigger eigenvalues.
+    */
+
+    nl = 0.5 * nl;
+    sum = 0.5 * sum + log(M_PI * nl);
+    bound = exp(-(sum + log(e1)) / nl);
+    bound += 5.0 / nl;
+
+    if (bound < 0) {
+	fprintf(stderr, "imhof_bound: got negative result\n");
+	*err = E_DATA;
+	bound = NADBL;
+    }
+
+    return bound;
+}
+
+static double vecsum (const double *x, int k)
+{
+    double sum = 0.0;
+    int i;
+
+    for (i=0; i<k; i++) {
+	sum += x[i];
+    }
+
+    return sum;
+}
+
+static double imhof_f (double u, const double *lambda, int k, double arg)
+{
+    double ul, rho = 0.0;
+    double theta = -u * arg;
+    int i;
+
+    /* The value at zero isn't directly computable as
+       it produces 0/0. The limit is computed below.
+    */
+    if (u == 0.0) {
+	return 0.5 * (-arg + vecsum(lambda, k));
+    }
+
+    for (i=0; i<k; i++) {
+	ul = u * lambda[i];
+	theta += atan(ul);
+	rho += log(1.0 + ul * ul);
+    }
+
+    return sin(0.5 * theta) / (u * exp(0.25 * rho));
+}
+
+#define gridlimit 2048
+
+/*
+  Adaptation of Abrahamse and Koert's Pascal code.  Evaluates the
+  integral by Simpson's rule with grid size halving each time until
+  the change from moving to a tighter grid is negligible. By halving
+  the grid, only the odd terms need to be computed, as the old ones
+  are already in the sum. Points entering get x 4 weights, which then
+  get reduced to x 2 on the next iteration.
+*/
+
+static double imhof_integral (double arg, const double *lambda, int k,
+			      double bound, int *err)
+{
+    double e3 = 0.0001;
+    double base, step, sum1;
+    double int0 = 0.0, int1 = 0.0;
+    double eps4 = 3.0 * M_PI * e3;
+    double sum4 = 0.0;
+    double ret = NADBL;
+    int j, n = 2;
+
+    base = imhof_f(0, lambda, k, arg);
+    base += imhof_f(bound, lambda, k, arg);
+
+    while (n < gridlimit) {
+	step = bound / n;
+	sum1 = base + sum4 * 2.0;
+	base = sum1;
+	sum4 = 0.0;
+	for (j=1; j<=n; j+=2) {
+	    sum4 += imhof_f(j * step, lambda, k, arg);
+	}
+	int1 = (sum1 + 4 * sum4) * step;
+	if (n > 8 && fabs(int1 - int0) < eps4) {
+	    break;
+	}
+	int0 = int1;
+	n *= 2;
+    }
+
+    if (n > gridlimit) {
+	fprintf(stderr, "n = %d, Imhof integral failed to converge\n", n);
+	*err = E_NOCONV;
+    } else {
+	ret = 0.5 - int1 / (3.0 * M_PI);
+	if (ret < 0 && ret > -1.0e-14) {
+	    ret = 0.0;
+	} else if (ret < 0) {
+	    fprintf(stderr, "n = %d, Imhof integral gave negative value %g\n", n, ret);
+	    *err = E_DATA;
+	    ret = NADBL;
+	}
+    }
+
+    return ret;
+}
+
+static int imhof_get_eigenvals (const gretl_matrix *m,
+				double **plam, int *pk)
+{
+    gretl_matrix *E, *A;
+    int err = 0;
+
+    A = gretl_matrix_copy(m);
+    if (A == NULL) {
+	return E_ALLOC;
+    }
+
+    E = gretl_general_matrix_eigenvals(A, 0, &err);
+
+    if (!err) {
+	*pk = E->rows;
+	*plam = gretl_matrix_steal_data(E);
+    }
+
+    gretl_matrix_free(A);
+    gretl_matrix_free(E);
+
+    return err;
+}
+
+/* Implements the "imhof" function in genr: computes the probability
+   P(u'Au < arg) for a quadratic form in Normal(0,1) variables.  The
+   argument @m may be either the square matrix A or a column
+   vector containing the precomputed eigenvalues of A.
+*/
+
+double imhof (const gretl_matrix *m, double arg, int *err)
+{
+    double *lambda = NULL;
+    double bound, ret = NADBL;
+    int k = 0, free_lambda = 0;
+
+    errno = 0;
+
+    if (m->cols == 1) {
+	/* we'll assume m is a column vector of eigenvalues */
+	lambda = m->val;
+	k = m->rows;
+    } else if (m->rows == m->cols) {
+	/* we'll assume m is the 'A' matrix */
+	*err = imhof_get_eigenvals(m, &lambda, &k);
+	free_lambda = 1;
+    } else {
+	/* huh? */
+	*err = E_DATA;
+    }
+
+    if (!*err) {
+	bound = imhof_bound(lambda, k, err);
+    }
+
+    if (!*err) {
+	ret = imhof_integral(arg, lambda, k, bound, err);
+    }
+
+    if (errno != 0) {
+	fprintf(stderr, "imhof: %s\n", strerror(errno));
+	if (!*err) {
+	    *err = E_NOCONV;
+	}
+	ret = NADBL;
+	errno = 0;
+    }
+
+    if (free_lambda) {
+	free(lambda);
+    }
+
+    return ret;
+}
+
+/* Implements the "dwpval" function in genr: given the residual vector
+   @u and the matrix of regressors, @X, calculates the Durbin-Watson
+   statistic then finds its p-value via the Imhof/Koerts/Abrahamse
+   procedure.
+*/
+
+double dw_pval (const gretl_matrix *u, const gretl_matrix *X, 
+		double *pDW, int *perr)
+{
+    gretl_matrix *M = NULL;
+    gretl_matrix *A = NULL;
+    gretl_matrix *MA = NULL;
+    gretl_matrix *XX = NULL;
+    gretl_matrix *E = NULL;
+    double uu, DW;
+    double pv = NADBL;
+    int k = X->cols;
+    int n = X->rows;
+    int i, err = 0;
+
+    M = gretl_identity_matrix_new(n);
+    A = gretl_DW_matrix_new(n);
+    MA = gretl_matrix_alloc(n, n);
+    XX = gretl_matrix_alloc(k, k);
+
+    if (M == NULL || A == NULL || MA == NULL || XX == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    gretl_matrix_multiply_mod(X, GRETL_MOD_TRANSPOSE,
+			      X, GRETL_MOD_NONE,
+			      XX, GRETL_MOD_NONE);
+
+    err = gretl_invert_symmetric_matrix(XX);
+
+    if (!err) {
+	/* M = I - X(X'X)^{-1}X' */
+	err = gretl_matrix_qform(X, GRETL_MOD_NONE,
+				 XX, M, GRETL_MOD_DECREMENT);
+    }
+
+    if (!err) {
+	err = gretl_matrix_multiply(M, A, MA);
+    }
+
+    if (!err) {
+	uu = gretl_matrix_dot_product(u, GRETL_MOD_TRANSPOSE,
+				      u, GRETL_MOD_NONE,
+				      &err);
+    }
+
+    if (!err) {
+	DW = gretl_scalar_qform(u, A, &err);
+    }
+
+    if (!err) {
+	DW /= uu;
+	E = gretl_general_matrix_eigenvals(MA, 0, &err);
+    }
+
+    if (!err) {
+	k = n - k;
+	for (i=0; i<k; i++) {
+	    E->val[i] -= DW;
+	}
+	gretl_matrix_reuse(E, k, 1);
+	pv = imhof(E, 0.0, &err);
+	if (!err && pDW != NULL) {
+	    *pDW = DW;
+	}
+    }
+
+ bailout:
+
+    gretl_matrix_free(M);
+    gretl_matrix_free(A);
+    gretl_matrix_free(MA);
+    gretl_matrix_free(XX);
+    gretl_matrix_free(E);
+
+    *perr = err;
+
+    return pv;
+}
+
+/* create a matrix containing ACF and PACF values for each
+   column of the input matrix, @m, with lag order @p.
+*/
+
+gretl_matrix *multi_acf (const gretl_matrix *m, 
+			 const int *list,
+			 const double **Z,
+			 const DATAINFO *pdinfo,
+			 int p, int *err)
+{
+    gretl_matrix *a, *A = NULL;
+    const double *x;
+    double xa;
+    int nv, T, acol, pcol;
+    int i, j;
+    
+    if (list == NULL && gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (m != NULL) {
+	nv = m->cols;
+    } else {
+	nv = list[0];
+    }
+
+    A = gretl_matrix_alloc(p, 2 * nv);
+    if (A == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    if (m != NULL) {
+	x = m->val;
+	T = m->rows;
+    } else {
+	x = Z[list[1]] + pdinfo->t1;
+	T = sample_size(pdinfo);
+    }
+
+    acol = 0;
+    pcol = nv;
+
+    for (j=0; j<nv; j++) {
+	/* get ACF/PACF for column/series */
+	a = acf_vec(x, p, NULL, T, err);
+	if (*err) {
+	    gretl_matrix_free(a);
+	    gretl_matrix_free(A);
+	    return NULL;
+	}
+
+	/* transcribe into A matrix and free */
+	for (i=0; i<p; i++) {
+	    xa = gretl_matrix_get(a, i, 0);
+	    gretl_matrix_set(A, i, acol, xa);
+	    xa = gretl_matrix_get(a, i, 1);
+	    gretl_matrix_set(A, i, pcol, xa);
+	}
+	acol++;
+	pcol++;
+
+	gretl_matrix_free(a);
+
+	/* move to next data read position */
+	if (j < nv - 1) {
+	    if (m != NULL) {
+		x += m->rows;
+	    } else {
+		x = Z[list[j+2]] + pdinfo->t1;
+	    }
+	}
+    }
+
+    return A;
+}
+
+gretl_matrix *multi_xcf (const void *px, int xtype,
+			 const void *py, int ytype,
+			 const double **Z,
+			 const DATAINFO *pdinfo,
+			 int p, int *err)
+{
+    const int *xlist = NULL;
+    const gretl_matrix *Xmat = NULL;
+    const double *xvec = NULL;
+    const double *yvec = NULL;
+    gretl_matrix *xj, *XCF = NULL;
+    int T = sample_size(pdinfo);
+    int np = 2 * p + 1;
+    int Ty, nx = 1;
+    int i, j;
+
+    if (xtype == LVEC) {
+	xlist = px;
+	nx = xlist[0];
+	if (nx < 1) {
+	    *err = E_DATA;
+	    return NULL;
+	}	    
+	xvec = Z[xlist[1]] + pdinfo->t1; 
+    } else if (xtype == MAT) {
+	Xmat = px;
+	if (gretl_is_null_matrix(Xmat)) {
+	    *err = E_DATA;
+	    return NULL;
+	}
+	nx = Xmat->cols;
+	T = Xmat->rows;
+	xvec = Xmat->val;
+    } else {
+	xvec = px;
+    }
+
+    if (ytype == MAT) {
+	const gretl_matrix *ymat = py;
+
+	if (ymat->cols != 1) {
+	    *err = E_NONCONF;
+	    return NULL;
+	}	  
+	yvec = ymat->val;
+	Ty = ymat->rows;
+    } else {
+	yvec = (const double *) py + pdinfo->t1;
+	Ty = sample_size(pdinfo);
+    }
+
+    if (Ty != T) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    if (nx > 1) {
+	XCF = gretl_matrix_alloc(np, nx);
+	if (XCF == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+    }
+
+    for (j=0; j<nx; j++) {
+	/* get XCF for left-hand column/series and y */
+	xj = xcf_vec(xvec, yvec, p, NULL, T, err);
+	if (*err) {
+	    gretl_matrix_free(XCF);
+	    return NULL;
+	}
+
+	if (nx == 1) {
+	    XCF = xj;
+	    break;
+	} 
+
+	/* transcribe into big XCF matrix and free */
+	for (i=0; i<np; i++) {
+	    gretl_matrix_set(XCF, i, j, xj->val[i]);
+	}
+	gretl_matrix_free(xj);
+
+	/* move to next data read position */
+	if (j < nx - 1) {
+	    if (Xmat != NULL) {
+		xvec += Xmat->rows;
+	    } else {
+		xvec = Z[xlist[j+2]] + pdinfo->t1;
+	    }
+	}
+    }
+
+    return XCF;
+}
+
