@@ -40,9 +40,15 @@ int cum_series (const double *x, double *y,
 int resample_series (const double *x, double *y, 
 		     const DATAINFO *pdinfo);
 
+int block_resample_series (const double *x, double *y, int blocklen,
+			   const DATAINFO *pdinfo);
+
 int fracdiff_series (const double *x, double *y, double d,
 		     const DATAINFO *pdinfo);
 
+int boxcox_series (const double *x, double *y, double d,
+		   const DATAINFO *pdinfo);
+
 int filter_series(const double *x, double *y, const DATAINFO *pdinfo, 
 		  gretl_matrix *A, gretl_matrix *C, double y0);
 
@@ -83,4 +89,21 @@ int list_linear_combo (double *y, const int *list,
 		       const gretl_vector *b, const double **Z, 
 		       const DATAINFO *pdinfo);
 
+double imhof (const gretl_matrix *m, double arg, int *err);
+
+double dw_pval (const gretl_matrix *u, const gretl_matrix *X, 
+		double *pDW, int *err);
+
+gretl_matrix *multi_acf (const gretl_matrix *m, 
+			 const int *list, 
+			 const double **Z,
+			 const DATAINFO *pdinfo,
+			 int p, int *err);
+
+gretl_matrix *multi_xcf (const void *px, int xtype,
+			 const void *py, int ytype,
+			 const double **Z,
+			 const DATAINFO *pdinfo,
+			 int p, int *err);
+
 #endif /* GENFUNCS_H */
