@@ -21,7 +21,6 @@
 
 #include "genparse.h"
 #include "usermat.h"
-#include "loop_private.h"
 #include "gretl_func.h"
 #include "gretl_string_table.h"
 
@@ -75,7 +74,9 @@ struct str_table dvars[] = {
     { R_TEST_STAT, "$test" },
     { R_TEST_PVAL, "$pvalue" },
     { R_TEST_LNL,  "$rlnl" },
-    { R_INDEX,     "t" },
+    { R_KLNL,      "$kalman_lnl" },
+    { R_KS2,       "$kalman_s2" },
+    { R_KSTEP,     "$kalman_t" },
     { R_INDEX,     "obs" },
     { R_T1,        "$t1" },
     { R_T2,        "$t2" },
@@ -100,6 +101,9 @@ struct str_table mvars[] = {
     { M_BIC,     "$bic" },
     { M_HQC,     "$hqc" },
     { M_TRSQ,    "$trsq" },
+    { M_DWPVAL,  "$dwpval" },
+    { M_FSTT,    "$Fstat" },
+    { M_CHISQ,   "$chisq" },
     { M_UHAT,    "$uhat" },
     { M_YHAT,    "$yhat" },
     { M_AHAT,    "$ahat" },
@@ -124,6 +128,8 @@ struct str_table mvars[] = {
     { M_FCAST,   "$fcast" },
     { M_FCERR,   "$fcerr" },
     { M_COEFF_CI,"$coeff_ci" },
+    { M_KLLT,    "$kalman_llt" },
+    { M_KUHAT,   "$kalman_uhat" },
     { M_XLIST,   "$xlist" },
     { 0,         NULL }
 };
@@ -185,12 +191,13 @@ struct str_table funcs[] = {
     { F_RESAMPLE, "resample" },
     { F_PNOBS,    "pnobs" },     /* per-unit nobs in panels */
     { F_PMIN,     "pmin" },      /* panel min */
-    { F_PMAX,     "pmax" },       /* panel max */
+    { F_PMAX,     "pmax" },      /* panel max */
     { F_PMEAN,    "pmean" },     /* panel mean */
     { F_PSD,      "psd" },       /* panel std dev */
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
     { F_BKFILT,   "bkfilt" },    /* Baxter-King filter */
     { F_FRACDIFF, "fracdiff" },  /* fractional difference */
+    { F_BOXCOX,   "boxcox" },    /* Box-Cox transformation */
     { F_COV,      "cov" },
     { F_COR,      "corr" },
     { F_MOVAVG,   "movavg" },
@@ -198,6 +205,7 @@ struct str_table funcs[] = {
     { F_ZEROS,    "zeros" },
     { F_ONES,     "ones" },
     { F_SEQ,      "seq" },
+    { F_REPLACE,  "replace" },
     { F_MUNIF,    "muniform" },
     { F_MNORM,    "mnormal" },
     { F_SUMR,     "sumr" },
@@ -222,6 +230,7 @@ struct str_table funcs[] = {
     { F_MXTAB,    "mxtab" },
     { F_CDEMEAN,  "cdemean" },
     { F_CHOL,     "cholesky" },
+    { F_PSDROOT,  "psdroot" },
     { F_INV,      "inv" },
     { F_INVPD,    "invpd" },
     { F_GINV,     "ginv" },
@@ -264,11 +273,13 @@ struct str_table funcs[] = {
     { F_PVAL,     "pvalue" },
     { F_CRIT,     "critical" },
     { F_RANDGEN,  "randgen" },
+    { F_URCPVAL,  "urcpval" },
     { F_MAKEMASK, "makemask" },
     { F_VALUES,   "values" },
     { F_MSHAPE,   "mshape" },
     { F_SVD,      "svd" },
     { F_MOLS,     "mols" },
+    { F_MPOLS,    "mpols" },
     { F_MREAD,    "mread" },
     { F_MWRITE,   "mwrite" },
     { F_MRSEL,    "selifr" },
@@ -280,6 +291,9 @@ struct str_table funcs[] = {
     { F_WSD,      "wsd" },
     { F_XPX,      "xpx" },
     { F_FILTER,   "filter" },
+    { F_KFILTER,  "kfilter" },
+    { F_KSMOOTH,  "ksmooth" },
+    { F_KSIMUL,   "ksimul" },
     { F_TRIMR,    "trimr" },
     { F_GETENV,   "getenv" },
     { F_ARGNAME,  "argname" },
@@ -287,14 +301,23 @@ struct str_table funcs[] = {
     { F_READFILE, "readfile" },
     { F_BACKTICK, "grab" },
     { F_STRSTR,   "strstr" },
+    { F_STRCMP,   "strcmp" },
+    { F_STRNCMP,  "strncmp" },
     { F_STRLEN,   "strlen" },
     { F_VARNAME,  "varname" },
     { F_VARNUM,   "varnum" },
+    { F_TOLOWER,  "tolower" },
     { F_COLNAMES, "colnames" },
     { F_LJUNGBOX, "ljungbox" },
     { F_MSORTBY,  "msortby" },
     { F_LINCOMB,  "lincomb" },
-    { 0,        NULL }
+    { F_IMHOF,    "imhof" },
+    { F_TOEPSOLV, "toepsolv" },
+    { F_XMIN,     "xmin" },
+    { F_XMAX,     "xmax" },
+    { F_CORRGM,   "corrgm" },
+    { F_MCOVG,    "mcovg" },
+    { 0,          NULL }
 };
 
 struct str_table func_alias[] = {
@@ -568,33 +591,47 @@ int genr_function_word (const char *s)
 static void undefined_symbol_error (const char *s, parser *p)
 {
     parser_print_input(p);
+
     if (p->ch == '.') {
 	sprintf(gretl_errmsg, _("%s: no such object\n"), s);
     } else {
 	sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
     }
+
     p->err = E_UNKVAR;
 }
 
 static void function_noargs_error (const char *s, parser *p)
 {
     parser_print_input(p);
+
     pprintf(p->prn, _("'%s': no argument was given\n"), s);
     sprintf(gretl_errmsg, _("'%s': no argument was given\n"), s);
+
     p->err = 1;
 }
 
 void context_error (int c, parser *p)
 {
-    parser_print_input(p);
     if (c != 0) {
+	parser_print_input(p);
 	pprintf(p->prn, _("The symbol '%c' is not valid in this context\n"), c);
+	if (c == '&') {
+	    pputs(p->prn, _("(for logical AND, use '&&')\n"));
+	} else if (c == '|') {
+	    pputs(p->prn, _("(for logical OR, use '||')\n"));
+	}
     } else {
-	pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
-		getsymb(p->sym, p));
+	const char *s = getsymb(p->sym, p);
+
+	if (s != NULL && *s != '\0') {
+	    pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
+		    getsymb(p->sym, p));
+	}
     }
-    if (p->err == 0) {
-	p->err = 1;
+
+    if (!p->err) {
+	p->err = E_PARSE;
     }
 }
 
@@ -757,8 +794,7 @@ static void look_up_word (const char *s, parser *p)
 	    if (p->idnum > 0) {
 		p->sym = DUM;
 	    } else {
-		p->idnum = series_index(p->dinfo, s);
-		if (p->idnum < p->dinfo->v) {
+		if ((p->idnum = current_series_index(p->dinfo, s)) >= 0) {
 		    p->sym = USERIES;
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
@@ -785,6 +821,10 @@ static void look_up_word (const char *s, parser *p)
 			   varname_match_any(p->dinfo, s)) {
 		    p->sym = LIST;
 		    p->idstr = gretl_strdup(s);
+		} else if (!strcmp(s, "t")) {
+		    /* "t" as synonym for internal "obs" */
+		    p->sym = DVAR;
+		    p->idnum = R_INDEX;
 		} else {
 		    err = 1;
 		}
@@ -820,8 +860,11 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
 	    p->sym = DMSTR;
-	} else if (!func1_symb(p->sym) && !func2_symb(p->sym) &&
-		   !funcn_symb(p->sym) && p->sym != UFUN) {
+	} else if (!func1_symb(p->sym) && 
+		   !func2_symb(p->sym) &&
+		   !func3_symb(p->sym) &&
+		   !funcn_symb(p->sym) && 
+		   p->sym != UFUN) {
 	    p->err = 1;
 	} 
     } else if (p->ch == '[') {
@@ -905,14 +948,7 @@ static void getword (parser *p)
 	return;
     }
 
-    /* handle loop index scalar */
-    if (word[1] == '\0' && is_active_index_loop_char(word[0])) {
-	p->sym = LOOPIDX;
-	p->idstr = gretl_strdup(word);
-	return;
-    }
-
-    if ((*word == '$' && word[1]) || !strcmp(word, "t") || !strcmp(word, "obs")) {
+    if ((*word == '$' && word[1]) || !strcmp(word, "obs")) {
 	look_up_dollar_word(word, p);
     } else if (*word == '@') {
 	look_up_string_variable(word, p);
@@ -1333,8 +1369,7 @@ const char *getsymb (int t, const parser *p)
 	    return p->dinfo->varname[p->idnum];
 	} else if (t == USCALAR) {
 	    return p->idstr;
-	} else if (t == UMAT || t == UOBJ ||
-		   t == LOOPIDX) {
+	} else if (t == UMAT || t == UOBJ) {
 	    return p->idstr;
 	} else if (t == CON) {
 	    return constname(p->idnum);
