@@ -101,12 +101,16 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 static void gen_write_warning (const parser *p, PRN *prn)
 {
     if (prn != NULL) {
+	const char *w[] = {
+	    N_("missing values were generated"),
+	    N_("non-finite values were generated")
+	};
+	const char *s = (p->warn == E_MISSDATA)? w[0] : w[1];
+
 	if (*p->warning != '\0') {
-	    pprintf(prn, "%s: %s\n", 
-		    p->warning, _("missing values were generated"));
+	    pprintf(prn, "%s: %s\n", p->warning, _(s));
 	} else {
-	    pprintf(prn, "%s: %s\n", _("Warning"),
-		    _("missing values were generated"));
+	    pprintf(prn, "%s: %s\n", _("Warning"), _(s));
 	}
     }
 }
@@ -258,59 +262,56 @@ int is_gretl_function_call (const char *s)
     }
 }
 
-/* "reserved word" data */
+/* "reserved word" data: 
 
-static const char *res1[] = {
+   smpl is special because it can legitimately be followed
+   by '+' or '-' and hence may be confused with the name
+   of a variable in parsing a command line: consider
+   "smpl +3 ;" (command) versus "smpl += 3" (revision of
+   value of variable).
+*/
+
+static const char *reswords[] = {
+    /* constants */
     "const",
     "CONST",
     "pi",
     "NA",
-    "null"
-};
-
-static const char *res2[] = {
-    "t",
-    "obs",
+    "null",
+    "obs", /* not exactly a constant, but hey */
+    /* types */
     "scalar",
     "series",
-    "matrix"
+    "matrix",
+    "string",
+    "list",
+    "kalman"
 };
 
 /**
  * gretl_reserved_word:
  * @str: string to be tested.
  *
- * Returns non-zero if @str is a reserved word that cannot 
+ * Returns: non-zero if @str is a reserved word that cannot 
  * figure as the name of a user-defined variable, otherwise 0.
  */
 
 int gretl_reserved_word (const char *str)
 {
-    const char *ruses[] = {
-	N_("a constant"),
-	N_("an internal variable")
-    };
-    static int n1 = sizeof res1 / sizeof res1[0];
-    static int n2 = sizeof res2 / sizeof res2[0];
-    int i, ret = 0;
+    static int n = sizeof reswords / sizeof reswords[0];
+    int i, ret = gretl_command_number(str);
 
-    for (i=0; i<n1 && !ret; i++) {
-	if (!strcmp(str, res1[i])) {
+    for (i=0; i<n && !ret; i++) {
+	if (!strcmp(str, reswords[i])) {
 	    ret = 1;
 	}
     }
 
-    for (i=0; i<n2 && !ret; i++) {
-	if (!strcmp(str, res2[i])) {
-	    ret = 2;
-	}
-    }
+    if (ret) {
+	gretl_errmsg_sprintf(_("'%s' may not be used as a "
+			       "variable name"), str);
+    }	
 
-    if (ret > 0) {
-	sprintf(gretl_errmsg, _("'%s' refers to %s and may not be used as a "
-			    "variable name"), str, _(ruses[ret - 1])); 
-    }
- 
     return ret;
 }
 
@@ -381,8 +382,13 @@ static int try_for_listvar (const DATAINFO *pdinfo, const char *s)
 int series_index (const DATAINFO *pdinfo, const char *varname)
 {
     const char *s = varname;
-    int i, fd;
-    int ret = pdinfo->v;
+    int i, fd, ret;
+
+    if (pdinfo == NULL) {
+	return -1;
+    }
+
+    ret = pdinfo->v;
 
     if (s == NULL || *s == 0 || isdigit(*s)) {
 	return ret;
@@ -403,7 +409,10 @@ int series_index (const DATAINFO *pdinfo, const char *varname)
 	    if (!strcmp(pdinfo->varname[i], s)) {
 		if (lists_protected() && var_is_listarg(pdinfo, i)) {
 		    /* variable is not visible by name in context */
+		    ;
+#if 0
 		    fprintf(stderr, "var %d (%s) is invisible\n", i, s);
+#endif
 		} else {
 		    ret = i;
 		    break;
@@ -420,6 +429,20 @@ int series_index (const DATAINFO *pdinfo, const char *varname)
     return ret;
 }
 
+int current_series_index (const DATAINFO *pdinfo, const char *vname)
+{
+    int v = -1;
+
+    if (pdinfo != NULL) {
+	v = series_index(pdinfo, vname);
+	if (v >= pdinfo->v) {
+	    v = -1;
+	}
+    }
+
+    return v;
+}
+
 int gretl_is_series (const char *name, const DATAINFO *pdinfo)
 {
     if (pdinfo == NULL) {
@@ -575,6 +598,13 @@ static int is_gen_special (const char *s, char *spec, const char **rem)
     return 0;
 }
 
+static int genr_last_type;
+
+int genr_get_last_output_type (void)
+{
+    return genr_last_type;
+}
+
 #define gen_verbose(f) (!(f & P_PRINT) && \
                         !(f & P_DISCARD) && \
                         !(f & P_PRIVATE) && \
@@ -587,8 +617,7 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 {
     char vname[VNAMELEN];
     const char *subline = NULL;
-    int oldv = pdinfo->v;
-    int flags = 0;
+    int oldv, flags = 0;
     parser p;
 
     if (opt & OPT_P) {
@@ -603,6 +632,8 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	flags |= P_QUIET;
     }
 
+    oldv = (pdinfo != NULL)? pdinfo->v : 0;
+
 #if GDEBUG
     fprintf(stderr, "\n*** generate: line = '%s'\n", line);
 #endif
@@ -630,10 +661,17 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!p.err && p.warn && gretl_warnings_on()) {
+    if (!p.err && p.warn && gretl_warnings_on() && !(flags & P_PRIVATE)) {
 	gen_write_warning(&p, prn);
     }
 
+    genr_last_type = genr_get_output_type(&p);
+
+    if (p.err == 1) {
+	/* a fairly good guess? */
+	p.err = E_PARSE;
+    }
+
     gen_cleanup(&p);
 
 #if GDEBUG
@@ -661,6 +699,8 @@ double generate_scalar (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -671,12 +711,13 @@ double generate_scalar (const char *s, double ***pZ,
 /* retrieve a series result directly */
 
 double *generate_series (const char *s, double ***pZ, 
-			 DATAINFO *pdinfo, int *err)
+			 DATAINFO *pdinfo, PRN *prn,
+			 int *err)
 {
     parser p;
     double *x = NULL;
 
-    *err = realgen(s, &p, pZ, pdinfo, NULL, P_SERIES | P_PRIVATE);
+    *err = realgen(s, &p, pZ, pdinfo, prn, P_SERIES | P_PRIVATE);
 
     if (!*err) {
 	NODE *n = p.ret;
@@ -692,6 +733,8 @@ double *generate_series (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -737,6 +780,8 @@ gretl_matrix *generate_matrix (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -760,6 +805,8 @@ char *generate_string (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -967,16 +1014,16 @@ double genr_get_output_scalar (const parser *p)
     }
 }
 
-const char *genr_get_formula (const GENERATOR *genr)
-{
-    return genr->input;
-}
-
 int genr_is_print (const parser *p)
 {
     return (p->flags & P_PRINT);
 }
 
+int genr_is_autoregressive (const parser *p)
+{
+    return (p->flags & P_AUTOREG);
+}
+
 void genr_set_na_check (parser *p)
 {
     p->flags |= P_NATEST;
