@@ -35,6 +35,9 @@ typedef enum {
     R_TEST_STAT,  /* test statistic from last explicit test performed */
     R_TEST_PVAL,  /* p-value from last explicit test performed */
     R_TEST_LNL,   /* log-likelihood from last test (if applicable) */
+    R_KLNL,       /* log-likelihood from Kalman filter (if applicable) */
+    R_KS2,        /* variance estimate from Kalman filter (if applicable) */
+    R_KSTEP,      /* current Kalman time-step (if applicable) */
     R_STOPWATCH,  /* stopwatch */ 
     R_NSCAN,      /* number of items scanned via sscanf */
     R_SCALAR_MAX, /* separator: scalars vs series */
@@ -56,12 +59,16 @@ typedef enum {
     M_BIC,        /* Bayesian info criterion */
     M_HQC,        /* Hannan-Quinn criterion */
     M_TRSQ,       /* T * R-squared, last model */
+    M_DWPVAL,     /* Durbin-Watson p-value, last model */
+    M_FSTT,       /* overall F-statistic, last model */
+    M_CHISQ,      /* overall chi-square stat, last model */   
     M_SCALAR_MAX, /* -- separator, scalars/series -- */
     M_UHAT,       /* residuals */
     M_YHAT,       /* fitted values */
     M_AHAT,       /* per-unit intercepts in panel model */
     M_H,          /* GARCH predicted variances */
     M_SAMPLE,     /* observations used in estimation */
+    M_UHAT2,      /* squared residuals */
     M_SERIES_MAX, /* -- separator, series/matrices -- */
     M_COEFF,      /* parameter estimates */
     M_SE,         /* parameter standard errors */
@@ -82,6 +89,8 @@ typedef enum {
     M_FCAST,      /* last forecast generated via fcasterr */
     M_FCERR,      /* standard errors associated with M_FCAST */
     M_COEFF_CI,   /* (asymmetric) confidence intervals for coeffs */
+    M_KLLT,       /* Kalman log-likelihood, per time-step */
+    M_KUHAT,      /* Kalman: current prediction error */
     M_MATRIX_MAX, /* -- separator, matrices/lists -- */
     M_XLIST,      /* list of regressors */
     M_MAX         /* sentinel */
@@ -109,18 +118,20 @@ void genr_set_loopline (GENERATOR *genr, int i);
 
 int genr_get_loopline (GENERATOR *genr);
 
-const char *genr_get_formula (const GENERATOR *genr);
-
 int genr_get_output_type (const GENERATOR *genr);
 
 int genr_get_output_varnum (const GENERATOR *genr);
 
 double genr_get_output_scalar (const GENERATOR *genr);
 
+int genr_get_last_output_type (void);
+
 gretl_matrix *genr_get_output_matrix (const GENERATOR *genr);
 
 int series_index (const DATAINFO *pdinfo, const char *varname);
 
+int current_series_index (const DATAINFO *pdinfo, const char *vname);
+
 int extract_varname (char *targ, const char *src, int *len);
 
 int genr_fit_resid (const MODEL *pmod, 
@@ -131,7 +142,8 @@ double generate_scalar (const char *s, double ***pZ,
 			DATAINFO *pdinfo, int *err);
 
 double *generate_series (const char *s, double ***pZ, 
-			 DATAINFO *pdinfo, int *err);
+			 DATAINFO *pdinfo, PRN *prn,
+			 int *err);
 
 gretl_matrix *generate_matrix (const char *s, double ***pZ, 
 			       DATAINFO *pdinfo, int *err);
@@ -156,6 +168,8 @@ int genr_function_word (const char *s);
 
 int genr_is_print (const GENERATOR *p);
 
+int genr_is_autoregressive (const GENERATOR *p);
+
 void genr_set_na_check (GENERATOR *genr);
 
 void genr_unset_na_check (GENERATOR *genr);
