@@ -89,7 +89,7 @@ enum {
 	      OBS,	  /* observation from a series */
 	      MSL,	  /* matrix plus subspec */
   /* 60 */    DMSL,	  /* "dollar" matrix plus subspec */
-	      DMSTR,	  /* "dollar" matrix plus old-style string subspec */
+	      DMSTR,	  /* "dollar" matrix plus string subspec */
 	      MSL2,	  /* unevaluated matrix subspec */
 	      MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
@@ -98,8 +98,7 @@ enum {
 	      DVAR,	  /* $ dataset variable (scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
 	      OVAR,	  /* object variable: variable "under" an object */
-  /* 70 */    LOOPIDX,	  /* loop index variable */
-	      LIST,	  /* reference to named list */
+  /* 70 */    LIST,	  /* reference to named list */
 	      LISTVAR,    /* variable in list */
 	      STR,	  /* string */
 	      FARGS,	  /* set of n function arguments */
@@ -108,8 +107,8 @@ enum {
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
 	      EROOT,	  /* dummy root for (...) expression */
               UFUN,	  /* user-defined function */
-  /* 80 */    VSTR,       /* string variable */
-	      INC,   
+              VSTR,       /* string variable */
+  /* 80 */    INC,   
 	      DEC,
 	      QUERY,
 	      EOT,	  /* end of transmission */
@@ -167,7 +166,6 @@ enum {
     F_LNGAMMA,
     F_HPFILT,
     F_BKFILT,
-    F_RESAMPLE,
     F_PNOBS,
     F_PMIN,
     F_PMAX,
@@ -183,6 +181,7 @@ enum {
     F_MCORR,
     F_CDEMEAN,
     F_CHOL,
+    F_PSDROOT,
     F_INV,
     F_INVPD,
     F_GINV,
@@ -211,7 +210,8 @@ enum {
     F_PVAL,
     F_CDF,
     F_INVCDF,
-    F_CRIT,  
+    F_CRIT, 
+    F_URCPVAL,
     F_RANDGEN,
     F_RPOISSON,
     F_MAKEMASK,
@@ -241,6 +241,7 @@ enum {
     F_STRLEN,
     F_VARNAME,
     F_VARNUM,
+    F_TOLOWER,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -249,9 +250,9 @@ enum {
     F_RUNIFORM,
     F_RNORMAL,
     F_FRACDIFF,
+    F_BOXCOX,
     F_ZEROS,
     F_ONES,
-    F_SEQ,
     F_MUNIF,
     F_MNORM,
     F_QFORM,
@@ -276,16 +277,32 @@ enum {
     F_WVAR,
     F_WSD,
     F_STRSTR,
+    F_STRCMP,
     F_COLNAMES,
     F_LJUNGBOX,
     F_MSORTBY,
     F_LINCOMB,
+    F_IMHOF,
+    F_XMIN,
+    F_XMAX,
+    F_RESAMPLE,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
     F_MSHAPE,
     F_SVD,
     F_MOLS,
-    F_FILTER,
+    F_MPOLS,
     F_TRIMR,
+    F_TOEPSOLV,
+    F_CORRGM,
+    F_SEQ,
+    F_REPLACE,
+    F_STRNCMP,
+    F3_MAX,       /* SEPARATOR: end of three-arg functions */
+    F_FILTER,
+    F_MCOVG,
+    F_KFILTER,
+    F_KSMOOTH,
+    F_KSIMUL,
     FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
@@ -308,7 +325,8 @@ enum {
 
 #define func1_symb(s) (s > F1_MIN && s < F1_MAX)
 #define func2_symb(s) (s > F1_MAX && s < F2_MAX)
-#define funcn_symb(s) (s > F2_MAX && s < FN_MAX)
+#define func3_symb(s) (s > F2_MAX && s < F3_MAX)
+#define funcn_symb(s) (s > F3_MAX && s < FN_MAX)
 
 #define string_arg_func(s) (s == F_ISSERIES || s == F_ISNULL || \
 			    s == F_ISLIST || s == F_LISTLEN || \
@@ -318,10 +336,12 @@ enum {
 #define string0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
                          s == F_CRIT || s == F_RANDGEN || s == F_PDF)
 
-#define unary_op(s) (s >= 1 && s < U_MAX)
+#define unary_op(s)  (s >= 1 && s < U_MAX)
 #define binary_op(s) (s > U_MAX && s < OP_MAX)
 #define bool_comp(s) (s >= B_EQ && s <= B_OR)
 
+#define evalb3(s) (func3_symb(s)) /* plus ternary op? */
+
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
                    s == MSL2 || s == SUBSL || s == LAG || \
                    s == OBS)
@@ -335,21 +355,26 @@ enum {
 #define b2sym(s) (evalb2(s) || s == DMSTR || s == DMSL || \
                   s == OVAR || s == UFUN || s == LISTVAR)
 
-#define b3sym(s) (s == QUERY)
+#define b3sym(s) (s == QUERY || func3_symb(s))
 
 #define bnsym(s) (s == MDEF || s == FARGS)
 
 #define freestr(s) (s == STR || s == USCALAR || s == UMAT || s == UOBJ || \
-                    s == LOOPIDX || s == LIST || s == VSTR)
+                    s == LIST || s == VSTR)
 
 #define bare_data_type(s) (s > PUNCT_MAX && s < DTYPE_MAX)
 
 #define closing_sym(s) (s == G_RPR || s == G_RBR || s == G_RCB)
 
 
-/* functions where the right-hand "argument" is actually a return
+/* functions where the right-hand argument is actually a return
    location */
-#define r_return(s) (s == F_QR || s == F_EIGSYM || s == F_EIGGEN)
+#define r_return(s) (s == F_QR || s == F_EIGSYM || s == F_EIGGEN || \
+                     s == F_MOLS || s == F_MPOLS || s == F_SVD)
+
+/* functions where the middle argument is actually a return
+   location */
+#define m_return(s) (s == F_SVD)
 
 #define dollar_node(n) (n->t == DVAR || n->t == MVAR || \
                         (n->t == MSL && n->v.b2.l->v.str[0] == '$'))
@@ -456,6 +481,7 @@ struct parser_ {
     int targ;          /* target type */
     int op;            /* assignment operator (possibly inflected) */
     struct lhinfo lh;  /* left-hand side info */
+    parser *subp;      /* left-hand side matrix subslice tree */
     NODE *tree;        /* parsed syntax tree */
     NODE *ret;         /* evaluated result node */
     NODE **aux;        /* auxiliary nodes used in evaluation */
