@@ -36,8 +36,8 @@ static NODE *powterm (parser *p);
 #if SDEBUG
 static void notify (const char *s, NODE *t, parser *p)
 {
-    fprintf(stderr, "%-8s: returning node at %p, err = %d\n", 
-	    s, (void *) t, p->err);
+    fprintf(stderr, "%-8s: returning node at %p (type %d), err = %d\n", 
+	    s, (void *) t, (t != NULL)? t->t : 0, p->err);
 }
 #endif
 
@@ -75,8 +75,7 @@ static NODE *newref (parser *p, int t)
 	    n->v.xvec = (*p->Z)[n->vnum];
 	} else {
 	    n->t = t;
-	    if (t == USCALAR || t == UMAT || t == UOBJ || 
-		t == LOOPIDX || t == LIST) {
+	    if (t == USCALAR || t == UMAT || t == UOBJ || t == LIST) {
 		n->v.str = p->idstr;
 	    } else {
 		n->v.idnum = p->idnum;
@@ -181,7 +180,7 @@ static NODE *newb2 (int t, NODE *l, NODE *r)
 
 /* node for ternary operator */
 
-static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
+static NODE *newb3 (int t, NODE *l)
 {  
     NODE *n = malloc(sizeof *n);
 
@@ -193,8 +192,8 @@ static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
     if (n != NULL) {
 	n->t = t;
 	n->v.b3.l = l;
-	n->v.b3.m = m;
-	n->v.b3.r = r;
+	n->v.b3.m = NULL;
+	n->v.b3.r = NULL;
 	n->flags = 0;
 	n->vnum = NO_VNUM;
     }
@@ -252,15 +251,20 @@ static void expected_symbol_error (int c, parser *p)
     const char *found = getsymb(p->sym, p);
 
     parser_print_input(p);
-    pprintf(p->prn, _("Expected '%c' but found '%s'\n"), c, 
-	    found);
-    if (found != NULL) {
-	if (!strcmp(found, "&")) {
-	    pputs(p->prn, "(for logical AND, please use \"&&\")\n");
-	} else if (!strcmp(found, "|")) {
-	    pputs(p->prn, "(for logical OR, please use \"||\")\n");
-	}
-    }   
+
+    if (*found == '\0') {
+	pprintf(p->prn, _("Expected '%c' but formula ended\n"), c);
+    } else {
+	pprintf(p->prn, _("Expected '%c' but found '%s'\n"), c, 
+		found);
+    }
+
+    if (!strcmp(found, "&")) {
+	pputs(p->prn, "(for logical AND, please use \"&&\")\n");
+    } else if (!strcmp(found, "|")) {
+	pputs(p->prn, "(for logical OR, please use \"||\")\n");
+    }
+
     p->err = 1;
 }
 
@@ -324,7 +328,6 @@ static NODE *base (parser *p, NODE *up)
     case MVAR:
     case OBS:
     case LIST:
-    case LOOPIDX:
 	t = newref(p, p->sym);
 	if (matrix_ref_node(p) && unary_apost(p)) {
 	    set_transpose(t);
@@ -541,58 +544,9 @@ static NODE *get_string_arg (parser *p)
 }
 
 enum {
-    BOTH_OPT = 1,
-    RIGHT_OPT,
-    RIGHT_STR
+    RIGHT_STR = 1
 };
 
-/* gather an unknown number of comma-separated arguments
-   for a (possibly user-defined) function */
-
-static void get_multi_args (NODE *t, parser *p)
-{
-    NODE *n;
-    char cexp = 0;
-
-#if SDEBUG
-    fprintf(stderr, "get_multi_args, p->sym = %d\n", p->sym);
-#endif    
-
-    if (p->sym == G_LPR) {
-	lex(p);
-	while (p->ch != 0 && !p->err) {
-	    n = expr(p);
-	    if (p->err) {
-		break;
-	    } else {
-		p->err = push_bn_node(t, n);
-	    }
-	    if (p->sym == P_COM) {
-		/* in case acceptance of plain strings was set,
-		   turn it off after the first arg */
-		p->flags &= ~P_GETSTR;
-		lex(p);
-	    } else if (p->sym == G_RPR) {
-		break;
-	    }
-	}
-    } else {
-	cexp = '(';
-    }
-
-    if (cexp == 0 && !p->err) {
-	if (p->sym == G_RPR) {
-	    lex(p);
-	} else {
-	    unmatched_symbol_error('(', p);
-	}
-    }
-	    
-    if (cexp && p->err == 0) {
-	expected_symbol_error(cexp, p);
-    }
-}
-
 static void get_matrix_def (NODE *t, parser *p, int *sub)
 {
     NODE *n;
@@ -640,7 +594,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	}
     }
 	    
-    if (cexp && p->err == 0) {
+    if (cexp && !p->err) {
 	expected_symbol_error(cexp, p);
     }
 }	
@@ -716,67 +670,123 @@ static void get_slice_parts (NODE *t, parser *p)
 	cexp = '[';
     }
 	    
-    if (cexp && p->err == 0) {
+    if (cexp && !p->err) {
 	expected_symbol_error(cexp, p);
     }
 
     set_matrix_slice_off();
 }
 
-/* get up to two comma-separated arguments 
-   (possibly optional) */
-
-static void get_args (NODE *t, parser *p, int opt)
+static void attach_child (NODE *parent, NODE *child, int k, int i,
+			  parser *p)
 {
+#if SDEBUG
+    fprintf(stderr, "attach_child: i=%d, type = %d\n", i, child->t);
+#endif
+
+    if (p->err) {
+	return;
+    }
+
+    if (k == 2) {
+	/* 2-place node */
+	if (i == 0) {
+	    parent->v.b2.l = child;
+	} else {
+	    parent->v.b2.r = child;
+	}
+    } else if (k == 3) {
+	/* 3-place node */
+	if (i == 0) {
+	    parent->v.b3.l = child;
+	} else if (i == 1) {
+	    parent->v.b3.m = child;
+	} else {
+	    parent->v.b3.r = child;
+	}
+    } else {
+	/* n-place node */
+	p->err = push_bn_node(parent, child);
+    }
+}
+
+static void pad_parent (NODE *parent, int k, int i, parser *p)
+{
+    NODE *n;
+    int j;
+
+    for (j=i; j<k; j++) {
+	n = newempty(EMPTY);
+	attach_child(parent, n, k, j, p);
+    }
+}
+
+/* Get up to k comma-separated arguments (possibly optional).
+   However, if k < 0 this is a signal for get as many arguments as we
+   can find (the number is unknown in advance).
+*/
+
+static void get_args (NODE *t, parser *p, int k, int opt, int *next)
+{
+    NODE *child;
     char cexp = 0;
+    int i = 0;
 
 #if SDEBUG
-    fprintf(stderr, "get_args...\n");
+    fprintf(stderr, "get_args: k = %d...\n", k);
 #endif    
 
-    if (p->sym == G_LPR) {
-	lex(p);
-	if (p->sym == G_RPR && opt == BOTH_OPT) {
-	    /* no args, but it's OK */
-	    t->v.b2.l = newempty(EMPTY);
-	    t->v.b2.r = newempty(EMPTY);
-	    lex(p);
-	    return;
+    if (p->sym != G_LPR) {
+	expected_symbol_error('(', p);
+	return;
+    }	
+
+    lex(p);
+
+    while (((k > 0 && i < k) || p->ch) && !p->err) {
+	if (p->sym == G_RPR) {
+	    break;
 	}
-	t->v.b2.l = expr(p);
-	if (p->sym == G_RPR && opt == RIGHT_OPT) {
-	    /* no second arg, but it's OK */
-	    t->v.b2.r = newempty(EMPTY);
-	    lex(p);
-	    return;
-	}	    
-	if (p->sym == P_COM) {
-	    if (opt == RIGHT_STR) {
-		/* the second arg must be a literal string */
-		t->v.b2.r = get_string_arg(p);
+	child = expr(p);
+	attach_child(t, child, k, i++, p);
+	if (p->err) {
+	    break;
+	}
+	if (p->sym == G_RPR) {
+	    break;
+	} else if (p->sym == P_COM) {
+	    /* turn off flag for accepting string as first arg */
+	    p->flags &= ~P_GETSTR;
+	    /* and handle final string arg if relevant */
+	    if (i == k - 1 && opt == RIGHT_STR) {
+		child = get_string_arg(p);
+		attach_child(t, child, k, i++, p);
 	    } else {
 		lex(p);
-		t->v.b2.r = expr(p);
-		if (p->sym == G_RPR) {
-		    if (p->ch == '\'') {
-			set_transpose(t);
-			parser_getc(p);
-		    }
-		    lex(p);
-		} else {
-		    cexp = ')';
-		}
 	    }
 	} else {
 	    cexp = ',';
 	}
-    } else {
-	cexp = '(';
-    }
-	    
-    if (cexp && p->err == 0) {
-	expected_symbol_error(cexp, p);
     }
+
+    if (!p->err) {
+	if (cexp) {
+	    expected_symbol_error(cexp, p);
+	} else if (p->sym == G_RPR) {
+	    if (i < k) {
+		pad_parent(t, k, i, p);
+	    } 
+	    /* handle trailing transpose */
+	    if (p->ch == '\'') {
+		set_transpose(t);
+		parser_getc(p);
+	    }
+	    lex(p);
+	    if (p->sym == G_LBR) {
+		*next = G_LBR;
+	    }
+	}
+    }	
 }
 
 static void get_ovar_ref (NODE *t, parser *p)
@@ -796,8 +806,8 @@ static void get_ovar_ref (NODE *t, parser *p)
 
     if (p->idnum == 0) {
 	p->err = E_PARSE;
-    } else if (p->sym == DMSL) {
-	/* followed by '[' slice mechanism? */
+    } else if (p->sym == DMSL || p->sym == DMSTR) {
+	/* followed by '[' or '(' matrix subspec? */
 	t->v.b2.r = powterm(p);
     } else {
 	t->v.b2.r = newref(p, p->sym);
@@ -809,9 +819,9 @@ static NODE *powterm (parser *p)
 { 
     /* watch out for unary operators */
     int sym = p->sym == B_SUB ? U_NEG : 
-	p->sym == B_ADD ? U_POS :
-	p->sym;
+	p->sym == B_ADD ? U_POS : p->sym;
     int opt = OPT_NONE;
+    int next = 0;
     NODE *t;
 
     if (p->err) {
@@ -823,13 +833,9 @@ static NODE *powterm (parser *p)
 	    p->sym, p->ch? p->ch : '0', p->ch);
 #endif
 
-    if (sym == F_RUNIFORM || sym == F_RNORMAL) {
-	opt = BOTH_OPT;
-    } else if (sym == F_FDJAC || sym == F_BFGSMAX) {
+    if (sym == F_FDJAC || sym == F_BFGSMAX) {
 	opt = RIGHT_STR;
-    } else if (sym == F_DUMIFY) {
-	opt = RIGHT_OPT;
-    }
+    } 
 
     if (unary_op(sym)) {
 	if (p->ch == 0) {
@@ -845,7 +851,13 @@ static NODE *powterm (parser *p)
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    get_args(t, p, opt);
+	    get_args(t, p, 2, opt, &next);
+	}
+    } else if (func3_symb(sym)) {
+	t = newb3(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    get_args(t, p, 3, opt, &next);
 	}
     } else if (string0_func(sym)) {
 	t = newb1(sym, NULL);
@@ -854,7 +866,16 @@ static NODE *powterm (parser *p)
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
 		p->flags |= P_GETSTR;
-		get_multi_args(t->v.b1.b, p);
+		get_args(t->v.b1.b, p, -1, opt, &next);
+	    }
+	}
+    } else if (sym == F_URCPVAL) {
+	t = newb1(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    t->v.b1.b = newbn(FARGS);
+	    if (t != NULL) {
+		get_args(t->v.b1.b, p, -1, opt, &next);
 	    }
 	}	
     } else if (string_arg_func(sym)) {
@@ -868,6 +889,9 @@ static NODE *powterm (parser *p)
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = base(p, t);
+	    if (p->sym == G_LBR) {
+		next = G_LBR;
+	    }
 	}
     } else if (sym == LAG || sym == OBS) {
 	t = newb2(sym, NULL, NULL);
@@ -876,12 +900,6 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
-    } else if (func2_symb(sym)) {
-	t = newb2(sym, NULL, NULL);
-	if (t != NULL) {
-	    lex(p);
-	    get_args(t, p, opt);
-	}
     } else if (sym == MSL || sym == DMSL) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
@@ -954,7 +972,7 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b2.r = newbn(FARGS);
 	    if (t != NULL) {
-		get_multi_args(t->v.b2.r, p);
+		get_args(t->v.b2.r, p, -1, opt, &next);
 	    }
 	}
     } else if (funcn_symb(sym)) {
@@ -963,7 +981,7 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		get_multi_args(t->v.b1.b, p);
+		get_args(t->v.b1.b, p, -1, opt, &next);
 	    }
 	}
     } else if (sym == STR || sym == VSTR) {
@@ -973,6 +991,17 @@ static NODE *powterm (parser *p)
 	t = base(p, NULL);
     }
 
+    if (next == G_LBR) {
+	/* support func(args)[slice] */
+	t = newb2(MSL, t, NULL);
+	if (t != NULL) {
+	    t->v.b2.r = newb2(MSL2, NULL, NULL);
+	    if (t->v.b2.r != NULL) {
+		get_slice_parts(t->v.b2.r, p);
+	    }
+	}	
+    }
+
 #if SDEBUG
     notify("powterm", t, p);
 #endif
@@ -980,11 +1009,37 @@ static NODE *powterm (parser *p)
     return t;
 }
 
+/* convert B_POW to right associativity: that is,
+   
+       pow           pow
+      L   R         L   R
+      |   |   ->    |   |
+     pow  c         a  pow
+     | |               | |
+     a b               b c
+
+*/
+
+static void convert_pow_term (NODE *n)
+{
+    NODE *L = n->v.b2.l;
+    NODE *a = L->v.b2.l;
+    NODE *b = L->v.b2.r;
+    NODE *c = n->v.b2.r;
+
+    n->v.b2.l = a;
+    n->v.b2.r = L;
+    L->v.b2.l = b;
+    L->v.b2.r = c;
+}
+
+#define pow_sym(t)     (t == B_POW || t == B_DOTPOW)
+#define pow_pow_node(n) (pow_sym(n->t) && pow_sym(n->v.b2.l->t))
+
 static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
-	p->sym == B_ADD ? U_POS : 
-	p->sym;
+	p->sym == B_ADD ? U_POS : p->sym;
     NODE *t;
 
     if (p->err) {
@@ -1016,6 +1071,10 @@ static NODE *factor (parser *p)
 			lex(p);
 			t->v.b2.r = powterm(p);
 		    }
+		    if (!p->err && pow_pow_node(t)) {
+			/* make exponentiation associate rightward */
+			convert_pow_term(t);
+		    }
 		}
 	    }
 	}
@@ -1183,7 +1242,7 @@ NODE *expr (parser *p)
     }
 
     while (!p->err && p->sym == QUERY) {
-	t = newb3(p->sym, t, NULL, NULL);
+	t = newb3(p->sym, t);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b3.m = expr(p);
