@@ -43,17 +43,16 @@ typedef enum {
     GPT_Y2AXIS         = 1 << 13, /* plot has second y-axis */
     GPT_AUTO_FIT       = 1 << 14, /* automatic (OLS) fitted line was added */
     GPT_FIT_HIDDEN     = 1 << 15, /* autofit line calculated, but suppressed */
-    GPT_MINIMAL_BORDER = 1 << 16, /* omitting top and right borders */
-    GPT_PNG_OUTPUT     = 1 << 17, /* output is to PNG file */
-    GPT_ALL_MARKERS    = 1 << 18, /* all observation markers displayed */
-    GPT_ALL_MARKERS_OK = 1 << 19, /* OK to show all observation markers */
-    GPT_LETTERBOX      = 1 << 20, /* special format for time series graphs */
-    GPT_PARAMETRIC     = 1 << 21, /* gnuplot should be in parametric mode */
-    GPT_XZEROAXIS      = 1 << 22, /* show x = 0 line */
-    GPT_YZEROAXIS      = 1 << 23, /* show y = 0 line */
-    GPT_FILL_SWITCH    = 1 << 24, /* switching from errorbars to fill */
-    GPT_ERR_SWITCH     = 1 << 25, /* switching from fill to errorbars */
-    GPT_MONO           = 1 << 26  /* monochrome output */
+    GPT_PNG_OUTPUT     = 1 << 16, /* output is to PNG file */
+    GPT_PRINT_MARKERS  = 1 << 17, /* print observation markers */
+    GPT_MARKERS_OK     = 1 << 18, /* OK to print observation markers */
+    GPT_LETTERBOX      = 1 << 19, /* special format for time series graphs */
+    GPT_PARAMETRIC     = 1 << 20, /* gnuplot should be in parametric mode */
+    GPT_XZEROAXIS      = 1 << 21, /* show x = 0 line */
+    GPT_YZEROAXIS      = 1 << 22, /* show y = 0 line */
+    GPT_FILL_SWITCH    = 1 << 23, /* switching from errorbars to fill */
+    GPT_ERR_SWITCH     = 1 << 24, /* switching from fill to errorbars */
+    GPT_MONO           = 1 << 25  /* monochrome output */
 } GptFlags; 
 
 typedef struct gretlRGB_ gretlRGB;
@@ -65,8 +64,7 @@ struct gretlRGB_ {
 };
 
 #define MAXTITLE 128
-#define MAX_PLOT_LABELS 3
-#define N_GP_COLORS 4
+#define N_GP_COLORS 7
 #define BOXCOLOR (N_GP_COLORS - 1)
 
 typedef enum {
@@ -95,6 +93,8 @@ typedef enum {
     PLOT_BI_GRAPH,
     PLOT_MANY_TS,
     PLOT_RQ_TAU,
+    PLOT_BOXPLOTS,
+    PLOT_CURVE,
     PLOT_TYPE_MAX
 } PlotType;
 
@@ -152,6 +152,10 @@ void gnuplot_missval_string (FILE *fp);
 
 int gnuplot_init (PlotType ptype, FILE **fpp);
 
+FILE *gnuplot_batch_init (int *err);
+
+int specified_gp_output_format (void);
+
 int write_plot_type_string (PlotType ptype, FILE *fp);
 
 void write_plot_line_styles (int ptype, FILE *fp);
@@ -164,6 +168,9 @@ int gnuplot_make_graph (void);
 
 void reset_plot_count (void);
 
+int matrix_plot (gretl_matrix *m, const int *list, const char *literal, 
+		 gretlopt opt);
+
 int gnuplot (const int *plotlist, const char *literal,
 	     const double **Z, const DATAINFO *pdinfo, 
 	     gretlopt opt);
@@ -189,10 +196,8 @@ int
 gretl_panel_ts_plot (const int *list, const double **Z, DATAINFO *pdinfo,
 		     gretlopt opt);
 
-int plot_fcast_errs (int t1, int t2, const double *obs, 
-		     const double *depvar, const double *yhat, 
-		     const double *maxerr, const char *varname, 
-		     int tsfreq, gretlopt opt);
+int plot_fcast_errs (const FITRESID *fr, const double *maxerr,
+		     const DATAINFO *pdinfo, gretlopt opt);
 
 int plot_tau_sequence (const MODEL *pmod, const DATAINFO *pdinfo,
 		       int k);
@@ -200,11 +205,13 @@ int plot_tau_sequence (const MODEL *pmod, const DATAINFO *pdinfo,
 int 
 gretl_VAR_plot_impulse_response (GRETL_VAR *var,
 				 int targ, int shock, int periods,
+				 double alpha,
 				 const double **Z,
 				 const DATAINFO *pdinfo);
 
 int 
 gretl_VAR_plot_multiple_irf (GRETL_VAR *var, int periods,
+			     double alpha,
 			     const double **Z,
 			     const DATAINFO *pdinfo);
 
@@ -252,6 +259,8 @@ void graph_palette_reset (int i);
 
 void print_rgb_hash (char *s, const gretlRGB *color);
 
+void gretl_rgb_get (gretlRGB *color, const char *s);
+
 void print_palette_string (char *s);
 
 const gretlRGB *get_graph_color (int i);
