@@ -33,7 +33,6 @@ struct gretl_cmd {
 static struct gretl_cmd gretl_cmds[] = {
     { SEMIC,    ";" },     
     { ADD,      "add" },
-    { ADDTO,    "addto" },
     { ADF,      "adf" }, 
     { APPEND,   "append" },
     { AR,       "ar" },  
@@ -70,14 +69,13 @@ static struct gretl_cmd gretl_cmds[] = {
     { FOREIGN,  "foreign" }, 
     { FREQ,     "freq" }, 
     { FUNC,     "function" },
-    { FUNCERR,  "error" },
+    { FUNCERR,  "funcerr" },
     { GARCH,    "garch" },
     { GENR,     "genr" },  
     { GMM,      "gmm" },
     { GNUPLOT,  "gnuplot" },  
     { GRAPH,    "graph" }, 
     { HAUSMAN,  "hausman" },
-    { HCCM,     "hccm" }, 
     { HECKIT,   "heckit" },
     { HELP,     "help" },    
     { HSK,      "hsk" }, 
@@ -85,13 +83,14 @@ static struct gretl_cmd gretl_cmds[] = {
     { IF,       "if" },
     { INCLUDE,  "include" },
     { INFO,     "info" }, 
+    { INTREG,   "intreg" }, 
+    { KALMAN,   "kalman" },
     { KPSS,     "kpss" },
     { LABELS,   "labels" },
     { LAD,      "lad" },
     { LAGS,     "lags" },    
     { LDIFF,    "ldiff" },
     { LEVERAGE, "leverage" },
-    { LMTEST,   "lmtest" }, 
     { LOGISTIC, "logistic" },
     { LOGIT,    "logit" },
     { LOGS,     "logs" },
@@ -100,13 +99,14 @@ static struct gretl_cmd gretl_cmds[] = {
     { MEANTEST, "meantest" },
     { MLE,      "mle" },
     { MODELTAB, "modeltab" },
+    { MODPRINT, "modprint" },
+    { MODTEST,  "modtest" },
     { MPOLS,    "mpols" },
     { NLS,      "nls" },
     { NORMTEST, "normtest" },
     { NULLDATA, "nulldata" },
     { OLS,      "ols" },     
     { OMIT,     "omit" },
-    { OMITFROM, "omitfrom" },
     { OPEN,     "open" },
     { ORTHDEV,  "orthdev" },
     { OUTFILE,  "outfile" },
@@ -125,7 +125,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { RENAME,   "rename" },
     { RESET,    "reset" },
     { RESTRICT, "restrict" },
-    { RHODIFF,  "rhodiff" }, 
     { RMPLOT,   "rmplot" },
     { RUN,      "run" },
     { RUNS,     "runs" },
@@ -145,9 +144,8 @@ static struct gretl_cmd gretl_cmds[] = {
     { SUMMARY,  "summary" },
     { SYSTEM,   "system" },
     { TABPRINT, "tabprint" },
-    { TESTUHAT, "testuhat" },
     { TOBIT,    "tobit" },
-    { TSLS,     "tsls" },    
+    { IVREG,    "tsls", },
     { VAR,      "var" },
     { VARLIST,  "varlist" },
     { VARTEST,  "vartest" },
