@@ -23,7 +23,6 @@
 typedef enum {
     SEMIC = 0,    
     ADD,
-    ADDTO,
     ADF,
     APPEND,
     AR,  
@@ -67,7 +66,6 @@ typedef enum {
     GNUPLOT, 
     GRAPH,
     HAUSMAN,
-    HCCM,    
     HECKIT,
     HELP,    
     HSK,
@@ -75,13 +73,14 @@ typedef enum {
     IF,
     INCLUDE,
     INFO,
+    INTREG,
+    KALMAN,
     KPSS,
     LABELS, 
     LAD,
     LAGS,    
     LDIFF,
     LEVERAGE,
-    LMTEST, 
     LOGISTIC,
     LOGIT,
     LOGS,
@@ -90,13 +89,14 @@ typedef enum {
     MEANTEST,
     MLE,
     MODELTAB,
+    MODPRINT,
+    MODTEST,
     MPOLS,
     NLS,
     NORMTEST,
     NULLDATA,
     OLS,     
     OMIT,
-    OMITFROM,
     OPEN,
     ORTHDEV,
     OUTFILE,
@@ -115,7 +115,6 @@ typedef enum {
     RENAME,
     RESET,
     RESTRICT,
-    RHODIFF, 
     RMPLOT,
     RUN,
     RUNS,
@@ -135,9 +134,8 @@ typedef enum {
     SUMMARY,
     SYSTEM,
     TABPRINT,
-    TESTUHAT,
     TOBIT,
-    TSLS,    
+    IVREG,
     VAR,
     VARLIST,
     VARTEST,
@@ -150,14 +148,13 @@ typedef enum {
 } GretlCmdIndex;
 
 #define NEEDS_TWO_VARS(c)  ((c) == AR || (c) == ARCH || (c) == COINT || \
-                            (c) == AR1 || (c) == CORR || (c) == HCCM || \
-                            (c) == HSK || (c) == LOGIT || (c) == PROBIT || \
+                            (c) == AR1 || (c) == CORR || (c) == HSK || \
+                            (c) == LOGIT || (c) == PROBIT || \
                             (c) == SPEARMAN || (c) == OLS || \
-                            (c) == TSLS || (c) == VAR || (c) == WLS || \
+                            (c) == IVREG || (c) == VAR || (c) == WLS || \
 			    (c) == XTAB)
 
 #define TEXTSAVE_OK(c) (c == ADD || \
-                        c == ADDTO || \
                         c == ADF || \
                         c == CHOW || \
                         c == COEFFSUM || \
@@ -173,12 +170,11 @@ typedef enum {
                         c == HAUSMAN || \
                         c == KPSS || \
                         c == LEVERAGE || \
-                        c == LMTEST || \
+                        c == MODTEST || \
                         c == MEANTEST || \
                         c == MPOLS || \
 			c == NORMTEST || \
                         c == OMIT || \
-                        c == OMITFROM || \
                         c == PCA || \
                         c == PERGM || \
                         c == PLOT || \
@@ -191,32 +187,31 @@ typedef enum {
                         c == RUNS || \
                         c == SPEARMAN || \
                         c == SUMMARY || \
-                        c == TESTUHAT || \
                         c == VIF || \
                         c == VARTEST || \
                         c == XCORRGM || \
                         c == XTAB)
 
 #define NEEDS_MODEL_CHECK(c) (c == ADD || \
-                              c == OMIT || \
+                              c == CHOW || \
                               c == COEFFSUM || \
                               c == CUSUM || \
-                              c == RESET || \
-                              c == CHOW || \
-                              c == QLRTEST || \
-                              c == VIF || \
-                              c == TABPRINT || \
                               c == EQNPRINT || \
                               c == FCAST || \
                               c == HAUSMAN || \
                               c == LEVERAGE || \
-                              c == LMTEST || \
-                              c == TESTUHAT)
+                              c == MODTEST || \
+                              c == OMIT || \
+                              c == QLRTEST || \
+                              c == RESET || \
+                              c == TABPRINT || \
+                              c == VIF)
 
 #define USES_BFGS(c) ((c) == ARMA || \
                       (c) == GARCH || \
                       (c) == GMM || \
 		      (c) == HECKIT || \
+		      (c) == INTREG || \
                       (c) == MLE || \
                       (c) == TOBIT)
 
