@@ -30,10 +30,11 @@
 
 #include <glib.h>
 
-#define FN_DEBUG 0
-#define PKG_DEBUG 0
-#define EXEC_DEBUG 0
-#define UDEBUG 0
+#define FNPARSE_DEBUG 0 /* debug parsing of function code */
+#define EXEC_DEBUG 0    /* debugging of function execution */
+#define UDEBUG 0        /* debug handling of args */
+#define PKG_DEBUG 0     /* debug handling of function packages */
+#define FN_DEBUG 0      /* miscellaneous debugging */
 
 typedef struct fn_param_ fn_param;
 
@@ -296,6 +297,10 @@ static void set_listargs_from_call (fncall *call, DATAINFO *pdinfo)
 {
     int i, v;
 
+    if (pdinfo == NULL) {
+	return;
+    }
+
     for (i=1; i<pdinfo->v; i++) {
 	unset_var_listarg(pdinfo, i);
     }
@@ -334,7 +339,9 @@ static void set_executing_off (fncall *call, DATAINFO *pdinfo)
 	callstack = NULL;
     }
 
-    set_listargs_from_call(popcall, pdinfo);
+    if (pdinfo != NULL) {
+	set_listargs_from_call(popcall, pdinfo);
+    }
 }
 
 /* general info accessors */
@@ -555,7 +562,7 @@ ufunc *get_user_function_by_name (const char *name)
 	}
     }	
 
-#if FN_DEBUG
+#if FN_DEBUG > 1
     if (fun != NULL) {
 	fprintf(stderr, "get_user_function_by_name: name = '%s' (n_ufuns = %d);"
 		" found match\n", name, n_ufuns);
@@ -795,7 +802,7 @@ static int arg_type_from_int (const char *s)
 
 static int field_to_type (const char *s)
 {
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     fprintf(stderr, "field_to_type: looking at '%s'\n", s);
 #endif
 
@@ -1178,6 +1185,8 @@ void adjust_indent (const char *s, int *this_indent, int *next_indent)
 	ni++;
     } else if (wordmatch(s, "foreign")) {
 	ni++;
+    } else if (wordmatch(s, "kalman")) {
+	ni++;
     } else if (wordmatch(s, "end") ||
 	       wordmatch(s, "endif") ||
 	       wordmatch(s, "endloop")) {
@@ -2483,8 +2492,10 @@ static int check_func_name (const char *fname, ufunc **pfun, PRN *prn)
     } else {
 	for (i=0; i<n_ufuns; i++) {
 	    if (!strcmp(fname, ufuns[i]->name)) {
-		pprintf(prn, _("Redefining function '%s'"), fname);
-		pputc(prn, '\n');
+		if (libset_get_bool(VERBOSE_INCLUDE)) {
+		    pprintf(prn, _("Redefining function '%s'"), fname);
+		    pputc(prn, '\n');
+		}
 		if (pfun != NULL) {
 		    clear_ufunc_data(ufuns[i]);
 		    *pfun = ufuns[i];
@@ -2588,7 +2599,7 @@ static int read_param_option (char **ps, fn_param *param)
 {
     int err = E_PARSE;
 
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     fprintf(stderr, "read_param_option: got '%s'\n", *ps);
 #endif
 
@@ -2636,7 +2647,7 @@ static int parse_function_param (char *s, fn_param *param, int i)
     int type, len;
     int err = 0;
 
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     fprintf(stderr, "parse_function_param: s = '%s'\n", s);
 #endif
 
@@ -2685,6 +2696,9 @@ static int parse_function_param (char *s, fn_param *param, int i)
 	name = gretl_strndup(s, len);
 	if (name == NULL) {
 	    err = E_ALLOC;
+	} else if (gretl_reserved_word(name)) {
+	    free(name);
+	    err = E_DATA;
 	}
     }	
 
@@ -2730,7 +2744,7 @@ static int parse_function_param (char *s, fn_param *param, int i)
 	param->descrip = NULL;
     }
 
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     if (!err) {
 	fprintf(stderr, " param[%d] = '%s', ptype = %d\n", 
 		i, name, type);
@@ -2928,7 +2942,7 @@ static int add_function_return (ufunc *fun, const char *line)
 
     type = field_to_type(s1);
 
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     fprintf(stderr, "add_function_return: s1='%s', s2='%s'\n", s1, s2);
     fprintf(stderr, "field_to_type on '%s' gives %d\n", s1, type);
 #endif
@@ -2966,6 +2980,48 @@ static int end_of_function (const char *s)
     return ret;
 }
 
+static int function_return_line (const char *s)
+{
+    return !strncmp(s, "return ", 7);
+}
+
+#define bare_quote(p,s)   (*p == '"' && (p-s==0 || *(p-1) != '\\'))
+#define starts_comment(p) (*p == '/' && *(p+1) == '*')
+#define ends_comment(p)   (*p == '*' && *(p+1) == '/')
+
+static int ignore_line (ufunc *fun)
+{
+    int i, quoted = 0, ignore = 0;
+    char *s, *p;
+
+    for (i=0; i<fun->n_lines; i++) {
+	s = p = fun->lines[i];
+	while (*p) {
+	    if (!quoted && !ignore && *p == '#') {
+		break;
+	    }
+	    if (!ignore && bare_quote(p, s)) {
+		quoted = !quoted;
+	    }
+	    if (!quoted) {
+		if (starts_comment(p)) {
+		    ignore = 1;
+		    p += 2;
+		} else if (ends_comment(p)) {
+		    ignore = 0;
+		    p += 2;
+		    p += strspn(p, " ");
+		}
+	    }
+	    if (*p) {
+		p++;
+	    }
+	}
+    }
+
+    return ignore;
+}
+
 static int real_function_append_line (const char *line, ufunc *fun)
 {
     int editing = 1;
@@ -2976,7 +3032,7 @@ static int real_function_append_line (const char *line, ufunc *fun)
 	editing = 0;
     } 
 
-#if FN_DEBUG
+#if FNPARSE_DEBUG
     fprintf(stderr, "gretl_function_append_line: '%s'\n", line);
 #endif
 
@@ -2988,9 +3044,8 @@ static int real_function_append_line (const char *line, ufunc *fun)
     }
 
     if (string_is_blank(line)) {
-	/* return 0; */
 	err = strings_array_add(&fun->lines, &fun->n_lines, "");
-    } else if (end_of_function(line)) {
+    } else if (end_of_function(line) && !ignore_line(fun)) {
 	if (fun->n_lines == 0) {
 	    sprintf(gretl_errmsg, "%s: empty function", fun->name);
 	    err = 1;
@@ -3005,7 +3060,7 @@ static int real_function_append_line (const char *line, ufunc *fun)
     } else if (!strncmp(line, "function", 8)) {
 	strcpy(gretl_errmsg, "You can't define a function within a function");
 	err = 1;
-    } else if (!strncmp(line, "return ", 7)) {
+    } else if (function_return_line(line) && !ignore_line(fun)) {
 	err = add_function_return(fun, line + 7);
     } else {  
 	err = strings_array_add(&fun->lines, &fun->n_lines, line);
@@ -3177,6 +3232,8 @@ static int localize_list (fncall *call, const char *oldname,
     if (list == NULL) {
 	err = E_DATA;
     } else {
+	/* note: it's OK if the two names are the same,
+	   since their "levels" will be different */
 	err = copy_named_list_as(oldname, fp->name);
     }
 
@@ -3433,20 +3490,20 @@ int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
 	return 1;
     }
 
-    if ((dreq == FN_NEEDS_TS) && 
-	!dataset_is_time_series(pdinfo)) {
+    if (dreq == FN_NEEDS_TS && 
+	(pdinfo == NULL || !dataset_is_time_series(pdinfo))) {
 	strcpy(gretl_errmsg, "This function needs time-series data");
 	return 1;
     }
 
-    if ((dreq == FN_NEEDS_PANEL) && 
-	!dataset_is_panel(pdinfo)) {
+    if (dreq == FN_NEEDS_PANEL && 
+	(pdinfo == NULL || !dataset_is_panel(pdinfo))) {
 	strcpy(gretl_errmsg, "This function needs panel data");
 	return 1;
     }
 
-    if ((dreq == FN_NEEDS_QM) && 
-	(!dataset_is_time_series(pdinfo) || 
+    if (dreq == FN_NEEDS_QM && 
+	(pdinfo == NULL || !dataset_is_time_series(pdinfo) || 
 	 (pdinfo->pd != 4 || pdinfo->pd != 12))) {
 	strcpy(gretl_errmsg, "This function needs quarterly or monthly data");
 	return 1;
@@ -3455,9 +3512,8 @@ int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
     return 0;
 }
 
-static int 
-maybe_check_function_needs (const DATAINFO *pdinfo,
-			    const ufunc *fun)
+static int maybe_check_function_needs (const DATAINFO *pdinfo,
+				       const ufunc *fun)
 {
     const fnpkg *pkg = ufunc_get_parent_package(fun);
 
@@ -3468,13 +3524,7 @@ maybe_check_function_needs (const DATAINFO *pdinfo,
     }
 }
 
-enum {
-    GET_PTR,
-    GET_COPY
-};
-
-static double 
-get_scalar_return (const char *vname, int *err)
+static double get_scalar_return (const char *vname, int *err)
 {
     if (gretl_is_scalar(vname)) {
 	return gretl_scalar_get_value(vname);
@@ -3484,21 +3534,26 @@ get_scalar_return (const char *vname, int *err)
     }
 }
 
-static double *
-get_series_return (const char *vname, double **Z, DATAINFO *pdinfo,
-		   int action, int *err)
+static double *get_series_return (const char *vname, double **Z, 
+				  DATAINFO *pdinfo, int copy, 
+				  int *err)
 {
     int v = series_index(pdinfo, vname);
     double *x = NULL;
 
-    if (v < pdinfo->v) {
-	if (action == GET_COPY) {
+    if (!copy && v == 0) {
+	copy = 1;
+    }
+
+    if (v >= 0 && v < pdinfo->v) {
+	if (copy) {
 	    x = copyvec(Z[v], pdinfo->n);
 	    if (x == NULL) {
 		*err = E_ALLOC;
 	    }
 	} else {
 	    x = Z[v];
+	    Z[v] = NULL;
 	}
     } else {
 	*err = E_UNKVAR;
@@ -3507,71 +3562,108 @@ get_series_return (const char *vname, double **Z, DATAINFO *pdinfo,
     return x;
 }
 
-static gretl_matrix *
-get_matrix_return (const char *mname, int action, int *err)
+static gretl_matrix *get_matrix_return (const char *mname, int copy, int *err)
 {
-    gretl_matrix *m = get_matrix_by_name(mname);
     gretl_matrix *ret = NULL;
 
-    if (m != NULL) {
-	if (action == GET_COPY) {
+    if (copy) {
+	gretl_matrix *m = get_matrix_by_name(mname);
+
+	if (m != NULL) {
 	    ret = gretl_matrix_copy(m);
 	    if (ret == NULL) {
 		*err = E_ALLOC;
 	    }
-	} else {
-	    return ret = m;
-	}
+	} 
     } else {
+	ret = steal_matrix_by_name(mname);
+    }
+
+    if (ret == NULL && !*err) {
 	*err = E_UNKVAR;
     }
 
     return ret;
 }
 
-static int unlocalize_list (const char *lname, double **Z, DATAINFO *pdinfo)
+enum {
+    LIST_DIRECT_RETURN,
+    LIST_WAS_ARG
+};
+
+/* Deal with a list that exists at the level of a user-defined
+   function whose execution is now terminating.  Note that this list
+   may be the direct return value of the function, or it may have been
+   given to the function as an argument.  This is flagged in the
+   @status argument.  
+*/
+
+static int unlocalize_list (const char *lname, int status,
+			    double **Z, DATAINFO *pdinfo)
 {
     const int *list = get_list_by_name(lname);
     int d = gretl_function_depth();
-    const char *vname;
     int upd = d - 1;
-    int i, j, t, vi;
+    int i, vi;
 
 #if UDEBUG
     fprintf(stderr, "unlocalize_list: '%s', function depth = %d\n", lname, d);
     printlist(list, lname);
-    fprintf(stderr, "pdinfo = %p, pdinfo->v = %d\n", (void *) pdinfo, pdinfo->v);
+    fprintf(stderr, " pdinfo = %p, pdinfo->v = %d\n", (void *) pdinfo, pdinfo->v);
 #endif	    
 
     if (list == NULL) {
 	return E_DATA;
     }
 
-    for (i=1; i<=list[0]; i++) {
-	int overwrite = 0;
+    /* Note, AC, 2009-04-04.  The following is tricky, but I think it
+       _may_ now be right. If the list we're looking at was given as a
+       function argument we simply shunt all its members to the prior
+       STACK_LEVEL.  But if the list is the direct return value from
+       the function, we need to overwrite any variables at caller
+       level that have been redefined within the function.
+    */
 
-	vi = list[i];
-	unset_var_listarg(pdinfo, vi);
-	vname = pdinfo->varname[vi];
-	if (vi > 0 && vi < pdinfo->v && STACK_LEVEL(pdinfo, vi) == d) {
+    if (status == LIST_DIRECT_RETURN) {
+	int t, j, overwrite;
+	const char *vname;
+	
+	for (i=1; i<=list[0]; i++) {
+	    overwrite = 0;
+	    vi = list[i];
+	    vname = pdinfo->varname[vi];
+	    unset_var_listarg(pdinfo, vi);
+	    if (vi > 0 && vi < pdinfo->v && STACK_LEVEL(pdinfo, vi) == d) {
+		for (j=1; j<pdinfo->v; j++) { 
+		    if (STACK_LEVEL(pdinfo, j) == upd && 
+			!strcmp(pdinfo->varname[j], vname)) {
+			overwrite = 1;
+			break;
+		    }
+		}
+		if (overwrite) {
+		    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+			Z[j][t] = Z[vi][t];
+		    }
+		} else {
+		    STACK_LEVEL(pdinfo, vi) = upd;
+		}
+	    }
 #if UDEBUG
-	    fprintf(stderr, "unlocalize_list: looking at var %d, '%s'\n",
-		    vi, vname);
-#endif
-	    for (j=1; j<pdinfo->v; j++) { 
-		if (STACK_LEVEL(pdinfo, j) == upd && 
-		    !strcmp(pdinfo->varname[j], vname)) {
-		    overwrite = 1;
-		    break;
-		}
-	    }
+	    fprintf(stderr, " list-member var %d, '%s': ", vi, vname); 
 	    if (overwrite) {
-		for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-		    Z[j][t] = Z[vi][t];
-		}
+		fprintf(stderr, "found match in caller, overwrote var %d\n", j);
 	    } else {
-		STACK_LEVEL(pdinfo, vi) = upd;
+		fprintf(stderr, "no match in caller\n");
 	    }
+#endif
+	}
+    } else {
+	/* LIST_WAS_ARG */
+	for (i=1; i<=list[0]; i++) {
+	    vi = list[i];
+	    unset_var_listarg(pdinfo, vi);
+	    STACK_LEVEL(pdinfo, vi) = upd;
 	}
     }
 
@@ -3611,6 +3703,28 @@ maybe_set_return_description (ufunc *u, int rtype, DATAINFO *pdinfo,
     }
 }
 
+#define types_match(pt,rt) ((pt==GRETL_TYPE_SERIES_REF && rt==GRETL_TYPE_SERIES) || \
+                            (pt==GRETL_TYPE_MATRIX_REF && rt==GRETL_TYPE_MATRIX))
+
+static int is_pointer_arg (ufunc *u, fnargs *args, int rtype)
+{
+    int i;
+
+    for (i=0; i<args->argc; i++) {
+	if (types_match(u->params[i].type, rtype)) {
+	    if (!strcmp(u->params[i].name, u->retname)) {
+		return 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
+#define needs_datainfo(t) (t == GRETL_TYPE_SERIES || \
+                           t == GRETL_TYPE_LIST || \
+                           t == GRETL_TYPE_SERIES_REF)
+
 static int 
 function_assign_returns (ufunc *u, fnargs *args, int rtype, 
 			 double **Z, DATAINFO *pdinfo, 
@@ -3619,7 +3733,7 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 {
     struct fnarg *arg;
     fn_param *fp;
-    int i, err = 0;
+    int copy, i, err = 0;
 
 #if UDEBUG
     fprintf(stderr, "function_assign_returns: retname = '%s', rtype = %d\n", 
@@ -3627,19 +3741,25 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 #endif
 
     if (*perr == 0) {
-	/* direct return value */
-	if (rtype == GRETL_TYPE_DOUBLE) {
+	/* first we work on the value directly returned by the
+	   function (but only if there's no error) */
+	if (needs_datainfo(rtype) && pdinfo == NULL) {
+	    /* "can't happen" */
+	    err = E_DATA;
+	} else if (rtype == GRETL_TYPE_DOUBLE) {
 	    *(double *) ret = get_scalar_return(u->retname, &err);
 	} else if (rtype == GRETL_TYPE_SERIES) {
-	    *(double **) ret = get_series_return(u->retname, Z, pdinfo, GET_COPY, &err);
+	    copy = is_pointer_arg(u, args, rtype);
+	    *(double **) ret = get_series_return(u->retname, Z, pdinfo, copy, &err);
 	} else if (rtype == GRETL_TYPE_MATRIX) {
-	    *(gretl_matrix **) ret = get_matrix_return(u->retname, GET_COPY, &err);
+	    copy = is_pointer_arg(u, args, rtype);
+	    *(gretl_matrix **) ret = get_matrix_return(u->retname, copy, &err);
 	} else if (rtype == GRETL_TYPE_LIST) {
 	    /* note: in this case the job is finished in
 	       stop_fncall(); here we just adjust the info on the
 	       listed variables so they don't get deleted
 	    */
-	    err = unlocalize_list(u->retname, Z, pdinfo);
+	    err = unlocalize_list(u->retname, LIST_DIRECT_RETURN, Z, pdinfo);
 	} else if (rtype == GRETL_TYPE_STRING) {
 	    *(char **) ret = get_string_return(u->retname, Z, pdinfo, &err);
 	}
@@ -3651,19 +3771,21 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 
 	*perr = err;
 
-	if (!err && descrip != NULL) {
+	if (!err && pdinfo != NULL && descrip != NULL) {
 	    maybe_set_return_description(u, rtype, pdinfo, descrip);
 	}
     }
 
     /* "indirect return" values: these should be restored even if the
-       function bombed
+       function bombed.
     */
 
     for (i=0; i<args->argc; i++) {
 	arg = args->arg[i];
 	fp = &u->params[i];
-	if (gretl_ref_type(fp->type)) {
+	if (needs_datainfo(fp->type) && pdinfo == NULL) {
+	    err = E_DATA;
+	} else if (gretl_ref_type(fp->type)) {
 	    if (arg->type == GRETL_TYPE_SERIES_REF) {
 		int v = arg->val.idnum;
 
@@ -3678,11 +3800,7 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 		user_matrix_set_name(u, arg->upname);
 	    }
 	} else if (fp->type == GRETL_TYPE_LIST) {
-#if UDEBUG
-	    fprintf(stderr, "function_assign_returns: calling unlocalize_list on '%s'\n",
-		    fp->name);
-#endif	    
-	    unlocalize_list(fp->name, Z, pdinfo);
+	    unlocalize_list(fp->name, LIST_WAS_ARG, Z, pdinfo);
 	}
     }
 
@@ -3695,15 +3813,18 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 
 static void record_obs_info (obsinfo *o, DATAINFO *pdinfo)
 {
-    o->structure = pdinfo->structure;
-    o->pd = pdinfo->pd;
-    o->t1 = pdinfo->t1;
-    o->t2 = pdinfo->t2;
-    strcpy(o->stobs, pdinfo->stobs);
     o->changed = 0;
+
+    if (pdinfo != NULL) {
+	o->structure = pdinfo->structure;
+	o->pd = pdinfo->pd;
+	o->t1 = pdinfo->t1;
+	o->t2 = pdinfo->t2;
+	strcpy(o->stobs, pdinfo->stobs);
+    }
 }
 
-static int restore_obs_info (obsinfo *o, double **Z, DATAINFO *pdinfo)
+static int restore_obs_info (obsinfo *o, double ***pZ, DATAINFO *pdinfo)
 {
     char line[128];
     gretlopt opt = OPT_NONE;
@@ -3720,7 +3841,7 @@ static int restore_obs_info (obsinfo *o, double **Z, DATAINFO *pdinfo)
 
     sprintf(line, "setobs %d %s", o->pd, o->stobs);
 
-    return set_obs(line, Z, pdinfo, opt);
+    return set_obs(line, pZ, pdinfo, opt);
 }
 
 static int stop_fncall (fncall *call, int rtype, void *ret,
@@ -3734,7 +3855,7 @@ static int stop_fncall (fncall *call, int rtype, void *ret,
 #if FN_DEBUG
     fprintf(stderr, "stop_fncall: terminating call to "
 	    "function '%s' at depth %d, pdinfo->v = %d\n", 
-	    call->fun->name, d, pdinfo->v);
+	    call->fun->name, d, (pdinfo != NULL)? pdinfo->v : 0);
 #endif
 
     call->args = NULL;
@@ -3768,29 +3889,30 @@ static int stop_fncall (fncall *call, int rtype, void *ret,
        level via their inclusion in a returned list
     */
 
-    for (i=orig_v, delv=0; i<pdinfo->v; i++) {
-	if (STACK_LEVEL(pdinfo, i) == d) {
-	    delv++;
+    if (pdinfo != NULL) {
+	for (i=orig_v, delv=0; i<pdinfo->v; i++) {
+	    if (STACK_LEVEL(pdinfo, i) == d) {
+		delv++;
+	    }
+	}
+	if (delv > 0) {
+	    if (delv == pdinfo->v - orig_v) {
+		/* deleting all added variables */
+		anyerr = dataset_drop_last_variables(delv, pZ, pdinfo);
+		if (anyerr && !err) {
+		    err = anyerr;
+		}
+	    } else {
+		for (i=pdinfo->v-1; i>=orig_v; i--) {
+		    if (STACK_LEVEL(pdinfo, i) == d) {
+			anyerr = dataset_drop_variable(i, pZ, pdinfo);
+			if (anyerr && !err) {
+			    err = anyerr;
+			}
+		    } 
+		}
+	    }    
 	}
-    }
-
-    if (delv > 0) {
-	if (delv == pdinfo->v - orig_v) {
-	    /* deleting all added variables */
-	    anyerr = dataset_drop_last_variables(delv, pZ, pdinfo);
-	    if (anyerr && !err) {
-		err = anyerr;
-	    }
-	} else {
-	    for (i=pdinfo->v-1; i>=orig_v; i--) {
-		if (STACK_LEVEL(pdinfo, i) == d) {
-		    anyerr = dataset_drop_variable(i, pZ, pdinfo);
-		    if (anyerr && !err) {
-			err = anyerr;
-		    }
-		} 
-	    }
-	}    
     }
 
     /* direct list return: write the possibly revised list to the
@@ -3817,9 +3939,11 @@ static int stop_fncall (fncall *call, int rtype, void *ret,
     }    
 
     pop_program_state();
-    if (call->obs.changed) {
-	restore_obs_info(&call->obs, *pZ, pdinfo);
+
+    if (pdinfo != NULL && call->obs.changed) {
+	restore_obs_info(&call->obs, pZ, pdinfo);
     }
+
     set_executing_off(call, pdinfo);
 
     return err;
@@ -3970,15 +4094,15 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     MODEL **models = NULL;
     char line[MAXLINE];
     CMD cmd;
-    int orig_v = pdinfo->v;
-    int orig_t1 = pdinfo->t1;
-    int orig_t2 = pdinfo->t2;
+    int orig_v = 0;
+    int orig_t1 = 0;
+    int orig_t2 = 0;
     int indent0, started = 0;
     int i, err = 0;
 
     *funcerr_msg = '\0';
 
-#if FN_DEBUG
+#if EXEC_DEBUG
     fprintf(stderr, "gretl_function_exec: starting %s\n", u->name);
 #endif
 
@@ -3987,10 +4111,16 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	return err;
     }
 
+    if (pdinfo != NULL) {
+	orig_v = pdinfo->v;
+	orig_t1 = pdinfo->t1;
+	orig_t2 = pdinfo->t2;
+    }
+
     /* precaution */
     fn_state_init(&cmd, &state, &indent0);
 
-#if FN_DEBUG
+#if EXEC_DEBUG
     fprintf(stderr, "gretl_function_exec: argc = %d\n", args->argc);
     fprintf(stderr, "u->n_params = %d\n", u->n_params);
 #endif
@@ -4000,7 +4130,8 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	return E_ALLOC;
     }
 
-    err = check_function_args(u, args, (const double **) *pZ, pdinfo, prn);
+    err = check_function_args(u, args, (pZ != NULL)? (const double **) *pZ : NULL,
+			      pdinfo, prn);
 
     if (!err) {
 	call->args = args;
@@ -4026,7 +4157,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	*line = '\0';
 	gretl_exec_state_init(&state, FUNCTION_EXEC, line, &cmd, 
 			      models, prn);
-	if (pdinfo->submask != NULL) {
+	if (pdinfo != NULL && pdinfo->submask != NULL) {
 	    state.submask = copy_datainfo_submask(pdinfo);
 	}
 	state.callback = func_exec_callback;
@@ -4039,6 +4170,10 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
     }
 
+#if EXEC_DEBUG
+    fprintf(stderr, "start_fncall: err = %d\n", err);
+#endif
+
     /* get function lines in sequence and check, parse, execute */
 
     for (i=0; i<u->n_lines && !err; i++) {
@@ -4052,6 +4187,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
 
 	err = maybe_exec_line(&state, pZ, pdinfo);
+
 	if (err) {
 	    if (*gretl_errmsg == '\0') {
 		gretl_errmsg_sprintf("error in function %s\n"
@@ -4062,7 +4198,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
 
 	if (state.funcerr) {
-#if UDEBUG
+#if EXEC_DEBUG
 	    fprintf(stderr, "funcerr: gretl_function_exec: i=%d, line: '%s'\n", 
 		    i, line);
 #endif
@@ -4076,7 +4212,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
 
 	if (gretl_execute_loop()) { 
-#if UDEBUG
+#if EXEC_DEBUG
 	    fprintf(stderr, "gretl_function_exec: calling gretl_loop_exec\n");
 #endif
 	    err = gretl_loop_exec(&state, pZ, pdinfo);
@@ -4089,41 +4225,41 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 		fprintf(stderr, "error on line %d of function %s\n", i+1, u->name);
 		break;
 	    }
-#if UDEBUG
+#if EXEC_DEBUG
 	    fprintf(stderr, "gretl_function_exec: gretl_loop_exec done, err = %d\n", err);
 #endif
 	}
     }
 
-#if UDEBUG
+#if EXEC_DEBUG
     fprintf(stderr, "gretl_function_exec: %s: finished main exec, err = %d, pdinfo->v = %d\n", 
-	    u->name, err, pdinfo->v);
+	    u->name, err, (pdinfo != NULL)? pdinfo->v : 0);
 #endif
 
-    /* restore the sample that was in place on entry */
-
-    if (complex_subsampled()) {
-	if (state.submask == NULL) {
-	    /* we were not sub-sampled on entry */
-	    restore_full_sample(pZ, pdinfo, NULL);
-	} else if (submask_cmp(state.submask, pdinfo->submask)) {
-	    /* we were sub-sampled differently on entry */
-	    restore_full_sample(pZ, pdinfo, NULL);
-	    restrict_sample_from_mask(state.submask, pZ, pdinfo);
-	} 
+    if (pdinfo != NULL) {
+	/* restore the sample that was in place on entry */
+	if (complex_subsampled()) {
+	    if (state.submask == NULL) {
+		/* we were not sub-sampled on entry */
+		restore_full_sample(pZ, pdinfo, NULL);
+	    } else if (submask_cmp(state.submask, pdinfo->submask)) {
+		/* we were sub-sampled differently on entry */
+		restore_full_sample(pZ, pdinfo, NULL);
+		restrict_sample_from_mask(state.submask, pZ, pdinfo);
+	    } 
+	}
+	pdinfo->t1 = orig_t1;
+	pdinfo->t2 = orig_t2;
     }
 
-    pdinfo->t1 = orig_t1;
-    pdinfo->t2 = orig_t2;
-
     if (err) {
 	gretl_if_state_clear();
     } else {
 	err = gretl_if_state_check(indent0);
     }
 
-    function_assign_returns(u, args, rtype, *pZ, pdinfo,
-			    ret, descrip, prn, &err);
+    function_assign_returns(u, args, rtype, (pZ != NULL)? *pZ : NULL, 
+			    pdinfo, ret, descrip, prn, &err);
 
     gretl_exec_state_clear(&state);
 
@@ -4135,7 +4271,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
     }
 
-#if FN_DEBUG
+#if EXEC_DEBUG
     fprintf(stderr, "gretl_function_exec: err = %d\n", err);
 #endif
 
