@@ -186,7 +186,7 @@ void set_gretl_charset (const char *s)
 
     if (using_utf8) {
 	set_tex_use_utf(1);
-	set_fopen_use_utf8();
+	set_stdio_use_utf8();
     }
 
     *gretl_charset = '\0';
@@ -216,14 +216,14 @@ void set_gretl_charset (const char *s)
 		gretl_cset_maj = gretl_cset_min = 0;
 	    }
 	} 
-#ifdef WIN32
+# ifdef WIN32
 	if (p == NULL) {
 	    sscanf(gretl_charset, "cp%d", &gretl_cpage);
 	}
-#endif
+# endif
     }
 
-#ifdef WIN32
+# ifdef WIN32
     fprintf(stderr, "codepage = %d\n", gretl_cpage);
     if (gretl_cpage != 1250) {
 	char *e = getenv("GRETL_CPAGE");
@@ -233,18 +233,27 @@ void set_gretl_charset (const char *s)
 	    fprintf(stderr, "revised codepage to 1250\n");
 	}
     }
-#endif
+# endif
 }
 
+# ifdef WIN32
+
+static void set_cp_from_locale (const char *loc)
+{
+    const char *p = strrchr(loc, '.');
+
+    if (p != NULL && strlen(p) > 3 && isdigit(p[1])) {
+        gretl_cpage = atoi(p + 1);
+        fprintf(stderr, "set_cp_from_locale: CP = %d\n", gretl_cpage);
+    }
+}
+
+# endif
+
 static const char *get_gretl_charset (void)
 {
     static char cset[12];
 
-    if (gretl_cset_maj > 0 && gretl_cset_min > 0) {
-	sprintf(cset, "ISO-%d-%d", gretl_cset_maj, gretl_cset_min);
-	return cset;
-    } 
-
 # ifdef WIN32
     if (gretl_cpage > 0) {
 	sprintf(cset, "CP%d", gretl_cpage);
@@ -252,6 +261,11 @@ static const char *get_gretl_charset (void)
     }
 # endif
 
+    if (gretl_cset_maj > 0 && gretl_cset_min > 0) {
+	sprintf(cset, "ISO-%d-%d", gretl_cset_maj, gretl_cset_min);
+	return cset;
+    } 
+
     return NULL;
 }
 
@@ -295,10 +309,28 @@ int iso_latin_version (void)
     return 1;
 }
 
-char *gp_gettext (const char *msgid)
+int chinese_locale (void)
 {
-    return gettext(msgid);
-} 
+# ifdef WIN32
+    return (gretl_cpage == 950);
+# else
+    char *lang = getenv("LANG");
+
+    return (lang != NULL && !strncmp(lang, "zh", 2));
+#endif
+}
+
+static int gui_native_printing;
+
+void set_gui_native_printing (void)
+{
+    gui_native_printing = 1;
+}
+
+void unset_gui_native_printing (void)
+{
+    gui_native_printing = 0;
+}
 
 char *iso_gettext (const char *msgid)
 {
@@ -309,8 +341,7 @@ char *iso_gettext (const char *msgid)
 
     /* command line program: switching of codesets should not be 
        required, since unlike the GUI program there's no need
-       to force UTF-8 as the default.  Though there might be 
-       a problem with gnuplot, perhaps?
+       to force UTF-8 as the default.  
     */
 
     if (!strcmp(msgid, "@CLI_INIT")) {
@@ -329,9 +360,8 @@ char *iso_gettext (const char *msgid)
        iso_switch is non-zero (once it's determinate) this makes the
        I_() gettext macro use the system encoding, otherwise I_() is
        equivalent to plain _(), which always spits out UTF-8.  There's
-       a possible FIXME here: perhaps we should always recode for RTF
-       files -- is a UTF-8 encoded RTF file kosher?  But this seems
-       like a corner case.
+       a possible gotcha here: I suspect that a UTF-8 encoded RTF file 
+       is never kosher, regardless of the system.  
     */
 
     if (iso_switch < 0) {
@@ -345,7 +375,7 @@ char *iso_gettext (const char *msgid)
 	iso_switch = (cset != NULL);
     }
 
-    if (iso_switch) {
+    if (iso_switch && !gui_native_printing) {
 	bind_textdomain_codeset(PACKAGE, cset);
 	ret = gettext(msgid);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -356,6 +386,249 @@ char *iso_gettext (const char *msgid)
     return ret;
 } 
 
+# ifdef WIN32
+
+struct localeinfo {
+    int id;
+    const char *code;
+};
+
+/* the following are the strings that are accepted by setlocale()
+   on win32 */
+
+static struct localeinfo locales[] = {
+    { LANG_AUTO,  NULL },
+    { LANG_C,     "english" },
+    { LANG_EU,    "basque" },
+    { LANG_DE,    "german" },
+    { LANG_ES,    "spanish" },
+    { LANG_FR,    "french" },
+    { LANG_IT,    "italian" },
+    { LANG_PL,    "polish" },
+    { LANG_TR,    "turkish" },
+    { LANG_PT,    "portuguese" },
+    { LANG_PT_BR, "portuguese-brazilian" },
+    { LANG_RU,    "russian" },
+    { LANG_ZH_TW, "chinese-traditional" },
+    { LANG_CS,    "czech" },
+    { LANG_MAX,    NULL }
+};
+
+const char *locale_code_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == locales[i].id) {
+	    return locales[i].code;
+	}
+    }
+
+    return NULL;
+}
+
+# endif
+
+struct langinfo {
+    int id;
+    const char *name;
+    const char *code;
+};
+
+static struct langinfo langs[] = {
+    { LANG_AUTO,  "Automatic",            NULL    },
+    { LANG_C,     "English",              "C"     },
+    { LANG_EU,    "Basque",               "eu_ES" },
+    { LANG_ZH_TW, "Chinese (Taiwan)",     "zh_TW" },
+    { LANG_CS,    "Czech",                "cs_CZ" },
+    { LANG_FR,    "French",               "fr_FR" },
+    { LANG_DE,    "German",               "de_DE" },
+    { LANG_IT,    "Italian",              "it_IT" },
+    { LANG_PL,    "Polish",               "pl_PL" },
+    { LANG_PT,    "Portuguese",           "pt_PT" },
+    { LANG_PT_BR, "Portuguese (Brazil)",  "pt_BR" },
+    { LANG_RU,    "Russian",              "ru_RU" },
+    { LANG_ES,    "Spanish",              "es_ES" },
+    { LANG_TR,    "Turkish",              "tr_TR" },
+    { LANG_MAX,    NULL,                   NULL   }
+};
+
+const char *lang_string_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == langs[i].id) {
+	    return langs[i].name;
+	}
+    }
+
+    return NULL;
+}
+
+int lang_id_from_name (const char *s)
+{
+    int i;
+
+    if (s == NULL || *s == '\0') {
+	return 0;
+    }
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (!strcmp(s, langs[i].name)) {
+	    return langs[i].id;
+	}
+    }
+
+    return 0;
+}
+
+const char *lang_code_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == langs[i].id) {
+	    return langs[i].code;
+	}
+    }
+
+    return NULL;
+}
+
+# ifdef WIN32
+
+static char *win32_set_numeric (const char *lang)
+{
+    char *set = NULL;
+    int i;
+
+    for (i=LANG_EU; i<LANG_MAX; i++) {
+	if (!strcmp(lang, langs[i].code) ||
+	    !strncmp(lang, langs[i].code, 2)) {
+	    set = setlocale(LC_NUMERIC, locales[i].code);
+	    if (set != NULL) {
+		break;
+	    }
+	}
+    }
+
+    return set;
+}
+
+# endif /* WIN32 */
+
+void set_lcnumeric (int langid, int lcnumeric)
+{
+    if (lcnumeric && langid != LANG_C) {
+	char *lang = getenv("LANG");
+	char *set = NULL;
+
+	if (lang != NULL) {
+# ifdef WIN32
+	    set = win32_set_numeric(lang);
+# else
+	    set = setlocale(LC_NUMERIC, lang);
+# endif
+	} 
+	if (set == NULL) {
+	    setlocale(LC_NUMERIC, "");
+	    putenv("LC_NUMERIC=");
+	}
+    } else {
+	setlocale(LC_NUMERIC, "C");
+	putenv("LC_NUMERIC=C");
+    }
+
+    reset_local_decpoint();
+}
+
+/* arg should be long English form of language, as displayed
+   in GUI, e.g. "German", "French" */
+
+int test_locale (const char *langstr)
+{
+    const char *lcode;
+    int langid;
+    char *orig, *test;
+    char ocpy[64];
+
+    langid = lang_id_from_name(langstr);
+
+#ifdef WIN32
+    lcode = locale_code_from_id(langid);
+#else
+    lcode = lang_code_from_id(langid);
+#endif
+    orig = setlocale(LC_ALL, NULL);
+
+    gretl_error_clear();
+
+    *ocpy = '\0';
+    strncat(ocpy, orig, 63);
+
+    if ((test = setlocale(LC_ALL, lcode)) != NULL) {
+        fprintf(stderr, "test_locale: '%s' -> '%s'\n", lcode, test);
+	setlocale(LC_ALL, ocpy); /* restore the original locale */
+	return 0;
+    } else {
+	gretl_errmsg_sprintf(_("%s: locale is not supported "
+			       "on this system"), lcode);
+	return 1;
+    }
+}
+
+void force_language (int langid)
+{
+    const char *lcode = NULL;
+
+    if (langid == LANG_C) {
+	putenv("LANGUAGE=english");
+	setlocale(LC_ALL, "C");
+    } else {
+# ifdef WIN32
+        lcode = locale_code_from_id(langid);
+# else
+	lcode = lang_code_from_id(langid);
+# endif
+	if (lcode != NULL) {  
+	    char *newloc = setlocale(LC_ALL, lcode);
+
+            fprintf(stderr, "lcode='%s', newloc='%s'\n", lcode, newloc);
+# ifdef WIN32
+	    if (newloc != NULL) {
+		set_cp_from_locale(newloc);
+	    }
+# endif
+	}
+    }
+
+# if defined(WIN32)
+    if (langid == LANG_C) {
+	SetEnvironmentVariable("LC_ALL", "C");
+	putenv("LC_ALL=C");
+	textdomain("none");
+    } else if (lcode != NULL) {
+        char estr[32];
+
+        lcode = lang_code_from_id(langid);
+	SetEnvironmentVariable("LC_ALL", lcode);
+	sprintf(estr, "LC_ALL=%s", lcode);
+	putenv(estr);
+	SetEnvironmentVariable("LANG", lcode);
+	sprintf(estr, "LANG=%s", lcode);
+	putenv(estr);
+    }
+# elif defined(OSX_BUILD)
+    if (langid != LANG_C) {
+	char estr[64];
+
+	sprintf(estr, "LANGUAGE=%s", lang_code_from_id(langid));
+	putenv(estr);
+    }
+# endif
+}
+
 #endif  /* ENABLE_NLS */
 
 static void 
@@ -681,7 +954,7 @@ char *utf8_to_latin (const char *s)
     return ret;
 }
 
-/* convert from UTF-8 to Windows codepage, for EMF graphs */
+/* convert from UTF-8 to Windows codepage, e.g. for EMF graphs */
 
 char *utf8_to_cp (const char *s)
 {
