@@ -23,6 +23,8 @@
 #include "libset.h"
 #include "gretl_xml.h"
 
+#include <errno.h>
+
 #define LDEBUG 0
 
 typedef struct saved_list_ saved_list;
@@ -239,15 +241,15 @@ int subtract_from_list_by_name (const char *targ, const int *sub)
 /**
  * replace_list_by_name:
  * @targ: the name of the target list.
- * @new: replacement list
+ * @src: replacement list
  *
  * If @targ is the name of a saved list, replace the
- * list of that name with @new.
+ * list of that name with @src.
  *
  * Returns: 0 on success, non-zero code on failure.
  */
 
-int replace_list_by_name (const char *targ, const int *new)
+int replace_list_by_name (const char *targ, const int *src)
 {
     saved_list *sl = get_saved_list_by_name(targ);
     int err = 0;
@@ -255,7 +257,7 @@ int replace_list_by_name (const char *targ, const int *new)
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
-	int *tmp = gretl_list_copy(new);
+	int *tmp = gretl_list_copy(src);
 
 	if (tmp == NULL) {
 	    err = E_ALLOC;
@@ -283,6 +285,13 @@ static int real_remember_list (const int *list, const char *name,
 	    " n_lists=%d\n", name, force_new, n_lists);
 #endif
 
+    /* Note: 'force_new' means that we'll add a new list even if it
+       has the same name as an existing one.  This makes sense only if
+       we're copying a list in the context of running a user-defined
+       function, since in that case the new list will exist at a
+       different "stack level" from any prior list.
+    */
+
     if (!force_new) {
 	orig = get_saved_list_by_name(name);
     }
@@ -345,8 +354,8 @@ static int real_remember_list (const int *list, const char *name,
  * @name: name to be given to the list.
  * @prn: printing struct.
  *
- * Adds @list to the stack of saved lists and associates it
- * with @name, unless there is already a list with the given
+ * Adds a copy of @list to the stack of saved lists and associates
+ * it with @name, unless there is already a list with the given
  * name in which case the original list is replaced.  A status
  * message is printed to @prn.
  *
@@ -402,15 +411,15 @@ static int destroy_saved_list (saved_list *sl)
 /**
  * rename_saved_list:
  * @orig: the original name of the list.
- * @new: the new name to be given.
+ * @newname: the new name to be given.
  *
  * Renames a saved list from @orig to @new.  If there is
- * already a list called @new, it is destroyed.
+ * already a list called @newname, it is destroyed.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int rename_saved_list (const char *orig, const char *new)
+int rename_saved_list (const char *orig, const char *newname)
 {
     saved_list *sl0, *sl1;
     int err = 0;
@@ -419,13 +428,13 @@ int rename_saved_list (const char *orig, const char *new)
     if (sl0 == NULL) {
 	err = 1;
     } else {
-	/* is there already a list called @new? */
-	sl1 = get_saved_list_by_name(new);
+	/* is there already a list called @newname? */
+	sl1 = get_saved_list_by_name(newname);
 	if (sl1 != NULL) {
 	    err = destroy_saved_list(sl1);
 	}
 	*sl0->name = '\0';
-	strncat(sl0->name, new, VNAMELEN - 1);
+	strncat(sl0->name, newname, VNAMELEN - 1);
     } 
 
     return err;
@@ -434,18 +443,18 @@ int rename_saved_list (const char *orig, const char *new)
 /**
  * copy_named_list_as:
  * @orig: the name of the original list.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * If a saved list is found by the name @orig, a copy of
  * this list is added to the stack of saved lists under the
- * name @new.  This is intended for use when a list is given
+ * name @newname.  This is intended for use when a list is given
  * as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_named_list_as (const char *orig, const char *new)
+int copy_named_list_as (const char *orig, const char *newname)
 {
     saved_list *sl;
     int err = 0;
@@ -454,7 +463,7 @@ int copy_named_list_as (const char *orig, const char *new)
     if (sl == NULL) {
 	err = 1;
     } else {
-	err = real_remember_list(sl->list, new, 1, NULL);
+	err = real_remember_list(sl->list, newname, 1, NULL);
 	if (!err) {
 	    sl = list_stack[n_lists - 1];
 	    sl->level += 1;
@@ -956,59 +965,158 @@ int *gretl_list_copy (const int *src)
     return targ;
 }
 
+/**
+ * gretl_list_copy_from_pos:
+ * @src: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ *
+ * Returns: an allocated copy @src from position @pos onward
+ * (or %NULL on failure).
+ */
+
+int *gretl_list_copy_from_pos (const int *src, int pos)
+{
+    int *targ = NULL;
+    int i, n;
+
+    if (src != NULL && ((n = src[0] - pos + 1) > 0)) {
+	targ = gretl_list_new(n);
+	if (targ != NULL) {
+	    for (i=1; i<=n; i++) {
+		targ[i] = src[i+pos-1];
+	    }
+	}
+    }
+
+    return targ;
+}
+
 /**
  * gretl_list_from_string:
- * @liststr: string representation of list of integers.
+ * @str: string representation of list of integers.
+ * @err: location to receive error code.
  *
  * Reads a string containing a list of integers, separated by
  * spaces and/or commas and possibly wrapped in parentheses,
  * and constructs an array of these integers.  The first 
  * element is the number of integers that follow.
+ * This function supports an abbreviation for consecutive
+ * (increasing) integers in the list, using the notation, e.g., 
+ * "1-4" as shorthand for "1 2 3 4".
  *
  * Returns: the allocated array, or %NULL on failure.
  */
 
-int *gretl_list_from_string (const char *liststr)
+int *gretl_list_from_string (const char *str, int *err)
 {
-    const char *s = liststr;
-    char numstr[8];
+    char *p, *q, *s, *next;
+    int i, r1, r2, rg;
     int *list;
     int n = 0;
 
-    if (liststr == NULL) {
+    if (str == NULL) {
+	*err = E_DATA;
 	return NULL;
     }
 
-    while (*s) {
-	s += strspn(s, " ,()");
-	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
-	    n++;
-	    s += strlen(numstr);
+    /* 'p' marks the memory to be freed */
+    p = s = gretl_strdup(str);
+    if (s == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *err = 0;
+
+    /* strip white space at both ends */
+    while (isspace(*s)) s++;
+    tailstrip(s);
+
+    /* strip parentheses, if present */
+    if (*s == '(') {
+	n = strlen(s);
+	if (s[n-1] != ')') {
+	    /* got opening paren but no close */
+	    *err = E_PARSE;
+	    return NULL;
+	}
+	s[n-1] = '\0';
+	s++;
+	while (isspace(*s)) s++;
+	tailstrip(s);
+    }
+
+    q = s; /* copy relevant starting point */
+
+    charsub(s, ',', ' ');
+
+    errno = 0;
+
+    /* first pass: figure out the number of values
+       in the list, checking for errors as we go
+    */
+
+    n = 0; /* value counter */
+
+    while (*s && !*err) {
+	r1 = strtol(s, &next, 10);
+	if (errno || next == s) {
+	    *err = E_PARSE;
+	} else {
+	    s = next;
+	    if (*s == '-') {
+		/* hyphen indicating range? */
+		s++;
+		r2 = strtol(s, &next, 10);
+		if (errno || next == s) {
+		    *err = E_PARSE;
+		} else if (r2 < r1) {
+		    *err = E_PARSE;
+		} else {
+		    n += r2 - r1 + 1;
+		}
+		s = next;
+	    } else {
+		/* single numerical value */
+		n++;
+	    }
 	}
     }
 
-    if (n == 0) {
+    if (*err || n == 0) {
+	free(p);
 	return NULL;
     }
 
     list = gretl_list_new(n);
     if (list == NULL) {
+	*err = E_ALLOC;
+	free(p);
 	return NULL;
     }
 
-    s = liststr;
-    n = 1;
+    /* second pass: fill out the list (no error
+       checking should be needed at this stage) 
+    */
+
+    s = q; /* back to start of string */
+    n = 1; /* list position indicator */
+
     while (*s) {
-	s += strspn(s, " ,()");
-	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
-	    if (*numstr == ';') {
-		list[n++] = LISTSEP;
-	    } else {
-		list[n++] = atoi(numstr);
+	r1 = strtol(s, &s, 10);
+	if (*s == '-') {
+	    s++;
+	    r2 = strtol(s, &s, 10);
+	    rg = r2 - r1 + 1;
+	    for (i=0; i<rg; i++) {
+		list[n++] = r1 + i;
 	    }
-	    s += strlen(numstr);
+	} else {
+	    list[n++] = r1;
 	}
-    }    
+    } 
+
+    free(p);
 
     return list;
 }
@@ -1023,7 +1131,7 @@ int *gretl_list_from_string (const char *liststr)
  * positive integers in a range that is sensible for ID 
  * numbers of variables, typically with three digits or less, 
  * and will fail if the list contains any numbers greater 
- * than 999.
+ * than 9998.
  *
  * Returns: The string representation of the list on success,
  * or %NULL on failure.
@@ -1035,7 +1143,7 @@ char *gretl_list_to_string (const int *list)
     char numstr[8];
     int len, i, err = 0;
 
-    len = 4 * (list[0] + 1);
+    len = 6 * (list[0] + 1);
     if (len > MAXLINE - 32) {
 	return NULL;
     }
@@ -1046,11 +1154,8 @@ char *gretl_list_to_string (const int *list)
     }
 
     *buf = '\0';
+
     for (i=1; i<=list[0]; i++) {
-	if (abs(list[i] > 999)) {
-	    err = 1;
-	    break;
-	}
 	if (list[i] == LISTSEP) {
 	    strcat(buf, " ;");
 	} else {
@@ -1067,6 +1172,62 @@ char *gretl_list_to_string (const int *list)
     return buf;
 }
 
+/**
+ * gretl_list_to_lags_string:
+ * @list: array of integers.
+ * @err: location to receive error code.
+ * 
+ * Prints the given @list of integers into a newly
+ * allocated string, separated by commas.  Will fail
+ * if the list contains any numbers greater than 998.
+ *
+ * Returns: The string representation of the list on success,
+ * or %NULL on failure.
+ */
+
+char *gretl_list_to_lags_string (const int *list, int *err)
+{
+    char *buf;
+    char numstr[8];
+    int len, i;
+
+    len = 4 * (list[0] + 1);
+
+    if (len > MAXLINE - 32) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    buf = malloc(len);
+    if (buf == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *buf = '\0';
+
+    for (i=1; i<=list[0]; i++) {
+	if (abs(list[i] >= 999)) {
+	    *err = E_DATA;
+	    break;
+	} else {
+	    if (i == 1) {
+		sprintf(numstr, "%d", list[i]);
+	    } else {
+		sprintf(numstr, ",%d", list[i]);
+	    }
+	    strcat(buf, numstr);
+	}
+    }
+
+    if (*err) {
+	free(buf);
+	buf = NULL;
+    }
+
+    return buf;
+}
+
 /**
  * in_gretl_list:
  * @list: an array of integers, the first element of which holds
@@ -1082,24 +1243,24 @@ char *gretl_list_to_string (const int *list)
 
 int in_gretl_list (const int *list, int k)
 {
-    int i, ret = 0;
+    int i;
 
     if (list != NULL) {
-	for (i=1; i<=list[0] && !ret; i++) {
+	for (i=1; i<=list[0]; i++) {
 	    if (list[i] == k) {
-		ret = i;
+		return i;
 	    }
 	}
     }
 
-    return ret;
+    return 0;
 }
 
-static void reglist_move_const (int *list, int from)
+static void reglist_move_const (int *list, int k)
 {
-    int i, cnum = list[from];
+    int i, cnum = list[k];
 
-    for (i=from; i>2; i--) {
+    for (i=k; i>2; i--) {
 	list[i] = list[i-1];
     }
 
@@ -1570,7 +1731,9 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
  *
  * Returns: a newly allocated list including the elements of @biglist,
  * from position @minpos onwards, that are not present in @sublist, 
- * again from @minpos onwards, or %NULL on failure.  
+ * again from @minpos onwards, or %NULL on failure.  Note that
+ * comparison stops whenever a list separator is found; i.e. only
+ * the pre-separator portions of the lists are compared.
  */
 
 int *gretl_list_diff_new (const int *biglist, const int *sublist,
@@ -1591,9 +1754,14 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist,
 
     for (i=minpos; i<=biglist[0]; i++) {
 	bi = biglist[i];
+	if (bi == LISTSEP) {
+	    break;
+	}
 	match = 0;
 	for (j=minpos; j<=sublist[0]; j++) {
-	    if (sublist[j] == bi) {
+	    if (sublist[j] == LISTSEP) {
+		break;
+	    } else if (sublist[j] == bi) {
 		match = 1;
 		break;
 	    }
@@ -1765,7 +1933,7 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 			    "model have been redefined");
     const char *label;
     char rword[16];
-    int j, mc, repl, err = 0;
+    int j, mc, repl;
 
     if (ref_id == 0) {
 	mc = get_model_count();
@@ -1777,6 +1945,10 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	if (list[j] == LISTSEP) {
 	    continue;
 	}
+	if (list[j] >= pdinfo->v) {
+	    strcpy(gretl_errmsg, _(errmsg));
+	    return E_DATA;
+	}
 	label = VARLABEL(pdinfo, list[j]);
 	*rword = '\0';
 	sscanf(label, "%15s", rword);
@@ -1785,13 +1957,12 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	    sscanf(label, "%*s %*s %*s %d", &repl);
 	    if (repl >= mc) {
 		strcpy(gretl_errmsg, _(errmsg));
-		err = E_DATA;
-		break;
+		return E_DATA;
 	    }
 	}
     }
 
-    return err;
+    return 0;
 }
 
 /**
@@ -1849,9 +2020,11 @@ int gretl_list_separator_position (const int *list)
 {
     int i;
 
-    for (i=1; i<=list[0]; i++) {
-        if (list[i] == LISTSEP) {
-	    return i;
+    if (list != NULL) {
+	for (i=1; i<=list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		return i;
+	    }
 	}
     }
 
@@ -1894,9 +2067,10 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     int i, n = 0;
     int err = 0;
 
-    for (i=1; i<=list[0] && n==0; i++) {
+    for (i=1; i<=list[0]; i++) {
 	if (list[i] == LISTSEP) {
 	    n = i;
+	    break;
 	}
     }
 
@@ -1932,6 +2106,56 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     return err;
 }
 
+/**
+ * gretl_lists_join_with_separator:
+ * @list1: first sub-list.
+ * @list2: second sub-list.
+ *
+ * Concatenates the content of @list2 onto @list1, after first
+ * appending the list separator.  It is acceptable that @list1
+ * be %NULL, in which case the returned list is just @list2
+ * with the separator prepended.  But it is not acceptable that
+ * @list2 be null; in that this function returns %NULL.
+ *
+ * Returns: alllcated list  on success or %NULL on failure.
+ */
+
+int *gretl_lists_join_with_separator (const int *list1, const int *list2)
+{
+    int *biglist;
+    int i, j, n;
+
+    if (list2 == NULL) {
+	return NULL;
+    }
+
+    n = (list1 != NULL)? list1[0] : 0;
+
+
+    n += list2[0] + 1;
+    biglist = gretl_list_new(n);
+
+    if (biglist == NULL) {
+	return NULL;
+    }
+
+    j = 1;
+
+    if (list1 != NULL) {
+	for (i=1; i<=list1[0]; i++) {
+	    biglist[j++] = list1[i];
+	}
+    }
+
+    biglist[j++] = LISTSEP;
+
+    for (i=1; i<=list2[0]; i++) {
+	biglist[j++] = list2[i];
+    }    
+
+    return biglist;
+}
+
 static int real_list_dup (const int *list, int start, int stop)
 {
     int i, j, ret = -1;
@@ -1987,7 +2211,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 		break;
 	    }
 	}
-    } else if (ci == TSLS || ci == HECKIT || ci == EQUATION) {
+    } else if (ci == IVREG || ci == HECKIT || ci == EQUATION) {
 	multi = 1;
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
@@ -2408,3 +2632,4 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
 	pputc(prn, '\n');
     }
 }
+
