@@ -36,10 +36,14 @@ int *gretl_consecutive_list_new (int lmin, int lmax);
 
 int *gretl_list_copy (const int *src);
 
-int *gretl_list_from_string (const char *liststr);
+int *gretl_list_copy_from_pos (const int *src, int pos);
+
+int *gretl_list_from_string (const char *str, int *err);
 
 char *gretl_list_to_string (const int *list);
 
+char *gretl_list_to_lags_string (const int *list, int *err);
+
 int in_gretl_list (const int *list, int k);
 
 int gretl_list_delete_at_pos (int *list, int pos);
@@ -88,6 +92,8 @@ int gretl_list_is_consecutive (const int *list);
 
 int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2);
 
+int *gretl_lists_join_with_separator (const int *list1, const int *list2);
+
 int gretl_list_duplicates (const int *list, GretlCmdIndex ci);
 
 int gretl_lists_share_members (const int *list1, const int *list2);
@@ -106,15 +112,15 @@ int append_to_list_by_name (const char *targ, const int *add);
 
 int subtract_from_list_by_name (const char *targ, const int *sub);
 
-int replace_list_by_name (const char *targ, const int *new);
+int replace_list_by_name (const char *targ, const int *src);
 
 int remember_list (const int *list, const char *name, PRN *prn);
 
-int copy_named_list_as (const char *orig, const char *new);
+int copy_named_list_as (const char *orig, const char *newname);
 
 int named_list_lower_level (const char *name);
 
-int rename_saved_list (const char *orig, const char *new); 
+int rename_saved_list (const char *orig, const char *newname); 
 
 int create_named_null_list (const char *name);
 
