@@ -20,7 +20,9 @@
 #ifndef GRETL_MATRIX_H
 #define GRETL_MATRIX_H
 
-/* #define LDEBUG 1 */
+#include <stdarg.h>
+
+#define M_NA (0.0/0.0)
 
 /* minimum value of diagonal element of R (as in X = QR) that counts
    as non-zero for the purpose of determining the rank of X */
@@ -32,7 +34,7 @@ typedef enum {
     GRETL_MOD_TRANSPOSE,
     GRETL_MOD_SQUARE,
     GRETL_MOD_CUMULATE,
-    GRETL_MOD_DECUMULATE
+    GRETL_MOD_DECREMENT
 } GretlMatrixMod;
 
 typedef enum {
@@ -55,6 +57,8 @@ struct _gretl_matrix {
     double *val;
 };
 
+typedef struct _gretl_matrix_block gretl_matrix_block;
+
 /**
  * gretl_matrix_get:
  * @m: matrix.
@@ -193,15 +197,22 @@ gretl_matrix *gretl_matrix_reuse (gretl_matrix *m, int rows, int cols);
 
 int gretl_matrix_realloc (gretl_matrix *m, int rows, int cols);
 
+void gretl_matrix_block_destroy (gretl_matrix_block *B);
+
+gretl_matrix_block *gretl_matrix_block_new (gretl_matrix **pm, ...);
+
 gretl_matrix *gretl_identity_matrix_new (int n);
 
+gretl_matrix *gretl_DW_matrix_new (int n);
+
 gretl_matrix *gretl_zero_matrix_new (int r, int c);
 
 gretl_matrix *gretl_unit_matrix_new (int r, int c);
 
 gretl_matrix *gretl_null_matrix_new (void);
 
-gretl_matrix *gretl_matrix_seq (int start, int end);
+gretl_matrix *gretl_matrix_seq (int start, int end, int step, 
+				int *err);
 
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
@@ -220,8 +231,11 @@ int gretl_matrix_random_fill (gretl_matrix *m, int dist);
 
 gretl_matrix *gretl_random_matrix_new (int r, int c, int dist);
 
-gretl_matrix *gretl_matrix_resample (const gretl_matrix *m, int r,
-				     int *err);
+gretl_matrix *gretl_matrix_resample (const gretl_matrix *m, int *err);
+
+gretl_matrix *gretl_matrix_block_resample (const gretl_matrix *m, 
+					   int blocklen, 
+					   int *err);
 
 double gretl_vector_mean (const gretl_vector *v);
 
@@ -264,6 +278,9 @@ void gretl_matrix_free (gretl_matrix *m);
 
 double *gretl_matrix_steal_data (gretl_matrix *m);
 
+int gretl_vector_copy_values (gretl_vector *targ, 
+			      const gretl_vector *src);
+
 int gretl_matrix_copy_values (gretl_matrix *targ, 
 			      const gretl_matrix *src);
 
@@ -283,6 +300,8 @@ gretl_matrix_subtract_reversed (const gretl_matrix *a, gretl_matrix *b);
 
 int gretl_matrix_I_minus (gretl_matrix *m);
 
+int gretl_matrix_transpose_in_place (gretl_matrix *m);
+
 int gretl_matrix_transpose (gretl_matrix *targ, const gretl_matrix *src);
 
 int gretl_square_matrix_transpose (gretl_matrix *m);
@@ -396,6 +415,13 @@ int gretl_cholesky_decomp_solve (gretl_matrix *a, gretl_matrix *b);
 
 int gretl_cholesky_solve (const gretl_matrix *a, gretl_vector *b);
 
+gretl_vector *gretl_toeplitz_solve (const gretl_vector *c, 
+				    const gretl_vector *r, 
+				    const gretl_vector *b, 
+				    int *err);
+
+gretl_matrix *gretl_matrix_XTX_new (const gretl_matrix *X);
+
 int gretl_inverse_from_cholesky_decomp (gretl_matrix *targ, 
 					const gretl_matrix *src);
 
@@ -421,6 +447,8 @@ int gretl_SVD_invert_matrix (gretl_matrix *a);
 
 int gretl_invpd (gretl_matrix *a);
 
+int gretl_maybe_invpd (gretl_matrix *a);
+
 int gretl_matrix_SVD (const gretl_matrix *a, gretl_matrix **pu, 
 		      gretl_vector **ps, gretl_matrix **pvt);
 
@@ -474,7 +502,8 @@ gretl_matrix *gretl_matrix_cumcol (const gretl_matrix *m, int *err);
 gretl_matrix *gretl_matrix_diffcol (const gretl_matrix *m, 
 				    double missval, int *err);
 
-gretl_matrix *gretl_matrix_lag (const gretl_matrix *m, int k, 
+gretl_matrix *gretl_matrix_lag (const gretl_matrix *m, 
+				const gretl_vector *k, 
 				double missval);
 
 int gretl_matrix_inplace_lag (gretl_matrix *targ,
@@ -483,9 +512,11 @@ int gretl_matrix_inplace_lag (gretl_matrix *targ,
 
 int gretl_matrix_cholesky_decomp (gretl_matrix *a);
 
+int gretl_matrix_psd_root (gretl_matrix *a);
+
 int gretl_matrix_QR_decomp (gretl_matrix *M, gretl_matrix *R);
 
-int gretl_check_QR_rank (const gretl_matrix *R, int *err);
+int gretl_check_QR_rank (const gretl_matrix *R, int *err, double *rcnd);
 
 int gretl_matrix_rank (const gretl_matrix *a, int *err);
 
@@ -530,7 +561,7 @@ gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
 			     gretl_vector *b, gretl_matrix *vcv,
 			     gretl_vector *uhat, double *s2);
 
-int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
+int gretl_matrix_SVD_ols (const gretl_vector *y, const gretl_matrix *X,
 			  gretl_vector *b, gretl_matrix *vcv,
 			  gretl_vector *uhat, double *s2);
 
@@ -570,10 +601,10 @@ int gretl_matrices_are_equal (const gretl_matrix *a, const gretl_matrix *b,
 gretl_matrix *gretl_covariance_matrix (const gretl_matrix *m, int corr,
 				       int *errp);
 
-gretl_matrix **gretl_matrix_array_alloc (int n);
+gretl_matrix **gretl_matrix_array_new (int n);
 
 gretl_matrix **
-gretl_matrix_array_alloc_with_size (int n, int rows, int cols);
+gretl_matrix_array_new_with_size (int n, int rows, int cols);
 
 void gretl_matrix_array_free (gretl_matrix **A, int n);
 
@@ -607,6 +638,11 @@ gretl_matrix *gretl_matrix_bool_sel(const gretl_matrix *A,
 gretl_matrix *gretl_matrix_sort_by_column (const gretl_matrix *m, 
 					   int k, int *err);
 
+gretl_matrix *gretl_matrix_covariogram (const gretl_matrix *X, 
+					const gretl_matrix *u,
+					const gretl_matrix *w,
+					int p, int *err);
+
 void lapack_mem_free (void);
 
 void set_blas_nmk_min (int n);
