@@ -47,13 +47,84 @@ struct CoeffIntervals_ {
     int ifc;
 };
 
+typedef enum {
+    VCV_CLASSICAL,
+    VCV_HC,
+    VCV_HAC,
+    VCV_ML,
+    VCV_PANEL,
+    VCV_RQ
+} VCVMajorType;
+
+typedef enum {
+    VCV_UNSET,
+    VCV_HESSIAN,
+    VCV_IM,
+    VCV_OP,
+    VCV_QML,
+    VCV_BW,
+    VCV_MAX
+} MLVCVType;
+
+typedef enum {
+    KERNEL_BARTLETT,
+    KERNEL_PARZEN,
+    KERNEL_QS,
+    KERNEL_MAX
+} HACKernel;
+
+typedef enum {
+    PANEL_HAC,
+    PANEL_BK   /* Beck-Katz */
+} PanelVCVType;
+
+typedef enum {
+    RQ_ASY,
+    RQ_NID    /* sandwich */
+} RQVCVType;
+
+typedef struct VCVInfo_ VCVInfo;
+
+struct VCVInfo_ {
+    int vmaj;
+    int vmin;
+    int order;
+    int flags;
+    double bw;
+};
+
+/* single-equation model commands */
+
+#define MODEL_COMMAND(c) (c == AR || \
+                          c == AR1 || \
+                          c == ARBOND || \
+                          c == ARCH || \
+                          c == ARMA || \
+                          c == GARCH || \
+                          c == GMM || \
+		          c == HECKIT || \
+                          c == HSK || \
+                          c == INTREG || \
+                          c == IVREG || \
+                          c == LAD || \
+                          c == LOGISTIC || \
+                          c == LOGIT || \
+                          c == MLE || \
+                          c == MPOLS || \
+                          c == NLS || \
+                          c == OLS || \
+                          c == PANEL || \
+                          c == POISSON || \
+                          c == PROBIT || \
+                          c == QUANTREG || \
+                          c == TOBIT || \
+                          c == WLS)
+
 #define AR_MODEL(c) (c == AR || \
 		     c == AR1 || \
                      c == ARMA || \
                      c == GARCH)
 
-#define AR1_MODEL(c) (c == AR1)
-
 #define SIMPLE_AR_MODEL(c) (c == AR || c == AR1)
 
 #define ML_ESTIMATOR(c) (c == ARMA || \
@@ -70,7 +141,6 @@ struct CoeffIntervals_ {
                    c == TOBIT)
 
 #define LSQ_MODEL(c) (c == AR1 || \
-                      c == HCCM || \
                       c == HSK || \
                       c == OLS || \
                       c == WLS)
@@ -80,12 +150,17 @@ struct CoeffIntervals_ {
                              c == GARCH || \
                              c == GMM || \
                              c == HECKIT || \
+                             c == INTREG || \
+                             c == IVREG || \
                              c == LOGIT || \
                              c == MLE || \
                              c == POISSON || \
                              c == PROBIT || \
-                             c == TOBIT || \
-                             c == TSLS)
+                             c == TOBIT)
+
+/* model where the specification is not based on a list
+   of variables */
+#define NONLIST_MODEL(c) (c == NLS || c == MLE || c == GMM)
 
 #define is_model_ref_cmd(c) (c == ADD || \
 	                     c == ARCH || \
@@ -93,15 +168,17 @@ struct CoeffIntervals_ {
 	                     c == CUSUM || \
 	                     c == FCAST || \
                              c == LEVERAGE || \
-	                     c == LMTEST || \
+	                     c == MODTEST || \
                              c == OMIT || \
 	                     c == RESTRICT || \
-	                     c == TESTUHAT || \
                              c == VIF)
 
 #define RQ_SPECIAL_MODEL(m) (m->ci == LAD && \
                              NULL != gretl_model_get_data(m, "rq_tauvec"))
 
+#define POOLED_MODEL(m) ((m->ci == OLS || m->ci == PANEL) && \
+                         gretl_model_get_int(m, "pooled"))
+
 typedef enum {
     GRETL_TEST_ADD,
     GRETL_TEST_ARCH,
@@ -117,13 +194,15 @@ typedef enum {
     GRETL_TEST_SQUARES,
     GRETL_TEST_WHITES,
     GRETL_TEST_SARGAN,
-    GRETL_TEST_TSLS_HAUSMAN,
+    GRETL_TEST_IV_HAUSMAN,
     GRETL_TEST_PANEL_HAUSMAN,
     GRETL_TEST_PANEL_F,
     GRETL_TEST_PANEL_BP,
     GRETL_TEST_PANEL_TIMEDUM,
     GRETL_TEST_HET_1,
     GRETL_TEST_BP,
+    GRETL_TEST_CHOWDUM,
+    GRETL_TEST_COMFAC,
     GRETL_TEST_MAX
 } ModelTestType;
 
@@ -133,6 +212,8 @@ int attach_model_tests_from_xml (MODEL *pmod, xmlNodePtr node);
 
 MODEL *gretl_model_new (void);
 
+void model_stats_init (MODEL *pmod);
+
 void gretl_model_init (MODEL *pmod);
 
 int gretl_model_allocate_storage (MODEL *pmod);
@@ -157,6 +238,8 @@ void gretl_model_free (MODEL *pmod);
 
 void gretl_model_free_on_exit (MODEL *pmod);
 
+void display_model_data_items (const MODEL *pmod);
+
 int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, void *ptr, 
 					  GretlType type, size_t size, 
 					  void (*destructor) (void *));
@@ -179,6 +262,11 @@ int gretl_model_set_int (MODEL *pmod, const char *key, int val);
 
 int gretl_model_set_double (MODEL *pmod, const char *key, double val);
 
+int gretl_model_set_full_vcv_info (MODEL *pmod, int vmaj, int vmin,
+				   int order, int flags, double bw);
+
+int gretl_model_set_vcv_info (MODEL *pmod, int vmaj, int vmin);
+
 void *gretl_model_get_data (const MODEL *pmod, const char *key);
 
 void *gretl_model_get_data_and_size (const MODEL *pmod, const char *key,
@@ -213,6 +301,8 @@ const char *gretl_model_get_depvar_name (const MODEL *pmod,
 
 int *gretl_model_get_x_list (const MODEL *pmod);
 
+int *gretl_model_get_secondary_list (const MODEL *pmod);
+
 int arma_model_nonseasonal_AR_order (const MODEL *pmod);
 
 int arma_model_nonseasonal_MA_order (const MODEL *pmod);
@@ -245,17 +335,15 @@ int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
 
+int gretl_model_write_coeffs (MODEL *pmod, double *b, int k);
+
 int gretl_model_add_arinfo (MODEL *pmod, int nterms);
 
 MODEL *gretl_model_copy (const MODEL *pmod);
 
 void swap_models (MODEL *targ, MODEL *src);
 
-int is_model_cmd (const char *line);
-
-int is_quiet_model_test (int ci, gretlopt opt);
-
-int command_ok_for_model (int test_ci, gretlopt opt, int model_ci);
+int command_ok_for_model (int test_ci, gretlopt opt, int mci);
 
 int model_test_ok (int ci, gretlopt opt, const MODEL *pmod, 
 		   const DATAINFO *pdinfo);
@@ -288,6 +376,7 @@ void model_test_set_dfd (ModelTest *test, int df);
 void model_test_set_value (ModelTest *test, double val);
 void model_test_set_pvalue (ModelTest *test, double pval);
 void model_test_set_param (ModelTest *test, const char *s);
+void model_test_set_opt (ModelTest *test, gretlopt opt);
 void model_test_set_allocated_param (ModelTest *test, char *s);
 void model_test_set_crit_and_alpha (ModelTest *test, 
 				    double crit,
@@ -335,10 +424,11 @@ void gretl_model_set_name (MODEL *pmod, const char *name);
 const char *gretl_model_get_name (const MODEL *pmod);
 
 double gretl_model_get_scalar (const MODEL *pmod, ModelDataIndex idx, 
+			       double ***pZ, DATAINFO *pdinfo,
 			       int *err);
 
 double *
-gretl_model_get_series (const MODEL *pmod, const DATAINFO *pdinfo, 
+gretl_model_get_series (MODEL *pmod, const DATAINFO *pdinfo, 
 			ModelDataIndex idx, int *err);
 
 gretl_matrix *gretl_model_get_matrix (MODEL *pmod, ModelDataIndex idx, 
@@ -353,7 +443,9 @@ int gretl_model_serialize (const MODEL *pmod, SavedObjectFlags flags,
 
 #ifndef GRETLCLI
 
-MODEL *gretl_model_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
+MODEL *gretl_model_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			     const DATAINFO *pdinfo,
+			     int *err);
 
 #endif
 
