@@ -36,8 +36,14 @@
 # include <errno.h>
 #endif
 
+#include <fcntl.h> /* for 'open' */
+
 #include <glib.h>
 
+#if (GLIB_MAJOR_VERSION > 2 || GLIB_MINOR_VERSION >= 6)
+#include <glib/gstdio.h>
+#endif
+
 struct INTERNAL_PATHS {
     char dotdir[MAXLEN];
     char workdir[MAXLEN];
@@ -67,7 +73,7 @@ static int add_suffix (char *fname, const char *sfx)
 /* Heuristic: filename contains non-ascii characters, and
    validates as UTF-8 */
 
-int fname_is_utf8 (const unsigned char *s)
+int string_is_utf8 (const unsigned char *s)
 {
     const unsigned char *p = s;
     int sevenbit = 1;
@@ -88,30 +94,79 @@ int fname_is_utf8 (const unsigned char *s)
     return ret;
 }
 
-static int fopen_use_utf8;
+static int stdio_use_utf8;
 
 /**
- * set_fopen_use_utf8:
+ * set_stdio_use_utf8:
  *
  * Sets gretl's internal state so as to ensure that filenames
- * are given in UTF-8 when passed to the C library's fopen().
+ * are given in UTF-8 when passed to functions such as the C 
+ * library's fopen().
  */
 
-void set_fopen_use_utf8 (void)
+void set_stdio_use_utf8 (void)
 {
-    fopen_use_utf8 = 1;
+    stdio_use_utf8 = 1;
 }
 
 /**
- * get_fopen_use_utf8:
+ * get_stdio_use_utf8:
  *
- * Returns: 1 if fienames should be in UTF-8 when passed to the C
- * library's fopen(), otherwise 0.
+ * Returns: 1 if filenames should be in UTF-8 when passed to the C
+ * library's fopen() and friends, otherwise 0.
  */
 
-int get_fopen_use_utf8 (void)
+int get_stdio_use_utf8 (void)
 {
-    return fopen_use_utf8;
+    return stdio_use_utf8;
+}
+
+#define FDEBUG 0
+
+/* Try to handle both possible 'cases of need': we should be using 
+   UTF-8 but the path is not in UTF-8, or the path is in UTF-8
+   but should be in locale encoding.
+*/
+
+static int maybe_recode_path (const char *path, int want_utf8, char **pconv)
+{
+    int err = 0;
+
+#if FDEBUG
+    fprintf(stderr, "maybe_recode_path: want_utf8 = %d\n", want_utf8);
+#endif
+
+    if (want_utf8) {
+	if (!g_utf8_validate(path, -1, NULL)) {
+	    /* need to convert from locale to UTF-8 */
+	    GError *gerr = NULL;
+	    gsize sz;
+	    
+	    *pconv = g_locale_to_utf8(path, -1, NULL, &sz, &gerr);
+	    if (*pconv == NULL) {
+		if (gerr != NULL) {
+		    gretl_errmsg_set(gerr->message);
+		    g_error_free(gerr);
+		}
+		err = 1;
+	    }
+	}
+    } else if (string_is_utf8((unsigned char *) path)) {
+	/* need to convert from UTF-8 to locale */
+	GError *gerr = NULL;
+	gsize sz;
+
+	*pconv = g_locale_from_utf8(path, -1, NULL, &sz, &gerr);
+	if (*pconv == NULL) {
+	    if (gerr != NULL) {
+		gretl_errmsg_set(gerr->message);
+		g_error_free(gerr);
+	    }
+	    err = 1;
+	}	
+    }	    
+
+    return err;
 }
 
 /**
@@ -119,49 +174,43 @@ int get_fopen_use_utf8 (void)
  * @fname: name of file to be opened.
  * @mode: mode in which to open the file.
  *
- * A wrapper for the C library's fopen(): provides a guard
- * against the situation where a filename is UTF-8 encoded, 
- * but on the current platform we should be using locale
- * encoding for the stdio functions.
+ * A wrapper for  the C library's fopen(), making allowance for
+ * the possibility that @fname has to be converted from
+ * UTF-8 to the locale encoding or vice versa.
  *
  * Returns: file pointer, or %NULL on failure.
  */
 
 FILE *gretl_fopen (const char *fname, const char *mode)
 {
-    gchar *fconv;
-    gsize wrote;
+    gchar *fconv = NULL;
     FILE *fp = NULL;
+    int err;
 
-    errno = 0;
+    gretl_error_clear();
 
-    if (mode != NULL && *mode == 'r') {
-	/* opening for reading */
-	fp = fopen(fname, mode);
-	if (fp == NULL && !fopen_use_utf8 && 
-	    fname_is_utf8((unsigned char *) fname)) {
-	    int save_errno = errno;
+#if FDEBUG
+    fprintf(stderr, "gretl_fopen: got '%s'\n", fname);
+#endif
 
-	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
-	    if (fconv != NULL) {
-		fp = fopen(fconv, mode);
-		g_free(fconv);
-	    }
-	    errno = save_errno;
-	}
-    } else {
-	/* opening for appending/writing */
-	if (!fopen_use_utf8 && fname_is_utf8((unsigned char *) fname)) {
-	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
-	    if (fconv != NULL) {
-		fp = fopen(fconv, mode);
-		g_free(fconv);
-	    }
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+
+    if (!err) {
+	if (fconv != NULL) {
+	    fp = fopen(fconv, mode);
+#if FDEBUG
+            fprintf(stderr, "using fconv, fp = %p\n", (void *) fp);
+#endif
+	    g_free(fconv);
 	} else {
 	    fp = fopen(fname, mode);
 	}
     }
 
+#if FDEBUG
+    fprintf(stderr, "after fopen, errno = %d\n", errno);
+#endif
+
     if (errno != 0) {
 	gretl_errmsg_set_from_errno(fname);
     }
@@ -169,74 +218,343 @@ FILE *gretl_fopen (const char *fname, const char *mode)
     return fp;
 }
 
+/**
+ * gretl_open:
+ * @pathname: name of file to be opened.
+ * @flags: flags to pass to the system open().
+ *
+ * A wrapper for the C library's open(), making allowance for
+ * the possibility that @pathname has to be converted from
+ * UTF-8 to the locale encoding or vice versa.
+ *
+ * Returns: new file descriptor, or -1 on error.
+ */
+
+int gretl_open (const char *pathname, int flags)
+{
+    gchar *pconv = NULL;
+    int fd = -1;
+    int err = 0;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(pathname, stdio_use_utf8, &pconv);
+
+    if (!err) {
+	if (pconv != NULL) {
+	    fd = open(pconv, flags);
+	    g_free(pconv);
+	} else {
+	    fd = open(pathname, flags);
+	}
+    }
+
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno(pathname);
+    }
+
+    return fd;
+}
+
+/**
+ * gretl_stat:
+ * @fname: name of file to be examined.
+ * @buf: pointer to %struct %stat.
+ *
+ * A wrapper for the C library's stat(), making allowance for
+ * the possibility that @fname has to be converted from UTF-8 
+ * to the locale encoding or vice versa.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int gretl_stat (const char *fname, struct stat *buf)
+{
+    gchar *pconv = NULL;
+    int err;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(fname, stdio_use_utf8, &pconv);
+    
+    if (err) {
+	/* emulate 'stat' */
+	err = -1;
+    } else {
+	if (pconv != NULL) {
+            err = stat(pconv, buf);
+ 	    g_free(pconv);
+	} else {
+            err = stat(fname, buf);
+ 	}
+    }
+
+    return err;
+}
+
+/**
+ * gretl_rename:
+ * @oldpath: name of file to be opened.
+ * @newpath: new name to give the file.
+ *
+ * A wrapper for the C library's rename(), making allowance for
+ * the possibility that @oldpath and/or @newpath have to be 
+ * converted from UTF-8 to the locale encoding or vice versa.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int gretl_rename (const char *oldpath, const char *newpath)
+{
+    gchar *oldconv = NULL, *newconv = NULL;
+    int err;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(oldpath, stdio_use_utf8, &oldconv);
+
+    if (!err) {
+	err = maybe_recode_path(newpath, stdio_use_utf8, &newconv);
+    }
+
+    if (!err) {
+	if (oldconv == NULL && newconv == NULL) {
+	    err = rename(oldpath, newpath);
+	} else if (oldconv != NULL && newconv != NULL) {
+	    err = rename(oldconv, newconv);
+	} else if (oldconv != NULL) {
+	    err = rename(oldconv, newpath);
+	} else if (newconv != NULL) {
+	    err = rename(oldpath, newconv);
+	}
+    }
+
+    if (oldconv != NULL || newconv != NULL) {
+	g_free(oldconv);
+	g_free(newconv);
+    }
+
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno("gretl_rename");
+    }
+
+    return err;
+}
+
+/**
+ * gretl_remove:
+ * @path: name of file or directory to remove.
+ *
+ * A wrapper for remove(), making allowance for
+ * the possibility that @path has to be converted from
+ * UTF-8 to the locale encoding or vice versa.
+ *
+ * Returns: 0 on sucess, non-zero on failure.
+ */
+
+int gretl_remove (const char *path)
+{
+    gchar *pconv = NULL;
+    int ret = -1;
+    int err;
+
+    err = maybe_recode_path(path, stdio_use_utf8, &pconv);
+
+    if (!err) {
+	if (pconv != NULL) {
+	    ret = remove(pconv);
+	    g_free(pconv);
+	} else {
+	    ret = remove(path);
+	}
+    }
+
+#ifdef WIN32
+    /* allow for the possibility that we're trying to remove a
+       directory on win32 -> use g_remove */
+    if (ret == -1) {
+	err = maybe_recode_path(path, 1, &pconv);
+	if (!err) {
+	    if (pconv != NULL) {
+		ret = g_remove(pconv);
+		g_free(pconv);
+	    } else {
+		ret = g_remove(path);
+	    }
+	}	    
+    }
+#endif
+
+    return ret;
+}
+
 /**
  * gretl_gzopen:
  * @fname: name of gzipped file to be opened.
  * @mode: mode in which to open the file.
  *
- * A wrapper for zlib's gzopen(): provides a guard
- * against the situation where a filename is UTF-8 encoded, 
- * but on the current platform we should be using local
- * encoding for the stdio functions.
+ * A wrapper for zlib's gzopen(), making allowance for
+ * the possibility that @fname has to be converted from
+ * UTF-8 to the locale encoding or vice versa. 
  *
  * Returns: pointer to gzip stream, or %NULL on failure.
  */
 
 gzFile gretl_gzopen (const char *fname, const char *mode)
 {
-    gchar *fconv;
-    gsize wrote;
+    gchar *fconv = NULL;
     gzFile fz = NULL;
+    int err;
 
-    if (mode != NULL && *mode == 'r') {
-	/* opening for reading */
-	fz = gzopen(fname, mode);
-	if (fz == NULL && !fopen_use_utf8 && 
-	    fname_is_utf8((unsigned char *) fname)) {
-	    int save_errno = errno;
+    gretl_error_clear();
 
-	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
-	    if (fconv != NULL) {
-		fz = gzopen(fconv, mode);
-		g_free(fconv);
-	    }
-	    errno = save_errno;
-	}
-    } else {
-	/* opening for writing */
-	if (!fopen_use_utf8 && fname_is_utf8((unsigned char *) fname)) {
-	    fconv = g_locale_from_utf8(fname, -1, NULL, &wrote, NULL);
-	    if (fconv != NULL) {
-		fz = gzopen(fconv, mode);
-		g_free(fconv);
-	    }
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+
+    if (!err) {
+	if (fconv != NULL) {
+	    fz = gzopen(fconv, mode);
+	    g_free(fconv);
 	} else {
 	    fz = gzopen(fname, mode);
 	}
     }
 
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno("gzopen");
+    }
+
     return fz;
 }
 
+/**
+ * gretl_chdir:
+ * @path: name of directory.
+ *
+ * A wrapper for POSIX chdir(), making allowance for
+ * the possibility that @path has to be converted from
+ * UTF-8 to the locale encoding or vice versa.
+ *
+ * Returns: 0 on sucess, non-zero on failure.
+ */
+
+int gretl_chdir (const char *path)
+{
+    gchar *pconv = NULL;
+    int err;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(path, stdio_use_utf8, &pconv);
+
+    if (!err) {
+	if (pconv != NULL) {
+	    err = chdir(pconv);
+	    g_free(pconv);
+	} else {
+	    err = chdir(path);
+	}
+    }
+
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno("chdir");
+    }
+    
+    return err;
+}
+
+/**
+ * gretl_isdir:
+ * @path: path to check.
+ *
+ * A test for whether or not @path is the name of a directory,
+ * allowing for the possibility that @path has to be converted 
+ * from UTF-8 to the locale encoding or vice versa.
+ *
+ * Returns: 1 if @path is the name of a directory, else 0.
+ */
+
+int gretl_isdir (const char *path)
+{
+    struct stat buf;
+    int err;
+
+    err = gretl_stat(path, &buf);
+
+    return (err)? 0 : S_ISDIR(buf.st_mode);
+}
+
 #ifdef WIN32
 
 int gretl_mkdir (const char *path)
 {
+    gchar *pconv = NULL;
     DIR *test;
-    int done;
+    int done = 0;
 
-    test = win32_opendir(path);
-    if (test != NULL) {
-	closedir(test);
-	return 0;
+    if (string_is_utf8((unsigned char *) path)) {
+	gsize wrote;
+
+	pconv = g_locale_from_utf8(path, -1, NULL, &wrote, NULL);
     }
 
-    done = CreateDirectory(path, NULL);
+    if (pconv != NULL) {
+	test = win32_opendir(pconv);
+	if (test != NULL) {
+	    closedir(test);
+	    done = 1;
+	} else {
+	    done = CreateDirectory(pconv, NULL);
+	}
+	g_free(pconv);
+    } else {
+	test = win32_opendir(path);
+	if (test != NULL) {
+	    closedir(test);
+	    done = 1;
+	} else {
+	    done = CreateDirectory(path, NULL);
+	}
+    }
     
     return !done;
 }
 
-#else
+/**
+ * gretl_mkstemp:
+ * @tmpl: template filename.
+ *
+ * Returns: A file handle (as from open()) to the file opened for 
+ * reading and writing, or -1 on failure.
+ */
+
+int gretl_mkstemp (char *tmpl)
+{
+    int fd = -1;
+
+    if (!g_utf8_validate(tmpl, -1, NULL)) {
+	/* g_mkstemp requires UTF-8 input */
+	gchar *pconv;
+	gsize bytes;
+
+	pconv = g_locale_to_utf8(tmpl, -1, NULL, &bytes, NULL);
+	if (pconv != NULL) {
+	    strcpy(tmpl, pconv);
+	    fd = g_mkstemp(tmpl);
+	    g_free(pconv);
+	    pconv = g_locale_from_utf8(tmpl, -1, NULL, &bytes, NULL);
+	    if (pconv != NULL) {
+		strcpy(tmpl, pconv);
+		g_free(pconv);
+	    }
+	}	
+    } else {
+	fd = g_mkstemp(tmpl);
+    }
+
+    return fd;
+}
+
+#else /* !win32 */
 
 /**
  * gretl_mkdir:
@@ -273,13 +591,6 @@ static const char *gretl_readd (DIR *d)
     return (e == NULL)? NULL : e->d_name;
 }
 
-static int gretl_isdir (const char *path)
-{
-    struct stat buf;
-
-    return (stat(path, &buf) == 0 && S_ISDIR(buf.st_mode)); 
-}
-
 /* recursive deletion of directory tree: must be located
    in the directory above the one to be deleted at the
    outset */
@@ -314,14 +625,14 @@ int gretl_deltree (const char *path)
 		if (gretl_isdir(fname)) {
 		    err = gretl_deltree(fname);
 		} else {
-		    err = remove(fname);
+		    err = gretl_remove(fname);
 		}
 	    }
 	}
 	if (!err) {
 	    closedir(dir);
 	    chdir("..");
-	    err = remove(path);
+	    err = gretl_remove(path);
 	}
     }
 
@@ -345,18 +656,37 @@ int gretl_deltree (const char *path)
 
 int gretl_write_access (char *fname)
 {
-    int err = 0;
+    gchar *fconv = NULL;
+    int err;
 
-#ifdef WIN32
-    err = !win32_write_access(fname);
-#else
-    err = access(fname, W_OK);
-#endif
+    gretl_error_clear();
 
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
     if (err) {
-	sprintf(gretl_errmsg, "Can't write to %s", fname);
+	return err;
     }
 
+    if (fconv != NULL) {
+#ifdef WIN32
+	err = !win32_write_access(fconv);
+#else
+	err = access(fconv, W_OK);
+#endif
+	g_free(fconv);
+    } else {
+#ifdef WIN32
+	err = !win32_write_access(fname);
+#else
+	err = access(fname, W_OK);
+#endif
+    }
+
+#ifndef WIN32
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno(fname);
+    }
+#endif
+
     return err;
 }
 
@@ -454,13 +784,22 @@ static int try_open_file (char *targ, const char *finddir,
     return found;
 }
 
-static void make_finddir (char *targ, const char *src)
+static void make_findname (char *targ, const char *src)
 {
-    int n = strlen(src);
-
     strcpy(targ, src);
 
-    if (targ[n-1] != '\\') {
+    if (string_is_utf8(targ)) {
+	gchar *tmp;
+	gsize sz;
+	
+	tmp = g_locale_from_utf8(src, -1, NULL, &sz, NULL);
+	if (tmp != NULL) {
+	    strcpy(targ, tmp);
+	    g_free(tmp);
+	}
+    }
+
+    if (targ[strlen(targ)-1] != '\\') {
 	strcat(targ, "\\*");
     } else {
 	strcat(targ, "*");
@@ -489,7 +828,7 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
     int found = 0;
 
     /* make find target */
-    make_finddir(finddir, topdir);
+    make_findname(finddir, topdir);
 
     handle = FindFirstFile(finddir, &fdata); 
     if (handle != INVALID_HANDLE_VALUE) {
@@ -537,7 +876,7 @@ static int try_open_file (char *targ, const char *finddir,
     return found;
 }
 
-static void make_finddir (char *targ, const char *src)
+static void make_findname (char *targ, const char *src)
 {
     int n = strlen(src);
 
@@ -576,7 +915,7 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
     int found = 0;
 
     /* make find target */
-    make_finddir(finddir, topdir);
+    make_findname(finddir, topdir);
 
     dir = opendir(finddir);
     if (dir != NULL) {
@@ -1263,7 +1602,7 @@ static int validate_writedir (const char *dirname)
 		err = 1;
 	    } else {
 		fclose(fp);
-		remove(testname);
+		gretl_remove(testname);
 	    }
 	    g_free(testname);
 	}
@@ -1548,7 +1887,11 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	}
 
 	strcpy(ppaths->gnuplot, "gnuplot");
+#ifdef OSX_BUILD
+	strcpy(ppaths->pngfont, "Sans 9");
+#else
 	strcpy(ppaths->pngfont, "Vera 9");
+#endif
 	ppaths->currdir[0] = '\0';	
 	shelldir_init(NULL);
 
@@ -1657,16 +2000,49 @@ const char *gretl_maybe_switch_dir (const char *fname)
 	    char *sdir = get_shelldir();
 
 	    if (sdir != NULL && *sdir != '\0') {
-		chdir(sdir);
+		gretl_chdir(sdir);
 	    }
 	} else {
-	    chdir(gretl_paths.workdir);
+	    gretl_chdir(gretl_paths.workdir);
 	}
     }
 
     return fname;
 }
 
+/* argument should be of length FILENAME_MAX */
+
+char *gretl_maybe_prepend_dir (char *fname)
+{
+    char tmp[FILENAME_MAX];
+
+    *tmp = '\0';
+
+    if (fname[0] == '~' && fname[1] == '/') {
+	char *home = getenv("HOME");
+	
+	if (home != NULL) {
+	    build_path(tmp, home, fname + 2, NULL);
+	}
+    } else if (!g_path_is_absolute(fname)) {
+	if (dotpath(fname) || libset_get_bool(USE_CWD)) {
+	    char *sdir = get_shelldir();
+
+	    if (sdir != NULL && *sdir != '\0') {
+		build_path(tmp, sdir, fname, NULL);
+	    }
+	} else {
+	    build_path(tmp, gretl_paths.workdir, fname, NULL);
+	}
+    }
+
+    if (*tmp != '\0') {
+	strcpy(fname, tmp);
+    }
+
+    return fname;
+}
+
 /* remove '.' and '..' from @path */
 
 int gretl_normalize_path (char *path)
