@@ -20,6 +20,8 @@
 #ifndef GRETL_PATHS_H
 #define GRETL_PATHS_H
 
+#include <sys/stat.h>
+
 typedef enum {
     CURRENT_DIR,
     DATA_SEARCH,
@@ -36,24 +38,36 @@ int gretl_path_prepend (char *file, const char *path);
 
 int gretl_normalize_path (char *path);
 
-void set_fopen_use_utf8 (void);
+void set_stdio_use_utf8 (void);
 
-int get_fopen_use_utf8 (void);
+int get_stdio_use_utf8 (void);
 
-int fname_is_utf8 (const unsigned char *s);
+int string_is_utf8 (const unsigned char *s);
 
 FILE *gretl_fopen (const char *fname, const char *mode);
 
+int gretl_open (const char *pathname, int flags);
+
+int gretl_rename (const char *oldpath, const char *newpath);
+
+int gretl_remove (const char *path);
+
 gzFile gretl_gzopen (const char *fname, const char *mode);
 
+int gretl_stat (const char *fname, struct stat *buf);
+
 int gretl_mkdir (const char *path);
 
+int gretl_chdir (const char *path);
+
 int gretl_deltree (const char *path);
 
 int gretl_write_access (char *fname);
 
 int gretl_is_xml_file (const char *fname);
 
+int gretl_isdir (const char *path);
+
 char *addpath (char *fname, PATHS *ppaths, int script);
 
 int getopenfile (const char *line, char *fname, PATHS *ppaths,
@@ -80,6 +94,8 @@ int set_gretl_work_dir (const char *path, PATHS *ppaths);
 
 const char *gretl_maybe_switch_dir (const char *fname);
 
+char *gretl_maybe_prepend_dir (char *fname);
+
 const char *gretl_gnuplot_path (void);
 
 const char *gretl_plotfile (void);
@@ -96,7 +112,11 @@ const char *gretl_png_font (void);
 
 void set_gretl_png_font (const char *s, PATHS *ppaths);
 
-#ifndef WIN32
+#ifdef WIN32
+
+int gretl_mkstemp (char *tmpl);
+
+#else
 
 int cli_read_rc (PATHS *paths);
 
