@@ -60,7 +60,7 @@ void gretl_print_destroy (PRN *prn)
     }
 
     if (prn->fname != NULL) {
-	remove(prn->fname);
+	gretl_remove(prn->fname);
 	free(prn->fname);
     }
 
@@ -90,7 +90,7 @@ static int prn_add_tempfile (PRN *prn)
     sprintf(fname, "%sprntmp.XXXXXX", gretl_dot_dir());
 
 #ifdef WIN32
-    fd = g_mkstemp(fname);
+    fd = gretl_mkstemp(fname);
 #else
     fd = mkstemp(fname);
 #endif
@@ -111,7 +111,6 @@ static int prn_add_tempfile (PRN *prn)
 
     if (errno != 0) {
 	gretl_errmsg_set_from_errno("prn_add_tempfile");
-	
     }
 
     return err;
@@ -120,6 +119,7 @@ static int prn_add_tempfile (PRN *prn)
 static PRN *real_gretl_print_new (PrnType ptype, 
 				  const char *fname,
 				  char *buf,
+				  FILE *fp,
 				  int *perr)
 {
     PRN *prn = malloc(sizeof *prn);
@@ -133,8 +133,8 @@ static PRN *real_gretl_print_new (PrnType ptype,
     }
 
 #if PRN_DEBUG
-    fprintf(stderr, "real_gretl_print_new: type=%d, fname='%s', buf=%p\n",
-	    ptype, fname, (void *) buf);
+    fprintf(stderr, "real_gretl_print_new: type=%d, fname='%s', buf=%p, fp=%d\n",
+	    ptype, fname, (void *) buf, (void *) fp);
 #endif    
 
     prn->fp = NULL;
@@ -148,7 +148,9 @@ static PRN *real_gretl_print_new (PrnType ptype,
     prn->delim = ',';
     prn->fname = NULL;
 
-    if (ptype == GRETL_PRINT_FILE) {
+    if (ptype == GRETL_PRINT_STREAM) {
+	prn->fp = fp;
+    } else if (ptype == GRETL_PRINT_FILE) {
 	prn->fp = gretl_fopen(fname, "w");
 	if (prn->fp == NULL) {
 	    err = E_FOPEN;
@@ -225,7 +227,7 @@ PRN *gretl_print_new (PrnType ptype, int *err)
     fprintf(stderr, "gretl_print_new() called, type = %d\n", ptype);
 #endif
 
-    return real_gretl_print_new(ptype, NULL, NULL, err);
+    return real_gretl_print_new(ptype, NULL, NULL, NULL, err);
 }
 
 /**
@@ -246,7 +248,7 @@ PRN *gretl_print_new_with_filename (const char *fname, int *err)
 	return NULL;
     }
 
-    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL, err);
+    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL, NULL, err);
 }
 
 /**
@@ -262,7 +264,7 @@ PRN *gretl_print_new_with_filename (const char *fname, int *err)
 
 PRN *gretl_print_new_with_tempfile (int *err)
 {
-    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL, err);
+    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL, NULL, err);
 }
 
 /**
@@ -277,6 +279,23 @@ int gretl_print_has_tempfile (PRN *prn)
     return (prn != NULL && prn->fname != NULL && prn->fp != NULL);
 }
 
+/**
+ * gretl_print_get_tempfile_name:
+ * @prn: printing struct to test.
+ * 
+ * Returns: if @prn has a tempfile attached, return the name
+ * of that file, otherwise return %NULL.
+ */
+
+const char * gretl_print_get_tempfile_name (PRN *prn)
+{
+    if (prn != NULL) {
+	return prn->fname;
+    } else {
+	return NULL;
+    }
+}
+
 /**
  * gretl_print_new_with_buffer:
  * @buf: pre-allocated text buffer.
@@ -297,10 +316,83 @@ PRN *gretl_print_new_with_buffer (char *buf)
     if (buf == NULL) {
 	return NULL;
     } else {
-	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf, NULL);
+	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf, NULL, NULL);
     }
 }
 
+/**
+ * gretl_print_new_with_stream:
+ * @fp: pre-opened stream.
+ * 
+ * Creates and initializes a gretl printing struct, with
+ * printing to @fp.
+ *
+ * Note that @fp will be closed if and when #gretl_print_destroy
+ * is called on the #PRN pointer obtained.
+ *
+ * Returns: pointer to newly created struct, or %NULL on failure.
+ */
+
+PRN *gretl_print_new_with_stream (FILE *fp)
+{
+    if (fp == NULL) {
+	return NULL;
+    } else {
+	return real_gretl_print_new(GRETL_PRINT_STREAM, NULL, NULL, fp, NULL);
+    }
+}  
+
+/**
+ * gretl_print_rename_file:
+ * @prn: printing struct to operate on.
+ * @oldpath: name of current file (or %NULL if @prn was
+ * set up using #gretl_print_new_with_tempfile).
+ * @newpath: new name for file.
+ * 
+ * If @prn is printing to a %FILE pointer, rename the
+ * file to which it's printing and switch the stream
+ * to the new file.
+ *
+ * Returns: 0 on success, 1 on error.
+ */
+
+int gretl_print_rename_file (PRN *prn, const char *oldpath,
+			     const char *newpath)
+{
+    int err = 0;
+
+    if (prn == NULL) {
+	fprintf(stderr, "gretl_print_rename_file: prn is NULL\n");
+	return E_DATA;
+    }
+
+    if (prn->fp == NULL || prn->fpaux != NULL) {
+	return E_DATA;
+    }
+
+    fclose(prn->fp);
+
+    if (oldpath == NULL && prn->fname != NULL) {
+	/* renaming from tempfile */
+	err = gretl_rename(prn->fname, newpath);
+    } else {
+	err = gretl_rename(oldpath, newpath);
+    }
+
+    if (!err) {
+	prn->fp = gretl_fopen(newpath, "a");
+	if (prn->fname != NULL) {
+	    /* @prn originally used a tempfile: the record of
+	       the temporary filename should be deleted 
+	    */
+	    free(prn->fname);
+	    prn->fname = NULL;
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_print_reset_buffer:
  * @prn: printing struct to operate on.
@@ -317,6 +409,7 @@ int gretl_print_reset_buffer (PRN *prn)
 
     if (prn != NULL && prn->buf != NULL) {
 	*prn->buf = '\0';
+	prn->blen = 0;
     } else {
 	err = 1;
     }
@@ -690,7 +783,7 @@ static int pprintf_init (PRN *prn)
 /**
  * pprintf:
  * @prn: gretl printing struct.
- * @template: as in printf().
+ * @format: as in printf().
  * @Varargs: arguments to be printed.
  *
  * Multi-purpose printing function: can output to stream, to buffer
@@ -702,7 +795,7 @@ static int pprintf_init (PRN *prn)
  * failure.
  */
 
-int pprintf (PRN *prn, const char *template, ...)
+int pprintf (PRN *prn, const char *format, ...)
 {
     va_list args;
     int rem, plen = 0;
@@ -712,13 +805,14 @@ int pprintf (PRN *prn, const char *template, ...)
     }
 
     if (prn->fp != NULL) {
-	va_start(args, template);
-	plen = vfprintf(prn->fp, template, args);
+	/* printing to stream: straightforward */
+	va_start(args, format);
+	plen = vfprintf(prn->fp, format, args);
 	va_end(args);
 	return plen;
     }
 
-    if (strncmp(template, "@init", 5) == 0) {
+    if (strncmp(format, "@init", 5) == 0) {
 	return pprintf_init(prn);
     }
 
@@ -736,9 +830,10 @@ int pprintf (PRN *prn, const char *template, ...)
     fprintf(stderr, "printing at %p\n", (void *) (prn->buf + prn->blen));
 #endif
 
+    /* printing to buffer: be careful not to overrun */
     rem = prn->bufsize - prn->blen - 1;
-    va_start(args, template);
-    plen = vsnprintf(prn->buf + prn->blen, rem, template, args);
+    va_start(args, format);
+    plen = vsnprintf(prn->buf + prn->blen, rem, format, args);
     va_end(args);
 
     if (plen >= rem) {
