@@ -25,7 +25,8 @@ typedef enum {
     GRETL_PRINT_STDERR,
     GRETL_PRINT_FILE,
     GRETL_PRINT_BUFFER,
-    GRETL_PRINT_TEMPFILE
+    GRETL_PRINT_TEMPFILE,
+    GRETL_PRINT_STREAM
 } PrnType;
 
 typedef enum {
@@ -55,10 +56,17 @@ PRN *gretl_print_new_with_tempfile (int *err);
 
 int gretl_print_has_tempfile (PRN *prn);
 
+const char *gretl_print_get_tempfile_name (PRN *prn);
+
 PRN *gretl_print_new_with_buffer (char *buf);
 
+PRN *gretl_print_new_with_stream (FILE *fp);
+
 int gretl_print_reset_buffer (PRN *prn);
 
+int gretl_print_rename_file (PRN *prn, const char *oldpath, 
+			     const char *newpath);
+
 const char *gretl_print_get_buffer (PRN *prn);
 
 const char *gretl_print_get_trimmed_buffer (PRN *prn);
@@ -87,7 +95,7 @@ int gretl_print_supports_utf (PRN *prn);
 
 void gretl_print_set_delim (PRN *prn, char delim);
 
-int pprintf (PRN *prn, const char *template, ...);
+int pprintf (PRN *prn, const char *format, ...);
 
 int pputs (PRN *prn, const char *s);
 
