@@ -25,6 +25,7 @@
 #include "matrix_extra.h"
 #include "gretl_restrict.h"
 #include "bootstrap.h"
+#include "gretl_scalar.h"
 
 #define RDEBUG 0
 
@@ -925,11 +926,39 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
     return err;
 }
 
-#define ok_letter(r, c) (c == 'b' || (r->vecm && c == 'a'))
+static int r_get_scalar (const char *s, double *x)
+{
+    if (sscanf(s, "%lf", x)) {
+	return 1;
+    } else {
+	int n;
+
+	s += strspn(s, " ");
+	n = gretl_namechar_spn(s);
+
+	if (n > 0) {
+	    char tmp[VNAMELEN];
+
+	    *tmp = '\0';
+	    strncat(tmp, s, n);
+	    if (gretl_is_scalar(tmp)) {
+		*x = gretl_scalar_get_value(tmp);
+		if (!na(*x)) {
+		    return 1;
+		}
+	    } 
+	}
+    }
+
+    return 0;
+}
+
+#define b_start(r,s) ((*s == 'b' || (r->vecm && *s == 'a')) && \
+		      (isdigit(*(s+1)) || *(s+1) == '['))
 
 static int 
 parse_coeff_chunk (gretl_restriction *r, const char *s, double *x, 
-		   int *eq, int *bnum, char *letter,
+		   int *eq, int *bnum, char *letter, 
 		   const DATAINFO *pdinfo)
 {
     const char *s0 = s;
@@ -943,7 +972,7 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
     fprintf(stderr, "parse_coeff_chunk: s='%s'\n", s);
 #endif
 
-    if (ok_letter(r, *s)) {
+    if (b_start(r, s)) {
 	*letter = *s;
 	s++;
 #if RDEBUG
@@ -951,14 +980,19 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
 #endif
 	err = parse_b_bit(r, s, eq, bnum, pdinfo);
 	*x = 1.0;
-    } else if (sscanf(s, "%lf", x)) {
+    } else if (r_get_scalar(s, x)) {
 	s += strspn(s, " ");
 	s += strcspn(s, " *");
 	s += strspn(s, " *");
-	if (ok_letter(r, *s)) {
+	if (b_start(r, s)) {
 	    *letter = *s;
 	    s++;
 	    err = parse_b_bit(r, s, eq, bnum, pdinfo);
+	} else if (*s == '\0') {
+	    /* plain numeric value, saved in *x */
+	    *bnum = 0;
+	    *letter = '\0';
+	    err = 0;
 	}
     }
 
@@ -1118,7 +1152,11 @@ static void print_restriction (const gretl_restriction *rset,
 	    MODEL *pmod = rset->obj;
 
 	    gretl_model_get_param_name(pmod, pdinfo, k, vname);
-	    pprintf(prn, "b[%s]", vname);
+	    if (NONLIST_MODEL(pmod->ci)) {
+		pputs(prn, vname);
+	    } else {
+		pprintf(prn, "b[%s]", vname);
+	    }
 	}
     }
 
@@ -1414,7 +1452,7 @@ real_restriction_set_parse_line (gretl_restriction *rset,
     const char *p = line;
     rrow *r;
     int sgn = 1;
-    int i, nt, err = 0;
+    int i, j, nt, err = 0;
 
     gretl_error_clear();
 
@@ -1460,9 +1498,12 @@ real_restriction_set_parse_line (gretl_restriction *rset,
 	return E_ALLOC;
     }
 
+    j = 0;
+
     for (i=0; i<nt; i++) {
 	char chunk[32];
 	int len, bnum = 1, eq = 1;
+	int numeric = 0;
 	char letter = 'b';
 	double mult;
 
@@ -1484,13 +1525,22 @@ real_restriction_set_parse_line (gretl_restriction *rset,
 				&letter, pdinfo);
 	if (err) {
 	    break;
+	} else if (letter == '\0') {
+	    numeric = 1;
 	} else if (bnum_out_of_bounds(rset, eq, bnum, letter)) {
 	    err = E_DATA;
 	    break;
 	}
 
 	mult *= sgn;
-	add_term_to_restriction(r, mult, eq, bnum, letter, i);
+
+	if (numeric) {
+	    /* got a numeric constant */
+	    r->rhs -= mult;
+	    r->nterms -= 1;
+	} else {
+	    add_term_to_restriction(r, mult, eq, bnum, letter, j++);
+	}
 
 	if (*p == '+') {
 	    sgn = 1.0;
@@ -1502,9 +1552,13 @@ real_restriction_set_parse_line (gretl_restriction *rset,
     }
 
     if (!err) {
-	if (!sscanf(p, " = %lf", &r->rhs)) {
+	double rhs = 0.0;
+
+	if (!sscanf(p, " = %lf", &rhs)) {
 	    err = E_PARSE;
-	} 
+	} else {
+	    r->rhs += rhs;
+	}
     }
 
     if (err) {
@@ -1690,8 +1744,8 @@ static int print_restricted_estimates (MODEL *pmod,
 	gretl_model_get_param_name(pmod, pdinfo, i, names[i]);
     }
 
-    plain_print_aux_coeffs(b, se, (const char **) names, nc, pmod->dfd + rk, 
-			   pmod->ci, prn);
+    print_coeffs(b, se, (const char **) names, nc, pmod->dfd + rk, 
+		 pmod->ci, prn);
 
     pputc(prn, '\n');
     pprintf(prn, "  %s = %.*g\n", _("Standard error of the regression"), 
@@ -1792,7 +1846,7 @@ restriction_set_print_result (gretl_restriction *rset,
     MODEL *pmod = rset->obj;
     int robust, asym;
 
-    robust = gretl_model_get_int(pmod, "robust");
+    robust = (pmod->opt & OPT_R);
     asym = ASYMPTOTIC_MODEL(pmod->ci);
 
     if (asym) {
