@@ -25,10 +25,10 @@
 #include "libset.h"
 #include "usermat.h"
 #include "gretl_panel.h"
-#include "loop_private.h"
 #include "gretl_scalar.h"
 #include "gretl_string_table.h"
 #include "forecast.h"
+#include "kalman.h"
 
 #include <errno.h>
 #include <time.h>
@@ -287,32 +287,35 @@ int gretl_isint (int t1, int t2, const double *x)
 
 #define FEWVALS 32
 
-static int few_vals (int t1, int t2, const double *x)
+static int few_vals (int t1, int t2, const double *x, double *ratio)
 {
     double test[FEWVALS];
     int match;
-    int i, t, n = 0;
+    int i, t, n = 0, nv = 0;
 
     for (t=t1; t<=t2; t++) {
 	if (!na(x[t])) {
 	    match = 0;
-	    for (i=0; i<n; i++) {
+	    for (i=0; i<nv; i++) {
 		if (x[t] == test[i]) {
 		    match = 1;
 		    break;
 		}
 	    }
 	    if (!match) {
-		if (n == FEWVALS) {
-		    n++;
+		if (nv == FEWVALS) {
+		    nv++;
 		    break;
 		}
-		test[n++] = x[t];
+		test[nv++] = x[t];
 	    }
+	    n++;
 	}
     }
 
-    return n;
+    *ratio = (double) nv / n;
+
+    return nv;
 }
 
  /**
@@ -362,9 +365,15 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
     }
 
     if (d) {
-	n = few_vals(t1, t2, x);
+	n = few_vals(t1, t2, x, &r);
 	if (n > FEWVALS) {
 	    d = 0;
+	} else if (r > 0.9) {
+	    /* somewhat arbitrary: but if r (= ratio of distinct
+	       values to number of cases) is "too high", perhaps we
+	       should not take the var as discrete
+	    */
+	    d = 0;
 	} else if (n < 5) {
 	    d = 2;
 	}
@@ -574,11 +583,15 @@ void printlist (const int *list, const char *msg)
     }
 
     if (list == NULL) {
-	fputs( "list is NULL", stderr);
+	fputs("list is NULL", stderr);
     } else {
 	fprintf(stderr, "%d : ", list[0]);
 	for (i=1; i<=list[0]; i++) {
-	    fprintf(stderr, "%d ", list[i]);
+	    if (list[i] == LISTSEP) {
+		fputs("; ", stderr);
+	    } else {
+		fprintf(stderr, "%d ", list[i]);
+	    }
 	}
     }
 
@@ -823,7 +836,7 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 /**
  * set_obs:
  * @line: command line.
- * @Z: data array.
+ * @pZ: pointer to data array.
  * @pdinfo: data information struct.
  * @opt: %OPT_S for stacked time-series, %OPT_C for stacked cross-section,
  * %OPT_T for time series, %OPT_X for cross section, %OPT_P to set
@@ -835,7 +848,7 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int set_obs (const char *line, double **Z, DATAINFO *pdinfo, 
+int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
 	     gretlopt opt)
 {
     char pdstr[VNAMELEN];
@@ -845,11 +858,19 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
     int pd, dated = 0;
     int err = 0;
 
+    if (pZ == NULL || pdinfo == NULL) {
+	return E_NODATA;
+    }
+
+    if ((opt & (OPT_R | OPT_P)) && *pZ == NULL) {
+	return E_NODATA;
+    }
+
     gretl_error_clear();
 
     if (opt & OPT_R) {
 	/* restructure panel: "hidden" option */
-	return switch_panel_orientation(Z, pdinfo);
+	return switch_panel_orientation(*pZ, pdinfo);
     }    
 
     if (!strcmp(line, "setobs")) {
@@ -858,7 +879,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
     }
 
     if (opt & OPT_P) {
-	return set_panel_structure_from_line(line, Z, pdinfo);
+	return set_panel_structure_from_line(line, *pZ, pdinfo);
     }
 
     /* now we get down to business */
@@ -984,7 +1005,11 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
        stacked time series form
     */
     if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	err = switch_panel_orientation(Z, pdinfo);
+	if (*pZ == NULL) {
+	    err = E_NODATA;
+	} else {
+	    err = switch_panel_orientation(*pZ, pdinfo);
+	}
     }	
 
     if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
@@ -1077,8 +1102,6 @@ int gretl_int_from_string (const char *s, int *err)
 
     if (*test == '\0') {
 	return n;
-    } else if (s[1] == '\0' && is_active_index_loop_char(*s)) {
-	n = loop_scalar_read(*s);
     } else if (gretl_is_scalar(s)) {
 	x = gretl_scalar_get_value(s);
 	if (na(x)) {
@@ -1326,84 +1349,6 @@ double **data_array_from_model (const MODEL *pmod, double **Z, int missv)
     return X;
 }
 
-/**
- * re_estimate:
- * @model_spec: estimation command line.
- * @tmpmod: pointer to model to recieve results.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- *
- * Returns: 0 on success, non-zero on failure.
- */
-
-int re_estimate (char *model_spec, MODEL *tmpmod, 
-		 double ***pZ, DATAINFO *pdinfo) 
-{
-    CMD cmd;
-    double rho = 0.0;
-    int err = 0;
-
-    if (gretl_cmd_init(&cmd)) {
-	return 1;
-    }
-
-    err = parse_command_line(model_spec, &cmd, pZ, pdinfo);
-    if (err) {
-	gretl_cmd_free(&cmd);
-	return err;
-    }
-
-    gretl_model_init(tmpmod);
-
-    switch (cmd.ci) {
-    case AR:
-	*tmpmod = ar_func(cmd.list, pZ, pdinfo, OPT_NONE, NULL);
-	break;
-    case AR1:
-	rho = estimate_rho(cmd.list, pZ, pdinfo, cmd.opt, NULL, &err);
-	if (!err) {
-	    *tmpmod = ar1_lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, rho);
-	}
-	break;
-    case HSK:
-	*tmpmod = hsk_func(cmd.list, pZ, pdinfo);
-	break;
-    case LOGIT:
-    case PROBIT:
-	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, 
-			       NULL);
-	break;
-    case TOBIT:
-	*tmpmod = tobit_model(cmd.list, pZ, pdinfo, NULL);
-	break;
-    case POISSON:
-	*tmpmod = poisson_model(cmd.list, pZ, pdinfo, NULL);
-	break;
-    case HECKIT:
-	*tmpmod = heckit_model(cmd.list, pZ, pdinfo, cmd.opt, NULL);
-	break;
-    case OLS:
-    case WLS:
-    case HCCM:
-	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
-	break;
-    case TSLS:
-	break;
-    default:
-	break;
-    }
-
-    if (tmpmod->errcode) {
-	err = 1;
-	clear_model(tmpmod);
-    }
-
-    gretl_cmd_free(&cmd);
-
-    return err;
-}
-
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 {
     return Z[i][pdinfo->t1];
@@ -1417,11 +1362,14 @@ static int non_fatal (const char *s)
        "Could not find/open font when opening font X, using default" 
        "gnuplot_x11: Some character sets not available" 
        "Warning: empty y2 range..."
+       pango warning for, e.g., FreeSans font w/o GPOS table
     */
 
-    if (strstr(s, "using default") || 
+    if (strstr(s, "using default") ||
+	strstr(s, "trying default") ||
 	strstr(s, "character sets not available") ||
-	strstr(s, "Warning: empty ")) {
+	strstr(s, "Warning: empty ") ||
+	strstr(s, "Pango-WARNING")) {
 	return 1;
     } else {
 	return 0;
@@ -1516,9 +1464,11 @@ int gretl_copy_file (const char *src, const char *dest)
 
 int gretl_delete_var_by_name (const char *s, PRN *prn)
 {
-    int err;
+    int err = 0;
 
-    if (gretl_is_scalar(s)) {
+    if (!strcmp(s, "kalman")) {
+	err = delete_kalman(prn);	
+    } else if (gretl_is_scalar(s)) {
 	err = gretl_scalar_delete(s, prn);
     } else if (get_matrix_by_name(s)) {
 	err = user_matrix_destroy_by_name(s, prn);
@@ -1586,13 +1536,15 @@ void libgretl_cleanup (void)
     saved_strings_cleanup();
     lapack_mem_free();
     forecast_matrix_cleanup();
+    option_flags_cleanup();
+    kalman_cleanup();
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
 	int pnum;
 
 	if (!sscanf(p, "gpttmp%d.plt", &pnum)) {
-	    remove(gretl_plotfile());
+	    gretl_remove(gretl_plotfile());
 	}
     }
 }
