@@ -46,7 +46,7 @@ int read_reg_val (HKEY tree, const char *base,
                      KEY_READ,                    /* access mask */
                      &regkey                      /* key handle */
                      ) != ERROR_SUCCESS) {
-        fprintf(stderr, _("Couldn't open registry\n"));
+        fprintf(stderr, "Couldn't read registry path %s\n", regpath);
         return 1;
     }
 
@@ -62,6 +62,8 @@ int read_reg_val (HKEY tree, const char *base,
 	err = 1;
     }
 
+    if (0) ;
+
     RegCloseKey(regkey);
 
     return err;
@@ -80,13 +82,30 @@ int read_reg_val_with_fallback (HKEY tree0, HKEY tree1, const char *base,
     return err;
 }
 
-int write_reg_val (HKEY tree, const char *base, 
-		   const char *keyname, const char *keyval)
+static gchar *get_windows_string (char *s)
 {
+    gsize bytes;
+
+    return g_locale_from_utf8(s, -1, NULL, &bytes, NULL);
+}
+
+int write_reg_val (HKEY tree, const char *base, 
+		   const char *keyname, char *keyval,
+		   int ktype)
+{
+    gchar *tmp = NULL;
     char regpath[64];
     int err = 0;
     HKEY regkey;
 
+    /* ensure that strings are in the locale encoding */
+    if (ktype == GRETL_TYPE_STRING && string_is_utf8(keyval)) {
+	tmp = get_windows_string(keyval);
+	if (tmp != NULL) {
+	    keyval = tmp;
+	}
+    }
+
     sprintf(regpath, "Software\\%s", base);
 
     if (RegCreateKeyEx(
@@ -114,6 +133,10 @@ int write_reg_val (HKEY tree, const char *base,
 	fprintf(stderr, "RegSetValueEx: failed on '%s'\n", keyname);
         err = 1;
     }
+
+    if (tmp != NULL) {
+	g_free(tmp);
+    }
                   
     RegCloseKey(regkey);
 
@@ -142,6 +165,22 @@ DIR *win32_opendir (const char *dname)
     return opendir(tmp);
 }
 
+/* 
+   relevant extract from gretl.iss:
+
+   HKLM; "Software\gretl"; "gretldir";   "{app}"
+   HKLM; "Software\gretl"; "Rcommand";   "RGui.exe"
+   HKCU; "Software\gretl"; "binbase";    "{app}\db\"
+   HKCU; "Software\gretl"; "ratsbase";   "f:\"
+   HKCU; "Software\gretl"; "dbhost";     "ricardo.ecn.wfu.edu"
+   HKCU; "Software\gretl"; "dbproxy";    ""
+   HKCU; "Software\gretl"; "useproxy";   "false"
+   HKCU; "Software\gretl"; "updater";    "false"
+   HKCU; "Software\gretl"; "Fixed_font"; "Courier New 10"
+   HKCU; "Software\gretl"; "Png_font";   "verdana 8"
+   HKCU; "Software\gretl"; "Gp_colors";  ""
+*/
+
 void cli_read_registry (char *callname, PATHS *ppaths)
 {
     char valstr[MAXLEN];
@@ -150,12 +189,14 @@ void cli_read_registry (char *callname, PATHS *ppaths)
     char *tmp;
     int drive = callname[0];
 
+    /* gretl installation directory */
     ppaths->gretldir[0] = '\0';
     read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", ppaths->gretldir);
     if (ppaths->gretldir[0] == '\0') {
 	sprintf(ppaths->gretldir, "%c:\\userdata\\gretl\\", drive);
     }
 
+    /* user's working directory */
     ppaths->workdir[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "userdir", ppaths->workdir);
     if (ppaths->workdir[0] == '\0') {
@@ -169,6 +210,7 @@ void cli_read_registry (char *callname, PATHS *ppaths)
 	}
     }
 
+    /* "hidden" working dir: not user-configurable */
     ppaths->dotdir[0] = '\0';
     tmp = appdata_path();
     if (tmp != NULL) {
@@ -178,15 +220,18 @@ void cli_read_registry (char *callname, PATHS *ppaths)
 	sprintf(ppaths->dotdir, "%c:\\userdata\\gretl\\user\\", drive);
     }    
 
+    /* base path for databases */
     ppaths->binbase[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "binbase", ppaths->binbase);
     if (ppaths->binbase[0] == '\0') {
 	sprintf(ppaths->binbase, "%c:\\userdata\\gretl\\db", drive);
     }
 
+    /* base path for RATS databases */
     ppaths->ratsbase[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "ratsbase", ppaths->ratsbase);
 
+    /* path to X-12-ARIMA */
     ppaths->x12a[0] = '\0';
     read_reg_val_with_fallback(HKEY_LOCAL_MACHINE, HKEY_CLASSES_ROOT,
 			       "x12arima", "x12a", ppaths->x12a);
@@ -194,21 +239,25 @@ void cli_read_registry (char *callname, PATHS *ppaths)
 	sprintf(ppaths->x12a, "%c:\\userdata\\x12arima\\x12a.exe", drive);
     }
 
+    /* remote database host */
     ppaths->dbhost[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "dbhost", ppaths->dbhost);
     if (ppaths->dbhost[0] == '\0') {
 	strcpy(ppaths->dbhost, "ricardo.ecn.wfu.edu");
     }
 
+    /* www proxy for reading remote databases */
     dbproxy[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "dbproxy", dbproxy);
 
+    /* should a proxy be used? */
     valstr[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "use_proxy", valstr);
     if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
 	use_proxy = 1;
     } 
 
+    /* do we allow the shell command within gretl? */
     valstr[0] = '\0';
     read_reg_val(HKEY_CURRENT_USER, "gretl", "shellok", valstr);
     if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
@@ -314,6 +363,10 @@ int gretl_spawn (char *cmdline)
     return winfork(cmdline, NULL, SW_SHOWMINIMIZED, 0);
 }
 
+/* Retrieve various special paths from the bowels of MS
+   Windows.  Note that these paths will be in the locale
+   encoding, not UTF-8 */
+
 static char *win_special_path (int folder)
 {
     TCHAR dpath[MAX_PATH];
@@ -643,22 +696,6 @@ int win32_write_access (char *path)
     return ok;
 }
 
-int win32_rename (const char *oldpath, const char *newpath)
-{
-    int err = 0;
-
-    if (strcmp(oldpath, newpath)) {
-	if (CopyFile(oldpath, newpath, FALSE) == 0) {
-	    err = 1;
-	    win_show_last_error();
-	} else {
-	    DeleteFile(oldpath);
-	}
-    }
-
-    return err;
-}
-
 int win32_delete_dir (const char *path)
 {
     SHFILEOPSTRUCT op;
