@@ -229,6 +229,10 @@ typedef enum {
 # endif
 #endif /* WIN32 */
 
+#ifdef STANDALONE
+# define gretl_remove(p) remove(p)
+#endif
+
 struct proto {
     char *name;
     uerr_t ind;
@@ -246,6 +250,7 @@ struct urlinfo_ {
     char *params;            /* parameters to be passed to CGI */
     char *localfile;         /* name of local file to write or NULL */
     char *getbuf;            /* buffer to which to write result or NULL */
+    char *gottype;           /* record of content type from server */
     const char *upload;      /* content of file to upload */
     int upsize;              /* size of the above */
     char agent[32];          /* to communicate gretl version */
@@ -338,6 +343,12 @@ static int iread (int fd, char *buf, int len);
 static int iwrite (int fd, const char *buf, int len);
 static char *print_option (int opt);
 
+#ifdef STANDALONE
+# define errmsg_set(s) fprintf(stderr, s)
+#else
+# define errmsg_set(s) gretl_errmsg_set(s)
+#endif
+
 static char *w_strdup (const char *src)
 {
     char *targ = NULL;
@@ -759,11 +770,15 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
 
     if (sp_ret == SP_RETURN_CANCELED) goto canceled;
 
-    /* Read from fd while there is available data. */
+    /* read from fd while there is available data */
+
     nchunks = 1;
     allocated = WBUFSIZE;
+
     do {
+	fprintf(stderr, "get_contents: calling iread...\n");
 	res = iread(fd, cbuf, sizeof cbuf);
+	fprintf(stderr, " got %d bytes\n", (int) res);
 	if (res > 0) {
 	    if (fp == NULL) {
 		if ((size_t) (*len + res) > allocated) {
@@ -780,6 +795,7 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
 		    return -2;
 		}
 	    }
+
 	    *len += res;
 
 	    if (show) {
@@ -832,6 +848,10 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 
     err = make_connection(&sock, conn->host, conn->port);
 
+#if WDEBUG
+    fprintf(stderr, "make_connection returned %d\n", err);
+#endif   
+
     switch (err) {
     case HOSTERR:
 	if (*conn->host != '\0') {
@@ -898,7 +918,6 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 	return NOTENOUGHMEM;
     }
 
-#if 1 /* changed 2008-09-23: use host name here, not IP:port */
     if (u->params != NULL && u->upload == NULL) {
 	sprintf(request, "%s %s?%s HTTP/1.0\r\n"
 		"User-Agent: %s\r\n"
@@ -912,21 +931,6 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 		"Accept: %s\r\n",
 		cmd, u->path, u->agent, u->hostname, HTTP_ACCEPT);
     }
-#else
-    if (u->params != NULL && u->upload == NULL) {
-	sprintf(request, "%s %s?%s HTTP/1.0\r\n"
-		"User-Agent: %s\r\n"
-		"Host: %s:%d\r\n"
-		"Accept: %s\r\n",
-		cmd, u->path, u->params, u->agent, u->host, u->port, HTTP_ACCEPT);
-    } else {
-	sprintf(request, "%s %s HTTP/1.0\r\n"
-		"User-Agent: %s\r\n"
-		"Host: %s:%d\r\n"
-		"Accept: %s\r\n",
-		cmd, u->path, u->agent, u->host, u->port, HTTP_ACCEPT);
-    }
-#endif
 
     if (posthead != NULL) {
 	strcat(request, posthead);
@@ -1132,7 +1136,10 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 	}
     }
 
-    free(type);
+    if (u->gottype != NULL) {
+	free(u->gottype);
+    }
+    u->gottype = type;
     type = NULL;	
 
     /* Return if we have no intention of further downloading */
@@ -1167,11 +1174,17 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 static uerr_t try_http (urlinfo *u)
 {
     struct http_stat hstat;
-    int i, dt = 0 | ACCEPTRANGES;
+    int i, dt, got_newloc = 0;
+    int redirs = 0;
     uerr_t err;
 
+ start_again:
+
+    dt = 0 | ACCEPTRANGES;
+    got_newloc = 0;
+
 #if WDEBUG
-    fprintf(stderr, "try_http: u->path = '%s'\n", u->path);
+    fprintf(stderr, "*** try_http: u->path = '%s'\n", u->path);
 #endif
 
     for (i=0; i<MAXTRY; i++) {
@@ -1179,7 +1192,7 @@ static uerr_t try_http (urlinfo *u)
 	err = real_get_http(u, &hstat, &dt);
 
 #if WDEBUG
-	fprintf(stderr, "try_http: err (from real_get_http) = %d\n"
+	fprintf(stderr, "try_http: real_get_http returned %d\n"
 		" errbuf = '%s'\n", err,
 		(wproxy)? gretlproxy.errbuf : u->errbuf);
 	if (err == RETRFINISHED) {
@@ -1200,11 +1213,31 @@ static uerr_t try_http (urlinfo *u)
 	    return err;
 	case RETRFINISHED:
 	    break;
+	case NEWLOCATION: /* ?? */
+	    if (redirs == 0 && hstat.newloc != NULL) {
+		char *p;
+
+		free(u->path);
+		u->path = hstat.newloc;
+		p = strchr(u->path, '?');
+		if (p != NULL) {
+		    *p = '\0';
+		}
+		hstat.newloc = NULL;
+		got_newloc = 1;
+		redirs++;
+	    }
+	    break;
 	default:
 	    FREEHSTAT(hstat);
 	    return err;
 	}
 
+	if (got_newloc) {
+	    FREEHSTAT(hstat);
+	    break;
+	}
+
 	if (!(dt & RETROKF)) {
 	    FREEHSTAT(hstat);
 	    return WRONGCODE;
@@ -1228,7 +1261,11 @@ static uerr_t try_http (urlinfo *u)
 	    }
 	}
 	break;
-    } 
+    }
+    
+    if (got_newloc) {
+	goto start_again;
+    }
 
     return TRYLIMEXC;
 }
@@ -1272,6 +1309,7 @@ static void url_init (urlinfo *u)
     u->params = NULL;
     u->localfile = NULL;
     u->getbuf = NULL;
+    u->gottype = NULL;
 
     u->proto = URLHTTP;
     u->port = DEFAULT_HTTP_PORT;
@@ -1315,13 +1353,14 @@ static void urlinfo_destroy (urlinfo *u, int delfile)
     free(u->url);
     free(u->path);
     free(u->params);
+    free(u->gottype);
 
     if (u->localfile != NULL) {
 	if (u->fp != NULL) {
 	    fclose(u->fp);
 	}
 	if (delfile) {
-	    remove(u->localfile);
+	    gretl_remove(u->localfile);
 	}
 	free(u->localfile);
     }
@@ -1440,6 +1479,10 @@ static char *print_option (int opt)
 	return "UPLOAD";
     case CHECK_DB:
 	return "CHECK_DB";
+    case LIST_PKGS:
+	return "LIST_PKGS";
+    case GRAB_PKG:
+	return "GRAB_PKG";
     default:
 	break;
     }
@@ -1520,6 +1563,8 @@ urlinfo_set_params (urlinfo *u, CGIOpt opt, const char *fname,
     if (fname != NULL) {
 	if (opt == GRAB_FILE || opt == GRAB_FUNC) {
 	    strcat(u->params, "&fname=");
+	} else if (opt == GRAB_PKG) {
+	    strcat(u->params, "&pkg=");
 	} else {
 	    strcat(u->params, "&dbase=");
 	}
@@ -1534,6 +1579,33 @@ urlinfo_set_params (urlinfo *u, CGIOpt opt, const char *fname,
     return (u->params != NULL);
 }
 
+/* when we're expecting binary data, check that we didn't get
+   text/plain instead, which presumably in fact means we got
+   an error message from the server
+*/
+
+static int maybe_register_type_error (urlinfo *u, CGIOpt opt)
+{
+    int err = 0;
+
+    if ((opt == GRAB_PKG || opt == GRAB_PDF) && 
+	!strcmp(u->gottype, "text/plain")) {
+	/* we shouldn't have got plain text */
+	err = 1;
+	if (u->fp != NULL) {
+	    char buf[256];
+
+	    fclose(u->fp);
+	    u->fp = fopen(u->localfile, "r");
+	    if (fgets(buf, sizeof buf, u->fp)) {
+		fprintf(stderr, "server says: '%s'\n", buf);
+	    }
+	}
+    }
+    
+    return err;
+}
+
 /* grab data from URL.  If saveopt = SAVE_TO_FILE then data is stored
    to a local file whose name is given by "savefile".  If saveopt =
    SAVE_TO_BUFFER then "getbuf" is presumed to point to a char buffer
@@ -1618,6 +1690,10 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 	*getbuf = u->getbuf;
     }    
 
+    if (!err && u->gottype != NULL && saveopt == SAVE_TO_FILE) {
+	err = maybe_register_type_error(u, opt);
+    }
+
     urlinfo_destroy(u, err);
 
     return err;
@@ -1830,6 +1906,12 @@ int list_remote_function_packages (char **getbuf)
 			 NULL, getbuf);
 }
 
+int list_remote_data_packages (char **getbuf)
+{
+    return retrieve_url (GRETLHOST, LIST_PKGS, NULL, NULL, SAVE_TO_BUFFER, 
+			 NULL, getbuf);
+}
+
 int retrieve_remote_db_index (const char *dbname, char **getbuf) 
 {
     return retrieve_url (dbhost, GRAB_IDX, dbname, NULL, SAVE_TO_BUFFER, 
@@ -1872,6 +1954,13 @@ int retrieve_remote_function_package (const char *pkgname,
 			 localname, NULL);
 }
 
+int retrieve_remote_datafiles_package (const char *pkgname, 
+				       const char *localname)
+{
+    return retrieve_url (GRETLHOST, GRAB_PKG, pkgname, NULL, SAVE_TO_FILE, 
+			 localname, NULL);
+}
+
 int retrieve_remote_db_data (const char *dbname,
 			     const char *varname,
 			     char **getbuf,
