@@ -385,7 +385,11 @@ void gretl_xml_put_tagged_list (const char *tag, const int *list, FILE *fp)
 
     fprintf(fp, "<%s>\n", tag);
     for (i=0; i<=list[0]; i++) {
-	fprintf(fp, "%d ", list[i]);
+	if (list[i] == LISTSEP) {
+	    fputs("; ", fp);
+	} else {
+	    fprintf(fp, "%d ", list[i]);
+	}
     }
     fprintf(fp, "</%s>\n", tag); 
 }
@@ -673,7 +677,7 @@ int gretl_xml_node_get_string (xmlNodePtr node, xmlDocPtr doc,
  * @pstr: location to receive string.
  * 
  * Reads a string from @node and trims both leading and trailing
- * whit space.
+ * white space.
  * 
  * Returns: 1 if a string is found and read successfully, 0
  * otherwise.
@@ -738,7 +742,7 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
 	*err = E_DATA;
     } else {
 	p = (const char *) tmp;
-	p += strspn(p, " \r\n");
+	p += strspn(p, " \r\n"); /* skip space (get to first value) */
 	if (sscanf(p, "%d", &n) != 1) {
 	    *err = E_DATA;
 	} else if (n == 0) {
@@ -747,21 +751,25 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
 	} else if (n < 0) {
 	    *err = E_DATA;
 	} else {
-	    p += strcspn(p, " \r\n");
+	    p += strcspn(p, " \r\n"); /* skip non-space (get beyond value) */
 	    list = gretl_list_new(n);
 	    if (list == NULL) {
 		*err = E_ALLOC;
 	    }
 	}
+
 	if (list != NULL && !*err) {
 	    for (i=1; i<=n && !*err; i++) {
-		if (sscanf(p, "%d", &list[i]) != 1) {
+		p += strspn(p, " \r\n"); /* skip space (get to next value) */
+		if (*p == ';') {
+		    list[i] = LISTSEP;
+		} else if (sscanf(p, "%d", &list[i]) != 1) {
 		    *err = E_DATA;
 		}
-		p += strspn(p, " \r\n");
-		p += strcspn(p, " \r\n");
+		p += strcspn(p, " \r\n"); /* skip non-space (get beyond value) */
 	    }
 	}
+
 	free(tmp);
     }
 
@@ -770,6 +778,8 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
 	list = NULL;
     }
 
+    fprintf(stderr, "returning list = %p\n", (void *) list);
+
     return list;
 }
 
@@ -1803,38 +1813,29 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 
 static int process_values (double **Z, DATAINFO *pdinfo, int t, char *s)
 {
-    char valstr[32];
+    char *test;
     double x;
     int i, err = 0;
 
     gretl_error_clear();
 
     for (i=1; i<pdinfo->v && !err; i++) {
-	s = strpbrk(s, "01234567890+-NA");
-	if (s == NULL) {
-	    fprintf(stderr, "i = %d: s == NULL in process_values()\n", i);
+	while (isspace(*s)) s++;
+	x = strtod(s, &test);
+	if (errno) {
+	    err = 1;
+	} else if (!strncmp(test, "NA", 2)) {
+	    x = NADBL;
+	    s = test + 2;
+	} else if (*test != '\0' && !isspace(*test)) {
 	    err = 1;
 	} else {
-	    if (*s == '\0' || sscanf(s, "%31s", valstr) != 1) {
-		fputs("s is blank in process_values()\n", stderr);
-		err = 1;
-	    } else {
-		if (!strcmp(valstr, "NA")) {
-		    x = NADBL;
-		} else if (check_atof(valstr)) {
-		    err = 1;
-		} else {
-		    sscanf(valstr, "%lf", &x);
-		}
-	    }
+	    s = test;
 	}
-	if (!err) {
-	    if (t < pdinfo->n) {
-		Z[i][t] = x;
-	    }
-	    s = strpbrk(s, " \t\n\r");
+	if (t < pdinfo->n) {
+	    Z[i][t] = x;
 	}
-    }
+    }	
 
     if (err && *gretl_errmsg == '\0') {
 	sprintf(gretl_errmsg, _("Failed to parse data values at obs %d"), t+1);
@@ -1972,7 +1973,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		pdinfo->paninfo->period[t] = s;
 	    }
 
-	    tmp = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+	    tmp = xmlNodeListGetRawString(doc, cur->xmlChildrenNode, 1);
 
 	    if (tmp) {
 		if (process_values(*pZ, pdinfo, t, (char *) tmp)) {
@@ -2015,20 +2016,6 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     return err;
 }
 
-static long get_filesize (const char *fname)
-{
-    struct stat buf;
-
-    errno = 0;
-
-    if (stat(fname, &buf) == 0) {
-        return buf.st_size;
-    } else {
-	gretl_errmsg_set_from_errno(NULL);
-        return -1;
-    }
-}
-
 static double get_gdt_version (xmlNodePtr node)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "version");
@@ -2238,6 +2225,16 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
     pputc(prn, '\n');
 }
 
+static long get_filesize (const char *fname)
+{
+    struct stat buf;
+    int err;
+
+    err = gretl_stat(fname, &buf);
+
+    return (err)? -1 : buf.st_size;
+}
+
 /**
  * gretl_read_gdt:
  * @fname: name of file to try.
