@@ -44,25 +44,18 @@ typedef enum {
     OPT_DBOPEN  = 1 << 4,
     OPT_WEBDB   = 1 << 5,
     OPT_DUMP    = 1 << 6,
-    OPT_DEBUG   = 1 << 7
+    OPT_DEBUG   = 1 << 7,
+    OPT_ERROR   = 1 << 8
 } ProgramOptions;
 
-typedef enum {
-    ENGLISH = 1,
-    BASQUE
-} ForcedLangs;
-
 typedef enum {
     CONSOLE_EXEC      = 1 << 0,
     SCRIPT_EXEC       = 1 << 1,
-    SESSION_EXEC      = 1 << 2,
-    INCLUDE_EXEC      = 1 << 3,
-    FUNCTION_EXEC     = 1 << 4
+    INCLUDE_EXEC      = 1 << 2,
+    FUNCTION_EXEC     = 1 << 3
 } ExecFlags;
 
-#define HIDDEN_COMMAND(c) (c == ADDTO || \
-                           c == FUNCERR || \
-                           c == OMITFROM)
+#define HIDDEN_COMMAND(c) (c == FUNCERR)
     
 /* functions follow */
 
@@ -93,8 +86,7 @@ int command_number (const char *cmd);
 
 int cli_help (const char *cmdword, PATHS *paths, gretlopt opt, PRN *prn);
 
-int parseopt (const char **argv, int argc, char *fname, 
-	      int *force_lang);
+int parseopt (int *pargc, char ***pargv, char *fname, int *force_lang);
 
 void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line, 
 	       unsigned char flags, PRN *prn);
@@ -109,8 +101,6 @@ int call_pca_plugin (VMatrix *cmat, double ***pZ,
 		     DATAINFO *pdinfo, gretlopt opt,
 		     PRN *prn);
 
-int ready_for_command (const char *line);
-
 int gretl_shell_grab (const char *arg, char **sout);
 
 #endif /* INTERACT_H */
