@@ -270,13 +270,7 @@ static int re_estimate_VAR (irfboot *b, GRETL_VAR *v, int targ, int shock,
 {
     int err;
 
-    if (v->qr) {
-	err = gretl_matrix_QR_ols(v->Y, v->X, v->B, v->E,
-				  NULL, NULL);
-    } else {
-	err = gretl_matrix_multi_ols(v->Y, v->X, v->B, v->E,
-				     NULL);
-    }
+    err = gretl_matrix_multi_ols(v->Y, v->X, v->B, v->E, NULL);
 
     if (!err) {
 	VAR_write_A_matrix(v);
@@ -286,7 +280,7 @@ static int re_estimate_VAR (irfboot *b, GRETL_VAR *v, int targ, int shock,
 	gretl_matrix_multiply_mod(v->E, GRETL_MOD_TRANSPOSE,
 				  v->E, GRETL_MOD_NONE,
 				  v->S, GRETL_MOD_NONE);
-	gretl_matrix_divide_by_scalar(v->S, v->T);
+	gretl_matrix_divide_by_scalar(v->S, v->df); /* was v->T in denom. */
 	err = gretl_VAR_do_error_decomp(v->S, v->C);
     }
 
@@ -603,6 +597,7 @@ static void compute_VAR_dataset (irfboot *b, GRETL_VAR *v,
 {
     double x;
     int i, j, k, t;
+    int nl = var_n_lags(v);
 
     for (t=0; t<v->T; t++) {
 	/* extract "X" at t */
@@ -624,7 +619,7 @@ static void compute_VAR_dataset (irfboot *b, GRETL_VAR *v,
 	k = v->ifc;
 	for (i=0; i<v->neqns; i++) {
 	    x = b->Yt->val[i];
-	    for (j=1; j<=v->order && t+j < v->T; j++) {
+	    for (j=1; j<=nl && t+j < v->T; j++) {
 		gretl_matrix_set(v->X, t+j, k++, x);
 	    }
 	}
@@ -669,9 +664,8 @@ static void resample_resids (irfboot *b, const GRETL_VAR *vbak)
     }
 }
 
-static int irf_boot_quantiles (irfboot *b, gretl_matrix *R)
+static int irf_boot_quantiles (irfboot *b, gretl_matrix *R, double alpha)
 {
-    double alpha = 0.05;
     double *rk;
     int k, ilo, ihi;
 
@@ -823,6 +817,7 @@ static void restore_VAR_data (GRETL_VAR *v, GRETL_VAR *vbak)
 
 gretl_matrix *irf_bootstrap (GRETL_VAR *var, 
 			     int targ, int shock, int periods,
+			     double alpha,
 			     const double **Z, 
 			     const DATAINFO *pdinfo)
 {
@@ -898,7 +893,7 @@ gretl_matrix *irf_bootstrap (GRETL_VAR *var,
     }
 
     if (!err) {
-	err = irf_boot_quantiles(boot, R);
+	err = irf_boot_quantiles(boot, R, alpha);
     }
 
     irf_boot_free(boot);
