@@ -48,28 +48,20 @@
 #endif
 
 #ifdef ENABLE_NLS
-# define G_(String) gp_gettext (String)
+# include "libintl.h"
+# include "locale.h"
+# define gettext_noop(String) String
+# define _(String) gettext (String)
+# define N_(String) gettext_noop (String)
 # define I_(String) iso_gettext (String) 
 # define M_(String) maybe_iso_gettext (String)
 #else
-# define G_(String) String
-# define I_(String) String
-# define M_(String) String
+# define _(String)  ((char *) String)
+# define N_(String) String
+# define I_(String) ((char *) String)
+# define M_(String) ((char *) String)
 #endif /* ENABLE_NLS */
 
-#ifndef __GNOME_I18N_H__
-# ifdef ENABLE_NLS
-#  include "libintl.h"
-#  include "locale.h"
-#  define gettext_noop(String) String
-#  define _(String) gettext (String)
-#  define N_(String) gettext_noop (String)
-# else
-#  define _(String) String
-#  define N_(String) String
-# endif /* ENABLE_NLS */
-#endif /* __GNOME_I18N_H__ */
-
 #define MAXLINE 8192  /* maximum length of command line */
 #define MAXLABEL 128  /* maximum length of descriptive labels for variables */
 #define MAXLEN   512  /* max length of regular "long" strings */
@@ -89,7 +81,7 @@
 #define LN_2_PI       1.837877066409345
 #define LN_SQRT_2_PI  0.9189385332056725
 
-#define LISTSEP            999
+#define LISTSEP            (-100)
 #define PMAX_NOT_AVAILABLE 666
 
 /* numbers smaller than the given limit will print as zero */
@@ -135,14 +127,6 @@ enum ts_codes {
     STRUCTURE_UNKNOWN
 };
 
-enum auto_genr {
-    GENR_RESID,
-    GENR_FITTED,
-    GENR_RESID2,
-    GENR_H,
-    GENR_AHAT
-};
-
 enum progress_flags {
     SP_NONE, 
     SP_LOAD_INIT,
@@ -199,6 +183,7 @@ enum gretl_opt_flags {
     OPT_W = 1 << 22,
     OPT_X = 1 << 23,
     OPT_Z = 1 << 24,
+    OPT_Y = 1 << 25, /* added 2009-03-15 */
     OPT_UNSET = 1 << 30
 };
 
@@ -233,7 +218,7 @@ struct _cmplx {
 };
 #endif
 
-typedef unsigned long gretlopt;
+typedef unsigned int gretlopt;
 
 typedef struct VARINFO_ VARINFO;
 typedef struct PANINFO_ PANINFO;
@@ -349,6 +334,8 @@ struct ARINFO_ {
 struct MODEL_ {
     int ID;                      /* ID number for model */
     int refcount;                /* for saving/deleting */
+    int ci;                      /* "command index" -- estimation method */
+    gretlopt opt;                /* record of options */
     int t1, t2, nobs;            /* starting observation, ending
                                     observation, and number of obs */
     char *submask;               /* keep track of sub-sample in force
@@ -362,8 +349,6 @@ struct MODEL_ {
     int *list;                   /* list of variables by ID number */
     int ifc;                     /* = 1 if the equation includes a constant,
 				    else = 0 */
-    int ci;                      /* "command index" -- depends on 
-				    estimation method */
     int nwt;                     /* ID number of the weight variable (WLS) */
     int aux;                     /* code representing the sort of
 				    auxiliary regression this is (or not) */
@@ -376,7 +361,8 @@ struct MODEL_ {
     double ess, tss;             /* Error and Total Sums of Squares */
     double sigma;                /* Standard error of regression */
     double rsq, adjrsq;          /* Unadjusted and adjusted R^2 */     
-    double fstt;                 /* F-statistic */
+    double fstt;                 /* overall F-statistic */
+    double chisq;                /* overall chi-square statistic */
     double lnL;                  /* log-likelihood */
     double ybar, sdy;            /* mean and std. dev. of dependent var. */
     double criterion[C_MAX];     /* array of model selection statistics */
@@ -437,7 +423,6 @@ struct MODEL_ {
 #include "nls.h"
 #include "missing.h"
 #include "transforms.h"
-#include "tsls.h"
 #ifdef WIN32
 # include "gretl_win32.h"
 #endif
