@@ -39,20 +39,24 @@ enum {
 };
 
 enum {
-    STATE_USE_CWD        = 1 << 0,  /* store: use current dir as default */
-    STATE_ECHO_ON        = 1 << 1,  /* echoing commands or not */
-    STATE_MSGS_ON        = 1 << 2,  /* emitting non-error messages or not */
-    STATE_FORCE_DECPOINT = 1 << 3,  /* override locale decimal separator */
-    STATE_USE_PCSE       = 1 << 4,  /* Beck-Katz panel-corrected std errs */
-    STATE_USE_QR         = 1 << 5,  /* QR decomposition is OLS default */
-    STATE_PREWHITEN      = 1 << 6,  /* HAC pre-whitening? */
-    STATE_FORCE_HC       = 1 << 7,  /* don't use HAC for time series */
-    STATE_HALT_ON_ERR    = 1 << 8,  /* errors fatal in batch mode */
-    STATE_USE_LBFGS      = 1 << 9,  /* prefer LBFGS to BFGS? */
-    STATE_SHELL_OK       = 1 << 10, /* "shell" facility is approved? */
-    STATE_MAX_VERBOSE    = 1 << 11, /* verbose output from maximizer? */
-    STATE_USE_FCP        = 1 << 12, /* use FCP garch code */
-    STATE_WARN_ON        = 1 << 13  /* print numerical warning messages */
+    STATE_USE_CWD         = 1 << 0,  /* store: use current dir as default */
+    STATE_ECHO_ON         = 1 << 1,  /* echoing commands or not */
+    STATE_MSGS_ON         = 1 << 2,  /* emitting non-error messages or not */
+    STATE_FORCE_DECPOINT  = 1 << 3,  /* override locale decimal separator */
+    STATE_USE_PCSE        = 1 << 4,  /* Beck-Katz panel-corrected std errs */
+    STATE_USE_SVD         = 1 << 5,  /* SVD decomposition is matrix OLS default */
+    STATE_PREWHITEN       = 1 << 6,  /* HAC pre-whitening? */
+    STATE_FORCE_HC        = 1 << 7,  /* don't use HAC for time series */
+    STATE_HALT_ON_ERR     = 1 << 8,  /* errors fatal in batch mode */
+    STATE_USE_LBFGS       = 1 << 9,  /* prefer LBFGS to BFGS? */
+    STATE_SHELL_OK        = 1 << 10, /* "shell" facility is approved? */
+    STATE_MAX_VERBOSE     = 1 << 11, /* verbose output from maximizer? */
+    STATE_USE_FCP         = 1 << 12, /* use FCP garch code */
+    STATE_WARN_ON         = 1 << 13, /* print numerical warning messages */
+    STATE_VERBOSE_INCLUDE = 1 << 14, /* verbose include */
+    STATE_SKIP_MISSING    = 1 << 15, /* skip NAs when building matrix from series */
+    STATE_LOOPING         = 1 << 16, /* loop is in progress at this level */
+    STATE_LOOP_QUIET      = 1 << 17  /* loop commands should be quiet */
 };    
 
 /* for values that really want a non-negative integer */
@@ -113,11 +117,13 @@ struct set_vars_ {
 			   !strcmp(s, USE_LBFGS) || \
 			   !strcmp(s, PCSE) || \
 			   !strcmp(s, PREWHITEN) || \
-			   !strcmp(s, USE_QR) || \
+			   !strcmp(s, USE_SVD) || \
 			   !strcmp(s, SHELL_OK) || \
 			   !strcmp(s, USE_CWD) || \
 			   !strcmp(s, USE_FCP) || \
-                           !strcmp(s, PROTECT_LISTS))
+                           !strcmp(s, PROTECT_LISTS) || \
+                           !strcmp(s, VERBOSE_INCLUDE) || \
+                           !strcmp(s, SKIP_MISSING))
 
 #define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
 			  !strcmp(s, BHHH_TOLER) || \
@@ -143,6 +149,7 @@ struct set_vars_ {
 set_vars *state;
 static int gretl_debug;
 static int protect_lists = 1;
+static int user_mp_bits;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -310,6 +317,11 @@ static void state_vars_copy (set_vars *sv)
     fprintf(stderr, "state_vars_copy() called\n");
 #endif
     sv->flags = state->flags;
+    /* We're not (yet) looping at the current level of execution (but
+       note that the STATE_LOOP_QUIET flag should be inherited).
+    */
+    sv->flags &= ~STATE_LOOPING;
+
     sv->seed = state->seed;
     sv->hp_lambda = state->hp_lambda;
     sv->horizon = state->horizon;
@@ -341,7 +353,8 @@ static void state_vars_init (set_vars *sv)
 #if PDEBUG
     fprintf(stderr, "state_vars_init called\n");
 #endif
-    sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | STATE_HALT_ON_ERR;
+    sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | 
+	STATE_HALT_ON_ERR | STATE_SKIP_MISSING;
     sv->seed = 0;
     sv->hp_lambda = NADBL;
     sv->horizon = UNSET_INT;
@@ -511,6 +524,34 @@ int lists_protected (void)
     return protect_lists;
 }
 
+#define DEFAULT_MP_BITS 256
+#define mp_bits_ok(b) (b >= 256 && b <= 8192)
+
+void set_mp_bits (int b)
+{
+    if (mp_bits_ok(b)) {
+	user_mp_bits = b;
+    }
+}
+
+int get_mp_bits (void)
+{
+    if (user_mp_bits >= 256) {
+	return user_mp_bits;
+    } else {
+	char *s = getenv("GRETL_MP_BITS");
+	int b;
+
+	if (s != NULL) {
+	    b = atoi(s);
+	    if (mp_bits_ok(b)) {
+		return b;
+	    }
+	}
+	return DEFAULT_MP_BITS;
+    }
+}
+
 char get_csv_delim (const DATAINFO *pdinfo)
 {
     check_for_state();
@@ -681,7 +722,6 @@ static int negval_invalid (const char *var)
 }
 
 static int libset_get_scalar (const char *var, const char *arg, 
-			      double **Z, DATAINFO *pdinfo,
 			      int *pi, double *px)
 {
     double x = NADBL;
@@ -866,10 +906,10 @@ static int set_line_width (const char *s0, const char *s1,
     if (isdigit((unsigned char) *s0)) {
 	v = atoi(s0);
     } else {
-	v = series_index(pdinfo, s0);
+	v = current_series_index(pdinfo, s0);
     }
 
-    if (v < 1 || v >= pdinfo->v) {
+    if (v < 0) {
 	return E_DATA;
     }
 
@@ -887,15 +927,14 @@ static int set_line_width (const char *s0, const char *s1,
 }
 
 static int set_bkbp_limits (const char *s0, const char *s1,
-			    double **Z, DATAINFO *pdinfo,
 			    PRN *prn)
 {
     int p0, p1;
     int err = 0;
 
-    err = libset_get_scalar(NULL, s0, Z, pdinfo, &p0, NULL);
+    err = libset_get_scalar(NULL, s0, &p0, NULL);
     if (!err) {
-	err = libset_get_scalar(NULL, s1, Z, pdinfo, &p1, NULL);
+	err = libset_get_scalar(NULL, s1, &p1, NULL);
     }
 
     if (err) {
@@ -918,7 +957,7 @@ static int set_bkbp_limits (const char *s0, const char *s1,
     return 0;
 }
 
-static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
+static int set_initvals (const char *s, PRN *prn)
 {
     gretl_matrix *m;
     char mname[VNAMELEN];
@@ -933,7 +972,8 @@ static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
     } else {
 	m = get_matrix_by_name(mname);
 	if (m == NULL) {
-	    pprintf(prn, _("'%s': no such matrix\n"), mname);
+	    pprintf(prn, _("'%s': no such matrix"), mname);
+	    pputc(prn, '\n');
 	    err = E_DATA;
 	} else {
 	    state->initvals = gretl_matrix_copy(m);
@@ -971,7 +1011,7 @@ void shelldir_init (const char *s)
 
 static int set_shelldir (const char *s)
 {
-    int err = 0;
+    int len = 0, err = 0;
 
     /* skip past "set shelldir" and space */
     s += 12;
@@ -979,22 +1019,52 @@ static int set_shelldir (const char *s)
 
     if (*s == '\0') {
 	*state->shelldir = '\0';
+	gretl_insert_builtin_string("shelldir", state->shelldir);
     } else if (*s == '"') {
-	int len = haschar('"', s + 1);
-
-	if (len <= 0 || len >= MAXLEN) {
+	s++;
+	len = haschar('"', s);
+	if (len <= 0) {
 	    err = E_PARSE;
-	} else {
-	    *state->shelldir = '\0';
-	    strncat(state->shelldir, s + 1, len);
-	}
+	} 
     } else {
-	*state->shelldir = '\0';
-	strncat(state->shelldir, s, MAXLEN - 1);
+	len = strlen(s);
     }
 
-    if (!err) {
-	gretl_insert_builtin_string("shelldir", state->shelldir);
+    if (!err && len > 0) {
+	char test[MAXLEN];
+	char *home = NULL;
+	int slen = len;
+
+	if (*s == '~') {
+	    home = getenv("HOME");
+	    if (home != NULL) {
+		s++;
+		slen--;
+		len = slen + strlen(home);
+	    }
+	} 
+
+	*test = '\0';
+    
+	if (len >= MAXLEN) {
+	    gretl_errmsg_set("shelldir: string is too long");
+	    err = E_DATA;
+	} else if (home != NULL) {
+	    strcat(test, home);
+	    strncat(test, s, slen);
+	} else {
+	    strncat(test, s, len);
+	}
+
+	if (!gretl_isdir(test)) {
+	    gretl_errmsg_sprintf("shelldir: '%s' no such directory", test);
+	    err = E_DATA;
+	}
+
+	if (!err) {
+	    strcpy(state->shelldir, test);
+	    gretl_insert_builtin_string("shelldir", state->shelldir);
+	}
     }
 
     return err;
@@ -1177,6 +1247,8 @@ static int display_settings (PRN *prn)
 
     libset_print_bool(USE_CWD, prn);
     libset_print_bool(PROTECT_LISTS, prn);
+    libset_print_bool(VERBOSE_INCLUDE, prn);
+    libset_print_bool(SKIP_MISSING, prn);
 
     libset_header(_("Numerical methods"), prn);
 
@@ -1188,7 +1260,7 @@ static int display_settings (PRN *prn)
     print_initvals(state->initvals, prn);
     libset_print_bool(USE_LBFGS, prn);
     libset_print_double(NLS_TOLER, prn);
-    libset_print_bool(USE_QR, prn);
+    libset_print_bool(USE_SVD, prn);
     libset_print_bool(USE_FCP, prn);
 
     libset_header(_("Random number generation"), prn);
@@ -1297,8 +1369,7 @@ libset_query_settings (const char *s, PRN *prn)
 #define boolean_off(s) (!strcmp(s, "off") || !strcmp(s, "0") || \
                         !strcmp(s, "false"))
 
-int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo, 
-		      PRN *prn)
+int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 {
     char setobj[32], setarg[32], setarg2[32];
     int k, nw, err = E_PARSE;
@@ -1319,7 +1390,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 	if (!strcmp(setobj, "plotfile")) {
 	    return parse_set_plotfile(line);
 	} else if (!strcmp(setobj, "initvals")) {
-	    return set_initvals(line, pdinfo, prn);
+	    return set_initvals(line, prn);
 	} else if (!strcmp(setobj, "shelldir")) {
 	    return set_shelldir(line);
 	} else if (!strcmp(setobj, "codevars")) {
@@ -1358,7 +1429,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		libset_set_double(setobj, NADBL);
 		err = 0;
 	    } else {
-		err = libset_get_scalar(NULL, setarg, Z, pdinfo, NULL, &x);
+		err = libset_get_scalar(NULL, setarg, NULL, &x);
 		if (!err) {
 		    err = libset_set_double(setobj, x);
 		}
@@ -1371,7 +1442,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "seed")) {
-	    err = libset_get_scalar(NULL, setarg, Z, pdinfo, &k, NULL);
+	    err = libset_get_scalar(NULL, setarg, &k, NULL);
 	    if (!err) {
 		gretl_rand_set_seed((unsigned int) k);
 		if (gretl_messages_on() && !gretl_looping_quietly()) {
@@ -1386,7 +1457,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		state->horizon = UNSET_INT;
 		err = 0;
 	    } else {
-		err = libset_get_scalar(NULL, setarg, Z, pdinfo, &k, NULL);
+		err = libset_get_scalar(NULL, setarg, &k, NULL);
 		if (!err) {
 		    state->horizon = k;
 		} else {
@@ -1396,7 +1467,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 	} else if (coded_intvar(setobj)) {
 	    err = parse_libset_int_code(setobj, setarg);
 	} else if (libset_int(setobj)) {
-	    err = libset_get_scalar(setobj, setarg, Z, pdinfo, &k, NULL);
+	    err = libset_get_scalar(setobj, setarg, &k, NULL);
 	    if (!err) {
 		err = libset_set_int(setobj, k);
 	    }
@@ -1405,7 +1476,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 	}
     } else if (nw == 3) {
 	if (!strcmp(setobj, "bkbp_limits")) {
-	    err = set_bkbp_limits(setarg, setarg2, Z, pdinfo, prn);
+	    err = set_bkbp_limits(setarg, setarg2, prn);
 	} else if (!strcmp(setobj, "linewidth")) {
 	    err = set_line_width(setarg, setarg2, pdinfo, prn);
 	} else {
@@ -1558,7 +1629,7 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*min = 1;
 	*var = &state->horizon;
     } else if (!strcmp(s, LONGDIGITS)) {
-	*min = 5;
+	*min = 1;
 	*max = 21;
 	*var = &state->longdigits;
     } else if (!strcmp(s, LOOP_MAXITER)) {
@@ -1635,8 +1706,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_MSGS_ON;
     } else if (!strcmp(s, WARNINGS)) {
 	return STATE_WARN_ON;
-    } else if (!strcmp(s, USE_QR)) {
-	return STATE_USE_QR;
+    } else if (!strcmp(s, USE_SVD)) {
+	return STATE_USE_SVD;
     } else if (!strcmp(s, USE_LBFGS)) {
 	return STATE_USE_LBFGS;
     } else if (!strcmp(s, FORCE_DECP)) {
@@ -1657,6 +1728,10 @@ static int boolvar_get_flag (const char *s)
 	return STATE_PREWHITEN;
     } else if (!strcmp(s, PCSE)) {
 	return STATE_USE_PCSE;
+    } else if (!strcmp(s, VERBOSE_INCLUDE)) {
+	return STATE_VERBOSE_INCLUDE;
+    } else if (!strcmp(s, SKIP_MISSING)) {
+	return STATE_SKIP_MISSING;
     } else {
 	fprintf(stderr, "libset_get_bool: unrecognized "
 		"variable '%s'\n", s);	
@@ -1686,8 +1761,8 @@ static void set_flag_from_env (int flag, const char *s, int neg)
 
 static void maybe_check_env (const char *s)
 {
-    if (!strcmp(s, USE_QR)) {
-	set_flag_from_env(STATE_USE_QR, "GRETL_USE_QR", 0);
+    if (!strcmp(s, USE_SVD)) {
+	set_flag_from_env(STATE_USE_SVD, "GRETL_USE_SVD", 0);
     } else if (!strcmp(s, USE_LBFGS)) {
 	set_flag_from_env(STATE_USE_LBFGS, "GRETL_USE_LBFGS", 0);
     } else if (!strcmp(s, HALT_ON_ERR)) {
@@ -1738,6 +1813,7 @@ int libset_get_bool (const char *s)
 
 static void libset_set_decpoint (int on)
 {
+#ifdef ENABLE_NLS
     static char num_locale[16];
 
     if (on) {
@@ -1751,6 +1827,7 @@ static void libset_set_decpoint (int on)
     }
 
     reset_local_decpoint();
+#endif
 }
 
 void libset_set_bool (const char *s, int set)
@@ -1921,56 +1998,71 @@ void libset_cleanup (void)
     n_states = 0;
 }
 
-/* switches for looping, batch mode, and pausing between screens of
-   output: these depend on the state of the program calling libgretl,
-   but are not user-settable
+/* switches for looping and batch mode: output: these depend on the
+   state of the program calling libgretl, but are not user-settable
 */
 
-enum {
-    LOOPING = 1,
-    LOOPING_QUIETLY
-};
-
-static int gretl_text_pause;
-static int loop_on;
 static int batch_mode;
 static int gui_mode;
 
-int gretl_get_text_pause (void)
-{
-    return gretl_text_pause;
-}
-
 void set_loop_on (int quiet)
 {
-    loop_on = (quiet)? LOOPING_QUIETLY : LOOPING;
-    gretl_text_pause = 0;
+    state->flags |= STATE_LOOPING;
+    if (quiet) {
+	state->flags |= STATE_LOOP_QUIET;
+    }
 }
 
 void set_loop_off (void)
 {
-    loop_on = 0;
-    if (!batch_mode && !gui_mode) {
-	gretl_text_pause = 1;
+    state->flags &= ~STATE_LOOPING;
+    
+    /* If we're not currently governed by "loop quietness" at
+       caller level, turn such quietness off too 
+    */
+    if (state->flags & STATE_LOOP_QUIET) {
+	int i = n_states - 1;
+
+	if (i <= 0 || !(state_stack[i-1]->flags & STATE_LOOP_QUIET)) {
+	    state->flags ^= STATE_LOOP_QUIET;
+	}
     }
 }
 
+/* returns 1 if there's a loop going on anywhere in the "caller
+   ancestry" of the current execution level, else 0.
+*/
+
 int gretl_looping (void)
 {
-    return loop_on;
+    int i, ns = n_states;
+
+    for (i=0; i<ns; i++) {
+	if (state_stack[i]->flags & STATE_LOOPING) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+/* returns 1 if there's a loop going on at the current execution
+   stack level, else 0.
+*/
+
+int gretl_looping_currently (void)
+{
+    return (state->flags & STATE_LOOPING)? 1 : 0;
 }
 
 int gretl_looping_quietly (void)
 {
-    return (loop_on == LOOPING_QUIETLY);
+    return (state->flags & STATE_LOOP_QUIET)? 1 : 0;
 }
 
 void gretl_set_batch_mode (int b)
 {
     batch_mode = b;
-    if (batch_mode) {
-	gretl_text_pause = 0;
-    }	
 }
 
 int gretl_in_batch_mode (void)
