@@ -20,23 +20,6 @@
 #ifndef LIBSET_H
 #define LIBSET_H
 
-typedef enum {
-    VCV_UNSET,
-    VCV_HESSIAN,
-    VCV_IM,
-    VCV_OP,
-    VCV_QML,
-    VCV_BW,
-    VCV_MAX
-} VcvType;
-
-typedef enum {
-    KERNEL_BARTLETT,
-    KERNEL_PARZEN,
-    KERNEL_QS,
-    KERNEL_MAX
-} HACKernel;
-
 typedef enum {
     NORM_PHILLIPS,
     NORM_DIAG,
@@ -47,37 +30,39 @@ typedef enum {
 
 /* guard against consequences of typos */
 
-#define BFGS_MAXITER "bfgs_maxiter"
-#define BFGS_TOLER   "bfgs_toler"
-#define BHHH_MAXITER "bhhh_maxiter"
-#define BHHH_TOLER   "bhhh_toler"
-#define BKBP_K       "bkbp_k"
-#define BOOTREP      "bootrep"
-#define FORCE_DECP   "force_decpoint"
-#define FORCE_HC     "force_hc"
-#define GARCH_VCV    "garch_vcv"
+#define BFGS_MAXITER     "bfgs_maxiter"
+#define BFGS_TOLER       "bfgs_toler"
+#define BHHH_MAXITER     "bhhh_maxiter"
+#define BHHH_TOLER       "bhhh_toler"
+#define BKBP_K           "bkbp_k"
+#define BOOTREP          "bootrep"
+#define FORCE_DECP       "force_decpoint"
+#define FORCE_HC         "force_hc"
+#define GARCH_VCV        "garch_vcv"
 #define GARCH_ROBUST_VCV "garch_robust_vcv"
-#define HAC_KERNEL   "hac_kernel"
-#define HAC_LAG      "hac_lag"
-#define HALT_ON_ERR  "halt_on_error"
-#define HC_VERSION   "hc_version"
-#define HORIZON      "horizon"
-#define HP_LAMBDA    "hp_lambda"
-#define USE_LBFGS    "lbfgs"
-#define LONGDIGITS   "longdigits"
-#define LOOP_MAXITER "loop_maxiter"
-#define RQ_MAXITER   "rq_maxiter"
-#define MAX_VERBOSE  "max_verbose"
-#define NLS_TOLER    "nls_toler"
-#define PCSE         "pcse"
-#define PREWHITEN    "hac_prewhiten"
-#define QS_BANDWIDTH "qs_bandwidth"
-#define SHELL_OK     "shell_ok"
-#define USE_CWD      "use_cwd"
-#define USE_QR       "qr"
-#define USE_FCP      "fcp"
-#define VECM_NORM    "vecm_norm"
-#define ARMA_VCV     "arma_vcv"
+#define HAC_KERNEL       "hac_kernel"
+#define HAC_LAG          "hac_lag"
+#define HALT_ON_ERR      "halt_on_error"
+#define HC_VERSION       "hc_version"
+#define HORIZON          "horizon"
+#define HP_LAMBDA        "hp_lambda"
+#define USE_LBFGS        "lbfgs"
+#define LONGDIGITS       "longdigits"
+#define LOOP_MAXITER     "loop_maxiter"
+#define RQ_MAXITER       "rq_maxiter"
+#define MAX_VERBOSE      "max_verbose"
+#define NLS_TOLER        "nls_toler"
+#define PCSE             "pcse"
+#define PREWHITEN        "hac_prewhiten"
+#define QS_BANDWIDTH     "qs_bandwidth"
+#define SHELL_OK         "shell_ok"
+#define USE_CWD          "use_cwd"
+#define USE_SVD          "svd"
+#define USE_FCP          "fcp"
+#define VECM_NORM        "vecm_norm"
+#define ARMA_VCV         "arma_vcv"
+#define VERBOSE_INCLUDE  "verbose_include"
+#define SKIP_MISSING     "skip_missing"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
 
@@ -115,7 +100,8 @@ int set_bkbp_periods (int bkl, int bku);
 void unset_bkbp_k (void);
 void unset_bkbp_periods (void);
 
-int gretl_get_text_pause (void);
+void set_mp_bits (int b);
+int get_mp_bits (void);
 
 const gretl_matrix *get_init_vals (void);
 int n_init_vals (void);
@@ -124,6 +110,7 @@ void free_init_vals (void);
 void set_loop_on (int quiet);
 void set_loop_off (void);
 int gretl_looping (void);
+int gretl_looping_currently (void);
 int gretl_looping_quietly (void);
 
 void gretl_set_batch_mode (int b);
@@ -147,8 +134,7 @@ char *get_shelldir (void);
 
 char get_csv_delim (const DATAINFO *pdinfo);
 
-int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo, 
-		      PRN *prn);
+int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn);
 
 void set_iter_print_func (ITER_PRINT_FUNC func);
 int iter_print_callback (int i, PRN *prn);
