@@ -19,9 +19,13 @@
 
 #include "libgretl.h"
 #include "usermat.h"
+#include "matrix_extra.h"
 
 /* functions that convert between gretl_matrix and other
-   datatypes; also printing of gretl_matrices */
+   datatypes; also printing of gretl_matrices and some
+   functions relating to masking and cutting rows and/or
+   columns of matrices.
+*/
 
 /**
  * gretl_vector_from_array:
@@ -127,19 +131,18 @@ gretl_matrix *gretl_matrix_from_2d_array (const double **X,
  * @x: scalar to be "promoted".
  *
  * Returns: allocated 1x1 gretl_matrix, the single element
- * of which is set to @x, or %NULL on allocation failure
- * or if @x = #NADBL.
+ * of which is set to @x, or %NULL on allocation failure.
+ * If @x = #NADBL the matrix value is #M_NA.
  */
 
 gretl_matrix *gretl_matrix_from_scalar (double x) 
 {
-    gretl_matrix *m = NULL;
+    gretl_matrix *m = gretl_matrix_alloc(1, 1);
 
-    if (!na(x)) {
-	m = gretl_matrix_alloc(1, 1);
-	if (m != NULL) {
-	    m->val[0] = x;
-	}
+    if (na(x)) {
+	m->val[0] = M_NA;
+    } else {
+	m->val[0] = x;
     }
 
     return m;
@@ -402,12 +405,6 @@ static int get_mask_count (const char *mask, int n)
     return k;
 }
 
-enum {
-    OBS_MISS_MASK,
-    OBS_MISS_ERR,
-    OBS_MISS_SKIP
-};
-
 static gretl_matrix *
 real_gretl_matrix_data_subset (const int *list, const double **Z,
 			       int t1, int t2, const char *mask,
@@ -415,26 +412,35 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 {
     gretl_matrix *M;
     double x;
-    int T = t2 - t1 + 1;
+    int T, Tmax = t2 - t1 + 1;
     int k = list[0];
     int skip;
     int j, s, t;
 
-    if (k <= 0 || T <= 0) {
+    if (k <= 0 || Tmax <= 0) {
 	*err = E_DATA;
 	return NULL;
     }	
 
     *err = 0;
 
+    T = Tmax;
+
     if (mask != NULL) {
-	T -= get_mask_count(mask, T);
-    } else if (op == OBS_MISS_SKIP || op == OBS_MISS_ERR) {
+	T -= get_mask_count(mask, Tmax);
+    } else if (op == M_MISSING_TRIM) {
+	*err = check_for_missing_obs(list, &t1, &t2, Z, NULL);
+	if (*err) {
+	    return NULL;
+	} else {
+	    Tmax = T = t2 - t1 + 1;
+	}
+    } else if (op == M_MISSING_SKIP || op == M_MISSING_ERROR) {
 	for (t=t1; t<=t2 && !*err; t++) {
 	    for (j=0; j<k; j++) {
 		x = Z[list[j+1]][t];
 		if (na(x)) {
-		    if (op == OBS_MISS_SKIP) {
+		    if (op == M_MISSING_SKIP) {
 			T--;
 		    } else {
 			*err = E_MISSDATA;
@@ -462,7 +468,7 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 	skip = 0;
 	if (mask != NULL) {
 	    skip = mask[t - t1];
-	} else if (op == OBS_MISS_SKIP) {
+	} else if (op == M_MISSING_SKIP) {
 	    for (j=0; j<k; j++) {
 		x = Z[list[j+1]][t];
 		if (na(x)) {
@@ -486,12 +492,17 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 	}
     }
 
-    if (!*err) {
+    if (!*err && (mask != NULL || op == M_MISSING_OK)) {
 	for (j=0; j<k && !*err; j++) {
 	    for (t=0; t<T && !*err; t++) {
 		x = gretl_matrix_get(M, t, j);
 		if (na(x)) {
-		    *err = E_MISSDATA;
+		    if (op == M_MISSING_OK) {
+			/* convert NA to NAN */
+			gretl_matrix_set(M, t, j, M_NA);
+		    } else {
+			*err = E_MISSDATA;
+		    }
 		}
 	    }
 	}
@@ -500,7 +511,10 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
     if (*err) {
 	gretl_matrix_free(M);
 	M = NULL;
-    } 
+    } else if (T == Tmax) {
+	M->t1 = t1;
+	M->t2 = t2;
+    }
 
     return M;
 }
@@ -511,74 +525,125 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
  * @Z: data array.
  * @t1: starting observation.
  * @t2: ending observation.
- * @mask: missing observations mask, or %NULL.
- * @err: location to recieve error code.
+ * @missop: how to handle missing observations.
+ * @err: location to receive error code.
  *
  * Creates a gretl matrix holding the subset of variables from
  * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If @mask is not
- * %NULL then it should be an array of char of length (@t2 - @t1
- * + 1) with 1s in the positions of observations to exclude
- * from the subset and zeros elsewhere. This apparatus can be
- * used to exclude missing observations.
+ * inclusive.  Variables are in columns. The @missop flag
+ * can be %M_MISSING_OK to indicate that it's OK to include
+ * missing values in the matrix (these become NaNs),
+ * %M_MISSING_ERROR (it's an error of any missing values are
+ * found), or %M_MISSING_SKIP (observations with any missing
+ * values are omitted from the matrix).
  *
  * Returns: allocated matrix or %NULL on failure. 
  */
 
 gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
-					int t1, int t2, const char *mask,
+					int t1, int t2, int missop, 
 					int *err)
+{
+    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
+					 missop, err);
+}
+
+/**
+ * gretl_matrix_data_subset_masked:
+ * @list: list of variable to process.
+ * @Z: data array.
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @mask: missing observations mask, or %NULL.
+ * @err: location to receive error code.
+ *
+ * Creates a gretl matrix holding the subset of variables from
+ * @Z specified by @list, over the sample range @t1 to @t2,
+ * inclusive.  Variables are in columns.  @mask should be an
+ * array of char of length (@t2 - @t1 + 1) with 1s in the positions 
+ * of observations to exclude from the subset and zeros elsewhere. 
+ * This apparatus can be used to exclude missing observations.
+ *
+ * Returns: allocated matrix or %NULL on failure. 
+ */
+
+gretl_matrix *
+gretl_matrix_data_subset_masked (const int *list, const double **Z,
+				 int t1, int t2, const char *mask,
+				 int *err)
 {
     return real_gretl_matrix_data_subset(list, Z, t1, t2, mask, 
-					 OBS_MISS_MASK, err);
+					 M_MISSING_ERROR, err);
 }
 
 /**
- * gretl_matrix_data_subset_no_missing:
- * @list: list of variable to process.
- * @Z: data array.
- * @t1: starting observation.
- * @t2: ending observation.
+ * gretl_dataset_from_matrix:
+ * @m: source matrix.
+ * @list: list of columns (1-based) to include, or %NULL.
+ * @pZ: location to receive data array.
  * @err: location to receive error code.
  *
- * Creates a gretl matrix holding the subset of variables from
- * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If any missing 
- * values are encountered this constitutes an error.
+ * Creates a gretl dataset from matrix @m, either using the
+ * columns specified in @list or using all columns if @list
+ * is %NULL.
  *
- * Returns: allocated matrix or %NULL on failure. 
+ * Returns: pointer to new dataset information struct on success, 
+ * or %NULL on failure.
  */
 
-gretl_matrix *
-gretl_matrix_data_subset_no_missing (const int *list, const double **Z,
-				     int t1, int t2, int *err)
+DATAINFO *gretl_dataset_from_matrix (gretl_matrix *m, const int *list,
+				     double ***pZ, int *err)
 {
-    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
-					 OBS_MISS_ERR, err);
-}
+    DATAINFO *dinfo = NULL;
+    const char **names;
+    int i, t, col, nv, T;
 
-/**
- * gretl_matrix_data_subset_skip_missing:
- * @list: list of variable to process.
- * @Z: data array.
- * @t1: starting observation.
- * @t2: ending observation.
- * @err: location to receive error code.
- *
- * Creates a gretl matrix holding the subset of variables from
- * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If there is a missing
- * value for any variable on a given row, that row is skipped.
- *
- * Returns: allocated matrix or %NULL on failure. 
- */
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+	return NULL;
+    }
 
-gretl_matrix *
-gretl_matrix_data_subset_skip_missing (const int *list, const double **Z,
-				       int t1, int t2, int *err)
-{
-    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
-					 OBS_MISS_SKIP, err);
+    T = gretl_matrix_rows(m);
+    nv = gretl_matrix_cols(m);
+
+    if (list != NULL) {
+	for (i=1; i<=list[0]; i++) {
+	    col = list[i];
+	    if (col < 1 || col > nv) {
+		fprintf(stderr, "column %d: out of bounds\n", col);
+		*err = E_DATA;
+		break;
+	    } 
+	}
+	nv = list[0];
+    } 
+
+    if (!*err) {
+	dinfo = create_auxiliary_dataset(pZ, nv + 1, T);
+	if (dinfo == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (*err) {
+	return NULL;
+    }
+
+    names = user_matrix_get_column_names(m);
+
+    for (i=1; i<=nv; i++) {
+	col = (list != NULL)? list[i] - 1 : i - 1;
+	for (t=0; t<T; t++) {
+	    (*pZ)[i][t] = gretl_matrix_get(m, t, col);
+	}
+	if (names != NULL) {
+	    strcpy(dinfo->varname[i], names[col]);
+	} else {
+	    sprintf(dinfo->varname[i], "col%d", col + 1);
+	}
+    }
+
+    return dinfo;
 }
 
 /**
@@ -1160,3 +1225,276 @@ void debug_print_matrix (const gretl_matrix *m, const char *msg)
     }
 }
 
+static int count_unmasked_rows (const char *mask, int n)
+{
+    int i, c = 0;
+
+    for (i=0; i<n; i++) {
+	c += (mask[i] == 0);
+    }
+
+    return c;
+}
+
+/**
+ * gretl_matrix_cut_rows:
+ * @m: matrix to process.
+ * @mask: character array of length equal to the rows of @m,
+ * with 1s indicating rows to be cut, 0s for rows to be
+ * retained.
+ *
+ * In-place reduction of @m based on @mask: the masked rows
+ * are cut out of @m.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_matrix_cut_rows (gretl_matrix *m, const char *mask)
+{
+    int i, j, k, n;
+    double x;
+
+    if (m == NULL || mask == NULL) {
+	return E_DATA;
+    }
+
+    n = count_unmasked_rows(mask, m->rows);
+
+    for (j=0; j<m->cols; j++) {
+	k = 0;
+	for (i=0; i<m->rows; i++) {
+	    if (!mask[i]) {
+		x = gretl_matrix_get(m, i, j);
+		m->val[j * n + k] = x;
+		k++;
+	    }
+	}
+    }
+
+    m->rows = n;
+
+    return 0;
+}
+
+/**
+ * gretl_matrix_cut_rows_cols:
+ * @m: square matrix to process.
+ * @mask: character array of length equal to the dimension
+ * of @m, with 1s indicating rows and columns to be cut, 0s
+ * for rows/columns to be retained.
+ *
+ * In-place reduction of @m based on @mask: the masked rows
+ * and columns are cut out of @m.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_matrix_cut_rows_cols (gretl_matrix *m, const char *mask)
+{
+    gretl_matrix *tmp;
+    double x;
+    int i, j, k, l, n;
+
+    if (m == NULL || mask == NULL || m->rows != m->cols) {
+	return E_DATA;
+    }
+
+    n = count_unmasked_rows(mask, m->rows);
+
+    /* allocate smaller temp matrix */
+    tmp = gretl_matrix_alloc(n, n);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    /* copy unmasked values into temp matrix */
+    k = 0;
+    for (i=0; i<m->rows; i++) {
+	if (!mask[i]) {
+	    l = 0;
+	    for (j=0; j<m->cols; j++) {
+		if (!mask[j]) {
+		    x = gretl_matrix_get(m, i, j);
+		    gretl_matrix_set(tmp, k, l++, x);
+		}
+	    }
+	    k++;
+	}
+    }
+
+    /* resize the original matrix, copy the values back,
+       and free the temp matrix */
+    gretl_matrix_reuse(m, n, n);
+    gretl_matrix_copy_values(m, tmp);
+    gretl_matrix_free(tmp);
+
+    return 0;
+}
+
+/**
+ * gretl_matrix_zero_row_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Checks matrix @m for rows that are all zero.  If there are
+ * any such rows, constructs a mask of length equal to the
+ * number of rows in @m, with 1s indicating zero rows, 0s
+ * elsewhere.  If there are no such rows, returns NULL.
+ * 
+ * %E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or %NULL.
+ */
+
+char *gretl_matrix_zero_row_mask (const gretl_matrix *m, int *err)
+{
+    char *mask = NULL;
+    int row0, any0 = 0;
+    int i, j;
+
+    mask = calloc(m->rows, 1);
+    if (mask == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+    
+    for (i=0; i<m->rows; i++) {
+	row0 = 1;
+	for (j=0; j<m->cols; j++) {
+	    if (gretl_matrix_get(m, i, j) != 0.0) {
+		row0 = 0;
+		break;
+	    }
+	}
+	if (row0) {
+	    mask[i] = 1;
+	    any0 = 1;
+	}
+    }
+
+    if (!any0) {
+	free(mask);
+	mask = NULL;
+    }
+
+    return mask;
+}
+
+/**
+ * gretl_matrix_rank_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Performs a QR decomposition of matrix @m and uses this
+ * to assess the rank of @m.  If @m is not of full rank,
+ * constructs a mask of length equal to the numbers of
+ * columns in @m, with 1s in positions corresponding 
+ * to diagonal elements of R that are effectively 0, and
+ * 0s elsewhere.  If @m is of full column rank, %NULL is
+ * returned.
+ * 
+ * %E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or %NULL.
+ */
+
+char *gretl_matrix_rank_mask (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *Q = NULL;
+    gretl_matrix *R = NULL;
+    char *mask = NULL;
+    double Ri;
+    int fullrank = 1;
+    int i, n = m->cols;
+
+    Q = gretl_matrix_copy(m);
+    if (Q == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    R = gretl_matrix_alloc(n, n);
+    if (R == NULL) {
+	gretl_matrix_free(Q);
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *err = gretl_matrix_QR_decomp(Q, R);
+
+    if (!*err) {
+	mask = calloc(n, 1);
+	if (mask == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	for (i=0; i<n; i++) {
+	    Ri = gretl_matrix_get(R, i, i);
+	    if (fabs(Ri) < R_DIAG_MIN) {
+		mask[i] = 1;
+		fullrank = 0;
+	    }
+	}
+    }
+
+    if (*err || fullrank) {
+	free(mask);
+	mask = NULL;
+    }
+
+    gretl_matrix_free(Q);
+    gretl_matrix_free(R);
+    
+    return mask;
+}
+
+/**
+ * gretl_matrix_mp_ols:
+ * @y: dependent variable vector.
+ * @X: matrix of independent variables.
+ * @b: vector to hold coefficient estimates.
+ * @vcv: matrix to hold the covariance matrix of the coefficients,
+ * or %NULL if this is not needed.
+ * @uhat: vector to hold the regression residuals, or %NULL if 
+ * these are not needed.
+ * @s2: pointer to receive residual variance, or %NULL.  Note:
+ * if @s2 is %NULL, the "vcv" estimate will be plain (X'X)^{-1}.
+ *
+ * Computes OLS estimates using Cholesky factorization, via
+ * the GMP multiple-precision library, and puts the
+ * coefficient estimates in @b.  Optionally, calculates the
+ * covariance matrix in @vcv and the residuals in @uhat.
+ * 
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int gretl_matrix_mp_ols (const gretl_vector *y, const gretl_matrix *X,
+			 gretl_vector *b, gretl_matrix *vcv, 
+			 gretl_vector *uhat, double *s2)
+{
+    void *handle = NULL;
+    int (*matrix_mp_ols) (const gretl_vector *,
+			  const gretl_matrix *,
+			  gretl_vector *,
+			  gretl_matrix *,
+			  gretl_vector *,
+			  double *);
+    int err;
+
+    matrix_mp_ols = get_plugin_function("matrix_mp_ols", &handle);
+    if (matrix_mp_ols == NULL) {
+	return 1;
+    }
+
+    err = (*matrix_mp_ols)(y, X, b, vcv, uhat, s2);
+
+    close_plugin(handle);
+
+    return err;
+}
+
+
