@@ -21,7 +21,6 @@
 
 #include "libgretl.h" 
 #include "monte_carlo.h"
-#include "loop_private.h"
 #include "libset.h"
 #include "compat.h"
 #include "cmd_private.h"
@@ -35,9 +34,6 @@
 
 #define LOOP_DEBUG 0
 #define SUBST_DEBUG 0
-#define IDX_DEBUG 0
-
-#define GENCOMPILE 1
 
 #if LOOP_DEBUG
 # undef ENABLE_GMP
@@ -68,7 +64,7 @@ enum loop_types {
 			 l->type == EACH_LOOP)
 
 typedef struct {
-    int loopline;  /* location: line number in loop */
+    int linenum;   /* location: line number in loop */
     int n;         /* number of repetitions */
     int *list;     /* list of vars to print */
     bigval *sum;   /* running sum of values */
@@ -80,7 +76,7 @@ typedef struct {
 /* below: used for special "progressive" loop */ 
 
 typedef struct {
-    int loopline;           /* location: line number in loop */
+    int linenum;            /* location: line number in loop */
     int n;                  /* number of repetitions */
     int nc;                 /* number of coefficients */
     MODEL *model0;          /* copy of initial model */
@@ -96,7 +92,7 @@ typedef struct {
 } LOOP_MODEL;
 
 typedef struct {
-    int loopline;     /* location: line number in loop */ 
+    int linenum;      /* location: line number in loop */ 
     int n;            /* number of observations */
     char *fname;      /* filename for output */
     gretlopt opt;     /* formatting option */
@@ -107,14 +103,15 @@ typedef struct {
 enum loop_flags {
     LOOP_PROGRESSIVE = 1 << 0,
     LOOP_VERBOSE     = 1 << 1,
-    LOOP_QUIET       = 1 << 2
+    LOOP_QUIET       = 1 << 2,
+    LOOP_DELVAR      = 1 << 3
 };
 
 struct controller_ {
-    double val;
-    char vname[VNAMELEN];
-    int vsign;
-    char *expr;
+    double val;            /* evaluated value */
+    char vname[VNAMELEN];  /* name of (scalar) variable, if used */
+    int vsign;             /* 1 or -1, if vname is used */
+    char *expr;            /* expression to pass to genr, if used */
 };
 
 typedef struct controller_ controller;
@@ -133,12 +130,15 @@ struct LOOPSET_ {
     int iter;
     int index;
 
-    /* control variables */
-    char ichar;
-    int ival;
-    char brk;
+    /* index/foreach control variables */
+    char idxname[VNAMELEN];
+    int idxval;
     char listname[VNAMELEN];
 
+    /* break signal */
+    char brk;
+
+    /* control structures */
     controller init;
     controller test;
     controller delta;
@@ -149,14 +149,14 @@ struct LOOPSET_ {
     int n_models;
     int n_loop_models;
     int n_prints;
-#if GENCOMPILE
     int n_genrs;
-    GENERATOR **genrs;
-#endif
+    int n_children;
 
+    /* subsidiary objects */
     char **lines;
     int *ci;
     char **eachstrs;
+    GENERATOR **genrs;
     MODEL **models;
     int *model_lines;
     LOOP_MODEL *lmodels;
@@ -164,7 +164,6 @@ struct LOOPSET_ {
     LOOP_STORE store;
     LOOPSET *parent;
     LOOPSET **children;
-    int n_children;
     int parent_line;
 };
 
@@ -186,7 +185,6 @@ static void loop_print_free (LOOP_PRINT *lprn);
 static void loop_store_free (LOOP_STORE *lstore);
 static int extend_loop_dataset (LOOP_STORE *lstore);
 static void controller_free (controller *clr);
-static void set_active_loop (LOOPSET *loop);
 
 #define LOOP_BLOCK 32
 
@@ -207,47 +205,29 @@ int gretl_execute_loop (void)
     return loop_execute;
 }
 
-/* get a value from a loop "limit" element, either a specified
-   scalar variable or a numeric constant */
+/* For indexed loops: get a value from a loop "limit" element (lower
+   or upper).  If we got the name of a scalar variable at setup time,
+   look up its current value (and modify the sign if wanted).
+   Otherwise we should have got a numerical constant at setup, in
+   which case we just return that value.
+*/
 
-static double 
-controller_get_value (controller *clr, double ***pZ, DATAINFO *pdinfo)
+static double controller_get_val (controller *clr)
 {
-    double x = NADBL;
-
     if (clr->vname[0] != '\0') {
 	if (gretl_is_scalar(clr->vname)) {
-	    x = gretl_scalar_get_value(clr->vname) * clr->vsign;
+	    clr->val = gretl_scalar_get_value(clr->vname) * clr->vsign;
 	} else {
-	   gretl_errmsg_sprintf(_("'%s': not a scalar"), clr->vname);
+	    gretl_errmsg_sprintf(_("'%s': not a scalar"), clr->vname);
 	} 
-    } else {
-	/* numerical constant? */
-	x = clr->val;
-    }
+    } 
 
 #if LOOP_DEBUG
-    fprintf(stderr, "controller_get_value: vname='%s', returning %g\n", 
-	    clr->vname, x);
+    fprintf(stderr, "controller_get_val: vname='%s', returning %g\n", 
+	    clr->vname, clr->val);
 #endif
 
-    return x;
-}
-
-static double loop_get_initval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
-{
-#if LOOP_DEBUG
-    fprintf(stderr, "getting loop initval...\n");
-#endif
-    return controller_get_value(&loop->init, pZ, pdinfo);
-}
-
-static double loop_get_finalval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
-{
-#if LOOP_DEBUG
-    fprintf(stderr, "getting loop finalval...\n");
-#endif
-    return controller_get_value(&loop->final, pZ, pdinfo);
+    return clr->val;
 }
 
 /* apply initialization in case of for-loop */
@@ -335,26 +315,10 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
     }
 }
 
-#define plain_model_ci(c) (c == AR ||		\
-			   c == AR1 ||          \
-			   c == ARBOND ||	\
-			   c == ARCH ||		\
-			   c == ARMA ||		\
-			   c == GARCH ||	\
-			   c == HCCM ||		\
-			   c == HECKIT ||	\
-			   c == HSK ||		\
-			   c == LAD ||		\
-			   c == LOGISTIC ||	\
-			   c == LOGIT ||	\
-			   c == MPOLS ||	\
-			   c == OLS ||		\
-			   c == PANEL ||	\
-			   c == POISSON ||	\
-			   c == PROBIT ||	\
-			   c == TOBIT ||	\
-			   c == TSLS ||		\
-			   c == WLS)
+#define plain_model_ci(c) (MODEL_COMMAND(c) && \
+                           c != NLS && \
+                           c != MLE && \
+                           c != GMM)
 
 /**
  * ok_in_loop:
@@ -367,21 +331,18 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
 int ok_in_loop (int c)
 {
     /* here are the commands we _don't_ currently allow */
-    if (c == BXPLOT ||
-	c == CORRGM ||
+    if (c == CORRGM ||
 	c == CUSUM ||
 	c == DATA ||
 	c == DELEET ||
 	c == EQNPRINT ||
 	c == FUNC ||
-	c == GNUPLOT ||
 	c == HURST ||
 	c == INCLUDE ||
 	c == LEVERAGE ||
 	c == MODELTAB ||
 	c == NULLDATA ||
 	c == OPEN ||
-	c == QLRTEST ||
 	c == RMPLOT ||
 	c == RUN ||
 	c == SCATTERS ||
@@ -424,7 +385,7 @@ static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
 
 static void loop_store_init (LOOP_STORE *lstore)
 {
-    lstore->loopline = -1;
+    lstore->linenum = -1;
     lstore->n = 0;
     lstore->fname = NULL;
     lstore->opt = OPT_NONE;
@@ -444,8 +405,8 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->itermax = 0;
     loop->iter = 0;
     loop->err = 0;
-    loop->ichar = 0;
-    loop->ival = 0;
+    *loop->idxname = 0;
+    loop->idxval = 0;
     loop->brk = 0;
     *loop->listname = '\0';
 
@@ -459,10 +420,8 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->n_loop_models = 0;
     loop->n_prints = 0;
 
-#if GENCOMPILE
     loop->n_genrs = 0;
     loop->genrs = NULL;
-#endif
 
     loop->lines = NULL;
     loop->ci = NULL;
@@ -559,17 +518,19 @@ static void gretl_loop_destroy (LOOPSET *loop)
 
     loop_store_free(&loop->store);
 
-#if GENCOMPILE
     for (i=0; i<loop->n_genrs; i++) {
 	destroy_genr(loop->genrs[i]);
     }
     free(loop->genrs);    
-#endif
 
     if (loop->children != NULL) {
 	free(loop->children);
     }
 
+    if (loop->flags & LOOP_DELVAR) {
+	gretl_scalar_delete(loop->idxname, NULL);
+    }
+
     free(loop);
 }
 
@@ -577,48 +538,40 @@ static int parse_as_while_loop (LOOPSET *loop,
 				double ***pZ, DATAINFO *pdinfo,
 				const char *s)
 {
+    int err = 0;
+
 #if LOOP_DEBUG
     fprintf(stderr, "parse_as_while_loop: cond = '%s'\n", s);
 #endif
 
     if (s == NULL || *s == '\0') {
-	return E_PARSE;
-    }
-
-    loop->type = WHILE_LOOP;
-    loop->test.expr = gretl_strdup(s);
-    if (loop->test.expr == NULL) {
-	return E_ALLOC;
-    }
-
-    return 0;
-}
-
-static const char *ichars = "ijklmpqrs"; 
-
-static int loop_index_char_pos (int c)
-{
-    int i;
-
-    for (i=0; ichars[i] != '\0'; i++) {
-	if (c == ichars[i]) {
-	    return i;
+	err = E_PARSE;
+    } else {
+	loop->type = WHILE_LOOP;
+	loop->test.expr = gretl_strdup(s);
+	if (loop->test.expr == NULL) {
+	    err = E_ALLOC;
 	}
     }
 
-    return -1;
+    return err;
 }
 
-static int bad_ichar (char c)
+static int loop_attach_index_var (LOOPSET *loop, const char *vname,
+				  const DATAINFO *pdinfo)
 {
     int err = 0;
 
-    if (loop_index_char_pos(c) < 0) {
-	sprintf(gretl_errmsg, _("The index in a 'for' loop must be a "
-				"single character from the set '%s'"),
-		ichars);
-	err = E_DATA;
-    }
+    if (gretl_is_scalar(vname)) {
+	strcpy(loop->idxname, vname);
+	gretl_scalar_set_value(vname, loop->init.val);
+    } else {
+	err = gretl_scalar_add_with_check(vname, loop->init.val, pdinfo);
+	if (!err) {
+	    strcpy(loop->idxname, vname);
+	    loop->flags |= LOOP_DELVAR;
+	}
+    } 
 
     return err;
 }
@@ -646,7 +599,7 @@ static int index_get_limit (LOOPSET *loop, controller *clr, const char *s,
 	    *clr->vname = '\0';
 	    strncat(clr->vname, s, VNAMELEN - 1);
 	    clr->val = (int) gretl_scalar_get_value(s);
-	} else if ((v = series_index(pdinfo, s)) < pdinfo->v) {
+	} else if ((v = current_series_index(pdinfo, s)) >= 0) {
 	    /* found a series by the name of s */
 	    gretl_errmsg_sprintf(_("'%s': not a scalar"), s);
 	} else if (loop->parent != NULL && strlen(s) == gretl_namechar_spn(s)) {
@@ -667,23 +620,11 @@ static int index_get_limit (LOOPSET *loop, controller *clr, const char *s,
 static int parse_as_indexed_loop (LOOPSET *loop,
 				  const double **Z,
 				  const DATAINFO *pdinfo,
-				  char ichar,
 				  const char *lvar, 
 				  const char *start,
 				  const char *end)
 {
-    int err;
-
-    if (lvar != NULL) {
-	if (strlen(lvar) > 1) {
-	    /* deliberately set invalid ichar */
-	    ichar = 'x';
-	} else {
-	    ichar = *lvar;
-	}
-    }
-
-    err = bad_ichar(ichar);
+    int err = 0;
 
     /* starting and ending values: try for dates first, then
        numeric constants, then variables */
@@ -692,7 +633,7 @@ static int parse_as_indexed_loop (LOOPSET *loop,
     fprintf(stderr, "parse_as_indexed_loop: start='%s', end='%s'\n", start, end);
 #endif
 
-    if (!err && maybe_date(start)) {
+    if (maybe_date(start)) {
 	loop->init.val = dateton(start, pdinfo);
 	if (loop->init.val < 0) {
 	    err = E_DATA;
@@ -704,7 +645,7 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 		loop->type = DATED_LOOP;
 	    }
 	}
-    } else if (!err) { 
+    } else {
 	err = index_get_limit(loop, &loop->init, start, Z, pdinfo);
 	if (!err) {
 	    err = index_get_limit(loop, &loop->final, end, Z, pdinfo);
@@ -715,7 +656,7 @@ static int parse_as_indexed_loop (LOOPSET *loop,
     }
 
     if (!err) {
-	loop->ichar = ichar;
+	err = loop_attach_index_var(loop, lvar, pdinfo);
     }
 
 #if LOOP_DEBUG
@@ -853,10 +794,38 @@ static int loop_list_refresh (LOOPSET *loop, const DATAINFO *pdinfo)
     return err;
 }
 
-/* Identify the named list, if any, but do not yet fill out the
-   variable-name (or variable-number) strings: these will be set when
-   the loop is actually run, since the list may have changed in the
-   meantime.
+static int find_list_in_parentage (LOOPSET *loop, const char *s)
+{
+    char lname[VNAMELEN];
+    int i;
+
+    while ((loop = loop->parent) != NULL) {
+	for (i=0; i<loop->n_cmds; i++) {
+	    if (sscanf(loop->lines[i], "list %15[^ =]", lname)) {
+		if (!strcmp(lname, s)) {
+		    return 1;
+		}
+	    }
+	}
+    }
+
+    return 0;
+}
+
+/* We're looking at a "foreach" loop with just one field after the
+   index variable, so it's most likely a loop over a list.
+
+   We begin by looking for a currently existing named list, but if
+   this fails we don't give up immediately.  If we're working on an
+   embedded loop, the list may be created within a parent loop whose
+   commands have not yet been executed, so we search upward among the
+   ancestors of this loop (if any) for a relevant list-creation
+   command.
+
+   Even if we find an already-existing list, we do not yet fill out
+   the variable-name (or variable-number) strings: these will be set
+   when the loop is actually run, since the list may have changed in
+   the meantime.
 */
 
 static int list_loop_setup (LOOPSET *loop, char *s, int *nf)
@@ -870,101 +839,111 @@ static int list_loop_setup (LOOPSET *loop, char *s, int *nf)
     list = get_list_by_name(s);
 
 #if LOOP_DEBUG
-    printlist(list, "list, in list_loop_setup");
+    fprintf(stderr, "list_loop_setup: s = '%s'\n", s);
+    printlist(list, "get_list_by_name");
 #endif
 
-    if (list == NULL) {
+    if (list == NULL && !find_list_in_parentage(loop, s)) {
 	err = E_UNKVAR;
     } else {
 	*loop->listname = '\0';
 	strncat(loop->listname, s, VNAMELEN - 1);
-	*nf = list[0];
+	*nf = (list != NULL)? list[0] : 0;
     } 
 
     return err;
 }
 
+enum {
+    DOTTED_LIST,
+    WILDCARD_LIST
+};
+
 static int
 each_strings_from_list_of_vars (LOOPSET *loop, const DATAINFO *pdinfo, 
-				char *s, int *nf)
+				char *s, int *pnf, int type)
 {
-    char vn1[VNAMELEN], vn2[VNAMELEN];
-    int v1, v2;
+    int *list = NULL;
     int err = 0;
 
-    *nf = 0;
-
-    delchar(' ', s);
-
-    if (sscanf(s, "%15[^.]..%15s", vn1, vn2) != 2) {
-	err = E_PARSE;
-    } else {
-	v1 = series_index(pdinfo, vn1);
-	v2 = series_index(pdinfo, vn2);
-
-	if (v1 < 0 || v2 < 0 || v1 >= pdinfo->v || v2 >= pdinfo->v) {
+    if (type == WILDCARD_LIST) {
+	s += strspn(s, " \t");
+	list = varname_match_list(pdinfo, s);
+	if (list == NULL) {
 	    err = 1;
+	}
+    } else {
+	char vn1[VNAMELEN], vn2[VNAMELEN];
+	
+	delchar(' ', s);
+
+	if (sscanf(s, "%15[^.]..%15s", vn1, vn2) != 2) {
+	    err = E_PARSE;
 	} else {
-	    *nf = v2 - v1 + 1;
-	    if (*nf <= 0) {
-		err = 1;
+	    int v1 = current_series_index(pdinfo, vn1);
+	    int v2 = current_series_index(pdinfo, vn2);
+
+	    if (v1 < 0 || v2 < 0) {
+		err = E_UNKVAR;
+	    } else if (v2 - v1 + 1 <= 0) {
+		err = E_DATA;
+	    } else {
+		list = gretl_consecutive_list_new(v1, v2);
+		if (list == NULL) {
+		    err = E_ALLOC;
+		}
 	    }
 	}
-
-	if (!err) {
-	    err = allocate_each_strings(loop, *nf);
+	if (err) {
+	    *pnf = 0;
 	}
+    }
 
+    if (list != NULL) {
+	int i, vi;
+
+	err = allocate_each_strings(loop, list[0]);
 	if (!err) {
-	    int i;
-
-	    for (i=v1; i<=v2 && !err; i++) {
-		loop->eachstrs[i-v1] = gretl_strdup(pdinfo->varname[i]);
-		if (loop->eachstrs[i-v1] == NULL) {
-		    free_strings_array(loop->eachstrs, *nf);
+	    for (i=1; i<=list[0] && !err; i++) {
+		vi = list[i];
+		loop->eachstrs[i-1] = gretl_strdup(pdinfo->varname[vi]);
+		if (loop->eachstrs[i-1] == NULL) {
+		    free_strings_array(loop->eachstrs, list[0]);
 		    loop->eachstrs = NULL;
 		    err = E_ALLOC;
 		}
 	    }
 	}
+	if (!err) {
+	    *pnf = list[0];
+	}
+	free(list);
     }
-    
+
     return err;
 }
 
 static int
 parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 {
-    char ivar[8] = {0};
-    char ichar = 0;
+    char ivar[VNAMELEN] = {0};
     int done = 0;
     int i, nf, err = 0;
 
+    /* we're looking at the string that follows "loop foreach" */
     if (*s == '\0') {
 	return E_PARSE;
     }
-
-    /* we're looking at the string that follows "loop foreach" */
-
-    s++; /* skip space */
+    
+    s += strspn(s, " "); /* skip any spaces */
 
 #if LOOP_DEBUG
     fprintf(stderr, "parse_as_each_loop: s = '%s'\n", s);
 #endif 
 
-    /* should be something like "i " */
-    if (sscanf(s, "%7s", ivar) != 1) {
-	err = E_PARSE;
-    } else if (strlen(ivar) > 1) {
-	err = E_PARSE;
-    } else {
-	ichar = *ivar;
-	err = bad_ichar(ichar);
-    }
-
-    if (err) {
-	return err;
-    }
+    if (sscanf(s, "%15s", ivar) != 1) {
+	return E_PARSE;
+    } 
 
     s += strlen(ivar);
     nf = count_fields(s);
@@ -973,13 +952,16 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 	return E_PARSE;
     }
 
-    loop->ichar = ichar;
-    
     if (nf <= 3 && strstr(s, "..") != NULL) {
 	/* range of values, foo..quux */
-	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf);
+	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf,
+					     DOTTED_LIST);
 	done = 1;
-    }
+    } else if (nf == 1 && strchr(s, '*')) {
+	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf,
+					     WILDCARD_LIST);
+	done = (err == 0);
+    }	
 
     if (!done && nf == 1) {
 	/* try for a named list? */
@@ -1012,6 +994,7 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 	loop->type = EACH_LOOP;
 	loop->init.val = 1;
 	loop->final.val = nf;
+	err = loop_attach_index_var(loop, ivar, pdinfo);
     }   
 
 #if LOOP_DEBUG
@@ -1071,7 +1054,7 @@ parse_as_for_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, char *s)
     }  
 
     if (!err && (sc != 2 || s != q)) {
-	/* must have two semi-colons; must have reached righmost ')' */
+	/* must have two semi-colons; must have reached rightmost ')' */
 	err = E_PARSE;
     }
 
@@ -1088,7 +1071,6 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 				 double ***pZ, DATAINFO *pdinfo)
 {
     char lvar[VNAMELEN], rvar[VNAMELEN], op[VNAMELEN];
-    char ichar;
     int err = 0;
 
     /* skip preliminary string */
@@ -1098,20 +1080,22 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 	while (isspace(*s)) s++;
     }
 
+    /* syntactic slop: accept "for i=lo..hi" -> "i=lo..hi" */
+    if (!strncmp(s, "for ", 4) && !strstr(s, ";")) {
+	s += 4;
+    }
+
 #if LOOP_DEBUG
     fprintf(stderr, "parse_first_loopline: '%s'\n", s);
 #endif
 
-    if (sscanf(s, "%c = %15[^.]..%15s", &ichar, op, rvar) == 3) {
+    if (!strncmp(s, "foreach ", 8)) {
+	err = parse_as_each_loop(loop, pdinfo, s + 8);
+    } else if (sscanf(s, "%15[^= ] = %15[^.]..%15s", lvar, op, rvar) == 3) {
 	err = parse_as_indexed_loop(loop, (const double **) *pZ, pdinfo, 
-				    ichar, NULL, op, rvar);
-    } else if (sscanf(s, "for %15[^= ] = %15[^.]..%15s", lvar, op, rvar) == 3) {
-	err = parse_as_indexed_loop(loop, (const double **) *pZ, pdinfo, 
-				    0, lvar, op, rvar);
-    } else if (!strncmp(s, "foreach", 7)) {
-	err = parse_as_each_loop(loop, pdinfo, s + 7);
+				    lvar, op, rvar);
     } else if (!strncmp(s, "for", 3)) {
-	err = parse_as_for_loop(loop, pZ, pdinfo, s + 3);
+	err = parse_as_for_loop(loop, pZ, pdinfo, s + 4);
     } else if (!strncmp(s, "while", 5)) {
 	err = parse_as_while_loop(loop, pZ, pdinfo, s + 6);
     } else if (sscanf(s, "%15s", lvar) == 1) {
@@ -1177,7 +1161,6 @@ start_new_loop (char *s, LOOPSET *inloop,
     *err = parse_first_loopline(s, loop, pZ, pdinfo);
 
     if (!*err) {
-	/* allocate loop->lines, loop->ci */
 	*err = gretl_loop_prepare(loop);
     }
 
@@ -1204,7 +1187,7 @@ static int loop_count_too_high (LOOPSET *loop)
 
     if (loop->type == FOR_LOOP) {
 	if (nt >= MAX_FOR_TIMES) {
-	    sprintf(gretl_errmsg, _("Reached maximum interations, %d"),
+	    sprintf(gretl_errmsg, _("Reached maximum iterations, %d"),
 		    MAX_FOR_TIMES);
 	    loop->err = 1;
 	}
@@ -1213,7 +1196,7 @@ static int loop_count_too_high (LOOPSET *loop)
 	    max_iters = libset_get_int(LOOP_MAXITER);
 	}
 	if (nt >= max_iters) {
-	    sprintf(gretl_errmsg, _("Warning: no convergence after %d interations"),
+	    sprintf(gretl_errmsg, _("Warning: no convergence after %d iterations"),
 		    max_iters);
 	    loop->err = 1;
 	}
@@ -1244,11 +1227,6 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
 	loop->brk = 0;
 	ok = 0;
     } else if (loop->type == COUNT_LOOP || indexed_loop(loop)) {
-#if LOOP_DEBUG
-	fprintf(stderr, "** count or indexed loop: iter = %d, itermax = %d\n",
-		loop->iter, loop->itermax);
-#endif
-	/* simple count loop or indexed loop */
 	if (loop->iter < loop->itermax) {
 	    ok = 1;
 	}
@@ -1354,7 +1332,7 @@ static void loop_model_zero (LOOP_MODEL *lmod)
 
 static void loop_model_init (LOOP_MODEL *lmod, int lno)
 {
-    lmod->loopline = lno;
+    lmod->linenum = lno;
     lmod->nc = 0;
     lmod->model0 = NULL;
     lmod->bigarray = NULL;
@@ -1457,7 +1435,7 @@ static void loop_print_zero (LOOP_PRINT *lprn)
 
 static void loop_print_init (LOOP_PRINT *lprn, int lno)
 {
-    lprn->loopline = lno;
+    lprn->linenum = lno;
     lprn->list = NULL;
     lprn->sum = NULL;
     lprn->ssq = NULL;
@@ -1508,7 +1486,7 @@ static LOOP_PRINT *get_loop_print_by_line (LOOPSET *loop, int lno, int *err)
     int i, np = loop->n_prints;
 
     for (i=0; i<np; i++) {
-	if (loop->prns[i].loopline == lno) {
+	if (loop->prns[i].linenum == lno) {
 	    return &loop->prns[i];
 	}
     }
@@ -1535,7 +1513,7 @@ static void loop_store_free (LOOP_STORE *lstore)
     free(lstore->fname);
     lstore->fname = NULL;
 
-    lstore->loopline = -1;
+    lstore->linenum = -1;
     lstore->n = 0;
     lstore->opt = OPT_NONE;
 }
@@ -1611,7 +1589,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
 {
     int i, t, err = 0;
 
-    if (loop->store.loopline >= 0 && loop->store.loopline != lno) {
+    if (loop->store.linenum >= 0 && loop->store.linenum != lno) {
 	strcpy(gretl_errmsg, "Only one 'store' command is allowed in a "
 	       "progressive loop");
 	return E_DATA;
@@ -1625,7 +1603,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
 	}
     }
 
-    loop->store.loopline = lno;
+    loop->store.linenum = lno;
     t = loop->store.n;
 
     if (t >= loop->store.dinfo->n) {
@@ -1726,7 +1704,7 @@ get_loop_model_by_line (LOOPSET *loop, int lno, int *err)
 #endif
 
     for (i=0; i<nlm; i++) {
-	if (loop->lmodels[i].loopline == lno) {
+	if (loop->lmodels[i].linenum == lno) {
 	    return &loop->lmodels[i];
 	}
     }
@@ -2322,9 +2300,8 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 	sprintf(targ, "$%s", loop->init.vname);
 	targlen = strlen(targ);
     } else if (indexed_loop(loop)) {
-	targ[0] = '$';
-	targ[1] = loop->ichar;
-	targlen = 2;
+	sprintf(targ, "$%s", loop->idxname);
+	targlen = strlen(targ);
 	idx = loop->init.val + loop->iter;
     } else {
 	return 1;
@@ -2447,28 +2424,28 @@ top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
     if (is_list_loop(loop)) {
 	err = loop_list_refresh(loop, pdinfo);
     } else if (loop->type == INDEX_LOOP) {
-	loop->init.val = loop_get_initval(loop, pZ, pdinfo);
+	loop->init.val = controller_get_val(&loop->init);
     } else if (loop->type == FOR_LOOP) {
 	forloop_init(loop, pZ, pdinfo, &err);
     }
 
     if (!err && (loop->type == COUNT_LOOP || indexed_loop(loop))) {
-	double maxval = loop_get_finalval(loop, pZ, pdinfo);
-
-	if (na(loop->init.val) || na(maxval)) {
+	loop->final.val = controller_get_val(&loop->final);
+	if (na(loop->init.val) || na(loop->final.val)) {
 	    err = 1;
 	} else {
-	    loop->itermax = maxval - loop->init.val + 1;
+	    loop->itermax = loop->final.val - loop->init.val + 1;
 #if LOOP_DEBUG
 	    fprintf(stderr, "*** itermax = %g - %g + 1 = %d\n",
-		    maxval, loop->init.val, loop->itermax);
+		    loop->final.val, loop->init.val, loop->itermax);
 #endif
 	}
     }
 
     if (!err) {
 	if (indexed_loop(loop)) {
-	    loop->ival = loop->init.val;
+	    loop->idxval = loop->init.val;
+	    gretl_scalar_set_value(loop->idxname, loop->idxval);
 	}
 
 	/* initialization in case this loop is being run more than
@@ -2481,8 +2458,6 @@ top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 	    loop->models = NULL;
 	    loop->n_models = 0;
 	}
-
-	set_active_loop(loop);
     }
 
     return err;
@@ -2495,12 +2470,12 @@ print_loop_progress (const LOOPSET *loop, const DATAINFO *pdinfo,
     int i = loop->init.val + loop->iter;
 
     if (loop->type == INDEX_LOOP) {
-	pprintf(prn, "loop: %c = %d\n\n", loop->ichar, i);
+	pprintf(prn, "loop: %s = %d\n\n", loop->idxname, i);
     } else if (loop->type == DATED_LOOP) {
 	char obs[OBSLEN];
 
 	ntodate(obs, i, pdinfo);
-	pprintf(prn, "loop: %c = %s\n\n", loop->ichar, obs);
+	pprintf(prn, "loop: %s = %s\n\n", loop->idxname, obs);
     }
 }
 
@@ -2579,8 +2554,6 @@ static LOOPSET *get_child_loop_by_line (LOOPSET *loop, int lno)
     return NULL;
 }
 
-#if GENCOMPILE
-
 static int add_loop_genr (LOOPSET *loop, GENERATOR *genr, int lno)
 {
     GENERATOR **genrs;
@@ -2623,8 +2596,6 @@ static GENERATOR *get_loop_genr_by_line (LOOPSET *loop, int lno,
     return genr;
 }
 
-#endif
-
 /* get the next command for a loop by pulling a line off the
    stack of loop commands.
 */
@@ -2658,6 +2629,7 @@ static int block_model (CMD *cmd)
 
 int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo) 
 {
+    GENERATOR *genr;
     LOOPSET *loop = currloop;
     char *line = s->line;
     CMD *cmd = s->cmd;
@@ -2665,9 +2637,6 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     MODEL *pmod;
     LOOP_MODEL *lmod;
     LOOP_PRINT *lprn;
-#if GENCOMPILE
-    GENERATOR *genr;
-#endif
     char errline[MAXLINE];
     int indent0, mod_id = 0;
     int subst, lrefresh;
@@ -2679,6 +2648,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     if (loop == NULL) {
 	pputs(prn, "Got a NULL loop\n");
+	set_loop_off();
 	return 1;
     }
 
@@ -2707,7 +2677,6 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	}
 
 	while (!err && next_command(line, loop, &j)) {
-	    set_active_loop(loop);
 #if LOOP_DEBUG
 	    fprintf(stderr, " j=%d, line='%s'\n", j, line);
 #endif
@@ -2730,7 +2699,14 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    if (cmd->ci < 0) {
 		continue;
 	    } else if (err) {
-		break;
+		if (libset_get_bool(HALT_ON_ERR)) {
+		    break;
+		} else {
+		    /* try soldiering on */
+		    errmsg(err, prn);
+		    err = 0;
+		    continue;
+		}
 	    }
 
 	    if (!subst && cmd_subst(cmd)) {
@@ -2745,7 +2721,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		if (s->cmd->ci == ENDLOOP) {
 		    pputc(prn, '\n');
 		} else if (!loop_is_quiet(loop)) {
-		    echo_cmd(cmd, pdinfo, line, 0, prn);
+		    echo_cmd(cmd, pdinfo, line, CMD_BATCH_MODE, prn);
 		}
 	    }
 
@@ -2817,8 +2793,10 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		if (!loop_is_verbose(loop)) {
 		    cmd->opt |= OPT_Q;
 		}
-#if GENCOMPILE
 		if (subst || (cmd->opt & OPT_U)) {
+		    /* can't use a "compiled" genr if string substitution
+		       has been done, since the genr expression will not
+		       be constant */
 		    err = generate(line, pZ, pdinfo, cmd->opt, prn);
 		} else {
 		    genr = get_loop_genr_by_line(loop, j, line, pZ, pdinfo, &err);
@@ -2826,9 +2804,6 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 			err = execute_genr(genr, pZ, pdinfo, OPT_L, prn);
 		    }
 		}
-#else
-		err = generate(line, pZ, pdinfo, cmd->opt, prn);
-#endif
 	    } else {
 		err = gretl_cmd_exec(s, pZ, pdinfo);
 		if (!err && block_model(cmd)) {
@@ -2840,21 +2815,26 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    }
 	} /* end execution of commands within loop */
 
-	if (err && !libset_get_bool(HALT_ON_ERR)) {
-	    errmsg(err, prn);
-	    err = 0;
-	}
-
 	if (err) {
-	    gretl_if_state_clear();
-	} else {
+	    if (!libset_get_bool(HALT_ON_ERR)) {
+		/* print error message but keep going */
+		errmsg(err, prn);
+		err = 0;
+		/* should we clear the "if state" here? */
+	    } else {
+		gretl_if_state_clear();
+	    }
+	} 
+
+	if (!err) {
 	    err = gretl_if_state_check(indent0);
 	}
 
 	if (!err) {
 	    loop->iter += 1;
 	    if (indexed_loop(loop)) {
-		loop->ival += 1;
+		loop->idxval += 1;
+		gretl_scalar_set_value(loop->idxname, loop->idxval);
 	    } else if (lrefresh) {
 		/* added 2008-01-11, AC */
 		loop_list_refresh(loop, pdinfo);
@@ -2896,13 +2876,11 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	*line = '\0';
     } 
 
-    set_active_loop(loop->parent);
-    set_loop_off();
-
     if (loop->parent == NULL) {
 	/* reached top of stack: clean up */
 	gretl_loop_destroy(loop);
 	currloop = NULL;
+	set_loop_off();
     }
 
     if (libset_get_bool(HALT_ON_ERR)) {
@@ -2912,60 +2890,5 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     }
 }
 
-static const LOOPSET *
-parent_with_ichar (const LOOPSET *loop, int c)
-{
-    while ((loop = loop->parent) != NULL) {
-	if (indexed_loop(loop) && loop->ichar == c) {
-	    return loop;
-	}
-    }
 
-    return NULL;
-}
 
-static const LOOPSET *
-indexed_loop_set_or_get (LOOPSET *loop, int set, int c)
-{
-    static LOOPSET *active_loop;
-    const LOOPSET *ret = NULL;
-
-    if (set) {
-	active_loop = loop;
-    } else if (active_loop != NULL) {
-	if (indexed_loop(active_loop) && active_loop->ichar == c) {
-	    ret = active_loop;
-	} else {
-	    ret = parent_with_ichar(active_loop, c);
-	}
-    }
-
-#if IDX_DEBUG > 1
-    if (set) {
-	fprintf(stderr, "indexed_loop_record: set active_loop = %p\n",
-		(void *) loop);
-    } else {
-	fprintf(stderr, "indexed_loop_record: returning %p for c='%c'\n",
-		(void *) ret, c);
-    }	
-#endif
-
-    return ret;
-}
-
-static void set_active_loop (LOOPSET *loop)
-{
-    indexed_loop_set_or_get(loop, 1, 0);
-}
-
-int is_active_index_loop_char (int c)
-{
-    return indexed_loop_set_or_get(NULL, 0, c) != NULL;
-}
-
-int loop_scalar_read (int c)
-{
-    const LOOPSET *loop = indexed_loop_set_or_get(NULL, 0, c);
-
-    return (loop == NULL)? -1 : loop->ival;
-}
