@@ -27,10 +27,12 @@ typedef struct ocset_ ocset;
 struct _nlspec {
     int ci;             /* NLS, MLE or GMM */
     int generr;         /* error from genr */
-    int mode;           /* derivatives: numeric or analytic */
+    int flags;          /* numeric or analytic derivatives, etc. */
     gretlopt opt;       /* can include OPT_V for verbose output; if ci = MLE
 			   can also include OPT_H (Hessian) or OPT_R (QML)
-			   to control the estimator of the variance matrix 
+			   to control the estimator of the variance matrix;
+			   new: can also include OPT_N to force use of 
+			   numerical derivatives.
 			*/
     int dv;             /* ID number of dependent variable (NLS) */
     char lhname[VNAMELEN]; /* name of LHS var in criterion function */
@@ -58,6 +60,8 @@ struct _nlspec {
     double tol;         /* tolerance for stopping iteration */
     parm *params;       /* array of information on function parameters
 			   (see the parm_ struct above) */
+    double *fvec;       /* function vector */
+    double *jac;        /* Jacobian array */
     double *coeff;      /* coefficient estimates */
     double *hessvec;    /* vech representation of negative inverse of
 			   Hessian */
@@ -80,14 +84,21 @@ int update_coeff_values (const double *x, nlspec *s);
 
 int check_gmm_requirements (nlspec *spec);
 
-int 
-nlspec_add_orthcond (nlspec *s, const char *str,
-		     const double **Z, const DATAINFO *pdinfo);
+int nlspec_add_orthcond (nlspec *s, const char *str,
+			 const double **Z, const DATAINFO *pdinfo);
+
+int nlspec_add_ivreg_oc (nlspec *s, int lhv, const int *rlist,
+			 const double **Z);
 
 int nlspec_add_weights (nlspec *s, const char *str);
 
+void nlspec_print_gmm_info (const nlspec *spec, PRN *prn);
+
+void maybe_add_gmm_residual (MODEL *pmod, const nlspec *spec, 
+			     const DATAINFO *pdinfo);
+
 int gmm_add_vcv (MODEL *pmod, nlspec *spec);
 
-int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn);
+int gmm_calculate (nlspec *s, PRN *prn);
 
 int gmm_missval_check_etc (nlspec *s);
