@@ -24,8 +24,8 @@
 #include "system.h"
 #include "objstack.h"
 #include "usermat.h"
-#include "modelspec.h"
 #include "forecast.h"
+#include "kalman.h"
 
 #define ODEBUG 0
 
@@ -92,16 +92,35 @@ void gretl_object_ref (void *ptr, GretlObjType type)
     }
 }
 
-static void gretl_object_unstack (void *ptr)
+enum {
+    UNSTACK_REMOVE,
+    UNSTACK_DESTROY
+};
+
+static void gretl_object_unstack (void *ptr, int action)
 {
     int i, pos = -1;
 
+    if (action == UNSTACK_DESTROY && ptr == last_model.ptr) {
+#if ODEBUG
+	fprintf(stderr, "gretl_object_unstack: %p is 'last_model'\n", ptr);
+#endif
+	/* avoid double-freeing */
+	last_model.ptr = NULL;
+	last_model.type = GRETL_OBJ_NULL;
+    }
+
     for (i=0; i<n_obj; i++) {
 	if (ptr == ostack[i].ptr) {
 	    pos = i;
 	    break;
 	}
     }
+
+#if ODEBUG
+    fprintf(stderr, "gretl_object_unstack: stack pos for %p = %d\n",
+	    ptr, pos);
+#endif
     
     if (pos >= 0) {
 	n_obj--;
@@ -130,7 +149,7 @@ static void gretl_object_destroy (void *ptr, GretlObjType type)
 	    ptr, type);
 #endif
 
-    gretl_object_unstack(ptr);
+    gretl_object_unstack(ptr, UNSTACK_DESTROY);
     
     if (type == GRETL_OBJ_EQN) {
 	gretl_model_free(ptr);
@@ -634,7 +653,7 @@ void remove_model_from_stack_on_exit (MODEL *pmod)
 
 void gretl_object_remove_from_stack (void *ptr, GretlObjType type)
 {
-    gretl_object_unstack(ptr);
+    gretl_object_unstack(ptr, UNSTACK_REMOVE);
     gretl_object_unref(ptr, type);
 }
 
@@ -743,14 +762,14 @@ int gretl_stack_object_as (void *ptr, GretlObjType type, const char *name)
 
 int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
 {
-    char vname[MAXSAVENAME];
+    char name[MAXSAVENAME];
     int ret = 0;
 
     if (var == NULL) {
 	return 0;
     }
 
-    gretl_cmd_get_savename(vname);
+    gretl_cmd_get_savename(name);
 
 #if ODEBUG
     fprintf(stderr, "\nmaybe_stack_var: initial refcount = %d\n", var->refcount);
@@ -763,8 +782,8 @@ int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
 	    (void *) var, var->refcount);
 #endif
 
-    if (*vname) {
-	ret = real_stack_object(var, GRETL_OBJ_VAR, vname, NULL);
+    if (*name) {
+	ret = real_stack_object(var, GRETL_OBJ_VAR, name, NULL);
     } 
 
     return ret;
@@ -787,25 +806,15 @@ int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
 
 int maybe_stack_model (MODEL *pmod, const CMD *cmd, PRN *prn)
 {
-    char mname[MAXSAVENAME];
+    char name[MAXSAVENAME];
     MODEL *cpy = NULL;
     int err = 0;
 
-    gretl_cmd_get_savename(mname);
-
-#if ODEBUG
-    fprintf(stderr, "\nmaybe_stack_model: initial refcount = %d\n", 
-	    pmod->refcount);
-#endif
+    gretl_cmd_get_savename(name);
 
     set_as_last_model(pmod, GRETL_OBJ_EQN);
 
-#if ODEBUG
-    fprintf(stderr, "maybe_stack_model: set %p as last model, refcount %d\n",
-	    (void *) pmod, pmod->refcount);
-#endif
-
-    if (*mname == 0) {
+    if (*name == 0) {
 	return 0;
     }
 
@@ -815,11 +824,13 @@ int maybe_stack_model (MODEL *pmod, const CMD *cmd, PRN *prn)
     }
 
     if (!err) {
-	err = real_stack_object(cpy, GRETL_OBJ_EQN, mname, NULL);
+	err = real_stack_object(cpy, GRETL_OBJ_EQN, name, NULL);
     }
 
     if (!err) {
-	pprintf(prn, _("%s saved\n"), mname);
+	pprintf(prn, _("%s saved\n"), name);
+    } else {
+	errmsg(err, prn);
     }
 
     return err;
@@ -830,7 +841,9 @@ int maybe_stack_model (MODEL *pmod, const CMD *cmd, PRN *prn)
 /* retrieve from an object some value that is stored on the object in
    the form of a simple scalar */
 
-static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
+static double real_get_obj_scalar (void *p, GretlObjType type, 
+				   double ***pZ, DATAINFO *pdinfo,
+				   int idx)
 {
     double x = INVALID_STAT;
     int err = 0;
@@ -842,7 +855,7 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) p;
 
-	x = gretl_model_get_scalar(pmod, idx, &err);
+	x = gretl_model_get_scalar(pmod, idx, pZ, pdinfo, &err);
 	if (err) {
 	    x = INVALID_STAT;
 	}
@@ -1033,7 +1046,9 @@ int *saved_object_get_list (const char *oname, int idx, int *err)
     return ret;
 }
 
-double saved_object_get_scalar (const char *oname, int idx, int *err)
+double saved_object_get_scalar (const char *oname, int idx, 
+				double ***pZ, DATAINFO *pdinfo,
+				int *err)
 {
     double ret = INVALID_STAT;
     stacker *smatch;
@@ -1041,7 +1056,8 @@ double saved_object_get_scalar (const char *oname, int idx, int *err)
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
-	ret = real_get_obj_scalar(smatch->ptr, smatch->type, idx);
+	ret = real_get_obj_scalar(smatch->ptr, smatch->type, 
+				  pZ, pdinfo, idx);
     }
 
     if (ret == INVALID_STAT) {
@@ -1085,6 +1101,8 @@ saved_object_get_matrix (const char *oname, int idx, int *err)
 
     if (idx == M_FCAST || idx == M_FCERR) {
 	M = get_forecast_matrix(idx, err);
+    } else if (idx == M_KLLT || idx == M_KUHAT) {
+	M = user_kalman_get_matrix(idx, err);
     } else {
 	stacker *smatch = find_smatch(oname);
 
@@ -1235,6 +1253,8 @@ static void saved_object_free (stacker *s)
     }
 }
 
+#define sys_modtest_opt_ok(o) (o & (OPT_A | OPT_H | OPT_N))
+
 int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo, 
 			PRN *prn)
 {
@@ -1245,7 +1265,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
     ptr = get_last_model(&type);  
     if (ptr == NULL) {
 	pputs(prn, _("Can't do this: no model has been estimated yet\n"));
-	return 1;
+	return E_DATA;
     }
 
     if (type == GRETL_OBJ_EQN) {
@@ -1257,13 +1277,13 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 	if (model_sample_problem(pmod, pdinfo)) {
 	    pputs(prn, _("Can't do: the current data set is different from "
 			 "the one on which\nthe reference model was estimated\n"));
-	    err = 1;
+	    err = E_DATA;
 	}
     } else if (type == GRETL_OBJ_SYS) {
 	err = E_NOTIMP;
-	if (ci == RESTRICT || ci == TESTUHAT || ci == FCAST) {
+	if (ci == RESTRICT || ci == FCAST) {
 	    err = 0;
-	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
+	} else if (ci == MODTEST && sys_modtest_opt_ok(opt)) {
 	    err = 0;
 	}
     } else if (type == GRETL_OBJ_VAR) {
@@ -1274,22 +1294,18 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 
 	if (ci == RESTRICT && r > 0) {
 	    err = 0;
-	} else if (ci == TESTUHAT || ci == FCAST) {
+	} else if (ci == FCAST) {
 	    err = 0;
-	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
+	} else if (ci == MODTEST && sys_modtest_opt_ok(opt)) {
 	    err = 0;
 	} 
     }
 
-    if (err == E_NOTIMP) {
-	pputs(prn, _("Sorry, command not available for this estimator"));
-	pputc(prn, '\n');
-    }
-
     return err;
 }
 
-int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
+int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, 
+			  gretlopt opt, PRN *prn)
 {
     GretlObjType type;
     void *ptr;
@@ -1307,7 +1323,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	    gretl_model_get_int(pmod, "ordered")) {
 	    err = E_NOTIMP;
 	} else if (!exact_fit_check(pmod, prn)) {
-	    err = model_error_dist(ptr, pZ, pdinfo, prn);
+	    err = model_error_dist(ptr, pZ, pdinfo, opt, prn);
 	}
     } else if (type == GRETL_OBJ_SYS) {
 	err = system_normality_test(ptr, prn);
@@ -1378,16 +1394,20 @@ void gretl_saved_objects_cleanup (void)
 {
     int i;
 
+#if ODEBUG
+    fprintf(stderr, "gretl_saved_objects_cleanup, n_obj = %d\n", n_obj);
+#endif
+
     for (i=0; i<n_obj; i++) {
 	if (ostack[i].ptr == last_model.ptr) {
 	    if (gretl_object_get_refcount(ostack[i].ptr, ostack[i].type) == 1) {
 		/* don't double-free! */
 		last_model.ptr = NULL;
+		
 	    }
 	}
 #if ODEBUG
-	fprintf(stderr, "gretl_saved_objects_cleanup:\n"
-		" calling saved_object_free on ostack[%d]\n", i);
+	fprintf(stderr, " calling saved_object_free on ostack[%d]\n", i);
 #endif
 	saved_object_free(&ostack[i]);
     }
