@@ -20,21 +20,6 @@
 #include "libgretl.h"
 #include "plotspec.h"
 
-/**
- * plotspec_label_init:
- * @lbl: pointer to gnuplot label struct.
- * 
- * Initializes the label.
- */
-
-void plotspec_label_init (GPT_LABEL *lbl)
-{
-    lbl->text[0] = '\0';
-    lbl->just = GP_JUST_LEFT;
-    lbl->pos[0] = NADBL;
-    lbl->pos[1] = NADBL;
-}
-
 GPT_SPEC *plotspec_new (void)
 {
     GPT_SPEC *spec;
@@ -46,6 +31,13 @@ GPT_SPEC *plotspec_new (void)
     }
 
     spec->lines = NULL;
+    spec->n_lines = 0;
+
+    spec->labels = NULL;
+    spec->n_labels = 0;
+
+    spec->literal = NULL;
+    spec->n_literal = 0;
 
     for (i=0; i<4; i++) {
 	spec->titles[i][0] = 0;
@@ -54,17 +46,12 @@ GPT_SPEC *plotspec_new (void)
     *spec->xvarname = '\0';
     *spec->yvarname = '\0';
 
-    spec->literal = NULL;
-    spec->n_literal = 0;
-
-    for (i=0; i<MAX_PLOT_LABELS; i++) {
-	plotspec_label_init(&(spec->labels[i]));
-    }
-
+    spec->xfmt[0] = 0;
     spec->xtics[0] = 0;
     spec->mxtics[0] = 0;
+    spec->yfmt[0] = 0;
     spec->fname[0] = 0;
-    strcpy(spec->keyspec, "left top");
+    spec->keyspec = GP_KEY_LEFT_TOP;
 
     for (i=0; i<4; i++) {
 	spec->range[i][0] = NADBL;
@@ -88,12 +75,13 @@ GPT_SPEC *plotspec_new (void)
     spec->labeled = NULL;
     spec->ptr = NULL;
     spec->reglist = NULL;
-    spec->n_lines = 0;
     spec->nobs = 0;
     spec->okobs = 0;
     spec->pd = 0;
     spec->boxwidth = 0;
     spec->samples = 0;
+    spec->border = GP_BORDER_DEFAULT;
+    spec->bmargin = 0;
 
     spec->termtype = GP_TERM_NONE;
 
@@ -110,6 +98,10 @@ void plotspec_destroy (GPT_SPEC *spec)
 	free(spec->lines);
     }
 
+    if (spec->labels != NULL) {
+	free(spec->labels);
+    }    
+
     if (spec->data != NULL) {
 	free(spec->data);
     }
@@ -137,6 +129,158 @@ void plotspec_destroy (GPT_SPEC *spec)
     free(spec);
 }
 
+static gp_style_spec style_specs[] = {
+    { GP_STYLE_LINES,        N_("lines") },
+    { GP_STYLE_POINTS,       N_("points") },
+    { GP_STYLE_LINESPOINTS,  N_("linespoints") },
+    { GP_STYLE_IMPULSES,     N_("impulses") },
+    { GP_STYLE_DOTS,         N_("dots") },
+    { GP_STYLE_STEPS,        N_("steps") },
+    { GP_STYLE_BOXES,        N_("boxes") },
+    { GP_STYLE_ERRORBARS,    "errorbars" },
+    { GP_STYLE_FILLEDCURVE,  "filledcurve" },
+    { GP_STYLE_CANDLESTICKS, "candlesticks" },
+    { 0,                     NULL }
+};
+
+const char *gp_line_style_string (int t)
+{
+    int i;
+
+    for (i=0; style_specs[i].sty != 0; i++) {
+	if (t == style_specs[i].sty) {
+	    return style_specs[i].str;
+	}
+    }
+
+    return N_("lines");
+}
+
+int gp_style_from_string (const char *s)
+{
+    int i;
+
+    for (i=0; style_specs[i].sty != 0; i++) {
+	if (!strcmp(s, style_specs[i].str)) {
+	    return style_specs[i].sty;
+	}
+    }
+
+    /* allowed abbreviations */
+
+    if (!strcmp(s, "l")) {
+	return GP_STYLE_LINES;
+    } else if (!strcmp(s, "p")) {
+	return GP_STYLE_POINTS;
+    } else if (!strcmp(s, "lp")) {
+	return GP_STYLE_LINESPOINTS;
+    } else if (!strcmp(s, "i")) {
+	return GP_STYLE_IMPULSES;
+    }
+
+    /* fallback */
+    return GP_STYLE_LINES;
+}
+
+int gp_style_from_translation (const char *s)
+{
+    int i;
+
+    for (i=0; style_specs[i].sty != 0; i++) {
+	if (!strcmp(s, _(style_specs[i].str))) {
+	    return style_specs[i].sty;
+	}
+    }
+
+    return GP_STYLE_LINES;
+}
+
+gp_style_spec *get_style_spec (int t)
+{
+    int i;
+
+    for (i=0; style_specs[i].sty != 0; i++) {
+	if (t == style_specs[i].sty) {
+	    return &style_specs[i];
+	}
+    }
+
+    return NULL;
+}
+
+static gp_style_spec key_specs[] = {
+    { GP_KEY_LEFT_TOP,     N_("left top") },
+    { GP_KEY_RIGHT_TOP,    N_("right top") },
+    { GP_KEY_LEFT_BOTTOM,  N_("left bottom") },
+    { GP_KEY_RIGHT_BOTTOM, N_("right bottom") },
+    { GP_KEY_OUTSIDE,      N_("outside") },
+    { GP_KEY_NONE,         N_("none") },
+    { -1,                  NULL }
+};
+
+static const char *gp_keypos_string (int t)
+{
+    int i;
+
+    for (i=0; key_specs[i].str != NULL; i++) {
+	if (t == key_specs[i].sty) {
+	    return key_specs[i].str;
+	}
+    }
+
+    return N_("none");
+}
+
+void print_keypos_string (int t, FILE *fp)
+{
+    const char *s = gp_keypos_string(t);
+
+    if (!strcmp(s, "none")) {
+	fputs("set nokey\n", fp);
+    } else {
+	fprintf(fp, "set key %s\n", s);
+    }
+}
+
+int gp_keypos_from_string (const char *s)
+{
+    int i;
+
+    for (i=0; key_specs[i].sty >= 0; i++) {
+	if (!strcmp(s, key_specs[i].str)) {
+	    return key_specs[i].sty;
+	}
+    }
+
+    return GP_KEY_NONE;
+}
+
+int gp_keypos_from_translation (const char *s)
+{
+    int i;
+
+    for (i=0; key_specs[i].sty >= 0; i++) {
+	if (!strcmp(s, _(key_specs[i].str))) {
+	    return key_specs[i].sty;
+	}
+    }
+
+    return GP_KEY_NONE;
+}
+
+gp_style_spec *get_keypos_spec (int t)
+{
+    int i;
+
+    for (i=0; key_specs[i].sty >= 0; i++) {
+	if (t == key_specs[i].sty) {
+	    return &key_specs[i];
+	}
+    }
+
+    return NULL;
+}
+
 int plotspec_add_line (GPT_SPEC *spec)
 {
     GPT_LINE *lines;
@@ -151,18 +295,168 @@ int plotspec_add_line (GPT_SPEC *spec)
     spec->n_lines += 1;
 
     lines[n].varnum = 0;
-    lines[n].title[0] = 0;
-    lines[n].formula[0] = 0;
-    lines[n].style[0] = 0;
-    lines[n].scale[0] = 0;
+    lines[n].style = 0;
+    lines[n].scale = 1.0;
+    lines[n].title[0] = '\0';
+    lines[n].formula[0] = '\0';
+    lines[n].rgb[0] = '\0';
     lines[n].yaxis = 1;
-    lines[n].type = 0;
+    lines[n].type = LT_NONE;
+    lines[n].ptype = 0;
     lines[n].width = 1;
     lines[n].ncols = 0;
+    lines[n].whiskwidth = 0;
+    lines[n].flags = 0;
 
     return 0;
 }
 
+static void copy_line_content (GPT_LINE *targ, GPT_LINE *src)
+{
+    targ->varnum = src->varnum;
+    targ->style = src->style;
+    targ->scale = src->scale;
+    strcpy(targ->title, src->title);
+    strcpy(targ->formula, src->formula);
+    strcpy(targ->rgb, src->rgb);
+    targ->yaxis = src->yaxis;
+    targ->type = src->type;
+    targ->ptype = src->ptype;
+    targ->width = src->width;
+    targ->ncols = src->ncols;
+    targ->whiskwidth = src->whiskwidth;
+    targ->flags = src->flags;
+}
+
+int plotspec_delete_line (GPT_SPEC *spec, int i)
+{
+    GPT_LINE *lines = spec->lines;
+    int j, n = spec->n_lines;
+
+    if (i < 0 || i >= n) {
+	return E_DATA;
+    }
+
+    for (j=i; j<n-1; j++) {
+	copy_line_content(&lines[j], &lines[j+1]);
+    }
+
+    spec->n_lines -= 1;
+
+    lines = realloc(spec->lines, (n - 1) * sizeof *lines);
+    if (lines == NULL) {
+	return E_ALLOC;
+    }
+
+    spec->lines = lines;
+
+    return 0;
+}
+
+GPT_LINE *plotspec_clone_lines (GPT_SPEC *spec, int *err)
+{
+    GPT_LINE *lines = NULL;
+    int i;
+
+    if (spec->n_lines == 0) {
+	/* no-op */
+	return NULL;
+    }
+
+    lines = malloc(spec->n_lines * sizeof *lines);
+    if (lines == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (i=0; i<spec->n_lines; i++) {
+	    copy_line_content(&lines[i], &spec->lines[i]);
+	}
+    }
+
+    return lines;
+}
+
+int plotspec_add_label (GPT_SPEC *spec)
+{
+    GPT_LABEL *labels;
+    int n = spec->n_labels;
+
+    labels = realloc(spec->labels, (n + 1) * sizeof *labels);
+    if (labels == NULL) {
+	return E_ALLOC;
+    }
+
+    spec->labels = labels;
+    spec->n_labels += 1;
+
+    labels[n].text[0] = '\0';
+    labels[n].pos[0] = NADBL;
+    labels[n].pos[1] = NADBL;
+    labels[n].just = GP_JUST_LEFT;
+
+    return 0;
+}
+
+static void copy_label_content (GPT_LABEL *targ, GPT_LABEL *src)
+{
+    strcpy(targ->text, src->text);
+    targ->pos[0] = src->pos[0];
+    targ->pos[1] = src->pos[1];
+    targ->just = src->just;
+}
+
+int plotspec_delete_label (GPT_SPEC *spec, int i)
+{
+    GPT_LABEL *labels = spec->labels;
+    int j, n = spec->n_labels;
+    int err = 0;
+
+    if (i < 0 || i >= n) {
+	return E_DATA;
+    }
+
+    for (j=i; j<n-1; j++) {
+	copy_label_content(&labels[j], &labels[j+1]);
+    }
+
+    spec->n_labels -= 1;
+
+    if (spec->n_labels == 0) {
+	free(spec->labels);
+	spec->labels = NULL;
+    } else {
+	labels = realloc(spec->labels, (n - 1) * sizeof *labels);
+	if (labels == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    spec->labels = labels;
+	}
+    }
+
+    return err;
+}
+
+GPT_LABEL *plotspec_clone_labels (GPT_SPEC *spec, int *err)
+{
+    GPT_LABEL *labels = NULL;
+    int i;
+
+    if (spec->n_labels == 0) {
+	/* no-op */
+	return NULL;
+    }
+
+    labels = malloc(spec->n_labels * sizeof *labels);
+    if (labels == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (i=0; i<spec->n_labels; i++) {
+	    copy_label_content(&labels[i], &spec->labels[i]);
+	}
+    }
+
+    return labels;
+}
+
 static char *escape_quotes (const char *s)
 {
     if (strchr(s, '"') == NULL) {
@@ -201,13 +495,13 @@ static char *escape_quotes (const char *s)
 const char *gp_justification_string (int j)
 {
     if (j == GP_JUST_LEFT) {
-	return "left";
+	return N_("left");
     } else if (j == GP_JUST_CENTER) {
-	return "center";
+	return N_("center");
     } else if (j == GP_JUST_RIGHT) {
-	return "right";
+	return N_("right");
     } else {
-	return "left";
+	return N_("left");
     }
 }
 
@@ -347,7 +641,7 @@ void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
 	    continue;
 	}
 
-	fprintf(fp, "set %srange [%.7g:%.7g]\n", rstrs[i], 
+	fprintf(fp, "set %srange [%.10g:%.10g]\n", rstrs[i], 
 		spec->range[i][0], spec->range[i][1]);
 
 	if (i == 4 && spec->code == PLOT_PROB_DIST && spec->samples == 0) {
@@ -359,11 +653,116 @@ void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
 
     if (spec->boxwidth > 0 && spec->boxwidth < 1) {
 	fprintf(fp, "set boxwidth %.3f\n", (double) spec->boxwidth);
+    } else if (spec->boxwidth < 0 && spec->boxwidth > -1) {
+	fprintf(fp, "set boxwidth %g absolute\n", (double) -spec->boxwidth);
     }
 
     gretl_pop_c_numeric_locale();
 }
 
+static int blank_user_line (const GPT_SPEC *spec, int i)
+{
+    return ((spec->lines[i].flags & GP_LINE_USER) && 
+	    spec->lines[i].formula[0] == '\0');
+}
+
+static void print_user_lines_info (const GPT_SPEC *spec, FILE *fp)
+{
+    int i, n = 0;
+
+    for (i=0; i<spec->n_lines; i++) {
+	if (spec->lines[i].flags & GP_LINE_USER) {
+	    n++;
+	}
+    }
+
+    if (n > 0) {
+	fprintf(fp, "# %d user-defined lines: ", n);
+	for (i=0; i<spec->n_lines; i++) {
+	    if (spec->lines[i].flags & GP_LINE_USER) {
+		fprintf(fp, "%d ", i);
+	    }
+	}
+	fputc('\n', fp);
+    }
+}
+
+static int more_lines (const GPT_SPEC *spec, int i, int skipline)
+{
+    if (i == spec->n_lines - 1) {
+	/* at last line */
+	return 0;
+    } else {
+	int j;
+
+	for (j=i+1; j<spec->n_lines; j++) {
+	    if (j != skipline && !blank_user_line(spec, j)) {
+		return 1;
+	    }
+	}
+	return 0;
+    }
+}
+
+static void print_linestyle (const GPT_SPEC *spec, int i, FILE *fp)
+{
+    GPT_LINE *line;
+    int done = 0;
+
+    fprintf(fp, "set style line %d ", i+1);
+
+    if (i < spec->n_lines) {
+	line = &spec->lines[i];
+	if (*line->rgb != '\0' && line->type != LT_NONE) {
+	    fprintf(fp, "lc rgb \"%s\" lt %d\n", line->rgb, line->type);
+	    done = 1;
+	} else if (*line->rgb != '\0') {
+	    fprintf(fp, "lc rgb \"%s\"\n", line->rgb);
+	    done = 1;
+	}
+    }
+
+    if (!done) {
+	const gretlRGB *color = get_graph_color(i);
+	char cstr[8];
+
+	print_rgb_hash(cstr, color);
+	fprintf(fp, "lc rgb \"%s\"\n", cstr);
+    }
+}
+
+static void write_styles_from_plotspec (const GPT_SPEC *spec, FILE *fp)
+{
+    char cstr[8];
+    int i;
+    
+    if (frequency_plot_code(spec->code)) {
+	const gretlRGB *color = get_graph_color(BOXCOLOR);
+
+	print_rgb_hash(cstr, color);
+	fprintf(fp, "set style line 1 lc rgb \"%s\"\n", cstr);
+	fputs("set style line 2 lc rgb \"#000000\"\n", fp);
+    } else if (spec->code == PLOT_RQ_TAU) {
+	fputs("set style line 1 lc rgb \"#000000\"\n", fp);
+	for (i=1; i<BOXCOLOR; i++) {
+	    print_linestyle(spec, i, fp);
+	}
+    } else {
+	for (i=0; i<BOXCOLOR; i++) {
+	    print_linestyle(spec, i, fp);
+	}
+    }
+
+    fputs("set style increment user\n", fp);
+}
+
+static int print_point_type (GPT_LINE *line)
+{
+    return line->ptype != 0 && 
+	(line->style == GP_STYLE_POINTS ||
+	 line->style == GP_STYLE_LINESPOINTS);
+}
+
 #define show_fit(s) (s->fit == PLOT_FIT_OLS || \
                      s->fit == PLOT_FIT_QUADRATIC || \
                      s->fit == PLOT_FIT_INVERSE || \
@@ -375,16 +774,12 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
     int png = get_png_output(spec);
     int mono = (spec->flags & GPT_MONO);
     int started_data_lines = 0;
-    int n_lines = spec->n_lines;
     double et, yt;
-    double *x[4];
+    double *x[5];
+    int skipline = -1;
     int any_y2 = 0;
     int miss = 0;
 
-    if (spec->flags & GPT_FIT_HIDDEN) {
-	n_lines--;
-    }
-
     if (spec->pd > 0) {
 	fprintf(fp, "# timeseries %d", spec->pd);
 	if (spec->flags & GPT_LETTERBOX) {
@@ -395,7 +790,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
     }
 
     if (!mono && gnuplot_has_rgb()) {
-	write_plot_line_styles(spec->code, fp);
+	write_styles_from_plotspec(spec, fp);
     }
 
     if (!string_is_blank(spec->titles[0])) {
@@ -414,9 +809,9 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	fprintf(fp, "set y2label \"%s\"\n", spec->titles[3]);
     }
 
-    for (i=0; i<MAX_PLOT_LABELS; i++) {
+    for (i=0; i<spec->n_labels; i++) {
 	if (!string_is_blank(spec->labels[i].text)) {
-	    print_plot_labelspec(&(spec->labels[i]), fp);
+	    print_plot_labelspec(&spec->labels[i], fp);
 	}
     }
 
@@ -437,35 +832,55 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
     gnuplot_missval_string(fp);
 
-    if (strcmp(spec->keyspec, "none") == 0) {
+    if (spec->keyspec == GP_KEY_NONE) {
 	fputs("set nokey\n", fp);
     } else {
-	fprintf(fp, "set key %s\n", spec->keyspec);
+	fprintf(fp, "set key %s\n", gp_keypos_string(spec->keyspec));
     }
 
     print_plot_ranges_etc(spec, fp);
 
-    /* customized xtics? */
-    if (!string_is_blank(spec->xtics)) {
-	fprintf(fp, "set xtics %s\n", spec->xtics);
+    /* special x and/or y format? */
+    if (*spec->xfmt != '\0') {
+	fprintf(fp, "set format x \"%s\"\n", spec->xfmt);
     }
-    if (!string_is_blank(spec->mxtics)) {
-	fprintf(fp, "set mxtics %s\n", spec->mxtics);
+    if (*spec->yfmt != '\0') {
+	fprintf(fp, "set format y \"%s\"\n", spec->yfmt);
+    }
+
+    /* customized xtics? */
+    if (!strcmp(spec->xtics, "none")) {
+	fputs("set noxtics\n", fp);
+    } else {
+	if (!string_is_blank(spec->xtics)) {
+	    fprintf(fp, "set xtics %s\n", spec->xtics);
+	}
+	if (!string_is_blank(spec->mxtics)) {
+	    fprintf(fp, "set mxtics %s\n", spec->mxtics);
+	}
     }
 
     if (spec->flags & GPT_Y2AXIS) {
 	/* using two y axes */
 	fputs("set ytics nomirror\n", fp);
 	fputs("set y2tics\n", fp);
-    } else if (spec->flags & GPT_MINIMAL_BORDER) {
-	/* suppressing part of border */
-	fputs("set border 3\n", fp);
+    } else if (spec->border != GP_BORDER_DEFAULT) {
+	/* suppressing all or part of border */
+	if (spec->border == 0) {
+	    fputs("unset border\n", fp);
+	} else {
+	    fprintf(fp, "set border %d\n", spec->border);
+	}
 	if (string_is_blank(spec->xtics)) {
 	    fputs("set xtics nomirror\n", fp);
 	}
 	fputs("set ytics nomirror\n", fp);
     }
 
+    if (spec->bmargin > 0) {
+	fprintf(fp, "set bmargin %d\n", spec->bmargin);
+    }
+
     /* in case of plots that are editable (see gui client), it is
        important to write out the comment string that identifies the
        sort of graph, so that it will be recognized by type when
@@ -488,6 +903,8 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	print_auto_fit_string(spec->fit, fp);
     }
 
+    print_user_lines_info(spec, fp);
+
     if ((spec->code == PLOT_FREQ_SIMPLE ||
 	 spec->code == PLOT_FREQ_NORMAL ||
 	 spec->code == PLOT_FREQ_GAMMA) && gnuplot_has_style_fill()) {
@@ -500,82 +917,116 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	fputs("set style fill solid 0.4\n", fp);
     }
 
-    if (spec->flags & GPT_ALL_MARKERS) {
+    if (spec->flags & GPT_PRINT_MARKERS) {
 	print_data_labels(spec, fp);
     }
 
+    if (spec->flags & GPT_FIT_HIDDEN) {
+	skipline = 1;
+    }
+
     fputs("plot \\\n", fp);
 
-    for (i=0; i<n_lines; i++) {
+    for (i=0; i<spec->n_lines; i++) {
+	if (i == skipline || blank_user_line(spec, i)) {
+	    continue;
+	}
 	if ((spec->flags & GPT_Y2AXIS) && spec->lines[i].yaxis != 1) {
 	    any_y2 = 1;
 	    break;
 	}
     }
 
-    for (i=0; i<n_lines; i++) {
+    gretl_push_c_numeric_locale();
 
-	if (strcmp(spec->lines[i].scale, "NA")) {
-	    if (!strcmp(spec->lines[i].scale, "1.0")) {
-		fputs("'-' using 1", fp);
-		if (spec->lines[i].ncols == 2) {
-		    fputs(":($2)", fp);
+    for (i=0; i<spec->n_lines; i++) {
+	GPT_LINE *line = &spec->lines[i];
+
+	if (i == skipline || blank_user_line(spec, i)) {
+	    if (i < spec->n_lines - 1) {
+		continue;
+	    } else {
+		break;
+	    }
+	}
+
+	if (na(line->scale)) {
+	    fprintf(fp, "%s ", line->formula); 
+	} else if (line->scale == 1.0) {
+	    fputs("'-' using 1", fp);
+	    if (line->ncols == 5) {
+		/* Note: boxplot candlesticks, hard-wired! */
+		fputs(":3:2:5:4", fp);
+	    } else if (line->ncols == 2) {
+		if (line->flags & GP_LINE_BOXDATA) {
+		    /* boxplot median, hard-wired */
+		    fputs(":2:2:2:2", fp);
 		} else {
-		    for (k=2; k<=spec->lines[i].ncols; k++) {
-			fprintf(fp, ":%d", k);
-		    }
+		    fputs(":($2)", fp);
 		}
-		fputc(' ', fp);
 	    } else {
-		fprintf(fp, "'-' using 1:($2*%s) ", 
-			spec->lines[i].scale);
+		for (k=2; k<=line->ncols; k++) {
+		    fprintf(fp, ":%d", k);
+		}
 	    }
+	    fputc(' ', fp);
 	} else {
-	    fprintf(fp, "%s ", spec->lines[i].formula); 
+	    fprintf(fp, "'-' using 1:($2*%g) ", line->scale);
 	}
 
-	if ((spec->flags & GPT_Y2AXIS) && spec->lines[i].yaxis != 1) {
-	    fprintf(fp, "axes x1y%d ", spec->lines[i].yaxis);
+	if ((spec->flags & GPT_Y2AXIS) && line->yaxis != 1) {
+	    fprintf(fp, "axes x1y%d ", line->yaxis);
 	}
 
-	fprintf(fp, "title \"%s", spec->lines[i].title);
-
-	if (any_y2) {
-	    if (spec->lines[i].yaxis == 1) {
-		fprintf(fp, " (%s)\" ", G_("left"));
+	if (*line->title == '\0') {
+	    fputs("notitle ", fp);
+	} else {
+	    fprintf(fp, "title \"%s", line->title);
+	    if (any_y2) {
+		if (line->yaxis == 1) {
+		    fprintf(fp, " (%s)\" ", _("left"));
+		} else {
+		    fprintf(fp, " (%s)\" ", _("right"));
+		}
 	    } else {
-		fprintf(fp, " (%s)\" ", G_("right"));
+		fputs("\" ", fp);
 	    }
-	} else {
-	    fputs("\" ", fp);
 	}
 
-	fprintf(fp, "w %s", spec->lines[i].style);
+	fprintf(fp, "w %s", gp_line_style_string(line->style));
 
-	if (spec->lines[i].type != 0) {
-	    fprintf(fp, " lt %d", spec->lines[i].type);
-	}
-	if (spec->lines[i].width != 1) {
-	    fprintf(fp, " lw %d", spec->lines[i].width);
+	if (line->type != LT_NONE) {
+	    fprintf(fp, " lt %d", line->type);
 	}
 
-	if (i == n_lines - 1) {
-	    fputc('\n', fp);
-	} else {
-	    fputs(", \\\n", fp);
+	if (print_point_type(line)) {
+	    fprintf(fp, " pt %d", line->ptype);
 	}
+
+	if (line->width != 1) {
+	    fprintf(fp, " lw %d", line->width);
+	}
+
+	if (line->whiskwidth > 0) {
+	    fprintf(fp, " whiskerbars %g", line->whiskwidth);
+	}
+
+	if (more_lines(spec, i, skipline)) {
+	    fputs(", \\", fp);
+	} 
+
+	fputc('\n', fp);
     } 
 
     miss = 0;
 
     /* supply the data to gnuplot inline */
 
-    gretl_push_c_numeric_locale();
-
     for (i=0; i<spec->n_lines; i++) { 
-	int j;
+	int j, ncols = spec->lines[i].ncols;
 
-	if (spec->lines[i].ncols == 0) {
+	if (ncols == 0) {
+	    /* no data to print */
 	    continue;
 	}
 
@@ -588,6 +1039,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
 	x[2] = x[1] + spec->nobs;
 	x[3] = x[2] + spec->nobs;
+	x[4] = x[3] + spec->nobs;
 
 	for (t=0; t<spec->nobs; t++) {
 	    /* print x-axis value */
@@ -595,7 +1047,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 		fputs("? ", fp);
 		miss = 1;
 	    } else {
-		fprintf(fp, "%.8g ", x[0][t]);
+		fprintf(fp, "%.10g ", x[0][t]);
 	    }
 
 	    /* conversion, if needed, between (y, ydelta) and
@@ -620,12 +1072,12 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	    } 
 
 	    /* print y-axis value(s) */
-	    for (j=1; j<spec->lines[i].ncols; j++) {
+	    for (j=1; j<ncols; j++) {
 		if (na(x[j][t])) {
 		    fputs("? ", fp);
 		    miss = 1;
 		} else {
-		    fprintf(fp, "%.8g ",  x[j][t]);
+		    fprintf(fp, "%.10g ",  x[j][t]);
 		}
 	    }
 
@@ -638,7 +1090,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
 	fputs("e\n", fp);
 
-	x[1] += (spec->lines[i].ncols - 1) * spec->nobs;
+	x[1] += (ncols - 1) * spec->nobs;
     }
 
     gretl_pop_c_numeric_locale();
@@ -670,9 +1122,9 @@ static int set_loess_fit (GPT_SPEC *spec, int d, double q, gretl_matrix *x,
     spec->data = data;
     spec->nobs = spec->okobs = T;
 
-    sprintf(spec->lines[1].title, G_("loess fit, d = %d, q = %g"), d, q);
-    strcpy(spec->lines[1].scale, "1.0");
-    strcpy(spec->lines[1].style, "lines");
+    sprintf(spec->lines[1].title, _("loess fit, d = %d, q = %g"), d, q);
+    spec->lines[1].scale = 1.0;
+    spec->lines[1].style = GP_STYLE_LINES;
     spec->lines[1].ncols = 2;
 
     spec->fit = PLOT_FIT_LOESS;
@@ -710,8 +1162,8 @@ static void set_fitted_line (GPT_SPEC *spec, FitType f)
 	gretl_pop_c_numeric_locale();
     }
 
-    strcpy(spec->lines[1].scale, "NA");
-    strcpy(spec->lines[1].style, "lines");
+    spec->lines[1].scale = NADBL;
+    spec->lines[1].style = GP_STYLE_LINES;
     spec->lines[1].ncols = 0;
 
     spec->fit = f;
