@@ -22,10 +22,6 @@
 #define GRETL_DIGITS 6
 #define GRETL_MP_DIGITS 12
 
-#define PAGELINES 21
-
-/* functions follow */
- 
 void session_time (PRN *prn);
 
 void logo (void);
@@ -40,6 +36,8 @@ void text_print_model_confints (const CoeffIntervals *cf, PRN *prn);
 
 void print_freq (const FreqDist *freq, PRN *prn);
 
+void print_freq_test (const FreqDist *freq, PRN *prn);
+
 void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn);
 
 void print_smpl (const DATAINFO *pdinfo, 
@@ -51,6 +49,8 @@ print_contemp_covariance_matrix (const gretl_matrix *m,
 
 int outcovmx (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn);
 
+int max_obs_label_length (const DATAINFO *pdinfo);
+
 void obs_marker_init (const DATAINFO *pdinfo);
 
 void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn);
@@ -59,8 +59,6 @@ void varlist (const DATAINFO *pdinfo, PRN *prn);
 
 void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn);
 
-int get_printdata_blocks (void);
-
 int printdata (const int *list, const char *mstr,
 	       const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
@@ -69,6 +67,10 @@ int print_data_sorted (const int *list, const int *obsvec,
 		       const double **Z, const DATAINFO *pdinfo, 
 		       PRN *prn);
 
+int print_series_with_format (const int *list, const double **Z, 
+			      const DATAINFO *pdinfo, 
+			      char fmt, int digits, PRN *prn);
+
 int text_print_fit_resid (const FITRESID *fr, 
 			  const DATAINFO *pdinfo, 
 			  PRN *prn);
@@ -108,6 +110,8 @@ void bufgets_init (const char *buf);
 
 void bufgets_finalize (const char *buf);
 
-void scroll_pause (void);
+void buf_rewind (const char *buf);
 
-int scroll_pause_or_quit (void);
+int bufseek (const char *buf, long offset);
+
+long buftell (const char *buf);
