@@ -30,6 +30,45 @@
 
 #define PSDEBUG 0
 
+/* For the point of "x *= (1 + 0x1p-52)" below, see
+   http://sourceware.org/bugzilla/show_bug.cgi?id=4943 ,
+   contribution from Eric Postpischil, 2007-10-05.
+*/
+
+static void printf_series (int v, const double **Z,
+			   const DATAINFO *pdinfo,
+			   const char *fmt, 
+			   int wid, int prec, 
+			   int wstar, int pstar,
+			   PRN *prn)
+{
+    char label[OBSLEN];
+    double x;
+    int n, t;
+
+    n = max_obs_label_length(pdinfo) + 1;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	get_obs_string(label, t, pdinfo);
+	pprintf(prn, "%*s ", n, label);
+	x = Z[v][t];
+	if (na(x)) {
+	    pputc(prn, '\n');
+	    continue;
+	}
+	x *= (1 + 0x1p-52);
+	if (wstar && pstar) {
+	    pprintf(prn, fmt, wid, prec, x);
+	} else if (wstar || pstar) {
+	    wid = (wstar)? wid : prec;
+	    pprintf(prn, fmt, wid, x);
+	} else {
+	    pprintf(prn, fmt, x);
+	}
+	pputc(prn, '\n');
+    }
+}
+
 static int printf_escape (int c, PRN *prn)
 {
     switch (c) {
@@ -68,7 +107,7 @@ static char *printf_get_string (char *s, double ***pZ,
 
     /* special, not yet in genr */
 
-    if (!strncmp(s, "marker", 6) && pdinfo->S != NULL) {
+    if (!strncmp(s, "marker", 6) && pdinfo != NULL && pdinfo->S != NULL) {
 	/* observation label */
 	int offset = 0;
 
@@ -111,13 +150,13 @@ static double printf_get_scalar (char *s, double ***pZ,
 #endif
 
     if (numeric_string(s)) {
-	x = atof(s);
+	x = dot_atof(s);
     } else if (gretl_is_scalar(s)) {
 	x = gretl_scalar_get_value(s);
     } else {
-	v = series_index(pdinfo, s);
+	v = current_series_index(pdinfo, s);
 
-	if (v < pdinfo->v) {
+	if (v >= 0) {
 	    char genstr[32];
 
 	    sprintf(genstr, "%s[%d]", s, t + 1);
@@ -296,6 +335,7 @@ static int print_arg (char **pfmt, char **pargs,
     char *arg = NULL;
     char *str = NULL;
     gretl_matrix *m = NULL;
+    int series_v = -1;
     double x = NADBL;
     int flen = 0, alen = 0;
     int wstar = 0, pstar = 0;
@@ -353,10 +393,14 @@ static int print_arg (char **pfmt, char **pargs,
     /* get next substantive arg */
     arg = get_next_arg(*pargs, &alen, &err);
     if (!err) {
+	int v;
+
 	if (fc == 's') {
 	    str = printf_get_string(arg, pZ, pdinfo, t, &err);
 	} else if ((m = get_matrix_by_name(arg)) != NULL) {
 	    ; /* OK, we'll print the matrix */
+	} else if ((v = current_series_index(pdinfo, arg)) >= 0) {
+	    series_v = v; /* OK, we'll print the series */
 	} else {
 	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
 	    if (!err && na(x)) {
@@ -375,10 +419,17 @@ static int print_arg (char **pfmt, char **pargs,
     /* do the actual printing */
 
     if (m != NULL) {
+	/* printing a matrix */
 	if (!wstar) wid = -1;
 	if (!pstar) prec = -1;
 	gretl_matrix_print_with_format(m, fmt, wid, prec, prn);
+    } else if (series_v > 0) {
+	/* printing a series */
+	printf_series(series_v, (const double **) *pZ,
+		      pdinfo, fmt, wid, prec, 
+		      wstar, pstar, prn);
     } else if (fc == 's') {
+	/* printing a string */
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, str);
 	} else if (wstar || pstar) {
@@ -388,6 +439,7 @@ static int print_arg (char **pfmt, char **pargs,
 	    pprintf(prn, fmt, str);
 	}
     } else if (strchr(intconv, fc)) {
+	/* printing a scalar as int */
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, (int) x);
 	} else if (wstar || pstar) {
@@ -397,6 +449,8 @@ static int print_arg (char **pfmt, char **pargs,
 	    pprintf(prn, fmt, (int) x);
 	}
     } else {
+	/* printing a scalar value */
+	x *= (1 + 0x1p-52);
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, x);
 	} else if (wstar || pstar) {
@@ -516,7 +570,7 @@ static int real_do_printf (const char *line, double ***pZ,
     *targ = '\0';
 
     if (t < 0) {
-	t = pdinfo->t1;
+	t = (pdinfo != NULL)? pdinfo->t1 : 0;
     }
 
     err = split_printf_line(line, targ, &sp, &format, &args);
@@ -560,6 +614,11 @@ static int real_do_printf (const char *line, double ***pZ,
 		p++;
 	    }
 	}
+
+	if (q != NULL && *q != '\0') {
+	    pprintf(prn, "\nunmatched argument '%s'", q);
+	    err = E_PARSE;
+	}
     }
 
     if (sp) {
@@ -585,7 +644,7 @@ static int real_do_printf (const char *line, double ***pZ,
  * @pdinfo: dataset information.
  * @prn: printing struct.
  *
- * Implement a somewhat limited version of C's printf
+ * Implements a somewhat limited version of C's printf()
  * for use in gretl scripts.
  *
  * Returns: 0 on success, non-zero on error.
@@ -1113,7 +1172,7 @@ int n_scanned_items (void)
  * @pdinfo: dataset information.
  * @prn: printing struct.
  *
- * Implement a somewhat limited version of C's sscanf
+ * Implements a somewhat limited version of C's sscanf()
  * for use in gretl scripts.
  *
  * Returns: 0 on success, non-zero on error.
