@@ -441,7 +441,7 @@ double student_pvalue_2 (int df, double x)
  * probability @a, or #NADBL on failure.
  */
 
-static double student_critval (double df, double a)
+double student_critval (double df, double a)
 {
     double x;
 
@@ -699,88 +699,207 @@ static double Binv (double p, double q)
     return f;
 }    
 
+static int snedecor_pdf_array (int v1, int v2, double *x, int n)
+{
+    int i, err = 0;
+
+    errno = 0;
+
+    if (v1 > 0 && v2 > 0) {
+	double xm = v1, xn = v2;
+	double vr = xm / xn;
+	double x1 = Binv(0.5*xm, 0.5*xn);
+	double x2 = pow(vr, 0.5*xm);
+	double x3, x4;
+
+	if (errno) {
+	    err = E_NAN;
+	} else {
+	    for (i=0; i<n; i++) {
+		if (!na(x[i]) && x[i] > 0) {
+		    errno = 0;
+		    x3 = pow(x[i], 0.5*xm - 1.0);
+		    x4 = pow(1.0 + vr * x[i], 0.5 * (xm+xn));
+		    x[i] = x1 * x2 * x3 / x4;
+		    if (errno) {
+			x[i] = NADBL;
+		    }
+		} else {
+		    x[i] = NADBL;
+		}
+	    }
+	}
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
+    }
+
+    errno = 0;
+
+    return err;
+}
+
+
 static double snedecor_pdf (int m, int n, double x)
 {
-    double fx = NADBL;
+    snedecor_pdf_array(m, n, &x, 1);
 
-    errno = 0;
-
-    if (m > 0 && n > 0 && x > 0) {
-	double xm = m, xn = n;
-	double x1 = Binv(0.5*xm, 0.5*xn);
-	double x2 = pow(xm / xn, 0.5*xm);
-	double x3 = pow(x, 0.5*xm - 1.0);
-	double x4 = pow(1.0 + xm/xn * x, 0.5 * (xm+xn));
-
-	if (!errno && !na(x1)) {
-	    fx = x1 * x2 * x3 / x4;
-	}
-    }
-
-    return fx;
+    return x;
 }
 
-static double chisq_pdf (int m, double x)
+static int chisq_pdf_array (int m, double *x, int n)
 {
-    double fx = NADBL;
-
-    errno = 0;
-
-    if (m > 0 && x >= 0) {
-	double m2 = m / 2.0;
-	double x1 = pow(.5, m2);
-	double x2 = gamma_function(m2);
-	double x3 = pow(x, m2 - 1.0);
-	double x4 = exp(-x / 2.0);
-
-	if (!errno && !na(x2)) {
-	    fx = (x1/x2) * x3 * x4;
-	}
-    }
-
-    return fx;
-}
-
-static double student_pdf (double m, double x)
-{
-    double fx = NADBL;
+    int i, err = 0;
 
     errno = 0;
 
     if (m > 0) {
-	double x1 = Binv(0.5 * m, 0.5) / sqrt(m);
-	double x2 = m / (m + x * x);
-	double x3 = 0.5 * (m + 1.0);
+	double m2 = m / 2.0;
+	double x1 = pow(.5, m2);
+	double x2 = gamma_function(m2);
+	double x3, x4;
 
-	if (!errno && !na(x1)) {
-	    fx = x1 * pow(x2, x3);
-	    if (errno) {
-		fx = NADBL;
+	if (errno) {
+	    err = E_NAN;
+	} else {
+	    for (i=0; i<n; i++) {
+		if (!na(x[i]) && x[i] > 0) {
+		    errno = 0;
+		    x3 = pow(x[i], m2 - 1.0);
+		    x4 = exp(-x[i] / 2.0);
+		    x[i] = (x1/x2) * x3 * x4;
+		    if (errno) {
+			x[i] = NADBL;
+		    }
+		} else {
+		    x[i] = NADBL;
+		}
 	    }
 	}
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
     }
 
-    return fx;
+    errno = 0;
+
+    return err;
+}
+
+static double chisq_pdf (int m, double x)
+{
+    chisq_pdf_array(m, &x, 1);
+
+    return x;
+}
+
+static int student_pdf_array (double m, double *x, int n)
+{
+    int i, err = 0;
+
+    errno = 0;
+
+    if (m > 0 && !na(m)) {
+	double x1 = Binv(0.5 * m, 0.5) / sqrt(m);
+	double x3 = 0.5 * (m + 1.0);
+	double x2;
+
+	if (errno) {
+	    err = E_NAN;
+	} else {
+	    for (i=0; i<n; i++) {
+		if (!na(x[i])) {
+		    errno = 0;
+		    x2 = m / (m + x[i] * x[i]);
+		    x[i] = x1 * pow(x2, x3);
+		    if (errno) {
+			x[i] = NADBL;
+		    }
+		}
+	    }
+	}
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
+    }
+
+    errno = 0;
+
+    return err;
+}
+
+static double student_pdf (double m, double x)
+{
+    student_pdf_array(m, &x, 1);
+
+    return x;
+}
+
+static int weibull_pdf_array (double k, double l, 
+			      double *x, int n)
+{
+    int i, err = 0;
+
+    errno = 0;
+
+    if (!na(k) && k > 0 && !na(l) && l > 0) {
+	double x1 = k / l;
+	double x2, x3, x4;
+
+	if (errno) {
+	    err = E_NAN;
+	} else {
+	    for (i=0; i<n; i++) {
+		if (!na(x[i]) && x[i] >= 0) {
+		    errno = 0;
+		    x3 = x[i] / l;
+		    x2 = pow(x3, k - 1.0);
+		    x3 *= x2;
+		    x4 = exp(-x3);
+		    x[i] = x1 * x2 * x4;
+		    if (errno) {
+			x[i] = NADBL;
+		    }
+		} else {
+		    x[i] = NADBL;
+		}
+	    }
+	}
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
+    }
+
+    errno = 0;
+
+    return err;
 }
 
 static double weibull_pdf (double k, double l, double x)
 {
-    double fx = NADBL;
+    weibull_pdf_array(k, l, &x, 1);
 
-    errno = 0;
-
-    if (k > 0 && l > 0 && x >= 0) {
-	double x1 = k / l;
-	double x2 = pow(x / l, k - 1.0);
-	double x3 = pow(x / l, k);
-	double x4 = exp(-x3);
-
-	if (!errno) {
-	    fx = x1 * x2 * x4;
-	}
-    }
-
-    return fx;
+    return x;
 }
 
 /**
@@ -797,6 +916,9 @@ double normal_cdf (double x)
 
     if (get_cephes_errno()) {
 	y = NADBL;
+    } else if (y == 1.0) {
+	/* do we want to do this? */
+	y = 0.9999999999999999;
     }
 
     return y;
@@ -848,6 +970,26 @@ double normal_critval (double a)
     return z;
 }
 
+static int normal_pdf_array (double *x, int n)
+{
+    double s = 1.0 / sqrt(M_2PI);
+    int i;
+
+    for (i=0; i<n; i++) {
+	if (!na(x[i])) {
+	    errno = 0;
+	    x[i] = s * exp(-0.5 * x[i] * x[i]);
+	    if (errno) {
+		x[i] = NADBL;
+	    }
+	}
+    }
+
+    errno = 0;
+
+    return 0;
+}
+
 /**
  * normal_pdf:
  * @x: double-precision value.
@@ -1043,6 +1185,50 @@ double gamma_cdf_comp (double s1, double s2, double x, int control)
     return p;
 }
 
+static int gamma_pdf_array (double shape, double scale, 
+			    double *x, int n)
+{
+    int i, err = 0;
+
+    errno = 0;
+
+    if (!na(shape) && shape > 0 && !na(scale) && scale > 0) {
+	double x1, x2;
+	double x3 = pow(scale, shape);
+	double x4 = gamma_function(shape);
+
+	if (errno || na(x4)) {
+	    err = E_NAN;
+	} else {
+	    for (i=0; i<n; i++) {
+		if (!na(x[i]) && x[i] > 0) {
+		    errno = 0;
+		    x1 = pow(x[i], shape - 1.0);
+		    x2 = exp(-x[i]/scale);
+		    x[i] = x1 * x2 / (x3 * x4);
+		    if (errno) {
+			x[i] = NADBL;
+		    }
+		} else {
+		    x[i] = NADBL;
+		}
+	    }
+	}
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
+    }
+
+    errno = 0;
+
+    return err;
+}
+
 /**
  * gamma_pdf:
  * @shape: shape parameter.
@@ -1054,22 +1240,9 @@ double gamma_cdf_comp (double s1, double s2, double x, int control)
 
 double gamma_pdf (double shape, double scale, double x)
 {
-    double p = NADBL;
+    gamma_pdf_array(shape, scale, &x, 1);
 
-    errno = 0;
-
-    if (shape > 0 && scale > 0 && x > 0) {
-	double x1 = pow(x, shape - 1.0);
-	double x2 = exp(-x/scale);
-	double x3 = pow(scale, shape);
-	double x4 = gamma_function(shape);
-
-	if (!errno && !na(x4)) {
-	    p = x1 * x2 / (x3 * x4);
-	}
-    }
-
-    return p;
+    return x;
 }
 
 /**
@@ -1408,6 +1581,27 @@ double gretl_get_pdf (char st, double *p)
     return x;
 }
 
+int gretl_fill_pdf_array (char st, double *p, double *x, int n)
+{
+    int err = E_DATA;
+
+    if (st == 'z') {
+	err = normal_pdf_array(x, n);
+    } else if (st == 't') {
+	err = student_pdf_array(p[0], x, n);
+    } else if (st == 'X') {
+	err = chisq_pdf_array(p[0], x, n);
+    } else if (st == 'F') {
+	err = snedecor_pdf_array((int) p[0], (int) p[1], x, n);
+    } else if (st == 'G') {
+	err = gamma_pdf_array(p[0], p[1], x, n); 
+    } else if (st == 'W') {
+	err = weibull_pdf_array(p[0], p[1], x, n);
+    }
+
+    return err;
+}
+
 double gretl_get_pvalue (char st, const double *p)
 {
     double x = NADBL;
@@ -1702,6 +1896,23 @@ int batch_pvalue (const char *str,
     return err;
 }
 
+/**
+ * gretl_get_DW:
+ * @n: number of observations
+ * @k: number of regressors excluding the constant.
+ * @err: location to receive error code.
+ *
+ * Consults a table of Durbin-Watson critical values and
+ * returns the results in a gretl_matrix.  
+ *
+ * Returns: on success, a 4-vector containing the lower
+ * and upper Durbin-Watson values, dl and du, along with
+ * the values actually used for @n and @k (which may differ
+ * from those given on input if the exact values are not
+ * found in the table and have to be approximated).  
+ * On error, returns %NULL.
+ */
+
 gretl_matrix *gretl_get_DW (int n, int k, int *err)
 {
     void *handle;
