@@ -58,6 +58,8 @@ double normal_pdf (double x);
 
 double normal_critval (double a);
 
+double student_critval (double df, double a);
+
 double log_normal_pdf (double x);
 
 double bvnorm_cdf (double a, double b, double rho);
@@ -76,6 +78,8 @@ double gretl_get_pvalue (char st, const double *p);
 
 double gretl_get_pdf (char st, double *p);
 
+int gretl_fill_pdf_array (char st, double *p, double *x, int n);
+
 double gretl_get_cdf (char st, double *p);
 
 double gretl_get_cdf_inverse (char st, double *p);
