@@ -28,7 +28,6 @@
 #include "gretl_f2c.h"
 #include "clapack_double.h"
 
-#define QR_RCOND_MIN 1e-15 /* experiment with this? */
 #define ESSZERO      1e-22 /* SSR less than this counts as zero */
 
 enum {
@@ -45,7 +44,7 @@ enum {
 static double qr_get_tss (MODEL *pmod, const double *y, int *ifc,
 			  int *yconst)
 {
-    int pwe = gretl_model_get_int(pmod, "pwe");
+    int pwe = (pmod->opt & OPT_P);
     double y0 = 0.0, ymean = 0.0;
     double x, tss = 0.0;
     double ctss = 0.0;
@@ -132,6 +131,7 @@ static void qr_compute_stats (MODEL *pmod, const double *y, int n,
     int yconst, ifc = pmod->ifc;
 
     pmod->tss = qr_get_tss(pmod, y, &ifc, &yconst);
+    pmod->chisq = NADBL;
 
     if (yconst && pmod->dfd > 0) {
 	double y0 = y[pmod->t1];
@@ -158,7 +158,7 @@ static void qr_compute_stats (MODEL *pmod, const double *y, int n,
 
     if (pmod->dfd > 0 && pmod->dfn > 0) {
 	if (opt & OPT_R) {
-	    pmod->fstt = robust_omit_F(NULL, pmod);
+	    pmod->fstt = wald_omit_F(NULL, pmod);
 	} else if (pmod->rsq == 1.0) {
 	    pmod->fstt = NADBL;
 	} else {
@@ -212,8 +212,9 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     int t, i = 0;
     int dwt = gretl_model_get_int(pmod, "wt_dummy");
     int qdiff = (pmod->rho != 0.0);
-    int pwe = gretl_model_get_int(pmod, "pwe");
+    int pwe = (pmod->opt & OPT_P);
     int yvar = pmod->list[1];
+    double *u = pmod->uhat;
     double y;
 
     if (dwt) {
@@ -225,7 +226,7 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     if (qdiff) {
 	for (t=0; t<fulln; t++) {
 	    if (t < pmod->t1 || t > pmod->t2) {
-		pmod->yhat[t] = pmod->uhat[t] = NADBL;
+		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
 		y = Z[yvar][t];
 		if (t == pmod->t1 && pwe) {
@@ -234,15 +235,15 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		    y -= pmod->rho * Z[yvar][t-1];
 		}
 		pmod->yhat[t] = yhat->val[i];
-		pmod->uhat[t] = y - yhat->val[i];
-		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
+		u[t] = y - yhat->val[i];
+		pmod->ess += u[t] * u[t];
 		i++;
 	    }
 	}
     } else if (pmod->nwt) {
 	for (t=0; t<fulln; t++) {
 	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
-		pmod->yhat[t] = pmod->uhat[t] = NADBL;
+		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
 		y = Z[yvar][t];
 		if (dwt && Z[dwt][t] == 0.0) {
@@ -252,8 +253,8 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 			y *= sqrt(Z[pmod->nwt][t]);
 		    }
 		    pmod->yhat[t] = yhat->val[i];
-		    pmod->uhat[t] = y - yhat->val[i];
-		    pmod->ess += pmod->uhat[t] * pmod->uhat[t];
+		    u[t] = y - yhat->val[i];
+		    pmod->ess += u[t] * u[t];
 		    i++;
 		}
 	    }
@@ -261,11 +262,11 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
     } else {
 	for (t=0; t<fulln; t++) {
 	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
-		pmod->yhat[t] = pmod->uhat[t] = NADBL;
+		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
 		pmod->yhat[t] = yhat->val[i];
-		pmod->uhat[t] = Z[yvar][t] - yhat->val[i];
-		pmod->ess += pmod->uhat[t] * pmod->uhat[t];
+		u[t] = Z[yvar][t] - yhat->val[i];
+		pmod->ess += u[t] * u[t];
 		i++;
 	    }
 	}
@@ -540,12 +541,12 @@ static int qr_make_hac (MODEL *pmod, const double **Z, gretl_matrix *xpxinv)
 	p = get_hac_lag(T);
     }
 
-    gretl_model_set_int(pmod, "hac_kernel", kern);
-    gretl_model_set_int(pmod, "hac_prewhiten", prewhiten);
     if (kern == KERNEL_QS) {
-	gretl_model_set_double(pmod, "qs_bandwidth", bt);
+	gretl_model_set_full_vcv_info(pmod, VCV_HAC, kern, 0, 
+				      prewhiten, bt);
     } else {
-	gretl_model_set_int(pmod, "hac_lag", p);
+	gretl_model_set_full_vcv_info(pmod, VCV_HAC, kern, p, 
+				      prewhiten, NADBL);
     }
 
     gretl_matrix_zero(vcv);
@@ -659,10 +660,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     }  
 
     hc_version = libset_get_int(HC_VERSION);
-    gretl_model_set_int(pmod, "hc", 1);
-    if (hc_version > 0) {
-	gretl_model_set_int(pmod, "hc_version", hc_version);
-    }
+    gretl_model_set_vcv_info(pmod, VCV_HC, hc_version);
 
     if (hc_version == 1) {
 	for (t=0; t<T; t++) {
@@ -704,6 +702,28 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     return err;
 }
 
+static int qr_dw_stats (MODEL *pmod, const double **Z,
+			gretl_matrix *X, gretl_matrix *u)
+{
+    double DW, pv;
+    int t, s, err = 0;
+
+    get_data_X(X, pmod, Z);
+
+    for (s=0, t=pmod->t1; t<=pmod->t2; s++, t++) {
+	gretl_vector_set(u, s, pmod->uhat[t]);
+    }
+    
+    pv = dw_pval(u, X, &DW, &err);
+
+    if (!err) {
+	pmod->dw = DW;
+	gretl_model_set_double(pmod, "dw_pval", pv);
+    }
+    
+    return err;
+}
+
 static int qr_make_regular_vcv (MODEL *pmod, gretl_matrix *v,
 				gretlopt opt)
 {
@@ -715,12 +735,11 @@ static int qr_make_regular_vcv (MODEL *pmod, gretl_matrix *v,
 static void get_model_data (MODEL *pmod, const double **Z, 
 			    gretl_matrix *Q, gretl_matrix *y)
 {
-    int i, j, t;
-    double x;
     int dwt = gretl_model_get_int(pmod, "wt_dummy");
     int qdiff = (pmod->rho != 0.0);
-    int pwe = gretl_model_get_int(pmod, "pwe");
-    double pw1 = 0.0;
+    int pwe = (pmod->opt & OPT_P);
+    double x, pw1 = 0.0;
+    int i, j, t;
 
     if (pwe) {
 	pw1 = sqrt(1.0 - pmod->rho * pmod->rho);
@@ -797,13 +816,21 @@ allocate_model_arrays (MODEL *pmod, int k, int T)
     return 0;
 }
 
+#define RCOND_WARN 1.0e-07
+
 /* perform QR decomposition plus some additional tasks */
 
-static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank)
+static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank,
+			   int *warn)
 {
     integer k = gretl_matrix_rows(R);
+    double rcond = 0;
     int r, err;
 
+    if (warn != NULL) {
+	*warn = 0;
+    }
+
     /* basic decomposition */
     err = gretl_matrix_QR_decomp(Q, R);
     if (err) {
@@ -811,7 +838,7 @@ static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank)
     }
 
     /* check rank of QR */
-    r = gretl_check_QR_rank(R, &err);
+    r = gretl_check_QR_rank(R, &err, &rcond);
     if (err) {
 	return err;
     }
@@ -828,6 +855,8 @@ static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank)
 	if (info != 0) {
 	    fprintf(stderr, "dtrtri: info = %d\n", (int) info);
 	    err = 1;
+	} else if (rcond < RCOND_WARN && warn != NULL) {
+	    *warn = 1;
 	}
     }
 
@@ -848,7 +877,7 @@ drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
     int nd = R->rows - rank;
     int i, j;
 
-    if (!(opt & OPT_A)) {
+    if (1 || !(opt & OPT_A)) {
 	dlist = gretl_list_new(nd);
 	if (dlist != NULL) {
 	    dlist[0] = 0;
@@ -891,7 +920,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     gretl_matrix *Q = NULL, *y = NULL;
     gretl_matrix *R = NULL, *g = NULL, *b = NULL;
     gretl_matrix *V = NULL;
-    int rank, err = 0;
+    int rank, warn = 0, err = 0;
 
     T = pmod->nobs;               /* # of rows (observations) */
     k = pmod->list[0] - 1;        /* # of cols (variables) */
@@ -910,7 +939,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     }
 
     get_model_data(pmod, Z, Q, y);
-    err = QR_decomp_plus(Q, R, &rank);
+    err = QR_decomp_plus(Q, R, &rank, &warn);
 
     /* handling of (near-)perfect collinearity */
     if (err == E_SINGULAR && !(opt & OPT_Z)) {
@@ -920,7 +949,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
 	gretl_matrix_reuse(R, k, k);
 	gretl_matrix_reuse(V, k, k);
 	get_model_data(pmod, Z, Q, y);
-	err = QR_decomp_plus(Q, R, NULL);
+	err = QR_decomp_plus(Q, R, NULL, &warn);
 	if (!err) {
 	    maybe_shift_ldepvar(pmod, Z, pdinfo);
 	}
@@ -960,7 +989,8 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
 
     /* standard error of regression */
     if (T - k > 0) {
-	if (gretl_model_get_int(pmod, "no-df-corr")) {
+	if (pmod->opt & OPT_N) {
+	    /* no-df-corr */
 	    pmod->sigma = sqrt(pmod->ess / T);
 	} else {
 	    pmod->sigma = sqrt(pmod->ess / (T - k));
@@ -976,7 +1006,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
 
     /* VCV and standard errors */
     if (opt & OPT_R) { 
-	gretl_model_set_int(pmod, "robust", 1);
+	pmod->opt |= OPT_R;
 	if ((opt & OPT_T) && !libset_get_bool(FORCE_HC)) {
 	    qr_make_hac(pmod, Z, V);
 	} else {
@@ -989,6 +1019,16 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     /* get R^2, F */
     qr_compute_stats(pmod, Z[pmod->list[1]], T, opt);
 
+    /* D-W stat and p-value */
+    if ((opt & OPT_I) && pmod->missmask == NULL) {
+	qr_dw_stats(pmod, Z, Q, y);
+    }
+
+    /* near-singularity? */
+    if (warn) {
+	gretl_model_set_int(pmod, "near-singular", 1);
+    }
+
  qr_cleanup:
 
     gretl_matrix_free(Q);
@@ -1025,7 +1065,7 @@ int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 	goto qr_cleanup;
     }
 
-    err = QR_decomp_plus(Q, R, NULL);
+    err = QR_decomp_plus(Q, R, NULL, NULL);
     if (err) {
 	goto qr_cleanup;
     }
@@ -1044,10 +1084,10 @@ int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 	    }
 	} else if (dataset_is_time_series(pdinfo) && 
 		   !libset_get_bool(FORCE_HC)) {
-	    gretl_model_set_int(pmod, "robust", 1);
+	    pmod->opt |= OPT_R;
 	    err = qr_make_hac(pmod, Z, V);
 	} else {
-	    gretl_model_set_int(pmod, "robust", 1);
+	    pmod->opt |= OPT_R;
 	    err = qr_make_hccme(pmod, Z, Q, V);
 	}
     } else {
