@@ -170,7 +170,7 @@ char *delchar (int c, char *str)
  * @idx: the starting point for deleting characters.
  * @count: the number of characters to delete.
  *
- * Deletes @count characters from @str, starting at position @indx.
+ * Deletes @count characters from @str, starting at position @idx.
  *
  * Returns: the modified string.
  */
@@ -484,7 +484,7 @@ char *gretl_strndup (const char *src, size_t n)
 
 /**
  * gretl_strdup_printf:
- * @template: as in printf().
+ * @format: as in printf().
  * @Varargs: arguments to be printed.
  *
  * Print the arguments according to @format.
@@ -492,7 +492,7 @@ char *gretl_strndup (const char *src, size_t n)
  * Returns: allocated result of the printing, or %NULL on failure.
  */
 
-char *gretl_strdup_printf (const char *template, ...)
+char *gretl_strdup_printf (const char *format, ...)
 {
     va_list args;
     int plen, bsize = 2048;
@@ -505,8 +505,8 @@ char *gretl_strdup_printf (const char *template, ...)
 
     memset(buf, 0, 1);
 
-    va_start(args, template);
-    plen = vsnprintf(buf, bsize, template, args);
+    va_start(args, format);
+    plen = vsnprintf(buf, bsize, format, args);
     va_end(args);
 
     if (plen >= bsize) {
@@ -641,13 +641,26 @@ char *gretl_word_strdup (const char *src, const char **ptr)
 
 char *gretl_quoted_string_strdup (const char *s, const char **ptr)
 {
-    char quote, *ret = NULL;
+    char q, *ret = NULL;
     const char *p = NULL;
 
     if (s != NULL && (*s == '"' || *s == '\'')) {
-	quote = *s;
+	int gotit = 0;
+
+	q = *s;
 	s++;
-	p = strchr(s, quote);
+	p = s;
+	while (*p && !gotit) {
+	    if (*p == q && *(p-1) != '\\') {
+		/* found non-escaped matching quote */
+		gotit = 1;
+	    } else {
+		p++;
+	    }
+	}
+	if (!gotit) {
+	    p = NULL;
+	}
     }
 
     if (p == NULL) {
@@ -923,18 +936,20 @@ int get_base (char *targ, const char *src, char c)
 /**
  * top_n_tail:
  * @str: the string to process.
- * @err: location to receive error code, or %NULL
+ * @maxlen: maximum length of string, including NUL termination.
+ * @err: location to receive error code, or %NULL.
  *
  * Drop leading space and trailing space and newline from string,
  * then replace a trailing backslash (if any) with a space.
- * If @str does not end with a newline, and @err is not %NULL,
- * then %E_TOOLONG is written to @err.
+ * If @str does not end with a newline within the limit set by
+ * @maxlen, and @err is not %NULL, then %E_TOOLONG is written 
+ * to @err.
  * 
  * Returns: 1 if a trailing backslash or comma was found, 
  * otherwise 0.
  */
 
-int top_n_tail (char *str, int *err)
+int top_n_tail (char *str, size_t maxlen, int *err)
 {
     int i, n, cont = 0;
 
@@ -944,7 +959,7 @@ int top_n_tail (char *str, int *err)
 
     n = strlen(str) - 1;
 
-    if (err != NULL && str[n] != '\n') {
+    if (err != NULL && n > maxlen - 2 && str[n] != '\n') {
 	*err = E_TOOLONG;
     }
 
@@ -1247,7 +1262,7 @@ int strings_array_add (char ***pS, int *n, const char *p)
     } else {
 	Tmp[m] = NULL;
     }
-
+    
     *n += 1;
 
     return 0;
@@ -1292,6 +1307,78 @@ char **strings_array_new_with_length (int nstrs, int len)
     return S;
 }
 
+/**
+ * strings_array_realloc_with_length:
+ * @pS: existing array to reallocate.
+ * @oldn: original number of strings in the array.
+ * @newn: new number of strings in array.
+ * @len: number of bytes per string.
+ *
+ * Adjusts the storage in @pS to a size of @newn
+ * strings, each of them @len bytes long.  The first 
+ * byte of any additional strings is initialized to 0.
+ * This function may be used either to expand or to
+ * shrink an existing array of strings.
+ * 
+ * Returns: the new array, or %NULL on failure.
+ */
+
+char **strings_array_realloc_with_length (char ***pS, 
+					  int oldn, 
+					  int newn,
+					  int len)
+{
+    char **S;
+    int i, j;
+
+    if (pS == NULL) {
+	/* huh? */
+	return NULL;
+    }
+
+    if (newn == oldn) {
+	/* no-op */
+	return *pS;
+    }
+
+    if (newn <= 0) {
+	free_strings_array(*pS, oldn);
+	*pS = NULL;
+	return NULL;
+    }
+
+    /* in case we're shrinking the array */
+    for (i=newn; i<oldn; i++) {
+	free((*pS)[i]);
+	(*pS)[i] = NULL;
+    }
+
+    S = realloc(*pS, newn * sizeof *S);
+    if (S == NULL) {
+	free_strings_array(*pS, oldn);
+	*pS = NULL;
+	return NULL;
+    }
+
+    *pS = S;
+
+    /* in case we're expanding the array */
+    for (i=oldn; i<newn; i++) {
+	S[i] = malloc(len);
+	if (S[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(S[j]);
+	    }
+	    free(*pS);
+	    *pS = NULL;
+	    return NULL;
+	}
+	S[i][0] = '\0';
+    }
+
+    return *pS;
+}
+
 /**
  * strings_array_dup:
  * @strs: array of strings to be copied.
@@ -1717,7 +1804,7 @@ void unescape_url (char *url)
  * @v: the ID number for the new variable.
  * @pdinfo: dataset information.
  *
- * Given a tenative name for a new variable, check that it
+ * Given a tentative name for a new variable, check that it
  * is not a duplicate of an existing varname.  If it is,
  * modify the new name so that it becomes unique. The ID
  * number @v is required so that, if the variable has already
@@ -1949,6 +2036,10 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     int *list = NULL;
     int i, n = 0;
 
+    if (pdinfo == NULL) {
+	return NULL;
+    }
+
     pspec = g_pattern_spec_new(pattern);
 
     for (i=1; i<pdinfo->v; i++) { 
