@@ -71,7 +71,7 @@ char *gretl_strdup (const char *src);
 
 char *gretl_strndup (const char *src, size_t n);
 
-char *gretl_strdup_printf (const char *template, ...);
+char *gretl_strdup_printf (const char *format, ...);
 
 char *gretl_word_strdup (const char *src, const char **ptr);
 
@@ -99,7 +99,7 @@ int get_base (char *targ, const char *src, char c);
 
 int equation_get_lhs_and_rhs (const char *s, char **plh, char **prh);
 
-int top_n_tail (char *str, int *err);
+int top_n_tail (char *str, size_t maxlen, int *err);
 
 char *tailstrip (char *str);
 
@@ -111,6 +111,11 @@ char *safecpy (char *targ, const char *src, int n);
 
 char **strings_array_new (int nstrs);
 
+char **strings_array_realloc_with_length (char ***pS, 
+					  int oldn, 
+					  int newn,
+					  int len);
+
 int strings_array_add (char ***pS, int *n, const char *p);
 
 char **strings_array_new_with_length (int nstrs, int len);
