@@ -200,6 +200,11 @@ int submask_cmp (const char *m1, const char *m2)
     return 0;
 }
 
+/* all values apart from the sentinel are initialized to zero; once
+   the mask is used, 1s will indicate included observations and
+   0s will indicate excluded observations
+*/
+
 static char *make_submask (int n)
 {
     char *mask = calloc(n + 1, 1);
@@ -502,12 +507,14 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
 	    sample_range_get_extrema(pdinfo, &t1min, &t2max);
 	}
 
-	pdinfo->t1 = t1min;
-	pdinfo->t2 = t2max;
+	if (pdinfo->t1 != t1min || pdinfo->t2 != t2max) {
+	    pdinfo->t1 = t1min;
+	    pdinfo->t2 = t2max;
 #if SUBDEBUG
-	fprintf(stderr, "restore_full_sample: just set t1=%d and t2=%d\n",
-		t1min, t2max);
+	    fprintf(stderr, "restore_full_sample: just set t1=%d and t2=%d\n",
+		    t1min, t2max);
 #endif
+	}
 	return 0;
     }
 
@@ -651,7 +658,7 @@ static int copy_dummy_to_mask (char *mask, const double *x, int n)
 }
 
 static int mask_from_temp_dummy (const char *s, double ***pZ, DATAINFO *pdinfo, 
-				 char *mask)
+				 char *mask, PRN *prn)
 {
     char formula[MAXLEN];
     double *x;
@@ -660,7 +667,7 @@ static int mask_from_temp_dummy (const char *s, double ***pZ, DATAINFO *pdinfo,
     *formula = '\0';
     strncat(formula, s, MAXLEN - 1);
 
-    x = generate_series(formula, pZ, pdinfo, &err);
+    x = generate_series(formula, pZ, pdinfo, prn, &err);
 
     if (!err) {
 	err = copy_dummy_to_mask(mask, x, pdinfo->n);
@@ -739,24 +746,29 @@ count_panel_units (const char *mask, const DATAINFO *pdinfo)
     return n;
 }
 
-/* construct mask for taking random sub-sample from dataset */
+/* construct mask for taking random sub-sample from dataset:
+   we're selecting 'subn' cases without replacement, and
+   there may or may not be an existing mask in place that
+   has to be respected.
+*/
 
 static int make_random_mask (const char *s, const char *oldmask, 
 			     double ***pZ, DATAINFO *pdinfo,
 			     char *mask)
 {
     unsigned u;
-    int i, subn;
-    int cases = 0;
+    int targ, avail, oldn = pdinfo->n;
+    int i, subn, cases, rejn;
     int err = 0;
 
+    /* how many cases are requested? */
     subn = smpl_get_int(s, pZ, pdinfo, &err);
 
     if (subn <= 0 || subn >= pdinfo->n) {
 	err = 1;
     } else if (oldmask != NULL) {
-	int oldn = 0;
-
+	/* dataset is already sub-sampled */
+	oldn = 0;
 	for (i=0; i<pdinfo->n; i++) {
 	    if (oldmask[i]) {
 		oldn++;
@@ -772,17 +784,47 @@ static int make_random_mask (const char *s, const char *oldmask,
 	return err;
     }	
 
-    for (i=0; i<pdinfo->n; i++) {
-	mask[i] = 0;
+    /* Which is smaller: the number of cases to be selected or the
+       complement, the number to be discarded?  For the sake of
+       efficiency we'll go for the smaller value.
+    */
+    rejn = oldn - subn;
+    if (rejn < subn) {
+	/* select rejn observations to discard */
+	targ = rejn;
+	avail = 1;
+    } else {
+	/* select subn observations to include */
+	targ = subn;
+	avail = 0;
     }
 
-    for (i=0; cases != subn; i++) {
-	u = gretl_rand_int_max(pdinfo->n);
-	if (oldmask == NULL || oldmask[u]) {
-	    mask[u] = 1;
+    for (i=0; i<pdinfo->n; i++) {
+	if (oldmask != NULL && oldmask[i] == 0) {
+	    /* obs is already excluded, hence not selectable */
+	    mask[i] = -1;
+	} else {
+	    mask[i] = avail;
 	}
-	if (i >= subn - 1) {
-	    cases = count_selected_cases(mask, pdinfo);
+    }
+
+    cases = 0;
+
+    while (cases < targ) {
+	u = gretl_rand_int_max(pdinfo->n);
+	if (mask[u] == avail) {
+	    /* obs is available, not yet selected */
+	    mask[u] = !avail;
+	    cases++;
+	}
+    }
+
+    if (oldmask != NULL) {
+	/* undo the 'already excluded' coding above */
+	for (i=0; i<pdinfo->n; i++) {
+	    if (mask[i] == -1) {
+		mask[i] = 0;
+	    }
 	}
     }
 
@@ -999,7 +1041,8 @@ make_panel_submask (char *mask, const DATAINFO *pdinfo, int *err)
 static int 
 make_restriction_mask (int mode, const char *s, const int *list, 
 		       double ***pZ, DATAINFO *pdinfo,
-		       PRN *prn, const char *oldmask, char **pmask)
+		       const char *oldmask, char **pmask,
+		       PRN *prn)
 {
     char *mask = NULL;
     int sn = 0, err = 0;
@@ -1021,7 +1064,7 @@ make_restriction_mask (int mode, const char *s, const int *list,
     } else if (mode == SUBSAMPLE_USE_DUMMY) {
 	err = mask_from_dummy(s, (const double **) *pZ, pdinfo, mask);
     } else if (mode == SUBSAMPLE_BOOLEAN) {
-	err = mask_from_temp_dummy(s, pZ, pdinfo, mask);
+	err = mask_from_temp_dummy(s, pZ, pdinfo, mask, prn);
     } else {
 	strcpy(gretl_errmsg, _("Sub-sample command failed mysteriously"));
 	err = 1;
@@ -1173,6 +1216,72 @@ restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo)
     return err;
 }
 
+/* Below: we do this "precompute" thing if the dataset is already
+   subsampled and the user wants to compound the restriction with a
+   new one of the form "obs=x" or "obs!=x".  The "obs" references may
+   get out of whack if we restore the full dataset first, as we
+   usually do.  For example, say the spec is "obs!=50" to exclude
+   observation 50: presumably the user means to exclude the 50th
+   observation in the current, subsampled dataset, which may not be
+   the same as the 50th observation in the full dataset.
+*/
+
+static char *precompute_mask (const char *s, const char *oldmask,
+			      double ***pZ, DATAINFO *pdinfo, 
+			      PRN *prn, int *err)
+{
+    char *tmp = make_submask(pdinfo->n);
+    char *mask = NULL;
+    int i, t;
+
+#if SUBDEBUG
+    fprintf(stderr, "restrict_sample: precomputing new mask\n");
+#endif
+
+    if (tmp == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    /* fill out mask relative to current, restricted dataset */
+    *err = mask_from_temp_dummy(s, pZ, pdinfo, tmp, prn);
+
+    if (!*err) {
+	/* make blank full-length mask */
+	mask = make_submask(fullinfo->n);
+	if (mask == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	/* map the new mask onto full data range */
+	i = 0;
+	for (t=0; t<fullinfo->n; t++) {
+	    if (oldmask[t]) {
+		mask[t] = tmp[i++];
+	    }
+	}
+    }
+
+    free(tmp);
+
+    return mask;
+}
+
+#if 1 /* let's be conservative here */
+
+# define restriction_uses_obs(s) (strstr(s, "obs") != NULL)
+
+#else
+
+static int restriction_uses_obs (const char *s)
+{
+    return gretl_namechar_spn(s) == 3 && !strncmp(s, "obs", 3);
+}
+
+#endif
+
 /* restrict_sample: 
  * @line: command line (or %NULL).  
  * @pZ: pointer to original data array.  
@@ -1243,31 +1352,51 @@ int restrict_sample (const char *line, const int *list,
 	oldmask = state->submask;
     }
 
+    if (mode == SUBSAMPLE_BOOLEAN && fullinfo != NULL && 
+	oldmask != NULL && restriction_uses_obs(line)) {
+	mask = precompute_mask(line, oldmask, pZ, pdinfo, prn, &err);
+    }
+
     /* restore the full data range, for housekeeping purposes */
-    err = restore_full_sample(pZ, pdinfo, NULL);
+    if (!err) {
+	err = restore_full_sample(pZ, pdinfo, NULL);
+    }
+
     if (err) {
 	return err;
     }
 
-    if (!err) {
+    if (mask == NULL) {
+	/* not already handled by "precompute" above */
 	err = make_restriction_mask(mode, line, list, pZ, pdinfo, 
-				    prn, oldmask, &mask);
+				    oldmask, &mask, prn);
     }
 
     if (!err && mask != NULL) {
 	int t1 = 0, t2 = 0;
 	int contig = 0;
 
-	if (mode != SUBSAMPLE_RANDOM &&
-	    (dataset_is_time_series(pdinfo) ||
-	     dataset_is_panel(pdinfo))) {
+	if (mode != SUBSAMPLE_RANDOM) {
+	    /* AC 2009-04-11: this check was not being done
+	       for cross-sectional data.  Why not?
+	    */
 	    contig = mask_contiguous(mask, pdinfo, &t1, &t2);
 	}
 
 	if (contig) {
+	    /* The specified subsample consists of contiguous
+	       observations, so we'll just adjust the range, avoiding
+	       the overhead of creating a parallel dataset.
+	    */
+#if SUBDEBUG
+	    fprintf(stderr, "restrict sample: got contiguous range\n");
+#endif
 	    pdinfo->t1 = t1;
 	    pdinfo->t2 = t2;
 	} else {
+#if SUBDEBUG
+	    fprintf(stderr, "restrict sample: using mask\n");
+#endif
 	    err = restrict_sample_from_mask(mask, pZ, pdinfo);
 	}
     }
@@ -1360,7 +1489,8 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 {
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
     int tmin = 0, tmax = 0;
-    char newstart[OBSLEN], newstop[OBSLEN];
+    char newstart[VNAMELEN+1];
+    char newstop[VNAMELEN+1];
 
     gretl_error_clear();
 
@@ -1373,8 +1503,8 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
     fprintf(stderr, "set_sample: line='%s', nf=%d, pdinfo=%p\n", 
 	    line, nf, (void *) pdinfo);
     if (pdinfo != NULL) {
-	fprintf(stderr, "pdinfo->v = %d, pdinfo->n = %d\n",
-		pdinfo->v, pdinfo->n);
+	fprintf(stderr, "pdinfo->v = %d, pdinfo->n = %d, pd = %d\n",
+		pdinfo->v, pdinfo->n, pdinfo->pd);
     }
 #endif
 
@@ -1389,9 +1519,13 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
     }
 
     sample_range_get_extrema(pdinfo, &tmin, &tmax);
+
+#if SUBDEBUG
+    fprintf(stderr, "sample extrema: lo = %d, hi = %d\n", tmin, tmax);
+#endif
 	
     if (nf == 1) {
-	if (sscanf(line, "%10s", newstart) != 1) {
+	if (sscanf(line, "%16s", newstart) != 1) {
 	    strcpy(gretl_errmsg, _("error reading smpl line"));
 	    return 1;
 	} else {
@@ -1407,7 +1541,7 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 
     /* now we're looking at nf = 2 (2 fields) case */
 
-    if (sscanf(line, "%10s %10s", newstart, newstop) != 2) {
+    if (sscanf(line, "%16s %16s", newstart, newstop) != 2) {
 	strcpy(gretl_errmsg, _("error reading smpl line"));
 	return 1;
     }
@@ -1439,15 +1573,42 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
     return 0;
 }
 
-int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
+/**
+ * count_missing_values:
+ * @Z: data array.
+ * @pdinfo: dataset information.
+ * @opt: use %OPT_V for verbose operation.
+ * @prn: printing struct.
+ * @err: location to receive error code.
+ *
+ * Prints a count of missing values (if any) in the current
+ * dataset over the currently defined sample range. If
+ * %OPT_V is given this includes a count of missing values
+ * at each observation; otherwise it just includes global
+ * and per-variable counts.
+ *
+ * Returns: 0 if no missing values are found (or on error),
+ * otherwise the total number of missing values.
+ */
+
+int count_missing_values (const double **Z, const DATAINFO *pdinfo, 
+			  gretlopt opt, PRN *prn, int *err)
 {
-    int i, t, tmiss;
     int missval = 0, missobs = 0, totvals = 0, oldmiss = 0;
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
     int *missvec;
+    double missfrac;
+    int i, t, tmiss;
 
     missvec = malloc(pdinfo->v * sizeof missvec);
-    if (missvec != NULL) {
-	for (i=0; i<pdinfo->v; i++) missvec[i] = 0;
+
+    if (missvec == NULL) {
+	*err = E_ALLOC;
+	return 0;
+    }
+
+    for (i=0; i<pdinfo->v; i++) {
+	missvec[i] = 0;
     }
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
@@ -1456,7 +1617,7 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	    if (var_is_hidden(pdinfo, i)) {
 		continue;
 	    }
-	    if (na((*pZ)[i][t])) {
+	    if (na(Z[i][t])) {
 		if (missvec[i] == 0) {
 		    missvec[0] += 1;
 		}
@@ -1471,32 +1632,41 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	if (tmiss) {
 	    missobs++;
 
-	    if (pdinfo->markers) { 
-		pprintf(prn, "%8s %4d %s\n", pdinfo->S[t], tmiss,
-			_("missing values"));
-	    } else {
-		char tmp[OBSLEN];
+	    if (opt & OPT_V) {
+		/* verbose print by observation */
+		if (pdinfo->markers) { 
+		    pprintf(prn, "%8s %4d %s\n", pdinfo->S[t], tmiss,
+			    _("missing values"));
+		} else {
+		    char tmp[OBSLEN];
 
-		ntodate(tmp, t, pdinfo);
-		pprintf(prn, "%8s %4d %s\n", tmp, tmiss,
-			_("missing values"));
+		    ntodate(tmp, t, pdinfo);
+		    pprintf(prn, "%8s %4d %s\n", tmp, tmiss,
+			    _("missing values"));
+		}
 	    }
 	}
 	oldmiss = missval;
     }
 
+    missfrac = 100.0 * (double) missobs / T;
+
     pprintf(prn, _("\nNumber of observations (rows) with missing data "
-	    "values = %d (%.2f%%)\n"), missobs, 
-	    (100.0 * missobs / (pdinfo->t2 - pdinfo->t1 + 1)));
+		   "values = %d (%.2f%%)\n"), missobs, missfrac);
+
     pprintf(prn, _("Total number of missing data values = %d (%.2f%% "
 	    "of total data values)\n"), missval, 
-	    (100.0 * missval / totvals));
+	    (100.0 * (double) missval / totvals));
+
     if (missvec[0] > 0) {
 	pputc(prn, '\n');
 	for (i=1; i<pdinfo->v; i++) {
 	    if (missvec[i] > 0) {
-		pprintf(prn, "%8s: %d %s\n", pdinfo->varname[i], 
-			missvec[i], _("missing values"));
+		missfrac = 100.0 * (double) missvec[i] / T;
+		pprintf(prn, "%8s: %d %s (%.2f%%); %d %s (%.2f%%)\n", 
+			pdinfo->varname[i], missvec[i], _("missing values"),
+			missfrac, T - missvec[i], _("valid values"),
+			100.0 - missfrac);
 	    }
 	}
     }
@@ -1660,3 +1830,62 @@ void free_model_dataset (MODEL *pmod)
 	pmod->dataset = NULL;
     }
 }
+
+static int submask_match (const char *s1, const char *s2, int n)
+{
+    int t;
+
+    if (s1 == RESAMPLED || s2 == RESAMPLED) {
+	return s1 == RESAMPLED && s2 == RESAMPLED;
+    }
+
+    for (t=0; t<n; t++) {
+	if (s1[t] != s2[t]) return 0;
+    }
+
+    return 1;
+}
+
+/* check the subsample mask from a model (or modelspec) against 
+   datainfo to see if it may have been estimated on a different
+   (subsampled) data set from the current one
+*/
+
+int model_sample_problem (MODEL *pmod, const DATAINFO *pdinfo)
+{
+    int n = pdinfo->n;
+    int ret = 1;
+
+    if (pmod->submask == NULL) {
+	/* the model has no sub-sampling info recorded */
+	if (pdinfo->submask == NULL) {
+	    /* data set is not sub-sampled either, OK */
+	    ret = 0;
+	} else {
+	    fputs(I_("dataset is subsampled, model is not\n"), stderr);
+	    strcpy(gretl_errmsg, _("dataset is subsampled, model is not\n"));
+	    ret = 1;
+	}
+    } else {
+	/* the model does have sub-sampling info recorded */
+	if (pdinfo->submask == NULL) {
+	    fputs(I_("model is subsampled, dataset is not\n"), stderr);
+	    strcpy(gretl_errmsg, _("model is subsampled, dataset is not\n"));
+	    ret = 1;
+	} else if (submask_match(pdinfo->submask, pmod->submask, n)) {
+	    /* the subsamples (model and current data set) agree, OK */
+	    ret = 0;
+	} else {
+	    /* the subsamples differ */
+	    strcpy(gretl_errmsg, _("model and dataset subsamples not the same\n"));
+	    ret = 1;
+	}
+    }
+
+    if (ret == 0 && pmod->dataset != NULL) {
+	/* the model carries a now redundant auxiliary dataset */
+	free_model_dataset(pmod);
+    }
+
+    return ret;
+}
