@@ -71,7 +71,8 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state);
 
 int backup_full_dataset (double **Z, DATAINFO *pdinfo);
 
-int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn);
+int count_missing_values (const double **Z, const DATAINFO *pdinfo, 
+			  gretlopt opt, PRN *prn, int *err);
 
 int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo);
 
@@ -79,4 +80,6 @@ void free_model_dataset (MODEL *pmod);
 
 void maybe_free_full_dataset (const DATAINFO *pdinfo);
 
+int model_sample_problem (MODEL *pmod, const DATAINFO *pdinfo);
+
 #endif /* SUBSAMPLE_H */
