@@ -23,6 +23,7 @@
 #include "system.h"
 #include "objstack.h"
 #include "gretl_xml.h"
+#include "tsls.h"
 
 #define SYSDEBUG 0
 
@@ -43,6 +44,11 @@ enum {
     SYS_TEST_NOTIMP
 } system_test_types;
 
+enum {
+    SYS_UHAT,
+    SYS_YHAT
+};
+
 struct id_atom_ {
     int op;         /* operator (plus or minus) */
     int varnum;     /* ID number of variable to right of operator */
@@ -107,7 +113,8 @@ static void destroy_ident (identity *ident);
 static int 
 add_predet_to_sys (equation_system *sys, const DATAINFO *pdinfo,
 		   int id, int src, int lag);
-static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo);
+static int sys_check_lists (equation_system *sys, const double **Z, 
+			    const DATAINFO *pdinfo);
 
 static void 
 print_system_equation (const int *list, const DATAINFO *pdinfo, 
@@ -349,6 +356,8 @@ equation_system_new (int method, const char *name, int *err)
     sys->ilist = NULL;
     sys->xlist = NULL;
     sys->plist = NULL;
+    sys->biglist = NULL;
+
     sys->pre_vars = NULL;
     sys->idents = NULL;
 
@@ -478,6 +487,8 @@ void equation_system_destroy (equation_system *sys)
     free(sys->ilist);
     free(sys->xlist);
     free(sys->plist);
+    free(sys->biglist);
+
     free(sys->pre_vars);
 
     free(sys->name);
@@ -645,33 +656,10 @@ static int get_estimation_method_from_line (const char *s)
     return method;
 }
 
-static void 
-system_set_flags (equation_system *sys, const char *s, gretlopt opt)
-{
-    if (opt & OPT_T) {
-	sys->flags |= SYSTEM_ITERATE;
-    }
-
-    s = strstr(s, " save");
-
-    if (s != NULL) {
-	s += 5;
-	if (*s != ' ' && *s != '=') {
-	    return;
-	}
-	if (strstr(s, "resids") || strstr(s, "uhat")) {
-	    sys->flags |= SYSTEM_SAVE_UHAT;
-	}
-	if (strstr(s, "fitted") || strstr(s, "yhat")) {
-	    sys->flags |= SYSTEM_SAVE_YHAT;
-	}
-    }
-}
-
 /**
  * equation_system_start:
  * @line: command line.
- * @opt: may include %OPT_T for iterative estimation, if the
+ * @opt: may include %OPT_I for iterative estimation, if the
  * estimation method supports this.
  * 
  * Start compiling an equation system: @line must specify a "method" 
@@ -710,12 +698,17 @@ equation_system *equation_system_start (const char *line,
 	*err = E_DATA;
     }
 
+    if (strstr(line, "save=")) {
+	/* obsolete: e.g. "save=fitted" */
+	*err = E_PARSE;
+    }
+
     if (!*err) {
 	sys = equation_system_new(method, sysname, err);
     }
 
-    if (sys != NULL) {
-	system_set_flags(sys, line, opt);
+    if (sys != NULL && (opt & OPT_I)) {
+	sys->flags |= SYSTEM_ITERATE;
     }
 
     if (sysname != NULL) {
@@ -792,8 +785,8 @@ system_print_F_test (const equation_system *sys,
     F /= dfn;
 
     pprintf(prn, "%s:\n", _("F test for the specified restrictions"));
-    pprintf(prn, "  F(%d,%d) = %g %s %.4f\n", dfn, dfu, F,
-	    _("with p-value"), snedecor_cdf_comp(dfn, dfu, F));
+    pprintf(prn, "  F(%d,%d) = %g [%.4f]\n", dfn, dfu, F,
+	    snedecor_cdf_comp(dfn, dfu, F));
     pputc(prn, '\n');    
 
  bailout:
@@ -820,8 +813,8 @@ system_print_LR_test (const equation_system *sys,
     pprintf(prn, "%s:\n", _("LR test for the specified restrictions"));
     pprintf(prn, "  %s = %g\n", _("Restricted log-likelihood"), llr);
     pprintf(prn, "  %s = %g\n", _("Unrestricted log-likelihood"), llu);
-    pprintf(prn, "  %s(%d) = %g %s %.4f\n", _("Chi-square"),
-	    df, X2, _("with p-value"), chisq_cdf_comp(df, X2));
+    pprintf(prn, "  %s(%d) = %g [%.4f]\n", _("Chi-square"),
+	    df, X2, chisq_cdf_comp(df, X2));
     pputc(prn, '\n');    
 }
 
@@ -986,13 +979,11 @@ adjust_sys_flags_for_method (equation_system *sys, int method)
 static void 
 set_sys_flags_from_opt (equation_system *sys, gretlopt opt)
 {
-    char oldflags = sys->flags;
-
     sys->flags = 0;
 
     /* the iterate option is available for WLS, SUR or 3SLS */
 
-    if (opt & OPT_T) {
+    if (opt & OPT_I) {
 	if (sys->method == SYS_METHOD_WLS || 
 	    sys->method == SYS_METHOD_SUR ||
 	    sys->method == SYS_METHOD_3SLS) {
@@ -1015,12 +1006,9 @@ set_sys_flags_from_opt (equation_system *sys, gretlopt opt)
 	sys->flags |= SYSTEM_VCV_GEOMEAN;
     } 
 
-    if (oldflags & SYSTEM_SAVE_UHAT) {
-	sys->flags |= SYSTEM_SAVE_UHAT;
-    }
-
-    if (oldflags & SYSTEM_SAVE_YHAT) {
-	sys->flags |= SYSTEM_SAVE_YHAT;
+    if (opt & OPT_S) {
+	/* estimating single equation */
+	sys->flags |= SYSTEM_SINGLE;
     }
 }
 
@@ -1051,10 +1039,10 @@ equation_system_estimate (equation_system *sys,
 
     gretl_error_clear();
 
-    if (sys->xlist == NULL) {
+    if (sys->xlist == NULL || sys->biglist == NULL) {
 	/* allow for the possibility that we're looking at a
 	   system restored from a session file */
-	err = sys_check_lists(sys, pdinfo);
+	err = sys_check_lists(sys, (const double **) *pZ, pdinfo);
 	if (err) {
 	    return err;
 	}
@@ -1105,28 +1093,132 @@ equation_system_estimate (equation_system *sys,
 	close_plugin(handle);
     }
 
-    if (!err) {
+    if (!err && sys->neqns >= 2) {
 	set_as_last_model(sys, GRETL_OBJ_SYS);
     } 
 
     return err;
 }
 
+int system_adjust_t1t2 (equation_system *sys, const double **Z, 
+			const DATAINFO *pdinfo)
+{
+    int err = 0;
+
+    if (sys->biglist == NULL) {
+	fprintf(stderr, "system_adjust_t1t2: no 'biglist' present!\n");
+	return E_DATA;
+    }
+
+    sys->t1 = pdinfo->t1;
+    sys->t2 = pdinfo->t2;
+
+    err = check_for_missing_obs(sys->biglist, &sys->t1, &sys->t2, Z, NULL);
+
+    if (!err) {
+	sys->T = sys->t2 - sys->t1 + 1;
+    }
+
+    return err;
+}
+
+/* construct a list containing all the variables referenced in the
+   system */
+
+static int sys_make_biglist (equation_system *sys)
+{
+    int *biglist;
+    const int *slist;
+    const identity *ident;
+    int bign = 0;
+    int i, j, vj, k;
+
+    for (i=0; i<sys->neqns; i++) {
+	bign += sys->lists[i][0] - gretl_list_has_separator(sys->lists[i]);
+    }
+
+    for (i=0; i<sys->nidents; i++) {
+	bign += 1 + sys->idents[i]->n_atoms;
+    }   
+
+    if (sys->ilist != NULL) {
+	bign += sys->ilist[0];
+    }
+
+    biglist = gretl_list_new(bign);
+    if (biglist == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=bign; i++) {
+	/* invalidate all elements */
+	biglist[i] = -1;
+    }
+
+    k = 0;    
+
+    /* system equations */
+    for (i=0; i<sys->neqns; i++) {
+	slist = sys->lists[i];
+	for (j=1; j<=slist[0]; j++) {
+	    vj = slist[j];
+	    if (vj != LISTSEP && !in_gretl_list(biglist, vj)) {
+		biglist[++k] = vj;
+	    }
+	}
+    }
+
+    /* system identities */
+    for (i=0; i<sys->nidents; i++) {
+	ident = sys->idents[i];
+	for (j=0; j<=ident->n_atoms; j++) {
+	    if (j == 0) {
+		vj = ident->depvar;
+	    } else {
+		vj = ident->atoms[j-1].varnum;
+	    }
+	    if (!in_gretl_list(biglist, vj)) {
+		biglist[++k] = vj;
+	    }
+	}
+    }    
+
+    if (sys->ilist != NULL) {
+	/* system instruments */
+	for (j=1; j<=sys->ilist[0]; j++) {
+	    vj = sys->ilist[j];
+	    if (!in_gretl_list(biglist, vj)) {
+		biglist[++k] = vj;
+	    }
+	}
+    }
+
+    biglist[0] = k;
+
+#if SYSDEBUG
+    printlist(biglist, "system biglist (all vars)");
+#endif
+
+    free(sys->biglist);
+    sys->biglist = biglist;
+
+    return 0;
+}
+
 /* prior to system estimation, get all the required lists of variables
-   in order 
+   in order
 */
 
-static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
+static int sys_check_lists (equation_system *sys, const double **Z, 
+			    const DATAINFO *pdinfo)
 {
     const int *slist;
     const char *vname;
     const identity *ident;
     int user_ylist = (sys->ylist != NULL);
-    int *biglist = NULL;
     int *ylist = NULL;
     int *xplist = NULL;
-    int src, lag;
-    int nxp, bign = 0;
+    int src, lag, nxp;
     int i, j, k, vj;
     int err = 0;
 
@@ -1139,10 +1231,7 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
 
     /* Compose a minimal (and possibly incomplete) list of endogenous
        variables, including all vars that appear on the left-hand side
-       of structural equations or identities.  While we're at it, get
-       an upper bound to the grand total number of variables that
-       appear in the system ("bign" may be an overstatement due to
-       double-counting).
+       of structural equations or identities.
     */
 
     ylist = gretl_list_new(sys->neqns + sys->nidents);
@@ -1154,7 +1243,6 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
 
     for (i=0; i<sys->neqns; i++) {
 	slist = sys->lists[i];
-	bign += slist[0] - gretl_list_has_separator(slist);
 	vj = slist[1];
 	if (!in_gretl_list(ylist, vj)) {
 	    ylist[++k] = vj;
@@ -1163,7 +1251,6 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
 
     for (i=0; i<sys->nidents; i++) {
 	ident = sys->idents[i];
-	bign += 1 + ident->n_atoms;
 	vj = ident->depvar;
 	if (!in_gretl_list(ylist, vj)) {
 	    ylist[++k] = vj;
@@ -1200,83 +1287,42 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
     /* If the user did not give an endogenous list, use the one we
        computed above.
     */
-
     if (sys->ylist == NULL) {
 	sys->ylist = ylist;
 	ylist = NULL;
     }
 
-    /* Now, for reference, compose a big list of all the variables in
-       the system (referenced in structural equations or identities).
+    /* Now compose a big list of all the variables in the system
+       (referenced in structural equations or identities, or specified
+       as instruments).
     */
-
-    biglist = gretl_list_new(bign);
-    if (biglist == NULL) {
-	err = E_ALLOC;
+    err = sys_make_biglist(sys);
+    if (err) {
 	goto bailout;
-    }
-
-    for (i=1; i<=bign; i++) {
-	biglist[i] = -1;
-    }
-
-    k = 0;
-
-    for (i=0; i<sys->neqns; i++) {
-	slist = sys->lists[i];
-	for (j=1; j<=slist[0]; j++) {
-	    vj = slist[j];
-	    if (vj != LISTSEP && !in_gretl_list(biglist, vj)) {
-		biglist[++k] = vj;
-	    }
-	}
-    }
-
-    for (i=0; i<sys->nidents; i++) {
-	ident = sys->idents[i];
-	for (j=0; j<=ident->n_atoms; j++) {
-	    if (j == 0) {
-		vj = ident->depvar;
-	    } else {
-		vj = ident->atoms[j-1].varnum;
-	    }
-	    if (!in_gretl_list(biglist, vj)) {
-		biglist[++k] = vj;
-	    }
-	}
     }    
 
-    biglist[0] = k;
-
-#if SYSDEBUG
-    printlist(biglist, "system biglist (all vars)");
-#endif
-
     /* If the user gave an endogenous list, check that it doesn't
        contain any variables that are not actually present in the
        system.
      */
-
     if (user_ylist) {
 	for (j=1; j<=sys->ylist[0]; j++) {
 	    vj = sys->ylist[j];
-	    if (!in_gretl_list(biglist, vj)) {
+	    if (!in_gretl_list(sys->biglist, vj)) {
 		sprintf(gretl_errmsg, "%s is marked as endogenous but is "
 			"not present in the system", pdinfo->varname[vj]);
 		err = E_DATA;
+		goto bailout;
 	    }
 	}
     }
 
-    if (err) {
-	goto bailout;
-    }
 
     /* By elimination from biglist, using ylist, compose a maximal
        list of exogenous and predetermined variables, "xplist".
     */
 
-    nxp = biglist[0] - sys->ylist[0];
+    nxp = sys->biglist[0] - sys->ylist[0];
     xplist = gretl_list_new(nxp);
     if (xplist == NULL) {
 	err = E_ALLOC;
@@ -1284,8 +1330,8 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
     }
 
     k = 0;
-    for (j=1; j<=biglist[0]; j++) {
-	vj = biglist[j];
+    for (j=1; j<=sys->biglist[0]; j++) {
+	vj = sys->biglist[j];
 	if (!in_gretl_list(sys->ylist, vj)) {
 	    xplist[++k] = vj;
 	}
@@ -1298,7 +1344,6 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
     /* If the user gave a list of instruments, check that it does
        not contain anything that is in fact clearly endogenous.
      */
-
     if (sys->ilist != NULL) {
 	for (j=1; j<=sys->ilist[0]; j++) {
 	    vj = sys->ilist[j];
@@ -1313,7 +1358,6 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
     /* If the user did not give a list of instruments, use the
        one we computed above.
     */
-
     if (!err && sys->ilist == NULL) {
 	sys->ilist = gretl_list_copy(xplist);
 	if (sys->ilist == NULL) {
@@ -1325,7 +1369,6 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
        lags of endogenous variables (and recording them as such): this
        should leave a list of truly exogenous variables.
     */
-
     for (j=1; j<=xplist[0] && !err; j++) {
 	vj = xplist[j];
 	lag = pdinfo->varinfo[vj]->lag;
@@ -1343,12 +1386,14 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
     printlist(xplist, "final system exog list");
 #endif
 
-    sys->xlist = xplist;
-    xplist = NULL;
+    if (!err) {
+	free(sys->xlist);
+	sys->xlist = xplist;
+	xplist = NULL;
+    }
 
  bailout:
 
-    free(biglist);
     free(ylist);
     free(xplist);
 
@@ -1360,7 +1405,8 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
  * @sys: pre-defined equation system.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
- * @opt: may include %OPT_V for verbose operaton.
+ * @opt: may include %OPT_V for verbose operation, %OPT_S
+ * to permit estimation of a single equation.
  * @prn: printing struct.
  * 
  * Finalize an equation system, e.g. in response to "end system".
@@ -1376,6 +1422,7 @@ int equation_system_finalize (equation_system *sys,
 			      double ***pZ, DATAINFO *pdinfo,
 			      gretlopt opt, PRN *prn)
 {
+    int mineq = (opt & OPT_S)? 1 : 2;
     int err = 0;
 
     gretl_error_clear();
@@ -1385,7 +1432,7 @@ int equation_system_finalize (equation_system *sys,
 	return 1;
     }
 
-    if (sys->neqns < 2) {
+    if (sys->neqns < mineq) {
 	strcpy(gretl_errmsg, _(toofew));
 	equation_system_destroy(sys);
 	return 1;
@@ -1397,7 +1444,7 @@ int equation_system_finalize (equation_system *sys,
 	return 1;
     } 
 
-    err = sys_check_lists(sys, pdinfo);
+    err = sys_check_lists(sys, (const double **) *pZ, pdinfo);
 
     if (!err && sys->name != NULL && *sys->name != '\0') {
 	/* save the system for subsequent estimation */
@@ -1459,7 +1506,7 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
     return equation_system_estimate(sys, pZ, pdinfo, opt, prn);
 }
 
-/* effective list length, allowance made for TSLS-style lists */
+/* effective list length, allowance made for IVREG-style lists */
 
 static int get_real_list_length (const int *list)
 {
@@ -1534,25 +1581,7 @@ void equation_system_set_name (equation_system *sys, const char *name)
     }
 }
 
-int system_adjust_t1t2 (equation_system *sys,
-			int *t1, int *t2, const double **Z)
-{
-    int i, err = 0;
-
-    for (i=0; i<sys->neqns && !err; i++) {
-	err = check_for_missing_obs(sys->lists[i], t1, t2, Z, NULL);
-    }
-
-    if (!err) {
-	sys->t1 = *t1;
-	sys->t2 = *t2;
-	sys->T = *t2 - *t1 + 1;
-    }
-
-    return err;
-}
-
-int *compose_tsls_list (const equation_system *sys, int i)
+int *compose_ivreg_list (const equation_system *sys, int i)
 {
     int *list;
     int j, k1, k2;
@@ -1603,24 +1632,12 @@ add_system_var_info (equation_system *sys, int i,
 {
     char *label = VARLABEL(pdinfo, v);
 
-    if (code == SYSTEM_SAVE_UHAT) {
+    if (code == SYS_UHAT) {
 	sprintf(pdinfo->varname[v], "uhat_s%02d", i);
-	if (sys->method == SYS_METHOD_SUR) {
-	    sprintf(label, _("SUR residual, equation %d"), i);
-	} else if (sys->method == SYS_METHOD_3SLS) {
-	    sprintf(label, _("3SLS residual, equation %d"), i);
-	} else {
-	    sprintf(label, _("system residual, equation %d"), i);
-	}
-    } else if (code == SYSTEM_SAVE_YHAT) {
+	sprintf(label, _("system residual, equation %d"), i);
+    } else if (code == SYS_YHAT) {
 	sprintf(pdinfo->varname[v], "yhat_s%02d", i);
-	if (sys->method == SYS_METHOD_SUR) {
-	    sprintf(label, _("SUR fitted value, equation %d"), i);
-	} else if (sys->method == SYS_METHOD_3SLS) {
-	    sprintf(label, _("3SLS fitted value, equation %d"), i);
-	} else {
-	    sprintf(label, _("system fitted value, equation %d"), i);
-	}
+	sprintf(label, _("system fitted value, equation %d"), i);
     }
 }
 
@@ -1649,7 +1666,7 @@ system_add_resids_to_dataset (equation_system *sys,
 	}
     }
 
-    add_system_var_info(sys, eqnum + 1, pdinfo, v, SYSTEM_SAVE_UHAT);
+    add_system_var_info(sys, eqnum + 1, pdinfo, v, SYS_UHAT);
 
     return 0;
 }
@@ -1925,8 +1942,8 @@ parse_identity (const char *str, double ***pZ, DATAINFO *pdinfo, int *err)
 	    /* left-hand side variable */
 	    *err = extract_varname(vname, p, &len);
 	    if (!*err) {
-		v = series_index(pdinfo, vname);
-		if (v == pdinfo->v) {
+		v = current_series_index(pdinfo, vname);
+		if (v < 0) {
 		    *err = E_UNKVAR;
 		} else {
 		    p += len;
@@ -1954,8 +1971,8 @@ parse_identity (const char *str, double ***pZ, DATAINFO *pdinfo, int *err)
 		*err = E_PARSE;
 	    }
 	    if (!*err) {
-		v = series_index(pdinfo, vname);
-		if (v == pdinfo->v) {
+		v = current_series_index(pdinfo, vname);
+		if (v < 0) {
 		    *err = E_UNKVAR;
 		} else {
 		    p += len;
@@ -2336,17 +2353,27 @@ int highest_numbered_var_in_system (const equation_system *sys,
 {
     int i, j, v, vmax = 0;
 
-    for (i=0; i<sys->neqns; i++) {
-	for (j=1; j<=sys->lists[i][0]; j++) {
-	    v = sys->lists[i][j];
-	    if (v == LISTSEP || v >= pdinfo->v) {
-		/* temporary variables, already gone? */
-		continue;
-	    }
+    if (sys->biglist != NULL) {
+	for (j=1; j<=sys->biglist[0]; j++) {
+	    v = sys->biglist[j];
 	    if (v > vmax) {
 		vmax = v;
 	    }
 	}
+    } else {
+	/* should not happen */
+	for (i=0; i<sys->neqns; i++) {
+	    for (j=1; j<=sys->lists[i][0]; j++) {
+		v = sys->lists[i][j];
+		if (v == LISTSEP || v >= pdinfo->v) {
+		    /* temporary variables, already gone? */
+		    continue;
+		}
+		if (v > vmax) {
+		    vmax = v;
+		}
+	    }
+	}
     }
 
     return vmax;
@@ -2404,7 +2431,9 @@ sys_retrieve_identity (xmlNodePtr node, xmlDocPtr doc, int *err)
 }
 
 equation_system *
-equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
+equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			  const DATAINFO *pdinfo,
+			  int *err)
 {
     equation_system *sys;
     xmlNodePtr cur;
@@ -2603,14 +2632,12 @@ print_system_overid_test (const equation_system *sys, PRN *prn)
 		pprintf(prn, "  %s = %g", I_("Unrestricted log-likelihood"), sys->llu);
 	    }
 	    gretl_prn_newline(prn);
-	    pprintf(prn, "  $\\chi^2(%d)$ = %g %s %.4f\n", 
-		    df, X2, I_("with p-value"), pv);
+	    pprintf(prn, "  $\\chi^2(%d)$ = %g [%.4f]\n", df, X2, pv);
 	} else {
 	    pprintf(prn, "%s:\n", _("LR over-identification test"));
 	    pprintf(prn, "  %s = %g\n", _("Restricted log-likelihood"), sys->ll);
 	    pprintf(prn, "  %s = %g\n", _("Unrestricted log-likelihood"), sys->llu);
-	    pprintf(prn, "  %s(%d) = %g %s %.4f\n\n", _("Chi-square"),
-		    df, X2, _("with p-value"), pv);
+	    pprintf(prn, "  %s(%d) = %g [%.4f]\n\n", _("Chi-square"), df, X2, pv);
 	}
     } else if ((sys->method == SYS_METHOD_3SLS || 
 		sys->method == SYS_METHOD_SUR) && df > 0) {
@@ -2629,12 +2656,11 @@ print_system_overid_test (const equation_system *sys, PRN *prn)
 	if (tex) {
 	    pprintf(prn, "\\noindent %s:\\\\\n", 
 		    I_("Hansen--Sargan over-identification test"));
-	    pprintf(prn, "  $\\chi^2(%d)$ = %g %s %.4f\\\\\n", 
-		    df, sys->X2, I_("with p-value"), pv);
+	    pprintf(prn, "  $\\chi^2(%d)$ = %g [%.4f]\\\\\n", df, sys->X2, pv);
 	} else {
 	    pprintf(prn, "%s:\n", _("Hansen-Sargan over-identification test"));
-	    pprintf(prn, "  %s(%d) = %g %s %.4f\n\n", _("Chi-square"),
-		    df, sys->X2, _("with p-value"), pv);
+	    pprintf(prn, "  %s(%d) = %g [%.4f]\n\n", _("Chi-square"),
+		    df, sys->X2, pv);
 	}
     }
 }
@@ -2660,13 +2686,12 @@ int system_print_sigma (const equation_system *sys, PRN *prn)
 
 	    if (tex) {
 		pprintf(prn, "%s:\\\\\n", I_("LR test for diagonal covariance matrix"));
-		pprintf(prn, "  $\\chi^2(%d)$ = %g %s %.4f", 
-			df, lr, I_("with p-value"), x);
+		pprintf(prn, "  $\\chi^2(%d)$ = %g [%.4f]", df, lr, x);
 		gretl_prn_newline(prn);
 	    } else {
 		pprintf(prn, "%s:\n", _("LR test for diagonal covariance matrix"));
-		pprintf(prn, "  %s(%d) = %g %s %.4f\n", _("Chi-square"),
-		    df, lr, _("with p-value"), x);
+		pprintf(prn, "  %s(%d) = %g [%.4f]\n", _("Chi-square"),
+			df, lr, x);
 	    }
 	}
     } else {
@@ -2676,16 +2701,15 @@ int system_print_sigma (const equation_system *sys, PRN *prn)
 	    x = chisq_cdf_comp(df, lm);
 	    if (tex) {
 		pprintf(prn, "%s:", 
-			I_("Breusch--Pagan test for diagonal covariance matrix"));
+			_("Breusch--Pagan test for diagonal covariance matrix"));
 		gretl_prn_newline(prn);
-		pprintf(prn, "  $\\chi^2(%d)$ = %g %s %.4f", df, lm, 
-			I_("with p-value"), x);
+		pprintf(prn, "  $\\chi^2(%d)$ = %g [%.4f]", df, lm, x);
 		gretl_prn_newline(prn);
 	    } else {
 		pprintf(prn, "%s:\n", 
 			_("Breusch-Pagan test for diagonal covariance matrix"));
-		pprintf(prn, "  %s(%d) = %g %s %.4f\n", _("Chi-square"),
-			df, lm, _("with p-value"), x);
+		pprintf(prn, "  %s(%d) = %g [%.4f]\n", _("Chi-square"),
+			df, lm, x);
 	    }
 	}
     }
@@ -3441,65 +3465,6 @@ system_get_forecast_matrix (equation_system *sys, int t1, int t2,
     return sys->F;
 }
 
-/* respond to "save=..." in system command: add residuals and/or
-   fitted values to dataset as series
-*/
-
-static int sys_save_uhat_yhat (equation_system *sys, double ***pZ,
-			       DATAINFO *pdinfo)
-{
-    int v = pdinfo->v;
-    int addvars = 0;
-    int err;
-
-    if (sys->flags & SYSTEM_SAVE_UHAT) {
-	addvars += sys->neqns;
-    } 
-
-    if (sys->flags & SYSTEM_SAVE_YHAT) {
-	addvars += sys->neqns;
-    }
-
-    if (addvars == 0) {
-	return 0;
-    }
-
-    err = dataset_add_series(addvars, pZ, pdinfo);
-
-    if (!err) {
-	const MODEL *pmod;
-	int i, t;
-
-	for (i=0; i<sys->neqns; i++) {
-	    pmod = sys->models[i];
-
-	    if (sys->flags & SYSTEM_SAVE_UHAT) {
-		for (t=0; t<pdinfo->n; t++) {
-		    if (t < pmod->t1 || t > pmod->t2) {
-			(*pZ)[v][t] = NADBL;
-		    } else {
-			(*pZ)[v][t] = pmod->uhat[t];
-		    }
-		}
-		add_system_var_info(sys, i + 1, pdinfo, v++, SYSTEM_SAVE_UHAT);
-	    }
-
-	    if (sys->flags & SYSTEM_SAVE_YHAT) {
-		for (t=0; t<pdinfo->n; t++) {
-		    if (t < pmod->t1 || t > pmod->t2) {
-			(*pZ)[v][t] = NADBL;
-		    } else {
-			(*pZ)[v][t] = pmod->yhat[t];
-		    }
-		}
-		add_system_var_info(sys, i + 1, pdinfo, v++, SYSTEM_SAVE_YHAT);
-	    }
-	}
-    }
-    
-    return err;
-}
-
 /* attach to the system some extra statistics generated in the course
    of estimation via LIML */
 
@@ -3574,7 +3539,7 @@ static int sys_print_reconstituted_models (const equation_system *sys,
 	sys->method == SYS_METHOD_3SLS ||
 	sys->method == SYS_METHOD_FIML ||
 	sys->method == SYS_METHOD_LIML) {
-	mod.ci = TSLS;
+	mod.ci = IVREG;
     } else {
 	mod.ci = OLS;
     }
@@ -3589,7 +3554,7 @@ static int sys_print_reconstituted_models (const equation_system *sys,
 	mod.ID = i;
 
 	if (print_insts && !gretl_list_has_separator(mlist)) {
-	    mod.list = compose_tsls_list(sys, i);
+	    mod.list = compose_ivreg_list(sys, i);
 	    if (mod.list == NULL) {
 		err = E_ALLOC;
 	    } else {
@@ -3755,6 +3720,17 @@ int gretl_system_print (equation_system *sys,
     return 0;
 }
 
+static void ensure_asy_printout (equation_system *sys)
+{
+    int i;
+
+    if (sys->models != NULL) {
+	for (i=0; i<sys->neqns; i++) {
+	    sys->models[i]->ci = IVREG;
+	}
+    }
+}
+
 int 
 system_save_and_print_results (equation_system *sys,
 			       double ***pZ, DATAINFO *pdinfo,
@@ -3781,13 +3757,10 @@ system_save_and_print_results (equation_system *sys,
     } 
 
     if (!(opt & OPT_Q)) {
-	gretl_system_print(sys, (const double **) *pZ, pdinfo, opt, prn);
-    }
-
-    if (!err) {
-	if (sys->flags & (SYSTEM_SAVE_UHAT | SYSTEM_SAVE_YHAT)) {
-	    err = sys_save_uhat_yhat(sys, pZ, pdinfo);
+	if (sys->method == SYS_METHOD_FIML) {
+	    ensure_asy_printout(sys);
 	}
+	gretl_system_print(sys, (const double **) *pZ, pdinfo, opt, prn);
     }
 
     return err;
@@ -3804,10 +3777,9 @@ int system_autocorrelation_test (equation_system *sys, int order,
 	u = sys->E->val + (i * sys->T);
 	lb = ljung_box(order, 0, sys->T - 1, u, &err);
 	if (!err) {
-	    pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
-		    lb, _("with p-value"), _("Chi-square"), order,
+	    pprintf(prn, "%s: %s(%d) = %g [%.4f]\n\n", 
+		    _("Ljung-Box Q'"), _("Chi-square"), order,
 		    lb, chisq_cdf_comp(order, lb));
-	    pputc(prn, '\n');
 	}
     }
 
@@ -3827,3 +3799,70 @@ int system_arch_test (equation_system *sys, int order, PRN *prn)
 
     return err;
 }
+
+static void finalize_liml_model (MODEL *pmod, equation_system *sys)
+{
+    *pmod = *sys->models[0];
+
+#if SYSDEBUG
+    display_model_data_items(pmod);
+#endif
+
+    gretl_model_destroy_data_item(pmod, "tslsX");
+    gretl_model_destroy_data_item(pmod, "endog");
+    gretl_model_destroy_data_item(pmod, "method");
+    gretl_model_destroy_data_item(pmod, "liml_y");
+
+#if SYSDEBUG
+    display_model_data_items(pmod);
+#endif
+
+    free(sys->models[0]);
+    free(sys->models);
+    sys->models = NULL;
+
+    pmod->aux = AUX_NONE;
+    pmod->opt |= OPT_L;
+    pmod->rsq = pmod->adjrsq = NADBL;
+    pmod->fstt = NADBL;
+    set_model_id(pmod);
+}
+
+/* implement the --liml option to "tsls" */
+
+MODEL single_equation_liml (const int *list, double ***pZ,
+			    DATAINFO *pdinfo, gretlopt opt)
+{
+    int *mlist = NULL, *ilist = NULL;
+    equation_system *sys = NULL;
+    MODEL model;
+    int err = 0;
+
+    gretl_model_init(&model);
+
+    err = ivreg_process_lists(list, &mlist, &ilist);
+
+    if (!err) {
+	sys = equation_system_new(SYS_METHOD_LIML, NULL, &err);
+    }
+
+    if (!err) {
+	err = equation_system_append(sys, mlist);
+    }
+
+    if (!err) {
+	sys->ilist = ilist;
+	err = equation_system_finalize(sys, pZ, pdinfo, OPT_S, NULL);
+    }
+
+    if (err) {
+	model.errcode = err;
+    } else {
+	finalize_liml_model(&model, sys);
+    }
+
+    equation_system_destroy(sys);
+    free(mlist);
+
+    return model;
+}
