@@ -32,13 +32,13 @@ typedef enum {
 } GretlSystemMethods;
 
 enum {
-    SYSTEM_SAVE_UHAT   = 1 << 0,
-    SYSTEM_SAVE_YHAT   = 1 << 1,
+    /* allow for obsoleted flags */
     SYSTEM_DFCORR      = 1 << 2,
     SYSTEM_VCV_GEOMEAN = 1 << 3,
     SYSTEM_RESTRICT    = 1 << 4,
     SYSTEM_ITERATE     = 1 << 5,
-    SYSTEM_SAVEIT      = 1 << 6
+    SYSTEM_SAVEIT      = 1 << 6,
+    SYSTEM_SINGLE      = 1 << 7
 };
 
 enum {
@@ -77,6 +77,7 @@ struct equation_system_ {
     int *ilist;                 /* list of instruments */
     int *xlist;                 /* list of truly exogenous variables */
     int *plist;                 /* list of predetermined variables */
+    int *biglist;               /* list of all variables, for data checking */
     predet *pre_vars;           /* array of info on predetermined regressors */
     identity **idents;          /* set of identities */
     gretl_matrix *b;            /* coefficient estimates */
@@ -91,10 +92,7 @@ struct equation_system_ {
     gretl_matrix *A;            /* structural form A matrix (lagged endogenous) */
     gretl_matrix *F;            /* forecast matrix */
     gretl_matrix *Sr;           /* reduced-form error covariance matrix */
-    MODEL **models;             /* set of pointers to per-equation models: just
-				   convenience pointers -- these should NOT be
-				   freed as part of sys cleanup
-				*/
+    MODEL **models;             /* set of pointers to per-equation models */
     liml_data *ldata;           /* extra info from LIML estimation */
 };
 
@@ -133,14 +131,11 @@ int system_n_restrictions (const equation_system *sys);
 int system_max_indep_vars (const equation_system *sys);
 int system_n_indep_vars (const equation_system *sys);
 
-int system_adjust_t1t2 (equation_system *sys,
-			int *t1, int *t2, const double **Z);
-
 int *system_get_list (const equation_system *sys, int i);
 
 int system_get_list_length (const equation_system *sys, int i);
 
-int *compose_tsls_list (const equation_system *sys, int i);
+int *compose_ivreg_list (const equation_system *sys, int i);
 
 int system_get_depvar (const equation_system *sys, int i);
 
@@ -227,10 +222,15 @@ gretl_matrix *sys_get_fitted_values (equation_system *sys,
 				     const DATAINFO *pdinfo,
 				     int *err);
 
+int system_adjust_t1t2 (equation_system *sys, const double **Z, 
+			const DATAINFO *pdinfo);
+
 #ifndef GRETLCLI
 
 equation_system *
-equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
+equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			  const DATAINFO *pdinfo,
+			  int *err);
 
 #endif
 
@@ -244,4 +244,7 @@ int system_autocorrelation_test (equation_system *sys, int order,
 
 int system_arch_test (equation_system *sys, int order, PRN *prn);
 
+MODEL single_equation_liml (const int *list, double ***pZ,
+			    DATAINFO *pdinfo, gretlopt opt);
+
 #endif /* EQUATION_SYSTEM_H */
