@@ -28,6 +28,8 @@ static char colspec[4][8];
 static int use_custom;
 static int use_pdf;
 
+#define tex_screen_zero(x)  ((fabs(x) > 1.0e-17)? x : 0.0)
+
 int tex_using_custom_tabular (void)
 {
     return use_custom;
@@ -48,92 +50,31 @@ void set_tex_use_pdf (const char *prog)
 	strstr(prog, "PDF") != NULL;
 }
 
-#define tex_screen_zero(x)  ((fabs(x) > 1.0e-17)? x : 0.0)
-
-static void tex_modify_exponent (char *numstr)
+static const char *tex_greek_var (const char *s)
 {
-    char *p = strchr(numstr, 'e');
-
-    if (p != NULL) {
-	int expon = atoi(p + 1);
-
-	sprintf(p, "\\mbox{e%s%d}", (expon > 0)? "+" : "", expon);
-    }
-}
-
-/* prints a floating point number as a TeX math string.  if tab != 0,
-   print the sign in front, separated by a tab symbol (for
-   equation-style regression printout).
-*/
-
-static void tex_print_float (double x, PRN *prn)
-{
-    char number[48];
-
-    x = tex_screen_zero(x);
-
-    sprintf(number, "%#.*g", GRETL_DIGITS, x);
-    tex_modify_exponent(number);
-    pprintf(prn, "%s", (x < 0.0)? (number + 1) : number);
-}
-
-static void tex_print_signed_float (double x, PRN *prn)
-{
-    char number[48];
-
-    x = tex_screen_zero(x);
-
-    sprintf(number, "%#.*g", GRETL_DIGITS, x);
-    tex_modify_exponent(number);
-
-    if (x < 0.0) {
-	pprintf(prn, "$-$%s", number + 1);
-    } else {
-	pputs(prn, number);
-    }
-}
-
-static void tex_print_math_float (double x, PRN *prn)
-{
-    char number[48];
-
-    x = tex_screen_zero(x);
-
-    sprintf(number, "%#.*g", GRETL_DIGITS, x);
-    tex_modify_exponent(number);
-
-    pprintf(prn, "$%s$", number);
-}
-
-/**
- * tex_float_string:
- * @x: value to be printed.
- * @targ: target string.
- *
- * Prints the value @x into @targ with @prec digits of precision.
- * The result is intended for use in a non-math environment: if
- * @x is negative, the minus sign is put into math mode.
- * 
- * Returns: @targ.
- */
-
-char *tex_float_string (double x, int prec, char *targ)
-{
-    int offset = 0;
-
-    *targ = '\0';
-
-    x = tex_screen_zero(x);
-    if (x < 0.0) {
-	strcat(targ, "$-$");
-	x = fabs(x);
-	offset = 3;
+    if (!strcmp(s, "alpha")) {
+	return "\\alpha";
+    } else if (!strcmp(s, "beta")) {
+	return "\\beta";
+    } else if (!strcmp(s, "gamma")) {
+	return "\\gamma";
+    } else if (!strcmp(s, "delta")) {
+	return "\\delta";
+    } else if (!strcmp(s, "epsilon")) {
+	return "\\epsilon";
+    } else if (!strcmp(s, "chi")) {
+	return "\\chi";
+    } else if (!strcmp(s, "pi")) {
+	return "\\pi";
+    } else if (!strcmp(s, "phi")) {
+	return "\\phi";
+    } else if (!strcmp(s, "psi")) {
+	return "\\psi";
+    } else if (!strcmp(s, "lambda")) {
+	return "\\lambda";
     }
 
-    sprintf(targ + offset, "%#.*g", prec, x);
-    tex_modify_exponent(targ);
-
-    return targ;
+    return NULL;
 }
 
 /**
@@ -169,43 +110,277 @@ char *tex_escape (char *targ, const char *src)
     return p;
 }
 
-#define UPPER_F_LIMIT (pow(10, GRETL_DIGITS))
-#define LOWER_F_LIMIT (pow(10, -4))
-
-static void cut_extra_zero (char *numstr)
+static int tex_math_pname (char *targ, const char *s)
 {
-    int s = strspn(numstr, "-.,0");
-    int p = (s == 0 && (strchr(numstr, '.') || strchr(numstr, ',')));
+    char base[16];
+    char op[2];
+    char mod[8];
+    int n;
 
-    numstr[s + p + GRETL_DIGITS] = '\0';
+    n = sscanf(s, "%15[^_^]%1[_^]%7s", base, op, mod);
+
+    if (n == 3 && (*mod == '{' || isdigit(*mod))) {
+	const char *tgreek = tex_greek_var(base);
+	const char *tbase = (tgreek != NULL)? tgreek : base;
+
+	if (*mod == '{') {
+	    sprintf(targ, "$%s%s%s$", tbase, op, mod);
+	} else {
+	    sprintf(targ, "$%s%s{%s}$", tbase, op, mod);
+	}
+	return 1;
+    }
+
+    return 0;
 }
 
-void tex_dcolumn_double (double x, char *numstr)
+/**
+ * tex_escape_special:
+ * @targ: target string (must be pre-allocated)
+ * @src: source string.
+ *
+ * Copies from @src to @targ, escaping characters in @src that are 
+ * special to TeX (by inserting a leading backslash).  Unlike
+ * tex_escape(), this function does not mess with '$' in the
+ * source string, and it attempts to handle greek letters
+ * correctly.
+ * 
+ * Returns: the transformed copy of the string.
+ */
+
+char *tex_escape_special (char *targ, const char *src)
 {
-    double a;
+    const char *tgreek;
+    char *p = targ;
+
+    if (strchr(src, '$')) {
+	/* don't mess with it */
+	strcpy(targ, src);
+	return targ;
+    }
+
+    tgreek = tex_greek_var(src);
+
+    if (tgreek != NULL) {
+	sprintf(targ, "$%s$", tgreek);
+    } else if (tex_math_pname(targ, src)) {
+	; /* handled */
+    } else {
+	/* regular escape routine */
+	while (*src) {
+	    if (*src == '&' || *src == '_' || 
+		*src == '%' || *src == '#') {
+		*p++ = '\\';
+	    }
+	    *p++ = *src++;
+	}
+
+	*p = '\0';
+    }
+
+    return targ;
+}
+
+/* Print the floating point number @x into the string @s, using the C
+   format "%*.f", with the digits following the decimal point given by
+   @dig.  This is intended for use with the LaTeX tabular column
+   format "r@{.}l", so the decimal point is replaced by '&'.  In
+   addition, it is presumed that we're _not_ in TeX math mode, so the
+   leading minus sign, if present, is "mathized" as "$-$".
+
+   NADBL is handled by printing a blank "r@{.}l" value.
+*/
+
+char *tex_rl_float (double x, char *s, int dig)
+{
+    char *p;
 
     if (na(x)) {
-	strcpy(numstr, "\\multicolumn{1}{c}{}");
-	return;
+	return strcpy(s, "\\multicolumn{2}{c}{}");
     }
 
     x = screen_zero(x);
-    a = fabs(x);
 
-    sprintf(numstr, "%#.*g", GRETL_DIGITS, x);
-
-    if (a != 0.0 && (a >= UPPER_F_LIMIT || a < LOWER_F_LIMIT)) {
-	int expon;
-	char *p, exponstr[8];
-
-	p = strchr(numstr, 'e');
-	expon = atoi(p + 2);
-	strcpy(p, "\\mbox{e");
-	sprintf(exponstr, "%s%02d}", (x > 10)? "+" : "-", expon);
-	strcat(numstr, exponstr);
+    if (x < 0) {
+	sprintf(s, "$-$%.*f", dig, -x);
     } else {
-	cut_extra_zero(numstr);
+	sprintf(s, "%.*f", dig, x);
     }
+
+    p = strchr(s, '.');
+
+    if (p == NULL) {
+	p = strchr(s, ',');
+    }
+
+    if (p != NULL) {
+	*p = '&';
+    } else {
+	strcat(s, "&");
+    }
+
+    return s;
+}
+
+/* When a floating point value is printed as TeX in the C format "%g",
+   process the exponent part.  It is assumed we're not in TeX math
+   mode, so in a negative exponent such as "e-06" the minus will
+   appear as a hyphen, which does not look very good.  Using a true
+   minus sign in the exponent doesn't look very good either (too
+   wide); we compromise by writing the minus as an en dash.
+*/
+
+char *tex_modify_exponent (char *s)
+{
+    char *p = strchr(s, 'e');
+
+    if (p != NULL) {
+	int minus = (*(p+1) == '-');
+	char tmp[16];
+
+	sprintf(tmp, "\\textrm{e%s%s}", (minus)? "--" : "+", p + 2);
+	strcpy(p, tmp);
+    }
+    
+    return s;
+}
+
+/* Print the floating point number @x into the string @s, using the C
+   format "%#*.g", with GRETL_DIGITS of precision.  This is intended
+   for use with the LaTeX tabular column format "r@{.}l", so the
+   decimal point is replaced by '&'.  In addition, it is presumed that
+   we're _not_ in TeX math mode, so the leading minus sign, if
+   present, is "mathized" as "$-$".
+
+   NADBL is handled by printing a blank "r@{.}l" value.
+*/
+
+char *tex_rl_double (double x, char *s)
+{
+    char *p;
+
+    if (na(x)) {
+	return strcpy(s, "\\multicolumn{2}{c}{}");
+    }
+
+    x = screen_zero(x);
+
+    if (x < 0) {
+	sprintf(s, "$-$%#.*g", GRETL_DIGITS, -x);
+    } else {
+	sprintf(s, "%#.*g", GRETL_DIGITS, x);
+    }
+
+    if (strchr(s, 'e') != NULL) {
+	tex_modify_exponent(s);
+    }
+
+    p = strchr(s, '.');
+
+    if (p == NULL) {
+	p = strchr(s, ',');
+    }
+
+    if (p != NULL) {
+	*p = '&';
+    } else {
+	strcat(s, "&");
+    }
+
+    return s;
+}
+
+/* Print the floating point number @x into the string @s, using the C
+   format "%#*.g", with @dig digits of precision.  It is presumed
+   that we're _not_ in TeX math mode, so the leading minus sign, if
+   present, is "mathized" as "$-$".
+
+   NADBL is handled by printing a space.
+*/
+
+char *tex_sprint_double_digits (double x, char *s, int dig)
+{
+    if (na(x)) {
+	return strcpy(s, " ");
+    }
+
+    x = screen_zero(x);
+
+    if (x < 0.0) {
+	sprintf(s, "$-$%#.*g", dig, -x);
+    } else {
+	sprintf(s, "%#.*g", dig, x);
+    }
+
+    if (strchr(s, 'e') != NULL) {
+	tex_modify_exponent(s);
+    }
+
+    return s;
+}
+
+/* Basically as above, but for use when in TeX math mode */
+
+static char *tex_sprint_math_double_digits (double x, char *s, int dig)
+{
+    if (na(x)) {
+	return strcpy(s, "\\mbox{NA}");
+    }
+
+    x = screen_zero(x);
+
+    sprintf(s, "%#.*g", dig, x);
+
+    if (strchr(s, 'e') != NULL) {
+	tex_modify_exponent(s);
+    }
+
+    return s;
+}
+
+/* Print the floating point number @x into the string @s, using the C
+   format "%#*.g", with GRETL_DIGITS of precision.  It is presumed
+   that we're _not_ in TeX math mode, so the leading minus sign, if
+   present, is "mathized" as "$-$".
+
+   NADBL is handled by printing a space.
+*/
+
+char *tex_sprint_double (double x, char *s)
+{
+    if (na(x)) {
+	return strcpy(s, " ");
+    }
+
+    x = screen_zero(x);
+
+    if (x < 0.0) {
+	sprintf(s, "$-$%#.*g", GRETL_DIGITS, -x);
+    } else {
+	sprintf(s, "%#.*g", GRETL_DIGITS, x);
+    }
+
+    if (strchr(s, 'e') != NULL) {
+	tex_modify_exponent(s);
+    }
+
+    return s;
+}
+
+/* Print the floating point number @x directly to @prn, using the C
+   format "%#*.g", with GRETL_DIGITS of precision.  It is presumed
+   that we're _not_ in TeX math mode, so the leading minus sign, if
+   present, is "mathized" as "$-$".
+
+   NADBL is handled by printing a space.
+*/
+
+void tex_print_double (double x, PRN *prn)
+{
+    char s[32];
+
+    tex_sprint_double(x, s);
+    pputs(prn, s);
 }
 
 static void tex_make_cname (char *cname, const char *src)
@@ -234,45 +409,14 @@ static void tex_make_cname (char *cname, const char *src)
 
 static int tex_greek_param (char *targ, const char *src)
 {
-    *targ = 0;
+    const char *tgreek = tex_greek_var(src);
 
-    if (!strcmp(src, "alpha")) {
-	strcpy(targ, "$\\alpha$");
-    } else if (!strcmp(src, "beta")) {
-	strcpy(targ, "$\\beta$");
-    } else if (!strcmp(src, "gamma")) {
-	strcpy(targ, "$\\gamma$");
-    } else if (!strcmp(src, "delta")) {
-	strcpy(targ, "$\\delta$");
-    } else if (!strcmp(src, "pi")) {
-	strcpy(targ, "$\\pi$");
-    } else if (!strcmp(src, "lambda")) {
-	strcpy(targ, "$\\lambda$");
-    }
-
-    return (*targ != 0);
-}
-
-static void tex_arbond_coeff_name (char *targ, const char *src,
-				   int inmath)
-{
-    char vname[VNAMELEN], vnesc[16];
-    int lag;
-
-    if (sscanf(src, "D%10[^(](%d)", vname, &lag) == 2) {
-	tex_escape(vnesc, vname);
-	if (!inmath) {
-	    sprintf(targ, "$\\Delta \\mbox{\\rm %s}_{%d}$", vnesc, lag);
-	} else {
-	    sprintf(targ, "\\Delta \\mbox{\\rm %s}_{%d}", vnesc, lag);
-	}
+    if (tgreek != NULL) {
+	sprintf(targ, "$%s$", tgreek);
+	return 1;
     } else {
-	tex_escape(vnesc, src);
-	if (inmath) {
-	    sprintf(targ, "\\mbox{%s}", vnesc);
-	} else {
-	    strcpy(targ, vnesc);
-	}
+	*targ = '\0';
+	return 0;
     }
 }
 
@@ -478,77 +622,101 @@ static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
 void make_tex_coeff_name (const MODEL *pmod, const DATAINFO *pdinfo, int i,
 			  char *name)
 {
-    int j = i + 2;
+    char pname[VNAMELEN];
+
+    gretl_model_get_param_name(pmod, pdinfo, i, pname);    
 
     if (pmod->aux == AUX_ARCH) {
-	tex_make_cname(name, pdinfo->varname[pmod->list[j]]);
+	tex_make_cname(name, pname);
     } else if (pmod->ci == NLS) {
-	if (!tex_greek_param(name, pmod->params[i])) {
-	    tex_escape(name, pmod->params[i]);
+	if (!tex_greek_param(name, pname)) {
+	    tex_escape(name, pname);
 	}
     } else if (pmod->ci == ARMA) {
-	tex_arma_coeff_name(name, pmod->params[i], 0);
+	tex_arma_coeff_name(name, pname, 0);
     } else if (pmod->ci == GARCH) {
-	tex_garch_coeff_name(name, pmod->params[i], 0);
+	tex_garch_coeff_name(name, pname, 0);
     } else if (pmod->ci == VAR) {
 	tex_VAR_varname(name, pmod, pdinfo, i);
     } else if (pmod->aux == AUX_VECM) {
 	tex_VECM_varname(name, pmod, pdinfo, i);
-    } else if (pmod->ci == MPOLS && pmod->params != NULL) {
-	tex_mp_coeff_name(name, pmod->params[i], 0);
-    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT || pmod->ci == HECKIT) &&
-	       pmod->params != NULL) {
-	tex_escape(name, pmod->params[i]);
-    } else if (pmod->ci == PANEL || pmod->ci == MLE || pmod->ci == GMM) {
-	tex_escape(name, pmod->params[i]);
-    } else if (pmod->ci == ARBOND) {
-	tex_arbond_coeff_name(name, pmod->params[i], 0);
+    } else if (pmod->ci == MPOLS) {
+	tex_mp_coeff_name(name, pname, 0);
     } else {
-	tex_escape(name, pdinfo->varname[pmod->list[j]]);
+	tex_escape(name, pname);
     }
 }
 
+static char *tex_multi_double (double x, char *numstr)
+{
+    char *p;
+
+    if (na(x)) {
+	strcpy(numstr, " ");
+    } else if (x < 0) {
+	sprintf(numstr, "$-$%.15E", -x);
+    } else {
+	sprintf(numstr, "%.15E", x);
+    }
+
+    if ((p = strstr(numstr, "E-")) != NULL) {
+	char tmp[8];
+
+	sprintf(tmp, "E--%s", p + 2);
+	strcpy(p, tmp);
+    }
+
+    return numstr;
+}
+
 void tex_print_coeff (const model_coeff *mc, PRN *prn)
 {
     char col1[64], col2[64], col3[64], col4[64];
     int ncols = 4;
 
+    if (mc->multi) {
+	tex_multi_double(mc->b, col1);
+	tex_multi_double(mc->se, col2);
+	pprintf(prn, "%s & %s & %s \\\\\n", mc->name, col1, col2);
+	return;
+    }
+
     if (use_custom) {
 	tex_print_coeff_custom(mc, prn);
 	return;
     }
 
     if (na(mc->b)) {
-	sprintf(col1, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	sprintf(col1, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
     } else {
-	tex_dcolumn_double(mc->b, col1);
+	tex_rl_double(mc->b, col1);
     }
 
     if (!na(mc->lo) && !na(mc->hi)) {
-	tex_dcolumn_double(mc->lo, col2);
-	tex_dcolumn_double(mc->hi, col3);
+	tex_rl_double(mc->lo, col2);
+	tex_rl_double(mc->hi, col3);
 	ncols = 3;
     } else {
 	if (na(mc->se)) {
-	    sprintf(col2, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	    sprintf(col2, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
 	} else {
-	    tex_dcolumn_double(mc->se, col2);
+	    tex_rl_double(mc->se, col2);
 	}
 
 	if (na(mc->tval)) {
-	    sprintf(col3, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	    sprintf(col3, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
 	} else {
-	    sprintf(col3, "%.4f", mc->tval);
+	    tex_rl_float(mc->tval, col3, 4);
 	}
     }
 
     *col4 = '\0';
 
     if (!na(mc->slope)) {
-	tex_dcolumn_double(mc->slope, col4);
+	tex_rl_double(mc->slope, col4);
     } else if (mc->show_pval) {
 	if (!na(mc->pval)) {
-	    sprintf(col4, "%.4f", mc->pval);
+	    tex_rl_float(mc->pval, col4, 4);
 	}
     }
 
@@ -572,19 +740,26 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
     }	
 }
 
-void tex_custom_coeff_table_start (const char **cols, PRN *prn)
+static int
+tex_custom_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 {
     int i, ncols = 0;
 
     for (i=0; i<4; i++) {
-	if (colspec[i][0]) ncols++;
+	if (colspec[i][0]) {
+	    ncols++;
+	}
     }
 
-    pputs(prn, "\\vspace{1em}\n\n"
-	  "\\begin{tabular}{l");
+    if (!(opt & OPT_U)) {
+	/* not a user-defined model */
+	pputs(prn, "\\vspace{1em}\n\n");
+    }
+
+    pputs(prn, "\\begin{tabular}{l");
 
     for (i=0; i<ncols; i++) {
-	pputs(prn, "r");
+	pputc(prn, 'r');
     }
 
     pputs(prn, "}\n");
@@ -597,7 +772,7 @@ void tex_custom_coeff_table_start (const char **cols, PRN *prn)
 
     if (!colspec[1][0] && !colspec[2][0] && !colspec[3][0]) {
 	pputs(prn, " \\\\\n");
-	return;
+	return ncols;
     }
 
     if (colspec[1][0]) {
@@ -609,7 +784,7 @@ void tex_custom_coeff_table_start (const char **cols, PRN *prn)
 
     if (!colspec[2][0] && !colspec[3][0]) {
 	pputs(prn, " \\\\\n");
-	return;
+	return ncols;
     }
 
     if (colspec[2][0]) {
@@ -627,60 +802,55 @@ void tex_custom_coeff_table_start (const char **cols, PRN *prn)
     }
 
     pputs(prn, " \\\\\n");
+
+    return ncols;
 }
 
-void tex_coeff_table_start (const char **cols, int binary, PRN *prn)
+/* returns the number of columns in the coeff table */
+
+int tex_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 {
-    char pt;
+    char pt = get_local_decpoint();
+    int i, mcols, binary = (opt & OPT_B);
+    int ncols = 1;
 
     if (use_custom) {
-	tex_custom_coeff_table_start(cols, prn);
-	return;
+	return tex_custom_coeff_table_start(cols, opt, prn);
     }
 
-    pt = get_local_decpoint();
-
-    if (cols[4] == NULL) {
-	pprintf(prn, "\\vspace{1em}\n\n"
-		"\\begin{tabular*}{\\textwidth}"
-		"{@{\\extracolsep{\\fill}}\n"
-		"l%% col 1: varname\n"
-		"  D{%c}{%c}{-1}%% col 2: coeff\n"
-		"    D{%c}{%c}{-1}%% col 3\n"
-		"      D{%c}{%c}{-1}}%% col 4\n"
-		"%s &\n"
-		"  \\multicolumn{1}{c}{%s} &\n"
-		"    \\multicolumn{1}{c}{%s} &\n"
-		"      \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
-		pt, pt, pt, pt, pt, pt, 
-		I_(cols[0]), I_(cols[1]), I_(cols[2]), I_(cols[3]));
-    } else {
-	pprintf(prn, "\\vspace{1em}\n\n"
-		"\\begin{tabular*}{\\textwidth}"
-		"{@{\\extracolsep{\\fill}}\n"
-		"l%% col 1: varname\n"
-		"  D{%c}{%c}{-1}%% col 2: coeff\n"
-		"    D{%c}{%c}{-1}%% col 3: sderr\n"
-		"      D{%c}{%c}{-1}%% col 4: t-stat\n"
-		"        D{%c}{%c}{4}}%% col 5: p-value (or slope)\n"
-		"%s &\n"
-		"  \\multicolumn{1}{c}{%s} &\n"
-		"    \\multicolumn{1}{c}{%s} &\n"
-		"      \\multicolumn{1}{c}{%s} &\n"
-		"        \\multicolumn{1}{c}{%s%s} \\\\[1ex]\n",
-		pt, pt, pt, pt, pt, pt, pt, pt, 
-		I_(cols[0]), I_(cols[1]), I_(cols[2]), I_(cols[3]), I_(cols[4]),
-		(binary)? "$^*$" : "");
+    if (!(opt & OPT_U)) {
+	/* not a user-defined model */
+	pputs(prn, "\\vspace{1em}\n\n");
     }
+
+    pputs(prn, "\\begin{tabular}{l");
+
+    for (i=1; cols[i] != NULL; i++) {
+	if (opt & OPT_M) {
+	    pputc(prn, 'r');
+	} else {
+	    pprintf(prn, "r@{%c}l", pt);
+	}
+	ncols += 2;
+    }
+
+    pprintf(prn, "}\n%s &\n", I_(cols[0]));
+
+    mcols = (opt & OPT_M)? 1 : 2;
+
+    for (i=1; cols[i] != NULL; i++) {
+	bufspace(i, prn);
+	pprintf(prn, "\\multicolumn{%d}{c}{%s%s} %s\n", mcols, I_(cols[i]),
+		(cols[i+1] == NULL && binary)? "$^*$" : "",
+		(cols[i+1] == NULL)? "\\\\[1ex]" : "&");
+    }
+
+    return ncols;
 }
 
 void tex_coeff_table_end (PRN *prn)
 {
-    if (use_custom) {
-	pputs(prn, "\\end{tabular}\n\n");
-    } else {
-	pputs(prn, "\\end{tabular*}\n\n");
-    }
+    pputs(prn, "\\end{tabular}\n\n");
 }
 
 void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
@@ -716,7 +886,7 @@ void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
 	pprintf(prn, "$\\Delta$%s & ", vname);
 	for (j=0; j<vecm->neqns; j++) {
 	    x = gretl_matrix_get(vecm->S, i, j);
-	    tex_print_math_float(x, prn);
+	    tex_print_double(x, prn);
 	    if (j == vecm->neqns - 1) {
 		pputs(prn, "\\\\\n");
 	    } else {
@@ -730,7 +900,7 @@ void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
 
     pputs(prn, "\\noindent\n");
     pprintf(prn, "%s = ", I_("determinant"));
-    tex_print_math_float(exp(vecm->ldet), prn);
+    tex_print_double(exp(vecm->ldet), prn);
     pputs(prn, "\\\\\n");
 }
 
@@ -761,9 +931,9 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
     double x;
 
     pputs(prn, "\\noindent\n");
-    pputs(prn, _("Cointegrating vectors"));
+    pputs(prn, I_("Cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, " (%s)\n", _("standard errors in parentheses"));
+	pprintf(prn, " (%s)\n", I_("standard errors in parentheses"));
     } else {
 	pputc(prn, '\n');
     }
@@ -786,7 +956,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 	    if (jv->Bse == NULL) {
 		x /= gretl_matrix_get(jv->Beta, j, j);
 	    }
-	    tex_print_signed_float(x, prn);
+	    tex_print_double(x, prn);
 	    if (j == jv->rank - 1) {
 		pputs(prn, "\\\\\n");
 	    } else {
@@ -800,7 +970,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 	    for (j=0; j<jv->rank; j++) {
 		x = gretl_matrix_get(jv->Bse, i, j);
 		pputc(prn, '(');
-		tex_print_float(x, prn);
+		tex_print_double(x, prn);
 		pputc(prn, ')');
 		if (j == jv->rank - 1) {
 		    pputs(prn, "\\\\\n");
@@ -817,9 +987,9 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
     rows = gretl_matrix_rows(jv->Alpha);
 
     pputs(prn, "\\noindent\n");
-    pprintf(prn, _("Adjustment vectors"));
+    pprintf(prn, I_("Adjustment vectors"));
     if (jv->Ase != NULL) {
-	pprintf(prn, " (%s)\n", _("standard errors in parentheses"));
+	pprintf(prn, " (%s)\n", I_("standard errors in parentheses"));
     } else {
 	pputc(prn, '\n');
     }
@@ -842,7 +1012,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 	    if (jv->Ase == NULL) {
 		x /= gretl_matrix_get(jv->Alpha, j, j);
 	    }
-	    tex_print_signed_float(x, prn);
+	    tex_print_double(x, prn);
 	    if (j == jv->rank - 1) {
 		pputs(prn, "\\\\\n");
 	    } else {
@@ -856,7 +1026,7 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 	    for (j=0; j<jv->rank; j++) {
 		x = gretl_matrix_get(jv->Ase, i, j);
 		pputc(prn, '(');
-		tex_print_float(x, prn);
+		tex_print_double(x, prn);
 		pputc(prn, ')');
 		if (j == jv->rank - 1) {
 		    pputs(prn, "\\\\\n");
@@ -875,11 +1045,11 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn)
 {
     pprintf(prn, "\\noindent\n%s = ", I_("Log-likelihood"));
-    tex_print_math_float(var->ll, prn);
+    tex_print_double(var->ll, prn);
     pputs(prn, "\\par\n");
 
     pprintf(prn, "\\noindent\n%s = ", I_("Determinant of covariance matrix"));
-    tex_print_math_float(exp(var->ldet), prn);
+    tex_print_double(exp(var->ldet), prn);
     pputs(prn, "\\par\n");
 
     pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("AIC"), var->AIC);
@@ -1028,7 +1198,9 @@ void set_tex_use_utf (int s)
 
 void gretl_tex_preamble (PRN *prn, int fmt)
 {
+#ifdef ENABLE_NLS
     char* lang = getenv("LANG");
+#endif
     FILE *fp = NULL;
     int userfile = 0;
 
@@ -1087,7 +1259,7 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 		pputs(prn, "\\usepackage[margin=2cm,dvips]{geometry}\n\n");
 	    }
 	} else {
-	    pputs(prn, "\\usepackage{dcolumn,longtable}\n\n");
+	    pputs(prn, "\\usepackage{longtable}\n\n");
 	}
 
 	pputs(prn, "\\begin{document}\n\n"
@@ -1095,6 +1267,15 @@ void gretl_tex_preamble (PRN *prn, int fmt)
     }
 }
 
+/* For use when printing a model in equation style: print the value
+   unsigned, since the sign will be handled separately.
+*/
+
+static void tex_print_unsigned_double (double x, PRN *prn)
+{
+    tex_print_double(fabs(x), prn);
+}
+
 #define MAXCOEFF 4
 
 /**
@@ -1116,12 +1297,16 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 			gretlopt opt, PRN *prn)
 {
     double x;
-    char tmp[48];
+    char tmp[48], vname[32];
     int i, nc = pmod->ncoeff;
     int split = 0, offvar = 0;
     int cchars = 0, ccount = 0;
     int sderr_ok = 1;
 
+    if (pmod->ci == HECKIT) {
+	return E_NOTIMP;
+    }
+
     if (pmod->ci == POISSON) {
 	offvar = gretl_model_get_int(pmod, "offset_var");
 	if (offvar > 0) {
@@ -1148,8 +1333,6 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     if (pmod->depvar != NULL) {
 	tex_escape(tmp, pmod->depvar);
     } else if (pmod->ci == POISSON) {
-	char vname[32];
-
 	tex_escape(vname, pdinfo->varname[pmod->list[1]]);
 	sprintf(tmp, "log(%s)", vname);
     } else {
@@ -1173,7 +1356,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     for (i=0; i<nc; i++) {
 	if (offvar > 0 && i == nc - 1) {
 	    pputc(prn, '+');
-	    tex_print_float(1.0, prn);
+	    tex_print_double(1.0, prn);
 	} else {
 	    if (na(pmod->sderr[i])) {
 		sderr_ok = 0;			
@@ -1186,51 +1369,38 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 			    (pmod->coeff[i] < 0.0)? "-" :
 			    (i > 0)? "+" : "", x);
 		} else {
-		    pprintf(prn, "%s\\underset{(%.5g)}{", 
+		    tex_sprint_math_double_digits(pmod->sderr[i], tmp, 5);
+		    pprintf(prn, "%s\\underset{(%s)}{", 
 			    (pmod->coeff[i] < 0.0)? "-" :
-			    (i > 0)? "+" : "", pmod->sderr[i]);
+			    (i > 0)? "+" : "", tmp);
 		}
 	    }
-	    tex_print_float(pmod->coeff[i], prn);
+	    tex_print_unsigned_double(pmod->coeff[i], prn);
 	    pputc(prn, '}');
 	}
 
 	if (i > 0 || pmod->ifc == 0) {
+	    /* regular coefficient, not const */
+	    if (offvar > 0 && i == nc - 1) {
+		strcpy(vname, pdinfo->varname[offvar]);
+	    } else {
+		gretl_model_get_param_name(pmod, pdinfo, i, vname);
+	    }
+	    cchars += strlen(vname);
+
 	    pputs(prn, "\\,");
+
 	    if (pmod->ci == ARMA) {
-		cchars += strlen(pmod->params[i]);
-		tex_arma_coeff_name(tmp, pmod->params[i], 1);
+		tex_arma_coeff_name(tmp, vname, 1);
 		pputs(prn, tmp);
 	    } else if (pmod->ci == GARCH) {
-		cchars += strlen(pmod->params[i]);
-		tex_garch_coeff_name(tmp, pmod->params[i], 1);
+		tex_garch_coeff_name(tmp, vname, 1);
 		pputs(prn, tmp);
-	    } else if (pmod->ci == PANEL) {
-		cchars += strlen(pmod->params[i]);
-		tex_escape(tmp, pmod->params[i]);
-		pprintf(prn, "\\mbox{%s}", tmp);
-	    } else if (pmod->ci == ARBOND) {
-		if (strcmp(pmod->params[i], "const")) {
-		    cchars += strlen(pmod->params[i]);
-		    tex_arbond_coeff_name(tmp, pmod->params[i], 1);
-		    pputs(prn, tmp);
-		}
-	    } else if (pmod->ci == MPOLS && pmod->params != NULL) {
-		cchars += strlen(pmod->params[i]);
-		tex_mp_coeff_name(tmp, pmod->params[i], 1);
+	    } else if (pmod->ci == MPOLS) {
+		tex_mp_coeff_name(tmp, vname, 1);
 		pputs(prn, tmp);
-	    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT) &&
-		       pmod->params != NULL) {
-		cchars += strlen(pmod->params[i]);
-		tex_escape(tmp, pmod->params[i]);
-		pprintf(prn, "\\mbox{%s}", tmp);
-	    } else if (offvar > 0 && i == nc - 1) {
-		cchars += strlen(pdinfo->varname[offvar]);
-		tex_escape(tmp, pdinfo->varname[offvar]);
-		pprintf(prn, "\\mbox{%s}", tmp);
 	    } else {
-		cchars += strlen(pdinfo->varname[pmod->list[i+2]]);
-		tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
+		tex_escape(tmp, vname);
 		pprintf(prn, "\\mbox{%s}", tmp);
 	    }
 	}
@@ -1259,8 +1429,9 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	    pprintf(prn, "\\hat{\\sigma}^2_t = \\underset{(%.3f)}{%g} ", 
 		    x, pmod->coeff[r]);
 	} else {
-	    pprintf(prn, "\\hat{\\sigma}^2_t = \\underset{(%.5g)}{%g} ", 
-		    pmod->sderr[r], pmod->coeff[r]);
+	    tex_sprint_math_double_digits(pmod->sderr[r], tmp, 5);
+	    pprintf(prn, "\\hat{\\sigma}^2_t = \\underset{(%s)}{%g} ", /* FIXME? */
+		    tmp, pmod->coeff[r]);
 	}	    
 
 	for (i=1; i<=q; i++) {
@@ -1269,11 +1440,11 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 		pprintf(prn, "%s\\underset{(%.3f)}{", 
 			(pmod->coeff[r+i] < 0.0)? "-" : "+", x);
 	    } else {
-		pprintf(prn, "%s\\underset{(%.5g)}{", 
-			(pmod->coeff[r+i] < 0.0)? "-" : "+", 
-			pmod->sderr[r+i]);
+		tex_sprint_math_double_digits(pmod->sderr[r+i], tmp, 5);
+		pprintf(prn, "%s\\underset{(%s)}{", 
+			(pmod->coeff[r+i] < 0.0)? "-" : "+", tmp);
 	    }		
-	    tex_print_float(pmod->coeff[r+i], prn);
+	    tex_print_unsigned_double(pmod->coeff[r+i], prn);
 	    pputs(prn, "}\\,");
 	    pprintf(prn, "\\varepsilon^2_{t-%d}", i);
 	}
@@ -1284,11 +1455,11 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 		pprintf(prn, "%s\\underset{(%.3f)}{", 
 			(pmod->coeff[q+r+i] < 0.0)? "-" : "+", x);
 	    } else {
-		pprintf(prn, "%s\\underset{(%.5g)}{", 
-			(pmod->coeff[q+r+i] < 0.0)? "-" : "+", 
-			pmod->sderr[q+r+i]);
+		tex_sprint_math_double_digits(pmod->sderr[q+r+i], tmp, 5);
+		pprintf(prn, "%s\\underset{(%s)}{", 
+			(pmod->coeff[q+r+i] < 0.0)? "-" : "+", tmp);
 	    }		
-	    tex_print_float(pmod->coeff[q+r+i], prn);
+	    tex_print_unsigned_double(pmod->coeff[q+r+i], prn);
 	    pputs(prn, "}\\,");
 	    pprintf(prn, "\\sigma^2_{t-%d}", i);
 	}
@@ -1303,8 +1474,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	double SAR = gretl_model_get_double(pmod, "ladsum");
 
 	if (!na(SAR)) {
-	    sprintf(tmp, "%g", SAR);
-	    tex_modify_exponent(tmp);
+	    tex_sprint_math_double_digits(SAR, tmp, 6);
 	    pprintf(prn, "\\quad \\sum |\\hat{u}_t| = %s ", tmp);
 	}
     } else {
@@ -1315,23 +1485,20 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	}
 
 	if (pmod->ci != LOGIT && pmod->ci != PROBIT && !na(pmod->fstt)) {
-	    sprintf(tmp, "%.5g", pmod->fstt);
-	    tex_modify_exponent(tmp);
+	    tex_sprint_math_double_digits(pmod->fstt, tmp, 5);
 	    pprintf(prn, "\\quad F(%d,%d) = %s ", 
 		    pmod->dfn, pmod->dfd, tmp);
 	}
 
 	if (!na(pmod->sigma)) {
-	    sprintf(tmp, "%.5g", pmod->sigma);
-	    tex_modify_exponent(tmp);
+	    tex_sprint_math_double_digits(pmod->sigma, tmp, 5);
 	    pprintf(prn, "\\quad \\hat{\\sigma} = %s ", tmp);
 	}
 
 	if (!na(gretl_model_get_double(pmod, "rho_in"))) {
 	    double r = gretl_model_get_double(pmod, "rho_in");
 
-	    sprintf(tmp, "%.5g", r);
-	    tex_modify_exponent(tmp);
+	    tex_sprint_math_double_digits(r, tmp, 5);
 	    pprintf(prn, " \\quad \\rho = %s", tmp);
 	}
     }
