@@ -159,16 +159,50 @@ int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
 {
     int pv = 0, ret = 0;
 
+    if (pdinfo == NULL || v <= 0 || v >= pdinfo->v) {
+	return 0;
+    }
+
     if (pdinfo->varinfo[v]->transform == LAGS) {
 	pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
 	pv = (pv < pdinfo->v)? pv : 0;
+	ret = pdinfo->varinfo[v]->lag;
     }
 
     if (pv > 0) {
-	if (parent != NULL) {
-	    *parent = pv;
+	int n = strcspn(pdinfo->varname[v], "_");
+
+	if (!strncmp(pdinfo->varname[v], pdinfo->varname[pv], n)) {
+	    if (parent != NULL) {
+		*parent = pv;
+	    }
+	} else {
+	    /* if the lag has a "custom" name, don't consider it
+	       a child to parent variable pv */
+	    ret = 0;
+	}
+    }
+
+    return ret;
+}
+
+int is_standard_diff (int v, const DATAINFO *pdinfo, int *parent)
+{
+    int pv = 0, ret = 0;
+
+    if (v <= 0 || v >= pdinfo->v) {
+	return 0;
+    }
+
+    if (pdinfo->varinfo[v]->transform == DIFF) {
+	pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
+	pv = (pv < pdinfo->v)? pv : 0;
+	if (pv > 0) {
+	    if (parent != NULL) {
+		*parent = pv;
+	    }
+	    ret = 1;
 	}
-	ret = 1;
     }
 
     return ret;
