@@ -23,7 +23,9 @@
 #include "qr_estimate.h"
 #include "libset.h"
 #include "estim_private.h"
+#include "missing_private.h"
 #include "matrix_extra.h"
+#include "tsls.h"
 
 #define TDEBUG 0
 
@@ -31,10 +33,10 @@ static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
 {
     int i, v;
 
-    for (i=2; i<=list[0]; i++) {
+    for (i=list[0]; i>1; i--) {
         v = list[i];
         if (gretl_iszero(t1, t2, Z[v])) {
-	    gretl_list_delete_at_pos(list, i--);
+	    gretl_list_delete_at_pos(list, i);
 	}
     }
 }
@@ -61,8 +63,8 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 {
     double **X = NULL;
     char *endog = NULL;
-    int i, j, v, err = 0;
-    size_t Xsize, esize;
+    int i, v, err = 0;
+    size_t esize, Xsize;
     size_t xs_old, es_old;
     int recycle_X = 0;
     int recycle_e = 0;
@@ -119,21 +121,15 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 
     for (i=0; i<pmod->ncoeff; i++) {
 	v = pmod->list[i+2];
-	endog[i] = 1;
-	for (j=1; j<=exolist[0]; j++) {
-	    if (exolist[j] == v) {
-		endog[i] = 0;
-		break;
-	    }
-	}
+	endog[i] = !in_gretl_list(exolist, v);
     }
 
-    /* now attach X and endog to the model */
-    if (!recycle_X) {
+     /* now attach X and endog to the model */
+    if (!recycle_X && X != NULL) {
 	gretl_model_set_data(pmod, "tslsX", X, GRETL_TYPE_DOUBLE_ARRAY,
 			     Xsize);
     }
-    if (!recycle_e) {
+    if (!recycle_e && endog != NULL) {
 	gretl_model_set_data(pmod, "endog", endog, GRETL_TYPE_CHAR_ARRAY,
 			     esize);
     }
@@ -141,34 +137,6 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
     return err;
 }
 
-double *tsls_get_Xi (const MODEL *pmod, double **Z, int i)
-{
-    const char *endog = gretl_model_get_data(pmod, "endog");
-    double **X;
-    double *ret;
-
-    if (endog == NULL) {
-	return NULL;
-    }
-
-    X = gretl_model_get_data(pmod, "tslsX");
-
-    if (!endog[i]) {
-	ret = Z[pmod->list[i+2]];
-    } else if (X == NULL) {
-	ret = NULL;
-    } else {
-	int j, k = 0;
-
-	for (j=0; j<i; j++) {
-	    if (endog[j]) k++;
-	}
-	ret = X[k];
-    }
-
-    return ret;
-}
-
 static void add_tsls_var (int *list, int v, gretlopt opt)
 {
     int pos = gretl_list_separator_position(list);
@@ -236,7 +204,7 @@ static void delete_tsls_var (int *list, int v, gretlopt opt)
     }
 }
 
-static int in_tsls_list (const int *list, int v, gretlopt opt)
+static int in_ivreg_list (const int *list, int v, gretlopt opt)
 {
     int pos = gretl_list_separator_position(list);
     int i, imin, imax;
@@ -266,36 +234,35 @@ static int in_tsls_list (const int *list, int v, gretlopt opt)
 }
 
 /**
- * tsls_list_omit:
- * @orig: list specifying original TSLS model.
+ * ivreg_list_omit:
+ * @orig: list specifying original IV model.
  * @drop: list of variables to be omitted.
  * @opt: may include %OPT_T (omit variable only as instrument)
  * or %OPT_B (omit both as regressor and instrument).  The default
  * is to omit (only) as regressor.
  * @err: pointer to receive error code.
  *
- * Creates a new TSLS specification list, by first copying @orig
+ * Creates a new IVREG specification list, by first copying @orig
  * then deleting from the copy the variables found in @drop.
  *
  * Returns: the new, reduced list or %NULL on error.
  */
 
 int *
-tsls_list_omit (const int *orig, const int *drop, gretlopt opt, int *err)
+ivreg_list_omit (const int *orig, const int *drop, gretlopt opt, int *err)
 {
     int *newlist;
     int i;
 
-    if ((opt & OPT_T) && (opt & OPT_B)) {
-	/* incoherent options */
-	*err = E_BADOPT;
+    *err = incompatible_options(opt, OPT_T | OPT_B);
+    if (*err) {
 	return NULL;
     }
 
     newlist = gretl_list_copy(orig);
 
     for (i=1; i<=drop[0]; i++) {
-	if (in_tsls_list(orig, drop[i], opt)) {
+	if (in_ivreg_list(orig, drop[i], opt)) {
 	    delete_tsls_var(newlist, drop[i], opt);
 	} else {
 	    *err = E_UNSPEC;
@@ -311,31 +278,30 @@ tsls_list_omit (const int *orig, const int *drop, gretlopt opt, int *err)
 }
 
 /**
- * tsls_list_add:
- * @orig: list specifying original TSLS model.
+ * ivreg_list_add:
+ * @orig: list specifying original IV model.
  * @add: list of variables to be added.
  * @opt: may include %OPT_T (add variable only as instrument)
  * or %OPT_B (add both as regressor and instrument).  The default
  * is to add as an endogenous regressor.
  * @err: pointer to receive error code.
  *
- * Creates a new TSLS specification list, by copying @orig
+ * Creates a new IVREG specification list, by copying @orig
  * and adding the variables found in @add.
  *
  * Returns: the new, augmented list or %NULL on error.
  */
 
 int *
-tsls_list_add (const int *orig, const int *add, gretlopt opt, int *err)
+ivreg_list_add (const int *orig, const int *add, gretlopt opt, int *err)
 {
     int norig = orig[0];
     int nadd = add[0];
     int *newlist;
     int i;
 
-    if ((opt & OPT_T) && (opt & OPT_B)) {
-	/* incoherent options */
-	*err = E_BADOPT;
+    *err = incompatible_options(opt, OPT_T | OPT_B);
+    if (*err) {
 	return NULL;
     }
 
@@ -350,7 +316,7 @@ tsls_list_add (const int *orig, const int *add, gretlopt opt, int *err)
     }
 
     for (i=1; i<=add[0]; i++) {
-	if (in_tsls_list(orig, add[i], opt)) {
+	if (in_ivreg_list(orig, add[i], opt)) {
 	    *err = E_ADDDUP;
 	} else {
 	    add_tsls_var(newlist, add[i], opt);
@@ -366,59 +332,177 @@ tsls_list_add (const int *orig, const int *add, gretlopt opt, int *err)
 }
 
 /*
-  tsls_make_hatlist: determines which variables in reglist, when
+  tsls_make_endolist: determines which variables in reglist, when
   checked against the predetermined and exogenous vars in instlist,
-  need to be instrumented, and populates hatlist accordingly
+  need to be instrumented, and populates the returned list 
+  accordingly
 */
 
-static int 
-tsls_make_hatlist (const int *reglist, int *instlist, int *hatlist)
+int *tsls_make_endolist (const int *reglist, int **instlist, 
+			 int *addconst, int *err)
 {
-    int i, j, k = 0;
-    int endog, addconst = 0;
+    int *hatlist = NULL;
+    int i, vi;
 
-    for (i=2; i<=reglist[0]; i++) {  
-	endog = 1;
-	for (j=1; j<=instlist[0]; j++) {
-	    if (instlist[j] == reglist[i]) {
-		endog = 0;
-		break;
-	    }
-	}
-	if (endog) {
-	    if (reglist[i] == 0) {
+    for (i=2; i<=reglist[0]; i++) {
+	vi = reglist[i];
+	if (!in_gretl_list(*instlist, vi)) {
+	    if (vi == 0) {
 		/* found const in reglist but not instlist: 
 		   needs fixing -- or is this debatable? */
-		addconst = 1;
+		if (addconst != NULL) {
+		    *addconst = 1;
+		}
 	    } else {
-		hatlist[++k] = reglist[i];
+		hatlist = gretl_list_append_term(&hatlist, vi);
+		if (hatlist == NULL) {
+		    *err = E_ALLOC;
+		    return NULL;
+		}
 	    }
 	} 
     }
 
-    hatlist[0] = k;
-
-    if (addconst) {
+    if (addconst != NULL && *addconst) {
 	/* add constant to list of instruments */
-	instlist[0] += 1;
-	for (i=instlist[0]; i>=2; i--) {
-	    instlist[i] = instlist[i - 1];
-	}
-	instlist[1] = 0;
+	int clist[2] = {1, 0};
+
+	*err = gretl_list_insert_list(instlist, clist, 1);
     }
 
-    return addconst;
+    return hatlist;
 }
 
-/* perform the Sargan overidentification test for a model
-   estimated via TSLS */
+/* fill the residuals matrix for tsls likelihood calculation */
+
+static int fill_E_matrix (gretl_matrix *E, MODEL *pmod, 
+			  const int *reglist, const int *instlist,
+			  double ***pZ, DATAINFO *pdinfo)
+{
+    MODEL emod;
+    int *elist;
+    double uit;
+    int nx = instlist[0];
+    int i, vi, s, t, j = 0;
+    int T = E->rows;
+    int err = 0;
+
+    elist = gretl_list_new(nx + 1);
+    if (elist == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=2; i<=elist[0]; i++) {
+	elist[i] = instlist[i-1];
+    }
+
+    for (i=1; i<=reglist[0] && !err; i++) {
+	vi = reglist[i];
+
+	if (in_gretl_list(instlist, vi)) {
+	    continue;
+	}
+
+	elist[1] = vi;
+
+	if (pmod->missmask != NULL) {
+	    set_reference_missmask_from_model(pmod);
+	}
+
+	/* regress the given endogenous var on all the instruments */
+	emod = lsq(elist, pZ, pdinfo, OLS, OPT_A);
+	if ((err = emod.errcode)) {
+	    clear_model(&emod);
+	    break;
+	}
+
+	/* put residuals into appropriate column of E and
+	   increment the column: we should get exactly the same
+	   count of non-NA values as for the original model
+	*/
+	s = 0;
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    uit = emod.uhat[t];
+	    if (!na(uit)) {
+		if (s < T) {
+		    gretl_matrix_set(E, s, j, uit);
+		}
+		s++;
+	    }
+	}
+	j++;
+
+	if (s != T) {
+	    err = E_DATA;
+	}
+
+	clear_model(&emod);
+    }
+
+    free(elist);
+
+    return err;
+}
+
+/* calculate the maximized log-likelihood for a just-identified
+   model */
+
+static int tsls_loglik (MODEL *pmod, 
+			int nendo,
+			const int *reglist,
+			const int *instlist,
+			double ***pZ, 
+			DATAINFO *pdinfo)
+{
+    gretl_matrix *E, *W;
+    int T = pmod->nobs;
+    int k = 1 + nendo;
+    int err = 0;
+
+    pmod->lnL = NADBL;
+
+    E = gretl_matrix_alloc(T, k);
+    W = gretl_matrix_alloc(k, k); 
+
+    if (E == NULL || W == NULL) {
+	err = E_ALLOC;
+    }
+
+    if (!err) {
+	err = fill_E_matrix(E, pmod, reglist, instlist, pZ, pdinfo);
+    }
+
+    if (!err) {
+	err = gretl_matrix_multiply_mod(E, GRETL_MOD_TRANSPOSE,
+					E, GRETL_MOD_NONE,
+					W, GRETL_MOD_NONE);
+    }
+
+    if (!err) {
+	double ldet = gretl_matrix_log_determinant(W, &err);
+
+	if (!err) {
+	    /* Davidson and MacKinnon, ETM, p. 538 */
+	    pmod->lnL = -(T / 2.0) * (LN_2_PI + ldet);
+	} 
+    }
+
+    mle_criteria(pmod, 0);
+
+    gretl_matrix_free(E);
+    gretl_matrix_free(W);
+
+    return err;
+}
+
+/* perform the Sargan overidentification test for an IV model */
 
 static int 
-tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist, 
-		  double ***pZ, DATAINFO *pdinfo)
+ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist, 
+		   double ***pZ, DATAINFO *pdinfo)
 {
-    int t1 = tsls_model->t1;
-    int t2 = tsls_model->t2;
+    int t1 = pmod->t1;
+    int t2 = pmod->t2;
     int ninst = instlist[0];
     int i, t, err = 0;
 
@@ -430,7 +514,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	return 0;
     }
 
-    if (tsls_model->list[0] == 2 && tsls_model->list[2] == 0) {
+    if (pmod->list[0] == 2 && pmod->list[2] == 0) {
 	/* degenerate model with const only */
 	return 0;
     }
@@ -441,7 +525,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
     }
 
     for (t=t1; t<=t2; t++) {
-	(*pZ)[nv][t] = tsls_model->uhat[t];
+	(*pZ)[nv][t] = pmod->uhat[t];
     }
 
     OT_list = gretl_list_new(ninst + 1);
@@ -457,7 +541,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 
     smod = lsq(OT_list, pZ, pdinfo, OLS, OPT_A);
     if (smod.errcode) {
-	fprintf(stderr, "tsls_sargan_test: smod.errcode = %d\n", smod.errcode);
+	fprintf(stderr, "ivreg_sargan_test: smod.errcode = %d\n", smod.errcode);
 	err = smod.errcode;
     } else {
 	ModelTest *test = model_test_new(GRETL_TEST_SARGAN);
@@ -468,7 +552,7 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	    model_test_set_dfn(test, Orank);
 	    model_test_set_value(test, OTest);
 	    model_test_set_pvalue(test, chisq_cdf_comp(Orank, OTest));
-	    maybe_add_test_to_model(tsls_model, test);
+	    maybe_add_test_to_model(pmod, test);
 	}	
     }
 
@@ -523,7 +607,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 	*/
 	double URSS = hmod.ess;
 	double HTest = (RRSS/URSS - 1) * hmod.nobs;
-	ModelTest *test = model_test_new(GRETL_TEST_TSLS_HAUSMAN);
+	ModelTest *test = model_test_new(GRETL_TEST_IV_HAUSMAN);
 
 	if (test != NULL) {
 	    model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
@@ -556,7 +640,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     double test;
     int i, j, k;
 
-    Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask, err);
+    Q = gretl_matrix_data_subset_masked(instlist, Z, t1, t2, mask, err);
     if (*err) {
 	return NULL;
     }
@@ -574,7 +658,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 	goto bailout;
     }
 
-    rank = gretl_check_QR_rank(R, err);
+    rank = gretl_check_QR_rank(R, err, NULL);
 
 #if TDEBUG
     fprintf(stderr, "tsls_Q: k = Q->cols = %d, rank = %d\n", k, rank);
@@ -613,7 +697,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
 	k = instlist[0];
 	gretl_matrix_free(Q);
-	Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask, err);
+	Q = gretl_matrix_data_subset_masked(instlist, Z, t1, t2, mask, err);
 	if (!*err) {
 	    R = gretl_matrix_reuse(R, k, k);
 	    *err = gretl_matrix_QR_decomp(Q, R);
@@ -722,7 +806,8 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
     }
 }
 
-static void tsls_extra_stats (MODEL *pmod, const double **Z)
+static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
+			      const DATAINFO *pdinfo)
 {
     int yno = pmod->list[1];
     double r;
@@ -732,12 +817,25 @@ static void tsls_extra_stats (MODEL *pmod, const double **Z)
 
     pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1.0) / pmod->dfd);
 
-    pmod->fstt = pmod->rsq * pmod->dfd / (pmod->dfn * r);
+    pmod->fstt = pmod->chisq = NADBL;
 
-    ls_criteria(pmod);
+    if (pmod->ncoeff > 1) {
+	if (overid && pmod->ncoeff == 2) {
+	    pmod->chisq = wald_omit_chisq(NULL, pmod);
+	} else {	
+	    pmod->fstt = wald_omit_F(NULL, pmod);
+	}
+    } 
 
-    if (pmod->missmask == NULL) {
-	/* no missing obs within sample range */
+    /* tsls is not a ML estimator unless it's exactly identified, 
+       and in that case we require a special calculation */
+    pmod->lnL = NADBL;
+    pmod->criterion[C_AIC] = NADBL;
+    pmod->criterion[C_BIC] = NADBL;
+    pmod->criterion[C_HQC] = NADBL;
+    
+     if (dataset_is_time_series(pdinfo) && pmod->missmask == NULL) {
+	/* time series, no missing obs within sample range */
 	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
 	pmod->dw = dwstat(1, pmod, Z);
     } else {
@@ -748,26 +846,13 @@ static void tsls_extra_stats (MODEL *pmod, const double **Z)
 static void tsls_recreate_full_list (MODEL *pmod, const int *reglist,
 				     const int *instlist)
 {
-    int len = reglist[0] + instlist[0] + 1;
     int *full_list;
 
-    full_list = gretl_list_new(len);
+    full_list = gretl_lists_join_with_separator(reglist, instlist);
 
     if (full_list == NULL) {
 	pmod->errcode = E_ALLOC;
     } else {
-	int i, j = 1;
-
-	for (i=1; i<=reglist[0]; i++) {
-	    full_list[j++] = reglist[i];
-	}
-
-	full_list[j++] = LISTSEP;
-
-	for (i=1; i<=instlist[0]; i++) {
-	    full_list[j++] = instlist[i];
-	}
-
 	free(pmod->list);
 	pmod->list = full_list;
     }
@@ -805,13 +890,225 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
     return 0;
 }
 
+/* Stock-Yogo: compute test statistic for weakness of instruments.
+   This is the Cragg-Donald statistic: the minimum eigenvalue of the
+   matrix defined below (which reduces to the first-stage F-statistic
+   when there is just one endogenous regressor).  See Stock and Yogo,
+   "Testing for Weak Instruments in Linear IV Regressions" (originally
+   NBER Technical Working Paper 284; revised February 2003), at
+   http://ksghome.harvard.edu/~JStock/pdf/rfa_6.pdf
+ */
+
+static int 
+compute_stock_yogo (MODEL *pmod, const int *endolist, 
+		    const int *instlist, const int *hatlist,
+		    const double **dZ)
+{
+    gretl_matrix_block *B = NULL;
+    gretl_matrix_block *B2 = NULL;
+    gretl_matrix *G, *S, *Y;
+    gretl_matrix *Ya, *YPY;
+    gretl_matrix *X, *Z, *E;
+    int T = pmod->nobs;
+    int n = endolist[0];
+    int K1 = pmod->ncoeff - n;
+    int K2 = instlist[0] - K1;
+    int i, s, vi, vj, t;
+    int err = 0;
+
+    B = gretl_matrix_block_new(&G, n, n,
+			       &S, n, n,
+			       &Y, T, n,
+			       &Ya, T, n,
+			       &YPY, n, n,
+			       NULL);
+    if (B == NULL) {
+	return E_ALLOC;
+    }
+
+    if (K1 > 0) {
+	gretl_matrix *M;
+	int j, ix = 0, iz = 0;
+
+	B2 = gretl_matrix_block_new(&X, T, K1,
+				    &Z, T, K2,
+				    &E, T, K2,
+				    NULL);
+	if (B2 == NULL) {
+	    gretl_matrix_block_destroy(B);
+	    return E_ALLOC;
+	}
+
+	/* form the X and Z matrices */
+	for (i=1; i<=instlist[0]; i++) {
+	    vi = instlist[i];
+	    if (in_gretl_list(pmod->list, vi)) {
+		/* included exogenous var -> X */
+		M = X;
+		j = ix++;
+	    } else {
+		/* excluded exogenous var -> Z */
+		M = Z;
+		j = iz++;
+	    }
+	    s = 0;
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!na(pmod->uhat[t])) {
+		    gretl_matrix_set(M, s++, j, dZ[vi][t]);
+		}
+	    }
+	}
+    }	
+
+    /* form Y matrix */
+    for (i=0; i<n; i++) {
+	vi = endolist[i+1];
+	s = 0;
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->uhat[t])) {
+		gretl_matrix_set(Y, s++, i, dZ[vi][t]);
+	    }
+	}
+    }
+
+    if (K1 == 0) {
+	/* form Y-hat matrix in Ya */
+	for (i=0; i<n; i++) {
+	    vi = hatlist[i+1];
+	    s = 0;
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!na(pmod->uhat[t])) {
+		    gretl_matrix_set(Ya, s++, i, dZ[vi][t]);
+		}
+	    }
+	}
+    } else {
+	int bdim = (K1 > K2)? K1 : K2;
+	gretl_matrix *b;
+
+	/* the leading dimension of b is the number of regressors */
+	b = gretl_matrix_alloc(bdim, K2);
+	if (b == NULL) {
+	    err = E_ALLOC;
+	}
+
+	if (!err) {
+	    /* partial X out of Y: Y <- M_x Y */
+	    gretl_matrix_reuse(b, K1, n);
+	    gretl_matrix_reuse(E, -1, n);
+	    err = gretl_matrix_multi_ols(Y, X, b, E, NULL);
+	    gretl_matrix_copy_values(Y, E);
+	}
+
+	if (!err) {
+	    /* partial X out of Z: Z <- M_x Z */
+	    gretl_matrix_reuse(b, K1, K2);
+	    gretl_matrix_reuse(E, -1, K2);
+	    err = gretl_matrix_multi_ols(Z, X, b, E, NULL);
+	    gretl_matrix_copy_values(Z, E);
+	}
+
+	if (!err) {
+	    /* form projection P_z Y, in Ya */
+	    gretl_matrix_reuse(b, K2, n);
+	    gretl_matrix_reuse(E, -1, n);
+	    err = gretl_matrix_multi_ols(Y, Z, b, E, NULL);
+	    gretl_matrix_copy_values(Ya, Y);
+	    gretl_matrix_subtract_from(Ya, E);
+	}
+
+	gretl_matrix_free(b);
+    } 
+
+    if (!err) {
+	/* form Y' P_z Y */
+	err = gretl_matrix_multiply_mod(Y, GRETL_MOD_TRANSPOSE,
+					Ya, GRETL_MOD_NONE,
+					YPY, GRETL_MOD_NONE);
+    }
+
+    /* now write first-stage residuals into Ya */
+    for (i=0; i<n && !err; i++) {
+	vi = endolist[i+1];
+	vj = hatlist[i+1];
+	s = 0;
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->uhat[t])) {
+		gretl_matrix_set(Ya, s++, i, dZ[vi][t] - dZ[vj][t]);
+	    }
+	}
+    }    
+
+    if (!err) {
+	/* form S = \hat{\Sigma}_{vv} */
+	err = gretl_matrix_multiply_mod(Y, GRETL_MOD_TRANSPOSE,
+					Ya, GRETL_MOD_NONE,
+					S, GRETL_MOD_NONE);
+	if (!err) {
+	    gretl_matrix_xtr_symmetric(S);
+	    gretl_matrix_divide_by_scalar(S, T - K1 - K2);
+	}
+    }
+
+    if (!err) {
+	/* S^{-1/2}: invert S and Cholesky-decompose */
+	err = gretl_invert_symmetric_matrix(S);
+	if (!err) {
+	    err = gretl_matrix_cholesky_decomp(S);
+	}
+    }
+
+    if (!err) {
+	/* finally, form big sandwich */
+	err = gretl_matrix_qform(S, GRETL_MOD_TRANSPOSE,
+				 YPY, G, GRETL_MOD_NONE);
+	if (!err) {
+	    gretl_matrix_divide_by_scalar(G, K2);
+	}
+    }
+
+    if (!err) {
+	gretl_matrix *e;
+
+	e = gretl_symmetric_matrix_eigenvals(G, 0, &err); 
+
+	if (!err) {
+	    double gmin = e->val[0];
+
+	    for (i=1; i<n; i++) {
+		if (e->val[i] < gmin) {
+		    gmin = e->val[i];
+		}
+	    }
+	    gretl_matrix_free(e);
+	    gretl_model_set_double(pmod, "gmin", gmin);
+	    gretl_model_set_int(pmod, "n", n);
+	    gretl_model_set_int(pmod, "K2", K2);
+	}	    
+    }
+
+    gretl_matrix_block_destroy(B);
+    gretl_matrix_block_destroy(B2);
+
+    if (err) {
+	fprintf(stderr, "compute_stock_yogo: err = %d\n", err);
+    }
+
+    return err;
+}
+
+/* Calculate first-stage F-test as diagnostic for weak instruments.
+   This variant can only handle one endogenous regressor, but it
+   supports robust estimation if the model in question uses a robust
+   covariance estimator.
+*/
+
 static int 
 compute_first_stage_F (MODEL *pmod, int v, const int *reglist, 
 		       const int *instlist, double ***pZ, 
 		       DATAINFO *pdinfo, gretlopt opt)
 {
     MODEL mod1;
-    gretlopt myopt;
     int *list1 = NULL;
     int *flist = NULL;
     double F;
@@ -820,6 +1117,11 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
 
     gretl_model_init(&mod1);
 
+    /* The regression list, list1, has endogenous regressor v as the
+       dependent variable and includes all instruments; the list for
+       omission, flist, includes all the excluded instruments.
+    */
+
     list1 = gretl_list_append_term(&list1, v);
     if (list1 == NULL) {
 	return E_ALLOC;
@@ -842,13 +1144,14 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     }
 
     if (!err) {
-	myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
+	gretlopt myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
+
 	mod1 = lsq(list1, pZ, pdinfo, OLS, myopt);
 	err = mod1.errcode;
     }
 
     if (!err) {
-	F = robust_omit_F(flist, &mod1);
+	F = wald_omit_F(flist, &mod1);
 	if (na(F)) {
 	    err = 1;
 	}
@@ -858,6 +1161,10 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
 	gretl_model_set_double(pmod, "stage1-F", F);
 	gretl_model_set_int(pmod, "stage1-dfn", flist[0]);
 	gretl_model_set_int(pmod, "stage1-dfd", mod1.dfd);
+	if (!(opt & OPT_R)) {
+	    /* flag lookup of Stock-Yogo critical values */
+	    gretl_model_set_double(pmod, "gmin", F);
+	}	
     }
     
     clear_model(&mod1);
@@ -968,109 +1275,135 @@ tsls_adjust_sample (const int *list, int *t1, int *t2,
 }
 
 /**
- * tsls_func:
+ * ivreg_process_lists:
+ * @list: original specification list.
+ * @reglist: location to receive regression list.
+ * @instlist: location to receive list of instruments.
+ *
+ * Split the incoming list into its two components and perform
+ * some basic checks; if the checks fail the two created
+ * lists are destroyed.
+ *
+ * Returns: 0, on success, non-zero error code on failure.
+ */
+
+int ivreg_process_lists (const int *list, int **reglist, int **instlist)
+{
+    int *rlist, *zlist;
+    int i, err, oid;
+
+    err = gretl_list_split_on_separator(list, reglist, instlist);
+    if (err) {
+	fprintf(stderr, "gretl_list_split_on_separator: failed\n");
+	return err;
+    }
+
+    rlist = *reglist;
+    zlist = *instlist;
+
+    if (rlist[0] < 2 || zlist[0] < 1) {
+	err = E_ARGS;
+    } else {
+	for (i=1; i<=zlist[0]; i++) {
+	    if (zlist[i] == list[1]) {
+		strcpy(gretl_errmsg, 
+		       "You can't use the dependent variable as an instrument");
+		err = E_DATA;
+		break;
+	    }
+	}
+    }
+    
+    if (!err) {
+	oid = zlist[0] - rlist[0] + 1;
+	if (oid < 0) {
+	    sprintf(gretl_errmsg, 
+		    _("The order condition for identification is not satisfied.\n"
+		      "At least %d more instruments are needed."), -oid);
+	    err = E_DATA; 
+	}
+    }
+
+    if (err) {
+	free(*reglist);
+	free(*instlist);
+	*reglist = NULL;
+	*instlist = NULL;
+    }
+
+    return err;
+}
+
+/**
+ * tsls:
  * @list: dependent variable plus list of regressors.
- * @ci: %TSLS for regular two-stage least squares, or
- * %SYSTEM if the 2SLS estimates are wanted in the context of
- * estimation of a system of equations.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @opt: may contain %OPT_O for printing covariance matrix, 
- * %OPT_R for robust VCV, %OPT_N for no df correction.
+ * @opt: may contain %OPT_R for robust VCV, %OPT_N for no df correction, 
+ * %OPT_A if this is an auxiliary reression, %OPT_E is we're
+ * estimating one equation within a system; and %OPT_H to
+ * add "hatlist" to model even under %OPT_E.
  *
  * Estimate the model given in @list by means of Two-Stage Least
- * Squares.  If @ci is %SYSTEM, fitted values from the first-stage
- * regressions are saved with the model: see tsls_get_Xi().
+ * Squares.  If %OPT_E is given, fitted values from the first-stage
+ * regressions are saved with the model.
  * 
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt)
+MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
+	    gretlopt opt)
 {
+    MODEL tsls;
     gretl_matrix *Q = NULL;
     char *missmask = NULL;
     int orig_t1 = pdinfo->t1, orig_t2 = pdinfo->t2;
     int *reglist = NULL, *instlist = NULL;
     int *hatlist = NULL, *s2list = NULL;
-    int *exolist = NULL;
+    int *exolist = NULL, *endolist = NULL;
     int *droplist = NULL;
     int addconst = 0;
-    int pos, orig_nvar = pdinfo->v;
-    int rlen, ninst, ev = 0;
+    int orig_nvar = pdinfo->v;
+    int sysest = (opt & OPT_E);
+    int nendo, ev = 0;
     int OverIdRank = 0;
-    int i;
+    int i, err = 0;
 
-    MODEL tsls;
+#if TDEBUG
+    printlist(list, "tsls: incoming list");
+#endif
 
     /* initialize model in case we bail out early on error */
     gretl_model_init(&tsls);
 
     gretl_error_clear();
 
-    pos = gretl_list_separator_position(list);
-    if (pos == 0) {
-	tsls.errcode = E_PARSE;
-	return tsls;
-    }
-
-    /* number of elements in regression list (vars preceding the
-       separator at "pos") */
-    rlen = pos - 1;
-
-    /* number of instruments (vars following the separator at
-       "pos") */
-    ninst = list[0] - pos;
-
-    if (rlen < 2 || ninst <= 0) {
-	tsls.errcode = E_ARGS;
-	return tsls;
-    }
-
-    /* adjust sample range for missing observations */
-    tsls.errcode = tsls_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
-				      (const double **) *pZ, &missmask); 
+    /* reglist: dependent var plus list of regressors
+       instlist: list of instruments
+    */
+    tsls.errcode = ivreg_process_lists(list, &reglist, &instlist);
     if (tsls.errcode) {
 	return tsls;
     }
 
-    /* 
-       reglist: dependent var plus list of regressors
-       instlist: list of instruments
-       s2list: regression list for second-stage regression
-       hatlist: list of vars to be replaced by fitted values
-                for the second-stage regression
-    */
-    reglist = gretl_list_new(rlen);
-    instlist = gretl_list_new(ninst + 1); /* allow for adding const */
-    s2list = gretl_list_new(rlen);
-    hatlist = gretl_list_new(rlen);
+    /* adjust sample range for missing observations */
+    err = tsls_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
+			     (const double **) *pZ, &missmask); 
 
-    if (reglist == NULL || instlist == NULL || 
-	s2list == NULL || hatlist == NULL) {
-	tsls.errcode = E_ALLOC;
-	goto bailout;
+    if (!err) {
+	/* allocate second stage regression list */
+	s2list = gretl_list_copy(reglist);
+	if (s2list == NULL) {
+	    err = E_ALLOC;
+	}
     }
 
-    /* dependent variable plus regressors: first portion of input
-       list */
-    for (i=1; i<=rlen; i++) {
-	reglist[i] = list[i];
-    }    
-
-    /* list of instruments: second portion of input list */
-    instlist[0] = ninst;
-    for (i=1; i<=instlist[0]; i++) {
-	instlist[i] = list[i + pos];
-	if (instlist[i] == list[1]) {
-	    strcpy(gretl_errmsg, "You can't use the dependent variable as an instrument");
-	    tsls.errcode = E_DATA;
-	    goto bailout;
-	}
-    }	
+    if (err) {
+	goto bailout;
+    }  
 
     /* in case we drop any instruments as redundant */
-    if (ci == SYSTEM) {
+    if (sysest) {
 	exolist = gretl_list_copy(instlist);
     }
 
@@ -1081,88 +1414,112 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     reglist_check_for_const(reglist, (const double **) *pZ, pdinfo);
     tsls_omitzero(instlist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
 
-    /* determine the list of variables (hatlist) for which we need to
-       obtain fitted values in the first stage 
+    /* Determine the list of variables (endolist) for which we need to
+       obtain fitted values in the first stage.  Note that we accept
+       the condition where there are no such variables (and hence
+       TSLS is in fact redundant); this may sometimes be required
+       when tsls is called in the context of system estimation.
     */
-    addconst = tsls_make_hatlist(reglist, instlist, hatlist);
+    endolist = tsls_make_endolist(reglist, &instlist, &addconst, &err);
+    if (err) {
+	goto bailout;
+    }
 
-    Q = tsls_Q(instlist, reglist, &droplist,
-	       (const double **) *pZ, pdinfo->t1, pdinfo->t2,
-	       missmask, &tsls.errcode);
-    if (tsls.errcode) {
+    if (endolist != NULL) {
+	nendo = endolist[0];
+	hatlist = gretl_list_copy(endolist);
+	if (hatlist == NULL) {
+	    err = E_ALLOC;
+	}
+    } else {
+	/* no endogenous regressors */
+	nendo = 0;
+    }
+
+    if (!err) {
+	Q = tsls_Q(instlist, reglist, &droplist,
+		   (const double **) *pZ, pdinfo->t1, pdinfo->t2,
+		   missmask, &err);
+    }
+
+    if (err) {
 	goto bailout;
     } 
 
-    /* check for order condition */
+    /* check (again) for order condition: we do this after tsls_Q in case
+       any vars get dropped at that stage
+    */
     OverIdRank = instlist[0] - reglist[0] + 1;
     if (OverIdRank < 0) {
         sprintf(gretl_errmsg, 
 		_("The order condition for identification is not satisfied.\n"
 		  "At least %d more instruments are needed."), -OverIdRank);
-	tsls.errcode = E_UNSPEC; 
+	err = E_DATA; 
 	goto bailout;
     }
 
-    /* hatlist[0] holds the number of fitted vars to create */
-    if (dataset_add_series(hatlist[0], pZ, pdinfo)) {
-	tsls.errcode = E_ALLOC;
-	goto bailout;
+    /* allocate storage for fitted vars, if needed */
+    if (nendo > 0) {
+	err = dataset_add_series(nendo, pZ, pdinfo);
+	if (err) {
+	    goto bailout;
+	}
     } 
 
-    /* initial composition of second-stage regression list (will be
-       modified in the loop below) 
-    */
-    for (i=0; i<=reglist[0]; i++) {
-	s2list[i] = reglist[i];
-    }
+    /* prepare for first-stage F-test, if just one endogenous regressor */
+    if (nendo == 1) {
+	ev = endolist[1];
+    }      
 
     /* 
        Deal with the variables for which instruments are needed: loop
-       across the list of variables to be instrumented (hatlist),
+       across the list of variables to be instrumented (endolist),
        form the fitted values as QQ'x_i, and add these fitted values
-       into the data matrix Z.
+       into the data array Z.
     */
 
-    /* prepare for first-stage F-test, if only one endogenous regressor */
-    if (hatlist[0] == 1) {
-	ev = hatlist[1];
-    }    
-
-    for (i=1; i<=hatlist[0]; i++) {
-	int orig = hatlist[i];
-	int newv = orig_nvar + i - 1;
+    for (i=0; i<nendo; i++) {
+	int v0 = endolist[i+1];
+	int v1 = orig_nvar + i;
 
 	/* form xhat_i = QQ'x_i */
-	tsls.errcode = tsls_form_xhat(Q, (*pZ)[orig], (*pZ)[newv], pdinfo, 
-				      missmask);
-	if (tsls.errcode) {
+	err = tsls_form_xhat(Q, (*pZ)[v0], (*pZ)[v1], pdinfo, 
+			     missmask);
+	if (err) {
 	    goto bailout;
 	}
 
 	/* give the fitted series the same name as the original */
-	strcpy(pdinfo->varname[newv], pdinfo->varname[orig]);
+	strcpy(pdinfo->varname[v1], pdinfo->varname[v0]);
 
-	/* substitute newv into the right place in the second-stage
+	/* substitute v1 into the right place in the second-stage
 	   regression list */
-	replace_list_element(s2list, orig, newv);
+	replace_list_element(s2list, v0, v1);
 
 	/* update hatlist */
-	hatlist[i] = newv;
+	hatlist[i+1] = v1;
     }
 
     /* second-stage regression */
-    tsls = lsq(s2list, pZ, pdinfo, OLS, (ci == TSLS)? OPT_NONE : OPT_Z);
+    tsls = lsq(s2list, pZ, pdinfo, OLS, (sysest)? OPT_Z : OPT_NONE);
     if (tsls.errcode) {
-	fprintf(stderr, "tsls_func, stage 2: tsls.errcode = %d\n", tsls.errcode);
+	fprintf(stderr, "tsls, stage 2: tsls.errcode = %d\n", tsls.errcode);
 	goto bailout;
     }
 
-    if (ci == TSLS && hatlist[0] == 1) {
-	compute_first_stage_F(&tsls, ev, reglist, instlist, pZ, pdinfo, opt);
+    if (!sysest || (opt & OPT_H)) {
+	if (nendo == 1) {
+	    /* handles robust estimation, for single endogenous regressor */
+	    compute_first_stage_F(&tsls, ev, reglist, instlist, pZ, pdinfo, opt);
+	} else if (!(opt & OPT_R)) {
+	    /* at present, only handles case of i.i.d. errors */
+	    compute_stock_yogo(&tsls, endolist, instlist, hatlist,
+			       (const double **) *pZ);
+	}
     } 
 
     if (tsls.list[0] < s2list[0]) {
-	/* collinear regressors dropped? If so, adjustments are needed */
+	/* Were collinear regressors dropped? If so, adjustments are needed */
 	OverIdRank += s2list[0] - tsls.list[0];
 	reglist_remove_redundant_vars(&tsls, s2list, reglist);
     }
@@ -1171,9 +1528,8 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
        residuals and associated statistics */
     tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
 
-    /* compute standard errors */
-    if ((opt & OPT_R) || libset_get_bool(USE_QR)) {
-	/* QR decomp in force, or robust standard errors called for */
+    if (opt & OPT_R) {
+	/* robust standard errors called for */
 	qr_tsls_vcv(&tsls, (const double **) *pZ, pdinfo, opt);
 	if (tsls.errcode) {
 	    goto bailout;
@@ -1181,17 +1537,21 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* compute additional statistics (R^2, F, etc.) */
-    tsls_extra_stats(&tsls, (const double **) *pZ);
+    tsls_extra_stats(&tsls, OverIdRank, (const double **) *pZ, pdinfo);
 
-    if (ci == TSLS) {
-	tsls_hausman_test(&tsls, reglist, hatlist, pZ, pdinfo);
+    if (!sysest) {
+	if (hatlist != NULL) {
+	    tsls_hausman_test(&tsls, reglist, hatlist, pZ, pdinfo);
+	}
 	if (OverIdRank > 0) {
-	    tsls_sargan_test(&tsls, OverIdRank, instlist, pZ, pdinfo);
+	    ivreg_sargan_test(&tsls, OverIdRank, instlist, pZ, pdinfo);
+	} else {
+	    tsls_loglik(&tsls, nendo, reglist, instlist, pZ, pdinfo);
 	}
     }
 
     /* set command code on the model */
-    tsls.ci = TSLS;
+    tsls.ci = IVREG;
 
     /* write the full 2SLS list (dep. var. and regressors, followed by
        instruments, possibly purged of redundant elements) into the model
@@ -1214,21 +1574,33 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     gretl_matrix_free(Q);
     free(missmask);
 
-    /* save first-stage fitted values, if wanted */
-    if (ci == SYSTEM && tsls.errcode == 0) {
-	tsls_save_data(&tsls, hatlist, exolist, *pZ, pdinfo);
+    if (err && !tsls.errcode) {
+	tsls.errcode = err;
     }
 
-    /* delete first-stage fitted values from dataset */
+    if (!tsls.errcode && endolist != NULL) {
+	if (sysest) {
+	    /* save first-stage fitted values */
+	    tsls_save_data(&tsls, hatlist, exolist, *pZ, pdinfo);
+	}
+	if (!sysest || (opt & OPT_H)) {
+	    /* save list of endogenous regressors on model */
+	    gretl_model_set_list_as_data(&tsls, "endolist", endolist);
+	    endolist = NULL; /* model takes ownership */
+	}
+    } 
+
+    /* delete any first-stage fitted values from dataset */
     dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
 
-    free(hatlist);
     free(reglist); 
     free(instlist);
+    free(hatlist);
     free(exolist);
+    free(endolist);
     free(s2list);
 
-    if (tsls.errcode) {
+    if ((opt & OPT_A) || tsls.errcode) {
 	model_count_minus();
     }
 
