@@ -108,6 +108,16 @@ static int matrix_is_user_matrix (const gretl_matrix *m)
 
 static void (*matrix_add_callback)(void);
 
+/**
+ * set_matrix_add_callback:
+ * @callback: function function to out in place.
+ *
+ * Sets the callback function to be invoked when a user-defined
+ * matrix is added to the stack of saved objects.  Intended
+ * for synchronizing the GUI program with the saved object
+ * state.
+ */
+
 void set_matrix_add_callback (void (*callback))
 {
     matrix_add_callback = callback; 
@@ -122,16 +132,6 @@ static int real_user_matrix_add (gretl_matrix *M, const char *name,
 	return 0;
     }
 
-    if (gretl_matrix_xna_check(M)) {
-	return E_NAN;
-    }
-
-#if 0
-    if (check_varname(name)) {
-	return E_DATA;
-    }
-#endif
-
     tmp = realloc(matrices, (n_matrices + 1) * sizeof *tmp);
     if (tmp == NULL) {
 	return E_ALLOC;
@@ -200,21 +200,19 @@ matrix_insert_diagonal (gretl_matrix *M, const gretl_matrix *S,
     return 0;
 }
 
-/* If slevel is LEVEL_AUTO, search at the current function execution
+/* If level is LEVEL_AUTO, search at the current function execution
    depth, otherwise search at the function execution depth given by
    slevel.
 */
 
-static gretl_matrix *
-real_get_matrix_by_name (const char *name, int slevel)
+static gretl_matrix *real_get_matrix_by_name (const char *name, 
+					      int level)
 {
-    int level, i;
+    int i;
 
-    if (slevel == LEVEL_AUTO) {
+    if (level == LEVEL_AUTO) {
 	level = gretl_function_depth();
-    } else {
-	level = slevel;
-    }
+    } 
 
     for (i=0; i<n_matrices; i++) {
 	if (matrices[i]->level == level && 
@@ -228,8 +226,7 @@ real_get_matrix_by_name (const char *name, int slevel)
 
 user_matrix *get_user_matrix_by_name (const char *name)
 {
-    int level = gretl_function_depth();
-    int i;
+    int i, level = gretl_function_depth();
 
     for (i=0; i<n_matrices; i++) {
 	if (matrices[i]->level == level && 
@@ -256,10 +253,6 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 	return E_UNKVAR;
     }
 
-    if (gretl_matrix_xna_check(M)) {
-	return E_NAN;
-    }
-
     if (M != u->M) {
 #if MDEBUG
 	fprintf(stderr, " freeing u->M at %p, replacing with "
@@ -347,7 +340,38 @@ static user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
 
 gretl_matrix *get_matrix_by_name (const char *name)
 {
-    return real_get_matrix_by_name(name, LEVEL_AUTO);
+    if (name == NULL || *name == '\0') {
+	return NULL;
+    } else {
+	return real_get_matrix_by_name(name, LEVEL_AUTO);
+    }
+}
+
+/**
+ * steal_matrix_by_name:
+ * @name: name of the matrix.
+ *
+ * Looks up a user-defined matrix by namem and if found,
+ * grabs the matrix, leaving the matrix pointer on the
+ * named matrix as %NULL.
+ *
+ * Returns: pointer to matrix, or %NULL if not found.
+ */
+
+gretl_matrix *steal_matrix_by_name (const char *name)
+{
+    gretl_matrix *ret = NULL;
+
+    if (name != NULL && *name != '\0') {
+	user_matrix *u = get_user_matrix_by_name(name);
+
+	if (u != NULL) {
+	    ret = u->M;
+	    u->M = NULL;
+	}
+    }
+
+    return ret;
 }
 
 /**
@@ -417,18 +441,18 @@ gretl_matrix *user_matrix_get_matrix (user_matrix *u)
 /**
  * copy_named_matrix_as:
  * @orig: the name of the original matrix.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * If a saved matrix is found by the name @orig, a copy of
  * this matrix is added to the stack of saved matrices under the
- * name @new.  This is intended for use when a matrix is given
+ * name @newname.  This is intended for use when a matrix is given
  * as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_named_matrix_as (const char *orig, const char *new)
+int copy_named_matrix_as (const char *orig, const char *newname)
 {
     user_matrix *u;
     int err = 0;
@@ -442,7 +466,7 @@ int copy_named_matrix_as (const char *orig, const char *new)
 	if (M == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    err = user_matrix_add(M, new);
+	    err = user_matrix_add(M, newname);
 	}
 	if (!err) {
 	    /* for use in functions: increment level of last-added matrix */
@@ -457,17 +481,17 @@ int copy_named_matrix_as (const char *orig, const char *new)
 /**
  * copy_matrix_as:
  * @m: the original matrix.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * A copy of matrix @m is added to the stack of saved matrices
- * under the name @new.  This is intended for use when a matrix is given
+ * under the name @newname.  This is intended for use when a matrix is given
  * is given as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_matrix_as (const gretl_matrix *m, const char *new)
+int copy_matrix_as (const gretl_matrix *m, const char *newname)
 {
     gretl_matrix *m2 = gretl_matrix_copy(m);
     int err;
@@ -475,7 +499,7 @@ int copy_matrix_as (const gretl_matrix *m, const char *new)
     if (m2 == NULL) {
 	err = E_ALLOC;
     } else {
-	err = matrix_copy_add(m2, new);
+	err = matrix_copy_add(m2, newname);
     }
 
     if (!err) {
@@ -507,6 +531,21 @@ static int msel_out_of_bounds (int *range, int n)
     }
 }
 
+static int vec_is_const (const gretl_matrix *m, int n)
+{
+    int i;
+
+    for (i=1; i<n; i++) {
+	if (m->val[i] != m->val[i-1]) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+/* convert matrix subspec into list of rows or columns */
+
 static int *mspec_to_list (int type, union msel *sel, int n,
 			   int *err)
 {
@@ -522,17 +561,33 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 	    sel->range[1] = n;
 	}
 	if (msel_out_of_bounds(sel->range, n)) {
-	    *err = 1;
-	    return NULL;
+	    *err = E_DATA;
+	} else {
+	    ns = sel->range[1] - sel->range[0] + 1;
+	    if (ns <= 0) {
+		sprintf(gretl_errmsg, _("Range %d to %d is non-positive!"),
+			sel->range[0], sel->range[1]); 
+		*err = E_DATA;
+	    }
 	}
-	ns = sel->range[1] - sel->range[0] + 1;
     } else {
-	ns = gretl_vector_get_length(sel->m);
+	/* SEL_MATRIX */
+	if (sel->m == NULL) {
+	    strcpy(gretl_errmsg, _("Range is non-positive!"));
+	    *err = E_DATA;
+	} else {
+	    ns = gretl_vector_get_length(sel->m);
+	    if (ns == 0) {
+		fprintf(stderr, "selection matrix is %d x %d\n", 
+			sel->m->rows, sel->m->cols);
+		*err = E_NONCONF;
+	    } else if (vec_is_const(sel->m, ns)) {
+		ns = 1;
+	    }
+	}
     }
 
-    if (ns <= 0) {
-	strcpy(gretl_errmsg, _("Range is non-positive!")); 
-	*err = 1;
+    if (*err) {
 	return NULL;
     }
 
@@ -599,6 +654,11 @@ static int get_slices (matrix_subspec *spec,
     return err;
 }
 
+/* @M is the target for partial replacement, @S is the source to
+   substitute, and @spec tells how/where to make the
+   substitution.
+*/
+
 static int matrix_replace_submatrix (gretl_matrix *M,
 				     const gretl_matrix *S,
 				     matrix_subspec *spec)
@@ -612,7 +672,15 @@ static int matrix_replace_submatrix (gretl_matrix *M,
     int sscalar = 0;
     int err = 0;
 
+    if (spec == NULL) {
+	fprintf(stderr, "matrix_replace_submatrix: spec is NULL!\n");
+	return E_DATA;
+    }
+
     if (sr > mr || sc > mc) {
+	/* the replacement matrix won't fit into M */
+	fprintf(stderr, "matrix_replace_submatrix: target is %d x %d but "
+		"replacement part is %d x %d\n", mr, mc, sr, sc);
 	return E_NONCONF;
     }
 
@@ -620,24 +688,30 @@ static int matrix_replace_submatrix (gretl_matrix *M,
 	return matrix_insert_diagonal(M, S, mr, mc);
     }
 
+    /* parse mspec into lists of affected rows and columns */
     err = get_slices(spec, &rslice, &cslice, M);
     if (err) {
 	return err;
     }
 
 #if MDEBUG
-    printlist(rslice, "rslice");
-    printlist(cslice, "cslice");
-    fprintf(stderr, "M = %d x %d, S = %d x %d\n", mr, mc, sr, sc);
+    printlist(rslice, "rslice (rows list)");
+    printlist(cslice, "cslice (cols list)");
+    fprintf(stderr, "orig M = %d x %d, S = %d x %d\n", mr, mc, sr, sc);
 #endif
 
     if (sr == 1 && sc == 1) {
+	/* selection matrix is a scalar */
 	sscalar = 1;
 	sr = (rslice == NULL)? mr : rslice[0];
 	sc = (cslice == NULL)? mc : cslice[0];
     } else if (rslice != NULL && rslice[0] != sr) {
+	fprintf(stderr, "mspec has %d rows but substitute matrix has %d\n", 
+		rslice[0], sr);
 	err = E_NONCONF;
     } else if (cslice != NULL && cslice[0] != sc) {
+	fprintf(stderr, "mspec has %d cols but substitute matrix has %d\n", 
+		cslice[0], sc);
 	err = E_NONCONF;
     }
 
@@ -748,12 +822,18 @@ gretl_matrix *user_matrix_get_submatrix (const char *name,
     return S;
 }
 
-int user_matrix_replace_submatrix (const char *name, gretl_matrix *S,
+/* Look up the existing matrix called @name, and substitute
+   the matrix @S for part of the original, as specified by
+   @spec.
+*/
+
+int user_matrix_replace_submatrix (const char *mname, 
+				   const gretl_matrix *S,
 				   matrix_subspec *spec)
 {
     gretl_matrix *M;
 
-    M = real_get_matrix_by_name(name, LEVEL_AUTO); 
+    M = real_get_matrix_by_name(mname, LEVEL_AUTO); 
     if (M == NULL) {
 	return E_UNKVAR;
     }
@@ -1123,6 +1203,8 @@ gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f,
 	    gretl_matrix_demean_by_column(R);
 	} else if (f == F_CHOL) {
 	    *err = gretl_matrix_cholesky_decomp(R);
+	} else if (f == F_PSDROOT) {
+	    *err = gretl_matrix_psd_root(R);
 	} else if (f == F_INVPD) {
 	    *err = gretl_invpd(R);
 	} else if (f == F_GINV) {
@@ -1153,12 +1235,6 @@ static void matrix_cannibalize (gretl_matrix *targ, gretl_matrix *src)
     free(targ->val);
     targ->val = src->val;
     src->val = NULL;
-
-#if USE_COLS
-    free(targ->col);
-    targ->col = src->col;
-    src->col = NULL;
-#endif
 }
 
 int matrix_invert_in_place (gretl_matrix *m)
@@ -1338,39 +1414,40 @@ real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q,
     return err;
 }
 
+#define nullarg(s) (s == NULL || !strcmp(s, "null"))
+
 gretl_matrix *
 user_matrix_QR_decomp (const gretl_matrix *m, const char *rname, int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
-    int wantR = 0;
+    gretl_matrix **pR = NULL;
 
     if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
 	return NULL;
     }
 
-    if (rname != NULL && strcmp(rname, "null")) {
-	wantR = 1;
+    if (!nullarg(rname)) {
 	if (get_matrix_by_name(rname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), rname);
 	    *err = E_UNKVAR;
+	} else {
+	    pR = &R;
 	}
     }
 
     if (!*err) {
-	*err = real_user_matrix_QR_decomp(m, &Q, (wantR)? &R : NULL);
+	*err = real_user_matrix_QR_decomp(m, &Q, pR);
     }
 
-    if (!*err && wantR) {
+    if (!*err && R != NULL) {
 	user_matrix_replace_matrix_by_name(rname, R);
     }
 
     return Q;
 }
 
-#define SVD_RESIZE 1
-
-#if SVD_RESIZE
 static int revise_SVD_V (gretl_matrix **pV, int r, int c)
 {
     gretl_matrix *V;
@@ -1394,7 +1471,6 @@ static int revise_SVD_V (gretl_matrix **pV, int r, int c)
 
     return 0;
 }
-#endif
 
 gretl_matrix *user_matrix_SVD (const gretl_matrix *m, 
 			       const char *uname, 
@@ -1412,16 +1488,18 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	return NULL;
     }
 
-    if (uname != NULL && strcmp(uname, "null")) {
+    if (!nullarg(uname)) {
 	if (get_matrix_by_name(uname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), uname);
 	    *err = E_UNKVAR;
 	} else {
 	    pU = &U;
 	}
     }
 
-    if (vname != NULL && strcmp(vname, "null")) {
+    if (!*err && !nullarg(vname)) {
 	if (get_matrix_by_name(vname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), vname);
 	    *err = E_UNKVAR;
 	} else {
 	    pV = &V;
@@ -1432,7 +1510,6 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	*err = gretl_matrix_SVD(m, pU, &S, pV);
     }
 
-#if SVD_RESIZE
     if (!*err && (U != NULL || V != NULL)) {
 	int tall = m->rows - m->cols;
 	int minrc = (m->rows > m->cols)? m->cols : m->rows;
@@ -1454,16 +1531,6 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	    }
 	}
     }
-#else
-    if (!*err) {
-	if (U != NULL) {
-	    user_matrix_replace_matrix_by_name(uname, U);
-	}
-	if (V != NULL) {
-	    user_matrix_replace_matrix_by_name(vname, V);
-	}
-    }
-#endif
 
     return S;
 }
@@ -1471,10 +1538,12 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
 			       const gretl_matrix *X, 
 			       const char *Uname, 
+			       gretlopt opt,
 			       int *err)
 {
     gretl_matrix *B = NULL;
     gretl_matrix *U = NULL;
+    int g, k, T;
     int newU = 0;
 
     if (gretl_is_null_matrix(X) || gretl_is_null_matrix(X)) {
@@ -1482,37 +1551,57 @@ gretl_matrix *user_matrix_ols (const gretl_matrix *Y,
 	return NULL;
     }
 
-    if (X->rows != Y->rows) {
+    T = Y->rows;
+    k = X->cols;
+    g = Y->cols;
+
+    if (X->rows != T) {
 	*err = E_NONCONF;
 	return NULL;
     }
 
-    if (Uname != NULL && strcmp(Uname, "null")) {
+    if (g > 1 && (opt & OPT_M)) {
+	/* multiple precision: only one y var wanted */
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (!nullarg(Uname)) {
 	U = get_matrix_by_name(Uname);
 	if (U == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), Uname);
 	    *err = E_UNKVAR;
 	    return NULL;
 	} 
-	if (U->rows != Y->rows || U->cols != Y->cols) {
+	if (U->rows != T || U->cols != g) {
 	    newU = 1;
 	}
     }
 
     if (newU) {
-	U = gretl_matrix_alloc(Y->rows, Y->cols);
+	U = gretl_matrix_alloc(T, g);
 	if (U == NULL) {
 	    *err = E_ALLOC;
 	    return NULL;
 	}
     }
 
-    B = gretl_matrix_alloc(X->cols, Y->cols);
+    B = gretl_matrix_alloc(k, g);
     if (B == NULL) {
 	*err = E_ALLOC;
     }
 
     if (!*err) {
-	*err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	if (g == 1) {
+	    if (opt & OPT_M) {
+		/* use multiple precision */
+		*err = gretl_matrix_mp_ols(Y, X, B, NULL, U, NULL);
+	    } else {
+		*err = gretl_matrix_ols(Y, X, B, NULL, U, NULL);
+	    }
+	} else {
+	    *err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	}
     }
 
     if (*err) {
@@ -1563,9 +1652,10 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 	return NULL;
     }
 
-    if (rname != NULL && strcmp(rname, "null")) {
+    if (!nullarg(rname)) {
 	vecs = 1;
 	if (get_matrix_by_name(rname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), rname);
 	    *err = E_UNKVAR;
 	    return NULL;
 	}
@@ -1624,7 +1714,7 @@ static void xml_put_user_matrix (user_matrix *u, FILE *fp)
 
     for (i=0; i<M->rows; i++) {
 	for (j=0; j<M->cols; j++) {
-	    fprintf(fp, "%.15g ", gretl_matrix_get(M, i, j));
+	    fprintf(fp, "%.16g ", gretl_matrix_get(M, i, j));
 	}
 	fputc('\n', fp);
     }
