@@ -44,6 +44,8 @@ struct matrix_subspec_ {
     union msel sel[2];
 };
 
+#define gretl_is_matrix(s) (get_matrix_by_name(s) != NULL)
+
 int n_user_matrices (void);
 
 const char *get_matrix_name_by_index (int idx);
@@ -52,6 +54,8 @@ gretl_matrix *get_matrix_by_name (const char *name);
 
 gretl_matrix *get_matrix_copy_by_name (const char *name, int *err);
 
+gretl_matrix *steal_matrix_by_name (const char *name);
+
 gretl_matrix *get_matrix_by_name_at_level (const char *name, int level);
 
 gretl_matrix *user_matrix_get_matrix (user_matrix *u);
@@ -79,14 +83,15 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M);
 int user_matrix_replace_matrix_by_name (const char *name, 
 					gretl_matrix *M);
 
-int user_matrix_replace_submatrix (const char *name, gretl_matrix *M,
+int user_matrix_replace_submatrix (const char *mname, 
+				   const gretl_matrix *S,
 				   matrix_subspec *spec);
 
 int add_or_replace_user_matrix (gretl_matrix *M, const char *name);
 
-int copy_named_matrix_as (const char *orig, const char *new);
+int copy_named_matrix_as (const char *orig, const char *newname);
 
-int copy_matrix_as (const gretl_matrix *m, const char *new);
+int copy_matrix_as (const gretl_matrix *m, const char *newname);
 
 int umatrix_set_colnames_from_string (const gretl_matrix *M, 
 				      const char *s);
@@ -125,6 +130,7 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
 			       const gretl_matrix *X, 
 			       const char *Uname, 
+			       gretlopt opt,
 			       int *err);
 
 gretl_matrix *
