@@ -175,9 +175,10 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->t1 = var->t2 = var->T = var->df = 0;
     var->ifc = var->ncoeff = 0;
     var->detflags = 0;
-    var->robust = var->qr = 0;
+    var->robust = 0;
     var->LBs = 0;
 
+    var->lags = NULL;
     var->ylist = NULL;
     var->xlist = NULL;
     var->rlist = NULL;
@@ -206,6 +207,8 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->name = NULL;
 }
 
+#define lag_wanted(v, i) (v->lags == NULL || in_gretl_list(v->lags, i))
+
 /* Construct the common X matrix (composed of lags of the core
    variables plus other terms if applicable)
 */
@@ -232,6 +235,9 @@ void VAR_fill_X (GRETL_VAR *v, int p, const double **Z,
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
 	for (j=1; j<=p; j++) {
+	    if (!lag_wanted(v, j)) {
+		continue;
+	    }
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
 		if (diff) {
@@ -257,7 +263,7 @@ void VAR_fill_X (GRETL_VAR *v, int p, const double **Z,
 	}
     }
 
-    /* add other deterministics */
+    /* add other deterministic terms */
 
     if (v->detflags & DET_SEAS) {
 	int per = get_subperiod(v->t1, pdinfo, NULL);
@@ -337,10 +343,9 @@ static void VAR_fill_Y (GRETL_VAR *v, int mod, const double **Z)
 #endif
 }
 
-/* split the user-supplied list, if need be, and construct
-   the lists of endogenous and (possibly) exogenous vars.
-   Deterministic terms are handled separately, via option
-   flags.
+/* Split the user-supplied list, if need be, and construct the lists
+   of endogenous and (possibly) exogenous vars.  Note that
+   deterministic terms are handled separately, via option flags.
 */
 
 static int VAR_make_lists (GRETL_VAR *v, const int *list,
@@ -358,7 +363,7 @@ static int VAR_make_lists (GRETL_VAR *v, const int *list,
 	}
     }
 
-    if (!err && (v->ylist == NULL || v->ylist[0] < 2)) {
+    if (!err && (v->ylist == NULL || v->ylist[0] < 1)) {
 	/* first test for at least 2 endog vars */
 	err = E_ARGS;
     }
@@ -385,7 +390,7 @@ static int VAR_make_lists (GRETL_VAR *v, const int *list,
 	}	
     }
 
-    if (!err && (v->ylist == NULL || v->ylist[0] < 2)) {
+    if (!err && (v->ylist == NULL || v->ylist[0] < 1)) {
 	/* re-test after (possibly) losing const */
 	err = E_ARGS;
     }
@@ -422,6 +427,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		miss = 1;
 	    }
 	}
+
 	if (v->xlist != NULL && !miss) {
 	    for (i=1; i<=v->xlist[0] && !miss; i++) {
 		if (na(Z[v->xlist[i]][t])) {
@@ -430,6 +436,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		}
 	    }
 	}
+
 	if (v->rlist != NULL && !miss) {
 	    for (i=1; i<=v->rlist[0] && !miss; i++) {
 		if (na(Z[v->rlist[i]][t])) {
@@ -438,6 +445,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		}
 	    }
 	}
+
 	if (!miss) {
 	    break;
 	}
@@ -454,6 +462,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		miss = 1;
 	    }
 	}
+
 	if (v->xlist != NULL && !miss) {
 	    for (i=1; i<=v->xlist[0] && !miss; i++) {
 		if (na(Z[v->xlist[i]][t])) {
@@ -462,6 +471,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		}
 	    }
 	}
+
 	if (v->rlist != NULL && !miss) {
 	    for (i=1; i<=v->rlist[0] && !miss; i++) {
 		if (na(Z[v->rlist[i]][t])) {
@@ -470,6 +480,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		}
 	    }
 	}
+
 	if (!miss) {
 	    break;
 	}
@@ -478,11 +489,13 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
     /* reject sample in case of internal missing values */
 
     for (t=v->t1; t<=v->t2 && !err; t++) {
+
 	for (i=1; i<=v->ylist[0] && !err; i++) {
 	    if (na(Z[v->ylist[i]][t])) {
 		err = E_MISSDATA;
 	    }
 	}
+
 	if (v->xlist != NULL && !err) {
 	    for (i=1; i<=v->xlist[0] && !err; i++) {
 		if (na(Z[v->xlist[i]][t])) {
@@ -490,6 +503,7 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 		}
 	    }
 	}
+
 	if (v->rlist != NULL && !err) {
 	    for (i=1; i<=v->rlist[0] && !err; i++) {
 		if (na(Z[v->rlist[i]][t])) {
@@ -509,10 +523,12 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 static int VAR_check_df_etc (GRETL_VAR *v, const DATAINFO *pdinfo,
 			     gretlopt opt)
 {
-    int err = 0;
+    int nl, err = 0;
 
     v->T = v->t2 - v->t1 + 1;
-    v->ncoeff = v->order * v->neqns;
+
+    nl = var_n_lags(v);
+    v->ncoeff = nl * v->neqns;
 
     if (v->xlist != NULL) {
 	/* user-specified exogenous vars */
@@ -550,7 +566,7 @@ static int VAR_check_df_etc (GRETL_VAR *v, const DATAINFO *pdinfo,
     return err;
 }
 
-/* Note: if "v" is a VECM, and it includes a restricted const
+/* Note: if @v is a VECM, and it includes a restricted const
    or trend, we make the Y and B matrices big enough to
    accommodate the additional restricted term.
 */
@@ -591,6 +607,7 @@ static int VAR_add_basic_matrices (GRETL_VAR *v, gretlopt opt)
    auxiliary tasks */
 
 static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
+				 const int *lags,
 				 const int *list,
 				 const double **Z,
 				 const DATAINFO *pdinfo,
@@ -617,8 +634,8 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
 
     var->ci = ci;
     var->order = order;
+    var->lags = gretl_list_copy(lags);
 
-    var->qr = libset_get_bool(USE_QR);
     if (ci == VAR && (opt & OPT_R)) {
 	var->robust = 1;
     }
@@ -636,7 +653,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
 	var->t2 = pdinfo->t2;
     }
 
-    /* FIXME below: some of these allocations are (still) redundant
+    /* FIXME below: some of these allocations are redundant
        for some uses of the VAR struct */
 
     if (!err) {
@@ -746,6 +763,7 @@ void gretl_VAR_free (GRETL_VAR *var)
 	return;
     }
 
+    free(var->lags);
     free(var->ylist);
     free(var->xlist);
     free(var->rlist);
@@ -913,6 +931,9 @@ VAR_add_forecast (GRETL_VAR *var, int t1, int t2,
 	    for (j=0; j<var->neqns; j++) {
 		vj = var->ylist[j+1];
 		for (lag=1; lag<=var->order; lag++) {
+		    if (!lag_wanted(var, lag)) {
+			continue;
+		    }
 		    xtid = NADBL;
 		    if (t < tdyn || s - lag < 0) {
 			/* pre-forecast value */
@@ -1043,6 +1064,7 @@ VECM_add_forecast (GRETL_VAR *var, int t1, int t2,
 	    for (j=0; j<var->neqns; j++) {
 		vj = var->ylist[j+1];
 		for (k=1; k<=order; k++) {
+		    /* FIXME gappy lags */
 		    if (t - k < 0) {
 			fti = NADBL;
 			break;
@@ -1071,7 +1093,9 @@ VECM_add_forecast (GRETL_VAR *var, int t1, int t2,
 		}
 	    }
 
-	    if (na(fti)) goto set_fcast;
+	    if (na(fti)) {
+		goto set_fcast;
+	    }
 
 	    /* exogenous vars, if present */
 	    for (j=0; j<nexo; j++) {
@@ -1085,7 +1109,9 @@ VECM_add_forecast (GRETL_VAR *var, int t1, int t2,
 		}
 	    }
 
-	    if (na(fti)) goto set_fcast;
+	    if (na(fti)) {
+		goto set_fcast;
+	    }
 
 	    /* seasonals, if present */
 	    for (j=0; j<nseas; j++) {
@@ -1151,7 +1177,7 @@ gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t1, int t2,
 			       gretlopt opt, int *err)
 {
     if (var->F != NULL) {
-	/* There's a forecast attached, but it may not be what we want */
+	/* there's a forecast attached, but it may not be what we want */
 	gretl_matrix_free(var->F);
 	var->F = NULL;
     }
@@ -1214,7 +1240,7 @@ int set_VAR_model_stats (GRETL_VAR *var, int i)
     for (t=0; t<var->T; t++) {
 	u = gretl_matrix_get(var->E, t, i);
 	SSR += u * u;
-	x = (var->ifc)? y[t] - pmod->ybar : y[t];
+	x = (var->ifc)? (y[t] - pmod->ybar) : y[t];
 	TSS += x * x;
 	pmod->uhat[t + pmod->t1] = u;
 	pmod->yhat[t + pmod->t1] = y[t] - u;
@@ -1224,9 +1250,10 @@ int set_VAR_model_stats (GRETL_VAR *var, int i)
     pmod->sigma = sqrt(SSR / pmod->dfd);
     pmod->tss = TSS;
     pmod->rsq = 1.0 - SSR / TSS;
+    pmod->adjrsq = 1.0 - (SSR / (pmod->dfd)) / (TSS / (pmod->nobs - 1));
     pmod->fstt = ((TSS - SSR) / pmod->dfn) / (SSR / pmod->dfd);
 
-    pmod->adjrsq = pmod->lnL = NADBL;
+    pmod->lnL = NADBL;
 
     VAR_dw_rho(pmod);
 
@@ -1300,22 +1327,22 @@ int gretl_VAR_get_variable_number (const GRETL_VAR *var, int k)
 
 int gretl_VAR_get_n_equations (const GRETL_VAR *var)
 {
-    return var->neqns;
+    return (var == NULL)? 0 : var->neqns;
 }
 
 int gretl_VAR_get_t1 (const GRETL_VAR *var)
 {
-    return var->t1;
+    return (var == NULL)? 0 : var->t1;
 }
 
 int gretl_VAR_get_t2 (const GRETL_VAR *var)
 {
-    return var->t2;
+    return (var == NULL)? 0 : var->t2;
 }
 
 const MODEL *gretl_VAR_get_model (const GRETL_VAR *var, int i)
 {
-    if (i < var->neqns) {
+    if (var != NULL && i < var->neqns) {
 	return var->models[i];
     } else {
 	return NULL;
@@ -1449,6 +1476,7 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
 gretl_matrix *
 gretl_VAR_get_impulse_response (GRETL_VAR *var, 
 				int targ, int shock, int periods,
+				double alpha,
 				const double **Z,
 				const DATAINFO *pdinfo)
 {
@@ -1463,7 +1491,7 @@ gretl_VAR_get_impulse_response (GRETL_VAR *var,
 	/* no data matrix provided: just return point estimate */
 	ret = point;
     } else if (point != NULL) {
-	full = irf_bootstrap(var, targ, shock, periods, Z, pdinfo);
+	full = irf_bootstrap(var, targ, shock, periods, alpha, Z, pdinfo);
 	if (full != NULL) {
 	    double p;
 
@@ -1675,7 +1703,7 @@ int var_max_order (const int *list, const DATAINFO *pdinfo)
 
 static int VAR_add_roots (GRETL_VAR *var)
 {
-    gretl_matrix *CompForm = NULL;
+    gretl_matrix *comp = NULL;
     int err = 0;
 
     if (var->A == NULL) {
@@ -1685,21 +1713,21 @@ static int VAR_add_roots (GRETL_VAR *var)
 
     var->L = NULL;
 
-    CompForm = gretl_matrix_copy(var->A);
-    if (CompForm == NULL) {
+    comp = gretl_matrix_copy(var->A);
+    if (comp == NULL) {
 	err = E_ALLOC;
     }
 
     /* save eigenvalues of companion form matrix */
     if (!err) {
-        var->L = gretl_general_matrix_eigenvals(CompForm, 0, &err);
+        var->L = gretl_general_matrix_eigenvals(comp, 0, &err);
 #if 0
 	gretl_matrix_print(var->A, "Companion form matrix");
 	gretl_matrix_print(var->L, "Eigenvalues");
 #endif
     }
 
-    gretl_matrix_free(CompForm);
+    gretl_matrix_free(comp);
 
     if (err) {
 	gretl_matrix_free(var->L);
@@ -1738,7 +1766,7 @@ double gretl_VAR_ldet (GRETL_VAR *var, int *err)
 	gretl_matrix_multiply_mod(var->F, GRETL_MOD_TRANSPOSE,
 				  var->F, GRETL_MOD_NONE,
 				  S, GRETL_MOD_NONE);
-	gretl_matrix_divide_by_scalar(S, var->T);
+	gretl_matrix_divide_by_scalar(S, var->T); /* or var->df? */
 	ldet = gretl_vcv_log_determinant(S);
 	if (na(ldet)) {
 	    *err = 1;
@@ -1749,7 +1777,7 @@ double gretl_VAR_ldet (GRETL_VAR *var, int *err)
     return ldet;
 }
 
-static int VAR_add_variance_matrix (GRETL_VAR *var)
+static int VAR_add_stats (GRETL_VAR *var)
 {
     int err = 0;
 
@@ -1762,23 +1790,15 @@ static int VAR_add_variance_matrix (GRETL_VAR *var)
 	gretl_matrix_multiply_mod(var->E, GRETL_MOD_TRANSPOSE,
 				  var->E, GRETL_MOD_NONE,
 				  var->S, GRETL_MOD_NONE);
-	gretl_matrix_divide_by_scalar(var->S, var->T);
+	gretl_matrix_divide_by_scalar(var->S, var->T); /* OK, for ldet? */
     }
 
-    return err;
-}
-
-static int VAR_add_stats (GRETL_VAR *var)
-{
-    int err;
-
-    err = VAR_add_variance_matrix(var);
-
     if (!err) {
 	var->ldet = gretl_vcv_log_determinant(var->S);
 	if (na(var->ldet)) {
 	    err = 1;
 	}
+	gretl_matrix_multiply_by_scalar(var->S, var->T / var->df);
     }    
 
     if (!err) {
@@ -1816,6 +1836,9 @@ void gretl_VAR_param_names (GRETL_VAR *v, char **params,
 
     for (i=1; i<=v->ylist[0]; i++) {
 	for (j=1; j<=v->order; j++) {
+	    if (!lag_wanted(v, j)) {
+		continue;
+	    }
 	    sprintf(lagstr, "_%d", j);
 	    n = strlen(lagstr);
 	    if (v->ci == VECM) {
@@ -1856,14 +1879,19 @@ void gretl_VAR_param_names (GRETL_VAR *v, char **params,
 
 void VAR_write_A_matrix (GRETL_VAR *v)
 {
-    int i, j, k, lag;
+    int i, ii, j, k, lag;
     int dim = v->neqns * v->order;
     double bij;
 
     for (j=0; j<v->neqns; j++) {
-	k = lag = 0;
+	k = lag = ii = 0;
 	for (i=0; i<dim; i++) {
-	    bij = gretl_matrix_get(v->B, i + v->ifc, j);
+	    if (lag_wanted(v, lag+1)) {
+		bij = gretl_matrix_get(v->B, ii + v->ifc, j);
+		ii++;
+	    } else {
+		bij = 0;
+	    }
 	    gretl_matrix_set(v->A, j, v->neqns * lag + k, bij);
 	    if (lag < v->order - 1) {
 		lag++;
@@ -1873,6 +1901,10 @@ void VAR_write_A_matrix (GRETL_VAR *v)
 	    }
 	}
     }
+
+#if 0
+    gretl_matrix_print(v->A, "v->A");
+#endif
 }
 
 static void VAR_write_vcv_matrix (GRETL_VAR *v)
@@ -1997,6 +2029,43 @@ int transcribe_VAR_models (GRETL_VAR *var,
     return err;
 }
 
+/* If OPT_S (for Specific lags) was given, try to retrieve a list of
+   lags (in string form) from the gretl option mechanism.  If we don't
+   find any such string, that's an error.  If we do, convert the
+   string to a numeric list and sort it.
+*/
+
+static int *maybe_get_lags_list (int *list, int order, int *err)
+{
+    int *lags = NULL;
+    const char *s;
+
+    s = get_optval_string(VAR, OPT_S);
+
+    if (s == NULL) {
+	*err = E_ARGS;
+	return NULL;
+    }
+
+    lags = gretl_list_from_string(s, err);
+    
+    if (lags != NULL) {
+	int i;
+
+	gretl_list_sort(lags);
+	for (i=lags[0]; i>1; i--) {
+	    if (lags[i] > order) {
+		gretl_list_delete_at_pos(lags, i);
+	    }
+	}
+	if (lags[0] == 0) {
+	    *err = E_DATA;
+	}
+    }
+
+    return lags;
+}
+
 /**
  * gretl_VAR:
  * @order: lag order for the VAR
@@ -2011,6 +2080,7 @@ int transcribe_VAR_models (GRETL_VAR *var,
  *       if includes %OPT_Q, do not show individual regressions.
  *       if includes %OPT_T, include a linear trend.
  *       if includes %OPT_L, test for optimal lag length (only).
+ *       if includes %OPT_S, look for a specific set of lags
  * @prn: gretl printing struct.
  * @errp: location to receive error code.
  *
@@ -2023,53 +2093,52 @@ int transcribe_VAR_models (GRETL_VAR *var,
 GRETL_VAR *gretl_VAR (int order, int *list, 
 		      const double **Z, const DATAINFO *pdinfo,
 		      gretlopt opt, PRN *prn, 
-		      int *errp)
+		      int *err)
 {
     GRETL_VAR *var = NULL;
     int code = (opt & OPT_L)? VAR_LAGSEL : VAR_ESTIMATE;
-    int err = 0;
+    int *lags = NULL;
 
-    var = gretl_VAR_new(code, order, 0, list, Z, pdinfo, 
-			opt, &err);
+    if (opt & OPT_S) {
+	lags = maybe_get_lags_list(list, order, err);
+	if (*err) {
+	    return NULL;
+	}
+    }
+
+    var = gretl_VAR_new(code, order, 0, lags, list, Z, pdinfo, 
+			opt, err);
     if (var == NULL) {
 	return NULL;
     }
 
-    /* run the regressions: use QR or Cholesky */
-    if (var->qr) {
-	err = gretl_matrix_QR_ols(var->Y, var->X, 
+    /* run the regressions, using Cholesky or QR */
+    *err = gretl_matrix_multi_ols(var->Y, var->X, 
 				  var->B, var->E,
-				  &var->XTX, NULL);
-    } else {
-	err = gretl_matrix_multi_ols(var->Y, var->X, 
-				     var->B, var->E,
-				     &var->XTX);
-    }
+				  &var->XTX);
 
-    if (!err) {
+    if (!*err) {
 	if (code == VAR_LAGSEL) {
-	    err = VAR_add_stats(var);
-	    if (!err) {
-		err = VAR_do_lagsel(var, Z, pdinfo, prn);
+	    *err = VAR_add_stats(var);
+	    if (!*err) {
+		*err = VAR_do_lagsel(var, Z, pdinfo, prn);
 	    }
 	} else {
-	    err = transcribe_VAR_models(var, Z, pdinfo, NULL);
-	    if (!err) {
-		err = VAR_finalize(var);
+	    *err = transcribe_VAR_models(var, Z, pdinfo, NULL);
+	    if (!*err) {
+		*err = VAR_finalize(var);
 	    }
-	    if (!err) {
+	    if (!*err) {
 		gretl_VAR_print(var, pdinfo, opt, prn);
 	    }
 	}
     }
 
-    if (code == VAR_LAGSEL || (err && var != NULL)) {
+    if (code == VAR_LAGSEL || (*err && var != NULL)) {
 	gretl_VAR_free(var);
 	var = NULL;
     }
 
-    *errp = err;
-
     return var;
 }
 
@@ -2280,13 +2349,8 @@ int johansen_stage_1 (GRETL_VAR *jvar, const double **Z,
     err = gretl_matrix_multi_SVD_ols(jvar->Y, jvar->X, jvar->B, 
 				     jvar->jinfo->R0, NULL);
 #else
-    if (jvar->qr) {
-	err = gretl_matrix_QR_ols(jvar->Y, jvar->X, jvar->B, 
-				  jvar->jinfo->R0, NULL, NULL);
-    } else {
-	err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
-				     jvar->jinfo->R0, NULL);
-    }
+    err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
+				 jvar->jinfo->R0, NULL);
 #endif
 
     if (nr > 0) {
@@ -2305,13 +2369,8 @@ int johansen_stage_1 (GRETL_VAR *jvar, const double **Z,
     err = gretl_matrix_multi_SVD_ols(jvar->Y, jvar->X, jvar->B, 
 				     jvar->jinfo->R1, NULL);
 #else
-    if (jvar->qr) {
-	err = gretl_matrix_QR_ols(jvar->Y, jvar->X, jvar->B, 
-				  jvar->jinfo->R1, NULL, NULL);
-    } else {
-	err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
-				     jvar->jinfo->R1, NULL);
-    } 
+    err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
+				 jvar->jinfo->R1, NULL);
 #endif 
 
     if (nr > 0) {
@@ -2488,8 +2547,8 @@ johansen_driver (GRETL_VAR *jvar,
 	jvar->err = jvar_check_allocation(jvar, pdinfo);
     }
 
-    /* Now get the johansen plugin to finish the job */
     if (!jvar->err) {
+	/* call the johansen plugin to finish the job */
 	if (r > 0) {
 	    jvar->err = johansen_estimate_complete(jvar, rset, Z, pdinfo, 
 						   prn);
@@ -2503,14 +2562,14 @@ johansen_driver (GRETL_VAR *jvar,
 
 static GRETL_VAR *
 johansen_wrapper (int code, int order, int rank, 
-		  const int *list, 
+		  const int *lags, const int *list, 
 		  gretl_restriction *rset, 
 		  const double **Z, const DATAINFO *pdinfo, 
 		  gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *jvar;
 
-    jvar = gretl_VAR_new(code, order - 1, rank, list, Z, pdinfo,
+    jvar = gretl_VAR_new(code, order - 1, rank, lags, list, Z, pdinfo,
 			 opt, err);
     if (jvar == NULL) {
 	return NULL;
@@ -2548,7 +2607,7 @@ GRETL_VAR *johansen_test (int order, const int *list,
 {
     int err = 0;
 
-    return johansen_wrapper(VECM_CTEST, order, 0, list, NULL, 
+    return johansen_wrapper(VECM_CTEST, order, 0, NULL, list, NULL, 
 			    Z, pdinfo, opt, prn, &err);
 }
 
@@ -2579,7 +2638,7 @@ int johansen_test_simple (int order, const int *list,
     GRETL_VAR *jvar = NULL;
     int err = 0;
 
-    jvar = johansen_wrapper(VECM_CTEST, order, 0, list, NULL, 
+    jvar = johansen_wrapper(VECM_CTEST, order, 0, NULL, list, NULL, 
 			    Z, pdinfo, opt, prn, &err);
 
     if (jvar != NULL) {
@@ -2591,17 +2650,18 @@ int johansen_test_simple (int order, const int *list,
 
 /**
  * gretl_VECM:
- * @order: lag order for test.
- * @rank: pre-specified cointegration rank.
- * @list: list of variables to test for cointegration.
+ * @order: lag order.
+ * @rank: cointegration rank.
+ * @list: list of endogenous variables, possibly plus
+ * exogenous variables.
  * @Z: pointer to data array.
  * @pdinfo: dataset information.
  * @opt:
  * @prn: gretl printing struct.
  * @err: location to receive error code.
  *
- * Returns: pointer to struct containing information on 
- * the VECM system or %NULL on failure.
+ * Returns: pointer to struct containing a VECM system,
+ * or %NULL on failure.
  */
 
 GRETL_VAR *gretl_VECM (int order, int rank, int *list, 
@@ -2609,6 +2669,7 @@ GRETL_VAR *gretl_VECM (int order, int rank, int *list,
 		       gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *jvar = NULL;
+    int *lags = NULL;
 
     if (rank <= 0) {
 	sprintf(gretl_errmsg, _("vecm: rank %d is out of bounds"), rank);
@@ -2616,8 +2677,15 @@ GRETL_VAR *gretl_VECM (int order, int rank, int *list,
 	return NULL;
     }
 
-    jvar = johansen_wrapper(VECM_ESTIMATE, order, rank, list, NULL, 
-			    Z, pdinfo, opt, prn, err);
+    if (opt & OPT_S) {
+	lags = maybe_get_lags_list(list, order, err);
+	if (*err) {
+	    return NULL;
+	}
+    }
+
+    jvar = johansen_wrapper(VECM_ESTIMATE, order, rank, lags, list, 
+			    NULL, Z, pdinfo, opt, prn, err);
     
     if (jvar != NULL && !jvar->err) {
 	gretl_VAR_print(jvar, pdinfo, opt, prn);
@@ -2716,8 +2784,8 @@ real_gretl_restricted_vecm (GRETL_VAR *orig,
     }
 
     jvar = johansen_wrapper(VECM_ESTIMATE, orig->order + 1, 
-			    orig->jinfo->rank, list,
-			    rset, Z, pdinfo, 
+			    orig->jinfo->rank, orig->lags,
+			    list, rset, Z, pdinfo, 
 			    jopt, prn, err);
 
     if (jvar != NULL && !jvar->err) {
@@ -2808,6 +2876,7 @@ int gretl_VAR_do_irf (GRETL_VAR *var, const char *line,
 {
     int targ = -1, shock = 1;
     int h = 0, boot = 0;
+    double alpha = 0.05; /* FIXME */
     int err = 0;
     char *s;
 
@@ -2840,10 +2909,10 @@ int gretl_VAR_do_irf (GRETL_VAR *var, const char *line,
     if (targ >= 0 && shock >= 0 && h > 0) {
         if (boot) {
 	    err = gretl_VAR_plot_impulse_response(var, targ, shock, h,
-						  Z, pdinfo);
+						  alpha, Z, pdinfo);
 	} else {
 	    err = gretl_VAR_plot_impulse_response(var, targ, shock, h, 
-						  NULL, pdinfo);
+						  0, NULL, pdinfo);
 	}
     }
 
@@ -3040,15 +3109,19 @@ double *gretl_VAR_get_series (const GRETL_VAR *var, const DATAINFO *pdinfo,
     return x;    
 }
 
-static gretl_matrix *VAR_get_coeff_matrix (const GRETL_VAR *var,
-					   int *err)
+/* get coefficients or standard errors from the models
+   in the VAR */
+
+static gretl_matrix *
+VAR_matrix_from_models (const GRETL_VAR *var, int idx, int *err)
 {
-    gretl_matrix *B = NULL;
+    gretl_matrix *m = NULL;
     const MODEL *pmod;
+    double x;
     int i, j;
 
-    B = gretl_matrix_alloc(var->models[0]->ncoeff, var->neqns);
-    if (B == NULL) {
+    m = gretl_matrix_alloc(var->models[0]->ncoeff, var->neqns);
+    if (m == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
@@ -3056,11 +3129,16 @@ static gretl_matrix *VAR_get_coeff_matrix (const GRETL_VAR *var,
     for (j=0; j<var->neqns; j++) {
 	pmod = var->models[j];
 	for (i=0; i<pmod->ncoeff; i++) {
-	    gretl_matrix_set(B, i, j, pmod->coeff[i]);
+	    if (idx == M_COEFF) {
+		x = pmod->coeff[i];
+	    } else {
+		x = pmod->sderr[i];
+	    }
+	    gretl_matrix_set(m, i, j, x);
 	}
     }
 
-    return B;
+    return m;
 }
 
 #define vecm_matrix(i) (i == M_JALPHA || i == M_JBETA || \
@@ -3083,8 +3161,8 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	src = gretl_VAR_get_residual_matrix(var);
     } else if (idx == M_COMPAN) {
 	src = var->A;
-    } else if (idx == M_COEFF) {
-	M = VAR_get_coeff_matrix(var, err);
+    } else if (idx == M_COEFF || idx == M_SE) {
+	M = VAR_matrix_from_models(var, idx, err);
 	copy = 0;
     } else if (idx == M_VCV) {
 	src = var->vcv;
@@ -3310,7 +3388,8 @@ static int VAR_retrieve_jinfo (xmlNodePtr node, xmlDocPtr doc,
 }
 
 static int VAR_retrieve_equations (xmlNodePtr node, xmlDocPtr doc,
-				   MODEL **models, int neqns)
+				   MODEL **models, int neqns,
+				   const DATAINFO *pdinfo)
 {
     xmlNodePtr cur = node->xmlChildrenNode;
     int i = 0, err = 0;
@@ -3319,7 +3398,7 @@ static int VAR_retrieve_equations (xmlNodePtr node, xmlDocPtr doc,
 	MODEL *pmod;
 
 	if (!xmlStrcmp(cur->name, (XUC) "gretl-model")) {
-	    pmod = gretl_model_from_XML(cur, doc, &err);
+	    pmod = gretl_model_from_XML(cur, doc, pdinfo, &err);
 	    if (pmod != NULL) {
 		models[i++] = pmod;
 	    }
@@ -3445,7 +3524,9 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
     return err;
 }
 
-GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
+GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			       const DATAINFO *pdinfo,
+			       int *err)
 {
     GRETL_VAR *var;
     MODEL *pmod;
@@ -3496,7 +3577,9 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
     cur = node->xmlChildrenNode;
 
     while (cur != NULL && !*err) {
-	if (!xmlStrcmp(cur->name, (XUC) "ylist")) {
+	if (!xmlStrcmp(cur->name, (XUC) "lags")) {
+	    var->lags = gretl_xml_node_get_list(cur, doc, err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "ylist")) {
 	    var->ylist = gretl_xml_node_get_list(cur, doc, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "xlist")) {
 	    var->xlist = gretl_xml_node_get_list(cur, doc, err);
@@ -3507,7 +3590,7 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 	} else if (!xmlStrcmp(cur->name, (XUC) "Ivals")) {
 	    var->Ivals = gretl_xml_get_double_array(cur, doc, &n, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "equations")) {
-	    *err = VAR_retrieve_equations(cur, doc, var->models, var->neqns);
+	    *err = VAR_retrieve_equations(cur, doc, var->models, var->neqns, pdinfo);
 	} else if (!xmlStrcmp(cur->name, (XUC) "gretl-johansen")) {
 	    *err = VAR_retrieve_jinfo(cur, doc, var);
 	} else if (!xmlStrcmp(cur->name, (XUC) "gretl-matrix")) {
@@ -3631,6 +3714,7 @@ int gretl_VAR_serialize (const GRETL_VAR *var, SavedObjectFlags flags,
 
     fputs(">\n", fp);
 
+    gretl_xml_put_tagged_list("lags", var->lags, fp);
     gretl_xml_put_tagged_list("ylist", var->ylist, fp);
     gretl_xml_put_tagged_list("xlist", var->xlist, fp);
     gretl_xml_put_tagged_list("rlist", var->rlist, fp);
