@@ -23,6 +23,12 @@
 #include "gretl_matrix.h"
 #include "gretl_restrict.h"
 
+enum Detflags {
+    DET_CONST = 1 << 0,
+    DET_TREND = 1 << 1,
+    DET_SEAS  = 1 << 2
+};
+
 typedef struct JohansenInfo_ JohansenInfo;
 
 struct GRETL_VAR_ {
@@ -30,19 +36,19 @@ struct GRETL_VAR_ {
     int refcount;        /* for saving/deleting */
     int err;             /* error code */
     int neqns;           /* number of equations in system */
-    int order;           /* lag order */
+    int order;           /* maximum lag order */
     int t1;              /* starting observation */
     int t2;              /* ending observation */
     int T;               /* number of observations */
     int df;              /* T - average coeffs per equation */
     int ifc;             /* equations include a constant (1) or not (0) */
     int ncoeff;          /* total coefficients per equation */
+    int *lags;           /* list of specific lags */
     int *ylist;          /* list of stochastic vars */
     int *xlist;          /* list of exogenous variables */
     int *rlist;          /* restricted exogenous variables (VECM only) */
     int detflags;        /* record of automatic deterministic vars added */
     int robust;          /* computing robust std errors? */
-    int qr;              /* using QR decomposition? */
     gretl_matrix *Y;     /* matrix of dependent variables */
     gretl_matrix *X;     /* matrix of independent variables */
     gretl_matrix *B;     /* basic coefficient matrix */
@@ -69,6 +75,9 @@ struct GRETL_VAR_ {
     char *name;          /* for use in session management */
 };
 
+#define var_n_lags(v) ((v->lags != NULL)? v->lags[0] : v->order)
+#define var_max_lag(v) ((v->lags != NULL)? v->lags[v->lags[0]] : v->order)
+
 int var_max_order (const int *list, const DATAINFO *pdinfo);
 
 GRETL_VAR *gretl_VAR (int order, int *list, 
@@ -124,6 +133,7 @@ gretl_matrix *
 gretl_VAR_get_impulse_response (GRETL_VAR *var, 
 				int targ, int shock,
 				int periods,
+				double alpha,
 				const double **Z,
 				const DATAINFO *pdinfo);
 
@@ -202,7 +212,9 @@ int transcribe_VAR_models (GRETL_VAR *var,
 
 #ifndef GRETLCLI
 
-GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
+GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			       const DATAINFO *pdinfo,
+			       int *err);
 
 #endif
 
