@@ -17,13 +17,13 @@
  * 
  */
 
-#ifndef GRETL_VERSION_H
-#define GRETL_VERSION_H
+#ifndef GRTL_VERSION_H
+#define GRTL_VERSION_H
 
 #define LIBGRETL_CURRENT   0
-#define LIBGRETL_REVISION  49
+#define LIBGRETL_REVISION  51
 #define LIBGRETL_AGE       0
 
-#define GRETL_VERSION "1.7.9"
+#define GRETL_VERSION "1.8.1"
 
-#endif
+#endif /* GRTL_VERSION_H */
