@@ -28,6 +28,8 @@
 #include "kalman.h"
 #include "matrix_extra.h"
 #include "gretl_scalar.h"
+#include "gretl_bfgs.h"
+
 #include "../cephes/libprob.h"
 
 #define ARMA_DEBUG 0
@@ -955,7 +957,7 @@ kalman_arma_score_callback (const double *b, int i, void *data)
     int err;
 
     rewrite_kalman_matrices(K, b, i);
-    err = kalman_forecast(K);
+    err = kalman_forecast(K, NULL);
 
 #if ARMA_DEBUG
     fprintf(stderr, "kalman_arma_score: kalman f'cast gave "
@@ -1068,7 +1070,7 @@ static double kalman_arma_ll (const double *b, void *p)
     K = (kalman *) p;
     err = rewrite_kalman_matrices(K, b, KALMAN_ALL);
     if (!err) {
-	err = kalman_forecast(K);
+	err = kalman_forecast(K, NULL);
 	ll = kalman_get_loglik(K);
     }
 
@@ -1157,7 +1159,8 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 	    err = 0;
 	    do_hess = 1;
 	} else {
-	    gretl_model_set_int(pmod, "ml_vcv", VCV_OP);
+	    gretl_model_set_vcv_info(pmod, VCV_ML, VCV_OP);
+	    pmod->opt |= OPT_G;
 	}
     }	
 
@@ -1170,7 +1173,7 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 	if (!err) {
 	    maybe_rescale_hessian(kopt, vcv, k, T);
 	    arma_hessian_vcv(pmod, vcv, k);
-	    gretl_model_set_int(pmod, "ml_vcv", VCV_HESSIAN);
+	    gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
 	}
     }
 
@@ -1269,7 +1272,7 @@ static gretl_matrix *form_arma_x_matrix (const int *alist,
 #endif
 
     x = gretl_matrix_data_subset(xlist, Z, ainfo->t1, ainfo->t2, 
-				 NULL, &err);
+				 M_MISSING_ERROR, &err);
     if (err) {
 	free(xlist);
 	return NULL;
@@ -1445,8 +1448,7 @@ static int kalman_arma (const int *alist, double *coeff,
     /* publish ainfo */
     kainfo = ainfo;
 
-    K = kalman_new(S, P, F, A, H, Q, NULL, y, x, E, ainfo->nc, 
-		   ainfo->ifc, &err);
+    K = kalman_new(S, P, F, A, H, Q, NULL, y, x, E, &err);
 
     if (err) {
 	fprintf(stderr, "kalman_new(): err = %d\n", err);
