@@ -12,7 +12,7 @@ static int audioprint_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 
     gretl_model_get_param_name(pmod, pdinfo, i, varname);
 
-    pprintf(prn, "Variable %d, %s.\n", pmod->list[i+2], varname);
+    pprintf(prn, "Variable %s.\n", varname);
 
     /* print coeff value if well-defined */
     if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
@@ -34,7 +34,8 @@ static int audioprint_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	tval = pmod->coeff[i] / pmod->sderr[i];
 	pval = coeff_pval(pmod->ci, tval, pmod->dfd);
 	pprintf(prn, "P-value %.3g.\n", pval);
-    } else { /* zero standard error */
+    } else { 
+	/* zero standard error */
 	pputs(prn, "Standard error is zero.\n");
     }
 
@@ -427,7 +428,7 @@ int read_window_text (windata_t *vwin, const DATAINFO *pdinfo,
 	GtkTextIter start, end;
 	gchar *window_text;
 
-	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	gtk_text_buffer_get_start_iter(tbuf, &start);
 	gtk_text_buffer_get_end_iter(tbuf, &end);
 	window_text = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
