@@ -94,51 +94,36 @@ write_garch_stats (MODEL *pmod, const int *list,
 		   const double *e, const double *h, 
 		   int npar, int nc, int pad, int ifc, PRN *prn)
 {
-    int err = 0;
-    double *coeff, *sderr, *vcv, *garch_h;
-    double x, den;
+    double *garch_h;
+    double den;
     int ynum = list[4];
     int nvp = list[1] + list[2];
     int xvars = list[0] - 4;
-    int nv = npar * (npar + 1) / 2;
-    int i, j, k;
-
-    coeff = realloc(pmod->coeff, npar * sizeof *pmod->coeff);
-    sderr = realloc(pmod->sderr, npar * sizeof *pmod->sderr);
-    vcv = realloc(pmod->vcv, nv * sizeof *pmod->vcv);
-
-    if (coeff == NULL || sderr == NULL || vcv == NULL) {
-	return E_ALLOC;
-    }
+    int i, err;
 
     if (scale != 1.0) {
 	rescale_results(theta, V, scale, npar, nc);
     }
 
-    for (i=0; i<npar; i++) {
-	coeff[i] = theta[i];
-	x = gretl_matrix_get(V, i, i);
-	sderr[i] = (x > 0.0)? sqrt(x) : 0.0;
-	for (j=0; j<=i; j++) {
-	    k = ijton(i, j, npar);
-	    vcv[k] = gretl_matrix_get(V, i, j);
-	}
+    err = gretl_model_write_coeffs(pmod, theta, npar);
+
+    if (!err) {
+	gretl_model_write_vcv(pmod, V);
+    }
+
+    if (err) {
+	return err;
     }
 
     /* verbose? */
     if (prn != NULL) {
 	for (i=0; i<npar; i++) {
 	    pprintf(prn, "theta[%d]: %#14.6g (%#.6g)\n", i, theta[i], 
-		    sderr[i]);
+		    pmod->sderr[i]);
 	}
 	pputc(prn, '\n'); 
     }   
 
-    pmod->coeff = coeff;
-    pmod->sderr = sderr;
-    pmod->vcv = vcv;
-    pmod->ncoeff = npar;
-
     pmod->ess = 0.0;
     for (i=pmod->t1; i<=pmod->t2; i++) {
 	pmod->uhat[i] = e[i + pad] * scale;
@@ -149,9 +134,9 @@ write_garch_stats (MODEL *pmod, const int *list,
     /* set sigma to its unconditional or steady-state value */
     den = 1.0;
     for (i=0; i<nvp; i++) {
-	den -= coeff[i+xvars+1];
+	den -= pmod->coeff[i+xvars+1];
     }
-    pmod->sigma = sqrt(coeff[xvars] / den);
+    pmod->sigma = sqrt(pmod->coeff[xvars] / den);
 
     pmod->adjrsq = NADBL; 
     pmod->fstt = NADBL;
@@ -448,7 +433,7 @@ garch_driver (const int *list, double **Z, double scale,
 	    gretl_model_set_int(pmod, "fncount", fnc);
 	    gretl_model_set_int(pmod, "grcount", grc);
 	}
-	gretl_model_set_int(pmod, "ml_vcv", vopt);
+	gretl_model_set_vcv_info(pmod, VCV_ML, vopt);
     }
 
  bailout:
@@ -624,10 +609,20 @@ static int *get_garch_list (const int *list, const double **Z,
 			    gretlopt opt, int *ifc, int *err)
 {
     int *glist = NULL;
-    int i, p = list[1], q = list[2];
+    int i, p, q;
     int cpos = 0;
     int add0 = 0;
 
+    /* is the list well-formed? */
+    if (list[0] < 4 || list[1] == LISTSEP ||
+	list[2] == LISTSEP || list[3] != LISTSEP) {
+	*err = E_PARSE;
+	return NULL;
+    }
+
+    p = list[1];
+    q = list[2];
+
     *err = 0;
 
     /* rule out pure AR in variance: the model is unidentified */
@@ -843,6 +838,32 @@ static MODEL garch_run_ols (const int *list, double ***pZ,
     return model;
 }
 
+static void garch_add_lr_test (MODEL *pmod, double llr,
+			       const int *list)
+{
+    if (!na(pmod->lnL) && llr <= pmod->lnL) {
+	double LR = 2.0 * (pmod->lnL - llr);
+	int LRdf = list[1] + list[2];
+
+	gretl_model_set_double(pmod, "garch_LR", LR);
+	gretl_model_set_int(pmod, "garch_LR_df", LRdf);
+    }
+}
+
+static void garch_standardize_residuals (MODEL *pmod)
+{
+    double *h = gretl_model_get_data(pmod, "garch_h");
+
+    if (h != NULL) {
+	int t;
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    pmod->uhat[t] /= sqrt(h[t]);
+	}
+	pmod->opt |= OPT_U;
+    }
+}
+
 /* the driver function for the plugin */
 
 MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
@@ -853,8 +874,9 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     double vparm[VPARM_MAX] = {0};
     double LMF = NADBL;
     double pvF = NADBL;
+    double llr = NADBL;
     double scale = 1.0;
-    int ifc, yno = 0;
+    int ols_T, ifc, yno = 0;
     int err = 0;
 
     list = get_garch_list(cmdlist, (const double **) *pZ,
@@ -872,6 +894,9 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 	return model;
     }
 
+    llr = model.lnL;
+    ols_T = model.nobs;
+
 #if GARCH_AUTOCORR_TEST
     /* pretest the residuals for autocorrelation */
     if (prn != NULL) {
@@ -909,6 +934,15 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     }
 #endif
 
+    if (model.errcode == 0) {
+	if (opt & OPT_U) {
+	    garch_standardize_residuals(&model);
+	}
+	if (!na(llr) && ols_T == model.nobs) {
+	    garch_add_lr_test(&model, llr, cmdlist);
+	}
+    }
+
     free(list);
 
     return model;
