@@ -21,6 +21,7 @@
 #include "matrix_extra.h"
 #include "libset.h"
 #include "missing_private.h"
+#include "gretl_bfgs.h"
 
 #define HDEBUG 0
 #define HTOL 1.0e-09
@@ -28,40 +29,40 @@
 typedef struct h_container_ h_container;
 
 struct h_container_ {
-    int kmain;			/* no. of params in the main eq. */
-    int ksel;			/* no. of params in the selection eq. */
-    double ll;			/* log-likelihood */
+    const int *list;         /* incoming model specification */
+    int kmain;		     /* no. of params in the main eq. */
+    int ksel;		     /* no. of params in the selection eq. */
+    double ll;		     /* log-likelihood */
 
-    int ntot, nunc;		/* total & uncensored obs */
-    int depvar;			/* location of y in array Z */
-    int selvar;			/* location of selection variable in array Z */
-    int *Xlist;			/* regressor list for the main eq. */
-    int *Zlist;			/* regressor list for the selection eq. */
+    int ntot, nunc;	     /* total & uncensored obs */
+    int depvar;		     /* location of y in array Z */
+    int selvar;		     /* location of selection variable in array Z */
+    int *Xlist;		     /* regressor list for the main eq. */
+    int *Zlist;		     /* regressor list for the selection eq. */
 
-    gretl_matrix *y;		/* dependent var */
-    gretl_matrix *reg;		/* main eq. regressors */
-    gretl_matrix *mills;	/* Mills ratios from selection eq */
-    gretl_matrix *delta;	/* used in 2-step vcv calculations */
-    gretl_matrix *d;		/* selection dummy variable */
-    gretl_matrix *selreg;	/* selection eq. regressors */ 
-    gretl_matrix *selreg_u;	/* selection eq. regressors (subsample d==1) */
+    gretl_matrix *y;	     /* dependent var */
+    gretl_matrix *reg;	     /* main eq. regressors */
+    gretl_matrix *mills;     /* Mills ratios from selection eq */
+    gretl_matrix *delta;     /* used in 2-step vcv calculations */
+    gretl_matrix *d;	     /* selection dummy variable */
+    gretl_matrix *selreg;    /* selection eq. regressors */ 
+    gretl_matrix *selreg_u;  /* selection eq. regressors (subsample d==1) */
     gretl_vector *fitted;
     gretl_vector *u;
     gretl_vector *ndx;
 
-    gretl_vector *beta;		/* main eq. parameters */
-    gretl_vector *gama;		/* selection eq. parameters */
+    gretl_vector *beta;	     /* main eq. parameters */
+    gretl_vector *gama;	     /* selection eq. parameters */
     double sigma;
     double rho;
-    double lambda;		/* rho*sigma by definition */
+    double lambda;	     /* rho*sigma by definition */
 
-    gretl_matrix *vcv;		/* Variance-covariance matrix */
-    gretl_matrix *VProbit;	/* 1st stage probit covariance matrix */
-
-    char *probmask;		/* mask NAs for initial probit */
-    char *fullmask;		/* mask NAs */
-    char *uncmask;		/* mask NAs and (d==0) */
+    gretl_matrix *vcv;	     /* Variance-covariance matrix */
+    gretl_matrix *VProbit;   /* 1st stage probit covariance matrix */
 
+    char *probmask;	     /* mask NAs for initial probit */
+    char *fullmask;	     /* mask NAs */
+    char *uncmask;	     /* mask NAs and (d==0) */
 };
 
 static void h_container_destroy (h_container *HC)
@@ -97,7 +98,7 @@ static void h_container_destroy (h_container *HC)
     free(HC);
 }
 
-static h_container *h_container_new (void)
+static h_container *h_container_new (const int *list)
 {
     h_container *HC = malloc(sizeof *HC);
 
@@ -105,6 +106,7 @@ static h_container *h_container_new (void)
 	return NULL;
     }
 
+    HC->list = list;
     HC->ll = NADBL;
 
     HC->Xlist = NULL;
@@ -150,7 +152,7 @@ make_heckit_NA_mask (h_container *HC, const int *Xlist, const int *Zlist,
 		     const double **Z, const DATAINFO *pdinfo)
 {
     int sel = Zlist[1];
-    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int T = sample_size(pdinfo);
     int i, t, s = 0, m = 0;
     int err = 0;
 
@@ -231,7 +233,7 @@ static int make_uncens_mask (h_container *HC, const int *Xl,
 			     const int *Zl, const double **Z, 
 			     DATAINFO *pdinfo)
 {
-    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int T = sample_size(pdinfo);
     int t, s = 0;
     int selvar = Zl[1];
 
@@ -273,7 +275,7 @@ static int h_container_fill (h_container *HC, const int *Xl,
     int v, i, err = 0;
 
     /* it is assumed that the Mills ratios have just been added to the dataset;
-       hence, they're the last variable
+       hence they're the last variable
     */
     v = pdinfo->v - 1;
 
@@ -352,22 +354,22 @@ static int h_container_fill (h_container *HC, const int *Xl,
 
     if (!err) {
 	tmplist[1] = depvar;
-	HC->y = gretl_matrix_data_subset(tmplist, Z, t1, t2, 
-					 HC->uncmask, &err);
+	HC->y = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
+						HC->uncmask, &err);
     }
 
     if (!err) {
 	HC->nunc = gretl_matrix_rows(HC->y);
 	tmplist[1] = selvar;
-	HC->d = gretl_matrix_data_subset(tmplist, Z, t1, t2, 
-					 HC->fullmask, &err);
+	HC->d = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
+						HC->fullmask, &err);
     }
 
     if (!err) {
 	HC->ntot = gretl_matrix_rows(HC->d);
 	tmplist[1] = v;
-	HC->mills = gretl_matrix_data_subset(tmplist, Z, t1, t2, 
-					     HC->uncmask, &err);
+	HC->mills = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
+						    HC->uncmask, &err);
     }
 
     if (!err) {
@@ -379,7 +381,7 @@ static int h_container_fill (h_container *HC, const int *Xl,
     }
 
     if (err) {
-	goto bailout;
+	return err;
     }
 
     for (i=1; i<=HC->Xlist[0]; i++) {
@@ -390,21 +392,23 @@ static int h_container_fill (h_container *HC, const int *Xl,
 	HC->Zlist[i] = Zl[i+1];
     }
 
-    HC->reg = gretl_matrix_data_subset(HC->Xlist, Z, t1, t2, 
-				       HC->uncmask, &err);
+    HC->reg = gretl_matrix_data_subset_masked(HC->Xlist, Z, t1, t2, 
+					      HC->uncmask, &err);
 
     if (!err) {
-	HC->selreg = gretl_matrix_data_subset(HC->Zlist, Z, t1, t2, 
-					      HC->fullmask, &err);
+	HC->selreg = 
+	    gretl_matrix_data_subset_masked(HC->Zlist, Z, t1, t2, 
+					    HC->fullmask, &err);
     }
 
     if (!err) {
-	HC->selreg_u = gretl_matrix_data_subset(HC->Zlist, Z, t1, t2, 
-						HC->uncmask, &err);
+	HC->selreg_u = 
+	    gretl_matrix_data_subset_masked(HC->Zlist, Z, t1, t2, 
+					    HC->uncmask, &err);
     }
 
     if (err) {
-	goto bailout;
+	return err;
     }    
 
     tmp = gretl_matrix_multiply_new(HC->selreg_u, HC->gama, &err);
@@ -443,8 +447,6 @@ static int h_container_fill (h_container *HC, const int *Xl,
 	}
     }
 
- bailout:
-
     return err;
 }
 
@@ -602,6 +604,14 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
 	    }
 	}
     }
+
+#if 0
+    /* R-squared based on correlation? */
+    x = gretl_corr(hm->t1, hm->t2, Z[depvar], hm->yhat, NULL);
+    if (!na(x)) {
+	hm->rsq = x * x;
+    }
+#endif
 }
 
 /*
@@ -640,20 +650,21 @@ static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATAINFO *pdinf
 	    strcpy(hm->params[i+ko+1], pdinfo->varname[HC->Zlist[i+1]]);
 	    fullcoeff[i+ko+1] = gretl_vector_get(HC->gama, i);
 	}
-
     }
 
     hm->lnL = HC->ll;
+    mle_criteria(hm, 0); /* FIXME? */
     hm->sigma = HC->sigma;
     hm->rho = HC->rho;
 
     if (!err) {
+	free(hm->list);
+	hm->list = gretl_list_copy(HC->list);
 	free(hm->coeff);
 	hm->coeff = fullcoeff;
 	hm->ncoeff = k;
 	gretl_model_set_coeff_separator(hm, N_("Selection equation"), ko + 1);
 	gretl_model_set_int(hm, "base-coeffs", ko);
-	hm->list[0] -= 1;
     }
     
     return err;
@@ -665,7 +676,7 @@ static int transcribe_2step_vcv (MODEL *pmod, const gretl_matrix *S,
     int m = S->rows;
     int n = Vp->rows;
     int nvc, tot = m + n;
-    int i, j, k = 0;
+    int i, j, k;
     double vij;
 
     if (pmod->vcv != NULL) {
@@ -685,10 +696,17 @@ static int transcribe_2step_vcv (MODEL *pmod, const gretl_matrix *S,
 	return E_ALLOC;
     }
 
+    /* build combined covariance matrix */
+
+    k = 0;
     for (i=0; i<tot; i++) {
 	for (j=i; j<tot; j++) {
 	    if (i < m) {
-		vij = (j < m)? gretl_matrix_get(S, i, j) : NADBL;
+		/* How to handle the "irrelevant" block?  In the line below
+		   these elements were being set to NADBL; for the moment
+		   I'm setting them to zero -- AC, 2009-04-11.
+		*/
+		vij = (j < m)? gretl_matrix_get(S, i, j) : 0.0;
 	    } else {
 		vij = gretl_matrix_get(Vp, i-m, j-m);
 	    }
@@ -957,8 +975,7 @@ static int transcribe_ml_vcv (MODEL *pmod, h_container *HC)
     return 0;
 }
 
-static MODEL heckit_init (int *list, double ***pZ, DATAINFO *pdinfo,
-			  h_container *HC) 
+static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 {
 #if HDEBUG
     PRN *prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
@@ -974,7 +991,7 @@ static MODEL heckit_init (int *list, double ***pZ, DATAINFO *pdinfo,
     gretl_model_init(&hm);
     gretl_model_init(&probmod);
 
-    err = gretl_list_split_on_separator(list, &Xlist, &Zlist);
+    err = gretl_list_split_on_separator(HC->list, &Xlist, &Zlist);
     if (err) {
 	goto bailout;
     } 
@@ -1061,7 +1078,7 @@ static MODEL heckit_init (int *list, double ***pZ, DATAINFO *pdinfo,
    ML.
 */
 
-MODEL heckit_estimate (int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		       gretlopt opt, PRN *prn) 
 {
     h_container *HC = NULL;
@@ -1071,13 +1088,13 @@ MODEL heckit_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
 
     gretl_model_init(&hm);
 
-    HC = h_container_new();
+    HC = h_container_new(list);
     if (HC == NULL) {
 	hm.errcode = E_ALLOC;
 	return hm;
     }
 
-    hm = heckit_init(list, pZ, pdinfo, HC);
+    hm = heckit_init(HC, pZ, pdinfo);
     if (hm.errcode) {
 	h_container_destroy(HC);
 	return hm;
@@ -1106,7 +1123,7 @@ MODEL heckit_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (hm.errcode == 0 && (opt & OPT_T)) {
-	gretl_model_set_int(&hm, "two-step", 1);
+	hm.opt |= OPT_T;
     }
 
     if (pdinfo->v > oldv) {
