@@ -26,14 +26,14 @@ static void invalid_varname (PRN *prn)
 #ifndef ODS_IMPORTER
 
 #ifdef EXCEL_IMPORTER
-# define cell_string(i,j) (rows[i].cells[j])
+# define cell_string(i,j) ((rows[i].cells != NULL)? rows[i].cells[j] : NULL)
 #else
 # define cell_string(i,j) (labels[i])
 #endif
 
 static int 
 importer_dates_check (int row_offset, int col_offset, 
-		      BookFlag flags, char **labels, 
+		      BookFlag *pflags, char **labels, 
 		      DATAINFO *newinfo, PRN *prn, int *err)
 {
     int d, t;
@@ -43,7 +43,7 @@ importer_dates_check (int row_offset, int col_offset,
 
     for (t=0; t<newinfo->n; t++) {
 	s = cell_string(t + row_offset, col_offset);
-	if (*s == '\0') {
+	if (s == NULL || *s == '\0') {
 	    fprintf(stderr, "importer_dates_check: got blank label\n");
 	    return 0;
 	}
@@ -57,9 +57,9 @@ importer_dates_check (int row_offset, int col_offset,
     for (t=0; t<newinfo->n && !*err; t++) {
 	s = cell_string(t + row_offset, col_offset);
 	if (*s == '"' || *s == '\'') s++;
-	if (flags & BOOK_NUMERIC_DATES) {
+	if (*pflags & BOOK_NUMERIC_DATES) {
 	    if (sscanf(s, "%d", &d)) {
-		MS_excel_date_string(dstr, d, 0, flags & BOOK_DATE_BASE_1904);
+		MS_excel_date_string(dstr, d, 0, *pflags & BOOK_DATE_BASE_1904);
 		s = dstr;
 	    } else {
 		pprintf(prn, "Bad date on row %d: '%s'\n", t+1, s);
@@ -70,7 +70,12 @@ importer_dates_check (int row_offset, int col_offset,
     }
 
     if (!*err) {
-	ret = test_markers_for_dates(NULL, newinfo, NULL, prn);
+	int reversed = 0;
+
+	ret = test_markers_for_dates(NULL, newinfo, &reversed, NULL, prn);
+	if (reversed) {
+	    *pflags |= BOOK_DATA_REVERSED;
+	}
     }
 
     if (newinfo->markers != DAILY_DATE_STRINGS) {
