@@ -40,11 +40,11 @@ static void remove_temp_dir (char *dname)
 #ifdef G_OS_WIN32
     char *fullpath = g_strdup_printf("%s%s", udir, dname);
 
-    chdir(udir);
+    gretl_chdir(udir);
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    chdir(udir);
+    gretl_chdir(udir);
     gretl_deltree(dname);
 #endif
 }
@@ -1005,19 +1005,24 @@ static int gretl_make_tempdir (char *dname)
 
 static int ts_check (ods_sheet *sheet, PRN *prn)
 {
+    int reversed = 0;
     int mpd = -1;
 
     if (sheet->dinfo->S == NULL) {
 	return 0;
     }
 
-    mpd = test_markers_for_dates(&sheet->Z, sheet->dinfo, NULL, prn);
+    mpd = test_markers_for_dates(&sheet->Z, sheet->dinfo, &reversed,
+				 NULL, prn);
 
     if (mpd > 0) {
 	pputs(prn, _("taking date information from row labels\n\n"));
 	if (sheet->dinfo->markers != DAILY_DATE_STRINGS) {
 	    dataset_destroy_obs_markers(sheet->dinfo);
 	}
+	if (reversed) {
+	    sheet->flags |= BOOK_DATA_REVERSED;
+	}
     } 
 
 #if ODEBUG
@@ -1201,6 +1206,9 @@ static int finalize_ods_import (double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	tprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
 	ts_check(sheet, tprn);
+	if (sheet->flags & BOOK_DATA_REVERSED) {
+	    reverse_data(sheet->Z, sheet->dinfo, tprn);
+	}
 	gretl_print_destroy(tprn);
     }
 
@@ -1256,16 +1264,16 @@ int ods_get_data (const char *fname, int *list, char *sheetname,
     }
 
     /* cd to user dir */
-    if (chdir(udir)) {
+    if (gretl_chdir(udir)) {
 	gretl_errmsg_set_from_errno(udir);
 	return E_FOPEN;
     }
 
     err = gretl_make_tempdir(dname);
     if (!err) {
-	err = chdir(dname);
+	err = gretl_chdir(dname);
 	if (err) {
-	    remove(dname);
+	    gretl_remove(dname);
 	}
     }
     
@@ -1276,7 +1284,7 @@ int ods_get_data (const char *fname, int *list, char *sheetname,
     gretl_unzip_file = get_plugin_function("gretl_unzip_file", 
 					   &handle);
     if (gretl_unzip_file == NULL) {
-	remove(dname);
+	gretl_remove(dname);
         return E_FOPEN;
     }
 
