@@ -19,6 +19,7 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "gretl_bfgs.h"
 
 #define TDEBUG 0
 
@@ -479,7 +480,8 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
     }
 
     chesher_irish_test(pmod, X);
-    pmod->fstt = pmod->rsq = pmod->adjrsq = NADBL;
+    pmod->fstt = pmod->adjrsq = NADBL;
+    pmod->fstt = pmod->chisq = NADBL;
     mle_criteria(pmod, 1);
 
     pmod->ci = TOBIT;
