@@ -276,9 +276,9 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 
     /* irregular component */
     if (opt == TRAMO_SEATS && !tramo_got_irfin) {
-	sprintf(title, "%s", G_("irregular"));
+	sprintf(title, "%s", _("irregular"));
     } else {
-	sprintf(title, "%s - 1", G_("irregular"));
+	sprintf(title, "%s - 1", _("irregular"));
     }
 
     fprintf(fp, "set bars 0\n"
@@ -299,7 +299,7 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
     fprintf(fp, "set origin 0.0,0.33\n"
 	    "plot '-' using 1:2 title '%s' w l, \\\n"
 	    " '-' using 1:2 title '%s' w l\n",
-	    pdinfo->varname[0], G_("trend/cycle"));
+	    pdinfo->varname[0], _("trend/cycle"));
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) { 
 	fprintf(fp, "%g %g\n", obs[t], Z[0][t]);
@@ -316,7 +316,7 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
     fprintf(fp, "set origin 0.0,0.66\n"
 	    "plot '-' using 1:2 title '%s' w l, \\\n"
 	    " '-' using 1:2 title '%s' w l\n",
-	    pdinfo->varname[0], G_("adjusted"));
+	    pdinfo->varname[0], _("adjusted"));
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	fprintf(fp, "%g %g\n", obs[t], Z[0][t]);
@@ -358,11 +358,11 @@ static void clear_tramo_files (const char *tpath, const char *varname)
     for (i=D11; i<=TRIGRAPH; i++) {
 	sprintf(tfname, "%s%cgraph%cseries%c%s", tpath, SLASH, SLASH, SLASH,
 		tramo_series_strings[i]);
-	remove(tfname);
+	gretl_remove(tfname);
     }
 
     sprintf(tfname, "%s%coutput%c%s.out", tpath, SLASH, SLASH, varname);
-    remove(tfname);
+    gretl_remove(tfname);
 }
 
 static int add_series_from_file (const char *fname, int code,
